-- Feature esia-device-notes
CREATE TABLE IF NOT EXISTS esia_device_notes_thread
(
    thread_id serial NOT NULL,
    thread_device_id bigint NOT NULL,
    thread_type character varying(16) NOT NULL DEFAULT 'TEXT',
    thread_title character varying(120),
    CONSTRAINT thread_primary_key PRIMARY KEY (thread_id),
    CONSTRAINT device_notes_thread_foreignKey_device_id FOREIGN KEY (thread_device_id)
        REFERENCES esia_device (device_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
);

ALTER TABLE IF EXISTS esia_device_notes_thread OWNER to esia;

CREATE TABLE IF NOT EXISTS esia_device_notes
(
    note_id bigserial NOT NULL,
    note_date timestamp without time zone NOT NULL DEFAULT now(),
    note_last_update timestamp without time zone NOT NULL DEFAULT now(),
    note_user_id bigint,
    note_user_name character varying NOT NULL,
    note_thread_id bigint NOT NULL,
    note_error_happen_id bigint DEFAULT NULL,
    note_message character varying(300) NOT NULL,
    note_auto_purge boolean DEFAULT true,
    CONSTRAINT note_primary_key PRIMARY KEY (note_id),
    CONSTRAINT device_notes_foreignKey_user_id FOREIGN KEY (note_user_id)
        REFERENCES esia_users (user_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION,
    CONSTRAINT device_notes_foreignKey_thread_id FOREIGN KEY (note_thread_id)
        REFERENCES esia_device_notes_thread (thread_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION,
    CONSTRAINT device_notes_foreignKey_happen_id FOREIGN KEY (note_error_happen_id)
        REFERENCES esia_error_happen (happen_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION
);

ALTER TABLE IF EXISTS esia_device_notes OWNER to esia;

INSERT INTO esia_privileges VALUES (DEFAULT, 'device_notes_view', 0, 'Autorise la vue des notes d''un noeud');
INSERT INTO esia_privilege_grantedto VALUES (1, (SELECT privilege_id FROM esia_privileges WHERE privilege_name='device_notes_view'), 1);
INSERT INTO esia_privileges VALUES (DEFAULT, 'device_notes_administration', 0, 'Autorise le post de messages dans les notes d''un noeud');
INSERT INTO esia_privilege_grantedto VALUES (1, (SELECT privilege_id FROM esia_privileges WHERE privilege_name='device_notes_administration'), 1);

-- Add user name to esia_error_happen for error assignment
ALTER TABLE IF EXISTS esia_error_happen ADD COLUMN happen_assign_user character varying;

-- Device notes purge
INSERT INTO esia_configuration(config_key, config_value, config_description, config_type, config_visibility) VALUES ('DEVICES_NOTES_PURGE', 365, 'Purge automatique des notes de noeud', 'smallint', true);

INSERT INTO esia_plugins(
    plugin_id, plugin_name, plugin_path, plugin_type, plugin_default_interval_time, plugin_description, plugin_priority, plugin_manufacturer, plugin_tag, plugin_cron_type, plugin_cron_only_existing
) VALUES (
    DEFAULT, 'CRON_DEVICES_NOTES_PURGE', '/usr/local/esia/public_html/cron/purge-devices-notes.php', 'cron', 86400000, 'Purge des notes de noeud', 5, 'ESIA', 'ESIA,DEVICES,CRON', 'day', true
);

INSERT INTO esia_cron_plugin(
	cron_id, cron_plugin_id, cron_interval, cron_next_exec, cron_log_file, cron_retries, cron_retries_interval
) VALUES (
    DEFAULT, (SELECT plugin_id FROM esia_plugins WHERE plugin_name='CRON_DEVICES_NOTES_PURGE'), '1 day'::interval, CURRENT_DATE + interval'25 hour', 'devices_notes_$SERVICEID_$DATE.log', 1, '30 minute'::interval
);

--Update Trigger utilisateur & DEVICE

CREATE OR REPLACE FUNCTION public.trigger_delete_user()
    RETURNS trigger
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE NOT LEAKPROOF
AS $BODY$
DECLARE
    ForKEY RECORD;
BEGIN
	UPDATE esia_device_notes SET note_user_id=NULL WHERE note_user_id=OLD.user_id;
	FOR ForKEY IN SELECT tc.table_name, kcu.column_name, ccu.table_name AS foreign_table_name,ccu.column_name AS foreign_column_name
		FROM information_schema.table_constraints  tc
		JOIN information_schema.key_column_usage kcu ON tc.constraint_name = kcu.constraint_name
		JOIN information_schema.constraint_column_usage ccu ON ccu.constraint_name = tc.constraint_name
		WHERE constraint_type = 'FOREIGN KEY' AND ccu.table_name='esia_users' LOOP

	EXECUTE 'DELETE FROM '|| ForKEY.table_name || ' WHERE '||ForKEY.column_name||' = '||OLD.user_id;
	END LOOP;
	RETURN OLD;
END;
$BODY$;

ALTER FUNCTION public.trigger_delete_user() OWNER TO esia;


CREATE OR REPLACE FUNCTION public.trigger_delete_device()
    RETURNS trigger
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE NOT LEAKPROOF
AS $BODY$
DECLARE
    ForKEY RECORD;
BEGIN
	DELETE FROM esia_device_notes WHERE note_thread_id IN(SELECT thread_id FROM esia_device_notes_thread WHERE thread_device_id= OLD.device_id );
	FOR ForKEY IN SELECT tc.table_name, kcu.column_name, ccu.table_name AS foreign_table_name,ccu.column_name AS foreign_column_name
		FROM information_schema.table_constraints  tc
		JOIN information_schema.key_column_usage kcu ON tc.constraint_name = kcu.constraint_name
		JOIN information_schema.constraint_column_usage ccu ON ccu.constraint_name = tc.constraint_name WHERE constraint_type = 'FOREIGN KEY' AND ccu.table_name='esia_device' LOOP

	EXECUTE 'DELETE FROM '|| ForKEY.table_name || ' WHERE '||ForKEY.column_name||' = '||OLD.device_id;
	END LOOP;
	RETURN OLD;
END;
$BODY$;

ALTER FUNCTION public.trigger_delete_device()
    OWNER TO esia;


CREATE OR REPLACE FUNCTION public.trigger_delete_error_happen()
    RETURNS trigger
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE NOT LEAKPROOF
AS $BODY$
DECLARE
    ForKEY RECORD;
BEGIN
    UPDATE esia_device_notes SET note_error_happen_id=NULL WHERE note_error_happen_id=OLD.happen_id;
    RETURN OLD;
END;
$BODY$;

ALTER FUNCTION public.trigger_delete_error_happen() OWNER TO esia;

CREATE TRIGGER "deleteErrorHappen"
    BEFORE DELETE
    ON public.esia_error_happen
    FOR EACH ROW
    EXECUTE FUNCTION public.trigger_delete_error_happen();


--- Modification dashboard
ALTER TABLE IF EXISTS public.dashboard ADD COLUMN IF NOT EXISTS dashboard_share_read_groups integer[];
ALTER TABLE IF EXISTS public.dashboard ADD COLUMN IF NOT EXISTS dashboard_share_write_groups integer[];

-- Trigger

CREATE TABLE IF NOT EXISTS public.esia_triggers
(
    trigger_id serial,
    trigger_name character varying(50) COLLATE pg_catalog."default" NOT NULL,
    trigger_table_name character varying(100) COLLATE pg_catalog."default" NOT NULL,
    trigger_when character varying(10) COLLATE pg_catalog."default" NOT NULL,
    trigger_operation character varying(20) COLLATE pg_catalog."default" NOT NULL,
    trigger_web_plugin integer,
    trigger_order smallint NOT NULL DEFAULT 10,
    trigger_file_path character varying(200) COLLATE pg_catalog."default" NOT NULL,
    trigger_function character varying(100) COLLATE pg_catalog."default" NOT NULL,
    CONSTRAINT esia_triggers_pkey PRIMARY KEY (trigger_id),
    CONSTRAINT "trigger_foreignKey_webid" FOREIGN KEY (trigger_web_plugin)
        REFERENCES public.esia_web_plugin (web_id) MATCH SIMPLE
        ON UPDATE NO ACTION
        ON DELETE NO ACTION,
    CONSTRAINT trigger_check_operation CHECK (trigger_operation::text = ANY (ARRAY['DELETE'::character varying, 'UPDATE'::character varying, 'INSERT'::character varying]::text[])),
    CONSTRAINT trigger_check_when CHECK (trigger_when::text = ANY (ARRAY['BEFORE'::character varying, 'AFTER'::character varying]::text[]))
);
ALTER TABLE IF EXISTS public.esia_triggers OWNER to esia;


INSERT INTO public.esia_triggers(trigger_id, trigger_name, trigger_table_name, trigger_when, trigger_operation, trigger_order, trigger_file_path, trigger_function)
VALUES (DEFAULT,'share_dashboard','esia_memberof','AFTER','INSERT',50,'triggers.php','triggers\afterInsertMemberof');


INSERT INTO public.esia_triggers(trigger_id, trigger_name, trigger_table_name, trigger_when, trigger_operation, trigger_order, trigger_file_path, trigger_function)
VALUES (DEFAULT,'unshare_dashboard','esia_memberof','AFTER','DELETE',50,'triggers.php','triggers\beforeDeleteMemberof');

ALTER TABLE IF EXISTS public.esia_configuration ADD COLUMN config_web_load boolean DEFAULT False;
INSERT INTO esia_configuration (config_key, config_value, config_description, config_type, config_visibility, config_web_load)
VALUES ('DEFAULT_LANG','en','Langue par défaut','lang',TRUE,TRUE);

INSERT INTO esia_configuration (config_key, config_value, config_description, config_type, config_visibility, config_web_load)
VALUES ('DEFAULT_TIMEZONE','Europe/Brussels','Fuseau horaire par défaut','timezone',TRUE,TRUE);

UPDATE esia_configuration
SET config_web_load= TRUE
WHERE config_key IN ('LICENCE','ESIA_URL','DEFAULT_BANNER_FONTCOLOR','ESIA_TYPE','SITE_LOCATION','DEFAULT_BANNER','DEFAULT_LANG','DEFAULT_TIMEZONE','LICENCE_RETEST');
UPDATE esia_configuration SET config_type='integer_unsigned' WHERE config_key='RSS_PURGE_MESSAGE';
UPDATE esia_configuration SET config_type='integer_unsigned' WHERE config_key='DEVICES_NOTES_PURGE';


ALTER TABLE IF EXISTS public.esia_maintenance_configuration ADD COLUMN IF NOT EXISTS maintenance_week_selected integer;
ALTER TABLE IF EXISTS public.esia_maintenance_configuration ADD COLUMN IF NOT EXISTS maintenance_day_selected integer DEFAULT 0;

CREATE OR REPLACE FUNCTION public.updatenextmaintenancenextmonth(
	confid bigint)
    RETURNS integer
    LANGUAGE 'plpgsql'
    COST 100
    VOLATILE PARALLEL UNSAFE
AS $BODY$
DECLARE
	v_row RECORD;
	v_date DATE;
BEGIN
	SELECT (date_trunc('month', maintenance_conf_start_date)+'1 month'::interval)as nexmonth,
			maintenance_week_selected,
			maintenance_day_selected
			INTO v_row
	FROM esia_maintenance_configuration
	WHERE maintenance_conf_id = confid;
	v_date = v_row.nexmonth::date;
	IF v_row.maintenance_week_selected < 4
	THEN
		--SELECTION du premier jour du mois demandé (0 = lundi et 6 pour dimanche)
		--					(8 + (le jour de la semaine) +  jour de la maintenance)
		v_date = v_date + ((8 - extract(dow from v_row.nexmonth)+ v_row.maintenance_day_selected)::integer%7);
		-- On passe sur la première, deuxième ou troisième semaine
		v_date = v_date + (7*v_row.maintenance_week_selected);
	ELSE
		--On veut le dernier jour du mois
		v_date = v_date +'1 month'::interval - '1 day'::interval;

		--On prend le dernier jour - ( 6 - (le jour de la semaine) - jour de la maintenance )
		-- exemple pour le dernier mardi de janvier 2025 (2025-01-28):
		--     (v_date vaut donc 2025-01-31)
		-- 	    le 31 est un vendredi dow = 5
		--		la maintenance doit avoir lieu mardi maintenance_day_selected = 1
		--     (2025-01-31) - ( 6 + 5 - 1)%7
		--     (2025-01-31) - (10%7)
		--	   (2025-01-31) - 3 jours = 2025-01-28
		v_date = v_date - ((6 + extract(dow from v_date) - v_row.maintenance_day_selected)::integer%7);
	END IF;
	UPDATE esia_maintenance_configuration
		SET maintenance_conf_start_date = v_date::timestamp + maintenance_conf_start_date::time
	WHERE maintenance_conf_id = confid;
	RETURN confid;
END;
$BODY$;

ALTER FUNCTION public.updatenextmaintenancenextmonth(bigint)
    OWNER TO esia;

-- Update dashboards linked to import module
UPDATE dashboard_widget
SET widget_option = replace(widget_option::text, '\/plugin\/importation\/import_control.php', '\/plugin\/ipam\/control_import.php')::json
WHERE widget_view_id=(SELECT view_id FROM dashboard_view WHERE view_name='DASHBOARD_FAST_LINK')
AND jsonb_path_match(widget_option::jsonb, 'exists($.link[*].text ? (@ == "/plugin/importation/import_control.php"))');

ALTER TABLE public.esia_device ALTER COLUMN device_type TYPE character varying(30) COLLATE pg_catalog."default";
