--
-- PostgreSQL database dump
--

-- Dumped from database version 15.7 (Debian 15.7-0+deb12u1)
-- Dumped by pg_dump version 15.7 (Debian 15.7-0+deb12u1)

SET statement_timeout = 0;
SET lock_timeout = 0;
SET idle_in_transaction_session_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SELECT pg_catalog.set_config('search_path', '', false);
SET check_function_bodies = false;
SET xmloption = content;
SET client_min_messages = warning;
SET row_security = off;

--
-- Name: tablefunc; Type: EXTENSION; Schema: -; Owner: -
--

CREATE EXTENSION IF NOT EXISTS tablefunc WITH SCHEMA public;


--
-- Name: EXTENSION tablefunc; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION tablefunc IS 'functions that manipulate whole tables, including crosstab';


--
-- Name: count_service(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.count_service() RETURNS integer
    LANGUAGE plpgsql
    AS $$
	DECLARE 
		v_service integer;
        BEGIN
		SELECT count(*) INTO v_service
		FROM esia_services
		JOIN esia_device ON service_device_id = device_id
		WHERE service_delete=FALSE AND service_name <> 'GESA_HEALTH' AND device_ip_addr<>2130706433;
		RETURN v_service;
		
   END;
$$;


ALTER FUNCTION public.count_service() OWNER TO esia;

--
-- Name: count_service(integer, integer); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.count_service(v_year integer, v_month integer) RETURNS integer
    LANGUAGE plpgsql
    AS $$
	DECLARE 
		v_sms integer;
        BEGIN
		SELECT count(*) INTO v_sms 
		FROM sms_send
		WHERE sms_date > date (v_year||'-'||v_month||'-01') 
			AND  sms_date < date (v_year||'-'||v_month||'-01')  +interval '1 month';
		RETURN v_sms;
		
   END;
$$;


ALTER FUNCTION public.count_service(v_year integer, v_month integer) OWNER TO esia;

--
-- Name: get_availability(bigint, bigint, timestamp without time zone, interval); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.get_availability(nodeid bigint, serviceid bigint, enddate timestamp without time zone, inter interval) RETURNS real
    LANGUAGE plpgsql
    AS $$
DECLARE 
	stardate timestamp without time zone;
BEGIN	
	--on calcul le nombre de seconde dans l'interval de temps
	SELECT (enddate - inter) INTO stardate;
	RETURN get_availability(nodeid,serviceid,stardate,enddate);
END;
$$;


ALTER FUNCTION public.get_availability(nodeid bigint, serviceid bigint, enddate timestamp without time zone, inter interval) OWNER TO esia;

--
-- Name: get_availability(bigint, bigint, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.get_availability(nodeid bigint, serviceid bigint, stardate timestamp without time zone, enddate timestamp without time zone) RETURNS real
    LANGUAGE plpgsql
    AS $$
DECLARE 
	nbrsec bigint;
	v_ava real = 100;
	percent real;
	r RECORD;
BEGIN		
	SELECT date_part('epoch', enddate - stardate) INTO nbrsec;
	--CALCUL du nombre de seconde d'indisponnibilité
	FOR r IN SELECT error_type,SUM (sec_in_error) as time_notavailable 
		FROM (
			--Si l'erreur c'est produite entre l'interval de temps
			SELECT error_type, date_part('epoch',SUM(happen_time_acquitte - happen_timestamp)) as sec_in_error
			FROM ( 
				SELECT error_type, happen_timestamp, happen_time_acquitte
				FROM esia_error_happen
				LEFT JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_service_id = serviceid AND happen_device_id = nodeid AND error_type <> 'unknown'
					AND happen_timestamp > stardate
					AND happen_timestamp < enddate
				) AS T2
			WHERE T2.happen_time_acquitte < enddate
			GROUP BY T2.error_type

			UNION
			--Si l'erreur c'est produite durant l'interval de temps et que le acuitte es§t toujours actif ou après le enddate
			SELECT error_type, date_part('epoch',SUM(happen_time_acquitte - happen_timestamp)) as sec_in_error
			FROM  ( 
				SELECT error_type, happen_timestamp, happen_time_acquitte
				FROM esia_error_happen
				LEFT JOIN esia_errors ON error_id = happen_error_id
				WHERE happen_service_id = serviceid AND happen_device_id = nodeid AND error_type <> 'unknown'
					AND happen_timestamp > stardate
					AND happen_timestamp < enddate
			) AS T2
			WHERE happen_time_acquitte > enddate 
				OR happen_time_acquitte IS NULL 
			GROUP BY error_type
			
			UNION
			--si l'erreur c'est produite avant la start date et est toujours active ou que le time acquite est après la date de fin
			SELECT error_type, date_part('epoch',SUM(enddate - stardate)) as sec_in_error
			FROM esia_error_happen
			LEFT JOIN esia_errors ON error_id = happen_error_id
			WHERE happen_service_id = serviceid AND error_type <> 'unknown'
				AND happen_timestamp < stardate
				AND ( happen_acquitte=0 OR happen_time_acquitte > enddate)
			GROUP BY error_type
				--service non présent
			UNION
				SELECT 'nopresent' as error_type,date_part('epoch',SUM(enddate -stardate )) as sec_in_error
				FROM esia_services
				WHERE service_id = serviceid
					AND service_time_added >  stardate
			) A GROUP BY error_type
	LOOP
		--RAISE NOTICE 'i want to print % and % %', r.time_notavailable ,r.error_type,nbrsec ;
		IF r.time_notavailable > 0
		THEN
			percent = (r.time_notavailable / nbrsec)*100;
		ELSE
			percent=0;
		END IF;
		--RAISE NOTICE 'percent: %',percent;
		v_ava = v_ava - percent;
		IF v_ava < 0
		THEN
			v_ava=0;
		END IF;
	END LOOP;
	return v_ava;
	


END;
$$;


ALTER FUNCTION public.get_availability(nodeid bigint, serviceid bigint, stardate timestamp without time zone, enddate timestamp without time zone) OWNER TO esia;

--
-- Name: get_timeerror(bigint, bigint, timestamp without time zone, timestamp without time zone); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.get_timeerror(nodeid bigint, serviceid bigint, stardate timestamp without time zone, enddate timestamp without time zone) RETURNS TABLE(type character varying, nbrsec double precision)
    LANGUAGE plpgsql
    AS $$
BEGIN	
	SELECT date_part('epoch', enddate - stardate) INTO nbrsec;
	RETURN QUERY SELECT error_type, date_part('epoch',SUM(date_trunc('second',happen_time_acquitte) - date_trunc('second',happen_timestamp))) as sec_in_error FROM (
                --L'erreur de fin coupée
		SELECT error_type,happen_timestamp,enddate AS happen_time_acquitte
		FROM esia_error_happen 
		LEFT JOIN esia_errors ON error_id = happen_error_id
		WHERE happen_service_id = serviceid AND happen_device_id = nodeid
		AND happen_timestamp < enddate
		AND happen_time_acquitte > enddate
		UNION
		SELECT error_type,stardate as happen_timestamp,enddate AS happen_time_acquitte
		FROM esia_error_happen 
		LEFT JOIN esia_errors ON error_id = happen_error_id
		WHERE happen_service_id = serviceid AND happen_device_id = nodeid
		AND happen_timestamp < stardate
		AND happen_time_acquitte IS NULL
		UNION
		SELECT error_type,happen_timestamp,enddate AS happen_time_acquitte
		FROM esia_error_happen 
		LEFT JOIN esia_errors ON error_id = happen_error_id
		WHERE happen_service_id = serviceid AND happen_device_id = nodeid
		AND happen_timestamp > stardate
		AND happen_time_acquitte IS NULL
		UNION
                --Les erreurs qui ne sont pas coupées
		SELECT error_type,happen_timestamp,happen_time_acquitte 
		FROM esia_error_happen
		LEFT JOIN esia_errors ON error_id = happen_error_id
		WHERE happen_service_id = serviceid AND happen_device_id = nodeid
		AND happen_timestamp > stardate
		AND happen_time_acquitte < enddate
		UNION
                --L'erreur de début coupée
		SELECT error_type,stardate AS happen_timestamp,happen_time_acquitte 
		FROM esia_error_happen
		LEFT JOIN esia_errors ON error_id = happen_error_id
		WHERE happen_service_id = serviceid AND happen_device_id = nodeid
		AND happen_timestamp < stardate
		AND happen_time_acquitte > stardate
		UNION		
		-- nopresent
		SELECT 'nopresent' as error_type,stardate AS happen_timestamp,service_time_added AS happen_time_acquitte
		FROM esia_services
		WHERE service_id = serviceid
		AND service_time_added >  stardate
	) AS T2
	GROUP BY T2.error_type
	UNION
	SELECT 'total', date_part('epoch', enddate - stardate);
END;
$$;


ALTER FUNCTION public.get_timeerror(nodeid bigint, serviceid bigint, stardate timestamp without time zone, enddate timestamp without time zone) OWNER TO esia;

--
-- Name: getallchilddevicegroup(integer); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.getallchilddevicegroup(groupid integer) RETURNS TABLE(group_id integer, group_name character varying, group_parent integer, group_order integer)
    LANGUAGE plpgsql
    AS $$
    BEGIN
IF groupid IS NULL
THEN
RETURN QUERY SELECT device_group_id, device_group_name,device_group_parent,device_group_order
       FROM esia_device_group;
ELSE
RETURN QUERY
WITH RECURSIVE q AS (SELECT device_group_id, device_group_name,device_group_parent,device_group_order
       FROM esia_device_group
WHERE device_group_id = groupid
      UNION ALL
     SELECT m.device_group_id, m.device_group_name,m.device_group_parent,m.device_group_order
       FROM esia_device_group m
       JOIN q ON q.device_group_id = m.device_group_parent)
SELECT * FROM q;
END IF;
   END;
$$;


ALTER FUNCTION public.getallchilddevicegroup(groupid integer) OWNER TO esia;

--
-- Name: getallchilddevicegroups(integer[]); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.getallchilddevicegroups(groupid integer[]) RETURNS TABLE(group_id integer, group_name character varying, group_parent integer, group_order integer)
    LANGUAGE plpgsql
    AS $$
    BEGIN
IF groupid IS NULL
THEN
RETURN QUERY SELECT device_group_id, device_group_name,device_group_parent,device_group_order
       FROM esia_device_group;
ELSE
	RETURN QUERY
		WITH RECURSIVE q AS (SELECT device_group_id, device_group_name,device_group_parent,device_group_order
			FROM esia_device_group
			WHERE device_group_id = ANY (groupid)
			UNION ALL
			SELECT m.device_group_id, m.device_group_name,m.device_group_parent,m.device_group_order
			FROM esia_device_group m
			JOIN q ON q.device_group_id = m.device_group_parent)
		SELECT * FROM q;
END IF;
   END;
$$;


ALTER FUNCTION public.getallchilddevicegroups(groupid integer[]) OWNER TO esia;

--
-- Name: getdeviceingroup(integer, integer); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.getdeviceingroup(userid integer, groupid integer) RETURNS TABLE(device_id integer)
    LANGUAGE plpgsql
    AS $$
    DECLARE
        v_service integer;
        BEGIN
	IF (groupid IS NOT NULL)
	THEN
		RETURN QUERY SELECT T1.device_id
				FROM esia_device AS T1
				JOIN esia_device_group_in ON T1.device_id = group_in_device_id
				JOIN esia_device_group ON device_group_id= group_in_device_group_id
				WHERE T1.device_id IN ( SELECT * FROM getUserAccessNode(userid)) AND device_delete=FALSE AND device_group_id = groupid ;
	ELSE
		RETURN QUERY SELECT T1.device_id
		FROM esia_device AS T1
		WHERE T1.device_id IN ( SELECT * FROM getUserAccessNode(userid)) AND device_delete=FALSE
			AND NOT EXISTS(SELECT group_in_device_id FROM esia_device_group_in WHERE group_in_device_id = T1.device_id);
	END IF;

   END;
$$;


ALTER FUNCTION public.getdeviceingroup(userid integer, groupid integer) OWNER TO esia;

--
-- Name: getnodesstate(integer); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.getnodesstate(userid integer) RETURNS TABLE(typerror character varying, nbr bigint)
    LANGUAGE plpgsql
    AS $$
DECLARE

BEGIN
--CREATE TEMP TABLE userNode AS SELECT * FROM view_node_state WHERE device_id IN ( SELECT * FROM getUserAccessNode(userid));
RETURN QUERY  WITH userNode AS (
	SELECT device_id,warning,critical,unknown,ok,maintenance,acq
	FROM view_node_state
	WHERE device_id IN ( SELECT * FROM getUserAccessNode(userid))
	)
SELECT 'ok'::varchar,count(*)::integer
FROM userNode
WHERE warning=0 AND critical=0 AND unknown=0 AND ok > 0
UNION
SELECT 'critical',count(*) FROM userNode
WHERE critical>0
UNION
SELECT 'warning',count(*) FROM userNode
WHERE critical=0 AND warning>0
UNION
SELECT 'unknown',count(*) FROM userNode
WHERE critical=0 AND warning=0 AND unknown>0;

END;
$$;


ALTER FUNCTION public.getnodesstate(userid integer) OWNER TO esia;

SET default_tablespace = '';

SET default_table_access_method = heap;

--
-- Name: esia_device_group; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_device_group (
    device_group_id integer NOT NULL,
    device_group_name character varying(35),
    device_group_parent integer,
    device_group_order integer,
    device_group_description text,
    device_group_path text
);


ALTER TABLE public.esia_device_group OWNER TO esia;

--
-- Name: gettopnodegroup(integer); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.gettopnodegroup(group_id integer) RETURNS public.esia_device_group
    LANGUAGE plpgsql
    AS $$
	DECLARE
		v_row esia_device_group%ROWTYPE;
        BEGIN

		SELECT * INTO v_row
		FROM esia_device_group
		WHERE device_group_id = (SELECT device_group_parent FROM esia_device_group WHERE device_group_id=group_id);
		--Si le group_id est null c'est qu'on a demandé le top Group d'un group de top niveau (lvl1)
		IF v_row.device_group_id IS NULL
		THEN
			SELECT * INTO v_row FROM esia_device_group WHERE device_group_id=group_id;
			RETURN v_row;
		ELSE
			--Si le groupe parrent n'est pas null on est pas sur le top groupe
			IF v_row.device_group_parent IS NOT NULL
			THEN
				SELECT * INTO v_row  FROM getTopNodeGroup(v_row.device_group_id);
			END IF;
			RETURN v_row;
		END IF;


   END;
$$;


ALTER FUNCTION public.gettopnodegroup(group_id integer) OWNER TO esia;

--
-- Name: getuseraccessnode(integer); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.getuseraccessnode(userid integer) RETURNS SETOF integer
    LANGUAGE plpgsql
    AS $$
    DECLARE
        v_service integer;
        BEGIN
        RETURN QUERY SELECT access_device_id
            FROM esia_group_device_access
            JOIN esia_groups ON access_group_id = group_id
            JOIN esia_memberof ON member_group_id = group_id
            WHERE member_user_id=userid;
   END;
$$;


ALTER FUNCTION public.getuseraccessnode(userid integer) OWNER TO esia;

--
-- Name: link_message_user(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.link_message_user() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
    ForKEY RECORD;
BEGIN
	--FOR ForKEY IN SELECT tc.table_name, kcu.column_name, ccu.table_name AS foreign_table_name,ccu.column_name AS foreign_column_name 
	--	FROM information_schema.table_constraints  tc 
	--	JOIN information_schema.key_column_usage kcu ON tc.constraint_name = kcu.constraint_name 
	--	JOIN information_schema.constraint_column_usage ccu ON ccu.constraint_name = tc.constraint_name WHERE constraint_type = 'FOREIGN KEY' AND ccu.table_name='esia_communication_message_link_user' LOOP
	--EXECUTE 'DELETE FROM '|| ForKEY.table_name || ' WHERE '||ForKEY.column_name||' = '||OLD.message_id;
	--LOOP
	
	EXECUTE 'DELETE FROM esia_communication_message_link_user  WHERE message_link_id = '||OLD.message_id;
	--END LOOP;
	RETURN OLD;
END;
$$;


ALTER FUNCTION public.link_message_user() OWNER TO esia;

--
-- Name: trigger_channels(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_channels() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
	v_var  character varying[];
BEGIN
	raise notice 'TYPE %',TG_OP;
	IF TG_OP = 'UPDATE' THEN
		raise notice 'UPDATE % TO % ',OLD.channel_group_link,NEW.channel_group_link;
		IF NEW.channel_group_link <> OLD.channel_group_link OR NEW.channel_group_link IS NULL
		THEN
			raise notice 'DELETE OLD ';
			DELETE FROM rss_user_subscribe WHERE subscribe_channel_id= NEW.channel_id;
		END IF;
	END IF;
	IF NEW.channel_group_link <> OLD.channel_group_link OR  NEW.channel_group_link IS NOT NULL
	THEN
		IF NEW.channel_group_link = 'RSS_GROUP_ALL'
		THEN
			INSERT INTO rss_user_subscribe(subscribe_user_id, subscribe_channel_id)
			SELECT user_id,NEW.channel_id
			FROM esia_users
			ON CONFLICT ON CONSTRAINT "subscribe_primaryKey" DO NOTHING;
		ELSE
			v_var = string_to_array(NEW.channel_group_link,',');
			INSERT INTO rss_user_subscribe(subscribe_user_id, subscribe_channel_id)
			SELECT user_id,NEW.channel_id
			FROM esia_users
			JOIN esia_memberof ON user_id = member_user_id
			JOIN esia_groups ON group_id = member_group_id
			WHERE group_name = ANY(v_var)
			ON CONFLICT ON CONSTRAINT "subscribe_primaryKey" DO NOTHING;
		END IF;
	END IF;
    RETURN NEW;
END;
$$;


ALTER FUNCTION public.trigger_channels() OWNER TO esia;

--
-- Name: trigger_delete_device(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_delete_device() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
    ForKEY RECORD;
BEGIN
	FOR ForKEY IN SELECT tc.table_name, kcu.column_name, ccu.table_name AS foreign_table_name,ccu.column_name AS foreign_column_name 
		FROM information_schema.table_constraints  tc 
		JOIN information_schema.key_column_usage kcu ON tc.constraint_name = kcu.constraint_name 
		JOIN information_schema.constraint_column_usage ccu ON ccu.constraint_name = tc.constraint_name WHERE constraint_type = 'FOREIGN KEY' AND ccu.table_name='esia_device' LOOP

	EXECUTE 'DELETE FROM '|| ForKEY.table_name || ' WHERE '||ForKEY.column_name||' = '||OLD.device_id;
	END LOOP;
	RETURN OLD;
END;
$$;


ALTER FUNCTION public.trigger_delete_device() OWNER TO esia;

--
-- Name: trigger_delete_service(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_delete_service() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
BEGIN
    DELETE FROM esia_error_happen WHERE happen_service_id = OLD.service_id;
    DELETE FROM esia_service_response_program WHERE response_service_id = OLD.service_id;
    DELETE FROM esia_maintenance_configuration WHERE maintenance_conf_service_id = OLD.service_id;
    DELETE FROM esia_maintenance_planned WHERE planned_service_id = OLD.service_id;
	DELETE FROM esia_service_rra_file WHERE rra_service_id = OLD.service_id;
    RETURN OLD;
END;
$$;


ALTER FUNCTION public.trigger_delete_service() OWNER TO esia;

--
-- Name: trigger_delete_user(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_delete_user() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
    ForKEY RECORD;
BEGIN
	FOR ForKEY IN SELECT tc.table_name, kcu.column_name, ccu.table_name AS foreign_table_name,ccu.column_name AS foreign_column_name 
		FROM information_schema.table_constraints  tc 
		JOIN information_schema.key_column_usage kcu ON tc.constraint_name = kcu.constraint_name 
		JOIN information_schema.constraint_column_usage ccu ON ccu.constraint_name = tc.constraint_name WHERE constraint_type = 'FOREIGN KEY' AND ccu.table_name='esia_users' LOOP

	EXECUTE 'DELETE FROM '|| ForKEY.table_name || ' WHERE '||ForKEY.column_name||' = '||OLD.user_id;
	END LOOP;
	RETURN OLD;
END;
$$;


ALTER FUNCTION public.trigger_delete_user() OWNER TO esia;

--
-- Name: trigger_insert_update_memberof(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_insert_update_memberof() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
BEGIN
	INSERT INTO rss_user_subscribe(subscribe_user_id, subscribe_channel_id)
	SELECT NEW.member_user_id,channel_id
	FROM rss_channel AS chan
	WHERE EXISTS (
		SELECT *
		FROM unnest(string_to_array(chan.channel_group_link,',')) AS elem(usergroup)
		WHERE usergroup IN (
			SELECT group_name
			FROM esia_groups
			WHERE group_id= NEW.member_group_id
			)
	)
	UNION
	SELECT NEW.member_user_id,channel_id
	FROM rss_channel
	WHERE channel_group_link='RSS_GROUP_ALL'
	ON CONFLICT ON CONSTRAINT "subscribe_primaryKey" DO NOTHING;
    RETURN NEW;
END;
$$;


ALTER FUNCTION public.trigger_insert_update_memberof() OWNER TO esia;

--
-- Name: trigger_notify_servicecron(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_notify_servicecron() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
 DECLARE v_payload JSON;
BEGIN
	IF TG_OP = 'INSERT' THEN
		v_payload := json_build_object('TYPE','SERVICE','COMMAND','ADD','ID',NEW.cron_id);
	ELSEIF TG_OP = 'UPDATE' THEN
		v_payload := json_build_object('TYPE','SERVICE','COMMAND','UPDATE','ID',NEW.cron_id);
	END IF;
	PERFORM pg_notify('esiacronchannel',v_payload::text);
    RETURN NEW;
END;
$$;


ALTER FUNCTION public.trigger_notify_servicecron() OWNER TO esia;

--
-- Name: trigger_update_group_path(); Type: FUNCTION; Schema: public; Owner: esia
--

CREATE FUNCTION public.trigger_update_group_path() RETURNS trigger
    LANGUAGE plpgsql
    AS $$
DECLARE
	v_var integer;
BEGIN
	--Si on est sur un top group
	v_var=0;
	IF NEW.device_group_parent IS NULL THEN
		UPDATE esia_device_group SET device_group_path = NEW.device_group_name WHERE device_group_id = NEW.device_group_id;
		v_var=1;
	END IF;
	IF v_var = 0 THEN
		--Si on est sur un top group
		WITH RECURSIVE paths (device_group_id,name,path) AS (
			SELECT master.device_group_id, master.device_group_name, ARRAY[master.device_group_path]::character varying[] AS path
       		FROM esia_device_group master
			WHERE device_group_id= (SELECT device_group_id FROM gettopnodegroup(NEW.device_group_id))
			UNION ALL
			SELECT sec.device_group_id, sec.device_group_name, paths.path || ARRAY[sec.device_group_name]
	 		FROM esia_device_group sec
	 		JOIN paths ON paths.device_group_id = sec.device_group_parent AND sec.device_group_name != ALL(paths.path)
		)
		UPDATE esia_device_group SET device_group_path=p.fullpath
		FROM (select device_group_id,array_to_string(path,' > ')as fullpath from paths) as p
		WHERE p.device_group_id = esia_device_group.device_group_id;
	END IF;
	IF TG_OP = 'UPDATE' THEN
		IF NEW.device_group_parent <> OLD.device_group_parent AND OLD.device_group_parent IS NOT NULL  THEN
		--Si on est sur un top group
		WITH RECURSIVE paths (device_group_id,name,path) AS (
			SELECT master.device_group_id, master.device_group_name, ARRAY[master.device_group_path]::character varying[] AS path
       		FROM esia_device_group master
			WHERE device_group_id= (SELECT device_group_id FROM gettopnodegroup(OLD.device_group_parent))
			UNION ALL
			SELECT sec.device_group_id, sec.device_group_name, paths.path || ARRAY[sec.device_group_name]
	 		FROM esia_device_group sec
	 		JOIN paths ON paths.device_group_id = sec.device_group_parent AND sec.device_group_name != ALL(paths.path)
		)
		UPDATE esia_device_group SET device_group_path=p.fullpath
		FROM (select device_group_id,array_to_string(path,' > ')as fullpath from paths) as p
		WHERE p.device_group_id = esia_device_group.device_group_id;
		END IF;
	END IF;
    RETURN NEW;
END;
$$;


ALTER FUNCTION public.trigger_update_group_path() OWNER TO esia;

--
-- Name: dashboard; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.dashboard (
    dashboard_id integer NOT NULL,
    dashboard_name character varying,
    dashboard_controller character varying,
    dashboard_description character varying,
    dashboard_link character varying(200),
    dashboard_default boolean DEFAULT false
);


ALTER TABLE public.dashboard OWNER TO esia;

--
-- Name: dashboard_dashboard_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.dashboard_dashboard_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dashboard_dashboard_id_seq OWNER TO esia;

--
-- Name: dashboard_dashboard_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.dashboard_dashboard_id_seq OWNED BY public.dashboard.dashboard_id;


--
-- Name: dashboard_tile; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.dashboard_tile (
    tile_id integer NOT NULL,
    tile_position integer,
    tile_height integer,
    tile_width double precision,
    tile_dashboard_id integer
);


ALTER TABLE public.dashboard_tile OWNER TO esia;

--
-- Name: dashboard_tile_tile_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.dashboard_tile_tile_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dashboard_tile_tile_id_seq OWNER TO esia;

--
-- Name: dashboard_tile_tile_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.dashboard_tile_tile_id_seq OWNED BY public.dashboard_tile.tile_id;


--
-- Name: dashboard_user_access; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.dashboard_user_access (
    board_user_id integer NOT NULL,
    board_dashboard_id integer NOT NULL,
    board_user_share boolean DEFAULT false,
    board_user_access character varying(10) DEFAULT 'WRITE'::character varying NOT NULL
);


ALTER TABLE public.dashboard_user_access OWNER TO esia;

--
-- Name: dashboard_view; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.dashboard_view (
    view_id integer NOT NULL,
    view_name character varying(30),
    view_file_path character varying,
    view_during integer,
    view_default_height integer,
    view_default_width integer,
    view_default_option json,
    view_group_id integer
);


ALTER TABLE public.dashboard_view OWNER TO esia;

--
-- Name: dashboard_view_view_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.dashboard_view_view_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dashboard_view_view_id_seq OWNER TO esia;

--
-- Name: dashboard_view_view_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.dashboard_view_view_id_seq OWNED BY public.dashboard_view.view_id;


--
-- Name: dashboard_widget; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.dashboard_widget (
    widget_id integer NOT NULL,
    widget_name character varying,
    widget_view_during integer,
    widget_position integer,
    widget_option json,
    widget_tile_id integer,
    widget_view_id integer,
    widget_description character varying(150)
);


ALTER TABLE public.dashboard_widget OWNER TO esia;

--
-- Name: dashboard_widget_group; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.dashboard_widget_group (
    widget_group_id integer NOT NULL,
    widget_group_name character varying(30),
    widget_group_order integer DEFAULT 99
);


ALTER TABLE public.dashboard_widget_group OWNER TO esia;

--
-- Name: dashboard_widget_group_widget_group_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.dashboard_widget_group_widget_group_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dashboard_widget_group_widget_group_id_seq OWNER TO esia;

--
-- Name: dashboard_widget_group_widget_group_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.dashboard_widget_group_widget_group_id_seq OWNED BY public.dashboard_widget_group.widget_group_id;


--
-- Name: dashboard_widget_widget_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.dashboard_widget_widget_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.dashboard_widget_widget_id_seq OWNER TO esia;

--
-- Name: dashboard_widget_widget_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.dashboard_widget_widget_id_seq OWNED BY public.dashboard_widget.widget_id;


--
-- Name: device_tab; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.device_tab (
    tab_id integer NOT NULL,
    tab_name character varying(50),
    tab_default_options json,
    tab_url character varying(200),
    tab_order smallint DEFAULT 99,
    tab_web_id integer
);


ALTER TABLE public.device_tab OWNER TO esia;

--
-- Name: device_tab_linked; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.device_tab_linked (
    tab_linked_device_id bigint NOT NULL,
    tab_linked_tab_id integer NOT NULL,
    tab_linked_order smallint,
    tab_linked_option json
);


ALTER TABLE public.device_tab_linked OWNER TO esia;

--
-- Name: device_tab_tab_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.device_tab_tab_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.device_tab_tab_id_seq OWNER TO esia;

--
-- Name: device_tab_tab_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.device_tab_tab_id_seq OWNED BY public.device_tab.tab_id;


--
-- Name: esia_communication_message; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_communication_message (
    message_id integer NOT NULL,
    type_message character varying NOT NULL,
    getcontent character varying NOT NULL,
    title character varying,
    message character varying,
    start_date timestamp without time zone,
    end_date timestamp without time zone,
    admin_message boolean,
    CONSTRAINT esia_communication_message_getcontent_check CHECK ((((getcontent)::text = 'rss'::text) OR ((getcontent)::text = 'html'::text) OR ((getcontent)::text = 'db'::text))),
    CONSTRAINT esia_communication_message_type_messahge_check CHECK ((((type_message)::text = 'report'::text) OR ((type_message)::text = 'info'::text) OR ((type_message)::text = 'tips'::text) OR ((type_message)::text = 'modal'::text) OR ((type_message)::text = 'license'::text)))
);


ALTER TABLE public.esia_communication_message OWNER TO esia;

--
-- Name: esia_communication_message_link_user; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_communication_message_link_user (
    message_link_id integer NOT NULL,
    user_link_id integer NOT NULL,
    show_link boolean
);


ALTER TABLE public.esia_communication_message_link_user OWNER TO esia;

--
-- Name: esia_communication_message_message_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_communication_message_message_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_communication_message_message_id_seq OWNER TO esia;

--
-- Name: esia_communication_message_message_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_communication_message_message_id_seq OWNED BY public.esia_communication_message.message_id;


--
-- Name: esia_configuration; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_configuration (
    config_key character varying(30) NOT NULL,
    config_value text,
    config_description text,
    config_type character varying(20) NOT NULL,
    config_visibility boolean DEFAULT true NOT NULL
);


ALTER TABLE public.esia_configuration OWNER TO esia;

--
-- Name: esia_consumption; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_consumption (
    conso_device_id bigint NOT NULL,
    conso_node integer DEFAULT 0,
    conso_last_update timestamp without time zone DEFAULT now(),
    conso_max_node integer
);


ALTER TABLE public.esia_consumption OWNER TO esia;

--
-- Name: esia_cron_error; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_cron_error (
    cron_error_id bigint NOT NULL,
    cron_id bigint NOT NULL,
    cron_date timestamp without time zone NOT NULL,
    cron_logfile character varying(200),
    cron_delete boolean DEFAULT false
);


ALTER TABLE public.esia_cron_error OWNER TO esia;

--
-- Name: esia_cron_error_cron_error_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_cron_error_cron_error_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_cron_error_cron_error_id_seq OWNER TO esia;

--
-- Name: esia_cron_error_cron_error_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_cron_error_cron_error_id_seq OWNED BY public.esia_cron_error.cron_error_id;


--
-- Name: esia_cron_plugin; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_cron_plugin (
    cron_id bigint NOT NULL,
    cron_plugin_id integer NOT NULL,
    cron_interval interval,
    cron_status character varying(10),
    cron_last_exec timestamp with time zone,
    cron_next_exec timestamp with time zone,
    cron_log_file character varying(200),
    cron_retries integer DEFAULT 1 NOT NULL,
    cron_retries_interval interval,
    cron_parameter character varying(200),
    cron_delete boolean DEFAULT false
);


ALTER TABLE public.esia_cron_plugin OWNER TO esia;

--
-- Name: esia_cron_plugin_cron_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_cron_plugin_cron_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_cron_plugin_cron_id_seq OWNER TO esia;

--
-- Name: esia_cron_plugin_cron_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_cron_plugin_cron_id_seq OWNED BY public.esia_cron_plugin.cron_id;


--
-- Name: esia_device; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_device (
    device_id integer NOT NULL,
    device_name character varying(150) NOT NULL,
    device_type character varying(16),
    device_group_name character varying,
    device_description text,
    device_ip_addr bigint DEFAULT 0,
    device_username character varying(20),
    device_passwd bytea,
    device_snmp_community character varying(20),
    device_snmp_version integer,
    device_protocol_connection character varying(10),
    device_snmp3_user character varying,
    device_snmp3_sec_lvl character varying(20),
    device_snmp3_auth_protocole character varying(10),
    device_snmp3_auth_key character varying,
    device_ip_version smallint DEFAULT 4 NOT NULL,
    device_snmp3_priv_protocol character varying(10),
    device_snmp3_priv_passphrase character varying(50),
    device_snmp_timer integer DEFAULT 1000000,
    device_delete boolean DEFAULT false,
    device_ip_address inet,
    device_snmp_port integer DEFAULT 161
);


ALTER TABLE public.esia_device OWNER TO esia;

--
-- Name: esia_device_advance_config; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_device_advance_config (
    adv_id integer NOT NULL,
    adv_device_id bigint,
    adv_config_type_id integer,
    adv_param json,
    adv_authfile_crypt character varying(250),
    adv_plugin_keyword_value character varying(250)
);


ALTER TABLE public.esia_device_advance_config OWNER TO esia;

--
-- Name: esia_device_advance_config_adv_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_device_advance_config_adv_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_device_advance_config_adv_id_seq OWNER TO esia;

--
-- Name: esia_device_advance_config_adv_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_device_advance_config_adv_id_seq OWNED BY public.esia_device_advance_config.adv_id;


--
-- Name: esia_device_advance_config_type; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_device_advance_config_type (
    device_config_type_id integer NOT NULL,
    device_config_type_name character varying(40),
    device_config_type_controller character varying(250),
    device_config_type_default_parameter json,
    device_config_type_webplugin_id integer,
    device_config_type_nodetab character varying(150)
);


ALTER TABLE public.esia_device_advance_config_type OWNER TO esia;

--
-- Name: esia_device_advance_config_type_device_config_type_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_device_advance_config_type_device_config_type_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_device_advance_config_type_device_config_type_id_seq OWNER TO esia;

--
-- Name: esia_device_advance_config_type_device_config_type_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_device_advance_config_type_device_config_type_id_seq OWNED BY public.esia_device_advance_config_type.device_config_type_id;


--
-- Name: esia_device_device_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_device_device_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_device_device_id_seq OWNER TO esia;

--
-- Name: esia_device_device_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_device_device_id_seq OWNED BY public.esia_device.device_id;


--
-- Name: esia_device_group_device_group_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_device_group_device_group_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_device_group_device_group_id_seq OWNER TO esia;

--
-- Name: esia_device_group_device_group_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_device_group_device_group_id_seq OWNED BY public.esia_device_group.device_group_id;


--
-- Name: esia_device_group_in; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_device_group_in (
    group_in_device_id integer NOT NULL,
    group_in_device_group_id integer NOT NULL
);


ALTER TABLE public.esia_device_group_in OWNER TO esia;

--
-- Name: esia_device_type; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_device_type (
    type_id integer NOT NULL,
    type_name character varying(30) NOT NULL,
    type_web_id integer,
    type_admin_page character varying(100),
    type_img_ok_path character varying(100),
    type_img_warning_path character varying(100),
    type_img_critical_path character varying(100),
    type_img_unknown_path character varying(100),
    type_list_page character varying
);


ALTER TABLE public.esia_device_type OWNER TO esia;

--
-- Name: esia_device_type_type_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_device_type_type_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_device_type_type_id_seq OWNER TO esia;

--
-- Name: esia_device_type_type_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_device_type_type_id_seq OWNED BY public.esia_device_type.type_id;


--
-- Name: esia_error_happen; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_error_happen (
    happen_error_id integer NOT NULL,
    happen_device_id integer NOT NULL,
    happen_interface_id integer,
    happen_message character varying,
    happen_acquitte smallint DEFAULT 0 NOT NULL,
    happen_value integer DEFAULT 0,
    happen_service_id bigint,
    happen_time_acquitte timestamp without time zone,
    happen_timestamp timestamp without time zone DEFAULT now(),
    happen_id bigint NOT NULL,
    happen_user_id integer
);


ALTER TABLE public.esia_error_happen OWNER TO esia;

--
-- Name: esia_error_happen_happen_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_error_happen_happen_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_error_happen_happen_id_seq OWNER TO esia;

--
-- Name: esia_error_happen_happen_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_error_happen_happen_id_seq OWNED BY public.esia_error_happen.happen_id;


--
-- Name: esia_errors; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_errors (
    error_id integer NOT NULL,
    error_name character varying NOT NULL,
    error_type character varying DEFAULT 'critical'::character varying NOT NULL
);


ALTER TABLE public.esia_errors OWNER TO esia;

--
-- Name: esia_errors_error_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_errors_error_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_errors_error_id_seq OWNER TO esia;

--
-- Name: esia_errors_error_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_errors_error_id_seq OWNED BY public.esia_errors.error_id;


--
-- Name: esia_group_device_access; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_group_device_access (
    access_group_id integer NOT NULL,
    access_device_id integer NOT NULL
);


ALTER TABLE public.esia_group_device_access OWNER TO esia;

--
-- Name: esia_groups; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_groups (
    group_id integer NOT NULL,
    group_name character varying NOT NULL,
    group_description character varying(200)
);


ALTER TABLE public.esia_groups OWNER TO esia;

--
-- Name: esia_groups_group_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_groups_group_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_groups_group_id_seq OWNER TO esia;

--
-- Name: esia_groups_group_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_groups_group_id_seq OWNED BY public.esia_groups.group_id;


--
-- Name: esia_interface; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_interface (
    interface_id integer NOT NULL,
    interface_name character varying(100) NOT NULL,
    interface_type character varying(30),
    interface_desc character varying(60),
    interface_plug character varying(20),
    interface_toip integer DEFAULT '-1'::integer,
    interface_lastchange_username character varying(30),
    interface_down_time bigint DEFAULT 0,
    interface_critical smallint DEFAULT 0,
    interface_ifindex integer DEFAULT '-1'::integer,
    interface_device_id integer,
    interface_lastchange_time timestamp without time zone,
    interface_mac_addr macaddr
);


ALTER TABLE public.esia_interface OWNER TO esia;

--
-- Name: esia_interface_connected_to; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_interface_connected_to (
    conint_switch_interface_id integer NOT NULL,
    conint_interface_id2 integer NOT NULL,
    conint_protocol character varying(10) DEFAULT 'MACTABLE'::character varying
);


ALTER TABLE public.esia_interface_connected_to OWNER TO esia;

--
-- Name: esia_interface_interface_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_interface_interface_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_interface_interface_id_seq OWNER TO esia;

--
-- Name: esia_interface_interface_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_interface_interface_id_seq OWNED BY public.esia_interface.interface_id;


--
-- Name: esia_interface_node_connected; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_interface_node_connected (
    connected_description character varying(200),
    connected_id bigint NOT NULL,
    connected_mac macaddr,
    connected_protocol character varying(10),
    connected_interface_id bigint
);


ALTER TABLE public.esia_interface_node_connected OWNER TO esia;

--
-- Name: esia_interface_node_connected_connected_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_interface_node_connected_connected_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_interface_node_connected_connected_id_seq OWNER TO esia;

--
-- Name: esia_interface_node_connected_connected_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_interface_node_connected_connected_id_seq OWNED BY public.esia_interface_node_connected.connected_id;


--
-- Name: esia_mac; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_mac (
    mac character varying NOT NULL
);


ALTER TABLE public.esia_mac OWNER TO esia;

--
-- Name: esia_mac_address; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_mac_address (
    mac_address character varying(50) NOT NULL,
    mac_interface_id integer NOT NULL,
    mac_vlan_id integer NOT NULL,
    mac_ip_address character varying(54),
    mac_time_connection integer,
    mac_time_out_connection bigint DEFAULT 0,
    mac_dot1dport integer
);


ALTER TABLE public.esia_mac_address OWNER TO esia;

--
-- Name: esia_mac_address_vendor; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_mac_address_vendor (
    vendor_mac_address character(6) NOT NULL,
    vendor_company character varying(100),
    vendor_mac macaddr
);


ALTER TABLE public.esia_mac_address_vendor OWNER TO esia;

--
-- Name: esia_maintenance_configuration; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_maintenance_configuration (
    maintenance_conf_id integer NOT NULL,
    maintenance_conf_type character varying,
    maintenance_conf_start_date timestamp without time zone,
    maintenance_conf_create timestamp without time zone DEFAULT now(),
    maintenance_conf_service_id integer,
    maintenance_conf_during interval,
    maintenance_conf_recurring interval,
    maintenance_conf_during_firsterror interval
);


ALTER TABLE public.esia_maintenance_configuration OWNER TO esia;

--
-- Name: esia_maintenance_configuration_maintenance_conf_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_maintenance_configuration_maintenance_conf_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_maintenance_configuration_maintenance_conf_id_seq OWNER TO esia;

--
-- Name: esia_maintenance_configuration_maintenance_conf_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_maintenance_configuration_maintenance_conf_id_seq OWNED BY public.esia_maintenance_configuration.maintenance_conf_id;


--
-- Name: esia_maintenance_planned; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_maintenance_planned (
    planned_id bigint NOT NULL,
    planned_type character varying(15),
    planned_start timestamp with time zone,
    planned_end timestamp with time zone,
    planned_during integer DEFAULT 0,
    planned_service_id integer
);


ALTER TABLE public.esia_maintenance_planned OWNER TO esia;

--
-- Name: esia_maintenance_planned_planned_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_maintenance_planned_planned_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_maintenance_planned_planned_id_seq OWNER TO esia;

--
-- Name: esia_maintenance_planned_planned_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_maintenance_planned_planned_id_seq OWNED BY public.esia_maintenance_planned.planned_id;


--
-- Name: esia_manufacturer; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_manufacturer (
    manufacturer_oid bigint NOT NULL,
    manufacturer_name character varying,
    manufacturer_logo character varying(100)
);


ALTER TABLE public.esia_manufacturer OWNER TO esia;

--
-- Name: esia_php_class; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_php_class (
    class_id bigint NOT NULL,
    class_name character varying,
    class_type character varying,
    class_path character varying,
    class_manufacturer_oid bigint
);


ALTER TABLE public.esia_php_class OWNER TO esia;

--
-- Name: esia_manufacturer_class_class_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_manufacturer_class_class_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_manufacturer_class_class_id_seq OWNER TO esia;

--
-- Name: esia_manufacturer_class_class_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_manufacturer_class_class_id_seq OWNED BY public.esia_php_class.class_id;


--
-- Name: esia_memberof; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_memberof (
    member_group_id integer NOT NULL,
    member_user_id integer NOT NULL
);


ALTER TABLE public.esia_memberof OWNER TO esia;

--
-- Name: esia_model; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_model (
    model_id bigint NOT NULL,
    model_name character varying(50),
    model_manufacturer_oid bigint,
    model_html_tpl character varying
);


ALTER TABLE public.esia_model OWNER TO esia;

--
-- Name: esia_model_model_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_model_model_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_model_model_id_seq OWNER TO esia;

--
-- Name: esia_model_model_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_model_model_id_seq OWNED BY public.esia_model.model_id;


--
-- Name: esia_os; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_os (
    os_id integer NOT NULL,
    os_name character varying(30),
    os_version character varying(20) NOT NULL,
    os_template_path character varying(200),
    os_manufacturer_oid bigint
);


ALTER TABLE public.esia_os OWNER TO esia;

--
-- Name: esia_os_os_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_os_os_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_os_os_id_seq OWNER TO esia;

--
-- Name: esia_os_os_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_os_os_id_seq OWNED BY public.esia_os.os_id;


--
-- Name: esia_plugins; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_plugins (
    plugin_id integer NOT NULL,
    plugin_name character varying(50),
    plugin_path character varying(150),
    plugin_type character varying(100),
    plugin_trap_oid character varying(150),
    plugin_default_parameter character varying(250),
    plugin_default_interval_time integer DEFAULT 300000,
    plugin_description text,
    plugin_delete boolean DEFAULT false NOT NULL,
    plugin_priority smallint DEFAULT 7,
    plugin_html_template character varying(250),
    plugin_manufacturer character varying(30),
    plugin_tag character varying(50),
    plugin_cron_type character varying(10),
    plugin_cron_only_existing boolean
);


ALTER TABLE public.esia_plugins OWNER TO esia;

--
-- Name: esia_plugin_plugin_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_plugin_plugin_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_plugin_plugin_id_seq OWNER TO esia;

--
-- Name: esia_plugin_plugin_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_plugin_plugin_id_seq OWNED BY public.esia_plugins.plugin_id;


--
-- Name: esia_plugin_react_program; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_plugin_react_program (
    react_plugin_id integer NOT NULL,
    react_program_id integer NOT NULL,
    react_parameter character varying(150),
    react_ok boolean,
    react_warning boolean,
    react_critical boolean,
    react_unknown boolean
);


ALTER TABLE public.esia_plugin_react_program OWNER TO esia;

--
-- Name: esia_privilege_grantedto; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_privilege_grantedto (
    grant_group_id integer NOT NULL,
    grant_privilege_id integer NOT NULL,
    grant_acces integer DEFAULT 0
);


ALTER TABLE public.esia_privilege_grantedto OWNER TO esia;

--
-- Name: esia_privileges; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_privileges (
    privilege_id integer NOT NULL,
    privilege_name character varying NOT NULL,
    privilege_site smallint DEFAULT 0 NOT NULL,
    privilege_description character varying(200)
);


ALTER TABLE public.esia_privileges OWNER TO esia;

--
-- Name: esia_privilege_privilege_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_privilege_privilege_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_privilege_privilege_id_seq OWNER TO esia;

--
-- Name: esia_privilege_privilege_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_privilege_privilege_id_seq OWNED BY public.esia_privileges.privilege_id;


--
-- Name: esia_program; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_program (
    program_id integer NOT NULL,
    program_name character varying(30),
    program_path character varying(150),
    program_default_parameter character varying(100),
    program_html_template character varying(300)
);


ALTER TABLE public.esia_program OWNER TO esia;

--
-- Name: esia_program_program_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_program_program_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_program_program_id_seq OWNER TO esia;

--
-- Name: esia_program_program_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_program_program_id_seq OWNED BY public.esia_program.program_id;


--
-- Name: esia_rdp; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_rdp (
    rdp_id integer NOT NULL,
    rdp_device_id integer NOT NULL,
    rdp_software character varying(50),
    rdp_soft_id character varying(100),
    CONSTRAINT "Name_Logicel_Pmd" CHECK (((rdp_software)::text = ANY ((ARRAY['logmein'::character varying, 'teamviewer'::character varying, 'none'::character varying])::text[])))
);


ALTER TABLE public.esia_rdp OWNER TO esia;

--
-- Name: esia_rdp_rdp_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_rdp_rdp_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_rdp_rdp_id_seq OWNER TO esia;

--
-- Name: esia_rdp_rdp_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_rdp_rdp_id_seq OWNED BY public.esia_rdp.rdp_id;


--
-- Name: esia_search; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_search (
    search_keyword character varying(20) NOT NULL,
    search_description character varying(200),
    search_file character varying(100),
    search_function character varying(40),
    search_name character varying(40)
);


ALTER TABLE public.esia_search OWNER TO esia;

--
-- Name: esia_service_pattern_included; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_service_pattern_included (
    included_id bigint NOT NULL,
    included_name character varying(50),
    included_pattern_id integer NOT NULL,
    included_plugin_id integer,
    included_parameter character varying(300),
    included_interval integer,
    included_program_id integer,
    included_program_parameter character varying(250),
    included_unknown boolean,
    included_warning boolean,
    included_critical boolean,
    included_priority smallint DEFAULT 7,
    included_ok boolean
);


ALTER TABLE public.esia_service_pattern_included OWNER TO esia;

--
-- Name: esia_service_patterns; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_service_patterns (
    pattern_id integer NOT NULL,
    pattern_name character varying(50),
    pattern_description character varying(300)
);


ALTER TABLE public.esia_service_patterns OWNER TO esia;

--
-- Name: esia_service_pending_execute; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_service_pending_execute (
    pending_service_id bigint NOT NULL,
    pending_type character varying(10) DEFAULT 'ADD'::character varying
);


ALTER TABLE public.esia_service_pending_execute OWNER TO esia;

--
-- Name: esia_service_response_program; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_service_response_program (
    response_service_id bigint NOT NULL,
    response_program_id integer NOT NULL,
    response_parameter character varying(150),
    response_ok boolean,
    response_warning boolean,
    response_critical boolean,
    response_unknown boolean
);


ALTER TABLE public.esia_service_response_program OWNER TO esia;

--
-- Name: esia_service_rra_file; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_service_rra_file (
    rra_file_id bigint NOT NULL,
    rra_service_id bigint,
    rra_graph_num integer,
    rra_file_path character varying(300)
);


ALTER TABLE public.esia_service_rra_file OWNER TO esia;

--
-- Name: esia_service_rra_file_rra_file_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_service_rra_file_rra_file_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_service_rra_file_rra_file_id_seq OWNER TO esia;

--
-- Name: esia_service_rra_file_rra_file_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_service_rra_file_rra_file_id_seq OWNED BY public.esia_service_rra_file.rra_file_id;


--
-- Name: esia_services; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_services (
    service_plugin_id bigint NOT NULL,
    service_device_id bigint NOT NULL,
    service_interval_time bigint,
    service_parameter character varying(250),
    service_previous_state integer DEFAULT 0,
    service_last_exec_time timestamp without time zone,
    service_last_stdout text,
    service_id bigint NOT NULL,
    service_name character varying(100),
    service_time_added timestamp without time zone DEFAULT now(),
    service_delete boolean DEFAULT false NOT NULL,
    service_included_id integer,
    service_priority smallint DEFAULT 7
);


ALTER TABLE public.esia_services OWNER TO esia;

--
-- Name: esia_services_service_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_services_service_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_services_service_id_seq OWNER TO esia;

--
-- Name: esia_services_service_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_services_service_id_seq OWNED BY public.esia_services.service_id;


--
-- Name: esia_switch; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_switch (
    switch_device_id bigint NOT NULL,
    switch_model_id bigint,
    switch_class_id bigint,
    switch_os_id bigint
);


ALTER TABLE public.esia_switch OWNER TO esia;

--
-- Name: esia_template_template_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_template_template_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_template_template_id_seq OWNER TO esia;

--
-- Name: esia_template_template_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_template_template_id_seq OWNED BY public.esia_service_patterns.pattern_id;


--
-- Name: esia_theme; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_theme (
    theme_id integer NOT NULL,
    theme_name character varying(20)
);


ALTER TABLE public.esia_theme OWNER TO esia;

--
-- Name: esia_theme_theme_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_theme_theme_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_theme_theme_id_seq OWNER TO esia;

--
-- Name: esia_theme_theme_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_theme_theme_id_seq OWNED BY public.esia_theme.theme_id;


--
-- Name: esia_tpl_service_sertpl_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_tpl_service_sertpl_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_tpl_service_sertpl_id_seq OWNER TO esia;

--
-- Name: esia_tpl_service_sertpl_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_tpl_service_sertpl_id_seq OWNED BY public.esia_service_pattern_included.included_id;


--
-- Name: esia_users; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_users (
    user_id integer NOT NULL,
    user_name character varying NOT NULL,
    user_password character varying(40),
    user_lang character varying(5),
    user_inscription_time timestamp without time zone DEFAULT now(),
    user_email character varying(50),
    user_change_password smallint DEFAULT 0,
    user_theme_id integer,
    user_tuto_view integer,
    visibility_user boolean DEFAULT true,
    home_page character varying DEFAULT '/index.php'::character varying,
    user_source character varying DEFAULT 'LOCAL'::character varying
);


ALTER TABLE public.esia_users OWNER TO esia;

--
-- Name: esia_users_user_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_users_user_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_users_user_id_seq OWNER TO esia;

--
-- Name: esia_users_user_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_users_user_id_seq OWNED BY public.esia_users.user_id;


--
-- Name: esia_web_plugin; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.esia_web_plugin (
    web_id integer NOT NULL,
    web_name character varying(30) NOT NULL,
    web_dir_lang character varying(250),
    web_php_menu character varying(250),
    web_index_page character varying(30),
    web_admin_page character varying(100),
    web_activate boolean DEFAULT true,
    web_tpl_method character varying(20),
    web_plugin_apache_path character varying(20) NOT NULL,
    web_page_control character varying(50),
    web_show_main_menu boolean DEFAULT true,
    web_icon character varying(200)
);


ALTER TABLE public.esia_web_plugin OWNER TO esia;

--
-- Name: esia_web_plugin_web_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.esia_web_plugin_web_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.esia_web_plugin_web_id_seq OWNER TO esia;

--
-- Name: esia_web_plugin_web_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.esia_web_plugin_web_id_seq OWNED BY public.esia_web_plugin.web_id;


--
-- Name: gdpr_register_register_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.gdpr_register_register_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.gdpr_register_register_id_seq OWNER TO esia;

--
-- Name: gdpr_register; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.gdpr_register (
    register_id integer DEFAULT nextval('public.gdpr_register_register_id_seq'::regclass) NOT NULL,
    register_user_name character varying(100),
    register_date timestamp without time zone DEFAULT now(),
    register_action character varying(30),
    register_type character varying(100),
    register_data text,
    register_finality text
);


ALTER TABLE public.gdpr_register OWNER TO esia;

--
-- Name: gesa_connected; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.gesa_connected (
    connected_gesa_id bigint NOT NULL,
    connected_device_id bigint NOT NULL,
    CONSTRAINT connected_gesa_id_not_equal_device_id CHECK ((connected_gesa_id <> connected_device_id))
);


ALTER TABLE public.gesa_connected OWNER TO esia;

--
-- Name: gesa_info; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.gesa_info (
    gesa_device_id bigint NOT NULL,
    gesa_max_slot integer,
    gesa_sn bigint,
    gesa_version_base character varying(10),
    gesa_version_nrpe character varying(10),
    gesa_version_plugin character varying(10),
    gesa_version_web character varying(10),
    gesa_repo_global character varying(100),
    gesa_repo_gesa character varying(100),
    gesa_last_update timestamp without time zone,
    gesa_rsbp character varying(10),
    gesa_local_ip inet,
    gesa_macadress macaddr
);


ALTER TABLE public.gesa_info OWNER TO esia;

--
-- Name: rrd_plugins; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rrd_plugins (
    rrd_plugin_id integer NOT NULL,
    rrd_type character varying(20),
    rrd_regex character varying(100),
    rrd_time integer,
    rrd_id integer NOT NULL,
    rrd_name character varying(30),
    rrd_min bigint,
    rrd_max bigint,
    rrd_vertical_label character varying(20),
    rrd_graph_type character varying(20),
    rrd_graph_color character varying(10),
    rrd_regex_threshold character varying(50),
    rrd_graph_num smallint,
    rrd_calc character varying
);


ALTER TABLE public.rrd_plugins OWNER TO esia;

--
-- Name: rrd_plugins_rrd_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.rrd_plugins_rrd_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rrd_plugins_rrd_id_seq OWNER TO esia;

--
-- Name: rrd_plugins_rrd_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.rrd_plugins_rrd_id_seq OWNED BY public.rrd_plugins.rrd_id;


--
-- Name: rss_category; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rss_category (
    category_id integer NOT NULL,
    category_name character varying(30),
    category_description text
);


ALTER TABLE public.rss_category OWNER TO esia;

--
-- Name: rss_category_category_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.rss_category_category_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rss_category_category_id_seq OWNER TO esia;

--
-- Name: rss_category_category_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.rss_category_category_id_seq OWNED BY public.rss_category.category_id;


--
-- Name: rss_channel; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rss_channel (
    channel_id integer NOT NULL,
    channel_name character varying(50) NOT NULL,
    channel_description text,
    channel_language character varying(10) DEFAULT 'fr-fr'::character varying,
    channel_last_publication timestamp with time zone,
    channel_image text,
    channel_type character varying(10) DEFAULT 'LOCAL'::character varying,
    channel_url text,
    channel_enable boolean DEFAULT true NOT NULL,
    channel_notdelete boolean DEFAULT false,
    channel_group_link text
);


ALTER TABLE public.rss_channel OWNER TO esia;

--
-- Name: rss_channel_channel_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.rss_channel_channel_id_seq
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rss_channel_channel_id_seq OWNER TO esia;

--
-- Name: rss_channel_channel_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.rss_channel_channel_id_seq OWNED BY public.rss_channel.channel_id;


--
-- Name: rss_link_channel_to_category; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rss_link_channel_to_category (
    link_channel_id integer NOT NULL,
    link_category_id integer NOT NULL
);


ALTER TABLE public.rss_link_channel_to_category OWNER TO esia;

--
-- Name: rss_message; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rss_message (
    message_id bigint NOT NULL,
    message_title text NOT NULL,
    message_description text,
    message_link text,
    message_author character varying(50) NOT NULL,
    message_image text,
    message_channel_id integer,
    message_sign character varying(64),
    message_date_publish timestamp with time zone,
    message_group_view integer[],
    message_categories character varying(100)
);


ALTER TABLE public.rss_message OWNER TO esia;

--
-- Name: rss_message_message_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.rss_message_message_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.rss_message_message_id_seq OWNER TO esia;

--
-- Name: rss_message_message_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.rss_message_message_id_seq OWNED BY public.rss_message.message_id;


--
-- Name: rss_user_read; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rss_user_read (
    read_user_id integer NOT NULL,
    read_message_id bigint NOT NULL
);


ALTER TABLE public.rss_user_read OWNER TO esia;

--
-- Name: rss_user_subscribe; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.rss_user_subscribe (
    subscribe_user_id integer NOT NULL,
    subscribe_channel_id integer NOT NULL
);


ALTER TABLE public.rss_user_subscribe OWNER TO esia;

--
-- Name: sms_send; Type: TABLE; Schema: public; Owner: esia
--

CREATE TABLE public.sms_send (
    sms_id integer NOT NULL,
    sms_to text,
    sms_message text,
    sms_date timestamp without time zone
);


ALTER TABLE public.sms_send OWNER TO esia;

--
-- Name: sms_send_sms_id_seq; Type: SEQUENCE; Schema: public; Owner: esia
--

CREATE SEQUENCE public.sms_send_sms_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.sms_send_sms_id_seq OWNER TO esia;

--
-- Name: sms_send_sms_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: esia
--

ALTER SEQUENCE public.sms_send_sms_id_seq OWNED BY public.sms_send.sms_id;


--
-- Name: view_actual_error; Type: VIEW; Schema: public; Owner: esia
--

CREATE VIEW public.view_actual_error AS
 SELECT esia_error_happen.happen_id,
    esia_error_happen.happen_device_id,
    esia_error_happen.happen_service_id,
    esia_errors.error_type
   FROM (public.esia_error_happen
     JOIN public.esia_errors ON ((esia_errors.error_id = esia_error_happen.happen_error_id)))
  WHERE (esia_error_happen.happen_acquitte = 0);


ALTER TABLE public.view_actual_error OWNER TO esia;

--
-- Name: view_node_state; Type: VIEW; Schema: public; Owner: esia
--

CREATE VIEW public.view_node_state AS
 SELECT t2.device_id,
    COALESCE(t2.warning, (0)::bigint) AS warning,
    COALESCE(t2.critical, (0)::bigint) AS critical,
    COALESCE(t2.unknown, (0)::bigint) AS unknown,
    COALESCE(t2.ok, (0)::bigint) AS ok,
    COALESCE(t2.maintenance, (0)::bigint) AS maintenance,
    COALESCE(t2.acq, (0)::bigint) AS acq
   FROM public.crosstab('SELECT device_id::integer as rowid,error_type::text as attribute,nbr::bigint
		FROM	(SELECT device_id,error_type,count (*) as nbr
					FROM esia_device
					JOIN esia_error_happen ON happen_device_id = device_id
				    JOIN esia_errors ON error_id = happen_error_id
					WHERE device_delete=false AND happen_acquitte=0
					GROUP BY device_id,error_type
			UNION
			SELECT service_device_id,''ok'',count(*) AS nbr
			FROM esia_services
			WHERE service_previous_state = 0
				AND service_delete = false
			GROUP BY service_device_id
			UNION
			SELECT service_device_id,''acq'',count(*) AS nbr
			FROM esia_services
			WHERE service_last_stdout IS NOT NULL AND service_previous_state <>0 AND service_id NOT IN (SELECT happen_service_id FROM view_actual_error)
				AND service_delete = false
			GROUP BY service_device_id

		) AS T2
		ORDER BY 1,2'::text, 'SELECT * FROM (SELECT error_type FROM  esia_errors UNION SELECT ''ok'' UNION SELECT ''acq'') AS T1 ORDER BY 1;'::text) t2(device_id integer, acq bigint, critical bigint, maintenance bigint, ok bigint, unknown bigint, warning bigint);


ALTER TABLE public.view_node_state OWNER TO esia;

--
-- Name: dashboard dashboard_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard ALTER COLUMN dashboard_id SET DEFAULT nextval('public.dashboard_dashboard_id_seq'::regclass);


--
-- Name: dashboard_tile tile_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_tile ALTER COLUMN tile_id SET DEFAULT nextval('public.dashboard_tile_tile_id_seq'::regclass);


--
-- Name: dashboard_view view_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_view ALTER COLUMN view_id SET DEFAULT nextval('public.dashboard_view_view_id_seq'::regclass);


--
-- Name: dashboard_widget widget_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_widget ALTER COLUMN widget_id SET DEFAULT nextval('public.dashboard_widget_widget_id_seq'::regclass);


--
-- Name: dashboard_widget_group widget_group_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_widget_group ALTER COLUMN widget_group_id SET DEFAULT nextval('public.dashboard_widget_group_widget_group_id_seq'::regclass);


--
-- Name: device_tab tab_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab ALTER COLUMN tab_id SET DEFAULT nextval('public.device_tab_tab_id_seq'::regclass);


--
-- Name: esia_communication_message message_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_communication_message ALTER COLUMN message_id SET DEFAULT nextval('public.esia_communication_message_message_id_seq'::regclass);


--
-- Name: esia_cron_error cron_error_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_cron_error ALTER COLUMN cron_error_id SET DEFAULT nextval('public.esia_cron_error_cron_error_id_seq'::regclass);


--
-- Name: esia_cron_plugin cron_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_cron_plugin ALTER COLUMN cron_id SET DEFAULT nextval('public.esia_cron_plugin_cron_id_seq'::regclass);


--
-- Name: esia_device device_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device ALTER COLUMN device_id SET DEFAULT nextval('public.esia_device_device_id_seq'::regclass);


--
-- Name: esia_device_advance_config adv_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config ALTER COLUMN adv_id SET DEFAULT nextval('public.esia_device_advance_config_adv_id_seq'::regclass);


--
-- Name: esia_device_advance_config_type device_config_type_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config_type ALTER COLUMN device_config_type_id SET DEFAULT nextval('public.esia_device_advance_config_type_device_config_type_id_seq'::regclass);


--
-- Name: esia_device_group device_group_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_group ALTER COLUMN device_group_id SET DEFAULT nextval('public.esia_device_group_device_group_id_seq'::regclass);


--
-- Name: esia_device_type type_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_type ALTER COLUMN type_id SET DEFAULT nextval('public.esia_device_type_type_id_seq'::regclass);


--
-- Name: esia_error_happen happen_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen ALTER COLUMN happen_id SET DEFAULT nextval('public.esia_error_happen_happen_id_seq'::regclass);


--
-- Name: esia_errors error_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_errors ALTER COLUMN error_id SET DEFAULT nextval('public.esia_errors_error_id_seq'::regclass);


--
-- Name: esia_groups group_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_groups ALTER COLUMN group_id SET DEFAULT nextval('public.esia_groups_group_id_seq'::regclass);


--
-- Name: esia_interface interface_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface ALTER COLUMN interface_id SET DEFAULT nextval('public.esia_interface_interface_id_seq'::regclass);


--
-- Name: esia_interface_node_connected connected_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface_node_connected ALTER COLUMN connected_id SET DEFAULT nextval('public.esia_interface_node_connected_connected_id_seq'::regclass);


--
-- Name: esia_maintenance_configuration maintenance_conf_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_maintenance_configuration ALTER COLUMN maintenance_conf_id SET DEFAULT nextval('public.esia_maintenance_configuration_maintenance_conf_id_seq'::regclass);


--
-- Name: esia_maintenance_planned planned_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_maintenance_planned ALTER COLUMN planned_id SET DEFAULT nextval('public.esia_maintenance_planned_planned_id_seq'::regclass);


--
-- Name: esia_model model_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_model ALTER COLUMN model_id SET DEFAULT nextval('public.esia_model_model_id_seq'::regclass);


--
-- Name: esia_os os_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_os ALTER COLUMN os_id SET DEFAULT nextval('public.esia_os_os_id_seq'::regclass);


--
-- Name: esia_php_class class_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_php_class ALTER COLUMN class_id SET DEFAULT nextval('public.esia_manufacturer_class_class_id_seq'::regclass);


--
-- Name: esia_plugins plugin_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_plugins ALTER COLUMN plugin_id SET DEFAULT nextval('public.esia_plugin_plugin_id_seq'::regclass);


--
-- Name: esia_privileges privilege_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_privileges ALTER COLUMN privilege_id SET DEFAULT nextval('public.esia_privilege_privilege_id_seq'::regclass);


--
-- Name: esia_program program_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_program ALTER COLUMN program_id SET DEFAULT nextval('public.esia_program_program_id_seq'::regclass);


--
-- Name: esia_rdp rdp_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_rdp ALTER COLUMN rdp_id SET DEFAULT nextval('public.esia_rdp_rdp_id_seq'::regclass);


--
-- Name: esia_service_pattern_included included_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_pattern_included ALTER COLUMN included_id SET DEFAULT nextval('public.esia_tpl_service_sertpl_id_seq'::regclass);


--
-- Name: esia_service_patterns pattern_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_patterns ALTER COLUMN pattern_id SET DEFAULT nextval('public.esia_template_template_id_seq'::regclass);


--
-- Name: esia_service_rra_file rra_file_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_rra_file ALTER COLUMN rra_file_id SET DEFAULT nextval('public.esia_service_rra_file_rra_file_id_seq'::regclass);


--
-- Name: esia_services service_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_services ALTER COLUMN service_id SET DEFAULT nextval('public.esia_services_service_id_seq'::regclass);


--
-- Name: esia_theme theme_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_theme ALTER COLUMN theme_id SET DEFAULT nextval('public.esia_theme_theme_id_seq'::regclass);


--
-- Name: esia_users user_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_users ALTER COLUMN user_id SET DEFAULT nextval('public.esia_users_user_id_seq'::regclass);


--
-- Name: esia_web_plugin web_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_web_plugin ALTER COLUMN web_id SET DEFAULT nextval('public.esia_web_plugin_web_id_seq'::regclass);


--
-- Name: rrd_plugins rrd_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rrd_plugins ALTER COLUMN rrd_id SET DEFAULT nextval('public.rrd_plugins_rrd_id_seq'::regclass);


--
-- Name: rss_category category_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_category ALTER COLUMN category_id SET DEFAULT nextval('public.rss_category_category_id_seq'::regclass);


--
-- Name: rss_channel channel_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_channel ALTER COLUMN channel_id SET DEFAULT nextval('public.rss_channel_channel_id_seq'::regclass);


--
-- Name: rss_message message_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_message ALTER COLUMN message_id SET DEFAULT nextval('public.rss_message_message_id_seq'::regclass);


--
-- Name: sms_send sms_id; Type: DEFAULT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.sms_send ALTER COLUMN sms_id SET DEFAULT nextval('public.sms_send_sms_id_seq'::regclass);


--
-- Data for Name: dashboard; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.dashboard (dashboard_id, dashboard_name, dashboard_controller, dashboard_description, dashboard_link, dashboard_default) FROM stdin;
1	LANG_DASHBOARD_HOME_NEW	/dashboard/default.php?id=:dashid	\N	\N	t
2	LANG_DASHBOARD_HOME_NEW_OLD	/dashboard/default.php?id=:dashid	\N	\N	t
\.


--
-- Data for Name: dashboard_tile; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.dashboard_tile (tile_id, tile_position, tile_height, tile_width, tile_dashboard_id) FROM stdin;
1	0	280	19.395322304621	1
2	1	280	18.539646320593	1
3	2	280	20.136908157444	1
4	3	280	17.683970336566	1
5	4	280	18.026240730177	1
6	5	650	41.414717626925	1
7	6	650	23.103251568739	1
8	7	650	30.347974900171	1
9	0	250	20.788302606484	2
10	1	250	22.314049586777	2
11	2	250	18.054672600127	2
12	3	250	32.422123331214	2
13	4	550	62.301335028608	2
14	5	550	32.48569612206	2
\.


--
-- Data for Name: dashboard_user_access; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.dashboard_user_access (board_user_id, board_dashboard_id, board_user_share, board_user_access) FROM stdin;
\.


--
-- Data for Name: dashboard_view; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.dashboard_view (view_id, view_name, view_file_path, view_during, view_default_height, view_default_width, view_default_option, view_group_id) FROM stdin;
3	DASHBOARD_ALARM	/dashboard/view_alert.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px; color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"}}	\N
11	DASHBOARD_MESSAGE	/dashboard/view_com_msg.php?id=:widgetid	30	100	300	{"message_id":""}	\N
12	DASHBOARD_IMAGE	/dashboard/view_wig_img.php?id=:widgetid	180	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"imageHeader":"","path":""}	\N
13	DASHBOARD_API_ALARM	/dashboard/view_api_alert.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px; color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"}}	\N
9	DASHBOARD_APPLICATIVE_VIEW	/dashboard/view_applicative.php?id=:widgetid	30	100	300	{"link":[],"service":""}	3
2	DASHBOARD_GRAPHIC_SE	/dashboard/view_graph.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"service":"","start":"-30m"}	4
14	DASHBOARD_NODE_TYPE_TAB	/dashboard/view_tab_devicetype.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","warning":1,"critical":1,"unknown":1,"statusonly":0,"order":[]}	5
1	DASHBOARD_NODE_STATE	/dashboard/view_4state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","nodeid":[]}	1
6	DASHBOARD_SERV_STATE	/dashboard/view_4state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","service":[]}	1
7	DASHBOARD_PLUGIN_STATE	/dashboard/view_4state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","pluginid":[]}	1
10	DASHBOARD_GROUPNODE_STATE	/dashboard/view_4state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","groupnodeid":[]}	1
8	DASHBOARD_VIEW_VALUE	/dashboard/view_graph.php?id=:widgetid&type=value	30	100	300	{"type": "value", "start": "-30m", "title": {"style": "text-align: center; font-size:15px;color:#575759;"}, "unity": "", "service": "", "description": {"style": "text-align: center; margin:auto;font-size:12px;"}}	4
15	DASHBOARD_TOP_NODE_ERROR	/dashboard/view_top_node_error.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px; color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"interval":1,"limit":10,"type":"warning,critique"}	7
16	DASHBOARD_FLAG_NODE_STATE	/dashboard/view_flag_state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"abr","nodeid":[]}	6
17	DASHBOARD_FLAG_GROUPNODE_STATE	/dashboard/view_flag_state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"abr","groupnodeid":[]}	6
18	DASHBOARD_FLAG_PLUGIN_STATE	/dashboard/view_flag_state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"abr","pluginid":[]}	6
19	DASHBOARD_FLAG_SERVICE_STATE	/dashboard/view_flag_state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"abr","service":[]}	6
20	DASHBOARD_GRAPHIC_SERV_GAUGE	/dashboard/view_graph.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"service":"","type":"gauge"}	4
21	DASHBOARD_TAB_SERVICE	/dashboard/view_tab_service.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"abr","service":[],"column":{"group":null,"message":"on","exec":"on"},"order":[]}	5
22	DASHBOARD_BACKUP_TABLE	/dashboard/view_backup_table.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px; color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"}}	\N
23	DASHBOARD_FAST_LINK	/dashboard/view_link.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px;"}}	\N
24	DASHBOARD_CRON_ERROR	/dashboard/view_cron_error.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"}}	8
25	DASHBOARD_CRON_STATE	/dashboard/view_cron_4state.php?id=:widgetid	30	100	300	{"title":{"style":"text-align: center; font-size:15px;color:#575759;"},"description":{"style":"text-align: center; margin:auto;font-size:12px; color:#575759;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","unknown":"unknown","blink":"none"}	8
\.


--
-- Data for Name: dashboard_widget; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.dashboard_widget (widget_id, widget_name, widget_view_during, widget_position, widget_option, widget_tile_id, widget_view_id, widget_description) FROM stdin;
1	LANG_DASHBOARD_DEFAULT_SERVICE_STATUS	30	\N	{"title":{"style":"text-align: left; font-size:15px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"pie","service":[],"show_unknown":"unknown","show_pending":"pending","blink":"none","service_or_node":"service","all_service":"on","all_node":""}	1	6	LANG_DASHBOARD_DEFAULT_SERVICE_STATUS_DESCR
2	LANG_DASHBOARD_DEFAULT_ESIA_STATUS	30	\N	{"title":{"style":"text-align: left; font-size:15px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","nodeid":["1"],"show_unknown":"unknown","show_pending":"pending","blink":"none","service_or_node":null,"all_service":"","all_node":""}	2	1	LANG_DASHBOARD_DEFAULT_ESIA_STATUS_DESCR
3	LANG_DASHBOARD_CRON	30	\N	{"title":{"style":"text-align: left; font-size:15px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759;"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"ring","unknown":"unknown","blink":"none","interval":"1","line":"2"}	3	25	LANG_DASHBOARD_CRON_DESC
4	LANG_DASHBOARD_RSS_NOREAD_MSG	30	\N	{"message_id":"","title":{"style":"text-align: left; font-size:14px; color:#575759"},"description":{"style":"text-align: left; font-size:12px; color:#575759"},"list_channel":["0"],"list_category":["0"],"filter":"chan","type":null,"affiche_msg":"show_msg_view","showchannel":"on","showmessage":null,"showdate":"on","showauthor":null}	4	11	\N
5	LANG_DASHBOARD_FAST_LINK	30	\N	{"title":{"style":"text-align: left; font-size:14px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"link":[{"text":"\\/administration.php","icon":"node","label":"LANG_DAHSBOARD_LINK_ADMINISTRATION"},{"text":"\\/plugin\\/importation\\/import_control.php","icon":"scanner","label":"LANG_DAHSBOARD_LINK_NETSCAN"},{"text":"https:\\/\\/wiki.esia-sa.com\\/","icon":"wiki","label":"LANG_WIKI"},{"text":"https:\\/\\/wiki.esia-sa.com\\/faq","icon":"faq","label":"LANG_FAQ"},{"text":"https:\\/\\/wiki.esia-sa.com\\/mobile","icon":"esia voyager","label":"LANG_ESIAVOYAGER"},{"text":"\\/rss_view.php","icon":"rss","label":"LANG_RSS_FLOW"}]}	5	23	\N
6	LANG_DASHBOARD_DEFAULT_CURRENT_ALERT	30	\N	{"title":{"style":"text-align: center; font-size:14px; color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"filters":{"node":"","ip":"","description":"","group":"","alert":"!maintenance","service":""},"column":{"group":1,"descr":0,"ip":0,"message":0,"date":0},"hierarch":true}	6	3	\N
7	LANG_DASHBOARD_DEFAULT_TOP	30	\N	{"title":{"style":"text-align: left; font-size:15px; color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"interval":"1","limit":"10","type":null}	7	15	LANG_DASHBOARD_DEFAULT_TOP_DESCR
8	LANG_DASHBOARD_RSS_ESIA_MSG	30	\N	{"message_id":"","title":{"style":"text-align: left; font-size:14px; color:#575759"},"description":{"style":"text-align: left; font-size:12px; color:#575759"},"list_channel":[],"list_category":[],"filter":"cat","type":null,"affiche_msg":"show_tab_view","showchannel":"on","showmessage":null,"showdate":"on","showauthor":null}	8	11	\N
9	LANG_DASHBOARD_DEFAULT_SERVICE_STATUS	30	\N	{"title":{"style":"text-align: left; font-size:15px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"pie","service":[],"show_unknown":"unknown","show_pending":"pending","blink":"none","service_or_node":"service","all_service":"on","all_node":""}	9	6	LANG_DASHBOARD_DEFAULT_SERVICE_STATUS_DESCR
10	LANG_DASHBOARD_DEFAULT_ESIA_STATUS	30	\N	{"title":{"style":"text-align: left; font-size:15px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","nodeid":["1"],"show_unknown":"unknown","show_pending":"pending","blink":"none","service_or_node":null,"all_service":"","all_node":""}	10	1	LANG_DASHBOARD_DEFAULT_ESIA_STATUS_DESCR
11	LANG_ADMIN	30	\N	{"title":{"style":"text-align: left; font-size:14px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"link":[{"text":"\\/administration.php","icon":"node","label":"LANG_ADM_LIST_DEV"},{"text":"\\/administration.php?tab=nodegroups","icon":"nodegroup","label":"LANG_ADM_NODE_GROUP"},{"text":"\\/administration.php?tab=usergroups","icon":"group","label":"LANG_DASHBOARD_USER_GROUP_MAN"},{"text":"\\/administration.php?tab=plugins","icon":"plugin","label":"LANG_PLUGINS"},{"text":"\\/plugin\\/importation\\/import_control.php","icon":"scanner","label":"LANG_DAHSBOARD_LINK_NETSCAN"}]}	11	23	
12	LANG_HELP	30	\N	{"title":{"style":"text-align: left; font-size:14px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"link":[{"text":"https:\\/\\/wiki.esia-sa.com\\/","icon":"wiki","label":"LANG_WIKI"},{"text":"https:\\/\\/wiki.esia-sa.com\\/faq","icon":"faq","label":"LANG_FAQ"},{"text":"https:\\/\\/wiki.esia-sa.com\\/mobile","icon":"esia voyager","label":"LANG_ESIAVOYAGER"},{"text":"\\/rss_view.php","icon":"rss","label":"LANG_RSS_FLOW"},{"text":"https:\\/\\/wiki.esia-sa.com\\/advanced\\/config_ip_unity","icon":"unity","label":"LANG_DASHBOARD_UNITY_CONFIG"}]}	12	23	
13	LANG_DASHBOARD_NODE_TYPE_TAB	30	\N	{"title":{"style":"text-align: left; font-size:14px;color:#575759;"},"description":{"style":"text-align: left; margin:auto;font-size:12px; color:#575759"},"table":{"class":"arraytop","style":"width:60%; height:60%; margin: auto; border-spacing: 10px 10px;"},"td":{"style":"font-size:50px;"},"type":"table","warning":1,"critical":1,"unknown":1,"statusonly":0,"order":[],"nbr_service":1,"nbr_device":1}	13	14	
14	LANG_DASHBOARD_RSS_ESIA_MSG	30	\N	{"message_id":"","title":{"style":"text-align: left; font-size:14px; color:#575759"},"description":{"style":"text-align: left; font-size:12px; color:#575759"},"list_channel":[],"list_category":[],"filter":"cat","type":null,"affiche_msg":"show_tab_view","showchannel":"on","showmessage":null,"showdate":"on","showauthor":null}	14	11	
\.


--
-- Data for Name: dashboard_widget_group; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.dashboard_widget_group (widget_group_id, widget_group_name, widget_group_order) FROM stdin;
6	FLAG	10
7	TOP	55
1	STATE	20
2	PLAN	60
3	APPLICATIVE	50
4	SERVICE	40
5	TAB	30
8	ESIA_CRON	75
\.


--
-- Data for Name: device_tab; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.device_tab (tab_id, tab_name, tab_default_options, tab_url, tab_order, tab_web_id) FROM stdin;
\.


--
-- Data for Name: device_tab_linked; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.device_tab_linked (tab_linked_device_id, tab_linked_tab_id, tab_linked_order, tab_linked_option) FROM stdin;
\.


--
-- Data for Name: esia_communication_message; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_communication_message (message_id, type_message, getcontent, title, message, start_date, end_date, admin_message) FROM stdin;
\.


--
-- Data for Name: esia_communication_message_link_user; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_communication_message_link_user (message_link_id, user_link_id, show_link) FROM stdin;
\.


--
-- Data for Name: esia_configuration; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_configuration (config_key, config_value, config_description, config_type, config_visibility) FROM stdin;
MAIL_USER	\N	\N	varchar	f
MAIL_SMTP_PORT	25	\N	integer	f
MAIL_PASSWORD	\N	\N	varchar	f
MAIL_FROM	\N	\N	varchar	f
SW_PATH_LOGFILE_CONFIGURATION	/usr/local/esia/public_html/data/log/switch/conf/	\N	string	f
SYS_ESIADAEMON_LOG	/var/log/esia/esiaDaemon.log	Chemin d'accès au fichier de log du démon principal	string	f
SYS_ESIATRAPD_LOG	/var/log/esia/esiaTrapd.log	Chemin d'accès au fichier de log du démon esiaTrapd	string	f
SYS_TRAP_LOG	/var/log/esia/esia_trap.log	Chemin d'accès au fichier de trap reçue 	string	f
SYS_TRAP_LOG_ENABLE	1	Log ou non les trap reçues (0 non et 1 oui)	bool	f
MAIL_SMTP	\N	\N	varchar	f
SMS_FORMAT	The %node.name (%node.group) has service %service.name in a %error.type state	\N	text	f
DEFAULT_BANNER	/theme/default-image/esia-ban.png	\N	file	t
ESIA_TYPE	cloud	\N	string	f
MAIL_FORMAT_SUBJECT	( %node.group ) %node.name:%service.name => %error.type state	Sujet du mail. <br /> Voir la description du MAIL_FORMAT_MESSAGE pour les mots-clés	string	t
SMS_HOUR_START	8	Les sms seront envoyés à partir de l'heure indiquée	hour	t
SMS_HOUR_STOP	18	Les sms ne seront plus envoyés à partir de l'heure indiquée	hour	t
SYS_RELOAD	0	\N	integer	f
MAP_CENTER_LATITUDE	50.301270521118425	Latitude du centre de la google map	double	f
MAP_CENTER_LONGITUDE	3.691040128469467	Longitude du centre de la google map	double	f
MAP_ZOOM	8	valeur par défaut du zoom	integer	f
MAP_ZOOM_CHANGED	9	Valeur du zoom où on affiche les bâtiments plutôt que les zones	integer	f
MAP_TYPE	ROADMAP	Type de carte de la Google Map	string	f
DEFAULT_SESSION_TIME	25	Durée par défaut d'une session	smallint	t
SMS_MAX_PER_DAY	9	Nombre maximum de sms envoyés par jour	smallint	t
DEFAULT_THEME	3	id du thème par défaut	integer	f
IMG_DIRECTORY	data/images/	Répertoire d'upload des images (l'utilisateur d'apache doit avoir les droits en écriture)	string	t
IMG_MAX_SIZE	1048576	Taille maximum des images uploadées (en octet)	integer	t
SRV_CPU_LOAD	\\S*LOAD	Nom du plugin permettant de tester le CPU 	string	t
MAP_IMG_DIR	data/images/	WEBPLUGIN MAP: Répertoire d'upload des plans de bâtiment (l'utilisateur d'apache doit avoir les droits en écriture)	string	f
MAP_IMG_TAILLE_MAX	1048576	WEBPLUGIN MAP: Taille maximum des images uploadées (en octets)	integer	t
SYS_TRAP_LOG_ARCHIVE	1	valeur 0 ou 1 (1= archive les fichiers quand ils ont dépassé la taille maximum et 0 supprime l'ancien fichier et recrée un nouveau)	bool	f
SYS_TRAP_LOG_MAX_SIZE	20000000	Taille maximum du fichier de log (en octets)	integer	f
DEFAULT_BANNER_FONTCOLOR	#575759	Couleur de fond du logo	color	t
SRV_RAM_LOAD	\\S*(MEM|RAM)$	Nom du plugin permettant de tester la RAM	string	t
LICENCE	\N	\N	text	f
LICENCE_RETEST	0	\N	text	f
SRV_HDD_LOAD	\\S*STORAGE	Nom du plugin permettant de tester l'espace disque	string	t
SRV_INT_LOAD	\\S*INTERFACE%	Nom du plugin permettant de tester les interfaces réseaux	string	t
RSS_PURGE_MESSAGE	60	Purge automatique des message RSS	integer	t
LICENCE_EGIDE	\N	\N	text	f
ESIA_URL	localhost	FQDN du serveur ESIA	string	t
MAIL_FORMAT_MESSAGE	<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">\n<html>\n%service.name=>%error.type state\n<h1>Node: %node.name</h1>\n<table>\n<tr>\n<td bgcolor="%color">%service.name</td>\n<td>%error.message</td>\n</tr>\n<tr>\n<td>Date:</td><td>%error.time</td>\n</tr>\n</table>\n\n<h2>Information: %node.name</h2>\n<table style="background-color:#D8D8D8">\n<tr>\n<td>IP: </td><td>%node.ip </td>\n</tr>\n<tr>\n<td>Description: </td><td>%node.description </td>\n</tr>\n<tr>\n<td>Group: </td><td>%node.group </td>\n</tr>\n<tr>\n<td>Group description: </td><td>%node.groupdesc </td>\n</tr>\n<tr>\n<td>Service: </td><td>%service.name</td>\n</tr>\n\n</table>\n<img style="width:72px; height:50px;" src="http://www.esia-sa.com/esia-logo_200px.png" />Power By Esia \n</html>	Corps du mail au format HTML.  <br /> Les mots-clés sont automatiquement remplacés. Voici la liste des mots-clés:  <br />\n%node.name => nom du noeud <br />\n%node.ip => adresse IP du noeud <br />\n%node.description => description du noeud <br />\n%node.group => groupe du noeud <br />\n%node.groupdesc => description du groupe du noeud\n<br />\n%service.name => nom du service <br />\n%service.parameter => paramètres d'exécution du service <br />\n<br />\n%error.message => Message d'erreur <br />\n%error.starttime => date de début de l'erreur <br />\n%error.time => date fin de l'erreur<br /><br />\n%error.type => type de l'erreur <br />\n%color => couleur de l'erreur suivant son niveau (gris,jaune,rouge) <br />	text	t
\.


--
-- Data for Name: esia_consumption; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_consumption (conso_device_id, conso_node, conso_last_update, conso_max_node) FROM stdin;
\.


--
-- Data for Name: esia_cron_error; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_cron_error (cron_error_id, cron_id, cron_date, cron_logfile, cron_delete) FROM stdin;
\.


--
-- Data for Name: esia_cron_plugin; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_cron_plugin (cron_id, cron_plugin_id, cron_interval, cron_status, cron_last_exec, cron_next_exec, cron_log_file, cron_retries, cron_retries_interval, cron_parameter, cron_delete) FROM stdin;
1	1	00:30:00	\N	\N	2024-07-17 10:08:44.193947+00	rss_$SERVICEID_$DATE.log	1	00:01:00	\N	f
\.


--
-- Data for Name: esia_device; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_device (device_id, device_name, device_type, device_group_name, device_description, device_ip_addr, device_username, device_passwd, device_snmp_community, device_snmp_version, device_protocol_connection, device_snmp3_user, device_snmp3_sec_lvl, device_snmp3_auth_protocole, device_snmp3_auth_key, device_ip_version, device_snmp3_priv_protocol, device_snmp3_priv_passphrase, device_snmp_timer, device_delete, device_ip_address, device_snmp_port) FROM stdin;
\.


--
-- Data for Name: esia_device_advance_config; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_device_advance_config (adv_id, adv_device_id, adv_config_type_id, adv_param, adv_authfile_crypt, adv_plugin_keyword_value) FROM stdin;
\.


--
-- Data for Name: esia_device_advance_config_type; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_device_advance_config_type (device_config_type_id, device_config_type_name, device_config_type_controller, device_config_type_default_parameter, device_config_type_webplugin_id, device_config_type_nodetab) FROM stdin;
1	RDP	/adv_node/adv_rdp.php	{"type":"teamviewer","id":""}	\N	\N
2	PASSWORD	/adv_node/adv_password.php	\N	\N	\N
\.


--
-- Data for Name: esia_device_group; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_device_group (device_group_id, device_group_name, device_group_parent, device_group_order, device_group_description, device_group_path) FROM stdin;
\.


--
-- Data for Name: esia_device_group_in; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_device_group_in (group_in_device_id, group_in_device_group_id) FROM stdin;
\.


--
-- Data for Name: esia_device_type; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_device_type (type_id, type_name, type_web_id, type_admin_page, type_img_ok_path, type_img_warning_path, type_img_critical_path, type_img_unknown_path, type_list_page) FROM stdin;
3	none	\N	\N	\N	\N	\N	\N	\N
7	UPS	\N	\N	\N	\N	\N	\N	\N
8	PDU	\N	\N	\N	\N	\N	\N	\N
10	imprimante	\N	\N	\N	\N	\N	\N	\N
17	evision	\N	\N	\N	\N	\N	\N	\N
18	application	\N	\N	\N	\N	\N	\N	\N
\.


--
-- Data for Name: esia_error_happen; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_error_happen (happen_error_id, happen_device_id, happen_interface_id, happen_message, happen_acquitte, happen_value, happen_service_id, happen_time_acquitte, happen_timestamp, happen_id, happen_user_id) FROM stdin;
\.


--
-- Data for Name: esia_errors; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_errors (error_id, error_name, error_type) FROM stdin;
1	default_critical_error	critical
2	default_warning_error	warning
3	default_unknow_error	unknown
10	temperature warning	warning
11	temperature critical	critical
12	temperature notFunctioning	warning
13	err_disable	critical
14	interface input error	warning
15	critical interface input error	critical
16	default_maintenance	maintenance
\.


--
-- Data for Name: esia_group_device_access; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_group_device_access (access_group_id, access_device_id) FROM stdin;
\.


--
-- Data for Name: esia_groups; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_groups (group_id, group_name, group_description) FROM stdin;
1	root	groupe des administrateurs
7	default_read_access	Acces par defaut en lecture
\.


--
-- Data for Name: esia_interface; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_interface (interface_id, interface_name, interface_type, interface_desc, interface_plug, interface_toip, interface_lastchange_username, interface_down_time, interface_critical, interface_ifindex, interface_device_id, interface_lastchange_time, interface_mac_addr) FROM stdin;
\.


--
-- Data for Name: esia_interface_connected_to; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_interface_connected_to (conint_switch_interface_id, conint_interface_id2, conint_protocol) FROM stdin;
\.


--
-- Data for Name: esia_interface_node_connected; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_interface_node_connected (connected_description, connected_id, connected_mac, connected_protocol, connected_interface_id) FROM stdin;
\.


--
-- Data for Name: esia_mac; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_mac (mac) FROM stdin;
\.


--
-- Data for Name: esia_mac_address; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_mac_address (mac_address, mac_interface_id, mac_vlan_id, mac_ip_address, mac_time_connection, mac_time_out_connection, mac_dot1dport) FROM stdin;
\.


--
-- Data for Name: esia_mac_address_vendor; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_mac_address_vendor (vendor_mac_address, vendor_company, vendor_mac) FROM stdin;
000000	XEROX CORPORATION	\N
000001	XEROX CORPORATION	\N
000002	XEROX CORPORATION	\N
000003	XEROX CORPORATION	\N
000004	XEROX CORPORATION	\N
000005	XEROX CORPORATION	\N
000006	XEROX CORPORATION	\N
000007	XEROX CORPORATION	\N
000008	XEROX CORPORATION	\N
000009	XEROX CORPORATION	\N
00000a	OMRON TATEISI ELECTRONICS CO.	\N
00000b	MATRIX CORPORATION	\N
00000c	CISCO SYSTEMS, INC.	\N
00000d	FIBRONICS LTD.	\N
00000e	FUJITSU LIMITED	\N
00000f	NEXT, INC.	\N
000010	SYTEK INC.	\N
000011	NORMEREL SYSTEMES	\N
000012	INFORMATION TECHNOLOGY LIMITED	\N
000013	CAMEX	\N
000014	NETRONIX	\N
000015	DATAPOINT CORPORATION	\N
000016	DU PONT PIXEL SYSTEMS     .	\N
000017	TEKELEC	\N
000018	WEBSTER COMPUTER CORPORATION	\N
000019	APPLIED DYNAMICS INTERNATIONAL	\N
00001a	ADVANCED MICRO DEVICES	\N
00001b	NOVELL INC.	\N
00001c	BELL TECHNOLOGIES	\N
00001d	CABLETRON SYSTEMS, INC.	\N
00001e	TELSIST INDUSTRIA ELECTRONICA	\N
00001f	Telco Systems, Inc.	\N
000020	DATAINDUSTRIER DIAB AB	\N
000021	SUREMAN COMP. & COMMUN. CORP.	\N
000022	VISUAL TECHNOLOGY INC.	\N
000023	ABB INDUSTRIAL SYSTEMS AB	\N
000024	CONNECT AS	\N
000025	RAMTEK CORP.	\N
000026	SHA-KEN CO., LTD.	\N
000027	JAPAN RADIO COMPANY	\N
000028	PRODIGY SYSTEMS CORPORATION	\N
000029	IMC NETWORKS CORP.	\N
00002a	TRW - SEDD/INP	\N
00002b	CRISP AUTOMATION, INC	\N
00002c	AUTOTOTE LIMITED	\N
00002d	CHROMATICS INC	\N
00002e	SOCIETE EVIRA	\N
00002f	TIMEPLEX INC.	\N
000030	VG LABORATORY SYSTEMS LTD	\N
000031	QPSX COMMUNICATIONS PTY LTD	\N
000032	Marconi plc	\N
000033	EGAN MACHINERY COMPANY	\N
000034	NETWORK RESOURCES CORPORATION	\N
000035	SPECTRAGRAPHICS CORPORATION	\N
000036	ATARI CORPORATION	\N
000037	OXFORD METRICS LIMITED	\N
000038	CSS LABS	\N
000039	TOSHIBA CORPORATION	\N
00003a	CHYRON CORPORATION	\N
00003b	i Controls, Inc.	\N
00003c	AUSPEX SYSTEMS INC.	\N
00003d	UNISYS	\N
00003e	SIMPACT	\N
00003f	SYNTREX, INC.	\N
000040	APPLICON, INC.	\N
000041	ICE CORPORATION	\N
000042	METIER MANAGEMENT SYSTEMS LTD.	\N
000043	MICRO TECHNOLOGY	\N
000044	CASTELLE CORPORATION	\N
000045	FORD AEROSPACE & COMM. CORP.	\N
000046	OLIVETTI NORTH AMERICA	\N
000047	NICOLET INSTRUMENTS CORP.	\N
000048	SEIKO EPSON CORPORATION	\N
000049	APRICOT COMPUTERS, LTD	\N
00004a	ADC CODENOLL TECHNOLOGY CORP.	\N
00004b	ICL DATA OY	\N
00004c	NEC CORPORATION	\N
00004d	DCI CORPORATION	\N
00004e	AMPEX CORPORATION	\N
00004f	LOGICRAFT, INC.	\N
000050	RADISYS CORPORATION	\N
000051	HOB ELECTRONIC GMBH & CO. KG	\N
000052	Intrusion.com, Inc.	\N
000053	COMPUCORP	\N
000054	MODICON, INC.	\N
000055	COMMISSARIAT A L`ENERGIE ATOM.	\N
000056	DR. B. STRUCK	\N
000057	SCITEX CORPORATION LTD.	\N
000058	RACORE COMPUTER PRODUCTS INC.	\N
000059	HELLIGE GMBH	\N
00005a	SysKonnect GmbH	\N
00005b	ELTEC ELEKTRONIK AG	\N
00005c	TELEMATICS INTERNATIONAL INC.	\N
00005d	CS TELECOM	\N
00005e	USC INFORMATION SCIENCES INST	\N
00005f	SUMITOMO ELECTRIC IND., LTD.	\N
000060	KONTRON ELEKTRONIK GMBH	\N
000061	GATEWAY COMMUNICATIONS	\N
000062	BULL HN INFORMATION SYSTEMS	\N
000063	BARCO CONTROL ROOMS GMBH	\N
000064	YOKOGAWA DIGITAL COMPUTER CORP	\N
000065	Network General Corporation	\N
000066	TALARIS SYSTEMS, INC.	\N
000067	SOFT * RITE, INC.	\N
000068	ROSEMOUNT CONTROLS	\N
000069	CONCORD COMMUNICATIONS INC	\N
00006a	COMPUTER CONSOLES INC.	\N
00006b	SILICON GRAPHICS INC./MIPS	\N
00006c	PRIVATE	\N
00006d	CRAY COMMUNICATIONS, LTD.	\N
00006e	ARTISOFT, INC.	\N
00006f	Madge Ltd.	\N
000070	HCL LIMITED	\N
000071	ADRA SYSTEMS INC.	\N
000072	MINIWARE TECHNOLOGY	\N
000073	SIECOR CORPORATION	\N
000074	RICOH COMPANY LTD.	\N
000075	Nortel Networks	\N
000076	ABEKAS VIDEO SYSTEM	\N
000077	INTERPHASE CORPORATION	\N
000078	LABTAM LIMITED	\N
000079	NETWORTH INCORPORATED	\N
00007a	DANA COMPUTER INC.	\N
00007b	RESEARCH MACHINES	\N
00007c	AMPERE INCORPORATED	\N
00007d	SUN MICROSYSTEMS, INC.	\N
00007e	CLUSTRIX CORPORATION	\N
00007f	LINOTYPE-HELL AG	\N
000080	CRAY COMMUNICATIONS A/S	\N
000081	BAY NETWORKS	\N
000082	LECTRA SYSTEMES SA	\N
000083	TADPOLE TECHNOLOGY PLC	\N
000084	SUPERNET	\N
000085	CANON INC.	\N
000086	MEGAHERTZ CORPORATION	\N
000087	HITACHI, LTD.	\N
000088	Brocade Communications Systems, Inc.	\N
000089	CAYMAN SYSTEMS INC.	\N
00008a	DATAHOUSE INFORMATION SYSTEMS	\N
00008b	INFOTRON	\N
00008c	Alloy Computer Products (Australia) Pty Ltd	\N
00008d	Cryptek Inc.	\N
00008e	SOLBOURNE COMPUTER, INC.	\N
00008f	Raytheon	\N
000090	MICROCOM	\N
000091	ANRITSU CORPORATION	\N
000092	COGENT DATA TECHNOLOGIES	\N
000093	PROTEON INC.	\N
000094	ASANTE TECHNOLOGIES	\N
000095	SONY TEKTRONIX CORP.	\N
000096	MARCONI ELECTRONICS LTD.	\N
000097	EMC Corporation	\N
000098	CROSSCOMM CORPORATION	\N
000099	MTX, INC.	\N
00009a	RC COMPUTER A/S	\N
00009b	INFORMATION INTERNATIONAL, INC	\N
00009c	ROLM MIL-SPEC COMPUTERS	\N
00009d	LOCUS COMPUTING CORPORATION	\N
00009e	MARLI S.A.	\N
00009f	AMERISTAR TECHNOLOGIES INC.	\N
0000a0	SANYO Electric Co., Ltd.	\N
0000a1	MARQUETTE ELECTRIC CO.	\N
0000a2	BAY NETWORKS	\N
0000a3	NETWORK APPLICATION TECHNOLOGY	\N
0000a4	ACORN COMPUTERS LIMITED	\N
0000a5	COMPATIBLE SYSTEMS CORP.	\N
0000a6	NETWORK GENERAL CORPORATION	\N
0000a7	NETWORK COMPUTING DEVICES INC.	\N
0000a8	STRATUS COMPUTER INC.	\N
0000a9	NETWORK SYSTEMS CORP.	\N
0000aa	XEROX CORPORATION	\N
0000ab	LOGIC MODELING CORPORATION	\N
0000ac	CONWARE COMPUTER CONSULTING	\N
0000ad	BRUKER INSTRUMENTS INC.	\N
0000ae	DASSAULT ELECTRONIQUE	\N
0000af	NUCLEAR DATA INSTRUMENTATION	\N
0000b0	RND-RAD NETWORK DEVICES	\N
0000b1	ALPHA MICROSYSTEMS INC.	\N
0000b2	TELEVIDEO SYSTEMS, INC.	\N
0000b3	CIMLINC INCORPORATED	\N
0000b4	EDIMAX COMPUTER COMPANY	\N
0000b5	DATABILITY SOFTWARE SYS. INC.	\N
0000b6	MICRO-MATIC RESEARCH	\N
0000b7	DOVE COMPUTER CORPORATION	\N
0000b8	SEIKOSHA CO., LTD.	\N
0000b9	MCDONNELL DOUGLAS COMPUTER SYS	\N
0000ba	SIIG, INC.	\N
0000bb	TRI-DATA	\N
0000bc	Rockwell Automation	\N
0000bd	MITSUBISHI CABLE COMPANY	\N
0000be	THE NTI GROUP	\N
0000bf	SYMMETRIC COMPUTER SYSTEMS	\N
0000c0	WESTERN DIGITAL CORPORATION	\N
0000c1	Madge Ltd.	\N
0000c2	INFORMATION PRESENTATION TECH.	\N
0000c3	HARRIS CORP COMPUTER SYS DIV	\N
0000c4	WATERS DIV. OF MILLIPORE	\N
0000c5	FARALLON COMPUTING/NETOPIA	\N
0000c6	EON SYSTEMS	\N
0000c7	ARIX CORPORATION	\N
0000c8	ALTOS COMPUTER SYSTEMS	\N
0000c9	Emulex Corporation	\N
0000ca	ARRIS International	\N
0000cb	COMPU-SHACK ELECTRONIC GMBH	\N
0000cc	DENSAN CO., LTD.	\N
0000cd	Allied Telesis Labs Ltd	\N
0000ce	MEGADATA CORP.	\N
0000cf	HAYES MICROCOMPUTER PRODUCTS	\N
0000d0	DEVELCON ELECTRONICS LTD.	\N
0000d1	ADAPTEC INCORPORATED	\N
0000d2	SBE, INC.	\N
0000d3	WANG LABORATORIES INC.	\N
0000d4	PURE DATA LTD.	\N
0000d5	MICROGNOSIS INTERNATIONAL	\N
0000d6	PUNCH LINE HOLDING	\N
0000d7	DARTMOUTH COLLEGE	\N
0000d8	NOVELL, INC.	\N
0000d9	NIPPON TELEGRAPH & TELEPHONE	\N
0000da	ATEX	\N
0000db	BRITISH TELECOMMUNICATIONS PLC	\N
0000dc	HAYES MICROCOMPUTER PRODUCTS	\N
0000dd	TCL INCORPORATED	\N
0000de	CETIA	\N
0000df	BELL & HOWELL PUB SYS DIV	\N
0000e0	QUADRAM CORP.	\N
0000e1	GRID SYSTEMS	\N
0000e2	ACER TECHNOLOGIES CORP.	\N
0000e3	INTEGRATED MICRO PRODUCTS LTD	\N
0000e4	IN2 GROUPE INTERTECHNIQUE	\N
0000e5	SIGMEX LTD.	\N
0000e6	APTOR PRODUITS DE COMM INDUST	\N
0000e7	STAR GATE TECHNOLOGIES	\N
0000e8	ACCTON TECHNOLOGY CORP.	\N
0000e9	ISICAD, INC.	\N
0000ea	UPNOD AB	\N
0000eb	MATSUSHITA COMM. IND. CO. LTD.	\N
0000ec	MICROPROCESS	\N
0000ed	APRIL	\N
0000ee	NETWORK DESIGNERS, LTD.	\N
0000ef	KTI	\N
0000f0	SAMSUNG ELECTRONICS CO., LTD.	\N
0000f1	MAGNA COMPUTER CORPORATION	\N
0000f2	SPIDER COMMUNICATIONS	\N
0000f3	GANDALF DATA LIMITED	\N
0000f4	Allied Telesis	\N
0000f5	DIAMOND SALES LIMITED	\N
0000f6	APPLIED MICROSYSTEMS CORP.	\N
0000f7	YOUTH KEEP ENTERPRISE CO LTD	\N
0000f8	DIGITAL EQUIPMENT CORPORATION	\N
0000f9	QUOTRON SYSTEMS INC.	\N
0000fa	MICROSAGE COMPUTER SYSTEMS INC	\N
0000fb	RECHNER ZUR KOMMUNIKATION	\N
0000fc	MEIKO	\N
0000fd	HIGH LEVEL HARDWARE	\N
0000fe	ANNAPOLIS MICRO SYSTEMS	\N
0000ff	CAMTEC ELECTRONICS LTD.	\N
000100	EQUIP'TRANS	\N
000101	PRIVATE	\N
000102	3COM CORPORATION	\N
000103	3COM CORPORATION	\N
000104	DVICO Co., Ltd.	\N
000105	Beckhoff Automation GmbH	\N
000106	Tews Datentechnik GmbH	\N
000107	Leiser GmbH	\N
000108	AVLAB Technology, Inc.	\N
000109	Nagano Japan Radio Co., Ltd.	\N
00010a	CIS TECHNOLOGY INC.	\N
00010b	Space CyberLink, Inc.	\N
00010c	System Talks Inc.	\N
00010d	CORECO, INC.	\N
00010e	Bri-Link Technologies Co., Ltd	\N
00010f	Brocade Communications Systems, Inc.	\N
000110	Gotham Networks	\N
000111	iDigm Inc.	\N
000112	Shark Multimedia Inc.	\N
000113	OLYMPUS CORPORATION	\N
000114	KANDA TSUSHIN KOGYO CO., LTD.	\N
000115	EXTRATECH CORPORATION	\N
000116	Netspect Technologies, Inc.	\N
000117	CANAL +	\N
000118	EZ Digital Co., Ltd.	\N
000119	RTUnet (Australia)	\N
00011a	EEH DataLink GmbH	\N
00011b	Unizone Technologies, Inc.	\N
00011c	Universal Talkware Corporation	\N
00011d	Centillium Communications	\N
00011e	Precidia Technologies, Inc.	\N
00011f	RC Networks, Inc.	\N
000120	OSCILLOQUARTZ S.A.	\N
000121	Watchguard Technologies, Inc.	\N
000122	Trend Communications, Ltd.	\N
000123	DIGITAL ELECTRONICS CORP.	\N
000124	Acer Incorporated	\N
000125	YAESU MUSEN CO., LTD.	\N
000126	PAC Labs	\N
000127	OPEN Networks Pty Ltd	\N
000128	EnjoyWeb, Inc.	\N
000129	DFI Inc.	\N
00012a	Telematica Sistems Inteligente	\N
00012b	TELENET Co., Ltd.	\N
00012c	Aravox Technologies, Inc.	\N
00012d	Komodo Technology	\N
00012e	PC Partner Ltd.	\N
00012f	Twinhead International Corp	\N
000130	Extreme Networks	\N
000131	Detection Systems, Inc.	\N
000132	Dranetz - BMI	\N
000133	KYOWA Electronic Instruments C	\N
000134	Selectron Systems AG	\N
000135	KDC Corp.	\N
000136	CyberTAN Technology, Inc.	\N
000137	IT Farm Corporation	\N
000138	XAVi Technologies Corp.	\N
000139	Point Multimedia Systems	\N
00013a	SHELCAD COMMUNICATIONS, LTD.	\N
00013b	BNA SYSTEMS	\N
00013c	TIW SYSTEMS	\N
00013d	RiscStation Ltd.	\N
00013e	Ascom Tateco AB	\N
00013f	Neighbor World Co., Ltd.	\N
000140	Sendtek Corporation	\N
000141	CABLE PRINT	\N
000142	Cisco Systems, Inc.	\N
000143	Cisco Systems, Inc.	\N
000144	EMC Corporation	\N
000145	WINSYSTEMS, INC.	\N
000146	Tesco Controls, Inc.	\N
000147	Zhone Technologies	\N
000148	X-traWeb Inc.	\N
000149	T.D.T. Transfer Data Test GmbH	\N
00014a	Sony Corporation	\N
00014b	Ennovate Networks, Inc.	\N
00014c	Berkeley Process Control	\N
00014d	Shin Kin Enterprises Co., Ltd	\N
00014e	WIN Enterprises, Inc.	\N
00014f	ADTRAN INC	\N
000150	GILAT COMMUNICATIONS, LTD.	\N
000151	Ensemble Communications	\N
000152	CHROMATEK INC.	\N
000153	ARCHTEK TELECOM CORPORATION	\N
000154	G3M Corporation	\N
000155	Promise Technology, Inc.	\N
000156	FIREWIREDIRECT.COM, INC.	\N
000157	SYSWAVE CO., LTD	\N
000158	Electro Industries/Gauge Tech	\N
000159	S1 Corporation	\N
00015a	Digital Video Broadcasting	\N
00015b	ITALTEL S.p.A/RF-UP-I	\N
00015c	CADANT INC.	\N
00015d	Sun Microsystems, Inc	\N
00015e	BEST TECHNOLOGY CO., LTD.	\N
00015f	DIGITAL DESIGN GmbH	\N
000160	ELMEX Co., LTD.	\N
000161	Meta Machine Technology	\N
000162	Cygnet Technologies, Inc.	\N
000163	Cisco Systems, Inc.	\N
000164	Cisco Systems, Inc.	\N
000165	AirSwitch Corporation	\N
000166	TC GROUP A/S	\N
000167	HIOKI E.E. CORPORATION	\N
000168	VITANA CORPORATION	\N
000169	Celestix Networks Pte Ltd.	\N
00016a	ALITEC	\N
00016b	LightChip, Inc.	\N
00016c	FOXCONN	\N
00016d	CarrierComm Inc.	\N
00016e	Conklin Corporation	\N
00016f	Inkel Corp.	\N
000170	ESE Embedded System Engineer'g	\N
000171	Allied Data Technologies	\N
000172	TechnoLand Co., LTD.	\N
000173	AMCC	\N
000174	CyberOptics Corporation	\N
000175	Radiant Communications Corp.	\N
000176	Orient Silver Enterprises	\N
000177	EDSL	\N
000178	MARGI Systems, Inc.	\N
000179	WIRELESS TECHNOLOGY, INC.	\N
00017a	Chengdu Maipu Electric Industrial Co., Ltd.	\N
00017b	Heidelberger Druckmaschinen AG	\N
00017c	AG-E GmbH	\N
00017d	ThermoQuest	\N
00017e	ADTEK System Science Co., Ltd.	\N
00017f	Experience Music Project	\N
000180	AOpen, Inc.	\N
000181	Nortel Networks	\N
000182	DICA TECHNOLOGIES AG	\N
000183	ANITE TELECOMS	\N
000184	SIEB & MEYER AG	\N
000185	Aloka Co., Ltd.	\N
000186	Uwe Disch	\N
000187	i2SE GmbH	\N
000188	LXCO Technologies ag	\N
000189	Refraction Technology, Inc.	\N
00018a	ROI COMPUTER AG	\N
00018b	NetLinks Co., Ltd.	\N
00018c	Mega Vision	\N
00018d	AudeSi Technologies	\N
00018e	Logitec Corporation	\N
00018f	Kenetec, Inc.	\N
000190	SMK-M	\N
000191	SYRED Data Systems	\N
000192	Texas Digital Systems	\N
000193	Hanbyul Telecom Co., Ltd.	\N
000194	Capital Equipment Corporation	\N
000195	Sena Technologies, Inc.	\N
000196	Cisco Systems, Inc.	\N
000197	Cisco Systems, Inc.	\N
000198	Darim Vision	\N
000199	HeiSei Electronics	\N
00019a	LEUNIG GmbH	\N
00019b	Kyoto Microcomputer Co., Ltd.	\N
00019c	JDS Uniphase Inc.	\N
00019d	E-Control Systems, Inc.	\N
00019e	ESS Technology, Inc.	\N
00019f	Phonex Broadband	\N
0001a0	Infinilink Corporation	\N
0001a1	Mag-Tek, Inc.	\N
0001a2	Logical Co., Ltd.	\N
0001a3	GENESYS LOGIC, INC.	\N
0001a4	Microlink Corporation	\N
0001a5	Nextcomm, Inc.	\N
0001a6	Scientific-Atlanta Arcodan A/S	\N
0001a7	UNEX TECHNOLOGY CORPORATION	\N
0001a8	Welltech Computer Co., Ltd.	\N
0001a9	BMW AG	\N
0001aa	Airspan Communications, Ltd.	\N
0001ab	Main Street Networks	\N
0001ac	Sitara Networks, Inc.	\N
0001ad	Coach Master International  d.b.a. CMI Worldwide, Inc.	\N
0001ae	Trex Enterprises	\N
0001af	Emerson Network Power	\N
0001b0	Fulltek Technology Co., Ltd.	\N
0001b1	General Bandwidth	\N
0001b2	Digital Processing Systems, Inc.	\N
0001b3	Precision Electronic Manufacturing	\N
0001b4	Wayport, Inc.	\N
0001b5	Turin Networks, Inc.	\N
0001b6	SAEJIN T&M Co., Ltd.	\N
0001b7	Centos, Inc.	\N
0001b8	Netsensity, Inc.	\N
0001b9	SKF Condition Monitoring	\N
0001ba	IC-Net, Inc.	\N
0001bb	Frequentis	\N
0001bc	Brains Corporation	\N
0001bd	Peterson Electro-Musical Products, Inc.	\N
0001be	Gigalink Co., Ltd.	\N
0001bf	Teleforce Co., Ltd.	\N
0001c0	CompuLab, Ltd.	\N
0001c1	Vitesse Semiconductor Corporation	\N
0001c2	ARK Research Corp.	\N
0001c3	Acromag, Inc.	\N
0001c4	NeoWave, Inc.	\N
0001c5	Simpler Networks	\N
0001c6	Quarry Technologies	\N
0001c7	Cisco Systems, Inc.	\N
0001c8	THOMAS CONRAD CORP.	\N
0001c9	Cisco Systems, Inc.	\N
0001ca	Geocast Network Systems, Inc.	\N
0001cb	EVR	\N
0001cc	Japan Total Design Communication Co., Ltd.	\N
0001cd	ARtem	\N
0001ce	Custom Micro Products, Ltd.	\N
0001cf	Alpha Data Parallel Systems, Ltd.	\N
0001d0	VitalPoint, Inc.	\N
0001d1	CoNet Communications, Inc.	\N
0001d2	MacPower Peripherals, Ltd.	\N
0001d3	PAXCOMM, Inc.	\N
0001d4	Leisure Time, Inc.	\N
0001d5	HAEDONG INFO & COMM CO., LTD	\N
0001d6	manroland AG	\N
0001d7	F5 Networks, Inc.	\N
0001d8	Teltronics, Inc.	\N
0001d9	Sigma, Inc.	\N
0001da	WINCOMM Corporation	\N
0001db	Freecom Technologies GmbH	\N
0001dc	Activetelco	\N
0001dd	Avail Networks	\N
0001de	Trango Systems, Inc.	\N
0001df	ISDN Communications, Ltd.	\N
0001e0	Fast Systems, Inc.	\N
0001e1	Kinpo Electronics, Inc.	\N
0001e2	Ando Electric Corporation	\N
0001e3	Siemens AG	\N
0001e4	Sitera, Inc.	\N
0001e5	Supernet, Inc.	\N
0001e6	Hewlett-Packard Company	\N
0001e7	Hewlett-Packard Company	\N
0001e8	Force10 Networks, Inc.	\N
0001e9	Litton Marine Systems B.V.	\N
0001ea	Cirilium Corp.	\N
0001eb	C-COM Corporation	\N
0001ec	Ericsson Group	\N
0001ed	SETA Corp.	\N
0001ee	Comtrol Europe, Ltd.	\N
0001ef	Camtel Technology Corp.	\N
0001f0	Tridium, Inc.	\N
0001f1	Innovative Concepts, Inc.	\N
0001f2	Mark of the Unicorn, Inc.	\N
0001f3	QPS, Inc.	\N
0001f4	Enterasys Networks	\N
0001f5	ERIM S.A.	\N
0001f6	Association of Musical Electronics Industry	\N
0001f7	Image Display Systems, Inc.	\N
0001f8	Adherent Systems, Ltd.	\N
0001f9	TeraGlobal Communications Corp.	\N
0001fa	HOROSCAS	\N
0001fb	DoTop Technology, Inc.	\N
0001fc	Keyence Corporation	\N
0001fd	Digital Voice Systems, Inc.	\N
0001fe	DIGITAL EQUIPMENT CORPORATION	\N
0001ff	Data Direct Networks, Inc.	\N
000200	Net & Sys Co., Ltd.	\N
000201	IFM Electronic gmbh	\N
000202	Amino Communications, Ltd.	\N
000203	Woonsang Telecom, Inc.	\N
000204	Bodmann Industries Elektronik GmbH	\N
000205	Hitachi Denshi, Ltd.	\N
000206	Telital R&D Denmark A/S	\N
000207	VisionGlobal Network Corp.	\N
000208	Unify Networks, Inc.	\N
000209	Shenzhen SED Information Technology Co., Ltd.	\N
00020a	Gefran Spa	\N
00020b	Native Networks, Inc.	\N
00020c	Metro-Optix	\N
00020d	Micronpc.com	\N
00020e	ECI Telecom, Ltd., NSD-US	\N
00020f	AATR	\N
000210	Fenecom	\N
000211	Nature Worldwide Technology Corp.	\N
000212	SierraCom	\N
000213	S.D.E.L.	\N
000214	DTVRO	\N
000215	Cotas Computer Technology A/B	\N
000216	Cisco Systems, Inc.	\N
000217	Cisco Systems, Inc.	\N
000218	Advanced Scientific Corp	\N
000219	Paralon Technologies	\N
00021a	Zuma Networks	\N
00021b	Kollmorgen-Servotronix	\N
00021c	Network Elements, Inc.	\N
00021d	Data General Communication Ltd.	\N
00021e	SIMTEL S.R.L.	\N
00021f	Aculab PLC	\N
000220	Canon Aptex, Inc.	\N
000221	DSP Application, Ltd.	\N
000222	Chromisys, Inc.	\N
000223	ClickTV	\N
000224	C-COR	\N
000225	One Stop Systems	\N
000226	XESystems, Inc.	\N
000227	ESD Electronic System Design GmbH	\N
000228	Necsom, Ltd.	\N
000229	Adtec Corporation	\N
00022a	Asound Electronic	\N
00022b	SAXA, Inc.	\N
00022c	ABB Bomem, Inc.	\N
00022d	Agere Systems	\N
00022e	TEAC Corp. R& D	\N
00022f	P-Cube, Ltd.	\N
000230	Intersoft Electronics	\N
000231	Ingersoll-Rand	\N
000232	Avision, Inc.	\N
000233	Mantra Communications, Inc.	\N
000234	Imperial Technology, Inc.	\N
000235	Paragon Networks International	\N
000236	INIT GmbH	\N
000237	Cosmo Research Corp.	\N
000238	Serome Technology, Inc.	\N
000239	Visicom	\N
00023a	ZSK Stickmaschinen GmbH	\N
00023b	Redback Networks	\N
00023c	Creative Technology, Ltd.	\N
00023d	Cisco Systems, Inc.	\N
00023e	Selta Telematica S.p.a	\N
00023f	Compal Electronics, Inc.	\N
000240	Seedek Co., Ltd.	\N
000241	Amer.com	\N
000242	Videoframe Systems	\N
000243	Raysis Co., Ltd.	\N
000244	SURECOM Technology Co.	\N
000245	Lampus Co, Ltd.	\N
000246	All-Win Tech Co., Ltd.	\N
000247	Great Dragon Information Technology (Group) Co., Ltd.	\N
000248	Pilz GmbH & Co.	\N
000249	Aviv Infocom Co, Ltd.	\N
00024a	Cisco Systems, Inc.	\N
00024b	Cisco Systems, Inc.	\N
00024c	SiByte, Inc.	\N
00024d	Mannesman Dematic Colby Pty. Ltd.	\N
00024e	Datacard Group	\N
00024f	IPM Datacom S.R.L.	\N
000250	Geyser Networks, Inc.	\N
000251	Soma Networks, Inc.	\N
000252	Carrier Corporation	\N
000253	Televideo, Inc.	\N
000254	WorldGate	\N
000255	IBM Corporation	\N
000256	Alpha Processor, Inc.	\N
000257	Microcom Corp.	\N
000258	Flying Packets Communications	\N
000259	Tsann Kuen China (Shanghai)Enterprise Co., Ltd. IT Group	\N
00025a	Catena Networks	\N
00025b	Cambridge Silicon Radio	\N
00025c	SCI Systems (Kunshan) Co., Ltd.	\N
00025d	Calix Networks	\N
00025e	High Technology Ltd	\N
00025f	Nortel Networks	\N
000260	Accordion Networks, Inc.	\N
000261	Tilgin AB	\N
000262	Soyo Group Soyo Com Tech Co., Ltd	\N
000263	UPS Manufacturing SRL	\N
000264	AudioRamp.com	\N
000265	Virditech Co. Ltd.	\N
000266	Thermalogic Corporation	\N
000267	NODE RUNNER, INC.	\N
000268	Harris Government Communications	\N
000269	Nadatel Co., Ltd	\N
00026a	Cocess Telecom Co., Ltd.	\N
00026b	BCM Computers Co., Ltd.	\N
00026c	Philips CFT	\N
00026d	Adept Telecom	\N
00026e	NeGeN Access, Inc.	\N
00026f	Senao International Co., Ltd.	\N
000270	Crewave Co., Ltd.	\N
000271	Zhone Technologies	\N
000272	CC&C Technologies, Inc.	\N
000273	Coriolis Networks	\N
000274	Tommy Technologies Corp.	\N
000275	SMART Technologies, Inc.	\N
000276	Primax Electronics Ltd.	\N
000277	Cash Systemes Industrie	\N
000278	Samsung Electro-Mechanics Co., Ltd.	\N
000279	Control Applications, Ltd.	\N
00027a	IOI Technology Corporation	\N
00027b	Amplify Net, Inc.	\N
00027c	Trilithic, Inc.	\N
00027d	Cisco Systems, Inc.	\N
00027e	Cisco Systems, Inc.	\N
00027f	ask-technologies.com	\N
000280	Mu Net, Inc.	\N
000281	Madge Ltd.	\N
000282	ViaClix, Inc.	\N
000283	Spectrum Controls, Inc.	\N
000284	AREVA T&D	\N
000285	Riverstone Networks	\N
000286	Occam Networks	\N
000287	Adapcom	\N
000288	GLOBAL VILLAGE COMMUNICATION	\N
000289	DNE Technologies	\N
00028a	Ambit Microsystems Corporation	\N
00028b	VDSL Systems OY	\N
00028c	Micrel-Synergy Semiconductor	\N
00028d	Movita Technologies, Inc.	\N
00028e	Rapid 5 Networks, Inc.	\N
00028f	Globetek, Inc.	\N
000290	Woorigisool, Inc.	\N
000291	Open Network Co., Ltd.	\N
000292	Logic Innovations, Inc.	\N
000293	Solid Data Systems	\N
000294	Tokyo Sokushin Co., Ltd.	\N
000295	IP.Access Limited	\N
000296	Lectron Co,. Ltd.	\N
000297	C-COR.net	\N
000298	Broadframe Corporation	\N
000299	Apex, Inc.	\N
00029a	Storage Apps	\N
00029b	Kreatel Communications AB	\N
00029c	3COM	\N
00029d	Merix Corp.	\N
00029e	Information Equipment Co., Ltd.	\N
00029f	L-3 Communication Aviation Recorders	\N
0002a0	Flatstack Ltd.	\N
0002a1	World Wide Packets	\N
0002a2	Hilscher GmbH	\N
0002a3	ABB Switzerland Ltd, Power Systems	\N
0002a4	AddPac Technology Co., Ltd.	\N
0002a5	Hewlett Packard	\N
0002a6	Effinet Systems Co., Ltd.	\N
0002a7	Vivace Networks	\N
0002a8	Air Link Technology	\N
0002a9	RACOM, s.r.o.	\N
0002aa	PLcom Co., Ltd.	\N
0002ab	CTC Union Technologies Co., Ltd.	\N
0002ac	3PAR data	\N
0002ad	HOYA Corporation	\N
0002ae	Scannex Electronics Ltd.	\N
0002af	TeleCruz Technology, Inc.	\N
0002b0	Hokubu Communication & Industrial Co., Ltd.	\N
0002b1	Anritsu, Ltd.	\N
0002b2	Cablevision	\N
0002b3	Intel Corporation	\N
0002b4	DAPHNE	\N
0002b5	Avnet, Inc.	\N
0002b6	Acrosser Technology Co., Ltd.	\N
0002b7	Watanabe Electric Industry Co., Ltd.	\N
0002b8	WHI KONSULT AB	\N
0002b9	Cisco Systems, Inc.	\N
0002ba	Cisco Systems, Inc.	\N
0002bb	Continuous Computing Corp	\N
0002bc	LVL 7 Systems, Inc.	\N
0002bd	Bionet Co., Ltd.	\N
0002be	Totsu Engineering, Inc.	\N
0002bf	dotRocket, Inc.	\N
0002c0	Bencent Tzeng Industry Co., Ltd.	\N
0002c1	Innovative Electronic Designs, Inc.	\N
0002c2	Net Vision Telecom	\N
0002c3	Arelnet Ltd.	\N
0002c4	Vector International BVBA	\N
0002c5	Evertz Microsystems Ltd.	\N
0002c6	Data Track Technology PLC	\N
0002c7	ALPS ELECTRIC Co., Ltd.	\N
0002c8	Technocom Communications Technology (pte) Ltd	\N
0002c9	Mellanox Technologies	\N
0002ca	EndPoints, Inc.	\N
0002cb	TriState Ltd.	\N
0002cc	M.C.C.I	\N
0002cd	TeleDream, Inc.	\N
0002ce	FoxJet, Inc.	\N
0002cf	ZyGate Communications, Inc.	\N
0002d0	Comdial Corporation	\N
0002d1	Vivotek, Inc.	\N
0002d2	Workstation AG	\N
0002d3	NetBotz, Inc.	\N
0002d4	PDA Peripherals, Inc.	\N
0002d5	ACR	\N
0002d6	NICE Systems	\N
0002d7	EMPEG Ltd	\N
0002d8	BRECIS Communications Corporation	\N
0002d9	Reliable Controls	\N
0002da	ExiO Communications, Inc.	\N
0002db	NETSEC	\N
0002dc	Fujitsu General Limited	\N
0002dd	Bromax Communications, Ltd.	\N
0002de	Astrodesign, Inc.	\N
0002df	Net Com Systems, Inc.	\N
0002e0	ETAS GmbH	\N
0002e1	Integrated Network Corporation	\N
0002e2	NDC Infared Engineering	\N
0002e3	LITE-ON Communications, Inc.	\N
0002e4	JC HYUN Systems, Inc.	\N
0002e5	Timeware Ltd.	\N
0002e6	Gould Instrument Systems, Inc.	\N
0002e7	CAB GmbH & Co KG	\N
0002e8	E.D.&A.	\N
0002e9	CS Systemes De Securite - C3S	\N
0002ea	Focus Enhancements	\N
0002eb	Pico Communications	\N
0002ec	Maschoff Design Engineering	\N
0002ed	DXO Telecom Co., Ltd.	\N
0002ee	Nokia Danmark A/S	\N
0002ef	CCC Network Systems Group Ltd.	\N
0002f0	AME Optimedia Technology Co., Ltd.	\N
0002f1	Pinetron Co., Ltd.	\N
0002f2	eDevice, Inc.	\N
0002f3	Media Serve Co., Ltd.	\N
0002f4	PCTEL, Inc.	\N
0002f5	VIVE Synergies, Inc.	\N
0002f6	Equipe Communications	\N
0002f7	ARM	\N
0002f8	SEAKR Engineering, Inc.	\N
0002f9	Mimos Semiconductor SDN BHD	\N
0002fa	DX Antenna Co., Ltd.	\N
0002fb	Baumuller Aulugen-Systemtechnik GmbH	\N
0002fc	Cisco Systems, Inc.	\N
0002fd	Cisco Systems, Inc.	\N
0002fe	Viditec, Inc.	\N
0002ff	Handan BroadInfoCom	\N
000300	NetContinuum, Inc.	\N
000301	Avantas Networks Corporation	\N
000302	Charles Industries, Ltd.	\N
000303	JAMA Electronics Co., Ltd.	\N
000304	Pacific Broadband Communications	\N
000305	MSC Vertriebs GmbH	\N
000306	Fusion In Tech Co., Ltd.	\N
000307	Secure Works, Inc.	\N
000308	AM Communications, Inc.	\N
000309	Texcel Technology PLC	\N
00030a	Argus Technologies	\N
00030b	Hunter Technology, Inc.	\N
00030c	Telesoft Technologies Ltd.	\N
00030d	Uniwill Computer Corp.	\N
00030e	Core Communications Co., Ltd.	\N
00030f	Digital China (Shanghai) Networks Ltd.	\N
000310	ITX E-Globaledge Corporation	\N
000311	Micro Technology Co., Ltd.	\N
000312	TR-Systemtechnik GmbH	\N
000313	Access Media SPA	\N
000314	Teleware Network Systems	\N
000315	Cidco Incorporated	\N
000316	Nobell Communications, Inc.	\N
000317	Merlin Systems, Inc.	\N
000318	Cyras Systems, Inc.	\N
000319	Infineon AG	\N
00031a	Beijing Broad Telecom Ltd., China	\N
00031b	Cellvision Systems, Inc.	\N
00031c	Svenska Hardvarufabriken AB	\N
00031d	Taiwan Commate Computer, Inc.	\N
00031e	Optranet, Inc.	\N
00031f	Condev Ltd.	\N
000320	Xpeed, Inc.	\N
000321	Reco Research Co., Ltd.	\N
000322	IDIS Co., Ltd.	\N
000323	Cornet Technology, Inc.	\N
000324	SANYO Consumer Electronics Co., Ltd.	\N
000325	Arima Computer Corp.	\N
000326	Iwasaki Information Systems Co., Ltd.	\N
000327	ACT'L	\N
000328	Mace Group, Inc.	\N
000329	F3, Inc.	\N
00032a	UniData Communication Systems, Inc.	\N
00032b	GAI Datenfunksysteme GmbH	\N
00032c	ABB Switzerland Ltd	\N
00032d	IBASE Technology, Inc.	\N
00032e	Scope Information Management, Ltd.	\N
00032f	Global Sun Technology, Inc.	\N
000330	Imagenics, Co., Ltd.	\N
000331	Cisco Systems, Inc.	\N
000332	Cisco Systems, Inc.	\N
000333	Digitel Co., Ltd.	\N
000334	Newport Electronics	\N
000335	Mirae Technology	\N
000336	Zetes Technologies	\N
000337	Vaone, Inc.	\N
000338	Oak Technology	\N
000339	Eurologic Systems, Ltd.	\N
00033a	Silicon Wave, Inc.	\N
00033b	TAMI Tech Co., Ltd.	\N
00033c	Daiden Co., Ltd.	\N
00033d	ILSHin Lab	\N
00033e	Tateyama System Laboratory Co., Ltd.	\N
00033f	BigBand Networks, Ltd.	\N
000340	Floware Wireless Systems, Ltd.	\N
000341	Axon Digital Design	\N
000342	Nortel Networks	\N
000343	Martin Professional A/S	\N
000344	Tietech.Co., Ltd.	\N
000345	Routrek Networks Corporation	\N
000346	Hitachi Kokusai Electric, Inc.	\N
000347	Intel Corporation	\N
000348	Norscan Instruments, Ltd.	\N
000349	Vidicode Datacommunicatie B.V.	\N
00034a	RIAS Corporation	\N
00034b	Nortel Networks	\N
00034c	Shanghai DigiVision Technology Co., Ltd.	\N
00034d	Chiaro Networks, Ltd.	\N
00034e	Pos Data Company, Ltd.	\N
00034f	Sur-Gard Security	\N
000350	BTICINO SPA	\N
000351	Diebold, Inc.	\N
000352	Colubris Networks	\N
000353	Mitac, Inc.	\N
000354	Fiber Logic Communications	\N
000355	TeraBeam Internet Systems	\N
000356	Wincor Nixdorf International GmbH	\N
000357	Intervoice-Brite, Inc.	\N
000358	Hanyang Digitech Co., Ltd.	\N
000359	DigitalSis	\N
00035a	Photron Limited	\N
00035b	BridgeWave Communications	\N
00035c	Saint Song Corp.	\N
00035d	Bosung Hi-Net Co., Ltd.	\N
00035e	Metropolitan Area Networks, Inc.	\N
00035f	Prueftechnik Condition Monitoring GmbH & Co. KG	\N
000360	PAC Interactive Technology, Inc.	\N
000361	Widcomm, Inc.	\N
000362	Vodtel Communications, Inc.	\N
000363	Miraesys Co., Ltd.	\N
000364	Scenix Semiconductor, Inc.	\N
000365	Kira Information & Communications, Ltd.	\N
000366	ASM Pacific Technology	\N
000367	Jasmine Networks, Inc.	\N
000368	Embedone Co., Ltd.	\N
000369	Nippon Antenna Co., Ltd.	\N
00036a	Mainnet, Ltd.	\N
00036b	Cisco Systems, Inc.	\N
00036c	Cisco Systems, Inc.	\N
00036d	Runtop, Inc.	\N
00036e	Nicon Systems (Pty) Limited	\N
00036f	Telsey SPA	\N
000370	NXTV, Inc.	\N
000371	Acomz Networks Corp.	\N
000372	ULAN	\N
000373	Aselsan A.S	\N
000374	Control Microsystems	\N
000375	NetMedia, Inc.	\N
000376	Graphtec Technology, Inc.	\N
000377	Gigabit Wireless	\N
000378	HUMAX Co., Ltd.	\N
000379	Proscend Communications, Inc.	\N
00037a	Taiyo Yuden Co., Ltd.	\N
00037b	IDEC IZUMI Corporation	\N
00037c	Coax Media	\N
00037d	Stellcom	\N
00037e	PORTech Communications, Inc.	\N
00037f	Atheros Communications, Inc.	\N
000380	SSH Communications Security Corp.	\N
000381	Ingenico International	\N
000382	A-One Co., Ltd.	\N
000383	Metera Networks, Inc.	\N
000384	AETA	\N
000385	Actelis Networks, Inc.	\N
000386	Ho Net, Inc.	\N
000387	Blaze Network Products	\N
000388	Fastfame Technology Co., Ltd.	\N
000389	Plantronics	\N
00038a	America Online, Inc.	\N
00038b	PLUS-ONE I&T, Inc.	\N
00038c	Total Impact	\N
00038d	PCS Revenue Control Systems, Inc.	\N
00038e	Atoga Systems, Inc.	\N
00038f	Weinschel Corporation	\N
000390	Digital Video Communications, Inc.	\N
000391	Advanced Digital Broadcast, Ltd.	\N
000392	Hyundai Teletek Co., Ltd.	\N
000393	Apple Computer, Inc.	\N
000394	Connect One	\N
000395	California Amplifier	\N
000396	EZ Cast Co., Ltd.	\N
000397	Watchfront Limited	\N
000398	WISI	\N
000399	Dongju Informations & Communications Co., Ltd.	\N
00039a	SiConnect	\N
00039b	NetChip Technology, Inc.	\N
00039c	OptiMight Communications, Inc.	\N
00039d	Qisda Corporation	\N
00039e	Tera System Co., Ltd.	\N
00039f	Cisco Systems, Inc.	\N
0003a0	Cisco Systems, Inc.	\N
0003a1	HIPER Information & Communication, Inc.	\N
0003a2	Catapult Communications	\N
0003a3	MAVIX, Ltd.	\N
0003a4	Imation Corp.	\N
0003a5	Medea Corporation	\N
0003a6	Traxit Technology, Inc.	\N
0003a7	Unixtar Technology, Inc.	\N
0003a8	IDOT Computers, Inc.	\N
0003a9	AXCENT Media AG	\N
0003aa	Watlow	\N
0003ab	Bridge Information Systems	\N
0003ac	Fronius Schweissmaschinen	\N
0003ad	Emerson Energy Systems AB	\N
0003ae	Allied Advanced Manufacturing Pte, Ltd.	\N
0003af	Paragea Communications	\N
0003b0	Xsense Technology Corp.	\N
0003b1	Hospira Inc.	\N
0003b2	Radware	\N
0003b3	IA Link Systems Co., Ltd.	\N
0003b4	Macrotek International Corp.	\N
0003b5	Entra Technology Co.	\N
0003b6	QSI Corporation	\N
0003b7	ZACCESS Systems	\N
0003b8	NetKit Solutions, LLC	\N
0003b9	Hualong Telecom Co., Ltd.	\N
0003ba	Sun Microsystems Inc.	\N
0003bb	Signal Communications Limited	\N
0003bc	COT GmbH	\N
0003bd	OmniCluster Technologies, Inc.	\N
0003be	Netility	\N
0003bf	Centerpoint Broadband Technologies, Inc.	\N
0003c0	RFTNC Co., Ltd.	\N
0003c1	Packet Dynamics Ltd	\N
0003c2	Solphone K.K.	\N
0003c3	Micronik Multimedia	\N
0003c4	Tomra Systems ASA	\N
0003c5	Mobotix AG	\N
0003c6	ICUE Systems, Inc.	\N
0003c7	hopf Elektronik GmbH	\N
0003c8	CML Emergency Services	\N
0003c9	TECOM Co., Ltd.	\N
0003ca	MTS Systems Corp.	\N
0003cb	Nippon Systems Development Co., Ltd.	\N
0003cc	Momentum Computer, Inc.	\N
0003cd	Clovertech, Inc.	\N
0003ce	ETEN Technologies, Inc.	\N
0003cf	Muxcom, Inc.	\N
0003d0	KOANKEISO Co., Ltd.	\N
0003d1	Takaya Corporation	\N
0003d2	Crossbeam Systems, Inc.	\N
0003d3	Internet Energy Systems, Inc.	\N
0003d4	Alloptic, Inc.	\N
0003d5	Advanced Communications Co., Ltd.	\N
0003d6	RADVision, Ltd.	\N
0003d7	NextNet Wireless, Inc.	\N
0003d8	iMPath Networks, Inc.	\N
0003d9	Secheron SA	\N
0003da	Takamisawa Cybernetics Co., Ltd.	\N
0003db	Apogee Electronics Corp.	\N
0003dc	Lexar Media, Inc.	\N
0003dd	Comark Corp.	\N
0003de	OTC Wireless	\N
0003df	Desana Systems	\N
0003e0	RadioFrame Networks, Inc.	\N
0003e1	Winmate Communication, Inc.	\N
0003e2	Comspace Corporation	\N
0003e3	Cisco Systems, Inc.	\N
0003e4	Cisco Systems, Inc.	\N
0003e5	Hermstedt SG	\N
0003e6	Entone, Inc.	\N
0003e7	Logostek Co. Ltd.	\N
0003e8	Wavelength Digital Limited	\N
0003e9	Akara Canada, Inc.	\N
0003ea	Mega System Technologies, Inc.	\N
0003eb	Atrica	\N
0003ec	ICG Research, Inc.	\N
0003ed	Shinkawa Electric Co., Ltd.	\N
0003ee	MKNet Corporation	\N
0003ef	Oneline AG	\N
0003f0	Redfern Broadband Networks	\N
0003f1	Cicada Semiconductor, Inc.	\N
0003f2	Seneca Networks	\N
0003f3	Dazzle Multimedia, Inc.	\N
0003f4	NetBurner	\N
0003f5	Chip2Chip	\N
0003f6	Allegro Networks, Inc.	\N
0003f7	Plast-Control GmbH	\N
0003f8	SanCastle Technologies, Inc.	\N
0003f9	Pleiades Communications, Inc.	\N
0003fa	TiMetra Networks	\N
0003fb	ENEGATE Co.,Ltd.	\N
0003fc	Intertex Data AB	\N
0003fd	Cisco Systems, Inc.	\N
0003fe	Cisco Systems, Inc.	\N
0003ff	Microsoft Corporation	\N
000400	LEXMARK INTERNATIONAL, INC.	\N
000401	Osaki Electric Co., Ltd.	\N
000402	Nexsan Technologies, Ltd.	\N
000403	Nexsi Corporation	\N
000404	Makino Milling Machine Co., Ltd.	\N
000405	ACN Technologies	\N
000406	Fa. Metabox AG	\N
000407	Topcon Positioning Systems, Inc.	\N
000408	Sanko Electronics Co., Ltd.	\N
000409	Cratos Networks	\N
00040a	Sage Systems	\N
00040b	3com Europe Ltd.	\N
00040c	KANNO Work's Ltd.	\N
00040d	Avaya, Inc.	\N
00040e	AVM GmbH	\N
00040f	Asus Network Technologies, Inc.	\N
000410	Spinnaker Networks, Inc.	\N
000411	Inkra Networks, Inc.	\N
000412	WaveSmith Networks, Inc.	\N
000413	SNOM Technology AG	\N
000414	Umezawa Musen Denki Co., Ltd.	\N
000415	Rasteme Systems Co., Ltd.	\N
000416	Parks S/A Comunicacoes Digitais	\N
000417	ELAU AG	\N
000418	Teltronic S.A.U.	\N
000419	Fibercycle Networks, Inc.	\N
00041a	Ines Test and Measurement GmbH & CoKG	\N
00041b	Bridgeworks Ltd.	\N
00041c	ipDialog, Inc.	\N
00041d	Corega of America	\N
00041e	Shikoku Instrumentation Co., Ltd.	\N
00041f	Sony Computer Entertainment, Inc.	\N
000420	Slim Devices, Inc.	\N
000421	Ocular Networks	\N
000422	Gordon Kapes, Inc.	\N
000423	Intel Corporation	\N
000424	TMC s.r.l.	\N
000425	Atmel Corporation	\N
000426	Autosys	\N
000427	Cisco Systems, Inc.	\N
000428	Cisco Systems, Inc.	\N
000429	Pixord Corporation	\N
00042a	Wireless Networks, Inc.	\N
00042b	IT Access Co., Ltd.	\N
00042c	Minet, Inc.	\N
00042d	Sarian Systems, Ltd.	\N
00042e	Netous Technologies, Ltd.	\N
00042f	International Communications Products, Inc.	\N
000430	Netgem	\N
000431	GlobalStreams, Inc.	\N
000432	Voyetra Turtle Beach, Inc.	\N
000433	Cyberboard A/S	\N
000434	Accelent Systems, Inc.	\N
000435	Comptek International, Inc.	\N
000436	ELANsat Technologies, Inc.	\N
000437	Powin Information Technology, Inc.	\N
000438	Nortel Networks	\N
000439	Rosco Entertainment Technology, Inc.	\N
00043a	Intelligent Telecommunications, Inc.	\N
00043b	Lava Computer Mfg., Inc.	\N
00043c	SONOS Co., Ltd.	\N
00043d	INDEL AG	\N
00043e	Telencomm	\N
00043f	ESTeem Wireless Modems, Inc	\N
000440	cyberPIXIE, Inc.	\N
000441	Half Dome Systems, Inc.	\N
000442	NACT	\N
000443	Agilent Technologies, Inc.	\N
000444	Western Multiplex Corporation	\N
000445	LMS Skalar Instruments GmbH	\N
000446	CYZENTECH Co., Ltd.	\N
000447	Acrowave Systems Co., Ltd.	\N
000448	Polaroid Corporation	\N
000449	Mapletree Networks	\N
00044a	iPolicy Networks, Inc.	\N
00044b	NVIDIA	\N
00044c	JENOPTIK	\N
00044d	Cisco Systems, Inc.	\N
00044e	Cisco Systems, Inc.	\N
00044f	Leukhardt Systemelektronik GmbH	\N
000450	DMD Computers SRL	\N
000451	Medrad, Inc.	\N
000452	RocketLogix, Inc.	\N
000453	YottaYotta, Inc.	\N
000454	Quadriga UK	\N
000455	ANTARA.net	\N
000456	Motorola PTP Inc	\N
000457	Universal Access Technology, Inc.	\N
000458	Fusion X Co., Ltd.	\N
000459	Veristar Corporation	\N
00045a	The Linksys Group, Inc.	\N
00045b	Techsan Electronics Co., Ltd.	\N
00045c	Mobiwave Pte Ltd	\N
00045d	BEKA Elektronik	\N
00045e	PolyTrax Information Technology AG	\N
00045f	Evalue Technology, Inc.	\N
000460	Knilink Technology, Inc.	\N
000461	EPOX Computer Co., Ltd.	\N
000462	DAKOS Data & Communication Co., Ltd.	\N
000463	Bosch Security Systems	\N
000464	Fantasma Networks, Inc.	\N
000465	i.s.t isdn-support technik GmbH	\N
000466	ARMITEL Co.	\N
000467	Wuhan Research Institute of MII	\N
000468	Vivity, Inc.	\N
000469	Innocom, Inc.	\N
00046a	Navini Networks	\N
00046b	Palm Wireless, Inc.	\N
00046c	Cyber Technology Co., Ltd.	\N
00046d	Cisco Systems, Inc.	\N
00046e	Cisco Systems, Inc.	\N
00046f	Digitel S/A Industria Eletronica	\N
000470	ipUnplugged AB	\N
000471	IPrad	\N
000472	Telelynx, Inc.	\N
000473	Photonex Corporation	\N
000474	LEGRAND	\N
000475	3 Com Corporation	\N
000476	3 Com Corporation	\N
000477	Scalant Systems, Inc.	\N
000478	G. Star Technology Corporation	\N
000479	Radius Co., Ltd.	\N
00047a	AXXESSIT ASA	\N
00047b	Schlumberger	\N
00047c	Skidata AG	\N
00047d	Pelco	\N
00047e	Optelecom=NKF	\N
00047f	Chr. Mayr GmbH & Co. KG	\N
000480	Brocade Communications Systems, Inc	\N
000481	Econolite Control Products, Inc.	\N
000482	Medialogic Corp.	\N
000483	Deltron Technology, Inc.	\N
000484	Amann GmbH	\N
000485	PicoLight	\N
000486	ITTC, University of Kansas	\N
000487	Cogency Semiconductor, Inc.	\N
000488	Eurotherm Controls	\N
000489	YAFO Networks, Inc.	\N
00048a	Temia Vertriebs GmbH	\N
00048b	Poscon Corporation	\N
00048c	Nayna Networks, Inc.	\N
00048d	Tone Commander Systems, Inc.	\N
00048e	Ohm Tech Labs, Inc.	\N
00048f	TD Systems Corporation	\N
000490	Optical Access	\N
000491	Technovision, Inc.	\N
000492	Hive Internet, Ltd.	\N
000493	Tsinghua Unisplendour Co., Ltd.	\N
000494	Breezecom, Ltd.	\N
000495	Tejas Networks India Limited	\N
000496	Extreme Networks	\N
000497	MacroSystem Digital Video AG	\N
000498	Mahi Networks	\N
000499	Chino Corporation	\N
00049a	Cisco Systems, Inc.	\N
00049b	Cisco Systems, Inc.	\N
00049c	Surgient Networks, Inc.	\N
00049d	Ipanema Technologies	\N
00049e	Wirelink Co., Ltd.	\N
00049f	Freescale Semiconductor	\N
0004a0	Verity Instruments, Inc.	\N
0004a1	Pathway Connectivity	\N
0004a2	L.S.I. Japan Co., Ltd.	\N
0004a3	Microchip Technology, Inc.	\N
0004a4	NetEnabled, Inc.	\N
0004a5	Barco Projection Systems NV	\N
0004a6	SAF Tehnika Ltd.	\N
0004a7	FabiaTech Corporation	\N
0004a8	Broadmax Technologies, Inc.	\N
0004a9	SandStream Technologies, Inc.	\N
0004aa	Jetstream Communications	\N
0004ab	Comverse Network Systems, Inc.	\N
0004ac	IBM CORP.	\N
0004ad	Malibu Networks	\N
0004ae	Sullair Corporation	\N
0004af	Digital Fountain, Inc.	\N
0004b0	ELESIGN Co., Ltd.	\N
0004b1	Signal Technology, Inc.	\N
0004b2	ESSEGI SRL	\N
0004b3	Videotek, Inc.	\N
0004b4	CIAC	\N
0004b5	Equitrac Corporation	\N
0004b6	Stratex Networks, Inc.	\N
0004b7	AMB i.t. Holding	\N
0004b8	Kumahira Co., Ltd.	\N
0004b9	S.I. Soubou, Inc.	\N
0004ba	KDD Media Will Corporation	\N
0004bb	Bardac Corporation	\N
0004bc	Giantec, Inc.	\N
0004bd	Motorola BCS	\N
0004be	OptXCon, Inc.	\N
0004bf	VersaLogic Corp.	\N
0004c0	Cisco Systems, Inc.	\N
0004c1	Cisco Systems, Inc.	\N
0004c2	Magnipix, Inc.	\N
0004c3	CASTOR Informatique	\N
0004c4	Allen & Heath Limited	\N
0004c5	ASE Technologies, USA	\N
0004c6	Yamaha Motor Co., Ltd.	\N
0004c7	NetMount	\N
0004c8	LIBA Maschinenfabrik GmbH	\N
0004c9	Micro Electron Co., Ltd.	\N
0004ca	FreeMs Corp.	\N
0004cb	Tdsoft Communication, Ltd.	\N
0004cc	Peek Traffic B.V.	\N
0004cd	Informedia Research Group	\N
0004ce	Patria Ailon	\N
0004cf	Seagate Technology	\N
0004d0	Softlink s.r.o.	\N
0004d1	Drew Technologies, Inc.	\N
0004d2	Adcon Telemetry GmbH	\N
0004d3	Toyokeiki Co., Ltd.	\N
0004d4	Proview Electronics Co., Ltd.	\N
0004d5	Hitachi Information & Communication Engineering, Ltd.	\N
0004d6	Takagi Industrial Co., Ltd.	\N
0004d7	Omitec Instrumentation Ltd.	\N
0004d8	IPWireless, Inc.	\N
0004d9	Titan Electronics, Inc.	\N
0004da	Relax Technology, Inc.	\N
0004db	Tellus Group Corp.	\N
0004dc	Nortel Networks	\N
0004dd	Cisco Systems, Inc.	\N
0004de	Cisco Systems, Inc.	\N
0004df	Teracom Telematica Ltda.	\N
0004e0	Procket Networks	\N
0004e1	Infinior Microsystems	\N
0004e2	SMC Networks, Inc.	\N
0004e3	Accton Technology Corp.	\N
0004e4	Daeryung Ind., Inc.	\N
0004e5	Glonet Systems, Inc.	\N
0004e6	Banyan Network Private Limited	\N
0004e7	Lightpointe Communications, Inc	\N
0004e8	IER, Inc.	\N
0004e9	Infiniswitch Corporation	\N
0004ea	Hewlett-Packard Company	\N
0004eb	Paxonet Communications, Inc.	\N
0004ec	Memobox SA	\N
0004ed	Billion Electric Co., Ltd.	\N
0004ee	Lincoln Electric Company	\N
0004ef	Polestar Corp.	\N
0004f0	International Computers, Ltd	\N
0004f1	WhereNet	\N
0004f2	Polycom	\N
0004f3	FS FORTH-SYSTEME GmbH	\N
0004f4	Infinite Electronics Inc.	\N
0004f5	SnowShore Networks, Inc.	\N
0004f6	Amphus	\N
0004f7	Omega Band, Inc.	\N
0004f8	QUALICABLE TV Industria E Com., Ltda	\N
0004f9	Xtera Communications, Inc.	\N
0004fa	NBS Technologies Inc.	\N
0004fb	Commtech, Inc.	\N
0004fc	Stratus Computer (DE), Inc.	\N
0004fd	Japan Control Engineering Co., Ltd.	\N
0004fe	Pelago Networks	\N
0004ff	Acronet Co., Ltd.	\N
000500	Cisco Systems, Inc.	\N
000501	Cisco Systems, Inc.	\N
000502	APPLE COMPUTER	\N
000503	ICONAG	\N
000504	Naray Information & Communication Enterprise	\N
000505	Systems Integration Solutions, Inc.	\N
000506	Reddo Networks AB	\N
000507	Fine Appliance Corp.	\N
000508	Inetcam, Inc.	\N
000509	AVOC Nishimura Ltd.	\N
00050a	ICS Spa	\N
00050b	SICOM Systems, Inc.	\N
00050c	Network Photonics, Inc.	\N
00050d	Midstream Technologies, Inc.	\N
00050e	3ware, Inc.	\N
00050f	Tanaka S/S Ltd.	\N
000510	Infinite Shanghai Communication Terminals Ltd.	\N
000511	Complementary Technologies Ltd	\N
000512	MeshNetworks, Inc.	\N
000513	VTLinx Multimedia Systems, Inc.	\N
000514	KDT Systems Co., Ltd.	\N
000515	Nuark Co., Ltd.	\N
000516	SMART Modular Technologies	\N
000517	Shellcomm, Inc.	\N
000518	Jupiters Technology	\N
000519	Siemens Building Technologies AG,	\N
00051a	3Com Europe Ltd.	\N
00051b	Magic Control Technology Corporation	\N
00051c	Xnet Technology Corp.	\N
00051d	Airocon, Inc.	\N
00051e	Brocade Communications Systems, Inc.	\N
00051f	Taijin Media Co., Ltd.	\N
000520	Smartronix, Inc.	\N
000521	Control Microsystems	\N
000522	LEA*D Corporation, Inc.	\N
000523	AVL List GmbH	\N
000524	BTL System (HK) Limited	\N
000525	Puretek Industrial Co., Ltd.	\N
000526	IPAS GmbH	\N
000527	SJ Tek Co. Ltd	\N
000528	New Focus, Inc.	\N
000529	Shanghai Broadan Communication Technology Co., Ltd	\N
00052a	Ikegami Tsushinki Co., Ltd.	\N
00052b	HORIBA, Ltd.	\N
00052c	Supreme Magic Corporation	\N
00052d	Zoltrix International Limited	\N
00052e	Cinta Networks	\N
00052f	Leviton Network Solutions	\N
000530	Andiamo Systems, Inc.	\N
000531	Cisco Systems, Inc.	\N
000532	Cisco Systems, Inc.	\N
000533	Brocade Communications Systems, Inc.	\N
000534	Northstar Engineering Ltd.	\N
000535	Chip PC Ltd.	\N
000536	Danam Communications, Inc.	\N
000537	Nets Technology Co., Ltd.	\N
000538	Merilus, Inc.	\N
000539	A Brand New World in Sweden AB	\N
00053a	Willowglen Services Pte Ltd	\N
00053b	Harbour Networks Ltd., Co. Beijing	\N
00053c	Xircom	\N
00053d	Agere Systems	\N
00053e	KID Systeme GmbH	\N
00053f	VisionTek, Inc.	\N
000540	FAST Corporation	\N
000541	Advanced Systems Co., Ltd.	\N
000542	Otari, Inc.	\N
000543	IQ Wireless GmbH	\N
000544	Valley Technologies, Inc.	\N
000545	Internet Photonics	\N
000546	KDDI Network & Solultions Inc.	\N
000547	Starent Networks	\N
000548	Disco Corporation	\N
000549	Salira Optical Network Systems	\N
00054a	Ario Data Networks, Inc.	\N
00054b	Micro Innovation AG	\N
00054c	RF Innovations Pty Ltd	\N
00054d	Brans Technologies, Inc.	\N
00054e	Philips	\N
00054f	PRIVATE	\N
000550	Vcomms Connect Limited	\N
000551	F & S Elektronik Systeme GmbH	\N
000552	Xycotec Computer GmbH	\N
000553	DVC Company, Inc.	\N
000554	Rangestar Wireless	\N
000555	Japan Cash Machine Co., Ltd.	\N
000556	360 Systems	\N
000557	Agile TV Corporation	\N
000558	Synchronous, Inc.	\N
000559	Intracom S.A.	\N
00055a	Power Dsine Ltd.	\N
00055b	Charles Industries, Ltd.	\N
00055c	Kowa Company, Ltd.	\N
00055d	D-Link Systems, Inc.	\N
00055e	Cisco Systems, Inc.	\N
00055f	Cisco Systems, Inc.	\N
000560	LEADER COMM.CO., LTD	\N
000561	nac Image Technology, Inc.	\N
000562	Digital View Limited	\N
000563	J-Works, Inc.	\N
000564	Tsinghua Bitway Co., Ltd.	\N
000565	Tailyn Communication Company Ltd.	\N
000566	Secui.com Corporation	\N
000567	Etymonic Design, Inc.	\N
000568	Piltofish Networks AB	\N
000569	VMware, Inc.	\N
00056a	Heuft Systemtechnik GmbH	\N
00056b	C.P. Technology Co., Ltd.	\N
00056c	Hung Chang Co., Ltd.	\N
00056d	Pacific Corporation	\N
00056e	National Enhance Technology, Inc.	\N
00056f	Innomedia Technologies Pvt. Ltd.	\N
000570	Baydel Ltd.	\N
000571	Seiwa Electronics Co.	\N
000572	Deonet Co., Ltd.	\N
000573	Cisco Systems, Inc.	\N
000574	Cisco Systems, Inc.	\N
000575	CDS-Electronics BV	\N
000576	NSM Technology Ltd.	\N
000577	SM Information & Communication	\N
000578	PRIVATE	\N
000579	Universal Control Solution Corp.	\N
00057a	Hatteras Networks	\N
00057b	Chung Nam Electronic Co., Ltd.	\N
00057c	RCO Security AB	\N
00057d	Sun Communications, Inc.	\N
00057e	Eckelmann Steuerungstechnik GmbH	\N
00057f	Acqis Technology	\N
000580	Fibrolan Ltd.	\N
000581	Snell & Wilcox Ltd.	\N
000582	ClearCube Technology	\N
000583	ImageCom Limited	\N
000584	AbsoluteValue Systems, Inc.	\N
000585	Juniper Networks, Inc.	\N
000586	Lucent Technologies	\N
000587	Locus, Incorporated	\N
000588	Sensoria Corp.	\N
000589	National Datacomputer	\N
00058a	Netcom Co., Ltd.	\N
00058b	IPmental, Inc.	\N
00058c	Opentech Inc.	\N
00058d	Lynx Photonic Networks, Inc.	\N
00058e	Flextronics International GmbH & Co. Nfg. KG	\N
00058f	CLCsoft co.	\N
000590	Swissvoice Ltd.	\N
000591	Active Silicon Ltd.	\N
000592	Pultek Corp.	\N
000593	Grammar Engine Inc.	\N
000594	IXXAT Automation GmbH	\N
000595	Alesis Corporation	\N
000596	Genotech Co., Ltd.	\N
000597	Eagle Traffic Control Systems	\N
000598	CRONOS S.r.l.	\N
000599	DRS Test and Energy Management or DRS-TEM	\N
00059a	Cisco Systems, Inc.	\N
00059b	Cisco Systems, Inc.	\N
00059c	Kleinknecht GmbH, Ing. Buero	\N
00059d	Daniel Computing Systems, Inc.	\N
00059e	Zinwell Corporation	\N
00059f	Yotta Networks, Inc.	\N
0005a0	MOBILINE Kft.	\N
0005a1	Zenocom	\N
0005a2	CELOX Networks	\N
0005a3	QEI, Inc.	\N
0005a4	Lucid Voice Ltd.	\N
0005a5	KOTT	\N
0005a6	Extron Electronics	\N
0005a7	Hyperchip, Inc.	\N
0005a8	WYLE ELECTRONICS	\N
0005a9	Princeton Networks, Inc.	\N
0005aa	Moore Industries International Inc.	\N
0005ab	Cyber Fone, Inc.	\N
0005ac	Northern Digital, Inc.	\N
0005ad	Topspin Communications, Inc.	\N
0005ae	Mediaport USA	\N
0005af	InnoScan Computing A/S	\N
0005b0	Korea Computer Technology Co., Ltd.	\N
0005b1	ASB Technology BV	\N
0005b2	Medison Co., Ltd.	\N
0005b3	Asahi-Engineering Co., Ltd.	\N
0005b4	Aceex Corporation	\N
0005b5	Broadcom Technologies	\N
0005b6	INSYS Microelectronics GmbH	\N
0005b7	Arbor Technology Corp.	\N
0005b8	Electronic Design Associates, Inc.	\N
0005b9	Airvana, Inc.	\N
0005ba	Area Netwoeks, Inc.	\N
0005bb	Myspace AB	\N
0005bc	Resorsys Ltd.	\N
0005bd	ROAX BV	\N
0005be	Kongsberg Seatex AS	\N
0005bf	JustEzy Technology, Inc.	\N
0005c0	Digital Network Alacarte Co., Ltd.	\N
0005c1	A-Kyung Motion, Inc.	\N
0005c2	Soronti, Inc.	\N
0005c3	Pacific Instruments, Inc.	\N
0005c4	Telect, Inc.	\N
0005c5	Flaga HF	\N
0005c6	Triz Communications	\N
0005c7	I/F-COM A/S	\N
0005c8	VERYTECH	\N
0005c9	LG Innotek Co., Ltd.	\N
0005ca	Hitron Technology, Inc.	\N
0005cb	ROIS Technologies, Inc.	\N
0005cc	Sumtel Communications, Inc.	\N
0005cd	Denon, Ltd.	\N
0005ce	Prolink Microsystems Corporation	\N
0005cf	Thunder River Technologies, Inc.	\N
0005d0	Solinet Systems	\N
0005d1	Metavector Technologies	\N
0005d2	DAP Technologies	\N
0005d3	eProduction Solutions, Inc.	\N
0005d4	FutureSmart Networks, Inc.	\N
0005d5	Speedcom Wireless	\N
0005d6	Titan Wireless	\N
0005d7	Vista Imaging, Inc.	\N
0005d8	Arescom, Inc.	\N
0005d9	Techno Valley, Inc.	\N
0005da	Apex Automationstechnik	\N
0005db	NENTEC Netzwerktechnologie GmbH	\N
0005dc	Cisco Systems, Inc.	\N
0005dd	Cisco Systems, Inc.	\N
0005de	Gi Fone Korea, Inc.	\N
0005df	Electronic Innovation, Inc.	\N
0005e0	Empirix Corp.	\N
0005e1	Trellis Photonics, Ltd.	\N
0005e2	Creativ Network Technologies	\N
0005e3	LightSand Communications, Inc.	\N
0005e4	Red Lion Controls Inc.	\N
0005e5	Renishaw PLC	\N
0005e6	Egenera, Inc.	\N
0005e7	Netrake an AudioCodes Company	\N
0005e8	TurboWave, Inc.	\N
0005e9	Unicess Network, Inc.	\N
0005ea	Rednix	\N
0005eb	Blue Ridge Networks, Inc.	\N
0005ec	Mosaic Systems Inc.	\N
0005ed	Technikum Joanneum GmbH	\N
0005ee	BEWATOR Group	\N
0005ef	ADOIR Digital Technology	\N
0005f0	SATEC	\N
0005f1	Vrcom, Inc.	\N
0005f2	Power R, Inc.	\N
0005f3	Weboyn	\N
0005f4	System Base Co., Ltd.	\N
0005f5	OYO Geospace	\N
0005f6	Young Chang Co. Ltd.	\N
0005f7	Analog Devices, Inc.	\N
0005f8	Real Time Access, Inc.	\N
0005f9	TOA Corporation	\N
0005fa	IPOptical, Inc.	\N
0005fb	ShareGate, Inc.	\N
0005fc	Schenck Pegasus Corp.	\N
0005fd	PacketLight Networks Ltd.	\N
0005fe	Traficon N.V.	\N
0005ff	SNS Solutions, Inc.	\N
000600	Toshiba Teli Corporation	\N
000601	Otanikeiki Co., Ltd.	\N
000602	Cirkitech Electronics Co.	\N
000603	Baker Hughes Inc.	\N
000604	@Track Communications, Inc.	\N
000605	Inncom International, Inc.	\N
000606	RapidWAN, Inc.	\N
000607	Omni Directional Control Technology Inc.	\N
000608	At-Sky SAS	\N
000609	Crossport Systems	\N
00060a	Blue2space	\N
00060b	Paceline Systems Corporation	\N
00060c	Melco Industries, Inc.	\N
00060d	Wave7 Optics	\N
00060e	IGYS Systems, Inc.	\N
00060f	Narad Networks Inc	\N
000610	Abeona Networks Inc	\N
000611	Zeus Wireless, Inc.	\N
000612	Accusys, Inc.	\N
000613	Kawasaki Microelectronics Incorporated	\N
000614	Prism Holdings	\N
000615	Kimoto Electric Co., Ltd.	\N
000616	Tel Net Co., Ltd.	\N
000617	Redswitch Inc.	\N
000618	DigiPower Manufacturing Inc.	\N
000619	Connection Technology Systems	\N
00061a	Zetari Inc.	\N
00061b	Notebook Development Lab.  Lenovo Japan Ltd.	\N
00061c	Hoshino Metal Industries, Ltd.	\N
00061d	MIP Telecom, Inc.	\N
00061e	Maxan Systems	\N
00061f	Vision Components GmbH	\N
000620	Serial System Ltd.	\N
000621	Hinox, Co., Ltd.	\N
000622	Chung Fu Chen Yeh Enterprise Corp.	\N
000623	MGE UPS Systems France	\N
000624	Gentner Communications Corp.	\N
000625	The Linksys Group, Inc.	\N
000626	MWE GmbH	\N
000627	Uniwide Technologies, Inc.	\N
000628	Cisco Systems, Inc.	\N
000629	IBM CORPORATION	\N
00062a	Cisco Systems, Inc.	\N
00062b	INTRASERVER TECHNOLOGY	\N
00062c	Bivio Networks	\N
00062d	TouchStar Technologies, L.L.C.	\N
00062e	Aristos Logic Corp.	\N
00062f	Pivotech Systems Inc.	\N
000630	Adtranz Sweden	\N
000631	Optical Solutions, Inc.	\N
000632	Mesco Engineering GmbH	\N
000633	Cross Match Technologies GmbH	\N
000634	GTE Airfone Inc.	\N
000635	PacketAir Networks, Inc.	\N
000636	Jedai Broadband Networks	\N
000637	Toptrend-Meta Information (ShenZhen) Inc.	\N
000638	Sungjin C&C Co., Ltd.	\N
000639	Newtec	\N
00063a	Dura Micro, Inc.	\N
00063b	Arcturus Networks, Inc.	\N
00063c	Intrinsyc Europe Ltd	\N
00063d	Microwave Data Systems Inc.	\N
00063e	Opthos Inc.	\N
00063f	Everex Communications Inc.	\N
000640	White Rock Networks	\N
000641	ITCN	\N
000642	Genetel Systems Inc.	\N
000643	SONO Computer Co., Ltd.	\N
000644	Neix,Inc	\N
000645	Meisei Electric Co. Ltd.	\N
000646	ShenZhen XunBao Network Technology Co Ltd	\N
000647	Etrali S.A.	\N
000648	Seedsware, Inc.	\N
000649	3M Deutschland GmbH	\N
00064a	Honeywell Co., Ltd. (KOREA)	\N
00064b	Alexon Co., Ltd.	\N
00064c	Invicta Networks, Inc.	\N
00064d	Sencore	\N
00064e	Broad Net Technology Inc.	\N
00064f	PRO-NETS Technology Corporation	\N
000650	Tiburon Networks, Inc.	\N
000651	Aspen Networks Inc.	\N
000652	Cisco Systems, Inc.	\N
000653	Cisco Systems, Inc.	\N
000654	Winpresa Building Automation Technologies GmbH	\N
000655	Yipee, Inc.	\N
000656	Tactel AB	\N
000657	Market Central, Inc.	\N
000658	Helmut Fischer GmbH Institut für Elektronik und Messtechnik	\N
000659	EAL (Apeldoorn) B.V.	\N
00065a	Strix Systems	\N
00065b	Dell Computer Corp.	\N
00065c	Malachite Technologies, Inc.	\N
00065d	Heidelberg Web Systems	\N
00065e	Photuris, Inc.	\N
00065f	ECI Telecom - NGTS Ltd.	\N
000660	NADEX Co., Ltd.	\N
000661	NIA Home Technologies Corp.	\N
000662	MBM Technology Ltd.	\N
000663	Human Technology Co., Ltd.	\N
000664	Fostex Corporation	\N
000665	Sunny Giken, Inc.	\N
000666	Roving Networks	\N
000667	Tripp Lite	\N
000668	Vicon Industries Inc.	\N
000669	Datasound Laboratories Ltd	\N
00066a	InfiniCon Systems, Inc.	\N
00066b	Sysmex Corporation	\N
00066c	Robinson Corporation	\N
00066d	Compuprint S.P.A.	\N
00066e	Delta Electronics, Inc.	\N
00066f	Korea Data Systems	\N
000670	Upponetti Oy	\N
000671	Softing AG	\N
000672	Netezza	\N
000673	Optelecom-nkf	\N
000674	Spectrum Control, Inc.	\N
000675	Banderacom, Inc.	\N
000676	Novra Technologies Inc.	\N
000677	SICK AG	\N
000678	Marantz Brand Company	\N
000679	Konami Corporation	\N
00067a	JMP Systems	\N
00067b	Toplink C&C Corporation	\N
00067c	CISCO SYSTEMS, INC.	\N
00067d	Takasago Ltd.	\N
00067e	WinCom Systems, Inc.	\N
00067f	Digeo, Inc.	\N
000680	Card Access, Inc.	\N
000681	Goepel Electronic GmbH	\N
000682	Convedia	\N
000683	Bravara Communications, Inc.	\N
000684	Biacore AB	\N
000685	NetNearU Corporation	\N
000686	ZARDCOM Co., Ltd.	\N
000687	Omnitron Systems Technology, Inc.	\N
000688	Telways Communication Co., Ltd.	\N
000689	yLez Technologies Pte Ltd	\N
00068a	NeuronNet Co. Ltd. R&D Center	\N
00068b	AirRunner Technologies, Inc.	\N
00068c	3Com Corporation	\N
00068d	SEPATON, Inc.	\N
00068e	HID Corporation	\N
00068f	Telemonitor, Inc.	\N
000690	Euracom Communication GmbH	\N
000691	PT Inovacao	\N
000692	Intruvert Networks, Inc.	\N
000693	Flexus Computer Technology, Inc.	\N
000694	Mobillian Corporation	\N
000695	Ensure Technologies, Inc.	\N
000696	Advent Networks	\N
000697	R & D Center	\N
000698	egnite Software GmbH	\N
000699	Vida Design Co.	\N
00069a	e & Tel	\N
00069b	AVT Audio Video Technologies GmbH	\N
00069c	Transmode Systems AB	\N
00069d	Petards Mobile Intelligence	\N
00069e	UNIQA, Inc.	\N
00069f	Kuokoa Networks	\N
0006a0	Mx Imaging	\N
0006a1	Celsian Technologies, Inc.	\N
0006a2	Microtune, Inc.	\N
0006a3	Bitran Corporation	\N
0006a4	INNOWELL Corp.	\N
0006a5	PINON Corp.	\N
0006a6	Artistic Licence (UK) Ltd	\N
0006a7	Primarion	\N
0006a8	KC Technology, Inc.	\N
0006a9	Universal Instruments Corp.	\N
0006aa	VT Miltope	\N
0006ab	W-Link Systems, Inc.	\N
0006ac	Intersoft Co.	\N
0006ad	KB Electronics Ltd.	\N
0006ae	Himachal Futuristic Communications Ltd	\N
0006af	Xalted Networks	\N
0006b0	Comtech EF Data Corp.	\N
0006b1	Sonicwall	\N
0006b2	Linxtek Co.	\N
0006b3	Diagraph Corporation	\N
0006b4	Vorne Industries, Inc.	\N
0006b5	Luminent, Inc.	\N
0006b6	Nir-Or Israel Ltd.	\N
0006b7	TELEM GmbH	\N
0006b8	Bandspeed Pty Ltd	\N
0006b9	A5TEK Corp.	\N
0006ba	Westwave Communications	\N
0006bb	ATI Technologies Inc.	\N
0006bc	Macrolink, Inc.	\N
0006bd	BNTECHNOLOGY Co., Ltd.	\N
0006be	Baumer Optronic GmbH	\N
0006bf	Accella Technologies Co., Ltd.	\N
0006c0	United Internetworks, Inc.	\N
0006c1	CISCO SYSTEMS, INC.	\N
0006c2	Smartmatic Corporation	\N
0006c3	Schindler Elevator Ltd.	\N
0006c4	Piolink Inc.	\N
0006c5	INNOVI Technologies Limited	\N
0006c6	lesswire AG	\N
0006c7	RFNET Technologies Pte Ltd (S)	\N
0006c8	Sumitomo Metal Micro Devices, Inc.	\N
0006c9	Technical Marketing Research, Inc.	\N
0006ca	American Computer & Digital Components, Inc. (ACDC)	\N
0006cb	Jotron Electronics A/S	\N
0006cc	JMI Electronics Co., Ltd.	\N
0006cd	Leaf Imaging Ltd.	\N
0006ce	DATENO	\N
0006cf	Thales Avionics In-Flight Systems, LLC	\N
0006d0	Elgar Electronics Corp.	\N
0006d1	Tahoe Networks, Inc.	\N
0006d2	Tundra Semiconductor Corp.	\N
0006d3	Alpha Telecom, Inc. U.S.A.	\N
0006d4	Interactive Objects, Inc.	\N
0006d5	Diamond Systems Corp.	\N
0006d6	Cisco Systems, Inc.	\N
0006d7	Cisco Systems, Inc.	\N
0006d8	Maple Optical Systems	\N
0006d9	IPM-Net S.p.A.	\N
0006da	ITRAN Communications Ltd.	\N
0006db	ICHIPS Co., Ltd.	\N
0006dc	Syabas Technology (Amquest)	\N
0006dd	AT & T Laboratories - Cambridge Ltd	\N
0006de	Flash Technology	\N
0006df	AIDONIC Corporation	\N
0006e0	MAT Co., Ltd.	\N
0006e1	Techno Trade s.a	\N
0006e2	Ceemax Technology Co., Ltd.	\N
0006e3	Quantitative Imaging Corporation	\N
0006e4	Citel Technologies Ltd.	\N
0006e5	Fujian Newland Computer Ltd. Co.	\N
0006e6	DongYang Telecom Co., Ltd.	\N
0006e7	Bit Blitz Communications Inc.	\N
0006e8	Optical Network Testing, Inc.	\N
0006e9	Intime Corp.	\N
0006ea	ELZET80 Mikrocomputer GmbH&Co. KG	\N
0006eb	Global Data	\N
0006ec	Harris Corporation	\N
0006ed	Inara Networks	\N
0006ee	Shenyang Neu-era Information & Technology Stock Co., Ltd	\N
0006ef	Maxxan Systems, Inc.	\N
0006f0	Digeo, Inc.	\N
0006f1	Optillion	\N
0006f2	Platys Communications	\N
0006f3	AcceLight Networks	\N
0006f4	Prime Electronics & Satellitics Inc.	\N
0006f8	CPU Technology, Inc.	\N
0006f9	Mitsui Zosen Systems Research Inc.	\N
0006fa	IP SQUARE Co, Ltd.	\N
0006fb	Hitachi Printing Solutions, Ltd.	\N
0006fc	Fnet Co., Ltd.	\N
0006fd	Comjet Information Systems Corp.	\N
0006fe	Ambrado, Inc	\N
0006ff	Sheba Systems Co., Ltd.	\N
000700	Zettamedia Korea	\N
000701	RACAL-DATACOM	\N
000702	Varian Medical Systems	\N
000703	CSEE Transport	\N
000705	Endress & Hauser GmbH & Co	\N
000706	Sanritz Corporation	\N
000707	Interalia Inc.	\N
000708	Bitrage Inc.	\N
000709	Westerstrand Urfabrik AB	\N
00070a	Unicom Automation Co., Ltd.	\N
00070b	Novabase SGPS, SA	\N
00070c	SVA-Intrusion.com Co. Ltd.	\N
00070d	Cisco Systems Inc.	\N
00070e	Cisco Systems Inc.	\N
00070f	Fujant, Inc.	\N
000710	Adax, Inc.	\N
000711	Acterna	\N
000712	JAL Information Technology	\N
000713	IP One, Inc.	\N
000714	Brightcom	\N
000715	General Research of Electronics, Inc.	\N
000716	J & S Marine Ltd.	\N
000717	Wieland Electric GmbH	\N
000718	iCanTek Co., Ltd.	\N
000719	Mobiis Co., Ltd.	\N
00071a	Finedigital Inc.	\N
00071b	CDV Americas Ltd	\N
00071c	AT&T Fixed Wireless Services	\N
00071d	Satelsa Sistemas Y Aplicaciones De Telecomunicaciones, S.A.	\N
00071e	Tri-M Engineering / Nupak Dev. Corp.	\N
00071f	European Systems Integration	\N
000720	Trutzschler GmbH & Co. KG	\N
000721	Formac Elektronik GmbH	\N
000722	The Nielsen Company	\N
000723	ELCON Systemtechnik GmbH	\N
000724	Telemax Co., Ltd.	\N
000725	Bematech International Corp.	\N
000727	Zi Corporation (HK) Ltd.	\N
000728	Neo Telecom	\N
000729	Kistler Instrumente AG	\N
00072a	Innovance Networks	\N
00072b	Jung Myung Telecom Co., Ltd.	\N
00072c	Fabricom	\N
00072d	CNSystems	\N
00072e	North Node AB	\N
00072f	Intransa, Inc.	\N
000730	Hutchison OPTEL Telecom Technology Co., Ltd.	\N
000731	Ophir-Spiricon Inc	\N
000732	AAEON Technology Inc.	\N
000733	DANCONTROL Engineering	\N
000734	ONStor, Inc.	\N
000735	Flarion Technologies, Inc.	\N
000736	Data Video Technologies Co., Ltd.	\N
000737	Soriya Co. Ltd.	\N
000738	Young Technology Co., Ltd.	\N
000739	Scotty Group Austria Gmbh	\N
00073a	Inventel Systemes	\N
00073b	Tenovis GmbH & Co KG	\N
00073c	Telecom Design	\N
00073d	Nanjing Postel Telecommunications Co., Ltd.	\N
00073e	China Great-Wall Computer Shenzhen Co., Ltd.	\N
00073f	Woojyun Systec Co., Ltd.	\N
000740	Buffalo, Inc	\N
000741	Sierra Automated Systems	\N
000742	Current Technologies, LLC	\N
000743	Chelsio Communications	\N
000744	Unico, Inc.	\N
000745	Radlan Computer Communications Ltd.	\N
000746	TURCK, Inc.	\N
000747	Mecalc	\N
000748	The Imaging Source Europe	\N
000749	CENiX Inc.	\N
00074a	Carl Valentin GmbH	\N
00074b	Daihen Corporation	\N
00074c	Beicom Inc.	\N
00074d	Zebra Technologies Corp.	\N
00074e	Naughty boy co., Ltd.	\N
00074f	Cisco Systems, Inc.	\N
000750	Cisco Systems, Inc.	\N
000751	m·u·t AG	\N
000752	Rhythm Watch Co., Ltd.	\N
000753	Beijing Qxcomm Technology Co., Ltd.	\N
000754	Xyterra Computing, Inc.	\N
000755	Lafon SA	\N
000756	Juyoung Telecom	\N
000757	Topcall International AG	\N
000758	Dragonwave	\N
000759	Boris Manufacturing Corp.	\N
00075a	Air Products and Chemicals, Inc.	\N
00075b	Gibson Guitars	\N
00075c	Eastman Kodak Company	\N
00075d	Celleritas Inc.	\N
00075e	Ametek Power Instruments	\N
00075f	VCS Video Communication Systems AG	\N
000760	TOMIS Information & Telecom Corp.	\N
000761	Logitech SA	\N
000762	Group Sense Limited	\N
000763	Sunniwell Cyber Tech. Co., Ltd.	\N
000764	YoungWoo Telecom Co. Ltd.	\N
000765	Jade Quantum Technologies, Inc.	\N
000766	Chou Chin Industrial Co., Ltd.	\N
000767	Yuxing Electronics Company Limited	\N
000768	Danfoss A/S	\N
000769	Italiana Macchi SpA	\N
00076a	NEXTEYE Co., Ltd.	\N
00076b	Stralfors AB	\N
00076c	Daehanet, Inc.	\N
00076d	Flexlight Networks	\N
00076e	Sinetica Corporation Limited	\N
00076f	Synoptics Limited	\N
000770	Locusnetworks Corporation	\N
000771	Embedded System Corporation	\N
000772	Alcatel Shanghai Bell Co., Ltd.	\N
000773	Ascom Powerline Communications Ltd.	\N
000774	GuangZhou Thinker Technology Co. Ltd.	\N
000775	Valence Semiconductor, Inc.	\N
000776	Federal APD	\N
000777	Motah Ltd.	\N
000778	GERSTEL GmbH & Co. KG	\N
000779	Sungil Telecom Co., Ltd.	\N
00077a	Infoware System Co., Ltd.	\N
00077b	Millimetrix Broadband Networks	\N
00077c	Westermo Teleindustri AB	\N
00077e	Elrest GmbH	\N
00077f	J Communications Co., Ltd.	\N
000780	Bluegiga Technologies OY	\N
000781	Itron Inc.	\N
000782	Sun Microsystems, Inc.	\N
000783	SynCom Network, Inc.	\N
000784	Cisco Systems Inc.	\N
000785	Cisco Systems Inc.	\N
000786	Wireless Networks Inc.	\N
000787	Idea System Co., Ltd.	\N
000788	Clipcomm, Inc.	\N
000789	Eastel Systems Corporation	\N
00078a	Mentor Data System Inc.	\N
00078b	Wegener Communications, Inc.	\N
00078c	Elektronikspecialisten i Borlange AB	\N
00078d	NetEngines Ltd.	\N
00078e	Garz & Friche GmbH	\N
00078f	Emkay Innovative Products	\N
000790	Tri-M Technologies (s) Limited	\N
000791	International Data Communications, Inc.	\N
000792	Suetron Electronic GmbH	\N
000793	Shin Satellite Public Company Limited	\N
000794	Simple Devices, Inc.	\N
000795	Elitegroup Computer System Co. (ECS)	\N
000796	LSI Systems, Inc.	\N
000797	Netpower Co., Ltd.	\N
000798	Selea SRL	\N
000799	Tipping Point Technologies, Inc.	\N
00079a	Verint Systems Inc	\N
00079b	Aurora Networks	\N
00079c	Golden Electronics Technology Co., Ltd.	\N
00079d	Musashi Co., Ltd.	\N
00079e	Ilinx Co., Ltd.	\N
00079f	Action Digital Inc.	\N
0007a0	e-Watch Inc.	\N
0007a1	VIASYS Healthcare GmbH	\N
0007a2	Opteon Corporation	\N
0007a3	Ositis Software, Inc.	\N
0007a4	GN Netcom Ltd.	\N
0007a5	Y.D.K Co. Ltd.	\N
0007a6	Home Automation, Inc.	\N
0007a7	A-Z Inc.	\N
0007a8	Haier Group Technologies Ltd.	\N
0007a9	Novasonics	\N
0007aa	Quantum Data Inc.	\N
0007ac	Eolring	\N
0007ad	Pentacon GmbH Foto-und Feinwerktechnik	\N
0007ae	Britestream Networks, Inc.	\N
0007af	N-Tron Corp.	\N
0007b0	Office Details, Inc.	\N
0007b1	Equator Technologies	\N
0007b2	Transaccess S.A.	\N
0007b3	Cisco Systems Inc.	\N
0007b4	Cisco Systems Inc.	\N
0007b5	Any One Wireless Ltd.	\N
0007b6	Telecom Technology Ltd.	\N
0007b7	Samurai Ind. Prods Eletronicos Ltda	\N
0007b8	Corvalent Corporation	\N
0007b9	Ginganet Corporation	\N
0007ba	UTStarcom, Inc.	\N
0007bb	Candera Inc.	\N
0007bc	Identix Inc.	\N
0007bd	Radionet Ltd.	\N
0007be	DataLogic SpA	\N
0007bf	Armillaire Technologies, Inc.	\N
0007c0	NetZerver Inc.	\N
0007c1	Overture Networks, Inc.	\N
0007c2	Netsys Telecom	\N
0007c3	Thomson	\N
0007c4	JEAN Co. Ltd.	\N
0007c5	Gcom, Inc.	\N
0007c6	VDS Vosskuhler GmbH	\N
0007c7	Synectics Systems Limited	\N
0007c8	Brain21, Inc.	\N
0007c9	Technol Seven Co., Ltd.	\N
0007ca	Creatix Polymedia Ges Fur Kommunikaitonssysteme	\N
0007cb	Freebox SA	\N
0007cc	Kaba Benzing GmbH	\N
0007cd	NMTEL Co., Ltd.	\N
0007ce	Cabletime Limited	\N
0007cf	Anoto AB	\N
0007d0	Automat Engenharia de Automaoa Ltda.	\N
0007d1	Spectrum Signal Processing Inc.	\N
0007d2	Logopak Systeme	\N
0007d3	Stork Digital Imaging B.V.	\N
0007d4	Zhejiang Yutong Network Communication Co Ltd.	\N
0007d5	3e Technologies Int;., Inc.	\N
0007d6	Commil Ltd.	\N
0007d7	Caporis Networks AG	\N
0007d8	Hitron Systems Inc.	\N
0007d9	Splicecom	\N
0007da	Neuro Telecom Co., Ltd.	\N
0007db	Kirana Networks, Inc.	\N
0007dc	Atek Co, Ltd.	\N
0007dd	Cradle Technologies	\N
0007de	eCopilt AB	\N
0007df	Vbrick Systems Inc.	\N
0007e0	Palm Inc.	\N
0007e1	WIS Communications Co. Ltd.	\N
0007e2	Bitworks, Inc.	\N
0007e3	Navcom Technology, Inc.	\N
0007e4	SoftRadio Co., Ltd.	\N
0007e5	Coup Corporation	\N
0007e6	edgeflow Canada Inc.	\N
0007e7	FreeWave Technologies	\N
0007e8	St. Bernard Software	\N
0007e9	Intel Corporation	\N
0007ea	Massana, Inc.	\N
0007eb	Cisco Systems Inc.	\N
0007ec	Cisco Systems Inc.	\N
0007ed	Altera Corporation	\N
0007ee	telco Informationssysteme GmbH	\N
0007ef	Lockheed Martin Tactical Systems	\N
0007f0	Beckett LogiSync LLC	\N
0007f1	TeraBurst Networks Inc.	\N
0007f2	IOA Corporation	\N
0007f3	Thinkengine Networks	\N
0007f4	Eletex Co., Ltd.	\N
0007f5	Bridgeco Co AG	\N
0007f6	Qqest Software Systems	\N
0007f7	Galtronics	\N
0007f8	ITDevices, Inc.	\N
0007f9	Phonetics, Inc.	\N
0007fa	ITT Co., Ltd.	\N
0007fb	Giga Stream UMTS Technologies GmbH	\N
0007fc	Adept Systems Inc.	\N
0007fd	LANergy Ltd.	\N
0007fe	Rigaku Corporation	\N
0007ff	Gluon Networks	\N
000800	MULTITECH SYSTEMS, INC.	\N
000801	HighSpeed Surfing Inc.	\N
000802	Hewlett Packard	\N
000803	Cos Tron	\N
000804	ICA Inc.	\N
000805	Techno-Holon Corporation	\N
000806	Raonet Systems, Inc.	\N
000807	Access Devices Limited	\N
000808	PPT Vision, Inc.	\N
000809	Systemonic AG	\N
00080a	Espera-Werke GmbH	\N
00080b	Birka BPA Informationssystem AB	\N
00080c	VDA Elettronica spa	\N
00080d	Toshiba	\N
00080e	Motorola, BCS	\N
00080f	Proximion Fiber Optics AB	\N
000810	Key Technology, Inc.	\N
000811	VOIX Corporation	\N
000812	GM-2 Corporation	\N
000813	Diskbank, Inc.	\N
000814	TIL Technologies	\N
000815	CATS Co., Ltd.	\N
000816	Bluetags A/S	\N
000817	EmergeCore Networks LLC	\N
000818	Pixelworks, Inc.	\N
000819	Banksys	\N
00081a	Sanrad Intelligence Storage Communications (2000) Ltd.	\N
00081b	Windigo Systems	\N
00081c	@pos.com	\N
00081d	Ipsil, Incorporated	\N
00081e	Repeatit AB	\N
00081f	Pou Yuen Tech Corp. Ltd.	\N
000820	Cisco Systems Inc.	\N
000821	Cisco Systems Inc.	\N
000822	InPro Comm	\N
000823	Texa Corp.	\N
000824	Copitrak Inc	\N
000825	Acme Packet	\N
000826	Colorado Med Tech	\N
000827	Pirelli Broadband Solutions	\N
000828	Koei Engineering Ltd.	\N
000829	Aval Nagasaki Corporation	\N
00082a	Powerwallz Network Security	\N
00082b	Wooksung Electronics, Inc.	\N
00082c	Homag AG	\N
00082d	Indus Teqsite Private Limited	\N
00082e	Multitone Electronics PLC	\N
00084e	DivergeNet, Inc.	\N
00084f	Qualstar Corporation	\N
000850	Arizona Instrument Corp.	\N
000851	Canadian Bank Note Company, Ltd.	\N
000852	Davolink Co. Inc.	\N
000853	Schleicher GmbH & Co. Relaiswerke KG	\N
000854	Netronix, Inc.	\N
000855	NASA-Goddard Space Flight Center	\N
000856	Gamatronic Electronic Industries Ltd.	\N
000857	Polaris Networks, Inc.	\N
000858	Novatechnology Inc.	\N
000859	ShenZhen Unitone Electronics Co., Ltd.	\N
00085a	IntiGate Inc.	\N
00085b	Hanbit Electronics Co., Ltd.	\N
00085c	Shanghai Dare Technologies Co. Ltd.	\N
00085d	Aastra	\N
00085e	PCO AG	\N
00085f	Picanol N.V.	\N
000860	LodgeNet Entertainment Corp.	\N
000861	SoftEnergy Co., Ltd.	\N
000862	NEC Eluminant Technologies, Inc.	\N
000863	Entrisphere Inc.	\N
000864	Fasy S.p.A.	\N
000865	JASCOM CO., LTD	\N
000866	DSX Access Systems, Inc.	\N
000867	Uptime Devices	\N
000868	PurOptix	\N
000869	Command-e Technology Co.,Ltd.	\N
00086a	Industrie Technik IPS GmbH	\N
00086b	MIPSYS	\N
00086c	Plasmon LMS	\N
00086d	Missouri FreeNet	\N
00086e	Hyglo AB	\N
00086f	Resources Computer Network Ltd.	\N
000870	Rasvia Systems, Inc.	\N
000871	NORTHDATA Co., Ltd.	\N
000872	Sorenson Communications	\N
000873	DapTechnology B.V.	\N
000874	Dell Computer Corp.	\N
000875	Acorp Electronics Corp.	\N
000876	SDSystem	\N
000877	Liebert-Hiross Spa	\N
000878	Benchmark Storage Innovations	\N
000879	CEM Corporation	\N
00087a	Wipotec GmbH	\N
00087b	RTX Telecom A/S	\N
00087c	Cisco Systems, Inc.	\N
00087d	Cisco Systems Inc.	\N
00087e	Bon Electro-Telecom Inc.	\N
00087f	SPAUN electronic GmbH & Co. KG	\N
000880	BroadTel Canada Communications inc.	\N
000881	DIGITAL HANDS CO.,LTD.	\N
000882	SIGMA CORPORATION	\N
000883	Hewlett-Packard Company	\N
000884	Index Braille AB	\N
000885	EMS Dr. Thomas Wuensche	\N
000886	Hansung Teliann, Inc.	\N
000887	Maschinenfabrik Reinhausen GmbH	\N
000888	OULLIM Information Technology Inc,.	\N
000889	Echostar Technologies Corp	\N
00088a	Minds@Work	\N
00088b	Tropic Networks Inc.	\N
00088c	Quanta Network Systems Inc.	\N
00088d	Sigma-Links Inc.	\N
00088e	Nihon Computer Co., Ltd.	\N
00088f	ADVANCED DIGITAL TECHNOLOGY	\N
000890	AVILINKS SA	\N
000891	Lyan Inc.	\N
000892	EM Solutions	\N
000893	LE INFORMATION COMMUNICATION INC.	\N
000894	InnoVISION Multimedia Ltd.	\N
000895	DIRC Technologie GmbH & Co.KG	\N
000896	Printronix, Inc.	\N
000897	Quake Technologies	\N
000898	Gigabit Optics Corporation	\N
000899	Netbind, Inc.	\N
00089a	Alcatel Microelectronics	\N
00089b	ICP Electronics Inc.	\N
00089c	Elecs Industry Co., Ltd.	\N
00089d	UHD-Elektronik	\N
00089e	Beijing Enter-Net co.LTD	\N
00089f	EFM Networks	\N
0008a0	Stotz Feinmesstechnik GmbH	\N
0008a1	CNet Technology Inc.	\N
0008a2	ADI Engineering, Inc.	\N
0008a3	Cisco Systems	\N
0008a4	Cisco Systems	\N
0008a5	Peninsula Systems Inc.	\N
0008a6	Multiware & Image Co., Ltd.	\N
0008a7	iLogic Inc.	\N
0008a8	Systec Co., Ltd.	\N
0008a9	SangSang Technology, Inc.	\N
0008aa	KARAM	\N
0008ab	EnerLinx.com, Inc.	\N
0008ac	Eltromat GmbH	\N
0008ad	Toyo-Linx Co., Ltd.	\N
0008ae	PacketFront Sweden AB	\N
0008af	Novatec Corporation	\N
0008b0	BKtel communications GmbH	\N
0008b1	ProQuent Systems	\N
0008b2	SHENZHEN COMPASS TECHNOLOGY DEVELOPMENT CO.,LTD	\N
0008b3	Fastwel	\N
0008b4	SYSPOL	\N
0008b5	TAI GUEN ENTERPRISE CO., LTD	\N
0008b6	RouteFree, Inc.	\N
0008b7	HIT Incorporated	\N
0008b8	E.F. Johnson	\N
0008b9	KAON MEDIA Co., Ltd.	\N
0008ba	Erskine Systems Ltd	\N
0008bb	NetExcell	\N
0008bc	Ilevo AB	\N
0008bd	TEPG-US	\N
0008be	XENPAK MSA Group	\N
0008bf	Aptus Elektronik AB	\N
0008c0	ASA SYSTEMS	\N
0008c1	Avistar Communications Corporation	\N
0008c2	Cisco Systems	\N
0008c3	Contex A/S	\N
0008c4	Hikari Co.,Ltd.	\N
0008c5	Liontech Co., Ltd.	\N
0008c6	Philips Consumer Communications	\N
0008c7	Hewlett Packard	\N
0008c8	Soneticom, Inc.	\N
0008c9	TechniSat Digital GmbH	\N
0008ca	TwinHan Technology Co.,Ltd	\N
0008cb	Zeta Broadband Inc.	\N
0008cc	Remotec, Inc.	\N
0008cd	With-Net Inc	\N
0008ce	IPMobileNet Inc.	\N
0008cf	Nippon Koei Power Systems Co., Ltd.	\N
0008d0	Musashi Engineering Co., LTD.	\N
0008d1	KAREL INC.	\N
0008d2	ZOOM Networks Inc.	\N
0008d3	Hercules Technologies S.A.	\N
0008d4	IneoQuest Technologies, Inc	\N
0008d5	Vanguard Networks Solutions, LLC	\N
0008d6	HASSNET Inc.	\N
0008d7	HOW CORPORATION	\N
0008d8	Dowkey Microwave	\N
0008d9	Mitadenshi Co.,LTD	\N
0008da	SofaWare Technologies Ltd.	\N
0008db	Corrigent Systems	\N
0008dc	Wiznet	\N
0008dd	Telena Communications, Inc.	\N
0008de	3UP Systems	\N
0008df	Alistel Inc.	\N
0008e0	ATO Technology Ltd.	\N
0008e1	Barix AG	\N
0008e2	Cisco Systems	\N
0008e3	Cisco Systems	\N
0008e4	Envenergy Inc	\N
0008e5	IDK Corporation	\N
0008e6	Littlefeet	\N
0008e7	SHI ControlSystems,Ltd.	\N
0008e8	Excel Master Ltd.	\N
0008e9	NextGig	\N
0008ea	Motion Control Engineering, Inc	\N
0008eb	ROMWin Co.,Ltd.	\N
0008ec	Optical Zonu Corporation	\N
0008ed	ST&T Instrument Corp.	\N
0008ee	Logic Product Development	\N
0008ef	DIBAL,S.A.	\N
0008f0	Next Generation Systems, Inc.	\N
0008f1	Voltaire	\N
0008f2	C&S Technology	\N
0008f3	WANY	\N
0008f4	Bluetake Technology Co., Ltd.	\N
0008f5	YESTECHNOLOGY Co.,Ltd.	\N
0008f6	Sumitomo Electric System Solutions Co.,Ltd.	\N
0008f7	Hitachi Ltd, Semiconductor &amp; Integrated Circuits Gr	\N
0008f8	Guardall Ltd	\N
0008f9	Padcom, Inc.	\N
0008fa	Karl E.Brinkmann GmbH	\N
0008fb	SonoSite, Inc.	\N
0008fc	Gigaphoton Inc.	\N
0008fd	BlueKorea Co., Ltd.	\N
0008fe	UNIK C&C Co.,Ltd.	\N
0008ff	Trilogy Communications Ltd	\N
000900	TMT	\N
000901	Shenzhen Shixuntong Information & Technoligy Co	\N
000902	Redline Communications Inc.	\N
000903	Panasas, Inc	\N
000904	MONDIAL electronic	\N
000905	iTEC Technologies Ltd.	\N
000906	Esteem Networks	\N
000907	Chrysalis Development	\N
000908	VTech Technology Corp.	\N
000909	Telenor Connect A/S	\N
00090a	SnedFar Technology Co., Ltd.	\N
00090b	MTL  Instruments PLC	\N
00090c	Mayekawa Mfg. Co. Ltd.	\N
00090d	LEADER ELECTRONICS CORP.	\N
00090e	Helix Technology Inc.	\N
00090f	Fortinet Inc.	\N
000910	Simple Access Inc.	\N
000911	Cisco Systems	\N
000912	Cisco Systems	\N
000913	SystemK Corporation	\N
000914	COMPUTROLS INC.	\N
000915	CAS Corp.	\N
000916	Listman Home Technologies, Inc.	\N
000917	WEM Technology Inc	\N
000918	SAMSUNG TECHWIN CO.,LTD	\N
000919	MDS Gateways	\N
00091a	Macat Optics & Electronics Co., Ltd.	\N
00091b	Digital Generation Inc.	\N
00091c	CacheVision, Inc	\N
00091d	Proteam Computer Corporation	\N
00091e	Firstech Technology Corp.	\N
00091f	A&amp;D Co., Ltd.	\N
000920	EpoX COMPUTER CO.,LTD.	\N
000921	Planmeca Oy	\N
000922	TST Biometrics GmbH	\N
000923	Heaman System Co., Ltd	\N
000924	Telebau GmbH	\N
000925	VSN Systemen BV	\N
000926	YODA COMMUNICATIONS, INC.	\N
000927	TOYOKEIKI CO.,LTD.	\N
000928	Telecore	\N
000929	Sanyo Industries (UK) Limited	\N
00092a	MYTECS Co.,Ltd.	\N
00092b	iQstor Networks, Inc.	\N
00092c	Hitpoint Inc.	\N
00092d	HTC Corporation	\N
00092e	B&Tech System Inc.	\N
00092f	Akom Technology Corporation	\N
000930	AeroConcierge Inc.	\N
000931	Future Internet, Inc.	\N
000932	Omnilux	\N
000933	OPTOVALLEY Co. Ltd.	\N
000934	Dream-Multimedia-Tv GmbH	\N
000935	Sandvine Incorporated	\N
000936	Ipetronik GmbH & Co.KG	\N
000937	Inventec Appliance Corp	\N
000938	Allot Communications	\N
000939	ShibaSoku Co.,Ltd.	\N
00093a	Molex Fiber Optics	\N
00093b	HYUNDAI NETWORKS INC.	\N
00093c	Jacques Technologies P/L	\N
00093d	Newisys,Inc.	\N
00093e	C&I Technologies	\N
00093f	Double-Win Enterpirse CO., LTD	\N
000940	AGFEO GmbH & Co. KG	\N
000941	Allied Telesis K.K.	\N
000942	CRESCO, LTD.	\N
000943	Cisco Systems	\N
000944	Cisco Systems	\N
000945	Palmmicro Communications Inc	\N
000946	Cluster Labs GmbH	\N
000947	Aztek, Inc.	\N
000948	Vista Control Systems, Corp.	\N
000949	Glyph Technologies Inc.	\N
00094a	Homenet Communications	\N
00094b	FillFactory NV	\N
00094c	Communication Weaver Co.,Ltd.	\N
00094d	Braintree Communications Pty Ltd	\N
00094e	BARTECH SYSTEMS INTERNATIONAL, INC	\N
00094f	elmegt GmbH & Co. KG	\N
000950	Independent Storage Corporation	\N
000951	Apogee Instruments, Inc	\N
000952	Auerswald GmbH & Co. KG	\N
000953	Linkage System Integration Co.Ltd.	\N
000954	AMiT spol. s. r. o.	\N
000955	Young Generation International Corp.	\N
000956	Network Systems Group, Ltd. (NSG)	\N
000957	Supercaller, Inc.	\N
000958	INTELNET S.A.	\N
000959	Sitecsoft	\N
00095a	RACEWOOD TECHNOLOGY	\N
00095b	Netgear, Inc.	\N
00095c	Philips Medical Systems - Cardiac and Monitoring Systems (CM	\N
00095d	Dialogue Technology Corp.	\N
00095e	Masstech Group Inc.	\N
00095f	Telebyte, Inc.	\N
000960	YOZAN Inc.	\N
000961	Switchgear and Instrumentation Ltd	\N
000962	Sonitor Technologies AS	\N
000963	Dominion Lasercom Inc.	\N
000964	Hi-Techniques, Inc.	\N
000965	HyunJu Computer Co., Ltd.	\N
000966	Thales Navigation	\N
000967	Tachyon, Inc	\N
000968	TECHNOVENTURE, INC.	\N
000969	Meret Optical Communications	\N
00096a	Cloverleaf Communications Inc.	\N
00096b	IBM Corporation	\N
00096c	Imedia Semiconductor Corp.	\N
00096d	Powernet Technologies Corp.	\N
00096e	GIANT ELECTRONICS LTD.	\N
00096f	Beijing Zhongqing Elegant Tech. Corp.,Limited	\N
000970	Vibration Research Corporation	\N
000971	Time Management, Inc.	\N
000972	Securebase,Inc	\N
000973	Lenten Technology Co., Ltd.	\N
000974	Innopia Technologies, Inc.	\N
000975	fSONA Communications Corporation	\N
000976	Datasoft ISDN Systems GmbH	\N
000977	Brunner Elektronik AG	\N
000978	AIJI System Co., Ltd.	\N
000979	Advanced Television Systems Committee, Inc.	\N
00097a	Louis Design Labs.	\N
00097b	Cisco Systems	\N
00097c	Cisco Systems	\N
00097d	SecWell Networks Oy	\N
00097e	IMI TECHNOLOGY CO., LTD	\N
00097f	Vsecure 2000 LTD.	\N
000980	Power Zenith Inc.	\N
000981	Newport Networks	\N
000982	Loewe Opta GmbH	\N
000983	Gvision Incorporated	\N
000984	MyCasa Network Inc.	\N
000985	Auto Telecom Company	\N
000986	Metalink LTD.	\N
000987	NISHI NIPPON ELECTRIC WIRE & CABLE CO.,LTD.	\N
000988	Nudian Electron Co., Ltd.	\N
000989	VividLogic Inc.	\N
00098a	EqualLogic Inc	\N
00098b	Entropic Communications, Inc.	\N
00098c	Option Wireless Sweden	\N
00098d	Velocity Semiconductor	\N
00098e	ipcas GmbH	\N
00098f	Cetacean Networks	\N
000990	ACKSYS Communications & systems	\N
000991	GE Fanuc Automation Manufacturing, Inc.	\N
000992	InterEpoch Technology,INC.	\N
000993	Visteon Corporation	\N
000994	Cronyx Engineering	\N
000995	Castle Technology Ltd	\N
000996	RDI	\N
000997	Nortel Networks	\N
000998	Capinfo Company Limited	\N
000999	CP GEORGES RENAULT	\N
00099a	ELMO COMPANY, LIMITED	\N
00099b	Western Telematic Inc.	\N
00099c	Naval Research Laboratory	\N
00099d	Haliplex Communications	\N
00099e	Testech, Inc.	\N
00099f	VIDEX INC.	\N
0009a0	Microtechno Corporation	\N
0009a1	Telewise Communications, Inc.	\N
0009a2	Interface Co., Ltd.	\N
0009a3	Leadfly Techologies Corp. Ltd.	\N
0009a4	HARTEC Corporation	\N
0009a5	HANSUNG ELETRONIC INDUSTRIES DEVELOPMENT CO., LTD	\N
0009a6	Ignis Optics, Inc.	\N
0009a7	Bang & Olufsen A/S	\N
0009a8	Eastmode Pte Ltd	\N
0009a9	Ikanos Communications	\N
0009aa	Data Comm for Business, Inc.	\N
0009ab	Netcontrol Oy	\N
0009ac	LANVOICE	\N
0009ad	HYUNDAI SYSCOMM, INC.	\N
0009ae	OKANO ELECTRIC CO.,LTD	\N
0009af	e-generis	\N
0009b0	Onkyo Corporation	\N
0009b1	Kanematsu Electronics, Ltd.	\N
0009b2	L&F Inc.	\N
0009b3	MCM Systems Ltd	\N
0009b4	KISAN TELECOM CO., LTD.	\N
0009b5	3J Tech. Co., Ltd.	\N
0009b6	Cisco Systems	\N
0009b7	Cisco Systems	\N
0009b8	Entise Systems	\N
0009b9	Action Imaging Solutions	\N
0009ba	MAKU Informationstechik GmbH	\N
0009bb	MathStar, Inc.	\N
0009bc	Digital Safety Technologies, Inc	\N
0009bd	Epygi Technologies, Ltd.	\N
0009be	Mamiya-OP Co.,Ltd.	\N
0009bf	Nintendo Co.,Ltd.	\N
0009c0	6WIND	\N
0009c1	PROCES-DATA A/S	\N
0009c2	Onity, Inc.	\N
0009c3	NETAS	\N
0009c4	Medicore Co., Ltd	\N
0009c5	KINGENE Technology Corporation	\N
0009c6	Visionics Corporation	\N
0009c7	Movistec	\N
0009c8	SINAGAWA TSUSHIN KEISOU SERVICE	\N
0009c9	BlueWINC Co., Ltd.	\N
0009ca	iMaxNetworks(Shenzhen)Limited.	\N
0009cb	HBrain	\N
0009cc	Moog GmbH	\N
0009cd	HUDSON SOFT CO.,LTD.	\N
0009ce	SpaceBridge Semiconductor Corp.	\N
0009cf	iAd GmbH	\N
0009d0	Solacom Technologies Inc.	\N
0009d1	SERANOA NETWORKS INC	\N
0009d2	Mai Logic Inc.	\N
0009d3	Western DataCom Co., Inc.	\N
0009d4	Transtech Networks	\N
0009d5	Signal Communication, Inc.	\N
0009d6	KNC One GmbH	\N
0009d7	DC Security Products	\N
0009d8	Fält Communications AB	\N
0009d9	Neoscale Systems, Inc	\N
0009da	Control Module Inc.	\N
0009db	eSpace	\N
0009dc	Galaxis Technology AG	\N
0009dd	Mavin Technology Inc.	\N
0009de	Samjin Information & Communications Co., Ltd.	\N
0009df	Vestel Komunikasyon Sanayi ve Ticaret A.S.	\N
0009e0	XEMICS S.A.	\N
0009e1	Gemtek Technology Co., Ltd.	\N
0009e2	Sinbon Electronics Co., Ltd.	\N
0009e3	Angel Iglesias S.A.	\N
0009e4	K Tech Infosystem Inc.	\N
0009e5	Hottinger Baldwin Messtechnik GmbH	\N
0009e6	Cyber Switching Inc.	\N
0009e7	ADC Techonology	\N
0009e8	Cisco Systems	\N
0009e9	Cisco Systems	\N
0009ea	YEM Inc.	\N
0009eb	HuMANDATA LTD.	\N
0009ec	Daktronics, Inc.	\N
0009ed	CipherOptics	\N
0009ee	MEIKYO ELECTRIC CO.,LTD	\N
0009ef	Vocera Communications	\N
0009f0	Shimizu Technology Inc.	\N
0009f1	Yamaki Electric Corporation	\N
0009f2	Cohu, Inc., Electronics Division	\N
0009f3	WELL Communication Corp.	\N
0009f4	Alcon Laboratories, Inc.	\N
0009f5	Emerson Network Power Co.,Ltd	\N
0009f6	Shenzhen Eastern Digital Tech Ltd.	\N
0009f7	SED, a division of Calian	\N
0009f8	UNIMO TECHNOLOGY CO., LTD.	\N
0009f9	ART JAPAN CO., LTD.	\N
0009fb	Philips Patient Monitoring	\N
0009fc	IPFLEX Inc.	\N
0009fd	Ubinetics Limited	\N
0009fe	Daisy Technologies, Inc.	\N
0009ff	X.net 2000 GmbH	\N
000a00	Mediatek Corp.	\N
000a01	SOHOware, Inc.	\N
000a02	ANNSO CO., LTD.	\N
000a03	ENDESA SERVICIOS, S.L.	\N
000a04	3Com Ltd	\N
000a05	Widax Corp.	\N
000a06	Teledex LLC	\N
000a07	WebWayOne Ltd	\N
000a08	ALPINE ELECTRONICS, INC.	\N
000a09	TaraCom Integrated Products, Inc.	\N
000a0a	SUNIX Co., Ltd.	\N
000a0b	Sealevel Systems, Inc.	\N
000a0c	Scientific Research Corporation	\N
000a0d	MergeOptics GmbH	\N
000a0e	Invivo Research Inc.	\N
000a0f	Ilryung Telesys, Inc	\N
000a10	FAST media integrations AG	\N
000a11	ExPet Technologies, Inc	\N
000a12	Azylex Technology, Inc	\N
000a13	Honeywell Video Systems	\N
000a14	TECO a.s.	\N
000a15	Silicon Data, Inc	\N
000a16	Lassen Research	\N
000a17	NESTAR COMMUNICATIONS, INC	\N
000a18	Vichel Inc.	\N
000a19	Valere Power, Inc.	\N
000a1a	Imerge Ltd	\N
000a1b	Stream Labs	\N
000a1c	Bridge Information Co., Ltd.	\N
000a1d	Optical Communications Products Inc.	\N
000a1e	Red-M Products Limited	\N
000a1f	ART WARE Telecommunication Co., Ltd.	\N
000a20	SVA Networks, Inc.	\N
000a21	Integra Telecom Co. Ltd	\N
000a22	Amperion Inc	\N
000a23	Parama Networks Inc	\N
000a24	Octave Communications	\N
000a25	CERAGON NETWORKS	\N
000a26	CEIA S.p.A.	\N
000a27	Apple Computer, Inc.	\N
000a28	Motorola	\N
000a29	Pan Dacom Networking AG	\N
000a2a	QSI Systems Inc.	\N
000a2b	Etherstuff	\N
000a2c	Active Tchnology Corporation	\N
000a2d	Cabot Communications Limited	\N
000a2e	MAPLE NETWORKS CO., LTD	\N
000a2f	Artnix Inc.	\N
000a30	Johnson Controls-ASG	\N
000a31	HCV Consulting	\N
000a32	Xsido Corporation	\N
000a33	Emulex Corporation	\N
000a34	Identicard Systems Incorporated	\N
000a35	Xilinx	\N
000a36	Synelec Telecom Multimedia	\N
000a37	Procera Networks, Inc.	\N
000a38	Apani Networks	\N
000a39	LoPA Information Technology	\N
000a3a	J-THREE INTERNATIONAL Holding Co., Ltd.	\N
000a3b	GCT Semiconductor, Inc	\N
000a3c	Enerpoint Ltd.	\N
000a3d	Elo Sistemas Eletronicos S.A.	\N
000a3e	EADS Telecom	\N
000a3f	Data East Corporation	\N
000a40	Crown Audio -- Harmanm International	\N
000a41	Cisco Systems	\N
000a42	Cisco Systems	\N
000a43	Chunghwa Telecom Co., Ltd.	\N
000a44	Avery Dennison Deutschland GmbH	\N
000a45	Audio-Technica Corp.	\N
000a46	ARO WELDING TECHNOLOGIES SAS	\N
000a47	Allied Vision Technologies	\N
000a48	Albatron Technology	\N
000a49	F5 Networks, Inc.	\N
000a4a	Targa Systems Ltd.	\N
000a4b	DataPower Technology, Inc.	\N
000a4c	Molecular Devices Corporation	\N
000a4d	Noritz Corporation	\N
000a4e	UNITEK Electronics INC.	\N
000a4f	Brain Boxes Limited	\N
000a50	REMOTEK CORPORATION	\N
000a51	GyroSignal Technology Co., Ltd.	\N
000a52	AsiaRF Ltd.	\N
000a53	Intronics, Incorporated	\N
000a54	Laguna Hills, Inc.	\N
000a55	MARKEM Corporation	\N
000a56	HITACHI Maxell Ltd.	\N
000a57	Hewlett-Packard Company - Standards	\N
000a58	Ingenieur-Buero Freyer & Siegel	\N
000a59	HW server	\N
000a5a	GreenNET Technologies Co.,Ltd.	\N
000a5b	Power-One as	\N
000a5c	Carel s.p.a.	\N
000a5d	PUC Founder (MSC) Berhad	\N
000a5e	3COM Corporation	\N
000a5f	almedio inc.	\N
000a60	Autostar Technology Pte Ltd	\N
000a61	Cellinx Systems Inc.	\N
000a62	Crinis Networks, Inc.	\N
000a63	DHD GmbH	\N
000a64	Eracom Technologies	\N
000a65	GentechMedia.co.,ltd.	\N
000a66	MITSUBISHI ELECTRIC SYSTEM & SERVICE CO.,LTD.	\N
000a67	OngCorp	\N
000a68	SolarFlare Communications, Inc.	\N
000a69	SUNNY bell Technology Co., Ltd.	\N
000a6a	SVM Microwaves s.r.o.	\N
000a6b	Tadiran Telecom Business Systems LTD	\N
000a6c	Walchem Corporation	\N
000a6d	EKS Elektronikservice GmbH	\N
000a6e	Broadcast Technology Limited	\N
000a6f	ZyFLEX Technologies Inc	\N
000a70	MPLS Forum	\N
000a71	Avrio Technologies, Inc	\N
000a72	STEC, INC.	\N
000a73	Scientific Atlanta	\N
000a74	Manticom Networks Inc.	\N
000a75	Caterpillar, Inc	\N
000a76	Beida Jade Bird Huaguang Technology Co.,Ltd	\N
000a77	Bluewire Technologies LLC	\N
000a78	OLITEC	\N
000a79	Allied Telesis K.K. corega division	\N
000a7a	Kyoritsu Electric Co., Ltd.	\N
000a7b	Cornelius Consult	\N
000a7c	Tecton Ltd	\N
000a7d	Valo, Inc.	\N
000a7e	The Advantage Group	\N
000a7f	Teradon Industries, Inc	\N
000a80	Telkonet Inc.	\N
000a81	TEIMA Audiotex S.L.	\N
000a82	TATSUTA SYSTEM ELECTRONICS CO.,LTD.	\N
000a83	SALTO SYSTEMS S.L.	\N
000a84	Rainsun Enterprise Co., Ltd.	\N
000a85	PLAT'C2,Inc	\N
000a86	Lenze	\N
000a87	Integrated Micromachines Inc.	\N
000a88	InCypher S.A.	\N
000a89	Creval Systems, Inc.	\N
000a8a	Cisco Systems	\N
000a8b	Cisco Systems	\N
000a8c	Guardware Systems Ltd.	\N
000a8d	EUROTHERM LIMITED	\N
000a8e	Invacom Ltd	\N
000a8f	Aska International Inc.	\N
000a90	Bayside Interactive, Inc.	\N
000a91	HemoCue AB	\N
000a92	Presonus Corporation	\N
000a93	W2 Networks, Inc.	\N
000a94	ShangHai cellink CO., LTD	\N
000a95	Apple Computer, Inc.	\N
000a96	MEWTEL TECHNOLOGY INC.	\N
000a97	SONICblue, Inc.	\N
000a98	M+F Gwinner GmbH & Co	\N
000a99	Calamp Wireless Networks Inc	\N
000a9a	Aiptek International Inc	\N
000a9b	Towa Meccs Corporation	\N
000a9c	Server Technology, Inc.	\N
000a9d	King Young Technology Co. Ltd.	\N
000a9e	BroadWeb Corportation	\N
000a9f	Pannaway Technologies, Inc.	\N
000aa0	Cedar Point Communications	\N
000aa1	V V S Limited	\N
000aa2	SYSTEK INC.	\N
000aa3	SHIMAFUJI ELECTRIC CO.,LTD.	\N
000aa4	SHANGHAI SURVEILLANCE TECHNOLOGY CO,LTD	\N
000aa5	MAXLINK INDUSTRIES LIMITED	\N
000aa6	Hochiki Corporation	\N
000aa7	FEI Electron Optics	\N
000aa8	ePipe Pty. Ltd.	\N
000aa9	Brooks Automation GmbH	\N
000aaa	AltiGen Communications Inc.	\N
000aab	Toyota Technical Development Corporation	\N
000aac	TerraTec Electronic GmbH	\N
000aad	Stargames Corporation	\N
000aae	Rosemount Process Analytical	\N
000aaf	Pipal Systems	\N
000ab0	LOYTEC electronics GmbH	\N
000ab1	GENETEC Corporation	\N
000ab2	Fresnel Wireless Systems	\N
000ab3	Fa. GIRA	\N
000ab4	ETIC Telecommunications	\N
000ab5	Digital Electronic Network	\N
000ab6	COMPUNETIX, INC	\N
000ab7	Cisco Systems	\N
000ab8	Cisco Systems	\N
000ab9	Astera Technologies Corp.	\N
000aba	Arcon Technology Limited	\N
000abb	Taiwan Secom Co,. Ltd	\N
000abc	Seabridge Ltd.	\N
000abd	Rupprecht & Patashnick Co.	\N
000abe	OPNET Technologies CO., LTD.	\N
000abf	HIROTA SS	\N
000ac0	Fuyoh Video Industry CO., LTD.	\N
000ac1	Futuretel	\N
000ac2	FiberHome Telecommunication Technologies CO.,LTD	\N
000ac3	eM Technics Co., Ltd.	\N
000ac4	Daewoo Teletech Co., Ltd	\N
000ac5	Color Kinetics	\N
000ac6	Ceterus Networks, Inc.	\N
000ac7	Unication Group	\N
000ac8	ZPSYS CO.,LTD. (Planning&Management)	\N
000ac9	Zambeel Inc	\N
000aca	YOKOYAMA SHOKAI CO.,Ltd.	\N
000acb	XPAK MSA Group	\N
000acc	Winnow Networks, Inc.	\N
000acd	Sunrich Technology Limited	\N
000ace	RADIANTECH, INC.	\N
000acf	PROVIDEO Multimedia Co. Ltd.	\N
000ad0	Niigata Develoment Center,  F.I.T. Co., Ltd.	\N
000ad1	MWS	\N
000ad2	JEPICO Corporation	\N
000ad3	INITECH Co., Ltd	\N
000ad4	CoreBell Systems Inc.	\N
000ad5	Brainchild Electronic Co., Ltd.	\N
000ad6	BeamReach Networks	\N
000ad7	Origin ELECTRIC CO.,LTD.	\N
000ad8	IPCserv Technology Corp.	\N
000ad9	Sony Ericsson Mobile Communications AB	\N
000ada	Vindicator Technologies	\N
000adb	SkyPilot Network, Inc	\N
000adc	RuggedCom Inc.	\N
000add	Allworx Corp.	\N
000ade	Happy Communication Co., Ltd.	\N
000adf	Gennum Corporation	\N
000ae0	Fujitsu Softek	\N
000ae1	EG Technology	\N
000ae2	Binatone Electronics International, Ltd	\N
000ae3	YANG MEI TECHNOLOGY CO., LTD	\N
000ae4	Wistron Corp.	\N
000ae5	ScottCare Corporation	\N
000ae6	Elitegroup Computer System Co. (ECS)	\N
000ae7	ELIOP S.A.	\N
000ae8	Cathay Roxus Information Technology Co. LTD	\N
000ae9	AirVast Technology Inc.	\N
000aea	ADAM ELEKTRONIK LTD.STI.	\N
000aeb	Shenzhen Tp-Link Technology Co; Ltd.	\N
000aec	Koatsu Gas Kogyo Co., Ltd.	\N
000aed	HARTING Systems GmbH & Co KG	\N
000aee	GCD Hard- & Software GmbH	\N
000aef	OTRUM ASA	\N
000af0	SHIN-OH ELECTRONICS CO., LTD. R&D	\N
000af1	Clarity Design, Inc.	\N
000af2	NeoAxiom Corp.	\N
000af3	Cisco Systems	\N
000af4	Cisco Systems	\N
000af5	Airgo Networks, Inc.	\N
000af6	Emerson Climate Technologies Retail Solutions, Inc.	\N
000af7	Broadcom Corp.	\N
000af8	American Telecare Inc.	\N
000af9	HiConnect, Inc.	\N
000afa	Traverse Technologies Australia	\N
000afb	Ambri Limited	\N
000afc	Core Tec Communications, LLC	\N
000afd	Viking Electronic Services	\N
000afe	NovaPal Ltd	\N
000aff	Kilchherr Elektronik AG	\N
000b00	FUJIAN START COMPUTER EQUIPMENT CO.,LTD	\N
000b01	DAIICHI ELECTRONICS CO., LTD.	\N
000b02	Dallmeier electronic	\N
000b03	Taekwang Industrial Co., Ltd	\N
000b04	Volktek Corporation	\N
000b05	Pacific Broadband Networks	\N
000b06	Motorola BCS	\N
000b07	Voxpath Networks	\N
000b08	Pillar Data Systems	\N
000b09	Ifoundry Systems Singapore	\N
000b0a	dBm Optics	\N
000b0b	Corrent Corporation	\N
000b0c	Agile Systems Inc.	\N
000b0d	Air2U, Inc.	\N
000b0e	Trapeze Networks	\N
000b0f	Nyquist Industrial Control BV	\N
000b10	11wave Technonlogy Co.,Ltd	\N
000b11	HIMEJI ABC TRADING CO.,LTD.	\N
000b12	NURI Telecom Co., Ltd.	\N
000b13	ZETRON INC	\N
000b14	ViewSonic Corporation	\N
000b15	Platypus Technology	\N
000b16	Communication Machinery Corporation	\N
000b17	MKS Instruments	\N
000b18	PRIVATE	\N
000b19	Vernier Networks, Inc.	\N
000b1a	Industrial Defender, Inc.	\N
000b1b	Systronix, Inc.	\N
000b1c	SIBCO bv	\N
000b1d	LayerZero Power Systems, Inc.	\N
000b1e	KAPPA opto-electronics GmbH	\N
000b1f	I CON Computer Co.	\N
000b20	Hirata corporation	\N
000b21	G-Star Communications Inc.	\N
000b22	Environmental Systems and Services	\N
000b23	Siemens Subscriber Networks	\N
000b24	AirLogic	\N
000b25	Aeluros	\N
000b26	Wetek Corporation	\N
000b27	Scion Corporation	\N
000b28	Quatech Inc.	\N
000b29	LS(LG) Industrial Systems co.,Ltd	\N
000b2a	HOWTEL Co., Ltd.	\N
000b2b	HOSTNET CORPORATION	\N
000b2c	Eiki Industrial Co. Ltd.	\N
000b2d	Danfoss Inc.	\N
000b2e	Cal-Comp Electronics (Thailand) Public Company Limited Taipe	\N
000b2f	bplan GmbH	\N
000b30	Beijing Gongye Science & Technology Co.,Ltd	\N
000b31	Yantai ZhiYang Scientific and technology industry CO., LTD	\N
000b32	VORMETRIC, INC.	\N
000b33	Vivato	\N
000b34	ShangHai Broadband Technologies CO.LTD	\N
000b35	Quad Bit System co., Ltd.	\N
000b36	Productivity Systems, Inc.	\N
000b37	MANUFACTURE DES MONTRES ROLEX SA	\N
000b38	Knuerr AG	\N
000b39	Keisoku Giken Co.,Ltd.	\N
000b3a	QuStream Corporation	\N
000b3b	devolo AG	\N
000b3c	Cygnal Integrated Products, Inc.	\N
000b3d	CONTAL OK Ltd.	\N
000b3e	BittWare, Inc	\N
000b3f	Anthology Solutions Inc.	\N
000b40	OpNext Inc.	\N
000b41	Ing. Buero Dr. Beutlhauser	\N
000b42	commax Co., Ltd.	\N
000b43	Microscan Systems, Inc.	\N
000b44	Concord IDea Corp.	\N
000b45	Cisco	\N
000b46	Cisco	\N
000b47	Advanced Energy	\N
000b48	sofrel	\N
000b49	RF-Link System Inc.	\N
000b4a	Visimetrics (UK) Ltd	\N
000b4b	VISIOWAVE SA	\N
000b4c	Clarion (M) Sdn Bhd	\N
000b4d	Emuzed	\N
000b4e	VertexRSI, General Dynamics SatCOM Technologies, Inc.	\N
000b4f	Verifone, INC.	\N
000b50	Oxygnet	\N
000b51	Micetek International Inc.	\N
000b52	JOYMAX ELECTRONICS CORP.	\N
000b53	INITIUM Co., Ltd.	\N
000b54	BiTMICRO Networks, Inc.	\N
000b55	ADInstruments	\N
000b56	Cybernetics	\N
000b57	Silicon Laboratories	\N
000b58	Astronautics C.A  LTD	\N
000b59	ScriptPro, LLC	\N
000b5a	HyperEdge	\N
000b5b	Rincon Research Corporation	\N
000b5c	Newtech Co.,Ltd	\N
000b5d	FUJITSU LIMITED	\N
000b5e	Audio Engineering Society Inc.	\N
000b5f	Cisco Systems	\N
000b60	Cisco Systems	\N
000b61	Friedrich Lütze GmbH &Co.	\N
000b62	Ingenieurbuero fuer Elektronikdesign Ingo Mohnen	\N
000b63	Kaleidescape	\N
000b64	Kieback & Peter GmbH & Co KG	\N
000b65	Sy.A.C. srl	\N
000b66	Teralink Communications	\N
000b67	Topview Technology Corporation	\N
000b68	Addvalue Communications Pte Ltd	\N
000b69	Franke Finland Oy	\N
000b6a	Asiarock Incorporation	\N
000b6b	Wistron Neweb Corp.	\N
000b6c	Sychip Inc.	\N
000b6d	SOLECTRON JAPAN NAKANIIDA	\N
000b6e	Neff Instrument Corp.	\N
000b6f	Media Streaming Networks Inc	\N
000b70	Load Technology, Inc.	\N
000b71	Litchfield Communications Inc.	\N
000b72	Lawo AG	\N
000b73	Kodeos Communications	\N
000b74	Kingwave Technology Co., Ltd.	\N
000b75	Iosoft Ltd.	\N
000b76	ET&T Technology Co. Ltd.	\N
000b77	Cogent Systems, Inc.	\N
000b78	TAIFATECH INC.	\N
000b79	X-COM, Inc.	\N
000b7a	Wave Science Inc.	\N
000b7b	Test-Um Inc.	\N
000b7c	Telex Communications	\N
000b7d	SOLOMON EXTREME INTERNATIONAL LTD.	\N
000b7e	SAGINOMIYA Seisakusho Inc.	\N
000b7f	OmniWerks	\N
000b80	Lycium Networks	\N
000b81	Kaparel Corporation	\N
000b82	Grandstream Networks, Inc.	\N
000b83	DATAWATT B.V.	\N
000b84	BODET	\N
000b85	Cisco Systems	\N
000b86	Aruba Networks	\N
000b87	American Reliance Inc.	\N
000b88	Vidisco ltd.	\N
000b89	Top Global Technology, Ltd.	\N
000b8a	MITEQ Inc.	\N
000b8b	KERAJET, S.A.	\N
000b8c	Flextronics	\N
000b8d	Avvio Networks	\N
000b8e	Ascent Corporation	\N
000b8f	AKITA ELECTRONICS SYSTEMS CO.,LTD.	\N
000b90	Adva Optical Networking Inc.	\N
000b91	Aglaia Gesellschaft für Bildverarbeitung und Kommunikation m	\N
000b92	Ascom Danmark A/S	\N
000b93	Ritter Elektronik	\N
000b94	Digital Monitoring Products, Inc.	\N
000b95	eBet Gaming Systems Pty Ltd	\N
000b96	Innotrac Diagnostics Oy	\N
000b97	Matsushita Electric Industrial Co.,Ltd.	\N
000b98	NiceTechVision	\N
000b99	SensAble Technologies, Inc.	\N
000b9a	Shanghai Ulink Telecom Equipment Co. Ltd.	\N
000b9b	Sirius System Co, Ltd.	\N
000b9c	TriBeam Technologies, Inc.	\N
000b9d	TwinMOS Technologies Inc.	\N
000b9e	Yasing Technology Corp.	\N
000b9f	Neue ELSA GmbH	\N
000ba0	T&L Information Inc.	\N
000ba1	SYSCOM Ltd.	\N
000ba2	Sumitomo Electric Networks, Inc	\N
000ba3	Siemens AG, I&S	\N
000ba4	Shiron Satellite Communications Ltd. (1996)	\N
000ba5	Quasar Cipta Mandiri, PT	\N
000ba6	Miyakawa Electric Works Ltd.	\N
000ba7	Maranti Networks	\N
000ba8	HANBACK ELECTRONICS CO., LTD.	\N
000ba9	CloudShield Technologies, Inc.	\N
000baa	Aiphone co.,Ltd	\N
000bab	Advantech Technology (CHINA) Co., Ltd.	\N
000bac	3Com Ltd	\N
000bad	PC-PoS Inc.	\N
000bae	Vitals System Inc.	\N
000baf	WOOJU COMMUNICATIONS Co,.Ltd	\N
000bb0	Sysnet Telematica srl	\N
000bb1	Super Star Technology Co., Ltd.	\N
000bb2	SMALLBIG TECHNOLOGY	\N
000bb3	RiT technologies Ltd.	\N
000bb4	RDC Semiconductor Inc.,	\N
000bb5	nStor Technologies, Inc.	\N
000bb6	Mototech Inc.	\N
000bb7	Micro Systems Co.,Ltd.	\N
000bb8	Kihoku Electronic Co.	\N
000bb9	Imsys AB	\N
000bba	Harmonic Broadband Access Networks	\N
000bbb	Etin Systems Co., Ltd	\N
000bbc	En Garde Systems, Inc.	\N
000bbd	Connexionz Limited	\N
000bbe	Cisco Systems	\N
000bbf	Cisco Systems	\N
000bc0	China IWNComm Co., Ltd.	\N
000bc1	Bay Microsystems, Inc.	\N
000bc2	Corinex Communication Corp.	\N
000bc3	Multiplex, Inc.	\N
000bc4	BIOTRONIK GmbH & Co	\N
000bc5	SMC Networks, Inc.	\N
000bc6	ISAC, Inc.	\N
000bc7	ICET S.p.A.	\N
000bc8	AirFlow Networks	\N
000bc9	Electroline Equipment	\N
000bca	DATAVAN International Corporation	\N
000bcb	Fagor Automation , S. Coop	\N
000bcc	JUSAN, S.A.	\N
000bcd	Hewlett Packard	\N
000bce	Free2move AB	\N
000bcf	AGFA NDT INC.	\N
000bd0	XiMeta Technology Americas Inc.	\N
000bd1	Aeronix, Inc.	\N
000bd2	Remopro Technology Inc.	\N
000bd3	cd3o	\N
000bd4	Beijing Wise Technology & Science Development Co.Ltd	\N
000bd5	Nvergence, Inc.	\N
000bd6	Paxton Access Ltd	\N
000bd7	DORMA Time + Access GmbH	\N
000bd8	Industrial Scientific Corp.	\N
000bd9	General Hydrogen	\N
000bda	EyeCross Co.,Inc.	\N
000bdb	Dell ESG PCBA Test	\N
000bdc	AKCP	\N
000bdd	TOHOKU RICOH Co., LTD.	\N
000bde	TELDIX GmbH	\N
000bdf	Shenzhen RouterD Networks Limited	\N
000be0	SercoNet Ltd.	\N
000be1	Nokia NET Product Operations	\N
000be2	Lumenera Corporation	\N
000be3	Key Stream Co., Ltd.	\N
000be4	Hosiden Corporation	\N
000be5	HIMS Korea Co., Ltd.	\N
000be6	Datel Electronics	\N
000be7	COMFLUX TECHNOLOGY INC.	\N
000be8	AOIP	\N
000be9	Actel Corporation	\N
000bea	Zultys Technologies	\N
000beb	Systegra AG	\N
000bec	NIPPON ELECTRIC INSTRUMENT, INC.	\N
000bed	ELM Inc.	\N
000bee	inc.jet, Incorporated	\N
000bef	Code Corporation	\N
000bf0	MoTEX Products Co., Ltd.	\N
000bf1	LAP Laser Applikations	\N
000bf2	Chih-Kan Technology Co., Ltd.	\N
000bf3	BAE SYSTEMS	\N
000bf4	PRIVATE	\N
000bf5	Shanghai Sibo Telecom Technology Co.,Ltd	\N
000bf6	Nitgen Co., Ltd	\N
000bf7	NIDEK CO.,LTD	\N
000bf8	Infinera	\N
000bf9	Gemstone communications, Inc.	\N
000bfa	EXEMYS SRL	\N
000bfb	D-NET International Corporation	\N
000bfc	Cisco Systems	\N
000bfd	Cisco Systems	\N
000bfe	CASTEL Broadband Limited	\N
000bff	Berkeley Camera Engineering	\N
000c00	BEB Industrie-Elektronik AG	\N
000c01	Abatron AG	\N
000c02	ABB Oy	\N
000c03	HDMI Licensing, LLC	\N
000c04	Tecnova	\N
000c05	RPA Reserch Co., Ltd.	\N
000c06	Nixvue Systems  Pte Ltd	\N
000c07	Iftest AG	\N
000c08	HUMEX Technologies Corp.	\N
000c09	Hitachi IE Systems Co., Ltd	\N
000c0a	Guangdong Province Electronic Technology Research Institute	\N
000c0b	Broadbus Technologies	\N
000c0c	APPRO TECHNOLOGY INC.	\N
000c0d	Communications & Power Industries / Satcom Division	\N
000c0e	XtremeSpectrum, Inc.	\N
000c0f	Techno-One Co., Ltd	\N
000c10	PNI Corporation	\N
000c11	NIPPON DEMPA CO.,LTD.	\N
000c12	Micro-Optronic-Messtechnik GmbH	\N
000c13	MediaQ	\N
000c14	Diagnostic Instruments, Inc.	\N
000c15	CyberPower Systems, Inc.	\N
000c16	Concorde Microsystems Inc.	\N
000c17	AJA Video Systems Inc	\N
000c18	Zenisu Keisoku Inc.	\N
000c19	Telio Communications GmbH	\N
000c1a	Quest Technical Solutions Inc.	\N
000c1b	ORACOM Co, Ltd.	\N
000c1c	MicroWeb Co., Ltd.	\N
000c1d	Mettler & Fuchs AG	\N
000c1e	Global Cache	\N
000c1f	Glimmerglass Networks	\N
000c20	Fi WIn, Inc.	\N
000c21	Faculty of Science and Technology, Keio University	\N
000c22	Double D Electronics Ltd	\N
000c23	Beijing Lanchuan Tech. Co., Ltd.	\N
000c24	ANATOR	\N
000c25	Allied Telesyn Networks	\N
000c26	Weintek Labs. Inc.	\N
000c27	Sammy Corporation	\N
000c28	RIFATRON	\N
000c29	VMware, Inc.	\N
000c2a	OCTTEL Communication Co., Ltd.	\N
000c2b	ELIAS Technology, Inc.	\N
000c2c	Enwiser Inc.	\N
000c2d	FullWave Technology Co., Ltd.	\N
000c2e	Openet information technology(shenzhen) Co., Ltd.	\N
000c2f	SeorimTechnology Co.,Ltd.	\N
000c30	Cisco	\N
000c31	Cisco	\N
000c32	Avionic Design Development GmbH	\N
000c33	Compucase Enterprise Co. Ltd.	\N
000c34	Vixen Co., Ltd.	\N
000c35	KaVo Dental GmbH & Co. KG	\N
000c36	SHARP TAKAYA ELECTRONICS INDUSTRY CO.,LTD.	\N
000c37	Geomation, Inc.	\N
000c38	TelcoBridges Inc.	\N
000c39	Sentinel Wireless Inc.	\N
000c3a	Oxance	\N
000c3b	Orion Electric Co., Ltd.	\N
000c3c	MediaChorus, Inc.	\N
000c3d	Glsystech Co., Ltd.	\N
000c3e	Crest Audio	\N
000c3f	Cogent Defence & Security Networks,	\N
000c40	Altech Controls	\N
000c41	Cisco-Linksys	\N
000c42	Routerboard.com	\N
000c43	Ralink Technology, Corp.	\N
000c44	Automated Interfaces, Inc.	\N
000c45	Animation Technologies Inc.	\N
000c46	Allied Telesyn Inc.	\N
000c47	SK Teletech(R&D Planning Team)	\N
000c48	QoStek Corporation	\N
000c49	Dangaard Telecom RTC Division A/S	\N
000c4a	Cygnus Microsystems (P) Limited	\N
000c4b	Cheops Elektronik	\N
000c4c	Arcor AG&Co.	\N
000c4d	ACRA CONTROL	\N
000c4e	Winbest Technology CO,LT	\N
000c4f	UDTech Japan Corporation	\N
000c50	Seagate Technology	\N
000c51	Scientific Technologies Inc.	\N
000c52	Roll Systems Inc.	\N
000c53	PRIVATE	\N
000c54	Pedestal Networks, Inc	\N
000c55	Microlink Communications Inc.	\N
000c56	Megatel Computer (1986) Corp.	\N
000c57	MACKIE Engineering Services Belgium BVBA	\N
000c58	M&S Systems	\N
000c59	Indyme Electronics, Inc.	\N
000c5a	IBSmm Industrieelektronik Multimedia	\N
000c5b	HANWANG TECHNOLOGY CO.,LTD	\N
000c5c	GTN Systems B.V.	\N
000c5d	CHIC TECHNOLOGY (CHINA) CORP.	\N
000c5e	Calypso Medical	\N
000c5f	Avtec, Inc.	\N
000c60	ACM Systems	\N
000c61	AC Tech corporation DBA Advanced Digital	\N
000c62	ABB Automation Technology Products AB, Control	\N
000c63	Zenith Electronics Corporation	\N
000c64	X2 MSA Group	\N
000c65	Sunin Telecom	\N
000c66	Pronto Networks Inc	\N
000c67	OYO ELECTRIC CO.,LTD	\N
000c68	SigmaTel, Inc.	\N
000c69	National Radio Astronomy Observatory	\N
000c6a	MBARI	\N
000c6b	Kurz Industrie-Elektronik GmbH	\N
000c6c	Elgato Systems LLC	\N
000c6d	Edwards Ltd.	\N
000c6e	ASUSTEK COMPUTER INC.	\N
000c6f	Amtek system co.,LTD.	\N
000c70	ACC GmbH	\N
000c71	Wybron, Inc	\N
000c72	Tempearl Industrial Co., Ltd.	\N
000c73	TELSON ELECTRONICS CO., LTD	\N
000c74	RIVERTEC CORPORATION	\N
000c75	Oriental integrated electronics. LTD	\N
000c76	MICRO-STAR INTERNATIONAL CO., LTD.	\N
000c77	Life Racing Ltd	\N
000c78	In-Tech Electronics Limited	\N
000c79	Extel Communications P/L	\N
000c7a	DaTARIUS Technologies GmbH	\N
000c7b	ALPHA PROJECT Co.,Ltd.	\N
000c7c	Internet Information Image Inc.	\N
000c7d	TEIKOKU ELECTRIC MFG. CO., LTD	\N
000c7e	Tellium Incorporated	\N
000c7f	synertronixx GmbH	\N
000c80	Opelcomm Inc.	\N
000c81	Nulec Industries Pty Ltd	\N
000c82	NETWORK TECHNOLOGIES INC	\N
000c83	Logical Solutions	\N
000c84	Eazix, Inc.	\N
000c85	Cisco Systems	\N
000c86	Cisco Systems	\N
000c87	AMD	\N
000c88	Apache Micro Peripherals, Inc.	\N
000c89	AC Electric Vehicles, Ltd.	\N
000c8a	Bose Corporation	\N
000c8b	Connect Tech Inc	\N
000c8c	KODICOM CO.,LTD.	\N
000c8d	MATRIX VISION GmbH	\N
000c8e	Mentor Engineering Inc	\N
000c8f	Nergal s.r.l.	\N
000c90	Octasic Inc.	\N
000c91	Riverhead Networks Inc.	\N
000c92	WolfVision Gmbh	\N
000c93	Xeline Co., Ltd.	\N
000c94	United Electronic Industries, Inc. (EUI)	\N
000c95	PrimeNet	\N
000c96	OQO, Inc.	\N
000c97	NV ADB TTV Technologies SA	\N
000c98	LETEK Communications Inc.	\N
000c99	HITEL LINK Co.,Ltd	\N
000c9a	Hitech Electronics Corp.	\N
000c9b	EE Solutions, Inc	\N
000c9c	Chongho information & communications	\N
000c9d	AirWalk Communications, Inc.	\N
000c9e	MemoryLink Corp.	\N
000c9f	NKE Corporation	\N
000ca0	StorCase Technology, Inc.	\N
000ca1	SIGMACOM Co., LTD.	\N
000ca2	Scopus Network Technologies Ltd	\N
000ca3	Rancho Technology, Inc.	\N
000ca4	Prompttec Product Management GmbH	\N
000ca5	Naman NZ LTd	\N
000ca6	Mintera Corporation	\N
000ca7	Metro (Suzhou) Technologies Co., Ltd.	\N
000ca8	Garuda Networks Corporation	\N
000ca9	Ebtron Inc.	\N
000caa	Cubic Transportation Systems Inc	\N
000cab	COMMEND International	\N
000cac	Citizen Watch Co., Ltd.	\N
000cad	BTU International	\N
000cae	Ailocom Oy	\N
000caf	TRI TERM CO.,LTD.	\N
000cb0	Star Semiconductor Corporation	\N
000cb1	Salland Engineering (Europe) BV	\N
000cb2	Comstar Co., Ltd.	\N
000cb3	ROUND Co.,Ltd.	\N
000cb4	AutoCell Laboratories, Inc.	\N
000cb5	Premier Technolgies, Inc	\N
000cb6	NANJING SEU MOBILE & INTERNET TECHNOLOGY CO.,LTD	\N
000cb7	Nanjing Huazhuo Electronics Co., Ltd.	\N
000cb8	MEDION AG	\N
000cb9	LEA	\N
000cba	Jamex, Inc.	\N
000cbb	ISKRAEMECO	\N
000cbc	Iscutum	\N
000cbd	Interface Masters, Inc	\N
000cbe	Innominate Security Technologies AG	\N
000cbf	Holy Stone Ent. Co., Ltd.	\N
000cc0	Genera Oy	\N
000cc1	Cooper Industries Inc.	\N
000cc2	ControlNet (India) Private Limited	\N
000cc3	BeWAN systems	\N
000cc4	Tiptel AG	\N
000cc5	Nextlink Co., Ltd.	\N
000cc6	Ka-Ro electronics GmbH	\N
000cc7	Intelligent Computer Solutions Inc.	\N
000cc8	Xytronix Research & Design, Inc.	\N
000cc9	ILWOO DATA & TECHNOLOGY CO.,LTD	\N
000cca	Hitachi Global Storage Technologies	\N
000ccb	Design Combus Ltd	\N
000ccc	Aeroscout Ltd.	\N
000ccd	IEC - TC57	\N
000cce	Cisco Systems	\N
000ccf	Cisco Systems	\N
000cd0	Symetrix	\N
000cd1	SFOM Technology Corp.	\N
000cd2	Schaffner EMV AG	\N
000cd3	Prettl Elektronik Radeberg GmbH	\N
000cd4	Positron Public Safety Systems inc.	\N
000cd5	Passave Inc.	\N
000cd6	PARTNER TECH	\N
000cd7	Nallatech Ltd	\N
000cd8	M. K. Juchheim GmbH & Co	\N
000cd9	Itcare Co., Ltd	\N
000cda	FreeHand Systems, Inc.	\N
000cdb	Brocade Communications Systems, Inc	\N
000cdc	BECS Technology, Inc	\N
000cdd	AOS Technologies AG	\N
000cde	ABB STOTZ-KONTAKT GmbH	\N
000cdf	PULNiX America, Inc	\N
000ce0	Trek Diagnostics Inc.	\N
000ce1	The Open Group	\N
000ce2	Rolls-Royce	\N
000ce3	Option International N.V.	\N
000ce4	NeuroCom International, Inc.	\N
000ce5	Motorola BCS	\N
000ce6	Meru Networks Inc	\N
000ce7	MediaTek Inc.	\N
000ce8	GuangZhou AnJuBao Co., Ltd	\N
000ce9	BLOOMBERG L.P.	\N
000cea	aphona Kommunikationssysteme	\N
000ceb	CNMP Networks, Inc.	\N
000cec	Spectracom Corp.	\N
000ced	Real Digital Media	\N
000cee	jp-embedded	\N
000cef	Open Networks Engineering Ltd	\N
000cf0	M & N GmbH	\N
000cf1	Intel Corporation	\N
000cf2	GAMESA EÓLICA	\N
000cf3	CALL IMAGE SA	\N
000cf4	AKATSUKI ELECTRIC MFG.CO.,LTD.	\N
000cf5	InfoExpress	\N
000cf6	Sitecom Europe BV	\N
000cf7	Nortel Networks	\N
000cf8	Nortel Networks	\N
000cf9	ITT Flygt AB	\N
000cfa	Digital Systems Corp	\N
000cfb	Korea Network Systems	\N
000cfc	S2io Technologies Corp	\N
000cfd	Hyundai ImageQuest Co.,Ltd.	\N
000cfe	Grand Electronic Co., Ltd	\N
000cff	MRO-TEK LIMITED	\N
000d00	Seaway Networks Inc.	\N
000d01	P&E Microcomputer Systems, Inc.	\N
000d02	NEC AccessTechnica,Ltd	\N
000d03	Matrics, Inc.	\N
000d04	Foxboro Eckardt Development GmbH	\N
000d05	cybernet manufacturing inc.	\N
000d06	Compulogic Limited	\N
000d07	Calrec Audio Ltd	\N
000d08	AboveCable, Inc.	\N
000d09	Yuehua(Zhuhai) Electronic CO. LTD	\N
000d0a	Projectiondesign as	\N
000d0b	Buffalo Inc.	\N
000d0c	MDI Security Systems	\N
000d0d	ITSupported, LLC	\N
000d0e	Inqnet Systems, Inc.	\N
000d0f	Finlux Ltd	\N
000d10	Embedtronics Oy	\N
000d11	DENTSPLY - Gendex	\N
000d12	AXELL Corporation	\N
000d13	Wilhelm Rutenbeck GmbH&Co.	\N
000d14	Vtech Innovation LP dba Advanced American Telephones	\N
000d15	Voipac s.r.o.	\N
000d16	UHS Systems Pty Ltd	\N
000d17	Turbo Networks Co.Ltd	\N
000d18	Mega-Trend Electronics CO., LTD.	\N
000d19	ROBE Show lighting	\N
000d1a	Mustek System Inc.	\N
000d1b	Kyoto Electronics Manufacturing Co., Ltd.	\N
000d1c	Amesys Defense	\N
000d1d	HIGH-TEK HARNESS ENT. CO., LTD.	\N
000d1e	Control Techniques	\N
000d1f	AV Digital	\N
000d20	ASAHIKASEI TECHNOSYSTEM CO.,LTD.	\N
000d21	WISCORE Inc.	\N
000d22	Unitronics LTD	\N
000d23	Smart Solution, Inc	\N
000d24	SENTEC E&E CO., LTD.	\N
000d25	SANDEN CORPORATION	\N
000d26	Primagraphics Limited	\N
000d27	MICROPLEX Printware AG	\N
000d28	Cisco	\N
000d29	Cisco	\N
000d2a	Scanmatic AS	\N
000d2b	Racal Instruments	\N
000d2c	Patapsco Designs Ltd	\N
000d2d	NCT Deutschland GmbH	\N
000d2e	Matsushita Avionics Systems Corporation	\N
000d2f	AIN Comm.Tech.Co., LTD	\N
000d30	IceFyre Semiconductor	\N
000d31	Compellent Technologies, Inc.	\N
000d32	DispenseSource, Inc.	\N
000d33	Prediwave Corp.	\N
000d34	Shell International Exploration and Production, Inc.	\N
000d35	PAC International Ltd	\N
000d36	Wu Han Routon Electronic Co., Ltd	\N
000d37	WIPLUG	\N
000d38	NISSIN INC.	\N
000d39	Network Electronics	\N
000d3a	Microsoft Corp.	\N
000d3b	Microelectronics Technology Inc.	\N
000d3c	i.Tech Dynamic Ltd	\N
000d3d	Hammerhead Systems, Inc.	\N
000d3e	APLUX Communications Ltd.	\N
000d3f	VTI Instruments Corporation	\N
000d40	Verint Loronix Video Solutions	\N
000d41	Siemens AG ICM MP UC RD IT KLF1	\N
000d42	Newbest Development Limited	\N
000d43	DRS Tactical Systems Inc.	\N
000d44	Audio BU - Logitech	\N
000d45	Tottori SANYO Electric Co., Ltd.	\N
000d46	Parker SSD Drives	\N
000d47	Collex	\N
000d48	AEWIN Technologies Co., Ltd.	\N
000d49	Triton Systems of Delaware, Inc.	\N
000d4a	Steag ETA-Optik	\N
000d4b	Roku, LLC	\N
000d4c	Outline Electronics Ltd.	\N
000d4d	Ninelanes	\N
000d4e	NDR Co.,LTD.	\N
000d4f	Kenwood Corporation	\N
000d50	Galazar Networks	\N
000d51	DIVR Systems, Inc.	\N
000d52	Comart system	\N
000d53	Beijing 5w Communication Corp.	\N
000d54	3Com Ltd	\N
000d55	SANYCOM Technology Co.,Ltd	\N
000d56	Dell PCBA Test	\N
000d57	Fujitsu I-Network Systems Limited.	\N
000d58	PRIVATE	\N
000d59	Amity Systems, Inc.	\N
000d5a	Tiesse SpA	\N
000d5b	Smart Empire Investments Limited	\N
000d5c	Robert Bosch GmbH, VT-ATMO	\N
000d5d	Raritan Computer, Inc	\N
000d5e	NEC Personal Products	\N
000d5f	Minds Inc	\N
000d60	IBM Corporation	\N
000d61	Giga-Byte Technology Co., Ltd.	\N
000d62	Funkwerk Dabendorf GmbH	\N
000d63	DENT Instruments, Inc.	\N
000d64	COMAG Handels AG	\N
000d65	Cisco Systems	\N
000d66	Cisco Systems	\N
000d67	BelAir Networks Inc.	\N
000d68	Vinci Systems, Inc.	\N
000d69	TMT&D Corporation	\N
000d6a	Redwood Technologies LTD	\N
000d6b	Mita-Teknik A/S	\N
000d6c	M-Audio	\N
000d6d	K-Tech Devices Corp.	\N
000d6e	K-Patents Oy	\N
000d6f	Ember Corporation	\N
000d70	Datamax Corporation	\N
000d71	boca systems	\N
000d72	2Wire, Inc	\N
000d73	Technical Support, Inc.	\N
000d74	Sand Network Systems, Inc.	\N
000d75	Kobian Pte Ltd - Taiwan Branch	\N
000d76	Hokuto Denshi Co,. Ltd.	\N
000d77	FalconStor Software	\N
000d78	Engineering & Security	\N
000d79	Dynamic Solutions Co,.Ltd.	\N
000d7a	DiGATTO Asia Pacific Pte Ltd	\N
000d7b	Consensys Computers Inc.	\N
000d7c	Codian Ltd	\N
000d7d	Afco Systems	\N
000d7e	Axiowave Networks, Inc.	\N
000d7f	MIDAS  COMMUNICATION TECHNOLOGIES PTE LTD ( Foreign Branch)	\N
000d80	Online Development Inc	\N
000d81	Pepperl+Fuchs GmbH	\N
000d82	PHS srl	\N
000d83	Sanmina-SCI Hungary  Ltd.	\N
000d84	Makus Inc.	\N
000d85	Tapwave, Inc.	\N
000d86	Huber + Suhner AG	\N
000d87	Elitegroup Computer System Co. (ECS)	\N
000d88	D-Link Corporation	\N
000d89	Bils Technology Inc	\N
000d8a	Winners Electronics Co., Ltd.	\N
000d8b	T&D Corporation	\N
000d8c	Shanghai Wedone Digital Ltd. CO.	\N
000d8d	ProLinx Communication Gateways, Inc.	\N
000d8e	Koden Electronics Co., Ltd.	\N
000d8f	King Tsushin Kogyo Co., LTD.	\N
000d90	Factum Electronics AB	\N
000d91	Eclipse (HQ Espana) S.L.	\N
000d92	Arima Communication Corporation	\N
000d93	Apple Computer	\N
000d94	AFAR Communications,Inc	\N
000d95	Opti-cell, Inc.	\N
000d96	Vtera Technology Inc.	\N
000d97	Tropos Networks, Inc.	\N
000d98	S.W.A.C. Schmitt-Walter Automation Consult GmbH	\N
000d99	Orbital Sciences Corp.; Launch Systems Group	\N
000d9a	INFOTEC LTD	\N
000d9b	Heraeus Electro-Nite International N.V.	\N
000d9c	Elan GmbH & Co KG	\N
000d9d	Hewlett Packard	\N
000d9e	TOKUDEN OHIZUMI SEISAKUSYO Co.,Ltd.	\N
000d9f	RF Micro Devices	\N
000da0	NEDAP N.V.	\N
000da1	MIRAE ITS Co.,LTD.	\N
000da2	Infrant Technologies, Inc.	\N
000da3	Emerging Technologies Limited	\N
000da4	DOSCH & AMAND SYSTEMS AG	\N
000da5	Fabric7 Systems, Inc	\N
000da6	Universal Switching Corporation	\N
000da7	PRIVATE	\N
000da8	Teletronics Technology Corporation	\N
000da9	T.E.A.M. S.L.	\N
000daa	S.A.Tehnology co.,Ltd.	\N
000dab	Parker Hannifin GmbH Electromechanical Division Europe	\N
000dac	Japan CBM Corporation	\N
000dad	Dataprobe Inc	\N
000dae	SAMSUNG HEAVY INDUSTRIES CO., LTD.	\N
000daf	Plexus Corp (UK) Ltd	\N
000db0	Olym-tech Co.,Ltd.	\N
000db1	Japan Network Service Co., Ltd.	\N
000db2	Ammasso, Inc.	\N
000db3	SDO Communication Corperation	\N
000db4	NETASQ	\N
000db5	GLOBALSAT TECHNOLOGY CORPORATION	\N
000db6	Teknovus, Inc.	\N
000db7	SANKO ELECTRIC CO,.LTD	\N
000db8	SCHILLER AG	\N
000db9	PC Engines GmbH	\N
000dba	Océ Document Technologies GmbH	\N
000dbb	Nippon Dentsu Co.,Ltd.	\N
000dbc	Cisco Systems	\N
000dbd	Cisco Systems	\N
000dbe	Bel Fuse Europe Ltd.,UK	\N
000dbf	TekTone Sound & Signal Mfg., Inc.	\N
000dc0	Spagat AS	\N
000dc1	SafeWeb Inc	\N
000dc2	PRIVATE	\N
000dc3	First Communication, Inc.	\N
000dc4	Emcore Corporation	\N
000dc5	EchoStar International Corporation	\N
000dc6	DigiRose Technology Co., Ltd.	\N
000dc7	COSMIC ENGINEERING INC.	\N
000dc8	AirMagnet, Inc	\N
000dc9	THALES Elektronik Systeme GmbH	\N
000dca	Tait Electronics	\N
000dcb	Petcomkorea Co., Ltd.	\N
000dcc	NEOSMART Corp.	\N
000dcd	GROUPE TXCOM	\N
000dce	Dynavac Technology Pte Ltd	\N
000dcf	Cidra Corp.	\N
000dd0	TetraTec Instruments GmbH	\N
000dd1	Stryker Corporation	\N
000dd2	Simrad Optronics ASA	\N
000dd3	SAMWOO Telecommunication Co.,Ltd.	\N
000dd4	Symantec Corporation	\N
000dd5	O'RITE TECHNOLOGY CO.,LTD	\N
000dd6	ITI    LTD	\N
000dd7	Bright	\N
000dd8	BBN	\N
000dd9	Anton Paar GmbH	\N
000dda	ALLIED TELESIS K.K.	\N
000ddb	AIRWAVE TECHNOLOGIES INC.	\N
000ddc	VAC	\N
000ddd	PROFÝLO TELRA ELEKTRONÝK SANAYÝ VE TÝCARET A.Þ.	\N
000dde	Joyteck Co., Ltd.	\N
000ddf	Japan Image & Network Inc.	\N
000de0	ICPDAS Co.,LTD	\N
000de1	Control Products, Inc.	\N
000de2	CMZ Sistemi Elettronici	\N
000de3	AT Sweden AB	\N
000de4	DIGINICS, Inc.	\N
000de5	Samsung Thales	\N
000de6	YOUNGBO ENGINEERING CO.,LTD	\N
000de7	Snap-on OEM Group	\N
000de8	Nasaco Electronics Pte. Ltd	\N
000de9	Napatech Aps	\N
000dea	Kingtel Telecommunication Corp.	\N
000deb	CompXs Limited	\N
000dec	Cisco Systems	\N
000ded	Cisco Systems	\N
000dee	Andrew RF Power Amplifier Group	\N
000def	Soc. Coop. Bilanciai	\N
000df0	QCOM TECHNOLOGY INC.	\N
000df1	IONIX INC.	\N
000df2	PRIVATE	\N
000df3	Asmax Solutions	\N
000df4	Watertek Co.	\N
000df5	Teletronics International Inc.	\N
000df6	Technology Thesaurus Corp.	\N
000df7	Space Dynamics Lab	\N
000df8	ORGA Kartensysteme GmbH	\N
000df9	NDS Limited	\N
000dfa	Micro Control Systems Ltd.	\N
000dfb	Komax AG	\N
000dfc	ITFOR Inc.	\N
000dfd	Huges Hi-Tech Inc.,	\N
000dfe	Hauppauge Computer Works, Inc.	\N
000dff	CHENMING MOLD INDUSTRY CORP.	\N
000e00	Atrie	\N
000e01	ASIP Technologies Inc.	\N
000e02	Advantech AMT Inc.	\N
000e03	Emulex	\N
000e04	CMA/Microdialysis AB	\N
000e05	WIRELESS MATRIX CORP.	\N
000e06	Team Simoco Ltd	\N
000e07	Sony Ericsson Mobile Communications AB	\N
000e08	Cisco Linksys LLC	\N
000e09	Shenzhen Coship Software Co.,LTD.	\N
000e0a	SAKUMA DESIGN OFFICE	\N
000e0b	Netac Technology Co., Ltd.	\N
000e0c	Intel Corporation	\N
000e0d	HESCH Schröder GmbH	\N
000e0e	ESA elettronica S.P.A.	\N
000e0f	ERMME	\N
000e10	C-guys, Inc.	\N
000e11	BDT Büro- und Datentechnik GmbH & Co. KG	\N
000e12	Adaptive Micro Systems Inc.	\N
000e13	Accu-Sort Systems inc.	\N
000e14	Visionary Solutions, Inc.	\N
000e15	Tadlys LTD	\N
000e16	SouthWing S.L.	\N
000e17	PRIVATE	\N
000e18	MyA Technology	\N
000e19	LogicaCMG Pty Ltd	\N
000e1a	JPS Communications	\N
000e1b	IAV GmbH	\N
000e1c	Hach Company	\N
000e1d	ARION Technology Inc.	\N
000e1e	NetXen, Inc.	\N
000e1f	TCL Networks Equipment Co., Ltd.	\N
000e20	ACCESS Systems Americas, Inc.	\N
000e21	MTU Friedrichshafen GmbH	\N
000e22	PRIVATE	\N
000e23	Incipient, Inc.	\N
000e24	Huwell Technology Inc.	\N
000e25	Hannae Technology Co., Ltd	\N
000e26	Gincom Technology Corp.	\N
000e27	Crere Networks, Inc.	\N
000e28	Dynamic Ratings P/L	\N
000e29	Shester Communications Inc	\N
000e2a	PRIVATE	\N
000e2b	Safari Technologies	\N
000e2c	Netcodec co.	\N
000e2d	Hyundai Digital Technology Co.,Ltd.	\N
000e2e	Edimax Technology Co., Ltd.	\N
000e2f	Disetronic Medical Systems AG	\N
000e30	AERAS Networks, Inc.	\N
000e31	Olympus Soft Imaging Solutions GmbH	\N
000e32	Kontron Medical	\N
000e33	Shuko Electronics Co.,Ltd	\N
000e34	NexGen City, LP	\N
000e35	Intel Corp	\N
000e36	HEINESYS, Inc.	\N
000e37	Harms & Wende GmbH & Co.KG	\N
000e38	Cisco Systems	\N
000e39	Cisco Systems	\N
000e3a	Cirrus Logic	\N
000e3b	Hawking Technologies, Inc.	\N
000e3c	Transact Technologies Inc	\N
000e3d	Televic N.V.	\N
000e3e	Sun Optronics Inc	\N
000e3f	Soronti, Inc.	\N
000e40	Nortel Networks	\N
000e41	NIHON MECHATRONICS CO.,LTD.	\N
000e42	Motic Incoporation Ltd.	\N
000e43	G-Tek Electronics Sdn. Bhd.	\N
000e44	Digital 5, Inc.	\N
000e45	Beijing Newtry Electronic Technology Ltd	\N
000e46	Niigata Seimitsu Co.,Ltd.	\N
000e47	NCI System Co.,Ltd.	\N
000e48	Lipman TransAction Solutions	\N
000e49	Forsway Scandinavia AB	\N
000e4a	Changchun Huayu WEBPAD Co.,LTD	\N
000e4b	atrium c and i	\N
000e4c	Bermai Inc.	\N
000e4d	Numesa Inc.	\N
000e4e	Waveplus Technology Co., Ltd.	\N
000e4f	Trajet GmbH	\N
000e50	Thomson Telecom Belgium	\N
000e51	tecna elettronica srl	\N
000e52	Optium Corporation	\N
000e53	AV TECH CORPORATION	\N
000e54	AlphaCell Wireless Ltd.	\N
000e55	AUVITRAN	\N
000e56	4G Systems GmbH & Co. KG	\N
000e57	Iworld Networking, Inc.	\N
000e58	Sonos, Inc.	\N
000e59	SAGEM SA	\N
000e5a	TELEFIELD inc.	\N
000e5b	ParkerVision - Direct2Data	\N
000e5c	Motorola BCS	\N
000e5d	Triple Play Technologies A/S	\N
000e5e	Raisecom Technology	\N
000e5f	activ-net GmbH & Co. KG	\N
000e60	360SUN Digital Broadband Corporation	\N
000e61	MICROTROL LIMITED	\N
000e62	Nortel Networks	\N
000e63	Lemke Diagnostics GmbH	\N
000e64	Elphel, Inc	\N
000e65	TransCore	\N
000e66	Hitachi Advanced Digital, Inc.	\N
000e67	Eltis Microelectronics Ltd.	\N
000e68	E-TOP Network Technology Inc.	\N
000e69	China Electric Power Research Institute	\N
000e6a	3Com Ltd	\N
000e6b	Janitza electronics GmbH	\N
000e6c	Device Drivers Limited	\N
000e6d	Murata Manufacturing Co., Ltd.	\N
000e6e	MICRELEC  ELECTRONICS S.A	\N
000e6f	IRIS Corporation Berhad	\N
000e70	in2 Networks	\N
000e71	Gemstar Technology Development Ltd.	\N
000e72	CTS electronics	\N
000e73	Tpack A/S	\N
000e74	Solar Telecom. Tech	\N
000e75	New York Air Brake Corp.	\N
000e76	GEMSOC INNOVISION INC.	\N
000e77	Decru, Inc.	\N
000e78	Amtelco	\N
000e79	Ample Communications Inc.	\N
000e7a	GemWon Communications Co., Ltd.	\N
000e7b	Toshiba	\N
000e7c	Televes S.A.	\N
000e7d	Electronics Line 3000 Ltd.	\N
000e7e	ionSign Oy	\N
000e7f	Hewlett Packard	\N
000e80	Thomson Technology Inc	\N
000e81	Devicescape Software, Inc.	\N
000e82	Commtech Wireless	\N
000e83	Cisco Systems	\N
000e84	Cisco Systems	\N
000e85	Catalyst Enterprises, Inc.	\N
000e86	Alcatel North America	\N
000e87	adp Gauselmann GmbH	\N
000e88	VIDEOTRON CORP.	\N
000e89	CLEMATIC	\N
000e8a	Avara Technologies Pty. Ltd.	\N
000e8b	Astarte Technology Co, Ltd.	\N
000e8c	Siemens AG A&D ET	\N
000e8d	Systems in Progress Holding GmbH	\N
000e8e	SparkLAN Communications, Inc.	\N
000e8f	Sercomm Corp.	\N
000e90	PONICO CORP.	\N
000e91	Navico Auckland Ltd	\N
000e92	Millinet Co., Ltd.	\N
000e93	Milénio 3 Sistemas Electrónicos, Lda.	\N
000e94	Maas International BV	\N
000e95	Fujiya Denki Seisakusho Co.,Ltd.	\N
000e96	Cubic Defense Applications, Inc.	\N
000e97	Ultracker Technology CO., Inc	\N
000e98	Vitec CC, INC.	\N
000e99	Spectrum Digital, Inc	\N
000e9a	BOE TECHNOLOGY GROUP CO.,LTD	\N
000e9b	Ambit Microsystems Corporation	\N
000e9c	Pemstar	\N
000e9d	Tiscali UK Ltd	\N
000e9e	Topfield Co., Ltd	\N
000e9f	TEMIC SDS GmbH	\N
000ea0	NetKlass Technology Inc.	\N
000ea1	Formosa Teletek Corporation	\N
000ea2	McAfee, Inc	\N
000ea3	CNCR-IT CO.,LTD,HangZhou P.R.CHINA	\N
000ea4	Certance Inc.	\N
000ea5	BLIP Systems	\N
000ea6	ASUSTEK COMPUTER INC.	\N
000ea7	Endace Technology	\N
000ea8	United Technologists Europe Limited	\N
000ea9	Shanghai Xun Shi Communications Equipment Ltd. Co.	\N
000eaa	Scalent Systems, Inc.	\N
000eab	Cray Inc	\N
000eac	MINTRON ENTERPRISE CO., LTD.	\N
000ead	Metanoia Technologies, Inc.	\N
000eae	GAWELL TECHNOLOGIES CORP.	\N
000eaf	CASTEL	\N
000eb0	Solutions Radio BV	\N
000eb1	Newcotech,Ltd	\N
000eb2	Micro-Research Finland Oy	\N
000eb3	Hewlett-Packard	\N
000eb4	GUANGZHOU GAOKE COMMUNICATIONS TECHNOLOGY CO.LTD.	\N
000eb5	Ecastle Electronics Co., Ltd.	\N
000eb6	Riverbed Technology, Inc.	\N
000eb7	Knovative, Inc.	\N
000eb8	Iiga co.,Ltd	\N
000eb9	HASHIMOTO Electronics Industry Co.,Ltd.	\N
000eba	HANMI SEMICONDUCTOR CO., LTD.	\N
000ebb	Everbee Networks	\N
000ebc	Paragon Fidelity GmbH	\N
000ebd	Burdick, a Quinton Compny	\N
000ebe	B&B Electronics Manufacturing Co.	\N
000ebf	Remsdaq Limited	\N
000ec0	Nortel Networks	\N
000ec1	MYNAH Technologies	\N
000ec2	Lowrance Electronics, Inc.	\N
000ec3	Logic Controls, Inc.	\N
000ec4	Iskra Transmission d.d.	\N
000ec5	Digital Multitools Inc	\N
000ec6	ASIX ELECTRONICS CORP.	\N
000ec7	Motorola Korea	\N
000ec8	Zoran Corporation	\N
000ec9	YOKO Technology Corp.	\N
000eca	WTSS Inc	\N
000ecb	VineSys Technology	\N
000ecc	Tableau, LLC	\N
000ecd	SKOV A/S	\N
000ece	S.I.T.T.I. S.p.A.	\N
000ecf	PROFIBUS Nutzerorganisation e.V.	\N
000ed0	Privaris, Inc.	\N
000ed1	Osaka Micro Computer.	\N
000ed2	Filtronic plc	\N
000ed3	Epicenter, Inc.	\N
000ed4	CRESITT INDUSTRIE	\N
000ed5	COPAN Systems Inc.	\N
000ed6	Cisco Systems	\N
000ed7	Cisco Systems	\N
000ed8	Aktino, Inc.	\N
000ed9	Aksys, Ltd.	\N
000eda	C-TECH UNITED CORP.	\N
000edb	XiNCOM Corp.	\N
000edc	Tellion INC.	\N
000edd	SHURE INCORPORATED	\N
000ede	REMEC, Inc.	\N
000edf	PLX Technology	\N
000ee0	Mcharge	\N
000ee1	ExtremeSpeed Inc.	\N
000ee2	Custom Engineering S.p.A.	\N
000ee3	Chiyu Technology Co.,Ltd	\N
000ee4	BOE TECHNOLOGY GROUP CO.,LTD	\N
000ee5	bitWallet, Inc.	\N
000ee6	Adimos Systems LTD	\N
000ee7	AAC ELECTRONICS CORP.	\N
000ee8	zioncom	\N
000ee9	WayTech Development, Inc.	\N
000eea	Shadong Luneng Jicheng Electronics,Co.,Ltd	\N
000eeb	Sandmartin(zhong shan)Electronics Co.,Ltd	\N
000eec	Orban	\N
000eed	Nokia Danmark A/S	\N
000eee	Muco Industrie BV	\N
000eef	PRIVATE	\N
000ef0	Festo AG & Co. KG	\N
000ef1	EZQUEST INC.	\N
000ef2	Infinico Corporation	\N
000ef3	Smarthome	\N
000ef4	Kasda Digital Technology Co.,Ltd	\N
000ef5	iPAC Technology Co., Ltd.	\N
000ef6	E-TEN Information Systems Co., Ltd.	\N
000ef7	Vulcan Portals Inc	\N
000ef8	SBC ASI	\N
000ef9	REA Elektronik GmbH	\N
000efa	Optoway Technology Incorporation	\N
000efb	Macey Enterprises	\N
000efc	JTAG Technologies B.V.	\N
000efd	FUJINON CORPORATION	\N
000efe	EndRun Technologies LLC	\N
000eff	Megasolution,Inc.	\N
000f00	Legra Systems, Inc.	\N
000f01	DIGITALKS INC	\N
000f02	Digicube Technology Co., Ltd	\N
000f03	COM&C CO., LTD	\N
000f04	cim-usa inc	\N
000f05	3B SYSTEM INC.	\N
000f06	Nortel Networks	\N
000f07	Mangrove Systems, Inc.	\N
000f08	Indagon Oy	\N
000f09	PRIVATE	\N
000f0a	Clear Edge Networks	\N
000f0b	Kentima Technologies AB	\N
000f0c	SYNCHRONIC ENGINEERING	\N
000f0d	Hunt Electronic Co., Ltd.	\N
000f0e	WaveSplitter Technologies, Inc.	\N
000f0f	Real ID Technology Co., Ltd.	\N
000f10	RDM Corporation	\N
000f11	Prodrive B.V.	\N
000f12	Panasonic Europe Ltd.	\N
000f13	Nisca corporation	\N
000f14	Mindray Co., Ltd.	\N
000f15	Kjaerulff1 A/S	\N
000f16	JAY HOW TECHNOLOGY CO.,	\N
000f17	Insta Elektro GmbH	\N
000f18	Industrial Control Systems	\N
000f19	Boston Scientific	\N
000f1a	Gaming Support B.V.	\N
000f1b	Ego Systems Inc.	\N
000f1c	DigitAll World Co., Ltd	\N
000f1d	Cosmo Techs Co., Ltd.	\N
000f1e	Chengdu KT Electric Co.of High & New Technology	\N
000f1f	WW PCBA Test	\N
000f20	Hewlett Packard	\N
000f21	Scientific Atlanta, Inc	\N
000f22	Helius, Inc.	\N
000f23	Cisco Systems	\N
000f24	Cisco Systems	\N
000f25	AimValley B.V.	\N
000f26	WorldAccxx  LLC	\N
000f27	TEAL Electronics, Inc.	\N
000f28	Itronix Corporation	\N
000f29	Augmentix Corporation	\N
000f2a	Cableware Electronics	\N
000f2b	GREENBELL SYSTEMS	\N
000f2c	Uplogix, Inc.	\N
000f2d	CHUNG-HSIN ELECTRIC & MACHINERY MFG.CORP.	\N
000f2e	Megapower International Corp.	\N
000f2f	W-LINX TECHNOLOGY CO., LTD.	\N
000f30	Raza Microelectronics Inc	\N
000f31	Allied Vision Technologies Canada Inc	\N
000f32	LuTong Electronic Technology Co.,Ltd	\N
000f33	DUALi Inc.	\N
000f34	Cisco Systems	\N
000f35	Cisco Systems	\N
000f36	Accurate Techhnologies, Inc.	\N
000f37	Xambala Incorporated	\N
000f38	Netstar	\N
000f39	IRIS SENSORS	\N
000f3a	HISHARP	\N
000f3b	Fuji System Machines Co., Ltd.	\N
000f3c	Endeleo Limited	\N
000f3d	D-Link Corporation	\N
000f3e	CardioNet, Inc	\N
000f3f	Big Bear Networks	\N
000f40	Optical Internetworking Forum	\N
000f41	Zipher Ltd	\N
000f42	Xalyo Systems	\N
000f43	Wasabi Systems Inc.	\N
000f44	Tivella Inc.	\N
000f45	Stretch, Inc.	\N
000f46	SINAR AG	\N
000f47	ROBOX SPA	\N
000f48	Polypix Inc.	\N
000f49	Northover Solutions Limited	\N
000f4a	Kyushu-kyohan co.,ltd	\N
000f4b	Virtual Iron Software, Inc.	\N
000f4c	Elextech INC	\N
000f4d	TalkSwitch	\N
000f4e	Cellink	\N
000f4f	Cadmus Technology Ltd	\N
000f50	StreamScale Limited	\N
000f51	Azul Systems, Inc.	\N
000f52	YORK Refrigeration, Marine & Controls	\N
000f53	Solarflare Communications Inc	\N
000f54	Entrelogic Corporation	\N
000f55	Datawire Communication Networks Inc.	\N
000f56	Continuum Photonics Inc	\N
000f57	CABLELOGIC Co., Ltd.	\N
000f58	Adder Technology Limited	\N
000f59	Phonak Communications AG	\N
000f5a	Peribit Networks	\N
000f5b	Delta Information Systems, Inc.	\N
000f5c	Day One Digital Media Limited	\N
000f5d	42Networks AB	\N
000f5e	Veo	\N
000f5f	Nicety Technologies Inc. (NTS)	\N
000f60	Lifetron Co.,Ltd	\N
000f61	Hewlett Packard	\N
000f62	Alcatel Bell Space N.V.	\N
000f63	Obzerv Technologies	\N
000f64	D&R Electronica Weesp BV	\N
000f65	icube Corp.	\N
000f66	Cisco-Linksys	\N
000f67	West Instruments	\N
000f68	Vavic Network Technology, Inc.	\N
000f69	SEW Eurodrive GmbH & Co. KG	\N
000f6a	Nortel Networks	\N
000f6b	GateWare Communications GmbH	\N
000f6c	ADDI-DATA GmbH	\N
000f6d	Midas Engineering	\N
000f6e	BBox	\N
000f6f	FTA Communication Technologies	\N
000f70	Wintec Industries, inc.	\N
000f71	Sanmei Electronics Co.,Ltd	\N
000f72	Sandburst	\N
000f73	RS Automation Co., Ltd	\N
000f74	Qamcom Technology AB	\N
000f75	First Silicon Solutions	\N
000f76	Digital Keystone, Inc.	\N
000f77	DENTUM CO.,LTD	\N
000f78	Datacap Systems Inc	\N
000f79	Bluetooth Interest Group Inc.	\N
000f7a	BeiJing NuQX Technology CO.,LTD	\N
000f7b	Arce Sistemas, S.A.	\N
000f7c	ACTi Corporation	\N
000f7d	Xirrus	\N
000f7e	Ablerex Electronics Co., LTD	\N
000f7f	UBSTORAGE Co.,Ltd.	\N
000f80	Trinity Security Systems,Inc.	\N
000f81	Secure Info Imaging	\N
000f82	Mortara Instrument, Inc.	\N
000f83	Brainium Technologies Inc.	\N
000f84	Astute Networks, Inc.	\N
000f85	ADDO-Japan Corporation	\N
000f86	Research In Motion Limited	\N
000f87	Maxcess International	\N
000f88	AMETEK, Inc.	\N
000f89	Winnertec System Co., Ltd.	\N
000f8a	WideView	\N
000f8b	Orion MultiSystems Inc	\N
000f8c	Gigawavetech Pte Ltd	\N
000f8d	FAST TV-Server AG	\N
000f8e	DONGYANG TELECOM CO.,LTD.	\N
000f8f	Cisco Systems	\N
000f90	Cisco Systems	\N
000f91	Aerotelecom Co.,Ltd.	\N
000f92	Microhard Systems Inc.	\N
000f93	Landis+Gyr Ltd.	\N
000f94	Genexis	\N
000f95	ELECOM Co.,LTD Laneed Division	\N
000f96	Critical Telecom Corp.	\N
000f97	Avanex Corporation	\N
000f98	Avamax Co. Ltd.	\N
000f99	APAC opto Electronics Inc.	\N
000f9a	Synchrony, Inc.	\N
000f9b	Ross Video Limited	\N
000f9c	Panduit Corp	\N
000f9d	DisplayLink (UK) Ltd	\N
000f9e	Murrelektronik GmbH	\N
000f9f	Motorola BCS	\N
000fa0	CANON KOREA BUSINESS SOLUTIONS INC.	\N
000fa1	Gigabit Systems Inc.	\N
000fa2	Digital Path Networks	\N
000fa3	Alpha Networks Inc.	\N
000fa4	Sprecher Automation GmbH	\N
000fa5	SMP / BWA Technology GmbH	\N
000fa6	S2 Security Corporation	\N
000fa7	Raptor Networks Technology	\N
000fa8	Photometrics, Inc.	\N
000fa9	PC Fabrik	\N
000faa	Nexus Technologies	\N
000fab	Kyushu Electronics Systems Inc.	\N
000fac	IEEE 802.11	\N
000fad	FMN communications GmbH	\N
000fae	E2O Communications	\N
000faf	Dialog Inc.	\N
000fb0	Compal Electronics,INC.	\N
000fb1	Cognio Inc.	\N
000fb2	Broadband Pacenet (India) Pvt. Ltd.	\N
000fb3	Actiontec Electronics, Inc	\N
000fb4	Timespace Technology	\N
000fb5	NETGEAR Inc	\N
000fb6	Europlex Technologies	\N
000fb7	Cavium Networks	\N
000fb8	CallURL Inc.	\N
000fb9	Adaptive Instruments	\N
000fba	Tevebox AB	\N
000fbb	Nokia Siemens Networks GmbH & Co. KG	\N
000fbc	Onkey Technologies, Inc.	\N
000fbd	MRV Communications (Networks) LTD	\N
000fbe	e-w/you Inc.	\N
000fbf	DGT Sp. z o.o.	\N
000fc0	DELCOMp	\N
000fc1	WAVE Corporation	\N
000fc2	Uniwell Corporation	\N
000fc3	PalmPalm Technology, Inc.	\N
000fc4	NST co.,LTD.	\N
000fc5	KeyMed Ltd	\N
000fc6	Eurocom Industries A/S	\N
000fc7	Dionica R&D Ltd.	\N
000fc8	Chantry Networks	\N
000fc9	Allnet GmbH	\N
000fca	A-JIN TECHLINE CO, LTD	\N
000fcb	3Com Ltd	\N
000fcc	Netopia, Inc.	\N
000fcd	Nortel Networks	\N
000fce	Kikusui Electronics Corp.	\N
000fcf	Datawind Research	\N
000fd0	ASTRI	\N
000fd1	Applied Wireless Identifications Group, Inc.	\N
000fd2	EWA Technologies, Inc.	\N
000fd3	Digium	\N
000fd4	Soundcraft	\N
000fd5	Schwechat - RISE	\N
000fd6	Sarotech Co., Ltd	\N
000fd7	Harman Music Group	\N
000fd8	Force, Inc.	\N
000fd9	FlexDSL Telecommunications AG	\N
000fda	YAZAKI CORPORATION	\N
000fdb	Westell Technologies	\N
000fdc	Ueda Japan  Radio Co., Ltd.	\N
000fdd	SORDIN AB	\N
000fde	Sony Ericsson Mobile Communications AB	\N
000fdf	SOLOMON Technology Corp.	\N
000fe0	NComputing Co.,Ltd.	\N
000fe1	ID DIGITAL CORPORATION	\N
000fe2	Hangzhou H3C Technologies Co., Ltd.	\N
000fe3	Damm Cellular Systems A/S	\N
000fe4	Pantech Co.,Ltd	\N
000fe5	MERCURY SECURITY CORPORATION	\N
000fe6	MBTech Systems, Inc.	\N
000fe7	Lutron Electronics Co., Inc.	\N
000fe8	Lobos, Inc.	\N
000fe9	GW TECHNOLOGIES CO.,LTD.	\N
000fea	Giga-Byte Technology Co.,LTD.	\N
000feb	Cylon Controls	\N
000fec	Arkus Inc.	\N
000fed	Anam Electronics Co., Ltd	\N
000fee	XTec, Incorporated	\N
000fef	Thales e-Transactions GmbH	\N
000ff0	Sunray Co. Ltd.	\N
000ff1	nex-G Systems Pte.Ltd	\N
000ff2	Loud Technologies Inc.	\N
000ff3	Jung Myoung Communications&Technology	\N
000ff4	Guntermann & Drunck GmbH	\N
000ff5	GN&S company	\N
000ff6	Darfon Electronics Corp.	\N
000ff7	Cisco Systems	\N
000ff8	Cisco  Systems	\N
000ff9	Valcretec, Inc.	\N
000ffa	Optinel Systems, Inc.	\N
000ffb	Nippon Denso Industry Co., Ltd.	\N
000ffc	Merit Li-Lin Ent.	\N
000ffd	Glorytek Network Inc.	\N
000ffe	G-PRO COMPUTER	\N
000fff	Control4	\N
001000	CABLE TELEVISION LABORATORIES, INC.	\N
001001	Citel	\N
001002	ACTIA	\N
001003	IMATRON, INC.	\N
001004	THE BRANTLEY COILE COMPANY,INC	\N
001005	UEC COMMERCIAL	\N
001006	Thales Contact Solutions Ltd.	\N
001007	CISCO SYSTEMS, INC.	\N
001008	VIENNA SYSTEMS CORPORATION	\N
001009	HORO QUARTZ	\N
00100a	WILLIAMS COMMUNICATIONS GROUP	\N
00100b	CISCO SYSTEMS, INC.	\N
00100c	ITO CO., LTD.	\N
00100d	CISCO SYSTEMS, INC.	\N
00100e	MICRO LINEAR COPORATION	\N
00100f	INDUSTRIAL CPU SYSTEMS	\N
001010	INITIO CORPORATION	\N
001011	CISCO SYSTEMS, INC.	\N
001012	PROCESSOR SYSTEMS (I) PVT LTD	\N
001013	Kontron America, Inc.	\N
001014	CISCO SYSTEMS, INC.	\N
001015	OOmon Inc.	\N
001016	T.SQWARE	\N
001017	Bosch Access Systems GmbH	\N
001018	BROADCOM CORPORATION	\N
001019	SIRONA DENTAL SYSTEMS GmbH & Co. KG	\N
00101a	PictureTel Corp.	\N
00101b	CORNET TECHNOLOGY, INC.	\N
00101c	OHM TECHNOLOGIES INTL, LLC	\N
00101d	WINBOND ELECTRONICS CORP.	\N
00101e	MATSUSHITA ELECTRONIC INSTRUMENTS CORP.	\N
00101f	CISCO SYSTEMS, INC.	\N
001020	Hand Held Products Inc	\N
001021	ENCANTO NETWORKS, INC.	\N
001022	SatCom Media Corporation	\N
001023	Network Equipment Technologies	\N
001024	NAGOYA ELECTRIC WORKS CO., LTD	\N
001025	Grayhill, Inc	\N
001026	ACCELERATED NETWORKS, INC.	\N
001027	L-3 COMMUNICATIONS EAST	\N
001028	COMPUTER TECHNICA, INC.	\N
001029	CISCO SYSTEMS, INC.	\N
00102a	ZF MICROSYSTEMS, INC.	\N
00102b	UMAX DATA SYSTEMS, INC.	\N
00102c	Lasat Networks A/S	\N
00102d	HITACHI SOFTWARE ENGINEERING	\N
00102e	NETWORK SYSTEMS & TECHNOLOGIES PVT. LTD.	\N
00102f	CISCO SYSTEMS, INC.	\N
001030	EION Inc.	\N
001031	OBJECTIVE COMMUNICATIONS, INC.	\N
001032	ALTA TECHNOLOGY	\N
001033	ACCESSLAN COMMUNICATIONS, INC.	\N
001034	GNP Computers	\N
001035	ELITEGROUP COMPUTER SYSTEMS CO., LTD	\N
001036	INTER-TEL INTEGRATED SYSTEMS	\N
001037	CYQ've Technology Co., Ltd.	\N
001038	MICRO RESEARCH INSTITUTE, INC.	\N
001039	Vectron Systems AG	\N
00103a	DIAMOND NETWORK TECH	\N
00103b	HIPPI NETWORKING FORUM	\N
00103c	IC ENSEMBLE, INC.	\N
00103d	PHASECOM, LTD.	\N
00103e	NETSCHOOLS CORPORATION	\N
00103f	TOLLGRADE COMMUNICATIONS, INC.	\N
001040	INTERMEC CORPORATION	\N
001041	BRISTOL BABCOCK, INC.	\N
001042	Alacritech, Inc.	\N
001043	A2 CORPORATION	\N
001044	InnoLabs Corporation	\N
001045	Nortel Networks	\N
001046	ALCORN MCBRIDE INC.	\N
001047	ECHO ELETRIC CO. LTD.	\N
001048	HTRC AUTOMATION, INC.	\N
001049	ShoreTel, Inc	\N
00104a	The Parvus Corporation	\N
00104b	3COM CORPORATION	\N
00104c	LeCroy Corporation	\N
00104d	SURTEC INDUSTRIES, INC.	\N
00104e	CEOLOGIC	\N
00104f	Sun Microsystems, Inc.	\N
001050	RION CO., LTD.	\N
001051	CMICRO CORPORATION	\N
001052	METTLER-TOLEDO (ALBSTADT) GMBH	\N
001053	COMPUTER TECHNOLOGY CORP.	\N
001054	CISCO SYSTEMS, INC.	\N
001055	FUJITSU MICROELECTRONICS, INC.	\N
001056	SODICK CO., LTD.	\N
001057	Rebel.com, Inc.	\N
001058	ArrowPoint Communications	\N
001059	DIABLO RESEARCH CO. LLC	\N
00105a	3COM CORPORATION	\N
00105b	NET INSIGHT AB	\N
00105c	QUANTUM DESIGNS (H.K.) LTD.	\N
00105d	Draeger Medical	\N
00105e	HEKIMIAN LABORATORIES, INC.	\N
00105f	IN-SNEC	\N
001060	BILLIONTON SYSTEMS, INC.	\N
001061	HOSTLINK CORP.	\N
001062	NX SERVER, ILNC.	\N
001063	STARGUIDE DIGITAL NETWORKS	\N
001064	DNPG, LLC	\N
001065	RADYNE CORPORATION	\N
001066	ADVANCED CONTROL SYSTEMS, INC.	\N
001067	REDBACK NETWORKS, INC.	\N
001068	COMOS TELECOM	\N
001069	HELIOSS COMMUNICATIONS, INC.	\N
00106a	DIGITAL MICROWAVE CORPORATION	\N
00106b	SONUS NETWORKS, INC.	\N
00106c	Infratec AG	\N
00106d	Axxcelera Broadband Wireless	\N
00106e	TADIRAN COM. LTD.	\N
00106f	TRENTON TECHNOLOGY INC.	\N
001070	CARADON TREND LTD.	\N
001071	ADVANET INC.	\N
001072	GVN TECHNOLOGIES, INC.	\N
001073	Technobox, Inc.	\N
001074	ATEN INTERNATIONAL CO., LTD.	\N
001075	Maxtor Corporation	\N
001076	EUREM GmbH	\N
001077	SAF DRIVE SYSTEMS, LTD.	\N
001078	NUERA COMMUNICATIONS, INC.	\N
001079	CISCO SYSTEMS, INC.	\N
00107a	AmbiCom, Inc.	\N
00107b	CISCO SYSTEMS, INC.	\N
00107c	P-COM, INC.	\N
00107d	AURORA COMMUNICATIONS, LTD.	\N
00107e	BACHMANN ELECTRONIC GmbH	\N
00107f	CRESTRON ELECTRONICS, INC.	\N
001080	METAWAVE COMMUNICATIONS	\N
001081	DPS, INC.	\N
001082	JNA TELECOMMUNICATIONS LIMITED	\N
001083	HEWLETT-PACKARD COMPANY	\N
001084	K-BOT COMMUNICATIONS	\N
001085	POLARIS COMMUNICATIONS, INC.	\N
001086	ATTO Technology, Inc.	\N
001087	Xstreamis PLC	\N
001088	AMERICAN NETWORKS INC.	\N
001089	WebSonic	\N
00108a	TeraLogic, Inc.	\N
00108b	LASERANIMATION SOLLINGER GmbH	\N
00108c	FUJITSU TELECOMMUNICATIONS EUROPE, LTD.	\N
00108d	Johnson Controls, Inc.	\N
00108e	HUGH SYMONS CONCEPT Technologies Ltd.	\N
00108f	RAPTOR SYSTEMS	\N
001090	CIMETRICS, INC.	\N
001091	NO WIRES NEEDED BV	\N
001092	NETCORE INC.	\N
001093	CMS COMPUTERS, LTD.	\N
001094	Performance Analysis Broadband, Spirent plc	\N
001095	Thomson Inc.	\N
001096	TRACEWELL SYSTEMS, INC.	\N
001097	WinNet Metropolitan Communications Systems, Inc.	\N
001098	STARNET TECHNOLOGIES, INC.	\N
001099	InnoMedia, Inc.	\N
00109a	NETLINE	\N
00109b	Emulex Corporation	\N
00109c	M-SYSTEM CO., LTD.	\N
00109d	CLARINET SYSTEMS, INC.	\N
00109e	AWARE, INC.	\N
00109f	PAVO, INC.	\N
0010a0	INNOVEX TECHNOLOGIES, INC.	\N
0010a1	KENDIN SEMICONDUCTOR, INC.	\N
0010a2	TNS	\N
0010a3	OMNITRONIX, INC.	\N
0010a4	XIRCOM	\N
0010a5	OXFORD INSTRUMENTS	\N
0010a6	CISCO SYSTEMS, INC.	\N
0010a7	UNEX TECHNOLOGY CORPORATION	\N
0010a8	RELIANCE COMPUTER CORP.	\N
0010a9	ADHOC TECHNOLOGIES	\N
0010aa	MEDIA4, INC.	\N
0010ab	KOITO INDUSTRIES, LTD.	\N
0010ac	IMCI TECHNOLOGIES	\N
0010ad	SOFTRONICS USB, INC.	\N
0010ae	SHINKO ELECTRIC INDUSTRIES CO.	\N
0010af	TAC SYSTEMS, INC.	\N
0010b0	MERIDIAN TECHNOLOGY CORP.	\N
0010b1	FOR-A CO., LTD.	\N
0010b2	COACTIVE AESTHETICS	\N
0010b3	NOKIA MULTIMEDIA TERMINALS	\N
0010b4	ATMOSPHERE NETWORKS	\N
0010b5	ACCTON TECHNOLOGY CORPORATION	\N
0010b6	ENTRATA COMMUNICATIONS CORP.	\N
0010b7	COYOTE TECHNOLOGIES, LLC	\N
0010b8	ISHIGAKI COMPUTER SYSTEM CO.	\N
0010b9	MAXTOR CORP.	\N
0010ba	MARTINHO-DAVIS SYSTEMS, INC.	\N
0010bb	DATA & INFORMATION TECHNOLOGY	\N
0010bc	Aastra Telecom	\N
0010bd	THE TELECOMMUNICATION TECHNOLOGY COMMITTEE (TTC)	\N
0010be	TELEXIS CORP.	\N
0010bf	InterAir Wireless	\N
0010c0	ARMA, Inc.	\N
0010c1	OI ELECTRIC CO., LTD.	\N
0010c2	WILLNET, INC.	\N
0010c3	CSI-CONTROL SYSTEMS	\N
0010c4	MEDIA LINKS CO., LTD.	\N
0010c5	PROTOCOL TECHNOLOGIES, INC.	\N
0010c6	USI	\N
0010c7	DATA TRANSMISSION NETWORK	\N
0010c8	COMMUNICATIONS ELECTRONICS SECURITY GROUP	\N
0010c9	MITSUBISHI ELECTRONICS LOGISTIC SUPPORT CO.	\N
0010ca	INTEGRAL ACCESS	\N
0010cb	FACIT K.K.	\N
0010cc	CLP COMPUTER LOGISTIK PLANUNG GmbH	\N
0010cd	INTERFACE CONCEPT	\N
0010ce	VOLAMP, LTD.	\N
0010cf	FIBERLANE COMMUNICATIONS	\N
0010d0	WITCOM, LTD.	\N
0010d1	Top Layer Networks, Inc.	\N
0010d2	NITTO TSUSHINKI CO., LTD	\N
0010d3	GRIPS ELECTRONIC GMBH	\N
0010d4	STORAGE COMPUTER CORPORATION	\N
0010d5	IMASDE CANARIAS, S.A.	\N
0010d6	ITT - A/CD	\N
0010d7	ARGOSY RESEARCH INC.	\N
0010d8	CALISTA	\N
0010d9	IBM JAPAN, FUJISAWA MT+D	\N
0010da	MOTION ENGINEERING, INC.	\N
0010db	Juniper Networks, Inc.	\N
0010dc	MICRO-STAR INTERNATIONAL CO., LTD.	\N
0010dd	ENABLE SEMICONDUCTOR, INC.	\N
0010de	INTERNATIONAL DATACASTING CORPORATION	\N
0010df	RISE COMPUTER INC.	\N
0010e0	Sun Microsystems, Inc.	\N
0010e1	S.I. TECH, INC.	\N
0010e2	ArrayComm, Inc.	\N
0010e3	Hewlett Packard	\N
0010e4	NSI CORPORATION	\N
0010e5	SOLECTRON TEXAS	\N
0010e6	APPLIED INTELLIGENT SYSTEMS, INC.	\N
0010e7	BreezeCom	\N
0010e8	TELOCITY, INCORPORATED	\N
0010e9	RAIDTEC LTD.	\N
0010ea	ADEPT TECHNOLOGY	\N
0010eb	SELSIUS SYSTEMS, INC.	\N
0010ec	RPCG, LLC	\N
0010ed	SUNDANCE TECHNOLOGY, INC.	\N
0010ee	CTI PRODUCTS, INC.	\N
0010ef	DBTEL INCORPORATED	\N
0010f1	I-O CORPORATION	\N
0010f2	ANTEC	\N
0010f3	Nexcom International Co., Ltd.	\N
0010f4	Vertical Communications	\N
0010f5	AMHERST SYSTEMS, INC.	\N
0010f6	CISCO SYSTEMS, INC.	\N
0010f7	IRIICHI TECHNOLOGIES Inc.	\N
0010f8	Niikke Techno System Co. Ltd	\N
0010f9	UNIQUE SYSTEMS, INC.	\N
0010fa	Apple Inc	\N
0010fb	ZIDA TECHNOLOGIES LIMITED	\N
0010fc	BROADBAND NETWORKS, INC.	\N
0010fd	COCOM A/S	\N
0010fe	DIGITAL EQUIPMENT CORPORATION	\N
0010ff	CISCO SYSTEMS, INC.	\N
001100	Schneider Electric	\N
001101	CET Technologies Pte Ltd	\N
001102	Aurora Multimedia Corp.	\N
001103	kawamura electric inc.	\N
001104	TELEXY	\N
001105	Sunplus Technology Co., Ltd.	\N
001106	Siemens NV (Belgium)	\N
001107	RGB Networks Inc.	\N
001108	Orbital Data Corporation	\N
001109	Micro-Star International	\N
00110a	Hewlett Packard	\N
00110b	Franklin Technology Systems	\N
00110c	Atmark Techno, Inc.	\N
00110d	SANBlaze Technology, Inc.	\N
00110e	Tsurusaki Sealand Transportation Co. Ltd.	\N
00110f	netplat,Inc.	\N
001110	Maxanna Technology Co., Ltd.	\N
001111	Intel Corporation	\N
001112	Honeywell CMSS	\N
001113	Fraunhofer FOKUS	\N
001114	EverFocus Electronics Corp.	\N
001115	EPIN Technologies, Inc.	\N
001116	COTEAU VERT CO., LTD.	\N
001117	CESNET	\N
001118	BLX IC Design Corp., Ltd.	\N
001119	Solteras, Inc.	\N
00111a	Motorola BCS	\N
00111b	Targa Systems Div L-3 Communications Canada	\N
00111c	Pleora Technologies Inc.	\N
00111d	Hectrix Limited	\N
00111e	EPSG (Ethernet Powerlink Standardization Group)	\N
00111f	Doremi Labs, Inc.	\N
001120	Cisco Systems	\N
001121	Cisco Systems	\N
001122	CIMSYS Inc	\N
001123	Appointech, Inc.	\N
001124	Apple Computer	\N
001125	IBM Corporation	\N
001126	Venstar Inc.	\N
001127	TASI, Inc	\N
001128	Streamit	\N
001129	Paradise Datacom Ltd.	\N
00112a	Niko NV	\N
00112b	NetModule	\N
00112c	IZT GmbH	\N
00112d	iPulse Systems	\N
00112e	CEICOM	\N
00112f	ASUSTek Computer Inc.	\N
001130	Allied Telesis (Hong Kong) Ltd.	\N
001131	UNATECH. CO.,LTD	\N
001132	Synology Incorporated	\N
001133	Siemens Austria SIMEA	\N
001134	MediaCell, Inc.	\N
001135	Grandeye Ltd	\N
001136	Goodrich Sensor Systems	\N
001137	AICHI ELECTRIC CO., LTD.	\N
001138	TAISHIN CO., LTD.	\N
001139	STOEBER ANTRIEBSTECHNIK GmbH + Co. KG.	\N
00113a	SHINBORAM	\N
00113b	Micronet Communications Inc.	\N
00113c	Micronas GmbH	\N
00113d	KN SOLTEC CO.,LTD.	\N
00113e	JL Corporation	\N
00113f	Alcatel DI	\N
001140	Nanometrics Inc.	\N
001141	GoodMan Corporation	\N
001142	e-SMARTCOM  INC.	\N
001143	DELL INC.	\N
001144	Assurance Technology Corp	\N
001145	ValuePoint Networks	\N
001146	Telecard-Pribor Ltd	\N
001147	Secom-Industry co.LTD.	\N
001148	Prolon Control Systems	\N
001149	Proliphix Inc.	\N
00114a	KAYABA INDUSTRY Co,.Ltd.	\N
00114b	Francotyp-Postalia GmbH	\N
00114c	caffeina applied research ltd.	\N
00114d	Atsumi Electric Co.,LTD.	\N
00114e	690885 Ontario Inc.	\N
00114f	US Digital Television, Inc	\N
001150	Belkin Corporation	\N
001151	Mykotronx	\N
001152	Eidsvoll Electronics AS	\N
001153	Trident Tek, Inc.	\N
001154	Webpro Technologies Inc.	\N
001155	Sevis Systems	\N
001156	Pharos Systems NZ	\N
001157	OF Networks Co., Ltd.	\N
001158	Nortel Networks	\N
001159	MATISSE NETWORKS INC	\N
00115a	Ivoclar Vivadent AG	\N
00115b	Elitegroup Computer System Co. (ECS)	\N
00115c	Cisco	\N
00115d	Cisco	\N
00115e	ProMinent Dosiertechnik GmbH	\N
00115f	ITX Security Co., Ltd.	\N
001160	ARTDIO Company Co., LTD	\N
001161	NetStreams, LLC	\N
001162	STAR MICRONICS CO.,LTD.	\N
001163	SYSTEM SPA DEPT. ELECTRONICS	\N
001164	ACARD Technology Corp.	\N
001165	Znyx Networks	\N
001166	Taelim Electronics Co., Ltd.	\N
001167	Integrated System Solution Corp.	\N
001168	HomeLogic LLC	\N
001169	EMS Satcom	\N
00116a	Domo Ltd	\N
00116b	Digital Data Communications Asia Co.,Ltd	\N
00116c	Nanwang Multimedia Inc.,Ltd	\N
00116d	American Time and Signal	\N
00116e	PePLink Ltd.	\N
00116f	Netforyou Co., LTD.	\N
001170	GSC SRL	\N
001171	DEXTER Communications, Inc.	\N
001172	COTRON CORPORATION	\N
001173	SMART Modular Technologies	\N
001174	Wibhu Technologies, Inc.	\N
001175	PathScale, Inc.	\N
001176	Intellambda Systems, Inc.	\N
001177	Coaxial Networks, Inc.	\N
001178	Chiron Technology Ltd	\N
001179	Singular Technology Co. Ltd.	\N
00117a	Singim International Corp.	\N
00117b	Büchi Labortechnik AG	\N
00117c	e-zy.net	\N
00117d	ZMD America, Inc.	\N
00117e	Progeny Inc.	\N
00117f	Neotune Information Technology Corporation,.LTD	\N
001180	Motorola BCS	\N
001181	InterEnergy Co.Ltd,	\N
001182	IMI Norgren Ltd	\N
001183	Datalogic Mobile, Inc.	\N
001184	Humo Laboratory,Ltd.	\N
001185	Hewlett Packard	\N
001186	Prime Systems, Inc.	\N
001187	Category Solutions, Inc	\N
001188	Enterasys	\N
001189	Aerotech Inc	\N
00118a	Viewtran Technology Limited	\N
00118b	Alcatel-Lucent, Enterprise Business Group	\N
00118c	Missouri Department of Transportation	\N
00118d	Hanchang System Corp.	\N
00118e	Halytech Mace	\N
00118f	EUTECH INSTRUMENTS PTE. LTD.	\N
001190	Digital Design Corporation	\N
001191	CTS-Clima Temperatur Systeme GmbH	\N
001192	Cisco Systems	\N
001193	Cisco Systems	\N
001194	Chi Mei Communication Systems, Inc.	\N
001195	D-Link Corporation	\N
001196	Actuality Systems, Inc.	\N
001197	Monitoring Technologies Limited	\N
001198	Prism Media Products Limited	\N
001199	2wcom GmbH	\N
00119a	Alkeria srl	\N
00119b	Telesynergy Research Inc.	\N
00119c	EP&T Energy	\N
00119d	Diginfo Technology Corporation	\N
00119e	Solectron Brazil	\N
00119f	Nokia Danmark A/S	\N
0011a0	Vtech Engineering Canada Ltd	\N
0011a1	VISION NETWARE CO.,LTD	\N
0011a2	Manufacturing Technology Inc	\N
0011a3	LanReady Technologies Inc.	\N
0011a4	JStream Technologies Inc.	\N
0011a5	Fortuna Electronic Corp.	\N
0011a6	Sypixx Networks	\N
0011a7	Infilco Degremont Inc.	\N
0011a8	Quest Technologies	\N
0011a9	MOIMSTONE Co., LTD	\N
0011aa	Uniclass Technology, Co., LTD	\N
0011ab	TRUSTABLE TECHNOLOGY CO.,LTD.	\N
0011ac	Simtec Electronics	\N
0011ad	Shanghai Ruijie Technology	\N
0011ae	Motorola BCS	\N
0011af	Medialink-i,Inc	\N
0011b0	Fortelink Inc.	\N
0011b1	BlueExpert Technology Corp.	\N
0011b2	2001 Technology Inc.	\N
0011b3	YOSHIMIYA CO.,LTD.	\N
0011b4	Westermo Teleindustri AB	\N
0011b5	Shenzhen Powercom Co.,Ltd	\N
0011b6	Open Systems International	\N
0011b7	Octalix B.V.	\N
0011b8	Liebherr - Elektronik GmbH	\N
0011b9	Inner Range Pty. Ltd.	\N
0011ba	Elexol Pty Ltd	\N
0011bb	Cisco Systems	\N
0011bc	Cisco Systems	\N
0011bd	Bombardier Transportation	\N
0011be	AGP Telecom Co. Ltd	\N
0011bf	AESYS S.p.A.	\N
0011c0	Aday Technology Inc	\N
0011c1	4P MOBILE DATA PROCESSING	\N
0011c2	United Fiber Optic Communication	\N
0011c3	Transceiving System Technology Corporation	\N
0011c4	Terminales de Telecomunicacion Terrestre, S.L.	\N
0011c5	TEN Technology	\N
0011c6	Seagate Technology LLC	\N
0011c7	Raymarine UK Ltd	\N
0011c8	Powercom Co., Ltd.	\N
0011c9	MTT Corporation	\N
0011ca	Long Range Systems, Inc.	\N
0011cb	Jacobsons AB	\N
0011cc	Guangzhou Jinpeng Group Co.,Ltd.	\N
0011cd	Axsun Technologies	\N
0011ce	Ubisense Limited	\N
0011cf	Thrane & Thrane A/S	\N
0011d0	Tandberg Data ASA	\N
0011d1	Soft Imaging System GmbH	\N
0011d2	Perception Digital Ltd	\N
0011d3	NextGenTel Holding ASA	\N
0011d4	NetEnrich, Inc	\N
0011d5	Hangzhou Sunyard System Engineering Co.,Ltd.	\N
0011d6	HandEra, Inc.	\N
0011d7	eWerks Inc	\N
0011d8	ASUSTek Computer Inc.	\N
0011d9	TiVo	\N
0011da	Vivaas Technology Inc.	\N
0011db	Land-Cellular Corporation	\N
0011dc	Glunz & Jensen	\N
0011dd	FROMUS TEC. Co., Ltd.	\N
0011de	EURILOGIC	\N
0011df	Current Energy	\N
0011e0	U-MEDIA Communications, Inc.	\N
0011e1	BEKO Electronics Co.	\N
0011e2	Hua Jung Components Co., Ltd.	\N
0011e3	Thomson, Inc.	\N
0011e4	Danelec Electronics A/S	\N
0011e5	KCodes Corporation	\N
0011e6	Scientific Atlanta	\N
0011e7	WORLDSAT - Texas de France	\N
0011e8	Tixi.Com	\N
0011e9	STARNEX CO., LTD.	\N
0011ea	IWICS Inc.	\N
0011eb	Innovative Integration	\N
0011ec	AVIX INC.	\N
0011ed	802 Global	\N
0011ee	Estari, Inc.	\N
0011ef	Conitec Datensysteme GmbH	\N
0011f0	Wideful Limited	\N
0011f1	QinetiQ Ltd	\N
0011f2	Institute of Network Technologies	\N
0011f3	NeoMedia Europe AG	\N
0011f4	woori-net	\N
0011f5	ASKEY COMPUTER CORP.	\N
0011f6	Asia Pacific Microsystems , Inc.	\N
0011f7	Shenzhen Forward Industry Co., Ltd	\N
0011f8	AIRAYA Corp	\N
0011f9	Nortel Networks	\N
0011fa	Rane Corporation	\N
0011fb	Heidelberg Engineering GmbH	\N
0011fc	HARTING Electric Gmbh & Co.KG	\N
0011fd	KORG INC.	\N
0011fe	Keiyo System Research, Inc.	\N
0011ff	Digitro Tecnologia Ltda	\N
001200	Cisco	\N
001201	Cisco	\N
001202	Decrane Aerospace - Audio International Inc.	\N
001203	Activ Networks	\N
001204	u10 Networks, Inc.	\N
001205	Terrasat Communications, Inc.	\N
001206	iQuest (NZ) Ltd	\N
001207	Head Strong International Limited	\N
001208	Gantner Instruments GmbH	\N
001209	Fastrax Ltd	\N
00120a	Emerson Electric GmbH & Co. OHG	\N
00120b	Chinasys Technologies Limited	\N
00120c	CE-Infosys Pte Ltd	\N
00120d	Advanced Telecommunication Technologies, Inc.	\N
00120e	AboCom	\N
00120f	IEEE 802.3	\N
001210	WideRay Corp	\N
001211	Protechna Herbst GmbH & Co. KG	\N
001212	PLUS  Corporation	\N
001213	Metrohm AG	\N
001214	Koenig & Bauer AG	\N
001215	iStor Networks, Inc.	\N
001216	ICP Internet Communication Payment AG	\N
001217	Cisco-Linksys, LLC	\N
001218	ARUZE Corporation	\N
001219	Ahead Communication Systems Inc	\N
00121a	Techno Soft Systemnics Inc.	\N
00121b	Sound Devices, LLC	\N
00121c	PARROT S.A.	\N
00121d	Netfabric Corporation	\N
00121e	Juniper Networks, Inc.	\N
00121f	Harding Intruments	\N
001220	Cadco Systems	\N
001221	B.Braun Melsungen AG	\N
001222	Skardin (UK) Ltd	\N
001223	Pixim	\N
001224	NexQL Corporation	\N
001225	Motorola BCS	\N
001226	Japan Direx Corporation	\N
001227	Franklin Electric Co., Inc.	\N
001228	Data Ltd.	\N
001229	BroadEasy Technologies Co.,Ltd	\N
00122a	VTech Telecommunications Ltd.	\N
00122b	Virbiage Pty Ltd	\N
00122c	Soenen Controls N.V.	\N
00122d	SiNett Corporation	\N
00122e	Signal Technology - AISD	\N
00122f	Sanei Electric Inc.	\N
001230	Picaso Infocommunication CO., LTD.	\N
001231	Motion Control Systems, Inc.	\N
001232	LeWiz Communications Inc.	\N
001233	JRC TOKKI Co.,Ltd.	\N
001234	Camille Bauer	\N
001235	Andrew Corporation	\N
001236	ConSentry Networks	\N
001237	Texas Instruments	\N
001238	SetaBox Technology Co., Ltd.	\N
001239	S Net Systems Inc.	\N
00123a	Posystech Inc., Co.	\N
00123b	KeRo Systems ApS	\N
00123c	Second Rule LLC	\N
00123d	GES	\N
00123e	ERUNE technology Co., Ltd.	\N
00123f	Dell Inc	\N
001240	AMOI ELECTRONICS CO.,LTD	\N
001241	a2i marketing center	\N
001242	Millennial Net	\N
001243	Cisco	\N
001244	Cisco	\N
001245	Zellweger Analytics, Inc.	\N
001246	T.O.M TECHNOLOGY INC..	\N
001247	Samsung Electronics Co., Ltd.	\N
001248	EMC Corporation (Kashya)	\N
001249	Delta Elettronica S.p.A.	\N
00124a	Dedicated Devices, Inc.	\N
00124b	Texas Instruments	\N
00124c	BBWM Corporation	\N
00124d	Inducon BV	\N
00124e	XAC AUTOMATION CORP.	\N
00124f	Tyco Thermal Controls LLC.	\N
001250	Tokyo Aircaft Instrument Co., Ltd.	\N
001251	SILINK	\N
001252	Citronix, LLC	\N
001253	AudioDev AB	\N
001254	Spectra Technologies Holdings Company Ltd	\N
001255	NetEffect Incorporated	\N
001256	LG INFORMATION & COMM.	\N
001257	LeapComm Communication Technologies Inc.	\N
001258	Activis Polska	\N
001259	THERMO ELECTRON KARLSRUHE	\N
00125a	Microsoft Corporation	\N
00125b	KAIMEI ELECTRONI	\N
00125c	Green Hills Software, Inc.	\N
00125d	CyberNet Inc.	\N
00125e	CAEN	\N
00125f	AWIND Inc.	\N
001260	Stanton Magnetics,inc.	\N
001261	Adaptix, Inc	\N
001262	Nokia Danmark A/S	\N
001263	Data Voice Technologies GmbH	\N
001264	daum electronic gmbh	\N
001265	Enerdyne Technologies, Inc.	\N
001266	Swisscom Hospitality Services SA	\N
001267	Matsushita Electronic Components Co., Ltd.	\N
001268	IPS d.o.o.	\N
001269	Value Electronics	\N
00126a	OPTOELECTRONICS Co., Ltd.	\N
00126b	Ascalade Communications Limited	\N
00126c	Visonic Ltd.	\N
00126d	University of California, Berkeley	\N
00126e	Seidel Elektronik GmbH Nfg.KG	\N
00126f	Rayson Technology Co., Ltd.	\N
001270	NGES Denro Systems	\N
001271	Measurement Computing Corp	\N
001272	Redux Communications Ltd.	\N
001273	Stoke Inc	\N
001274	NIT lab	\N
001275	Sentilla Corporation	\N
001276	Microsol Holdings Ltd.	\N
001277	Korenix Technologies Co., Ltd.	\N
001278	International Bar Code	\N
001279	Hewlett Packard	\N
00127a	Sanyu Industry Co.,Ltd.	\N
00127b	VIA Networking Technologies, Inc.	\N
00127c	SWEGON AB	\N
00127d	MobileAria	\N
00127e	Digital Lifestyles Group, Inc.	\N
00127f	Cisco	\N
001280	Cisco	\N
001281	CIEFFE srl	\N
001282	Qovia	\N
001283	Nortel Networks	\N
001284	Lab33 Srl	\N
001285	Gizmondo Europe Ltd	\N
001286	ENDEVCO CORP	\N
001287	Digital Everywhere Unterhaltungselektronik GmbH	\N
001288	2Wire, Inc	\N
001289	Advance Sterilization Products	\N
00128a	Motorola PCS	\N
00128b	Sensory Networks Inc	\N
00128c	Woodward Governor	\N
00128d	STB Datenservice GmbH	\N
00128e	Q-Free ASA	\N
00128f	Montilio	\N
001290	KYOWA Electric & Machinery Corp.	\N
001291	KWS Computersysteme GmbH	\N
001292	Griffin Technology	\N
001293	GE Energy	\N
001294	SUMITOMO ELECTRIC DEVICE INNOVATIONS, INC	\N
001295	Aiware Inc.	\N
001296	Addlogix	\N
001297	O2Micro, Inc.	\N
001298	MICO ELECTRIC(SHENZHEN) LIMITED	\N
001299	Ktech Telecommunications Inc	\N
00129a	IRT Electronics Pty Ltd	\N
00129b	E2S Electronic Engineering Solutions, S.L.	\N
00129c	Yulinet	\N
00129d	First International Computer do Brasil	\N
00129e	Surf Communications Inc.	\N
00129f	RAE Systems, Inc.	\N
0012a0	NeoMeridian Sdn Bhd	\N
0012a1	BluePacket Communications Co., Ltd.	\N
0012a2	VITA	\N
0012a3	Trust International B.V.	\N
0012a4	ThingMagic, LLC	\N
0012a5	Stargen, Inc.	\N
0012a6	Dolby Australia	\N
0012a7	ISR TECHNOLOGIES Inc	\N
0012a8	intec GmbH	\N
0012a9	3Com Ltd	\N
0012aa	IEE, Inc.	\N
0012ab	WiLife, Inc.	\N
0012ac	ONTIMETEK INC.	\N
0012ad	IDS GmbH	\N
0012ae	HLS HARD-LINE Solutions Inc.	\N
0012af	ELPRO Technologies	\N
0012b0	Efore Oyj   (Plc)	\N
0012b1	Dai Nippon Printing Co., Ltd	\N
0012b2	AVOLITES LTD.	\N
0012b3	Advance Wireless Technology Corp.	\N
0012b4	Work Microwave GmbH	\N
0012b5	Vialta, Inc.	\N
0012b6	Santa Barbara Infrared, Inc.	\N
0012b7	PTW Freiburg	\N
0012b8	G2 Microsystems	\N
0012b9	Fusion Digital Technology	\N
0012ba	FSI Systems, Inc.	\N
0012bb	Telecommunications Industry Association TR-41 Committee	\N
0012bc	Echolab LLC	\N
0012bd	Avantec Manufacturing Limited	\N
0012be	Astek Corporation	\N
0012bf	Arcadyan Technology Corporation	\N
0012c0	HotLava Systems, Inc.	\N
0012c1	Check Point Software Technologies	\N
0012c2	Apex Electronics Factory	\N
0012c3	WIT S.A.	\N
0012c4	Viseon, Inc.	\N
0012c5	V-Show  Technology (China) Co.,Ltd	\N
0012c6	TGC America, Inc	\N
0012c7	SECURAY Technologies Ltd.Co.	\N
0012c8	Perfect tech	\N
0012c9	Motorola BCS	\N
0012ca	Mechatronic Brick Aps	\N
0012cb	CSS Inc.	\N
0012cc	Bitatek CO., LTD	\N
0012cd	ASEM SpA	\N
0012ce	Advanced Cybernetics Group	\N
0012cf	Accton Technology Corporation	\N
0012d0	Gossen-Metrawatt-GmbH	\N
0012d1	Texas Instruments Inc	\N
0012d2	Texas Instruments	\N
0012d3	Zetta Systems, Inc.	\N
0012d4	Princeton Technology, Ltd	\N
0012d5	Motion Reality Inc.	\N
0012d6	Jiangsu Yitong High-Tech Co.,Ltd	\N
0012d7	Invento Networks, Inc.	\N
0012d8	International Games System Co., Ltd.	\N
0012d9	Cisco Systems	\N
0012da	Cisco Systems	\N
0012db	ZIEHL industrie-elektronik GmbH + Co KG	\N
0012dc	SunCorp Industrial Limited	\N
0012dd	Shengqu Information Technology (Shanghai) Co., Ltd.	\N
0012de	Radio Components Sweden AB	\N
0012df	Novomatic AG	\N
0012e0	Codan Limited	\N
0012e1	Alliant Networks, Inc	\N
0012e2	ALAXALA Networks Corporation	\N
0012e3	Agat-RT, Ltd.	\N
0012e4	ZIEHL industrie-electronik GmbH + Co KG	\N
0012e5	Time America, Inc.	\N
0012e6	SPECTEC COMPUTER CO., LTD.	\N
0012e7	Projectek Networking Electronics Corp.	\N
0012e8	Fraunhofer IMS	\N
0012e9	Abbey Systems Ltd	\N
0012ea	Trane	\N
0012eb	R2DI, LLC	\N
0012ec	Movacolor b.v.	\N
0012ed	AVG Advanced Technologies	\N
0012ee	Sony Ericsson Mobile Communications AB	\N
0012ef	OneAccess SA	\N
0012f0	Intel Corporate	\N
0012f1	IFOTEC	\N
0012f2	Brocade Communications Systems, Inc	\N
0012f3	connectBlue AB	\N
0012f4	Belco International Co.,Ltd.	\N
0012f5	Imarda New Zealand Limited	\N
0012f6	MDK CO.,LTD.	\N
0012f7	Xiamen Xinglian Electronics Co., Ltd.	\N
0012f8	WNI Resources, LLC	\N
0012f9	URYU SEISAKU, LTD.	\N
0012fa	THX LTD	\N
0012fb	Samsung Electronics	\N
0012fc	PLANET System Co.,LTD	\N
0012fd	OPTIMUS IC S.A.	\N
0012fe	Lenovo Mobile Communication Technology Ltd.	\N
0012ff	Lely Industries N.V.	\N
001300	IT-FACTORY, INC.	\N
001301	IronGate S.L.	\N
001302	Intel Corporate	\N
001303	GateConnect Technologies GmbH	\N
001304	Flaircomm Technologies Co. LTD	\N
001305	Epicom, Inc.	\N
001306	Always On Wireless	\N
001307	Paravirtual Corporation	\N
001308	Nuvera Fuel Cells	\N
001309	Ocean Broadband Networks	\N
00130a	Nortel	\N
00130b	Mextal B.V.	\N
00130c	HF System Corporation	\N
00130d	GALILEO AVIONICA	\N
00130e	Focusrite Audio Engineering Limited	\N
00130f	EGEMEN Bilgisayar Muh San ve Tic LTD STI	\N
001310	Cisco-Linksys, LLC	\N
001311	ARRIS International	\N
001312	Amedia Networks Inc.	\N
001313	GuangZhou Post & Telecom Equipment ltd	\N
001314	Asiamajor Inc.	\N
001315	SONY Computer Entertainment inc,	\N
001316	L-S-B Broadcast Technologies GmbH	\N
001317	GN Netcom as	\N
001318	DGSTATION Co., Ltd.	\N
001319	Cisco Systems	\N
00131a	Cisco Systems	\N
00131b	BeCell Innovations Corp.	\N
00131c	LiteTouch, Inc.	\N
00131d	Scanvaegt International A/S	\N
00131e	Peiker acustic GmbH & Co. KG	\N
00131f	NxtPhase T&D, Corp.	\N
001320	Intel Corporate	\N
001321	Hewlett Packard	\N
001322	DAQ Electronics, Inc.	\N
001323	Cap Co., Ltd.	\N
001324	Schneider Electric Ultra Terminal	\N
001325	Cortina Systems Inc	\N
001326	ECM Systems Ltd	\N
001327	Data Acquisitions limited	\N
001328	Westech Korea Inc.,	\N
001329	VSST Co., LTD	\N
00132a	Sitronics Telecom Solutions	\N
00132b	Phoenix Digital	\N
00132c	MAZ Brandenburg GmbH	\N
00132d	iWise Communications	\N
00132e	ITian Coporation	\N
00132f	Interactek	\N
001330	EURO PROTECTION SURVEILLANCE	\N
001331	CellPoint Connect	\N
001332	Beijing Topsec Network Security Technology Co., Ltd.	\N
001333	BaudTec Corporation	\N
001334	Arkados, Inc.	\N
001335	VS Industry Berhad	\N
001336	Tianjin 712 Communication Broadcasting co., ltd.	\N
001337	Orient Power Home Network Ltd.	\N
001338	FRESENIUS-VIAL	\N
001339	EL-ME AG	\N
00133a	VadaTech Inc.	\N
00133b	Speed Dragon Multimedia Limited	\N
00133c	QUINTRON SYSTEMS INC.	\N
00133d	Micro Memory Curtiss Wright Co	\N
00133e	MetaSwitch	\N
00133f	Eppendorf Instrumente GmbH	\N
001340	AD.EL s.r.l.	\N
001341	Shandong New Beiyang Information Technology Co.,Ltd	\N
001342	Vision Research, Inc.	\N
001343	Matsushita Electronic Components (Europe) GmbH	\N
001344	Fargo Electronics Inc.	\N
001345	Eaton Corporation	\N
001346	D-Link Corporation	\N
001347	BlueTree Wireless Data Inc.	\N
001348	Artila Electronics Co., Ltd.	\N
001349	ZyXEL Communications Corporation	\N
00134a	Engim, Inc.	\N
00134b	ToGoldenNet Technology Inc.	\N
00134c	YDT Technology International	\N
00134d	IPC systems	\N
00134e	Valox Systems, Inc.	\N
00134f	Tranzeo Wireless Technologies Inc.	\N
001350	Silver Spring Networks, Inc	\N
001351	Niles Audio Corporation	\N
001352	Naztec, Inc.	\N
001353	HYDAC Filtertechnik GMBH	\N
001354	Zcomax Technologies, Inc.	\N
001355	TOMEN Cyber-business Solutions, Inc.	\N
001356	target systemelectronic gmbh	\N
001357	Soyal Technology Co., Ltd.	\N
001358	Realm Systems, Inc.	\N
001359	ProTelevision Technologies A/S	\N
00135a	Project T&E Limited	\N
00135b	PanelLink Cinema, LLC	\N
00135c	OnSite Systems, Inc.	\N
00135d	NTTPC Communications, Inc.	\N
00135e	EAB/RWI/K	\N
00135f	Cisco Systems	\N
001360	Cisco Systems	\N
001361	Biospace Co., Ltd.	\N
001362	ShinHeung Precision Co., Ltd.	\N
001363	Verascape, Inc.	\N
001364	Paradigm Technology Inc..	\N
001365	Nortel	\N
001366	Neturity Technologies Inc.	\N
001367	Narayon. Co., Ltd.	\N
001368	Maersk Data Defence	\N
001369	Honda Electron Co., LED.	\N
00136a	Hach Lange SA	\N
00136b	E-TEC	\N
00136c	TomTom	\N
00136d	Tentaculus AB	\N
00136e	Techmetro Corp.	\N
00136f	PacketMotion, Inc.	\N
001370	Nokia Danmark A/S	\N
001371	Motorola CHS	\N
001372	Dell Inc.	\N
001373	BLwave Electronics Co., Ltd	\N
001374	Atheros Communications, Inc.	\N
001375	American Security Products Co.	\N
001376	Tabor Electronics Ltd.	\N
001377	Samsung Electronics CO., LTD	\N
001378	QSAN Technology, Inc.	\N
001379	PONDER INFORMATION INDUSTRIES LTD.	\N
00137a	Netvox Technology Co., Ltd.	\N
00137b	Movon Corporation	\N
00137c	Kaicom co., Ltd.	\N
00137d	Dynalab, Inc.	\N
00137e	CorEdge Networks, Inc.	\N
00137f	Cisco Systems	\N
001380	Cisco Systems	\N
001381	CHIPS & Systems, Inc.	\N
001382	Cetacea Networks Corporation	\N
001383	Application Technologies and Engineering Research Laboratory	\N
001384	Advanced Motion Controls	\N
001385	Add-On Technology Co., LTD.	\N
001386	ABB Inc./Totalflow	\N
001387	27M Technologies AB	\N
001388	WiMedia Alliance	\N
001389	Redes de Telefonía Móvil S.A.	\N
00138a	QINGDAO GOERTEK ELECTRONICS CO.,LTD.	\N
00138b	Phantom Technologies LLC	\N
00138c	Kumyoung.Co.Ltd	\N
00138d	Kinghold	\N
00138e	FOAB Elektronik AB	\N
00138f	Asiarock Incorporation	\N
001390	Termtek Computer Co., Ltd	\N
001391	OUEN CO.,LTD.	\N
001392	Ruckus Wireless	\N
001393	Panta Systems, Inc.	\N
001394	Infohand Co.,Ltd	\N
001395	congatec AG	\N
001396	Acbel Polytech Inc.	\N
001397	Xsigo Systems, Inc.	\N
001398	TrafficSim Co.,Ltd	\N
001399	STAC Corporation.	\N
00139a	K-ubique ID Corp.	\N
00139b	ioIMAGE Ltd.	\N
00139c	Exavera Technologies, Inc.	\N
00139d	Design of Systems on Silicon S.A.	\N
00139e	Ciara Technologies Inc.	\N
00139f	Electronics Design Services, Co., Ltd.	\N
0013a0	ALGOSYSTEM Co., Ltd.	\N
0013a1	Crow Electronic Engeneering	\N
0013a2	MaxStream, Inc	\N
0013a3	Siemens Com CPE Devices	\N
0013a4	KeyEye Communications	\N
0013a5	General Solutions, LTD.	\N
0013a6	Extricom Ltd	\N
0013a7	BATTELLE MEMORIAL INSTITUTE	\N
0013a8	Tanisys Technology	\N
0013a9	Sony Corporation	\N
0013aa	ALS  & TEC Ltd.	\N
0013ab	Telemotive AG	\N
0013ac	Sunmyung Electronics Co., LTD	\N
0013ad	Sendo Ltd	\N
0013ae	Radiance Technologies, Inc.	\N
0013af	NUMA Technology,Inc.	\N
0013b0	Jablotron	\N
0013b1	Intelligent Control Systems (Asia) Pte Ltd	\N
0013b2	Carallon Limited	\N
0013b3	Ecom Communications Technology Co., Ltd.	\N
0013b4	Appear TV	\N
0013b5	Wavesat	\N
0013b6	Sling Media, Inc.	\N
0013b7	Scantech ID	\N
0013b8	RyCo Electronic Systems Limited	\N
0013b9	BM SPA	\N
0013ba	ReadyLinks Inc	\N
0013bb	Smartvue Corporation	\N
0013bc	Artimi Ltd	\N
0013bd	HYMATOM SA	\N
0013be	Virtual Conexions	\N
0013bf	Media System Planning Corp.	\N
0013c0	Trix Tecnologia Ltda.	\N
0013c1	Asoka USA Corporation	\N
0013c2	WACOM Co.,Ltd	\N
0013c3	Cisco Systems	\N
0013c4	Cisco Systems	\N
0013c5	LIGHTRON FIBER-OPTIC DEVICES INC.	\N
0013c6	OpenGear, Inc	\N
0013c7	IONOS Co.,Ltd.	\N
0013c8	PIRELLI BROADBAND SOLUTIONS S.P.A.	\N
0013c9	Beyond Achieve Enterprises Ltd.	\N
0013ca	X-Digital Systems, Inc.	\N
0013cb	Zenitel Norway AS	\N
0013cc	Tall Maple Systems	\N
0013cd	MTI co. LTD	\N
0013ce	Intel Corporate	\N
0013cf	4Access Communications	\N
0013d0	t+ Medical Ltd	\N
0013d1	KIRK telecom A/S	\N
0013d2	PAGE IBERICA, S.A.	\N
0013d3	MICRO-STAR INTERNATIONAL CO., LTD.	\N
0013d4	ASUSTek COMPUTER INC.	\N
0013d5	WiNetworks LTD	\N
0013d6	TII NETWORK TECHNOLOGIES, INC.	\N
0013d7	SPIDCOM Technologies SA	\N
0013d8	Princeton Instruments	\N
0013d9	Matrix Product Development, Inc.	\N
0013da	Diskware Co., Ltd	\N
0013db	SHOEI Electric Co.,Ltd	\N
0013dc	IBTEK INC.	\N
0013dd	Abbott Diagnostics	\N
0013de	Adapt4, LLC	\N
0013df	Ryvor Corp.	\N
0013e0	Murata Manufacturing Co., Ltd.	\N
0013e1	Iprobe AB	\N
0013e2	GeoVision Inc.	\N
0013e3	CoVi Technologies, Inc.	\N
0013e4	YANGJAE SYSTEMS CORP.	\N
0013e5	TENOSYS, INC.	\N
0013e6	Technolution	\N
0013e7	Halcro	\N
0013e8	Intel Corporate	\N
0013e9	VeriWave, Inc.	\N
0013ea	Kamstrup A/S	\N
0013eb	Sysmaster Corporation	\N
0013ec	Sunbay Software AG	\N
0013ed	PSIA	\N
0013ee	JBX Designs Inc.	\N
0013ef	Kingjon Digital Technology Co.,Ltd	\N
0013f0	Wavefront Semiconductor	\N
0013f1	AMOD Technology Co., Ltd.	\N
0013f2	Klas Ltd	\N
0013f3	Giga-byte Communications Inc.	\N
0013f4	Psitek (Pty) Ltd	\N
0013f5	Akimbi Systems	\N
0013f6	Cintech	\N
0013f7	SMC Networks, Inc.	\N
0013f8	Dex Security Solutions	\N
0013f9	Cavera Systems	\N
0013fa	LifeSize Communications, Inc	\N
0013fb	RKC INSTRUMENT INC.	\N
0013fc	SiCortex, Inc	\N
0013fd	Nokia Danmark A/S	\N
0013fe	GRANDTEC ELECTRONIC CORP.	\N
0013ff	Dage-MTI of MC, Inc.	\N
001400	MINERVA KOREA CO., LTD	\N
001401	Rivertree Networks Corp.	\N
001402	kk-electronic a/s	\N
001403	Renasis, LLC	\N
001404	Motorola CHS	\N
001405	OpenIB, Inc.	\N
001406	Go Networks	\N
001407	Sperian Protection Instrumentation	\N
001408	Eka Systems Inc.	\N
001409	MAGNETI MARELLI   S.E. S.p.A.	\N
00140a	WEPIO Co., Ltd.	\N
00140b	FIRST INTERNATIONAL COMPUTER, INC.	\N
00140c	GKB CCTV CO., LTD.	\N
00140d	Nortel	\N
00140e	Nortel	\N
00140f	Federal State Unitary Enterprise Leningrad R&D Institute of	\N
001410	Suzhou Keda Technology CO.,Ltd	\N
001411	Deutschmann Automation GmbH & Co. KG	\N
001412	S-TEC electronics AG	\N
001413	Trebing & Himstedt Prozessautomation GmbH & Co. KG	\N
001414	Jumpnode Systems LLC.	\N
001415	Intec Automation Inc.	\N
001416	Scosche Industries, Inc.	\N
001417	RSE Informations Technologie GmbH	\N
001418	C4Line	\N
001419	SIDSA	\N
00141a	DEICY CORPORATION	\N
00141b	Cisco Systems	\N
00141c	Cisco Systems	\N
00141d	Lust Antriebstechnik GmbH	\N
00141e	P.A. Semi, Inc.	\N
00141f	SunKwang Electronics Co., Ltd	\N
001420	G-Links networking company	\N
001421	Total Wireless Technologies Pte. Ltd.	\N
001422	Dell Inc.	\N
001423	J-S Co. NEUROCOM	\N
001424	Merry Electrics CO., LTD.	\N
001425	Galactic Computing Corp.	\N
001426	NL Technology	\N
001427	JazzMutant	\N
001428	Vocollect, Inc	\N
001429	V Center Technologies Co., Ltd.	\N
00142a	Elitegroup Computer System Co., Ltd	\N
00142b	Edata Communication Inc.	\N
00142c	Koncept International, Inc.	\N
00142d	Toradex AG	\N
00142e	77 Elektronika Kft.	\N
00142f	WildPackets	\N
001430	ViPowER, Inc	\N
001431	PDL Electronics Ltd	\N
001432	Tarallax Wireless, Inc.	\N
001433	Empower Technologies(Canada) Inc.	\N
001434	Keri Systems, Inc	\N
001435	CityCom Corp.	\N
001436	Qwerty Elektronik AB	\N
001437	GSTeletech Co.,Ltd.	\N
001438	Hewlett Packard	\N
001439	Blonder Tongue Laboratories, Inc.	\N
00143a	RAYTALK INTERNATIONAL SRL	\N
00143b	Sensovation AG	\N
00143c	Rheinmetall Canada Inc.	\N
00143d	Aevoe Inc.	\N
00143e	AirLink Communications, Inc.	\N
00143f	Hotway Technology Corporation	\N
001440	ATOMIC Corporation	\N
001441	Innovation Sound Technology Co., LTD.	\N
001442	ATTO CORPORATION	\N
001443	Consultronics Europe Ltd	\N
001444	Grundfos Electronics	\N
001445	Telefon-Gradnja d.o.o.	\N
001446	SuperVision Solutions LLC	\N
001447	BOAZ Inc.	\N
001448	Inventec Multimedia & Telecom Corporation	\N
001449	Sichuan Changhong Electric Ltd.	\N
00144a	Taiwan Thick-Film Ind. Corp.	\N
00144b	Hifn, Inc.	\N
00144c	General Meters Corp.	\N
00144d	Intelligent Systems	\N
00144e	SRISA	\N
00144f	Sun Microsystems, Inc.	\N
001450	Heim Systems GmbH	\N
001451	Apple Computer Inc.	\N
001452	CALCULEX,INC.	\N
001453	ADVANTECH TECHNOLOGIES CO.,LTD	\N
001454	Symwave	\N
001455	Coder Electronics Corporation	\N
001456	Edge Products	\N
001457	T-VIPS AS	\N
001458	HS Automatic ApS	\N
001459	Moram Co., Ltd.	\N
00145a	Neratec AG	\N
00145b	SeekerNet Inc.	\N
00145c	Intronics B.V.	\N
00145d	WJ Communications, Inc.	\N
00145e	IBM	\N
00145f	ADITEC CO. LTD	\N
001460	Kyocera Wireless Corp.	\N
001461	CORONA CORPORATION	\N
001462	Digiwell Technology, inc	\N
001463	IDCS N.V.	\N
001464	Cryptosoft	\N
001465	Novo Nordisk A/S	\N
001466	Kleinhenz Elektronik GmbH	\N
001467	ArrowSpan Inc.	\N
001468	CelPlan International, Inc.	\N
001469	Cisco Systems	\N
00146a	Cisco Systems	\N
00146b	Anagran, Inc.	\N
00146c	Netgear Inc.	\N
00146d	RF Technologies	\N
00146e	H. Stoll GmbH & Co. KG	\N
00146f	Kohler Co	\N
001470	Prokom Software SA	\N
001471	Eastern Asia Technology Limited	\N
001472	China Broadband Wireless IP Standard Group	\N
001473	Bookham Inc	\N
001474	K40 Electronics	\N
001475	Wiline Networks, Inc.	\N
001476	MultiCom Industries Limited	\N
001477	Nertec  Inc.	\N
001478	ShenZhen TP-LINK Technologies Co., Ltd.	\N
001479	NEC Magnus Communications,Ltd.	\N
00147a	Eubus GmbH	\N
00147b	Iteris, Inc.	\N
00147c	3Com Ltd	\N
00147d	Aeon Digital International	\N
00147e	InnerWireless	\N
00147f	Thomson Telecom Belgium	\N
001480	Hitachi-LG Data Storage Korea, Inc	\N
001481	Multilink Inc	\N
001482	GoBackTV, Inc	\N
001483	eXS Inc.	\N
001484	Cermate Technologies Inc.	\N
001485	Giga-Byte	\N
001486	Echo Digital Audio Corporation	\N
001487	American Technology Integrators	\N
001488	Akorri	\N
001489	B15402100 - JANDEI, S.L.	\N
00148a	Elin Ebg Traction Gmbh	\N
00148b	Globo Electronic GmbH & Co. KG	\N
00148c	Fortress Technologies	\N
00148d	Cubic Defense Simulation Systems	\N
00148e	Tele Power Inc.	\N
00148f	Protronic (Far East) Ltd.	\N
001490	ASP Corporation	\N
001491	Daniels Electronics Ltd.	\N
001492	Liteon, Mobile Media Solution SBU	\N
001493	Systimax Solutions	\N
001494	ESU AG	\N
001495	2Wire, Inc.	\N
001496	Phonic Corp.	\N
001497	ZHIYUAN Eletronics co.,ltd.	\N
001498	Viking Design Technology	\N
001499	Helicomm Inc	\N
00149a	Motorola Mobile Devices Business	\N
00149b	Nokota Communications, LLC	\N
00149c	HF Company	\N
00149d	Sound ID Inc.	\N
00149e	UbONE Co., Ltd	\N
00149f	System and Chips, Inc.	\N
0014a0	Accsense, Inc.	\N
0014a1	Synchronous Communication Corp	\N
0014a2	Core Micro Systems Inc.	\N
0014a3	Vitelec BV	\N
0014a4	Hon Hai Precision Ind. Co., Ltd.	\N
0014a5	Gemtek Technology Co., Ltd.	\N
0014a6	Teranetics, Inc.	\N
0014a7	Nokia Danmark A/S	\N
0014a8	Cisco Systems	\N
0014a9	Cisco Systems	\N
0014aa	Ashly Audio, Inc.	\N
0014ab	Senhai Electronic Technology Co., Ltd.	\N
0014ac	Bountiful WiFi	\N
0014ad	Gassner Wiege- u. Meßtechnik GmbH	\N
0014ae	Wizlogics Co., Ltd.	\N
0014af	Datasym Inc.	\N
0014b0	Naeil Community	\N
0014b1	Avitec AB	\N
0014b2	mCubelogics Corporation	\N
0014b3	CoreStar International Corp	\N
0014b4	General Dynamics United Kingdom Ltd	\N
0014b5	PHYSIOMETRIX,INC	\N
0014b6	Enswer Technology Inc.	\N
0014b7	AR Infotek Inc.	\N
0014b8	Hill-Rom	\N
0014b9	MSTAR SEMICONDUCTOR	\N
0014ba	Carvers SA de CV	\N
0014bb	Open Interface North America	\N
0014bc	SYNECTIC TELECOM EXPORTS PVT. LTD.	\N
0014bd	incNETWORKS, Inc	\N
0014be	Wink communication technology CO.LTD	\N
0014bf	Cisco-Linksys LLC	\N
0014c0	Symstream Technology Group Ltd	\N
0014c1	U.S. Robotics Corporation	\N
0014c2	Hewlett Packard	\N
0014c3	Seagate Technology LLC	\N
0014c4	Vitelcom Mobile Technology	\N
0014c5	Alive Technologies Pty Ltd	\N
0014c6	Quixant Ltd	\N
0014c7	Nortel	\N
0014c8	Contemporary Research Corp	\N
0014c9	Brocade Communications Systems, Inc.	\N
0014ca	Key Radio Systems Limited	\N
0014cb	LifeSync Corporation	\N
0014cc	Zetec, Inc.	\N
0014cd	DigitalZone Co., Ltd.	\N
0014ce	NF CORPORATION	\N
0014cf	Nextlink.to A/S	\N
0014d0	BTI Systems Inc.	\N
0014d1	TRENDnet	\N
0014d2	KYUKI CORPORATION	\N
0014d3	SEPSA	\N
0014d4	K Technology Corporation	\N
0014d5	Datang Telecom Technology CO. , LCD,Optical Communication Br	\N
0014d6	Jeongmin Electronics Co.,Ltd.	\N
0014d7	Datastore Technology Corp	\N
0014d8	bio-logic SA	\N
0014d9	IP Fabrics, Inc.	\N
0014da	Huntleigh Healthcare	\N
0014db	Elma Trenew Electronic GmbH	\N
0014dc	Communication System Design & Manufacturing (CSDM)	\N
0014dd	Covergence Inc.	\N
0014de	Sage Instruments Inc.	\N
0014df	HI-P Tech Corporation	\N
0014e0	LET'S Corporation	\N
0014e1	Data Display AG	\N
0014e2	datacom systems inc.	\N
0014e3	mm-lab GmbH	\N
0014e4	Integral Technologies	\N
0014e5	Alticast	\N
0014e6	AIM Infrarotmodule GmbH	\N
0014e7	Stolinx,. Inc	\N
0014e8	Motorola CHS	\N
0014e9	Nortech International	\N
0014ea	S Digm Inc. (Safe Paradigm Inc.)	\N
0014eb	AwarePoint Corporation	\N
0014ec	Acro Telecom	\N
0014ed	Airak, Inc.	\N
0014ee	Western Digital Technologies, Inc.	\N
0014ef	TZero Technologies, Inc.	\N
0014f0	Business Security OL AB	\N
0014f1	Cisco Systems	\N
0014f2	Cisco Systems	\N
0014f3	ViXS Systems Inc	\N
0014f4	DekTec Digital Video B.V.	\N
0014f5	OSI Security Devices	\N
0014f6	Juniper Networks, Inc.	\N
0014f7	Crevis	\N
0014f8	Scientific Atlanta	\N
0014f9	Vantage Controls	\N
0014fa	AsGa S.A.	\N
0014fb	Technical Solutions Inc.	\N
0014fc	Extandon, Inc.	\N
0014fd	Thecus Technology Corp.	\N
0014fe	Artech Electronics	\N
0014ff	Precise Automation, Inc.	\N
001500	Intel Corporate	\N
001501	LexBox	\N
001502	BETA tech	\N
001503	PROFIcomms s.r.o.	\N
001504	GAME PLUS CO., LTD.	\N
001505	Actiontec Electronics, Inc	\N
001506	Neo Photonics	\N
001507	Renaissance Learning Inc	\N
001508	Global Target Enterprise Inc	\N
001509	Plus Technology Co., Ltd	\N
00150a	Sonoa Systems, Inc	\N
00150b	SAGE INFOTECH LTD.	\N
00150c	AVM GmbH	\N
00150d	Hoana Medical, Inc.	\N
00150e	OPENBRAIN TECHNOLOGIES CO., LTD.	\N
00150f	mingjong	\N
001510	Techsphere Co., Ltd	\N
001511	Data Center Systems	\N
001512	Zurich University of Applied Sciences	\N
001513	EFS sas	\N
001514	Hu Zhou NAVA Networks&Electronics Ltd.	\N
001515	Leipold+Co.GmbH	\N
001516	URIEL SYSTEMS INC.	\N
001517	Intel Corporate	\N
001518	Shenzhen 10MOONS Technology Development CO.,Ltd	\N
001519	StoreAge Networking Technologies	\N
00151a	Hunter Engineering Company	\N
00151b	Isilon Systems Inc.	\N
00151c	LENECO	\N
00151d	M2I CORPORATION	\N
00151e	Ethernet Powerlink Standardization Group (EPSG)	\N
00151f	Multivision Intelligent Surveillance (Hong Kong) Ltd	\N
001520	Radiocrafts AS	\N
001521	Horoquartz	\N
001522	Dea Security	\N
001523	Meteor Communications Corporation	\N
001524	Numatics, Inc.	\N
001525	Chamberlain Access Solutions	\N
001526	Remote Technologies Inc	\N
001527	Balboa Instruments	\N
001528	Beacon Medical Products LLC d.b.a. BeaconMedaes	\N
001529	N3 Corporation	\N
00152a	Nokia GmbH	\N
00152b	Cisco Systems	\N
00152c	Cisco Systems	\N
00152d	TenX Networks, LLC	\N
00152e	PacketHop, Inc.	\N
00152f	Motorola CHS	\N
001530	Bus-Tech, Inc.	\N
001531	KOCOM	\N
001532	Consumer Technologies Group, LLC	\N
001533	NADAM.CO.,LTD	\N
001534	A BELTRÓNICA, Companhia de Comunicações, Lda	\N
001535	OTE Spa	\N
001536	Powertech co.,Ltd	\N
001537	Ventus Networks	\N
001538	RFID, Inc.	\N
001539	Technodrive SRL	\N
00153a	Shenzhen Syscan Technology Co.,Ltd.	\N
00153b	EMH Elektrizitätszähler GmbH & CoKG	\N
00153c	Kprotech Co., Ltd.	\N
00153d	ELIM PRODUCT CO.	\N
00153e	Q-Matic Sweden AB	\N
00153f	Alcatel Alenia Space Italia	\N
001540	Nortel	\N
001541	StrataLight Communications, Inc.	\N
001542	MICROHARD S.R.L.	\N
001543	Aberdeen Test Center	\N
001544	coM.s.a.t. AG	\N
001545	SEECODE Co., Ltd.	\N
001546	ITG Worldwide Sdn Bhd	\N
001547	AiZen Solutions Inc.	\N
001548	CUBE TECHNOLOGIES	\N
001549	Dixtal Biomedica Ind. Com. Ltda	\N
00154a	WANSHIH ELECTRONIC CO., LTD	\N
00154b	Wonde Proud Technology Co., Ltd	\N
00154c	Saunders Electronics	\N
00154d	Netronome Systems, Inc.	\N
00154e	IEC	\N
00154f	one RF Technology	\N
001550	Nits Technology Inc	\N
001551	RadioPulse Inc.	\N
001552	Wi-Gear Inc.	\N
001553	Cytyc Corporation	\N
001554	Atalum Wireless S.A.	\N
001555	DFM GmbH	\N
001556	SAGEM COMMUNICATION	\N
001557	Olivetti	\N
001558	FOXCONN	\N
001559	Securaplane Technologies, Inc.	\N
00155a	DAINIPPON PHARMACEUTICAL CO., LTD.	\N
00155b	Sampo Corporation	\N
00155c	Dresser Wayne	\N
00155d	Microsoft Corporation	\N
00155e	Morgan Stanley	\N
00155f	GreenPeak Technologies	\N
001560	Hewlett Packard	\N
001561	JJPlus Corporation	\N
001562	Cisco Systems	\N
001563	Cisco Systems	\N
001564	BEHRINGER Spezielle Studiotechnik GmbH	\N
001565	XIAMEN YEALINK NETWORK TECHNOLOGY CO.,LTD	\N
001566	A-First Technology Co., Ltd.	\N
001567	RADWIN Inc.	\N
001568	Dilithium Networks	\N
001569	PECO II, Inc.	\N
00156a	DG2L Technologies Pvt. Ltd.	\N
00156b	Perfisans Networks Corp.	\N
00156c	SANE SYSTEM CO., LTD	\N
00156d	Ubiquiti Networks Inc.	\N
00156e	A. W. Communication Systems Ltd	\N
00156f	Xiranet Communications GmbH	\N
001570	Symbol TechnologiesWholly owned Subsidiary of Motorola	\N
001571	Nolan Systems	\N
001572	Red-Lemon	\N
001573	NewSoft  Technology Corporation	\N
001574	Horizon Semiconductors Ltd.	\N
001575	Nevis Networks Inc.	\N
001576	scil animal care company GmbH	\N
001577	Allied Telesis	\N
001578	Audio / Video Innovations	\N
001579	Lunatone Industrielle Elektronik GmbH	\N
00157a	Telefin S.p.A.	\N
00157b	Leuze electronic GmbH + Co. KG	\N
00157c	Dave Networks, Inc.	\N
00157d	POSDATA CO., LTD.	\N
00157e	HEYFRA ELECTRONIC gmbH	\N
00157f	ChuanG International Holding CO.,LTD.	\N
001580	U-WAY CORPORATION	\N
001581	MAKUS Inc.	\N
001582	TVonics Ltd	\N
001583	IVT corporation	\N
001584	Schenck Process GmbH	\N
001585	Aonvision Technolopy Corp.	\N
001586	Xiamen Overseas Chinese Electronic Co., Ltd.	\N
001587	Takenaka Seisakusho Co.,Ltd	\N
001588	Balda-Thong Fook Solutions Sdn. Bhd.	\N
001589	D-MAX Technology Co.,Ltd	\N
00158a	SURECOM Technology Corp.	\N
00158b	Park Air Systems Ltd	\N
00158c	Liab ApS	\N
00158d	Jennic Ltd	\N
00158e	Plustek.INC	\N
00158f	NTT Advanced Technology Corporation	\N
001590	Hectronic GmbH	\N
001591	RLW Inc.	\N
001592	Facom UK Ltd (Melksham)	\N
001593	U4EA Technologies Inc.	\N
001594	BIXOLON CO.,LTD	\N
001595	Quester Tangent Corporation	\N
001596	ARRIS International	\N
001597	AETA AUDIO SYSTEMS	\N
001598	Kolektor group	\N
001599	Samsung Electronics Co., LTD	\N
00159a	Motorola CHS	\N
00159b	Nortel	\N
00159c	B-KYUNG SYSTEM Co.,Ltd.	\N
00159d	Minicom Advanced Systems ltd	\N
00159e	Mad Catz Interactive Inc	\N
00159f	Terascala, Inc.	\N
0015a0	Nokia Danmark A/S	\N
0015a1	ECA-SINTERS	\N
0015a2	ARRIS International	\N
0015a3	ARRIS International	\N
0015a4	ARRIS International	\N
0015a5	DCI Co., Ltd.	\N
0015a6	Digital Electronics Products Ltd.	\N
0015a7	Robatech AG	\N
0015a8	Motorola Mobile Devices	\N
0015a9	KWANG WOO I&C CO.,LTD	\N
0015aa	Rextechnik International Co.,	\N
0015ab	PRO CO SOUND INC	\N
0015ac	Capelon AB	\N
0015ad	Accedian Networks	\N
0015ae	kyung il	\N
0015af	AzureWave Technologies, Inc.	\N
0015b0	AUTOTELENET CO.,LTD	\N
0015b1	Ambient Corporation	\N
0015b2	Advanced Industrial Computer, Inc.	\N
0015b3	Caretech AB	\N
0015b4	Polymap  Wireless LLC	\N
0015b5	CI Network Corp.	\N
0015b6	ShinMaywa Industries, Ltd.	\N
0015b7	Toshiba	\N
0015b8	Tahoe	\N
0015b9	Samsung Electronics Co., Ltd.	\N
0015ba	iba AG	\N
0015bb	SMA Solar Technology AG	\N
0015bc	Develco	\N
0015bd	Group 4 Technology Ltd	\N
0015be	Iqua Ltd.	\N
0015bf	technicob	\N
0015c0	DIGITAL TELEMEDIA CO.,LTD.	\N
0015c1	SONY Computer Entertainment inc,	\N
0015c2	3M Germany	\N
0015c3	Ruf Telematik AG	\N
0015c4	FLOVEL CO., LTD.	\N
0015c5	Dell Inc	\N
0015c6	Cisco Systems	\N
0015c7	Cisco Systems	\N
0015c8	FlexiPanel Ltd	\N
0015c9	Gumstix, Inc	\N
0015ca	TeraRecon, Inc.	\N
0015cb	Surf Communication Solutions Ltd.	\N
0015cc	TEPCO UQUEST, LTD.	\N
0015cd	Exartech International Corp.	\N
0015ce	ARRIS International	\N
0015cf	ARRIS International	\N
0015d0	ARRIS International	\N
0015d1	ARRIS Group, Inc.	\N
0015d2	Xantech Corporation	\N
0015d3	Pantech&Curitel Communications, Inc.	\N
0015d4	Emitor AB	\N
0015d5	NICEVT	\N
0015d6	OSLiNK Sp. z o.o.	\N
0015d7	Reti Corporation	\N
0015d8	Interlink Electronics	\N
0015d9	PKC Electronics Oy	\N
0015da	IRITEL A.D.	\N
0015db	Canesta Inc.	\N
0015dc	KT&C Co., Ltd.	\N
0015dd	IP Control Systems Ltd.	\N
0015de	Nokia Danmark A/S	\N
0015df	Clivet S.p.A.	\N
0015e0	ST-Ericsson	\N
0015e1	picoChip Designs Ltd	\N
0015e2	Dr.Ing. Herbert Knauer GmbH	\N
0015e3	Dream Technologies Corporation	\N
0015e4	Zimmer Elektromedizin	\N
0015e5	Cheertek Inc.	\N
0015e6	MOBILE TECHNIKA Inc.	\N
0015e7	Quantec ProAudio	\N
0015e8	Nortel	\N
0015e9	D-Link Corporation	\N
0015ea	Tellumat (Pty) Ltd	\N
0015eb	ZTE CORPORATION	\N
0015ec	Boca Devices LLC	\N
0015ed	Fulcrum Microsystems, Inc.	\N
0015ee	Omnex Control Systems	\N
0015ef	NEC TOKIN Corporation	\N
0015f0	EGO BV	\N
0015f1	KYLINK Communications Corp.	\N
0015f2	ASUSTek COMPUTER INC.	\N
0015f3	PELTOR AB	\N
0015f4	Eventide	\N
0015f5	Sustainable Energy Systems	\N
0015f6	SCIENCE AND ENGINEERING SERVICES, INC.	\N
0015f7	Wintecronics Ltd.	\N
0015f8	Kingtronics Industrial Co. Ltd.	\N
0015f9	Cisco Systems	\N
0015fa	Cisco Systems	\N
0015fb	setex schermuly textile computer gmbh	\N
0015fc	Littelfuse Startco	\N
0015fd	Complete Media Systems	\N
0015fe	SCHILLING ROBOTICS LLC	\N
0015ff	Novatel Wireless, Inc.	\N
001600	CelleBrite Mobile Synchronization	\N
001601	Buffalo Inc.	\N
001602	CEYON TECHNOLOGY CO.,LTD.	\N
001603	COOLKSKY Co., LTD	\N
001604	Sigpro	\N
001605	YORKVILLE SOUND INC.	\N
001606	Ideal Industries	\N
001607	Curves International Inc.	\N
001608	Sequans Communications	\N
001609	Unitech electronics co., ltd.	\N
00160a	SWEEX Europe BV	\N
00160b	TVWorks LLC	\N
00160c	LPL  DEVELOPMENT S.A. DE C.V	\N
00160d	Be Here Corporation	\N
00160e	Optica Technologies Inc.	\N
00160f	BADGER METER INC	\N
001610	Carina Technology	\N
001611	Altecon Srl	\N
001612	Otsuka Electronics Co., Ltd.	\N
001613	LibreStream Technologies Inc.	\N
001614	Picosecond Pulse Labs	\N
001615	Nittan Company, Limited	\N
001616	BROWAN COMMUNICATION INC.	\N
001617	MSI	\N
001618	HIVION Co., Ltd.	\N
001619	La Factoría de Comunicaciones Aplicadas,S.L.	\N
00161a	Dametric AB	\N
00161b	Micronet Corporation	\N
00161c	e:cue	\N
00161d	Innovative Wireless Technologies, Inc.	\N
00161e	Woojinnet	\N
00161f	SUNWAVETEC Co., Ltd.	\N
001620	Sony Ericsson Mobile Communications AB	\N
001621	Colorado Vnet	\N
001622	BBH SYSTEMS GMBH	\N
001623	Interval Media	\N
001624	Teneros, Inc.	\N
001625	Impinj, Inc.	\N
001626	Motorola CHS	\N
001627	embedded-logic DESIGN AND MORE GmbH	\N
001628	Ultra Electronics Manufacturing and Card Systems	\N
001629	Nivus GmbH	\N
00162a	Antik computers & communications s.r.o.	\N
00162b	Togami Electric Mfg.co.,Ltd.	\N
00162c	Xanboo	\N
00162d	STNet Co., Ltd.	\N
00162e	Space Shuttle Hi-Tech Co., Ltd.	\N
00162f	Geutebrück GmbH	\N
001630	Vativ Technologies	\N
001631	Xteam	\N
001632	SAMSUNG ELECTRONICS CO., LTD.	\N
001633	Oxford Diagnostics Ltd.	\N
001634	Mathtech, Inc.	\N
001635	Hewlett Packard	\N
001636	Quanta Computer Inc.	\N
001637	Citel Srl	\N
001638	TECOM Co., Ltd.	\N
001639	UBIQUAM Co.,Ltd	\N
00163a	YVES TECHNOLOGY CO., LTD.	\N
00163b	VertexRSI/General Dynamics	\N
00163c	Rebox B.V.	\N
00163d	Tsinghua Tongfang Legend Silicon Tech. Co., Ltd.	\N
00163e	Xensource, Inc.	\N
00163f	CReTE SYSTEMS Inc.	\N
001640	Asmobile Communication Inc.	\N
001641	USI	\N
001642	Pangolin	\N
001643	Sunhillo Corporation	\N
001644	LITE-ON Technology Corp.	\N
001645	Power Distribution, Inc.	\N
001646	Cisco Systems	\N
001647	Cisco Systems	\N
001648	SSD Company Limited	\N
001649	SetOne GmbH	\N
00164a	Vibration Technology Limited	\N
00164b	Quorion Data Systems GmbH	\N
00164c	PLANET INT Co., Ltd	\N
00164d	Alcatel North America IP Division	\N
00164e	Nokia Danmark A/S	\N
00164f	World Ethnic Broadcastin Inc.	\N
001650	Herley General Microwave Israel. 	\N
001651	Exeo Systems	\N
001652	Hoatech Technologies, Inc.	\N
001653	LEGO System A/S IE Electronics Division	\N
001654	Flex-P Industries Sdn. Bhd.	\N
001655	FUHO TECHNOLOGY Co., LTD	\N
001656	Nintendo Co., Ltd.	\N
001657	Aegate Ltd	\N
001658	Fusiontech Technologies Inc.	\N
001659	Z.M.P. RADWAG	\N
00165a	Harman Specialty Group	\N
00165b	Grip Audio	\N
00165c	Trackflow Ltd	\N
00165d	AirDefense, Inc.	\N
00165e	Precision I/O	\N
00165f	Fairmount Automation	\N
001660	Nortel	\N
001661	Novatium Solutions (P) Ltd	\N
001662	Liyuh Technology Ltd.	\N
001663	KBT Mobile	\N
001664	Prod-El SpA	\N
001665	Cellon France	\N
001666	Quantier Communication Inc.	\N
001667	A-TEC Subsystem INC.	\N
001668	Eishin Electronics	\N
001669	MRV Communication (Networks) LTD	\N
00166a	TPS	\N
00166b	Samsung Electronics	\N
00166c	Samsung Electonics Digital Video System Division	\N
00166d	Yulong Computer Telecommunication Scientific(shenzhen)Co.,Lt	\N
00166e	Arbitron Inc.	\N
00166f	Intel Corporation	\N
001670	SKNET Corporation	\N
001671	Symphox Information Co.	\N
001672	Zenway enterprise ltd	\N
001673	Bury GmbH & Co. KG	\N
001674	EuroCB (Phils.), Inc.	\N
001675	Motorola MDb	\N
001676	Intel Corporation	\N
001677	Bihl+Wiedemann GmbH	\N
001678	SHENZHEN BAOAN GAOKE ELECTRONICS CO., LTD	\N
001679	eOn Communications	\N
00167a	Skyworth Overseas Dvelopment Ltd.	\N
00167b	Haver&Boecker	\N
00167c	iRex Technologies BV	\N
00167d	Sky-Line Information Co., Ltd.	\N
00167e	DIBOSS.CO.,LTD	\N
00167f	Bluebird Soft Inc.	\N
001680	Bally Gaming + Systems	\N
001681	Vector Informatik GmbH	\N
001682	Pro Dex, Inc	\N
001683	WEBIO International Co.,.Ltd.	\N
001684	Donjin Co.,Ltd.	\N
001685	Elisa Oyj	\N
001686	Karl Storz Imaging	\N
001687	Chubb CSC-Vendor AP	\N
001688	ServerEngines LLC	\N
001689	Pilkor Electronics Co., Ltd	\N
00168a	id-Confirm Inc	\N
00168b	Paralan Corporation	\N
00168c	DSL Partner AS	\N
00168d	KORWIN CO., Ltd.	\N
00168e	Vimicro corporation	\N
00168f	GN Netcom as	\N
001690	J-TEK INCORPORATION	\N
001691	Moser-Baer AG	\N
001692	Scientific-Atlanta, Inc.	\N
001693	PowerLink Technology Inc.	\N
001694	Sennheiser Communications A/S	\N
001695	AVC Technology Limited	\N
001696	QDI Technology (H.K.) Limited	\N
001697	NEC Corporation	\N
001698	T&A Mobile Phones	\N
001699	Tonic DVB Marketing Ltd	\N
00169a	Quadrics Ltd	\N
00169b	Alstom Transport	\N
00169c	Cisco Systems	\N
00169d	Cisco Systems	\N
00169e	TV One Ltd	\N
00169f	Vimtron Electronics Co., Ltd.	\N
0016a0	Auto-Maskin	\N
0016a1	3Leaf Networks	\N
0016a2	CentraLite Systems, Inc.	\N
0016a3	Ingeteam Transmission&Distribution, S.A.	\N
0016a4	Ezurio Ltd	\N
0016a5	Tandberg Storage ASA	\N
0016a6	Dovado FZ-LLC	\N
0016a7	AWETA G&P	\N
0016a8	CWT CO., LTD.	\N
0016a9	2EI	\N
0016aa	Kei Communication Technology Inc.	\N
0016ab	PBI-Dansensor A/S	\N
0016ac	Toho Technology Corp.	\N
0016ad	BT-Links Company Limited	\N
0016ae	INVENTEL	\N
0016af	Shenzhen Union Networks Equipment Co.,Ltd.	\N
0016b0	VK Corporation	\N
0016b1	KBS	\N
0016b2	DriveCam Inc	\N
0016b3	Photonicbridges (China) Co., Ltd.	\N
0016b4	PRIVATE	\N
0016b5	Motorola CHS	\N
0016b6	Cisco-Linksys	\N
0016b7	Seoul Commtech	\N
0016b8	Sony Ericsson Mobile Communications	\N
0016b9	ProCurve Networking	\N
0016ba	WEATHERNEWS INC.	\N
0016bb	Law-Chain Computer Technology Co Ltd	\N
0016bc	Nokia Danmark A/S	\N
0016bd	ATI Industrial Automation	\N
0016be	INFRANET, Inc.	\N
0016bf	PaloDEx Group Oy	\N
0016c0	Semtech Corporation	\N
0016c1	Eleksen Ltd	\N
0016c2	Avtec Systems Inc	\N
0016c3	BA Systems Inc	\N
0016c4	SiRF Technology, Inc.	\N
0016c5	Shenzhen Xing Feng Industry Co.,Ltd	\N
0016c6	North Atlantic Industries	\N
0016c7	Cisco Systems	\N
0016c8	Cisco Systems	\N
0016c9	NAT Seattle, Inc.	\N
0016ca	Nortel	\N
0016cb	Apple Computer	\N
0016cc	Xcute Mobile Corp.	\N
0016cd	HIJI HIGH-TECH CO., LTD.	\N
0016ce	Hon Hai Precision Ind. Co., Ltd.	\N
0016cf	Hon Hai Precision Ind. Co., Ltd.	\N
0016d0	ATech elektronika d.o.o.	\N
0016d1	ZAT a.s.	\N
0016d2	Caspian	\N
0016d3	Wistron Corporation	\N
0016d4	Compal Communications, Inc.	\N
0016d5	Synccom Co., Ltd	\N
0016d6	TDA Tech Pty Ltd	\N
0016d7	Sunways AG	\N
0016d8	Senea AB	\N
0016d9	NINGBO BIRD CO.,LTD.	\N
0016da	Futronic Technology Co. Ltd.	\N
0016db	Samsung Electronics Co., Ltd.	\N
0016dc	ARCHOS	\N
0016dd	Gigabeam Corporation	\N
0016de	FAST Inc	\N
0016df	Lundinova AB	\N
0016e0	3Com Ltd	\N
0016e1	SiliconStor, Inc.	\N
0016e2	American Fibertek, Inc.	\N
0016e3	ASKEY COMPUTER CORP.	\N
0016e4	VANGUARD SECURITY ENGINEERING CORP.	\N
0016e5	FORDLEY DEVELOPMENT LIMITED	\N
0016e6	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
0016e7	Dynamix Promotions Limited	\N
0016e8	Sigma Designs, Inc.	\N
0016e9	Tiba Medical Inc	\N
0016ea	Intel Corporation	\N
0016eb	Intel Corporation	\N
0016ec	Elitegroup Computer Systems Co., Ltd.	\N
0016ed	Digital Safety Technologies, Inc	\N
0016ee	RoyalDigital Inc.	\N
0016ef	Koko Fitness, Inc.	\N
0016f0	Dell	\N
0016f1	OmniSense, LLC	\N
0016f2	Dmobile System Co., Ltd.	\N
0016f3	CAST Information Co., Ltd	\N
0016f4	Eidicom Co., Ltd.	\N
0016f5	Dalian Golden Hualu Digital Technology Co.,Ltd	\N
0016f6	Video Products Group	\N
0016f7	L-3 Communications, Electrodynamics, Inc.	\N
0016f8	AVIQTECH TECHNOLOGY CO., LTD.	\N
0016f9	CETRTA POT, d.o.o., Kranj	\N
0016fa	ECI Telecom Ltd.	\N
0016fb	SHENZHEN MTC CO.,LTD.	\N
0016fc	TOHKEN CO.,LTD.	\N
0016fd	Jaty Electronics	\N
0016fe	Alps Electric Co., Ltd	\N
0016ff	Wamin Optocomm Mfg Corp	\N
001700	Motorola MDb	\N
001701	KDE, Inc.	\N
001702	Osung Midicom Co., Ltd	\N
001703	MOSDAN Internation Co.,Ltd	\N
001704	Shinco Electronics Group Co.,Ltd	\N
001705	Methode Electronics	\N
001706	Techfaith Wireless Communication Technology Limited.	\N
001707	InGrid, Inc	\N
001708	Hewlett Packard	\N
001709	Exalt Communications	\N
00170a	INEW DIGITAL COMPANY	\N
00170b	Contela, Inc.	\N
00170c	GeoSentric OYj	\N
00170d	Dust Networks Inc.	\N
00170e	Cisco Systems	\N
00170f	Cisco Systems	\N
001710	Casa Systems Inc.	\N
001711	GE Healthcare Bio-Sciences AB	\N
001712	ISCO International	\N
001713	Tiger NetCom	\N
001714	BR Controls Nederland bv	\N
001715	Qstik	\N
001716	Qno Technology Inc.	\N
001717	Leica Geosystems AG	\N
001718	Vansco Electronics Oy	\N
001719	AudioCodes USA, Inc	\N
00171a	Winegard Company	\N
00171b	Innovation Lab Corp.	\N
00171c	NT MicroSystems, Inc.	\N
00171d	DIGIT	\N
00171e	Theo Benning GmbH & Co. KG	\N
00171f	IMV Corporation	\N
001720	Image Sensing Systems, Inc.	\N
001721	FITRE S.p.A.	\N
001722	Hanazeder Electronic GmbH	\N
001723	Summit Data Communications	\N
001724	Studer Professional Audio GmbH	\N
001725	Liquid Computing	\N
001726	m2c Electronic Technology Ltd.	\N
001727	Thermo Ramsey Italia s.r.l.	\N
001728	Selex Communications	\N
001729	Ubicod Co.LTD	\N
00172a	Proware Technology Corp.	\N
00172b	Global Technologies Inc.	\N
00172c	TAEJIN INFOTECH	\N
00172d	Axcen Photonics Corporation	\N
00172e	FXC Inc.	\N
00172f	NeuLion Incorporated	\N
001730	Automation Electronics	\N
001731	ASUSTek COMPUTER INC.	\N
001732	Science-Technical Center "RISSA"	\N
001733	SFR	\N
001734	ADC Telecommunications	\N
001735	PRIVATE	\N
001736	iiTron Inc.	\N
001737	Industrie Dial Face S.p.A.	\N
001738	International Business Machines	\N
001739	Bright Headphone Electronics Company	\N
00173a	Reach Systems Inc.	\N
00173b	Arched Rock Corporation	\N
00173c	Extreme Engineering Solutions	\N
00173d	Neology	\N
00173e	LeucotronEquipamentos Ltda.	\N
00173f	Belkin Corporation	\N
001740	Technologies Labtronix	\N
001741	DEFIDEV	\N
001742	FUJITSU LIMITED	\N
001743	Deck Srl	\N
001744	Araneo Ltd.	\N
001745	INNOTZ CO., Ltd	\N
001746	Freedom9 Inc.	\N
001747	Trimble	\N
001748	Neokoros Brasil Ltda	\N
001749	HYUNDAE YONG-O-SA CO.,LTD	\N
00174a	SOCOMEC	\N
00174b	Nokia Danmark A/S	\N
00174c	Millipore	\N
00174d	DYNAMIC NETWORK FACTORY, INC.	\N
00174e	Parama-tech Co.,Ltd.	\N
00174f	iCatch Inc.	\N
001750	GSI Group, MicroE Systems	\N
001751	Online Corporation	\N
001752	DAGS, Inc	\N
001753	nFore Technology Inc.	\N
001754	Arkino Corporation., Ltd	\N
001755	GE Security	\N
001756	Vinci Labs Oy	\N
001757	RIX TECHNOLOGY LIMITED	\N
001758	ThruVision Ltd	\N
001759	Cisco Systems	\N
00175a	Cisco Systems	\N
00175b	ACS Solutions Switzerland Ltd.	\N
00175c	SHARP CORPORATION	\N
00175d	Dongseo system.	\N
00175e	Zed-3	\N
00175f	XENOLINK Communications Co., Ltd.	\N
001760	Naito Densei Machida MFG.CO.,LTD	\N
001761	ZKSoftware Inc.	\N
001762	Solar Technology, Inc.	\N
001763	Essentia S.p.A.	\N
001764	ATMedia GmbH	\N
001765	Nortel	\N
001766	Accense Technology, Inc.	\N
001767	Earforce AS	\N
001768	Zinwave Ltd	\N
001769	Cymphonix Corp	\N
00176a	Avago Technologies	\N
00176b	Kiyon, Inc.	\N
00176c	Pivot3, Inc.	\N
00176d	CORE CORPORATION	\N
00176e	DUCATI SISTEMI	\N
00176f	PAX Computer Technology(Shenzhen) Ltd.	\N
001770	Arti Industrial Electronics Ltd.	\N
001771	APD Communications Ltd	\N
001772	ASTRO Strobel Kommunikationssysteme GmbH	\N
001773	Laketune Technologies Co. Ltd	\N
001774	Elesta GmbH	\N
001775	TTE Germany GmbH	\N
001776	Meso Scale Diagnostics, LLC	\N
001777	Obsidian Research Corporation	\N
001778	Central Music Co.	\N
001779	QuickTel	\N
00177a	ASSA ABLOY AB	\N
00177b	Azalea Networks inc	\N
00177c	Smartlink Network Systems Limited	\N
00177d	IDT International Limited	\N
00177e	Meshcom Technologies Inc.	\N
00177f	Worldsmart Retech	\N
001780	Applera Holding B.V. Singapore Operations	\N
001781	Greystone Data System, Inc.	\N
001782	LoBenn Inc.	\N
001783	Texas Instruments	\N
001784	Motorola Mobile Devices	\N
001785	Sparr Electronics Ltd	\N
001786	wisembed	\N
001787	Brother, Brother & Sons ApS	\N
001788	Philips Lighting BV	\N
001789	Zenitron Corporation	\N
00178a	DARTS TECHNOLOGIES CORP.	\N
00178b	Teledyne Technologies Incorporated	\N
00178c	Independent Witness, Inc	\N
00178d	Checkpoint Systems, Inc.	\N
00178e	Gunnebo Cash Automation AB	\N
00178f	NINGBO YIDONG ELECTRONIC CO.,LTD.	\N
001790	HYUNDAI DIGITECH Co, Ltd.	\N
001791	LinTech GmbH	\N
001792	Falcom Wireless Comunications Gmbh	\N
001793	Tigi Corporation	\N
001794	Cisco Systems	\N
001795	Cisco Systems	\N
001796	Rittmeyer AG	\N
001797	Telsy Elettronica S.p.A.	\N
001798	Azonic Technology Co., LTD	\N
001799	SmarTire Systems Inc.	\N
00179a	D-Link Corporation	\N
00179b	Chant Sincere CO., LTD.	\N
00179c	DEPRAG SCHULZ GMBH u. CO.	\N
00179d	Kelman Limited	\N
00179e	Sirit Inc	\N
00179f	Apricorn	\N
0017a0	RoboTech srl	\N
0017a1	3soft inc.	\N
0017a2	Camrivox Ltd.	\N
0017a3	MIX s.r.l.	\N
0017a4	Hewlett Packard	\N
0017a5	TrendChip Technologies Corp.	\N
0017a6	YOSIN ELECTRONICS CO., LTD.	\N
0017a7	Mobile Computing Promotion Consortium	\N
0017a8	EDM Corporation	\N
0017a9	Sentivision	\N
0017aa	elab-experience inc.	\N
0017ab	Nintendo Co., Ltd.	\N
0017ac	O'Neil Product Development Inc.	\N
0017ad	AceNet Corporation	\N
0017ae	GAI-Tronics	\N
0017af	Enermet	\N
0017b0	Nokia Danmark A/S	\N
0017b1	ACIST Medical Systems, Inc.	\N
0017b2	SK Telesys	\N
0017b3	Aftek Infosys Limited	\N
0017b4	Remote Security Systems, LLC	\N
0017b5	Peerless Systems Corporation	\N
0017b6	Aquantia	\N
0017b7	Tonze Technology Co.	\N
0017b8	NOVATRON CO., LTD.	\N
0017b9	Gambro Lundia AB	\N
0017ba	SEDO CO., LTD.	\N
0017bb	Syrinx Industrial Electronics	\N
0017bc	Touchtunes Music Corporation	\N
0017bd	Tibetsystem	\N
0017be	Tratec Telecom B.V.	\N
0017bf	Coherent Research Limited	\N
0017c0	PureTech Systems, Inc.	\N
0017c1	CM Precision Technology LTD.	\N
0017c2	Pirelli Broadband Solutions	\N
0017c3	KTF Technologies Inc.	\N
0017c4	Quanta Microsystems, INC.	\N
0017c5	SonicWALL	\N
0017c6	Cross Match Technologies Inc	\N
0017c7	MARA Systems Consulting AB	\N
0017c8	Kyocera Mita Corporation	\N
0017c9	Samsung Electronics Co., Ltd.	\N
0017ca	Qisda Corporation	\N
0017cb	Juniper Networks	\N
0017cc	Alcatel-Lucent	\N
0017cd	CEC Wireless R&D Ltd.	\N
0017ce	MB International Telecom Labs srl	\N
0017cf	iMCA-GmbH	\N
0017d0	Opticom Communications, LLC	\N
0017d1	Nortel	\N
0017d2	THINLINX PTY LTD	\N
0017d3	Etymotic Research, Inc.	\N
0017d4	Monsoon Multimedia, Inc	\N
0017d5	Samsung Electronics Co., Ltd.	\N
0017d6	Bluechips Microhouse Co.,Ltd.	\N
0017d7	ION Geophysical Corporation Inc.	\N
0017d8	Magnum Semiconductor, Inc.	\N
0017d9	AAI Corporation	\N
0017da	Spans Logic	\N
0017db	CANKO TECHNOLOGIES INC.	\N
0017dc	DAEMYUNG ZERO1	\N
0017dd	Clipsal Australia	\N
0017de	Advantage Six Ltd	\N
0017df	Cisco Systems	\N
0017e0	Cisco Systems	\N
0017e1	DACOS Technologies Co., Ltd.	\N
0017e2	Motorola Mobile Devices	\N
0017e3	Texas Instruments	\N
0017e4	Texas Instruments	\N
0017e5	Texas Instruments	\N
0017e6	Texas Instruments	\N
0017e7	Texas Instruments	\N
0017e8	Texas Instruments	\N
0017e9	Texas Instruments	\N
0017ea	Texas Instruments	\N
0017eb	Texas Instruments	\N
0017ec	Texas Instruments	\N
0017ed	WooJooIT Ltd.	\N
0017ee	Motorola CHS	\N
0017ef	Blade Network Technologies, Inc.	\N
0017f0	SZCOM Broadband Network Technology Co.,Ltd	\N
0017f1	Renu Electronics Pvt Ltd	\N
0017f2	Apple Computer	\N
0017f3	Harris Corparation	\N
0017f4	ZERON ALLIANCE	\N
0017f5	LIG NEOPTEK	\N
0017f6	Pyramid Meriden Inc.	\N
0017f7	CEM Solutions Pvt Ltd	\N
0017f8	Motech Industries Inc.	\N
0017f9	Forcom Sp. z o.o.	\N
0017fa	Microsoft Corporation	\N
0017fb	FA	\N
0017fc	Suprema Inc.	\N
0017fd	Amulet Hotkey	\N
0017fe	TALOS SYSTEM INC.	\N
0017ff	PLAYLINE Co.,Ltd.	\N
001800	UNIGRAND LTD	\N
001801	Actiontec Electronics, Inc	\N
001802	Alpha Networks Inc.	\N
001803	ArcSoft Shanghai Co. LTD	\N
001804	E-TEK DIGITAL TECHNOLOGY LIMITED	\N
001805	Beijing InHand Networking Technology Co.,Ltd.	\N
001806	Hokkei Industries Co., Ltd.	\N
001807	Fanstel Corp.	\N
001808	SightLogix, Inc.	\N
001809	CRESYN	\N
00180a	Meraki, Inc.	\N
00180b	Brilliant Telecommunications	\N
00180c	Optelian Access Networks	\N
00180d	Terabytes Server Storage Tech Corp	\N
00180e	Avega Systems	\N
00180f	Nokia Danmark A/S	\N
001810	IPTrade S.A.	\N
001811	Neuros Technology International, LLC.	\N
001812	Beijing Xinwei Telecom Technology Co., Ltd.	\N
001813	Sony Ericsson Mobile Communications	\N
001814	Mitutoyo Corporation	\N
001815	GZ Technologies, Inc.	\N
001816	Ubixon Co., Ltd.	\N
001817	D. E. Shaw Research, LLC	\N
001818	Cisco Systems	\N
001819	Cisco Systems	\N
00181a	AVerMedia Information Inc.	\N
00181b	TaiJin Metal Co., Ltd.	\N
00181c	Exterity Limited	\N
00181d	ASIA ELECTRONICS CO.,LTD	\N
00181e	GDX Technologies Ltd.	\N
00181f	Palmmicro Communications	\N
001820	w5networks	\N
001821	SINDORICOH	\N
001822	CEC TELECOM CO.,LTD.	\N
001823	Delta Electronics, Inc.	\N
001824	Kimaldi Electronics, S.L.	\N
001825	Wavion LTD	\N
001826	Cale Access AB	\N
001827	NEC PHILIPS UNIFIED SOLUTIONS NEDERLAND BV	\N
001828	e2v technologies (UK) ltd.	\N
001829	Gatsometer	\N
00182a	Taiwan Video & Monitor	\N
00182b	Softier	\N
00182c	Ascend Networks, Inc.	\N
00182d	Artec Group OÜ	\N
00182e	XStreamHD, LLC	\N
00182f	Texas Instruments	\N
001830	Texas Instruments	\N
001831	Texas Instruments	\N
001832	Texas Instruments	\N
001833	Texas Instruments	\N
001834	Texas Instruments	\N
001835	Thoratec / ITC	\N
001836	Reliance Electric Limited	\N
001837	Universal ABIT Co., Ltd.	\N
001838	PanAccess Communications,Inc.	\N
001839	Cisco-Linksys LLC	\N
00183a	Westell Technologies	\N
00183b	CENITS Co., Ltd.	\N
00183c	Encore Software Limited	\N
00183d	Vertex Link Corporation	\N
00183e	Digilent, Inc	\N
00183f	2Wire, Inc	\N
001840	3 Phoenix, Inc.	\N
001841	High Tech Computer Corp	\N
001842	Nokia Danmark A/S	\N
001843	Dawevision Ltd	\N
001844	Heads Up Technologies, Inc.	\N
001845	NPL Pulsar Ltd.	\N
001846	Crypto S.A.	\N
001847	AceNet Technology Inc.	\N
001848	Vecima Networks Inc.	\N
001849	Pigeon Point Systems	\N
00184a	Catcher, Inc.	\N
00184b	Las Vegas Gaming, Inc.	\N
00184c	Bogen Communications	\N
00184d	Netgear Inc.	\N
00184e	Lianhe Technologies, Inc.	\N
00184f	8 Ways Technology Corp.	\N
001850	Secfone Kft	\N
001851	SWsoft	\N
001852	StorLink Semiconductors, Inc.	\N
001853	Atera Networks LTD.	\N
001854	Argard Co., Ltd	\N
001855	Aeromaritime Systembau GmbH	\N
001856	EyeFi, Inc	\N
001857	Unilever R&D	\N
001858	TagMaster AB	\N
001859	Strawberry Linux Co.,Ltd.	\N
00185a	uControl, Inc.	\N
00185b	Network Chemistry, Inc	\N
00185c	EDS Lab Pte Ltd	\N
00185d	TAIGUEN TECHNOLOGY (SHEN-ZHEN) CO., LTD.	\N
00185e	Nexterm Inc.	\N
00185f	TAC Inc.	\N
001860	SIM Technology Group Shanghai Simcom Ltd.,	\N
001861	Ooma, Inc.	\N
001862	Seagate Technology	\N
001863	Veritech Electronics Limited	\N
001864	Cybectec Inc.	\N
001865	Siemens Healthcare Diagnostics Manufacturing Ltd	\N
001866	Leutron Vision	\N
001867	Evolution Robotics Retail	\N
001868	Scientific Atlanta, A Cisco Company	\N
001869	KINGJIM	\N
00186a	Global Link Digital Technology Co,.LTD	\N
00186b	Sambu Communics CO., LTD.	\N
00186c	Neonode AB	\N
00186d	Zhenjiang Sapphire Electronic Industry CO.	\N
00186e	3Com Ltd	\N
00186f	Setha Industria Eletronica LTDA	\N
001870	E28 Shanghai Limited	\N
001871	Hewlett Packard	\N
001872	Expertise Engineering	\N
001873	Cisco Systems	\N
001874	Cisco Systems	\N
001875	AnaCise Testnology Pte Ltd	\N
001876	WowWee Ltd.	\N
001877	Amplex A/S	\N
001878	Mackware GmbH	\N
001879	dSys	\N
00187a	Wiremold	\N
00187b	4NSYS Co. Ltd.	\N
00187c	INTERCROSS, LLC	\N
00187d	Armorlink shanghai Co. Ltd	\N
00187e	RGB Spectrum	\N
00187f	ZODIANET	\N
001880	Maxim Integrated Circuits	\N
001881	Buyang Electronics Industrial Co., Ltd	\N
001882	Huawei Technologies Co., Ltd.	\N
001883	FORMOSA21 INC.	\N
001884	FON	\N
001885	Avigilon Corporation	\N
001886	EL-TECH, INC.	\N
001887	Metasystem SpA	\N
001888	GOTIVE a.s.	\N
001889	WinNet Solutions Limited	\N
00188a	Infinova LLC	\N
00188b	Dell	\N
00188c	Mobile Action Technology Inc.	\N
00188d	Nokia Danmark A/S	\N
00188e	Ekahau, Inc.	\N
00188f	Montgomery Technology, Inc.	\N
001890	RadioCOM, s.r.o.	\N
001891	Zhongshan General K-mate Electronics Co., Ltd	\N
001892	ads-tec GmbH	\N
001893	SHENZHEN PHOTON BROADBAND TECHNOLOGY CO.,LTD	\N
001894	zimocom	\N
001895	Hansun Technologies Inc.	\N
001896	Great Well Electronic LTD	\N
001897	JESS-LINK PRODUCTS Co., LTD	\N
001898	KINGSTATE ELECTRONICS CORPORATION	\N
001899	ShenZhen jieshun Science&Technology Industry CO,LTD.	\N
00189a	HANA Micron Inc.	\N
00189b	Thomson Inc.	\N
00189c	Weldex Corporation	\N
00189d	Navcast Inc.	\N
00189e	OMNIKEY GmbH.	\N
00189f	Lenntek Corporation	\N
0018a0	Cierma Ascenseurs	\N
0018a1	Tiqit Computers, Inc.	\N
0018a2	XIP Technology AB	\N
0018a3	ZIPPY TECHNOLOGY CORP.	\N
0018a4	Motorola Mobile Devices	\N
0018a5	ADigit Technologies Corp.	\N
0018a6	Persistent Systems, LLC	\N
0018a7	Yoggie Security Systems LTD.	\N
0018a8	AnNeal Technology Inc.	\N
0018a9	Ethernet Direct Corporation	\N
0018aa	Protec Fire Detection plc	\N
0018ab	BEIJING LHWT MICROELECTRONICS INC.	\N
0018ac	Shanghai Jiao Da HISYS Technology Co. Ltd.	\N
0018ad	NIDEC SANKYO CORPORATION	\N
0018ae	TVT CO.,LTD	\N
0018af	Samsung Electronics Co., Ltd.	\N
0018b0	Nortel	\N
0018b1	Blade Network Technologies	\N
0018b2	ADEUNIS RF	\N
0018b3	TEC WizHome Co., Ltd.	\N
0018b4	Dawon Media Inc.	\N
0018b5	Magna Carta	\N
0018b6	S3C, Inc.	\N
0018b7	D3 LED, LLC	\N
0018b8	New Voice International AG	\N
0018b9	Cisco Systems	\N
0018ba	Cisco Systems	\N
0018bb	Eliwell Controls srl	\N
0018bc	ZAO NVP Bolid	\N
0018bd	SHENZHEN DVBWORLD TECHNOLOGY CO., LTD.	\N
0018be	ANSA Corporation	\N
0018bf	Essence Technology Solution, Inc.	\N
0018c0	Motorola CHS	\N
0018c1	Almitec Informática e Comércio Ltda.	\N
0018c2	Firetide, Inc	\N
0018c3	C&S Microwave	\N
0018c4	Raba Technologies LLC	\N
0018c5	Nokia Danmark A/S	\N
0018c6	OPW Fuel Management Systems	\N
0018c7	Real Time Automation	\N
0018c8	ISONAS Inc.	\N
0018c9	EOps Technology Limited	\N
0018ca	Viprinet GmbH	\N
0018cb	Tecobest Technology Limited	\N
0018cc	AXIOHM SAS	\N
0018cd	Erae Electronics Industry Co., Ltd	\N
0018ce	Dreamtech Co., Ltd	\N
0018cf	Baldor Electric Company	\N
0018d0	AtRoad,  A Trimble Company	\N
0018d1	Siemens Home & Office Comm. Devices	\N
0018d2	High-Gain Antennas LLC	\N
0018d3	TEAMCAST	\N
0018d4	Unified Display Interface SIG	\N
0018d5	REIGNCOM	\N
0018d6	Swirlnet A/S	\N
0018d7	Javad Navigation Systems Inc.	\N
0018d8	ARCH METER Corporation	\N
0018d9	Santosha Internatonal, Inc	\N
0018da	AMBER wireless GmbH	\N
0018db	EPL Technology Ltd	\N
0018dc	Prostar Co., Ltd.	\N
0018dd	Silicondust Engineering Ltd	\N
0018de	Intel Corporation	\N
0018df	The Morey Corporation	\N
0018e0	ANAVEO	\N
0018e1	Verkerk Service Systemen	\N
0018e2	Topdata Sistemas de Automacao Ltda	\N
0018e3	Visualgate Systems, Inc.	\N
0018e4	YIGUANG	\N
0018e5	Adhoco AG	\N
0018e6	Computer Hardware Design SIA	\N
0018e7	Cameo Communications, INC.	\N
0018e8	Hacetron Corporation	\N
0018e9	Numata Corporation	\N
0018ea	Alltec GmbH	\N
0018eb	BroVis Wireless Networks	\N
0018ec	Welding Technology Corporation	\N
0018ed	Accutech Ultrasystems Co., Ltd.	\N
0018ee	Videology Imaging Solutions, Inc.	\N
0018ef	Escape Communications, Inc.	\N
0018f0	JOYTOTO Co., Ltd.	\N
0018f1	Chunichi Denshi Co.,LTD.	\N
0018f2	Beijing Tianyu Communication Equipment Co., Ltd	\N
0018f3	ASUSTek COMPUTER INC.	\N
0018f4	EO TECHNICS Co., Ltd.	\N
0018f5	Shenzhen Streaming Video Technology Company Limited	\N
0018f6	Thomson Telecom Belgium	\N
0018f7	Kameleon Technologies	\N
0018f8	Cisco-Linksys LLC	\N
0018f9	VVOND, Inc.	\N
0018fa	Yushin Precision Equipment Co.,Ltd.	\N
0018fb	Compro Technology	\N
0018fc	Altec Electronic AG	\N
0018fd	Optimal Technologies International Inc.	\N
0018fe	Hewlett Packard	\N
0018ff	PowerQuattro Co.	\N
001900	Intelliverese - DBA Voicecom	\N
001901	F1MEDIA	\N
001902	Cambridge Consultants Ltd	\N
001903	Bigfoot Networks Inc	\N
001904	WB Electronics Sp. z o.o.	\N
001905	SCHRACK Seconet AG	\N
001906	Cisco Systems	\N
001907	Cisco Systems	\N
001908	Duaxes Corporation	\N
001909	Devi A/S	\N
00190a	HASWARE INC.	\N
00190b	Southern Vision Systems, Inc.	\N
00190c	Encore Electronics, Inc.	\N
00190d	IEEE 1394c	\N
00190e	Atech Technology Co., Ltd.	\N
00190f	Advansus Corp.	\N
001910	Knick Elektronische Messgeraete GmbH & Co. KG	\N
001911	Just In Mobile Information Technologies (Shanghai) Co., Ltd.	\N
001912	Welcat Inc	\N
001913	Chuang-Yi Network Equipment Co.Ltd.	\N
001914	Winix Co., Ltd	\N
001915	TECOM Co., Ltd.	\N
001916	PayTec AG	\N
001917	Posiflex Inc.	\N
001918	Interactive Wear AG	\N
001919	ASTEL Inc.	\N
00191a	IRLINK	\N
00191b	Sputnik Engineering AG	\N
00191c	Sensicast Systems	\N
00191d	Nintendo Co.,Ltd.	\N
00191e	Beyondwiz Co., Ltd.	\N
00191f	Microlink communications Inc.	\N
001920	KUME electric Co.,Ltd.	\N
001921	Elitegroup Computer System Co.	\N
001922	CM Comandos Lineares	\N
001923	Phonex Korea Co., LTD.	\N
001924	LBNL  Engineering	\N
001925	Intelicis Corporation	\N
001926	BitsGen Co., Ltd.	\N
001927	ImCoSys Ltd	\N
001928	Siemens AG, Transportation Systems	\N
001929	2M2B Montadora de Maquinas Bahia Brasil LTDA	\N
00192a	Antiope Associates	\N
00192b	Aclara RF Systems Inc.	\N
00192c	Motorola Mobile Devices	\N
00192d	Nokia Corporation	\N
00192e	Spectral Instruments, Inc.	\N
00192f	Cisco Systems	\N
001930	Cisco Systems	\N
001931	Balluff GmbH	\N
001932	Gude Analog- und Digialsysteme GmbH	\N
001933	Strix Systems, Inc.	\N
001934	TRENDON TOUCH TECHNOLOGY CORP.	\N
001935	Duerr Dental GmbH & Co. KG	\N
001936	STERLITE OPTICAL TECHNOLOGIES LIMITED	\N
001937	CommerceGuard AB	\N
001938	UMB Communications Co., Ltd.	\N
001939	Gigamips	\N
00193a	OESOLUTIONS	\N
00193b	Wilibox Deliberant Group LLC	\N
00193c	HighPoint Technologies Incorporated	\N
00193d	GMC Guardian Mobility Corp.	\N
00193e	PIRELLI BROADBAND SOLUTIONS	\N
00193f	RDI technology(Shenzhen) Co.,LTD	\N
001940	Rackable Systems	\N
001941	Pitney Bowes, Inc	\N
001942	ON SOFTWARE INTERNATIONAL LIMITED	\N
001943	Belden	\N
001944	Fossil Partners, L.P.	\N
001945	Ten-Tec Inc.	\N
001946	Cianet Industria e Comercio S/A	\N
001947	Scientific Atlanta, A Cisco Company	\N
001948	AireSpider Networks	\N
001949	TENTEL  COMTECH CO., LTD.	\N
00194a	TESTO AG	\N
00194b	SAGEM COMMUNICATION	\N
00194c	Fujian Stelcom information & Technology CO.,Ltd	\N
00194d	Avago Technologies Sdn Bhd	\N
00194e	Ultra Electronics - TCS (Tactical Communication Systems)	\N
00194f	Nokia Danmark A/S	\N
001950	Harman Multimedia	\N
001951	NETCONS, s.r.o.	\N
001952	ACOGITO Co., Ltd	\N
001953	Chainleader Communications Corp.	\N
001954	Leaf Corporation.	\N
001955	Cisco Systems	\N
001956	Cisco Systems	\N
001957	Saafnet Canada Inc.	\N
001958	Bluetooth SIG, Inc.	\N
001959	Staccato Communications Inc.	\N
00195a	Jenaer Antriebstechnik GmbH	\N
00195b	D-Link Corporation	\N
00195c	Innotech Corporation	\N
00195d	ShenZhen XinHuaTong Opto Electronics Co.,Ltd	\N
00195e	Motorola CHS	\N
00195f	Valemount Networks Corporation	\N
001960	DoCoMo Systems, Inc.	\N
001961	Blaupunkt GmbH	\N
001962	Commerciant, LP	\N
001963	Sony Ericsson Mobile Communications AB	\N
001964	Doorking Inc.	\N
001965	YuHua TelTech (ShangHai) Co., Ltd.	\N
001966	Asiarock Technology Limited	\N
001967	TELDAT Sp.J.	\N
001968	Digital Video Networks(Shanghai) CO. LTD.	\N
001969	Nortel	\N
00196a	MikroM GmbH	\N
00196b	Danpex Corporation	\N
00196c	ETROVISION TECHNOLOGY	\N
00196d	Raybit Systems Korea, Inc	\N
00196e	Metacom (Pty) Ltd.	\N
00196f	SensoPart GmbH	\N
001970	Z-Com, Inc.	\N
001971	Guangzhou Unicomp Technology Co.,Ltd	\N
001972	Plexus (Xiamen) Co.,ltd	\N
001973	Zeugma Systems	\N
001974	AboCom Systems, Inc.	\N
001975	Beijing Huisen networks technology Inc	\N
001976	Xipher Technologies, LLC	\N
001977	Aerohive Networks, Inc.	\N
001978	Datum Systems, Inc.	\N
001979	Nokia Danmark A/S	\N
00197a	MAZeT GmbH	\N
00197b	Picotest Corp.	\N
00197c	Riedel Communications GmbH	\N
00197d	Hon Hai Precision Ind. Co., Ltd	\N
00197e	Hon Hai Precision Ind. Co., Ltd	\N
00197f	PLANTRONICS, INC.	\N
001980	Gridpoint Systems	\N
001981	Vivox Inc	\N
001982	SmarDTV	\N
001983	CCT R&D Limited	\N
001984	ESTIC Corporation	\N
001985	IT Watchdogs, Inc	\N
001986	Cheng Hongjian	\N
001987	Panasonic Mobile Communications Co., Ltd.	\N
001988	Wi2Wi, Inc	\N
001989	Sonitrol Corporation	\N
00198a	Northrop Grumman Systems Corp.	\N
00198b	Novera Optics Korea, Inc.	\N
00198c	iXSea	\N
00198d	Ocean Optics, Inc.	\N
00198e	Oticon A/S	\N
00198f	Alcatel Bell N.V.	\N
001990	ELM DATA Co., Ltd.	\N
001991	avinfo	\N
001992	Bluesocket, Inc	\N
001993	Changshu Switchgear MFG. Co.,Ltd. (Former Changshu Switchgea	\N
001994	Jorjin Technologies Inc.	\N
001995	Jurong Hi-Tech (Suzhou)Co.ltd	\N
001996	TurboChef Technologies Inc.	\N
001997	Soft Device Sdn Bhd	\N
001998	SATO CORPORATION	\N
001999	Fujitsu Technology Solutions	\N
00199a	EDO-EVI	\N
00199b	Diversified Technical Systems, Inc.	\N
00199c	CTRING	\N
00199d	VIZIO, Inc.	\N
00199e	SHOWADENSHI ELECTRONICS,INC.	\N
00199f	DKT A/S	\N
0019a0	NIHON DATA SYSTENS, INC.	\N
0019a1	LG INFORMATION & COMM.	\N
0019a2	ORDYN TECHNOLOGIES	\N
0019a3	asteel electronique atlantique	\N
0019a4	Austar Technology (hang zhou) Co.,Ltd	\N
0019a5	RadarFind Corporation	\N
0019a6	Motorola CHS	\N
0019a7	ITU-T	\N
0019a8	WiQuest Communications	\N
0019a9	Cisco Systems	\N
0019aa	Cisco Systems	\N
0019ab	Raycom CO ., LTD	\N
0019ac	GSP SYSTEMS Inc.	\N
0019ad	BOBST SA	\N
0019ae	Hopling Technologies b.v.	\N
0019af	Rigol Technologies, Inc.	\N
0019b0	HanYang System	\N
0019b1	Arrow7 Corporation	\N
0019b2	XYnetsoft Co.,Ltd	\N
0019b3	Stanford Research Systems	\N
0019b4	VideoCast Ltd.	\N
0019b5	Famar Fueguina S.A.	\N
0019b6	Euro Emme s.r.l.	\N
0019b7	Nokia Danmark A/S	\N
0019b8	Boundary Devices	\N
0019b9	Dell Inc.	\N
0019ba	Paradox Security Systems Ltd	\N
0019bb	Hewlett Packard	\N
0019bc	ELECTRO CHANCE SRL	\N
0019bd	New Media Life	\N
0019be	Altai Technologies Limited	\N
0019bf	Citiway technology Co.,ltd	\N
0019c0	Motorola Mobile Devices	\N
0019c1	Alps Electric Co., Ltd	\N
0019c2	Equustek Solutions, Inc.	\N
0019c3	Qualitrol	\N
0019c4	Infocrypt Inc.	\N
0019c5	SONY Computer Entertainment inc,	\N
0019c6	ZTE Corporation	\N
0019c7	Cambridge Industries(Group) Co.,Ltd.	\N
0019c8	AnyDATA Corporation	\N
0019c9	S&C ELECTRIC COMPANY	\N
0019ca	Broadata Communications, Inc	\N
0019cb	ZyXEL Communications Corporation	\N
0019cc	RCG (HK) Ltd	\N
0019cd	Chengdu ethercom information technology Ltd.	\N
0019ce	Progressive Gaming International	\N
0019cf	SALICRU, S.A.	\N
0019d0	Cathexis	\N
0019d1	Intel Corporation	\N
0019d2	Intel Corporation	\N
0019d3	TRAK Microwave	\N
0019d4	ICX Technologies	\N
0019d5	IP Innovations, Inc.	\N
0019d6	LS Cable Ltd.	\N
0019d7	FORTUNETEK CO., LTD	\N
0019d8	MAXFOR	\N
0019d9	Zeutschel GmbH	\N
0019da	Welltrans O&E Technology Co. , Ltd.	\N
0019db	MICRO-STAR INTERNATIONAL CO., LTD.	\N
0019dc	ENENSYS Technologies	\N
0019dd	FEI-Zyfer, Inc.	\N
0019de	MOBITEK	\N
0019df	Thomson Inc.	\N
0019e0	TP-LINK Technologies Co., Ltd.	\N
0019e1	Nortel	\N
0019e2	Juniper Networks	\N
0019e3	Apple Computer Inc.	\N
0019e4	2Wire, Inc	\N
0019e5	Lynx Studio Technology, Inc.	\N
0019e6	TOYO MEDIC CO.,LTD.	\N
0019e7	Cisco Systems	\N
0019e8	Cisco Systems	\N
0019e9	S-Information Technolgy, Co., Ltd.	\N
0019ea	TeraMage Technologies Co., Ltd.	\N
0019eb	Pyronix Ltd	\N
0019ec	Sagamore Systems, Inc.	\N
0019ed	Axesstel Inc.	\N
0019ee	CARLO GAVAZZI CONTROLS SPA-Controls Division	\N
0019ef	SHENZHEN LINNKING ELECTRONICS CO.,LTD	\N
0019f0	UNIONMAN TECHNOLOGY CO.,LTD	\N
0019f1	Star Communication Network Technology Co.,Ltd	\N
0019f2	Teradyne K.K.	\N
0019f3	Telematrix, Inc	\N
0019f4	Convergens Oy Ltd	\N
0019f5	Imagination Technologies Ltd	\N
0019f6	Acconet (PTE) Ltd	\N
0019f7	Onset Computer Corporation	\N
0019f8	Embedded Systems Design, Inc.	\N
0019f9	TDK-Lambda	\N
0019fa	Cable Vision Electronics CO., LTD.	\N
0019fb	AMSTRAD PLC	\N
0019fc	PT. Ufoakses Sukses Luarbiasa	\N
0019fd	Nintendo Co., Ltd.	\N
0019fe	SHENZHEN SEECOMM TECHNOLOGY CO.,LTD.	\N
0019ff	Finnzymes	\N
001a00	MATRIX INC.	\N
001a01	Smiths Medical	\N
001a02	SECURE CARE PRODUCTS, INC	\N
001a03	Angel Electronics Co., Ltd.	\N
001a04	Interay Solutions BV	\N
001a05	OPTIBASE LTD	\N
001a06	OpVista, Inc.	\N
001a07	Arecont Vision	\N
001a08	Dalman Technical Services	\N
001a09	Wayfarer Transit Systems Ltd	\N
001a0a	Adaptive Micro-Ware Inc.	\N
001a0b	BONA TECHNOLOGY INC.	\N
001a0c	Swe-Dish Satellite Systems AB	\N
001a0d	HandHeld entertainment, Inc.	\N
001a0e	Cheng Uei Precision Industry Co.,Ltd	\N
001a0f	Sistemas Avanzados de Control, S.A.	\N
001a10	LUCENT TRANS ELECTRONICS CO.,LTD	\N
001a11	Google Inc.	\N
001a12	Essilor	\N
001a13	Wanlida Group Co., LTD	\N
001a14	Xin Hua Control Engineering Co.,Ltd.	\N
001a15	gemalto e-Payment	\N
001a16	Nokia Danmark A/S	\N
001a17	Teak Technologies, Inc.	\N
001a18	Advanced Simulation Technology inc.	\N
001a19	Computer Engineering Limited	\N
001a1a	Gentex Corporation/Electro-Acoustic Products	\N
001a1b	Motorola Mobile Devices	\N
001a1c	GT&T Engineering Pte Ltd	\N
001a1d	PChome Online Inc.	\N
001a1e	Aruba Networks	\N
001a1f	Coastal Environmental Systems	\N
001a20	CMOTECH Co. Ltd.	\N
001a21	Indac B.V.	\N
001a22	eq-3 GmbH	\N
001a23	Ice Qube, Inc	\N
001a24	Galaxy Telecom Technologies Ltd	\N
001a25	DELTA DORE	\N
001a26	Deltanode Solutions AB	\N
001a27	Ubistar	\N
001a28	ASWT Co., LTD. Taiwan Branch H.K.	\N
001a29	Techsonic Industries d/b/a Humminbird	\N
001a2a	Arcadyan Technology Corporation	\N
001a2b	Ayecom Technology Co., Ltd.	\N
001a2c	SATEC Co.,LTD	\N
001a2d	The Navvo Group	\N
001a2e	Ziova Coporation	\N
001a2f	Cisco Systems	\N
001a30	Cisco Systems	\N
001a31	SCAN COIN Industries AB	\N
001a32	ACTIVA MULTIMEDIA	\N
001a33	ASI Communications, Inc.	\N
001a34	Konka Group Co., Ltd.	\N
001a35	BARTEC GmbH	\N
001a36	Aipermon GmbH & Co. KG	\N
001a37	Lear Corporation	\N
001a38	Sanmina-SCI	\N
001a39	Merten GmbH&CoKG	\N
001a3a	Dongahelecomm	\N
001a3b	Doah Elecom Inc.	\N
001a3c	Technowave Ltd.	\N
001a3d	Ajin Vision Co.,Ltd	\N
001a3e	Faster Technology LLC	\N
001a3f	intelbras	\N
001a40	A-FOUR TECH CO., LTD.	\N
001a41	INOCOVA Co.,Ltd	\N
001a42	Techcity Technology co., Ltd.	\N
001a43	Logical Link Communications	\N
001a44	JWTrading Co., Ltd	\N
001a45	GN Netcom as	\N
001a46	Digital Multimedia Technology Co., Ltd	\N
001a47	Agami Systems, Inc.	\N
001a48	Takacom Corporation	\N
001a49	Micro Vision Co.,LTD	\N
001a4a	Qumranet Inc.	\N
001a4b	Hewlett Packard	\N
001a4c	Crossbow Technology, Inc	\N
001a4d	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
001a4e	NTI AG / LinMot	\N
001a4f	AVM GmbH	\N
001a50	PheeNet Technology Corp.	\N
001a51	Alfred Mann Foundation	\N
001a52	Meshlinx Wireless Inc.	\N
001a53	Zylaya	\N
001a54	Hip Shing Electronics Ltd.	\N
001a55	ACA-Digital Corporation	\N
001a56	ViewTel Co,. Ltd.	\N
001a57	Matrix Design Group, LLC	\N
001a58	CCV Deutschland GmbH - Celectronic eHealth Div.	\N
001a59	Ircona	\N
001a5a	Korea Electric Power Data Network  (KDN) Co., Ltd	\N
001a5b	NetCare Service Co., Ltd.	\N
001a5c	Euchner GmbH+Co. KG	\N
001a5d	Mobinnova Corp.	\N
001a5e	Thincom Technology Co.,Ltd	\N
001a5f	KitWorks.fi Ltd.	\N
001a60	Wave Electronics Co.,Ltd.	\N
001a61	PacStar Corp.	\N
001a62	Data Robotics, Incorporated	\N
001a63	Elster Solutions, LLC,	\N
001a64	IBM Corp.	\N
001a65	Seluxit	\N
001a66	Motorola CHS	\N
001a67	Infinite QL Sdn Bhd	\N
001a68	Weltec Enterprise Co., Ltd.	\N
001a69	Wuhan Yangtze Optical Technology CO.,Ltd.	\N
001a6a	Tranzas, Inc.	\N
001a6b	USI	\N
001a6c	Cisco Systems	\N
001a6d	Cisco Systems	\N
001a6e	Impro Technologies	\N
001a6f	MI.TEL s.r.l.	\N
001a70	Cisco-Linksys, LLC	\N
001a71	Diostech Co., Ltd.	\N
001a72	Mosart Semiconductor Corp.	\N
001a73	Gemtek Technology Co., Ltd.	\N
001a74	Procare International Co	\N
001a75	Sony Ericsson Mobile Communications	\N
001a76	SDT information Technology Co.,LTD.	\N
001a77	Motorola Mobile Devices	\N
001a78	ubtos	\N
001a79	TELECOMUNICATION TECHNOLOGIES LTD.	\N
001a7a	Lismore Instruments Limited	\N
001a7b	Teleco, Inc.	\N
001a7c	Hirschmann Automation and Control B.V.	\N
001a7d	cyber-blue(HK)Ltd	\N
001a7e	LN Srithai Comm Ltd.	\N
001a7f	GCI Science&Technology Co.,Ltd.	\N
001a80	Sony Corporation	\N
001a81	Zelax	\N
001a82	PROBA Building Automation Co.,LTD	\N
001a83	Pegasus Technologies Inc.	\N
001a84	V One Multimedia Pte Ltd	\N
001a85	NV Michel Van de Wiele	\N
001a86	AdvancedIO Systems Inc	\N
001a87	Canhold International Limited	\N
001a88	Venergy,Co,Ltd	\N
001a89	Nokia Danmark A/S	\N
001a8a	Samsung Electronics Co., Ltd.	\N
001a8b	CHUNIL ELECTRIC IND., CO.	\N
001a8c	Astaro AG	\N
001a8d	AVECS Bergen GmbH	\N
001a8e	3Way Networks Ltd	\N
001a8f	Nortel	\N
001a90	Trópico Sistemas e Telecomunicações da Amazônia LTDA.	\N
001a91	FusionDynamic Ltd.	\N
001a92	ASUSTek COMPUTER INC.	\N
001a93	ERCO Leuchten GmbH	\N
001a94	Votronic GmbH	\N
001a95	Hisense Mobile Communications Technoligy Co.,Ltd.	\N
001a96	ECLER S.A.	\N
001a97	fitivision technology Inc.	\N
001a98	Asotel Communication Limited Taiwan Branch	\N
001a99	Smarty (HZ) Information Electronics Co., Ltd	\N
001a9a	Skyworth Digital technology(shenzhen)co.ltd.	\N
001a9b	ADEC & Parter AG	\N
001a9c	RightHand Technologies, Inc.	\N
001a9d	Skipper Wireless, Inc.	\N
001a9e	ICON Digital International Limited	\N
001a9f	A-Link Europe Ltd	\N
001aa0	Dell Inc	\N
001aa1	Cisco Systems	\N
001aa2	Cisco Systems	\N
001aa3	DELORME	\N
001aa4	Future University-Hakodate	\N
001aa5	BRN Phoenix	\N
001aa6	Telefunken Radio Communication Systems GmbH &CO.KG	\N
001aa7	Torian Wireless	\N
001aa8	Mamiya Digital Imaging Co., Ltd.	\N
001aa9	FUJIAN STAR-NET COMMUNICATION CO.,LTD	\N
001aaa	Analogic Corp.	\N
001aab	eWings s.r.l.	\N
001aac	Corelatus AB	\N
001aad	Motorola CHS	\N
001aae	Savant Systems LLC	\N
001aaf	BLUSENS TECHNOLOGY	\N
001ab0	Signal Networks Pvt. Ltd.,	\N
001ab1	Asia Pacific Satellite Industries Co., Ltd.	\N
001ab2	Cyber Solutions Inc.	\N
001ab3	VISIONITE INC.	\N
001ab4	FFEI Ltd.	\N
001ab5	Home Network System	\N
001ab6	Texas Instruments	\N
001ab7	Ethos Networks LTD.	\N
001ab8	Anseri Corporation	\N
001ab9	PMC	\N
001aba	Caton Overseas Limited	\N
001abb	Fontal Technology Incorporation	\N
001abc	U4EA Technologies Ltd	\N
001abd	Impatica Inc.	\N
001abe	COMPUTER HI-TECH INC.	\N
001abf	TRUMPF Laser Marking Systems AG	\N
001ac0	JOYBIEN TECHNOLOGIES CO., LTD.	\N
001ac1	3Com Ltd	\N
001ac2	YEC Co.,Ltd.	\N
001ac3	Scientific-Atlanta, Inc	\N
001ac4	2Wire, Inc	\N
001ac5	BreakingPoint Systems, Inc.	\N
001ac6	Micro Control Designs	\N
001ac7	UNIPOINT	\N
001ac8	ISL (Instrumentation Scientifique de Laboratoire)	\N
001ac9	SUZUKEN CO.,LTD	\N
001aca	Tilera Corporation	\N
001acb	Autocom Products Ltd	\N
001acc	Celestial Semiconductor, Ltd	\N
001acd	Tidel Engineering LP	\N
001ace	YUPITERU CORPORATION	\N
001acf	C.T. ELETTRONICA	\N
001ad0	Albis Technologies AG	\N
001ad1	FARGO CO., LTD.	\N
001ad2	Eletronica Nitron Ltda	\N
001ad3	Vamp Ltd.	\N
001ad4	iPOX Technology Co., Ltd.	\N
001ad5	KMC CHAIN INDUSTRIAL CO., LTD.	\N
001ad6	JIAGNSU AETNA ELECTRIC CO.,LTD	\N
001ad7	Christie Digital Systems, Inc.	\N
001ad8	AlsterAero GmbH	\N
001ad9	International Broadband Electric Communications, Inc.	\N
001ada	Biz-2-Me Inc.	\N
001adb	Motorola Mobile Devices	\N
001adc	Nokia Danmark A/S	\N
001add	PePWave Ltd	\N
001ade	Motorola CHS	\N
001adf	Interactivetv Pty Limited	\N
001ae0	Mythology Tech Express Inc.	\N
001ae1	EDGE ACCESS INC	\N
001ae2	Cisco Systems	\N
001ae3	Cisco Systems	\N
001ae4	Medicis Technologies Corporation	\N
001ae5	Mvox Technologies Inc.	\N
001ae6	Atlanta Advanced Communications Holdings Limited	\N
001ae7	Aztek Networks, Inc.	\N
001ae8	Siemens Enterprise Communications GmbH & Co. KG	\N
001ae9	Nintendo Co., Ltd.	\N
001aea	Radio Terminal Systems Pty Ltd	\N
001aeb	Allied Telesis K.K.	\N
001aec	Keumbee Electronics Co.,Ltd.	\N
001aed	INCOTEC GmbH	\N
001aee	Shenztech Ltd	\N
001aef	Loopcomm Technology, Inc.	\N
001af0	Alcatel - IPD	\N
001af1	Embedded Artists AB	\N
001af2	Dynavisions Schweiz AG	\N
001af3	Samyoung Electronics	\N
001af4	Handreamnet	\N
001af5	PENTAONE. CO., LTD.	\N
001af6	Woven Systems, Inc.	\N
001af7	dataschalt e+a GmbH	\N
001af8	Copley Controls Corporation	\N
001af9	AeroVIronment (AV Inc)	\N
001afa	Welch Allyn, Inc.	\N
001afb	Joby Inc.	\N
001afc	ModusLink Corporation	\N
001afd	EVOLIS	\N
001afe	SOFACREAL	\N
001aff	Wizyoung Tech.	\N
001b00	Neopost Technologies	\N
001b01	Applied Radio Technologies	\N
001b02	ED Co.Ltd	\N
001b03	Action Technology (SZ) Co., Ltd	\N
001b04	Affinity International S.p.a	\N
001b05	YMC AG	\N
001b06	Ateliers R. LAUMONIER	\N
001b07	Mendocino Software	\N
001b08	Danfoss Drives A/S	\N
001b09	Matrix Telecom Pvt. Ltd.	\N
001b0a	Intelligent Distributed Controls Ltd	\N
001b0b	Phidgets Inc.	\N
001b0c	Cisco Systems	\N
001b0d	Cisco Systems	\N
001b0e	InoTec GmbH Organisationssysteme	\N
001b0f	Petratec	\N
001b10	ShenZhen Kang Hui Technology Co.,ltd	\N
001b11	D-Link Corporation	\N
001b12	Apprion	\N
001b13	Icron Technologies Corporation	\N
001b14	Carex Lighting Equipment Factory	\N
001b15	Voxtel, Inc.	\N
001b16	Celtro Ltd.	\N
001b17	Palo Alto Networks	\N
001b18	Tsuken Electric Ind. Co.,Ltd	\N
001b19	IEEE 1588 Standard	\N
001b1a	e-trees Japan, Inc.	\N
001b1b	Siemens AG,	\N
001b1c	Coherent	\N
001b1d	Phoenix International Co., Ltd	\N
001b1e	HART Communication Foundation	\N
001b1f	DELTA - Danish Electronics, Light & Acoustics	\N
001b20	TPine Technology	\N
001b21	Intel Corporate	\N
001b22	Palit Microsystems ( H.K.) Ltd.	\N
001b23	SimpleComTools	\N
001b24	Quanta Computer Inc.	\N
001b25	Nortel	\N
001b26	RON-Telecom ZAO	\N
001b27	Merlin CSI	\N
001b28	POLYGON, JSC	\N
001b29	Avantis.Co.,Ltd	\N
001b2a	Cisco Systems	\N
001b2b	Cisco Systems	\N
001b2c	ATRON electronic GmbH	\N
001b2d	Med-Eng Systems Inc.	\N
001b2e	Sinkyo Electron Inc	\N
001b2f	NETGEAR Inc.	\N
001b30	Solitech Inc.	\N
001b31	Neural Image. Co. Ltd.	\N
001b32	QLogic Corporation	\N
001b33	Nokia Danmark A/S	\N
001b34	Focus System Inc.	\N
001b35	ChongQing JINOU Science & Technology Development CO.,Ltd	\N
001b36	Tsubata Engineering Co.,Ltd. (Head Office)	\N
001b37	Computec Oy	\N
001b38	COMPAL INFORMATION (KUNSHAN) CO., LTD.	\N
001b39	Proxicast	\N
001b3a	SIMS Corp.	\N
001b3b	Yi-Qing CO., LTD	\N
001b3c	Software Technologies Group,Inc.	\N
001b3d	EuroTel Spa	\N
001b3e	Curtis, Inc.	\N
001b3f	ProCurve Networking by HP	\N
001b40	Network Automation mxc AB	\N
001b41	General Infinity Co.,Ltd.	\N
001b42	Wise & Blue	\N
001b43	Beijing DG Telecommunications equipment Co.,Ltd	\N
001b44	SanDisk Corporation	\N
001b45	ABB AS, Division Automation Products	\N
001b46	Blueone Technology Co.,Ltd	\N
001b47	Futarque A/S	\N
001b48	Shenzhen Lantech Electronics Co., Ltd.	\N
001b49	Roberts Radio limited	\N
001b4a	W&W Communications, Inc.	\N
001b4b	SANION Co., Ltd.	\N
001b4c	Signtech	\N
001b4d	Areca Technology Corporation	\N
001b4e	Navman New Zealand	\N
001b4f	Avaya Inc.	\N
001b50	Nizhny Novgorod Factory named after M.Frunze, FSUE (NZiF)	\N
001b51	Vector Technology Corp.	\N
001b52	Motorola Mobile Devices	\N
001b53	Cisco Systems	\N
001b54	Cisco Systems	\N
001b55	Hurco Automation Ltd.	\N
001b56	Tehuti Networks Ltd.	\N
001b57	SEMINDIA SYSTEMS PRIVATE LIMITED	\N
001b58	ACE CAD Enterprise Co., Ltd.	\N
001b59	Sony Ericsson Mobile Communications AB	\N
001b5a	Apollo Imaging Technologies, Inc.	\N
001b5b	2Wire, Inc.	\N
001b5c	Azuretec Co., Ltd.	\N
001b5d	Vololink Pty Ltd	\N
001b5e	BPL Limited	\N
001b5f	Alien Technology	\N
001b60	NAVIGON AG	\N
001b61	Digital Acoustics, LLC	\N
001b62	JHT Optoelectronics Co.,Ltd.	\N
001b63	Apple Computer Inc.	\N
001b64	IsaacLandKorea Co., Ltd,	\N
001b65	China Gridcom Co., Ltd	\N
001b66	Sennheiser electronic GmbH & Co. KG	\N
001b67	Ubiquisys Ltd	\N
001b68	Modnnet Co., Ltd	\N
001b69	Equaline Corporation	\N
001b6a	Powerwave Technologies Sweden AB	\N
001b6b	Swyx Solutions AG	\N
001b6c	LookX Digital Media BV	\N
001b6d	Midtronics, Inc.	\N
001b6e	Anue Systems, Inc.	\N
001b6f	Teletrak Ltd	\N
001b70	IRI Ubiteq, INC.	\N
001b71	Telular Corp.	\N
001b72	Sicep s.p.a.	\N
001b73	DTL Broadcast Ltd	\N
001b74	MiraLink Corporation	\N
001b75	Hypermedia Systems	\N
001b76	Ripcode, Inc.	\N
001b77	Intel Corporate	\N
001b78	Hewlett Packard	\N
001b79	FAIVELEY TRANSPORT	\N
001b7a	Nintendo Co., Ltd.	\N
001b7b	The Tintometer Ltd	\N
001b7c	A & R Cambridge	\N
001b7d	CXR Anderson Jacobson	\N
001b7e	Beckmann GmbH	\N
001b7f	TMN Technologies Telecomunicacoes Ltda	\N
001b80	LORD Corporation	\N
001b81	DATAQ Instruments, Inc.	\N
001b82	Taiwan Semiconductor Co., Ltd.	\N
001b83	Finsoft Ltd	\N
001b84	Scan Engineering Telecom	\N
001b85	MAN Diesel SE	\N
001b86	Bosch Access Systems GmbH	\N
001b87	Deepsound Tech. Co., Ltd	\N
001b88	Divinet Access Technologies Ltd	\N
001b89	EMZA Visual Sense Ltd.	\N
001b8a	2M Electronic A/S	\N
001b8b	NEC AccessTechnica,Ltd.	\N
001b8c	JMicron Technology Corp.	\N
001b8d	Electronic Computer Systems, Inc.	\N
001b8e	Hulu Sweden AB	\N
001b8f	Cisco Systems	\N
001b90	Cisco Systems	\N
001b91	EFKON AG	\N
001b92	l-acoustics	\N
001b93	JC Decaux SA DNT	\N
001b94	T.E.M.A. S.p.A.	\N
001b95	VIDEO SYSTEMS SRL	\N
001b96	General Sensing	\N
001b97	Violin Technologies	\N
001b98	Samsung Electronics Co., Ltd.	\N
001b99	KS System GmbH	\N
001b9a	Apollo Fire Detectors Ltd	\N
001b9b	Hose-McCann Communications	\N
001b9c	SATEL sp. z o.o.	\N
001b9d	Novus Security Sp. z o.o.	\N
001b9e	ASKEY  COMPUTER  CORP	\N
001b9f	Calyptech Pty Ltd	\N
001ba0	Awox	\N
001ba1	Åmic AB	\N
001ba2	IDS Imaging Development Systems GmbH	\N
001ba3	Flexit Group GmbH	\N
001ba4	S.A.E Afikim	\N
001ba5	MyungMin Systems, Inc.	\N
001ba6	intotech inc.	\N
001ba7	Lorica Solutions	\N
001ba8	UBI&MOBI,.Inc	\N
001ba9	BROTHER INDUSTRIES, LTD.	\N
001baa	XenICs nv	\N
001bab	Telchemy, Incorporated	\N
001bac	Curtiss Wright Controls Embedded Computing	\N
001bad	iControl Incorporated	\N
001bae	Micro Control Systems, Inc	\N
001baf	Nokia Danmark A/S	\N
001bb0	BHARAT ELECTRONICS	\N
001bb1	Wistron Neweb Corp.	\N
001bb2	Intellect International NV	\N
001bb3	Condalo GmbH	\N
001bb4	Airvod Limited	\N
001bb5	ZF Electronics GmbH	\N
001bb6	Bird Electronic Corp.	\N
001bb7	Alta Heights Technology Corp.	\N
001bb8	BLUEWAY ELECTRONIC CO;LTD	\N
001bb9	Elitegroup Computer System Co.	\N
001bba	Nortel	\N
001bbb	RFTech Co.,Ltd	\N
001bbc	Silver Peak Systems, Inc.	\N
001bbd	FMC Kongsberg Subsea AS	\N
001bbe	ICOP Digital	\N
001bbf	SAGEM COMMUNICATION	\N
001bc0	Juniper Networks	\N
001bc1	HOLUX Technology, Inc.	\N
001bc2	Integrated Control Technology Limitied	\N
001bc3	Mobisolution Co.,Ltd	\N
001bc4	Ultratec, Inc.	\N
001bc5	IEEE Registration Authority	\N
001bc6	Strato Rechenzentrum AG	\N
001bc7	StarVedia Technology Inc.	\N
001bc8	MIURA CO.,LTD	\N
001bc9	FSN DISPLAY INC	\N
001bca	Beijing Run Technology LTD. Company	\N
001bcb	PEMPEK SYSTEMS PTY LTD	\N
001bcc	KINGTEK CCTV ALLIANCE CO., LTD.	\N
001bcd	DAVISCOMMS (S) PTE LTD	\N
001bce	Measurement Devices Ltd	\N
001bcf	Dataupia Corporation	\N
001bd0	IDENTEC SOLUTIONS	\N
001bd1	SOGESTMATIC	\N
001bd2	ULTRA-X ASIA PACIFIC Inc.	\N
001bd3	Matsushita Electric Panasonic AVC	\N
001bd4	Cisco Systems	\N
001bd5	Cisco Systems	\N
001bd6	Kelvin Hughes Ltd	\N
001bd7	Scientific Atlanta, A Cisco Company	\N
001bd8	DVTel LTD	\N
001bd9	Edgewater Computer Systems	\N
001bda	UTStarcom Inc	\N
001bdb	Valeo VECS	\N
001bdc	Vencer Co., Ltd.	\N
001bdd	Motorola CHS	\N
001bde	Renkus-Heinz, Inc.	\N
001bdf	Iskra MIS	\N
001be0	TELENOT ELECTRONIC GmbH	\N
001be1	ViaLogy	\N
001be2	AhnLab,Inc.	\N
001be3	Health Hero Network, Inc.	\N
001be4	TOWNET SRL	\N
001be5	802automation Limited	\N
001be6	VR AG	\N
001be7	Postek Electronics Co., Ltd.	\N
001be8	Ultratronik GmbH	\N
001be9	Broadcom Corporation	\N
001bea	Nintendo Co., Ltd.	\N
001beb	DMP Electronics INC.	\N
001bec	Netio Technologies Co., Ltd	\N
001bed	Brocade Communications Systems, Inc	\N
001bee	Nokia Danmark A/S	\N
001bef	Blossoms Digital Technology Co.,Ltd.	\N
001bf0	Value Platforms Limited	\N
001bf1	Nanjing SilverNet Software Co., Ltd.	\N
001bf2	KWORLD COMPUTER CO., LTD	\N
001bf3	TRANSRADIO SenderSysteme Berlin AG	\N
001bf4	KENWIN INDUSTRIAL(HK) LTD.	\N
001bf5	Tellink Sistemas de Telecomunicación S.L.	\N
001bf6	CONWISE Technology Corporation Ltd.	\N
001bf7	Lund IP Products AB	\N
001bf8	Digitrax Inc.	\N
001bf9	Intellitect Water Ltd	\N
001bfa	G.i.N. mbH	\N
001bfb	Alps Electric Co., Ltd	\N
001bfc	ASUSTek COMPUTER INC.	\N
001bfd	Dignsys Inc.	\N
001bfe	Zavio Inc.	\N
001bff	Millennia Media inc.	\N
001c00	Entry Point, LLC	\N
001c01	ABB Oy Drives	\N
001c02	Atto Devices, Inc	\N
001c03	Betty TV Technology AG	\N
001c04	Airgain, Inc.	\N
001c05	Nonin Medical Inc.	\N
001c06	Siemens Numerical Control Ltd., Nanjing	\N
001c07	Cwlinux Limited	\N
001c08	Anystream, Inc.	\N
001c09	SAE Electronic Co.,Ltd.	\N
001c0a	Shenzhen AEE Technology Co.,Ltd.	\N
001c0b	SmartAnt Telecom	\N
001c0c	TANITA Corporation	\N
001c0d	G-Technology, Inc.	\N
001c0e	Cisco Systems	\N
001c0f	Cisco Systems	\N
001c10	Cisco-Linksys, LLC	\N
001c11	Motorola CHS	\N
001c12	Motorola Mobile Devices	\N
001c13	OPTSYS TECHNOLOGY CO., LTD.	\N
001c14	VMware, Inc	\N
001c15	TXP Corporation	\N
001c16	ThyssenKrupp Elevator	\N
001c17	Nortel	\N
001c18	Sicert S.r.L.	\N
001c19	secunet Security Networks AG	\N
001c1a	Thomas Instrumentation, Inc	\N
001c1b	Hyperstone GmbH	\N
001c1c	Center Communication Systems GmbH	\N
001c1d	CHENZHOU GOSPELL DIGITAL TECHNOLOGY CO.,LTD	\N
001c1e	emtrion GmbH	\N
001c1f	Quest Retail Technology Pty Ltd	\N
001c20	CLB Benelux	\N
001c21	Nucsafe Inc.	\N
001c22	Aeris Elettronica s.r.l.	\N
001c23	Dell Inc	\N
001c24	Formosa Wireless Systems Corp.	\N
001c25	Hon Hai Precision Ind. Co.,Ltd.	\N
001c26	Hon Hai Precision Ind. Co.,Ltd.	\N
001c27	Sunell Electronics Co.	\N
001c28	Sphairon Access Systems GmbH	\N
001c29	CORE DIGITAL ELECTRONICS CO., LTD	\N
001c2a	Envisacor Technologies Inc.	\N
001c2b	Alertme.com Limited	\N
001c2c	Synapse	\N
001c2d	FlexRadio Systems	\N
001c2e	ProCurve Networking by HP	\N
001c2f	Pfister GmbH	\N
001c30	Mode Lighting (UK ) Ltd.	\N
001c31	Mobile XP Technology Co., LTD	\N
001c32	Telian Corporation	\N
001c33	Sutron	\N
001c34	HUEY CHIAO INTERNATIONAL CO., LTD.	\N
001c35	Nokia Danmark A/S	\N
001c36	iNEWiT NV	\N
001c37	Callpod, Inc.	\N
001c38	Bio-Rad Laboratories, Inc.	\N
001c39	S Netsystems Inc.	\N
001c3a	Element Labs, Inc.	\N
001c3b	AmRoad Technology Inc.	\N
001c3c	Seon Design Inc.	\N
001c3d	WaveStorm	\N
001c3e	ECKey Limited	\N
001c3f	International Police Technologies, Inc.	\N
001c40	VDG-Security bv	\N
001c41	scemtec Transponder Technology GmbH	\N
001c42	Parallels, Inc.	\N
001c43	Samsung Electronics Co.,Ltd	\N
001c44	Bosch Security Systems BV	\N
001c45	Chenbro Micom Co., Ltd.	\N
001c46	QTUM	\N
001c47	Hangzhou Hollysys Automation Co., Ltd	\N
001c48	WiDeFi, Inc.	\N
001c49	Zoltan Technology Inc.	\N
001c4a	AVM GmbH	\N
001c4b	Gener8, Inc.	\N
001c4c	Petrotest Instruments	\N
001c4d	Zeetoo, Inc.	\N
001c4e	TASA International Limited	\N
001c4f	MACAB AB	\N
001c50	TCL Technoly Electronics(Huizhou)Co.,Ltd	\N
001c51	Celeno Communications	\N
001c52	VISIONEE SRL	\N
001c53	Synergy Lighting Controls	\N
001c54	Hillstone Networks Inc	\N
001c55	Shenzhen Kaifa Technology Co.	\N
001c56	Pado Systems, Inc.	\N
001c57	Cisco Systems	\N
001c58	Cisco Systems	\N
001c59	DEVON IT	\N
001c5a	Advanced Relay Corporation	\N
001c5b	Chubb Electronic Security Systems Ltd	\N
001c5c	Integrated Medical Systems, Inc.	\N
001c5d	Leica Microsystems	\N
001c5e	ASTON France	\N
001c5f	Winland Electronics, Inc.	\N
001c60	CSP Frontier Technologies,Inc.	\N
001c61	Galaxy Technology (HK) Ltd.	\N
001c62	LG Electronics Inc	\N
001c63	TRUEN	\N
001c64	Cellnet+Hunt	\N
001c65	JoeScan, Inc.	\N
001c66	UCAMP CO.,LTD	\N
001c67	Pumpkin Networks, Inc.	\N
001c68	Anhui Sun Create Electronics Co., Ltd	\N
001c69	Packet Vision Ltd	\N
001c6a	Weiss Engineering Ltd.	\N
001c6b	COVAX  Co. Ltd	\N
001c6c	Jabil Circuit (Guangzhou) Limited	\N
001c6d	KYOHRITSU ELECTRONIC INDUSTRY CO., LTD.	\N
001c6e	Newbury Networks, Inc.	\N
001c6f	Emfit Ltd	\N
001c70	NOVACOMM LTDA	\N
001c71	Emergent Electronics	\N
001c72	Mayer & Cie GmbH & Co KG	\N
001c73	Arista Networks, Inc.	\N
001c74	Syswan Technologies Inc.	\N
001c75	RF Systems GmbH	\N
001c76	The Wandsworth Group Ltd	\N
001c77	Prodys	\N
001c78	WYPLAY SAS	\N
001c79	Cohesive Financial Technologies LLC	\N
001c7a	Perfectone Netware Company Ltd	\N
001c7b	Castlenet Technology Inc.	\N
001c7c	PERQ SYSTEMS CORPORATION	\N
001c7d	Excelpoint Manufacturing Pte Ltd	\N
001c7e	Toshiba	\N
001c7f	Check Point Software Technologies	\N
001c80	New Business Division/Rhea-Information CO., LTD.	\N
001c81	NextGen Venturi LTD	\N
001c82	Genew Technologies	\N
001c83	New Level Telecom Co., Ltd.	\N
001c84	STL Solution Co.,Ltd.	\N
001c85	Eunicorn	\N
001c86	Cranite Systems, Inc.	\N
001c87	Uriver Inc.	\N
001c88	TRANSYSTEM INC.	\N
001c89	Force Communications, Inc.	\N
001c8a	Verari Systems Inc	\N
001c8b	MJ Innovations Ltd.	\N
001c8c	DIAL TECHNOLOGY LTD.	\N
001c8d	Mesa Imaging	\N
001c8e	Alcatel-Lucent IPD	\N
001c8f	Advanced Electronic Design, Inc.	\N
001c90	Empacket Corporation	\N
001c91	Gefen Inc.	\N
001c92	Tervela	\N
001c93	ExaDigm Inc	\N
001c94	LI-COR Biosciences	\N
001c95	Opticomm Corporation	\N
001c96	Linkwise Technology Pte Ltd	\N
001c97	Enzytek Technology Inc.,	\N
001c98	LUCKY TECHNOLOGY (HK) COMPANY LIMITED	\N
001c99	Shunra Software Ltd.	\N
001c9a	Nokia Danmark A/S	\N
001c9b	FEIG ELECTRONIC GmbH	\N
001c9c	Nortel	\N
001c9d	Liecthi AG	\N
001c9e	Dualtech IT AB	\N
001c9f	Razorstream, LLC	\N
001ca0	Production Resource Group, LLC	\N
001ca1	AKAMAI TECHNOLOGIES, INC.	\N
001ca2	PIRELLI BROADBAND SOLUTIONS	\N
001ca3	Terra	\N
001ca4	Sony Ericsson Mobile Communications	\N
001ca5	Zygo Corporation	\N
001ca6	Win4NET	\N
001ca7	International Quartz Limited	\N
001ca8	AirTies Wireless Networks	\N
001ca9	Audiomatica Srl	\N
001caa	Bellon Pty Ltd	\N
001cab	Meyer Sound Laboratories, Inc.	\N
001cac	Qniq Technology Corp.	\N
001cad	Wuhan Telecommunication Devices Co.,Ltd	\N
001cae	WiChorus, Inc.	\N
001caf	Plato Networks Inc.	\N
001cb0	Cisco Systems	\N
001cb1	Cisco Systems	\N
001cb2	BPT SPA	\N
001cb3	APPLE, INC	\N
001cb4	Iridium Satellite LLC	\N
001cb5	Neihua Network Technology Co.,LTD.(NHN)	\N
001cb6	Duzon CNT Co., Ltd.	\N
001cb7	USC DigiArk Corporation	\N
001cb8	CBC Co., Ltd	\N
001cb9	KWANG SUNG ELECTRONICS CO., LTD.	\N
001cba	VerScient, Inc.	\N
001cbb	MusicianLink	\N
001cbc	CastGrabber, LLC	\N
001cbd	Ezze Mobile Tech., Inc.	\N
001cbe	Nintendo Co., Ltd.	\N
001cbf	Intel Corporate	\N
001cc0	Intel Corporate	\N
001cc1	Motorola Mobile Devices	\N
001cc2	Part II Research, Inc.	\N
001cc3	Pace Micro Technology plc	\N
001cc4	Hewlett Packard	\N
001cc5	3COM LTD	\N
001cc6	ProStor Systems	\N
001cc7	Rembrandt Technologies, LLC d/b/a REMSTREAM	\N
001cc8	INDUSTRONIC Industrie-Electronic GmbH & Co. KG	\N
001cc9	Kaise Electronic Technology Co., Ltd.	\N
001cca	Shanghai Gaozhi Science & Technology Development Co.	\N
001ccb	Forth Corporation Public Company Limited	\N
001ccc	Research In Motion Limited	\N
001ccd	Alektrona Corporation	\N
001cce	By Techdesign	\N
001ccf	LIMETEK	\N
001cd0	Circleone Co.,Ltd.	\N
001cd1	Waves Audio LTD	\N
001cd2	King Champion (Hong Kong) Limited	\N
001cd3	ZP Engineering SEL	\N
001cd4	Nokia Danmark A/S	\N
001cd5	ZeeVee, Inc.	\N
001cd6	Nokia Danmark A/S	\N
001cd7	Harman/Becker Automotive Systems GmbH	\N
001cd8	BlueAnt Wireless	\N
001cd9	GlobalTop Technology Inc.	\N
001cda	Exegin Technologies Limited	\N
001cdb	CARPOINT CO.,LTD	\N
001cdc	Custom Computer Services, Inc.	\N
001cdd	COWBELL ENGINEERING CO., LTD.	\N
001cde	Interactive Multimedia eXchange Inc.	\N
001cdf	Belkin International Inc.	\N
001ce0	DASAN TPS	\N
001ce1	INDRA SISTEMAS, S.A.	\N
001ce2	Attero Tech, LLC.	\N
001ce3	Optimedical Systems	\N
001ce4	EleSy JSC	\N
001ce5	MBS Electronic Systems GmbH	\N
001ce6	INNES	\N
001ce7	Rocon PLC Research Centre	\N
001ce8	Cummins Inc	\N
001ce9	Galaxy Technology Limited	\N
001cea	Scientific-Atlanta, Inc	\N
001ceb	Nortel	\N
001cec	Mobilesoft (Aust.) Pty Ltd	\N
001ced	ENVIRONNEMENT SA	\N
001cee	SHARP Corporation	\N
001cef	Primax Electronics LTD	\N
001cf0	D-Link Corporation	\N
001cf1	SUPoX Technology Co. , LTD.	\N
001cf2	Tenlon Technology Co.,Ltd.	\N
001cf3	EVS BROADCAST EQUIPMENT	\N
001cf4	Media Technology Systems Inc	\N
001cf5	Wiseblue Technology Limited	\N
001cf6	Cisco Systems	\N
001cf7	AudioScience	\N
001cf8	Parade Technologies, Ltd.	\N
001cf9	Cisco Systems	\N
001cfa	Alarm.com	\N
001cfb	Motorola CHS	\N
001dfe	Palm, Inc	\N
001cfc	Suminet Communication Technologies (Shanghai) Co., Ltd.	\N
001cfd	Universal Electronics	\N
001cfe	Quartics Inc	\N
001cff	Napera Networks Inc	\N
001d00	Brivo Systems, LLC	\N
001d01	Neptune Digital	\N
001d02	Cybertech Telecom Development	\N
001d03	Design Solutions Inc.	\N
001d04	Zipit Wireless, Inc.	\N
001d05	iLight	\N
001d06	HM Electronics, Inc.	\N
001d07	Shenzhen Sang Fei Consumer Communications Co.,Ltd	\N
001d08	JIANGSU YINHE ELECTRONICS CO., LTD	\N
001d09	Dell Inc	\N
001d0a	Davis Instruments, Inc.	\N
001d0b	Power Standards Lab	\N
001d0c	MobileCompia	\N
001d0d	Sony Computer Entertainment inc.	\N
001d0e	Agapha Technology co., Ltd.	\N
001d0f	TP-LINK Technologies Co., Ltd.	\N
001d10	LightHaus Logic, Inc.	\N
001d11	Analogue & Micro Ltd	\N
001d12	ROHM CO., LTD.	\N
001d13	NextGTV	\N
001d14	SPERADTONE INFORMATION TECHNOLOGY LIMITED	\N
001d15	Shenzhen Dolphin Electronic Co., Ltd	\N
001d16	Efixo	\N
001d17	Digital Sky Corporation	\N
001d18	Power Innovation GmbH	\N
001d19	Arcadyan Technology Corporation	\N
001d1a	OvisLink S.A.	\N
001d1b	Sangean Electronics Inc.	\N
001d1c	Gennet s.a.	\N
001d1d	Inter-M Corporation	\N
001d1e	KYUSHU TEN CO.,LTD	\N
001d1f	Siauliu Tauro Televizoriai, JSC	\N
001d20	COMTREND CO.	\N
001d21	Alcad SL	\N
001d22	Foss Analytical A/S	\N
001d23	SENSUS 	\N
001d24	Aclara Power-Line Systems Inc.	\N
001d25	Samsung Electronics Co.,Ltd	\N
001d26	Rockridgesound Technology Co.	\N
001d27	NAC-INTERCOM	\N
001d28	Sony Ericsson Mobile Communications AB	\N
001d29	Doro AB	\N
001d2a	Tideway Electronic LTD	\N
001d2b	Wuhan Pont Technology CO. , LTD	\N
001d2c	Wavetrend Technologies (Pty) Limited	\N
001d2d	Pylone, Inc.	\N
001d2e	Ruckus Wireless	\N
001d2f	QuantumVision Corporation	\N
001d30	YX Wireless S.A.	\N
001d31	HIGHPRO INTERNATIONAL R&D CO,.LTD.	\N
001d32	Longkay Communication & Technology (Shanghai) Co. Ltd	\N
001d33	Maverick Systems Inc.	\N
001d34	SYRIS Technology Corp	\N
001d35	Viconics Electronics Inc.	\N
001d36	ELECTRONICS CORPORATION OF INDIA LIMITED	\N
001d37	Thales-Panda Transportation System	\N
001d38	Seagate Technology	\N
001d39	MOOHADIGITAL CO., LTD	\N
001d3a	mh acoustics LLC	\N
001d3b	Nokia Danmark A/S	\N
001d3c	Muscle Corporation	\N
001d3d	Avidyne Corporation	\N
001d3e	SAKA TECHNO SCIENCE CO.,LTD	\N
001d3f	Mitron Pty Ltd	\N
001d40	Living Independently Group, Inc.	\N
001d41	Hardy Instruments	\N
001d42	Nortel	\N
001d43	Shenzhen G-link Digital Technology Co., Ltd.	\N
001d44	Krohne	\N
001d45	Cisco Systems	\N
001d46	Cisco Systems	\N
001d47	Covote GmbH & Co KG	\N
001d48	Sensor-Technik Wiedemann GmbH	\N
001d49	Innovation Wireless Inc.	\N
001d4a	Carestream Health, Inc.	\N
001d4b	Grid Connect Inc.	\N
001d4c	Alcatel-Lucent	\N
001d4d	Adaptive Recognition Hungary, Inc	\N
001d4e	TCM Mobile LLC	\N
001d4f	Apple Computer Inc.	\N
001d50	SPINETIX SA	\N
001d51	GE Energy	\N
001d52	Defzone B.V.	\N
001d53	S&O Electronics (Malaysia) Sdn. Bhd.	\N
001d54	Sunnic Technology & Merchandise INC.	\N
001d55	ZANTAZ, Inc	\N
001d56	Kramer Electronics Ltd.	\N
001d57	CAETEC Messtechnik	\N
001d58	CQ Inc	\N
001d59	Mitra Energy & Infrastructure	\N
001d5a	2Wire Inc.	\N
001d5b	Tecvan Informatica Ltda	\N
001d5c	Tom Communication Industrial Co.,Ltd.	\N
001d5d	Control Dynamics Pty. Ltd.	\N
001d5e	COMING MEDIA CORP.	\N
001d5f	OverSpeed SARL	\N
001d60	ASUSTek COMPUTER INC.	\N
001d61	BIJ Corporation	\N
001d62	InPhase Technologies	\N
001d63	Miele & Cie. KG	\N
001d64	Adam Communications Systems Int Ltd	\N
001d65	Microwave Radio Communications	\N
001d66	Hyundai Telecom	\N
001d67	AMEC	\N
001d68	Thomson Telecom Belgium	\N
001d69	Knorr-Bremse AG	\N
001d6a	Alpha Networks Inc.	\N
001d6b	Motorola (formerly Netopia, Inc	\N
001d6c	ClariPhy Communications, Inc.	\N
001d6d	Confidant International LLC	\N
001d6e	Nokia Danmark A/S	\N
001d6f	Chainzone Technology Co., Ltd	\N
001d70	Cisco Systems	\N
001d71	Cisco Systems	\N
001d72	Wistron Corporation	\N
001d73	Buffalo Inc.	\N
001d74	Tianjin China-Silicon Microelectronics Co., Ltd.	\N
001d75	Radioscape PLC	\N
001d76	Eyeheight Ltd.	\N
001d77	NSGate	\N
001d78	Invengo Information Technology Co.,Ltd	\N
001d79	SIGNAMAX LLC	\N
001d7a	Wideband Semiconductor, Inc.	\N
001d7b	Ice Energy, Inc.	\N
001d7c	ABE Elettronica S.p.A.	\N
001d7d	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
001d7e	Cisco-Linksys, LLC	\N
001d7f	Tekron International Ltd	\N
001d80	Beijing Huahuan Eletronics Co.,Ltd	\N
001d81	GUANGZHOU GATEWAY ELECTRONICS CO., LTD	\N
001d82	GN A/S (GN Netcom A/S)	\N
001d83	Emitech Corporation	\N
001d84	Gateway, Inc.	\N
001d85	Call Direct Cellular Solutions	\N
001d86	Shinwa Industries(China) Ltd.	\N
001d87	VigTech Labs Sdn Bhd	\N
001d88	Clearwire	\N
001d89	VaultStor Corporation	\N
001d8a	TechTrex Inc	\N
001d8b	PIRELLI BROADBAND SOLUTIONS	\N
001d8c	La Crosse Technology LTD	\N
001d8d	Raytek GmbH	\N
001d8e	Alereon, Inc.	\N
001d8f	PureWave Networks	\N
001d90	EMCO Flow Systems	\N
001d91	Digitize, Inc	\N
001d92	MICRO-STAR INT'L CO.,LTD.	\N
001d93	Modacom	\N
001d94	Climax Technology Co., Ltd	\N
001d95	Flash, Inc.	\N
001d96	WatchGuard Video	\N
001d97	Alertus Technologies LLC	\N
001d98	Nokia Danmark A/S	\N
001d99	Cyan Optic, Inc.	\N
001d9a	GODEX INTERNATIONAL CO., LTD	\N
001d9b	Hokuyo Automatic Co., Ltd.	\N
001d9c	Rockwell Automation	\N
001d9d	ARTJOY INTERNATIONAL LIMITED	\N
001d9e	AXION TECHNOLOGIES	\N
001d9f	MATT   R.P.Traczynscy Sp.J.	\N
001da0	Heng Yu Electronic Manufacturing Company Limited	\N
001da1	Cisco Systems	\N
001da2	Cisco Systems	\N
001da3	SabiOso	\N
001da4	Hangzhou System Technology CO., LTD	\N
001da5	WB Electronics	\N
001da6	Media Numerics Limited	\N
001da7	Seamless Internet	\N
001da8	Takahata Electronics Co.,Ltd	\N
001da9	Castles Technology, Co., LTD	\N
001daa	DrayTek Corp.	\N
001dab	SwissQual License AG	\N
001dac	Gigamon Systems LLC	\N
001dad	Sinotech Engineering Consultants, Inc.  Geotechnical Enginee	\N
001dae	CHANG TSENG TECHNOLOGY CO., LTD	\N
001daf	Nortel	\N
001db0	FuJian HengTong Information Technology Co.,Ltd	\N
001db1	Crescendo Networks	\N
001db2	HOKKAIDO ELECTRIC ENGINEERING CO.,LTD.	\N
001db3	ProCurve Networking by HP	\N
001db4	KUMHO ENG CO.,LTD	\N
001db5	Juniper networks	\N
001db6	BestComm Networks, Inc.	\N
001db7	Tendril Networks, Inc.	\N
001db8	Intoto Inc.	\N
001db9	Wellspring Wireless	\N
001dba	Sony Corporation	\N
001dbb	Dynamic System Electronics Corp.	\N
001dbc	Nintendo Co., Ltd.	\N
001dbd	Versamed Inc.	\N
001dbe	Motorola Mobile Devices	\N
001dbf	Radiient Technologies, Inc.	\N
001dc0	Enphase Energy	\N
001dc1	Audinate	\N
001dc2	XORTEC OY	\N
001dc3	RIKOR TV, Ltd	\N
001dc4	AIOI Systems Co., Ltd.	\N
001dc5	Beijing Jiaxun Feihong Electricial Co., Ltd.	\N
001dc6	SNR Inc.	\N
001dc7	L-3 Communications Geneva Aerospace	\N
001dc8	ScadaMetrcs, LLC.	\N
001dc9	GainSpan Corp.	\N
001dca	PAV Electronics Limited	\N
001dcb	Exéns Development Oy	\N
001dcc	Hetra Secure Solutions	\N
001dcd	ARRIS Group, Inc.	\N
001dce	ARRIS Group, Inc.	\N
001dcf	ARRIS Group, Inc.	\N
001dd0	ARRIS Group, Inc.	\N
001dd1	ARRIS Group, Inc.	\N
001dd2	ARRIS Group, Inc.	\N
001dd3	ARRIS Group, Inc.	\N
001dd4	ARRIS Group, Inc.	\N
001dd5	ARRIS Group, Inc.	\N
001dd6	ARRIS Group, Inc.	\N
001dd7	Algolith	\N
001dd8	Microsoft Corporation	\N
001dd9	Hon Hai Precision Ind.Co.,Ltd.	\N
001dda	Mikroelektronika spol. s r. o.	\N
001ddb	C-BEL Corporation	\N
001ddc	HangZhou DeChangLong Tech&Info Co.,Ltd	\N
001ddd	DAT H.K. LIMITED	\N
001dde	Zhejiang Broadcast&Television Technology Co.,Ltd.	\N
001ddf	Sunitec Enterprise Co., Ltd.	\N
001de0	Intel Corporate	\N
001de1	Intel Corporate	\N
001de2	Radionor Communications	\N
001de3	Intuicom	\N
001de4	Visioneered Image Systems	\N
001de5	Cisco Systems	\N
001de6	Cisco Systems	\N
001de7	Marine Sonic Technology, Ltd.	\N
001de8	Nikko Denki Tsushin Company(NDTC)	\N
001de9	Nokia Danmark A/S	\N
001dea	Commtest Instruments Ltd	\N
001deb	DINEC International	\N
001dec	Marusys	\N
001ded	Grid Net, Inc.	\N
001dee	NEXTVISION SISTEMAS DIGITAIS DE TELEVISÃO LTDA.	\N
001def	TRIMM, INC.	\N
001df0	Vidient Systems, Inc.	\N
001df1	Intego Systems, Inc.	\N
001df2	Netflix, Inc.	\N
001df3	SBS Science & Technology Co., Ltd	\N
001df4	Magellan Technology Pty Limited	\N
001df5	Sunshine Co,LTD	\N
001df6	Samsung Electronics Co.,Ltd	\N
001df7	R. STAHL Schaltgeräte GmbH	\N
001df8	Webpro Vision Technology Corporation	\N
001df9	Cybiotronics (Far East) Limited	\N
001dfa	Fujian LANDI Commercial Equipment Co.,Ltd	\N
001dfb	NETCLEUS Systems Corporation	\N
001dfc	KSIC	\N
001dfd	Nokia Danmark A/S	\N
001dff	Network Critical Solutions Ltd	\N
001e00	Shantou Institute of Ultrasonic Instruments	\N
001e01	Renesas Technology Sales Co., Ltd.	\N
001e02	Sougou Keikaku Kougyou Co.,Ltd.	\N
001e03	LiComm Co., Ltd.	\N
001e04	Hanson Research Corporation	\N
001e05	Xseed Technologies & Computing	\N
001e06	WIBRAIN	\N
001e07	Winy Technology Co., Ltd.	\N
001e08	Centec Networks Inc	\N
001e09	ZEFATEK Co.,LTD	\N
001e0a	Syba Tech Limited	\N
001e0b	Hewlett Packard	\N
001e0c	Sherwood Information Partners, Inc.	\N
001e0d	Micran Ltd.	\N
001e0e	MAXI VIEW HOLDINGS LIMITED	\N
001e0f	Briot International	\N
001e10	ShenZhen Huawei Communication Technologies Co.,Ltd.	\N
001e11	ELELUX INTERNATIONAL LTD	\N
001e12	Ecolab	\N
001e13	Cisco Systems	\N
001e14	Cisco Systems	\N
001e15	Beech Hill Electronics	\N
001e16	Keytronix	\N
001e17	STN BV	\N
001e18	Radio Activity srl	\N
001e19	GTRI	\N
001e1a	Best Source Taiwan Inc.	\N
001e1b	Digital Stream Technology, Inc.	\N
001e1c	SWS Australia Pty Limited	\N
001e1d	East Coast Datacom, Inc.	\N
001e1e	Honeywell Life Safety	\N
001e1f	Nortel	\N
001e20	Intertain Inc.	\N
001e21	Qisda Co.	\N
001e22	ARVOO Imaging Products BV	\N
001e23	Electronic Educational Devices, Inc	\N
001e24	Zhejiang Bell Technology Co.,ltd	\N
001e25	Intek Digital Inc	\N
001e26	Digifriends Co. Ltd	\N
001e27	SBN TECH Co.,Ltd.	\N
001e28	Lumexis Corporation	\N
001e29	Hypertherm Inc	\N
001e2a	Netgear Inc.	\N
001e2b	Radio Systems Design, Inc.	\N
001e2c	CyVerse Corporation	\N
001e2d	STIM	\N
001e2e	SIRTI S.p.A.	\N
001e2f	DiMoto Pty Ltd	\N
001e30	Shireen Inc	\N
001e31	INFOMARK CO.,LTD.	\N
001e32	Zensys	\N
001e33	Inventec Corporation	\N
001e34	CryptoMetrics	\N
001e35	Nintendo Co., Ltd.	\N
001e36	IPTE	\N
001e37	USI	\N
001e38	Bluecard Software Technology Co., Ltd.	\N
001e39	Comsys Communication Ltd.	\N
001e3a	Nokia Danmark A/S	\N
001e3b	Nokia Danmark A/S	\N
001e3c	Lyngbox Media AB	\N
001e3d	Alps Electric Co., Ltd	\N
001e3e	KMW Inc.	\N
001e3f	TrellisWare Technologies, Inc.	\N
001e40	Shanghai DareGlobal Technologies  Co.,Ltd.	\N
001e41	Microwave Communication & Component, Inc.	\N
001e42	Teltonika	\N
001e43	AISIN AW CO.,LTD.	\N
001e44	SANTEC	\N
001e45	Sony Ericsson Mobile Communications AB	\N
001e46	Motorola CHS	\N
001e47	PT. Hariff Daya Tunggal Engineering	\N
001e48	Wi-Links	\N
001e49	Cisco Systems	\N
001e4a	Cisco Systems	\N
001e4b	City Theatrical	\N
001e4c	Hon Hai Precision Ind.Co., Ltd.	\N
001e4d	Welkin Sciences, LLC	\N
001e4e	DAKO EDV-Ingenieur- und Systemhaus GmbH	\N
001e4f	Dell Inc.	\N
001e50	BATTISTONI RESEARCH	\N
001e51	Converter Industry Srl	\N
001e52	Apple Computer Inc	\N
001e53	Further Tech Co., LTD	\N
001e54	TOYO ELECTRIC Corporation	\N
001e55	COWON SYSTEMS,Inc.	\N
001e56	Bally Wulff Entertainment GmbH	\N
001e57	ALCOMA, spol. s r.o.	\N
001e58	D-Link Corporation	\N
001e59	Silicon Turnkey Express, LLC	\N
001e5a	Motorola CHS	\N
001e5b	Unitron Company, Inc.	\N
001e5c	RB GeneralEkonomik	\N
001e5d	Holosys d.o.o.	\N
001e5e	COmputime Ltd.	\N
001e5f	KwikByte, LLC	\N
001e60	Digital Lighting Systems, Inc	\N
001e61	ITEC GmbH	\N
001e62	Siemon	\N
001e63	Vibro-Meter SA	\N
001e64	Intel Corporate	\N
001e65	Intel Corporate	\N
001e66	RESOL Elektronische Regelungen GmbH	\N
001e67	Intel Corporate	\N
001e68	Quanta Computer	\N
001e69	Thomson Inc.	\N
001e6a	Beijing Bluexon Technology Co.,Ltd	\N
001e6b	Scientific Atlanta, A Cisco Company	\N
001e6c	Carbon Mountain LLC	\N
001e6d	IT R&D Center	\N
001e6e	Shenzhen First Mile Communications Ltd	\N
001e6f	Magna-Power Electronics, Inc.	\N
001e70	Cobham Defence Communications Ltd	\N
001e71	IgeaCare Systems Inc.	\N
001e72	PCS	\N
001e73	ZTE CORPORATION	\N
001e74	SAGEM COMMUNICATION	\N
001e75	LG Electronics	\N
001e76	Thermo Fisher Scientific	\N
001e77	Air2App	\N
001e78	Owitek Technology Ltd.,	\N
001e79	Cisco Systems	\N
001e7a	Cisco Systems	\N
001e7b	R.I.CO. S.r.l.	\N
001e7c	Taiwick Limited	\N
001e7d	Samsung Electronics Co.,Ltd	\N
001e7e	Nortel	\N
001e7f	CBM of America	\N
001e80	Last Mile Ltd.	\N
001e81	CNB Technology Inc.	\N
001e82	Pliant Technology, Inc.	\N
001e83	LAN/MAN Standards Association (LMSC)	\N
001e84	Pika Technologies Inc.	\N
001e85	Lagotek Corporation	\N
001e86	MEL Co.,Ltd.	\N
001e87	Realease Limited	\N
001e88	ANDOR SYSTEM SUPPORT CO., LTD.	\N
001e89	CRFS Limited	\N
001e8a	eCopy, Inc	\N
001e8b	Infra Access Korea Co., Ltd.	\N
001e8c	ASUSTek COMPUTER INC.	\N
001e8d	Motorola Mobile Devices	\N
001e8e	Hunkeler AG	\N
001e8f	CANON INC.	\N
001e90	Elitegroup Computer Systems Co	\N
001e91	KIMIN Electronic Co., Ltd.	\N
001e92	JEULIN S.A.	\N
001e93	CiriTech Systems Inc	\N
001e94	SUPERCOM TECHNOLOGY CORPORATION	\N
001e95	SIGMALINK	\N
001e96	Sepura Plc	\N
001e97	Medium Link System Technology CO., LTD,	\N
001e98	GreenLine Communications	\N
001e99	Vantanol Industrial Corporation	\N
001e9a	HAMILTON Bonaduz AG	\N
001e9b	San-Eisha, Ltd.	\N
001e9c	Fidustron INC	\N
001e9d	Recall Technologies, Inc.	\N
001e9e	ddm hopt + schuler Gmbh + Co. KG	\N
001e9f	Visioneering Systems, Inc.	\N
001ea0	XLN-t	\N
001ea1	Brunata a/s	\N
001ea2	Symx Systems, Inc.	\N
001ea3	Nokia Danmark A/S	\N
001ea4	Nokia Danmark A/S	\N
001ea5	ROBOTOUS, Inc.	\N
001ea6	Best IT World (India) Pvt. Ltd.	\N
001ea7	ActionTec Electronics, Inc	\N
001ea8	Datang Mobile Communications Equipment CO.,LTD	\N
001ea9	Nintendo Co., Ltd.	\N
001eaa	E-Senza Technologies GmbH	\N
001eab	TeleWell Oy	\N
001eac	Armadeus Systems	\N
001ead	Wingtech Group Limited	\N
001eae	Continental Automotive Systems	\N
001eaf	Ophir Optronics Ltd	\N
001eb0	ImesD Electronica S.L.	\N
001eb1	Cryptsoft Pty Ltd	\N
001eb2	LG innotek	\N
001eb3	Primex Wireless	\N
001eb4	UNIFAT TECHNOLOGY LTD.	\N
001eb5	Ever Sparkle Technologies Ltd	\N
001eb6	TAG Heuer SA	\N
001eb7	TBTech, Co., Ltd.	\N
001eb8	Fortis, Inc.	\N
001eb9	Sing Fai Technology Limited	\N
001eba	High Density Devices AS	\N
001ebb	BLUELIGHT TECHNOLOGY INC.	\N
001ebc	WINTECH AUTOMATION CO.,LTD.	\N
001ebd	Cisco Systems	\N
001ebe	Cisco Systems	\N
001ebf	Haas Automation Inc.	\N
001ec0	ZeroG Wireless Inc.	\N
001ec1	3COM EUROPE LTD	\N
001ec2	Apple, Inc	\N
001ec3	Kozio, Inc.	\N
001ec4	Celio Corp	\N
001ec5	Middle Atlantic Products Inc	\N
001ec6	Obvius Holdings LLC	\N
001ec7	2Wire	\N
001ec8	Rapid Mobile (Pty) Ltd	\N
001ec9	Dell Inc	\N
001eca	Nortel	\N
001ecb	"RPC "Energoautomatika" Ltd	\N
001ecc	CDVI	\N
001ecd	KYLAND	\N
001ece	BISA Technologies (Hong Kong) Limited	\N
001ecf	PHILIPS ELECTRONICS UK LTD	\N
001ed0	CONNEXIUM	\N
001ed1	Keyprocessor B.V.	\N
001ed2	Ray Shine Video Technology Inc	\N
001ed3	Dot Technology Int'l Co., Ltd.	\N
001ed4	Doble Engineering	\N
001ed5	Tekon-Automatics	\N
001ed6	Alentec & Orion AB	\N
001ed7	H-Stream Wireless, Inc.	\N
001ed8	Digital United Inc.	\N
001ed9	Mitsubishi Precision Co.,LTd.	\N
001eda	Wesemann Elektrotechniek B.V.	\N
001edb	Giken Trastem Co., Ltd.	\N
001edc	Sony Ericsson Mobile Communications AB	\N
001edd	WASKO S.A.	\N
001ede	BYD COMPANY LIMITED	\N
001edf	Master Industrialization Center Kista	\N
001ee0	Urmet Domus SpA	\N
001ee1	Samsung Electronics Co.,Ltd	\N
001ee2	Samsung Electronics Co.,Ltd	\N
001ee3	T&W Electronics (ShenZhen) Co.,Ltd	\N
001ee4	ACS Solutions France	\N
001ee5	Cisco-Linksys, LLC	\N
001ee6	Shenzhen Advanced Video Info-Tech Co., Ltd.	\N
001ee7	Epic Systems Inc	\N
001ee8	Mytek	\N
001ee9	Stoneridge Electronics AB	\N
001eea	Sensor Switch, Inc.	\N
001eeb	Talk-A-Phone Co.	\N
001eec	COMPAL INFORMATION (KUNSHAN) CO., LTD.	\N
001eed	Adventiq Ltd.	\N
001eee	ETL Systems Ltd	\N
001eef	Cantronic International Limited	\N
001ef0	Gigafin Networks	\N
001ef1	Servimat	\N
001ef2	Micro Motion Inc	\N
001ef3	From2	\N
001ef4	L-3 Communications Display Systems	\N
001ef5	Hitek Automated Inc.	\N
001ef6	Cisco Systems	\N
001ef7	Cisco Systems	\N
001ef8	Emfinity Inc.	\N
001ef9	Pascom Kommunikations systeme GmbH.	\N
001efa	PROTEI Ltd.	\N
001efb	Trio Motion Technology Ltd	\N
001efc	JSC "MASSA-K"	\N
001efd	Microbit 2.0 AB	\N
001efe	LEVEL s.r.o.	\N
001eff	Mueller-Elektronik GmbH & Co. KG	\N
001f00	Nokia Danmark A/S	\N
001f01	Nokia Danmark A/S	\N
001f02	Pixelmetrix Corporation Pte Ltd	\N
001f03	NUM AG	\N
001f04	Granch Ltd.	\N
001f05	iTAS Technology Corp.	\N
001f06	Integrated Dispatch Solutions	\N
001f07	AZTEQ Mobile	\N
001f08	RISCO LTD	\N
001f09	JASTEC CO., LTD.	\N
001f0a	Nortel	\N
001f0b	Federal State Unitary Enterprise Industrial Union"Electropribor"	\N
001f0c	Intelligent Digital Services GmbH	\N
001f0d	L3 Communications - Telemetry West	\N
001f0e	Japan Kyastem Co., Ltd	\N
001f0f	Select Engineered Systems	\N
001f10	TOLEDO DO BRASIL INDUSTRIA DE BALANCAS  LTDA	\N
001f11	OPENMOKO, INC.	\N
001f12	Juniper Networks	\N
001f13	S.& A.S. Ltd.	\N
001f14	NexG	\N
001f15	Bioscrypt Inc	\N
001f16	Wistron Corporation	\N
001f17	IDX Company, Ltd.	\N
001f18	Hakusan.Mfg.Co,.Ltd	\N
001f19	BEN-RI ELECTRONICA S.A.	\N
001f1a	Prominvest	\N
001f1b	RoyalTek Company Ltd.	\N
001f1c	KOBISHI ELECTRIC Co.,Ltd.	\N
001f1d	Atlas Material Testing Technology LLC	\N
001f1e	Astec Technology Co., Ltd	\N
001f1f	Edimax Technology Co. Ltd.	\N
001f20	Logitech Europe SA	\N
001f21	Inner Mongolia Yin An Science & Technology Development Co.,L	\N
001f22	Fiberxon, Inc.	\N
001f23	Interacoustics	\N
001f24	DIGITVIEW TECHNOLOGY CO., LTD.	\N
001f25	MBS GmbH	\N
001f26	Cisco Systems	\N
001f27	Cisco Systems	\N
001f28	ProCurve Networking by HP	\N
001f29	Hewlett Packard	\N
001f2a	ACCM	\N
001f2b	Orange Logic	\N
001f2c	Starbridge Networks	\N
001f2d	Electro-Optical Imaging, Inc.	\N
001f2e	Triangle Research Int'l Pte Ltd	\N
001f2f	Berker GmbH & Co. KG	\N
001f30	Travelping	\N
001f31	Radiocomp	\N
001f32	Nintendo Co., Ltd.	\N
001f33	Netgear Inc.	\N
001f34	Lung Hwa Electronics Co., Ltd.	\N
001f35	AIR802 LLC	\N
001f36	Bellwin Information Co. Ltd.,	\N
001f37	Genesis I&C	\N
001f38	POSITRON	\N
001f39	Construcciones y Auxiliar de Ferrocarriles, S.A.	\N
001f3a	Hon Hai Precision Ind.Co., Ltd.	\N
001f3b	Intel Corporate	\N
001f3c	Intel Corporate	\N
001f3d	Qbit GmbH	\N
001f3e	RP-Technik e.K.	\N
001f3f	AVM GmbH	\N
001f40	Speakercraft Inc.	\N
001f41	Ruckus Wireless	\N
001f42	Etherstack Pty Ltd	\N
001f43	ENTES ELEKTRONIK	\N
001f44	GE Transportation Systems	\N
001f45	Enterasys	\N
001f46	Nortel	\N
001f47	MCS Logic Inc.	\N
001f48	Mojix Inc.	\N
001f49	Eurosat Distribution Ltd	\N
001f4a	Albentia Systems S.A.	\N
001f4b	Lineage Power	\N
001f4c	Roseman Engineering Ltd	\N
001f4d	Segnetics LLC	\N
001f4e	ConMed Linvatec	\N
001f4f	Thinkware Co. Ltd.	\N
001f50	Swissdis AG	\N
001f51	HD Communications Corp	\N
001f52	UVT Unternehmensberatung für Verkehr und Technik GmbH	\N
001f53	GEMAC Gesellschaft für Mikroelektronikanwendung Chemnitz mbH	\N
001f54	Lorex Technology Inc.	\N
001f55	Honeywell Security (China) Co., Ltd.	\N
001f56	DIGITAL FORECAST	\N
001f57	Phonik Innovation Co.,LTD	\N
001f58	EMH Energiemesstechnik GmbH	\N
001f59	Kronback Tracers	\N
001f5a	Beckwith Electric Co.	\N
001f5b	Apple, Inc.	\N
001f5c	Nokia Danmark A/S	\N
001f5d	Nokia Danmark A/S	\N
001f5e	Dyna Technology Co.,Ltd.	\N
001f5f	Blatand GmbH	\N
001f60	COMPASS SYSTEMS CORP.	\N
001f61	Talent Communication Networks Inc.	\N
001f62	JSC "Stilsoft"	\N
001f63	JSC Goodwin-Europa	\N
001f64	Beijing Autelan Technology Inc.	\N
001f65	KOREA ELECTRIC TERMINAL CO., LTD.	\N
001f66	PLANAR LLC	\N
001f67	Hitachi,Ltd.	\N
001f68	Martinsson Elektronik AB	\N
001f69	Pingood Technology Co., Ltd.	\N
001f6a	PacketFlux Technologies, Inc.	\N
001f6b	LG Electronics	\N
001f6c	Cisco Systems	\N
001f6d	Cisco Systems	\N
001f6e	Vtech Engineering Corporation	\N
001f6f	Fujian Sunnada Communication Co.,Ltd.	\N
001f70	Botik Technologies LTD	\N
001f71	xG Technology, Inc.	\N
001f72	QingDao Hiphone Technology Co,.Ltd	\N
001f73	Teraview Technology Co., Ltd.	\N
001f74	Eigen Development	\N
001f75	GiBahn Media	\N
001f76	AirLogic Systems Inc.	\N
001f77	HEOL DESIGN	\N
001f78	Blue Fox Porini Textile	\N
001f79	Lodam Electronics A/S	\N
001f7a	WiWide Inc.	\N
001f7b	TechNexion Ltd.	\N
001f7c	Witelcom AS	\N
001f7d	embedded wireless GmbH	\N
001f7e	Motorola Mobile Devices	\N
001f7f	Phabrix Limited	\N
001f80	Lucas Holding bv	\N
001f81	Accel Semiconductor Corp	\N
001f82	Cal-Comp Electronics & Communications Co., Ltd	\N
001f83	Teleplan Technology Services Sdn Bhd	\N
001f84	Gigle Semiconductor	\N
001f85	Apriva ISS, LLC	\N
001f86	digEcor	\N
001f87	Skydigital Inc.	\N
001f88	FMS Force Measuring Systems AG	\N
001f89	Signalion GmbH	\N
001f8a	Ellion Digital Inc.	\N
001f8b	Storspeed, Inc.	\N
001f8c	CCS Inc.	\N
001f8d	Ingenieurbuero Stark GmbH und Ko. KG	\N
001f8e	Metris USA Inc.	\N
001f8f	Shanghai Bellmann Digital Source Co.,Ltd.	\N
001f90	Actiontec Electronics, Inc	\N
001f91	DBS Lodging Technologies, LLC	\N
001f92	VideoIQ, Inc.	\N
001f93	Xiotech Corporation	\N
001f94	Lascar Electronics Ltd	\N
001f95	SAGEM COMMUNICATION	\N
001f96	APROTECH CO.LTD	\N
001f97	BERTANA SRL	\N
001f98	DAIICHI-DENTSU LTD.	\N
001f99	SERONICS co.ltd	\N
001f9a	Nortel Networks	\N
001f9b	POSBRO	\N
001f9c	LEDCO	\N
001f9d	Cisco Systems	\N
001f9e	Cisco Systems	\N
001f9f	Thomson Telecom Belgium	\N
001fa0	A10 Networks	\N
001fa1	Gtran Inc	\N
001fa2	Datron World Communications, Inc.	\N
001fa3	T&W Electronics(Shenzhen)Co.,Ltd.	\N
001fa4	ShenZhen Gongjin Electronics Co.,Ltd	\N
001fa5	Blue-White Industries	\N
001fa6	Stilo srl	\N
001fa7	Sony Computer Entertainment Inc.	\N
001fa8	ANI Technologies Corp.	\N
001fa9	Atlanta DTH, Inc.	\N
001faa	Taseon, Inc.	\N
001fab	I.S HIGH TECH.INC	\N
001fac	Goodmill Systems Ltd	\N
001fad	Brown Innovations, Inc	\N
001fae	Blick South Africa (Pty) Ltd	\N
001faf	NextIO, Inc.	\N
001fb0	TimeIPS, Inc.	\N
001fb1	Cybertech Inc.	\N
001fb2	Sontheim Industrie Elektronik GmbH	\N
001fb3	2Wire	\N
001fb4	SmartShare Systems	\N
001fb5	I/O Interconnect Inc.	\N
001fb6	Chi Lin Technology Co., Ltd.	\N
001fb7	WiMate Technologies Corp.	\N
001fb8	Universal Remote Control, Inc.	\N
001fb9	Paltronics	\N
001fba	BoYoung Tech. & Marketing, Inc.	\N
001fbb	Xenatech Co.,LTD	\N
001fbc	EVGA Corporation	\N
001fbd	Kyocera Wireless Corp.	\N
001fbe	Shenzhen Mopnet Industrial Co.,Ltd	\N
001fbf	Fulhua Microelectronics Corp. Taiwan Branch	\N
001fc0	Control Express Finland Oy	\N
001fc1	Hanlong Technology Co.,LTD	\N
001fc2	Jow Tong Technology Co Ltd	\N
001fc3	SmartSynch, Inc	\N
001fc4	Motorola CHS	\N
001fc5	Nintendo Co., Ltd.	\N
001fc6	ASUSTek COMPUTER INC.	\N
001fc7	Casio Hitachi Mobile Comunications Co., Ltd.	\N
001fc8	Up-Today Industrial Co., Ltd.	\N
001fc9	Cisco Systems	\N
001fca	Cisco Systems	\N
001fcb	NIW Solutions	\N
001fcc	Samsung Electronics Co.,Ltd	\N
001fcd	Samsung Electronics	\N
001fce	QTECH LLC	\N
001fcf	MSI Technology GmbH	\N
001fd0	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
001fd1	OPTEX CO.,LTD.	\N
001fd2	COMMTECH TECHNOLOGY MACAO COMMERCIAL OFFSHORE LTD.	\N
001fd3	RIVA Networks Inc.	\N
001fd4	4IPNET, INC.	\N
001fd5	MICRORISC s.r.o.	\N
001fd6	Shenzhen Allywll	\N
001fd7	TELERAD SA	\N
001fd8	A-TRUST COMPUTER CORPORATION	\N
001fd9	RSD Communications Ltd	\N
001fda	Nortel Networks	\N
001fdb	Network Supply Corp.,	\N
001fdc	Mobile Safe Track Ltd	\N
001fdd	GDI LLC	\N
001fde	Nokia Danmark A/S	\N
001fdf	Nokia Danmark A/S	\N
001fe0	EdgeVelocity Corp	\N
001fe1	Hon Hai Precision Ind. Co., Ltd.	\N
001fe2	Hon Hai Precision Ind. Co., Ltd.	\N
001fe3	LG Electronics	\N
001fe4	Sony Ericsson Mobile Communications	\N
001fe5	In-Circuit GmbH	\N
001fe6	Alphion Corporation	\N
001fe7	Simet	\N
001fe8	KURUSUGAWA Electronics Industry Inc,.	\N
001fe9	Printrex, Inc.	\N
001fea	Applied Media Technologies Corporation	\N
001feb	Trio Datacom Pty Ltd	\N
001fec	Synapse électronique	\N
001fed	Tecan Systems Inc.	\N
001fee	ubisys technologies GmbH	\N
001fef	SHINSEI INDUSTRIES CO.,LTD	\N
001ff0	Audio Partnership	\N
001ff1	Paradox Hellas S.A.	\N
001ff2	VIA Technologies, Inc.	\N
001ff3	Apple, Inc	\N
001ff4	Power Monitors, Inc.	\N
001ff5	Kongsberg Defence & Aerospace	\N
001ff6	PS Audio International	\N
001ff7	Nakajima All Precision Co., Ltd.	\N
001ff8	Siemens AG, Sector Industry, Drive Technologies, Motion Control Systems	\N
001ff9	Advanced Knowledge Associates	\N
001ffa	Coretree, Co, Ltd	\N
001ffb	Green Packet Bhd	\N
001ffc	Riccius+Sohn GmbH	\N
001ffd	Indigo Mobile Technologies Corp.	\N
001ffe	ProCurve Networking by HP	\N
001fff	Respironics, Inc.	\N
002000	LEXMARK INTERNATIONAL, INC.	\N
002001	DSP SOLUTIONS, INC.	\N
002002	SERITECH ENTERPRISE CO., LTD.	\N
002003	PIXEL POWER LTD.	\N
002004	YAMATAKE-HONEYWELL CO., LTD.	\N
002005	SIMPLE TECHNOLOGY	\N
002006	GARRETT COMMUNICATIONS, INC.	\N
002007	SFA, INC.	\N
002008	CABLE & COMPUTER TECHNOLOGY	\N
002009	PACKARD BELL ELEC., INC.	\N
00200a	SOURCE-COMM CORP.	\N
00200b	OCTAGON SYSTEMS CORP.	\N
00200c	ADASTRA SYSTEMS CORP.	\N
00200d	CARL ZEISS	\N
00200e	SATELLITE TECHNOLOGY MGMT, INC	\N
00200f	TANBAC CO., LTD.	\N
002010	JEOL SYSTEM TECHNOLOGY CO. LTD	\N
002011	CANOPUS CO., LTD.	\N
002012	CAMTRONICS MEDICAL SYSTEMS	\N
002013	DIVERSIFIED TECHNOLOGY, INC.	\N
002014	GLOBAL VIEW CO., LTD.	\N
002015	ACTIS COMPUTER SA	\N
002016	SHOWA ELECTRIC WIRE & CABLE CO	\N
002017	ORBOTECH	\N
002018	CIS TECHNOLOGY INC.	\N
002019	OHLER GmbH	\N
00201a	MRV Communications, Inc.	\N
00201b	NORTHERN TELECOM/NETWORK	\N
00201c	EXCEL, INC.	\N
00201d	KATANA PRODUCTS	\N
00201e	NETQUEST CORPORATION	\N
00201f	BEST POWER TECHNOLOGY, INC.	\N
002020	MEGATRON COMPUTER INDUSTRIES PTY, LTD.	\N
002021	ALGORITHMS SOFTWARE PVT. LTD.	\N
002022	NMS Communications	\N
002023	T.C. TECHNOLOGIES PTY. LTD	\N
002024	PACIFIC COMMUNICATION SCIENCES	\N
002025	CONTROL TECHNOLOGY, INC.	\N
002026	AMKLY SYSTEMS, INC.	\N
002027	MING FORTUNE INDUSTRY CO., LTD	\N
002028	WEST EGG SYSTEMS, INC.	\N
002029	TELEPROCESSING PRODUCTS, INC.	\N
00202a	N.V. DZINE	\N
00202b	ADVANCED TELECOMMUNICATIONS MODULES, LTD.	\N
00202c	WELLTRONIX CO., LTD.	\N
00202d	TAIYO CORPORATION	\N
00202e	DAYSTAR DIGITAL	\N
00202f	ZETA COMMUNICATIONS, LTD.	\N
002030	ANALOG & DIGITAL SYSTEMS	\N
002031	ERTEC GmbH	\N
002032	ALCATEL TAISEL	\N
002033	SYNAPSE TECHNOLOGIES, INC.	\N
002034	ROTEC INDUSTRIEAUTOMATION GMBH	\N
002035	IBM CORPORATION	\N
002036	BMC SOFTWARE	\N
002037	SEAGATE TECHNOLOGY	\N
002038	VME MICROSYSTEMS INTERNATIONAL CORPORATION	\N
002039	SCINETS	\N
00203a	DIGITAL BI0METRICS INC.	\N
00203b	WISDM LTD.	\N
00203c	EUROTIME AB	\N
00203d	Honeywell ECC	\N
00203e	LogiCan Technologies, Inc.	\N
00203f	JUKI CORPORATION	\N
002040	Motorola Broadband Communications Sector	\N
002041	DATA NET	\N
002042	DATAMETRICS CORP.	\N
002043	NEURON COMPANY LIMITED	\N
002044	GENITECH PTY LTD	\N
002045	ION Networks, Inc.	\N
002046	CIPRICO, INC.	\N
002047	STEINBRECHER CORP.	\N
002048	Marconi Communications	\N
002049	COMTRON, INC.	\N
00204a	PRONET GMBH	\N
00204b	AUTOCOMPUTER CO., LTD.	\N
00204c	MITRON COMPUTER PTE LTD.	\N
00204d	INOVIS GMBH	\N
00204e	NETWORK SECURITY SYSTEMS, INC.	\N
00204f	DEUTSCHE AEROSPACE AG	\N
002050	KOREA COMPUTER INC.	\N
002051	Verilink Corporation	\N
002052	RAGULA SYSTEMS	\N
002053	HUNTSVILLE MICROSYSTEMS, INC.	\N
002054	Sycamore Networks	\N
002055	ALTECH CO., LTD.	\N
002056	NEOPRODUCTS	\N
002057	TITZE DATENTECHNIK GmbH	\N
002058	ALLIED SIGNAL INC.	\N
002059	MIRO COMPUTER PRODUCTS AG	\N
00205a	COMPUTER IDENTICS	\N
00205b	Kentrox, LLC	\N
00205c	InterNet Systems of Florida, Inc.	\N
00205d	NANOMATIC OY	\N
00205e	CASTLE ROCK, INC.	\N
00205f	GAMMADATA COMPUTER GMBH	\N
002060	ALCATEL ITALIA S.p.A.	\N
002061	GarrettCom, Inc.	\N
002062	SCORPION LOGIC, LTD.	\N
002063	WIPRO INFOTECH LTD.	\N
002064	PROTEC MICROSYSTEMS, INC.	\N
002065	SUPERNET NETWORKING INC.	\N
002066	GENERAL MAGIC, INC.	\N
002067	PRIVATE	\N
002068	ISDYNE	\N
002069	ISDN SYSTEMS CORPORATION	\N
00206a	OSAKA COMPUTER CORP.	\N
00206b	KONICA MINOLTA HOLDINGS, INC.	\N
00206c	EVERGREEN TECHNOLOGY CORP.	\N
00206d	DATA RACE, INC.	\N
00206e	XACT, INC.	\N
00206f	FLOWPOINT CORPORATION	\N
002070	HYNET, LTD.	\N
002071	IBR GMBH	\N
002072	WORKLINK INNOVATIONS	\N
002073	FUSION SYSTEMS CORPORATION	\N
002074	SUNGWOON SYSTEMS	\N
002075	MOTOROLA COMMUNICATION ISRAEL	\N
002076	REUDO CORPORATION	\N
002077	KARDIOS SYSTEMS CORP.	\N
002078	RUNTOP, INC.	\N
002079	MIKRON GMBH	\N
00207a	WiSE Communications, Inc.	\N
00207b	Intel Corporation	\N
00207c	AUTEC GmbH	\N
00207d	ADVANCED COMPUTER APPLICATIONS	\N
00207e	FINECOM Co., Ltd.	\N
00207f	KYOEI SANGYO CO., LTD.	\N
002080	SYNERGY (UK) LTD.	\N
002081	TITAN ELECTRONICS	\N
002082	ONEAC CORPORATION	\N
002083	PRESTICOM INCORPORATED	\N
002084	OCE PRINTING SYSTEMS, GMBH	\N
002085	EXIDE ELECTRONICS	\N
002086	MICROTECH ELECTRONICS LIMITED	\N
002087	MEMOTEC, INC.	\N
002088	GLOBAL VILLAGE COMMUNICATION	\N
002089	T3PLUS NETWORKING, INC.	\N
00208a	SONIX COMMUNICATIONS, LTD.	\N
00208b	LAPIS TECHNOLOGIES, INC.	\N
00208c	GALAXY NETWORKS, INC.	\N
00208d	CMD TECHNOLOGY	\N
00208e	CHEVIN SOFTWARE ENG. LTD.	\N
00208f	ECI TELECOM LTD.	\N
002090	ADVANCED COMPRESSION TECHNOLOGY, INC.	\N
002091	J125, NATIONAL SECURITY AGENCY	\N
002092	CHESS ENGINEERING B.V.	\N
002093	LANDINGS TECHNOLOGY CORP.	\N
002094	CUBIX CORPORATION	\N
002095	RIVA ELECTRONICS	\N
002096	Invensys	\N
002097	APPLIED SIGNAL TECHNOLOGY	\N
002098	HECTRONIC AB	\N
002099	BON ELECTRIC CO., LTD.	\N
00209a	THE 3DO COMPANY	\N
00209b	ERSAT ELECTRONIC GMBH	\N
00209c	PRIMARY ACCESS CORP.	\N
00209d	LIPPERT AUTOMATIONSTECHNIK	\N
00209e	BROWN'S OPERATING SYSTEM SERVICES, LTD.	\N
00209f	MERCURY COMPUTER SYSTEMS, INC.	\N
0020a0	OA LABORATORY CO., LTD.	\N
0020a1	DOVATRON	\N
0020a2	GALCOM NETWORKING LTD.	\N
0020a3	DIVICOM INC.	\N
0020a4	MULTIPOINT NETWORKS	\N
0020a5	API ENGINEERING	\N
0020a6	Proxim Wireless	\N
0020a7	PAIRGAIN TECHNOLOGIES, INC.	\N
0020a8	SAST TECHNOLOGY CORP.	\N
0020a9	WHITE HORSE INDUSTRIAL	\N
0020aa	DIGIMEDIA VISION LTD.	\N
0020ab	MICRO INDUSTRIES CORP.	\N
0020ac	INTERFLEX DATENSYSTEME GMBH	\N
0020ad	LINQ SYSTEMS	\N
0020ae	ORNET DATA COMMUNICATION TECH.	\N
0020af	3COM CORPORATION	\N
0020b0	GATEWAY DEVICES, INC.	\N
0020b1	COMTECH RESEARCH INC.	\N
0020b2	GKD Gesellschaft Fur Kommunikation Und Datentechnik	\N
0020b3	SCLTEC COMMUNICATIONS SYSTEMS	\N
0020b4	TERMA ELEKTRONIK AS	\N
0020b5	YASKAWA ELECTRIC CORPORATION	\N
0020b6	AGILE NETWORKS, INC.	\N
0020b7	NAMAQUA COMPUTERWARE	\N
0020b8	PRIME OPTION, INC.	\N
0020b9	METRICOM, INC.	\N
0020ba	CENTER FOR HIGH PERFORMANCE	\N
0020bb	ZAX CORPORATION	\N
0020bc	Long Reach Networks Pty Ltd	\N
0020bd	NIOBRARA R & D CORPORATION	\N
0020be	LAN ACCESS CORP.	\N
0020bf	AEHR TEST SYSTEMS	\N
0020c0	PULSE ELECTRONICS, INC.	\N
0020c1	SAXA, Inc.	\N
0020c2	TEXAS MEMORY SYSTEMS, INC.	\N
0020c3	COUNTER SOLUTIONS LTD.	\N
0020c4	INET,INC.	\N
0020c5	EAGLE TECHNOLOGY	\N
0020c6	NECTEC	\N
0020c7	AKAI Professional M.I. Corp.	\N
0020c8	LARSCOM INCORPORATED	\N
0020c9	VICTRON BV	\N
0020ca	DIGITAL OCEAN	\N
0020cb	PRETEC ELECTRONICS CORP.	\N
0020cc	DIGITAL SERVICES, LTD.	\N
0020cd	HYBRID NETWORKS, INC.	\N
0020ce	LOGICAL DESIGN GROUP, INC.	\N
0020cf	TEST & MEASUREMENT SYSTEMS INC	\N
0020d0	VERSALYNX CORPORATION	\N
0020d1	MICROCOMPUTER SYSTEMS (M) SDN.	\N
0020d2	RAD DATA COMMUNICATIONS, LTD.	\N
0020d3	OST (OUEST STANDARD TELEMATIQU	\N
0020d4	CABLETRON - ZEITTNET INC.	\N
0020d5	VIPA GMBH	\N
0020d6	BREEZECOM	\N
0020d7	JAPAN MINICOMPUTER SYSTEMS CO., Ltd.	\N
0020d8	Nortel Networks	\N
0020d9	PANASONIC TECHNOLOGIES, INC./MIECO-US	\N
0020da	Alcatel North America ESD	\N
0020db	XNET TECHNOLOGY, INC.	\N
0020dc	DENSITRON TAIWAN LTD.	\N
0020dd	Cybertec Pty Ltd	\N
0020de	JAPAN DIGITAL LABORAT'Y CO.LTD	\N
0020df	KYOSAN ELECTRIC MFG. CO., LTD.	\N
0020e0	Actiontec Electronics, Inc.	\N
0020e1	ALAMAR ELECTRONICS	\N
0020e2	INFORMATION RESOURCE ENGINEERING	\N
0020e3	MCD KENCOM CORPORATION	\N
0020e4	HSING TECH ENTERPRISE CO., LTD	\N
0020e5	APEX DATA, INC.	\N
0020e6	LIDKOPING MACHINE TOOLS AB	\N
0020e7	B&W NUCLEAR SERVICE COMPANY	\N
0020e8	DATATREK CORPORATION	\N
0020e9	DANTEL	\N
0020ea	EFFICIENT NETWORKS, INC.	\N
0020eb	CINCINNATI MICROWAVE, INC.	\N
0020ec	TECHWARE SYSTEMS CORP.	\N
0020ed	GIGA-BYTE TECHNOLOGY CO., LTD.	\N
0020ee	GTECH CORPORATION	\N
0020ef	USC CORPORATION	\N
0020f0	UNIVERSAL MICROELECTRONICS CO.	\N
0020f1	ALTOS INDIA LIMITED	\N
0020f2	SUN MICROSYSTEMS, INC.	\N
0020f3	RAYNET CORPORATION	\N
0020f4	SPECTRIX CORPORATION	\N
0020f5	PANDATEL AG	\N
0020f6	NET TEK  AND KARLNET, INC.	\N
0020f7	CYBERDATA	\N
0020f8	CARRERA COMPUTERS, INC.	\N
0020f9	PARALINK NETWORKS, INC.	\N
0020fa	GDE SYSTEMS, INC.	\N
0020fb	OCTEL COMMUNICATIONS CORP.	\N
0020fc	MATROX	\N
0020fd	ITV TECHNOLOGIES, INC.	\N
0020fe	TOPWARE INC. / GRAND COMPUTER	\N
0020ff	SYMMETRICAL TECHNOLOGIES	\N
002100	GemTek Technology Co., Ltd.	\N
002101	Aplicaciones Electronicas Quasar (AEQ)	\N
002102	UpdateLogic Inc.	\N
002103	GHI Electronics, LLC	\N
002104	Gigaset Communications GmbH	\N
002105	Alcatel-Lucent	\N
002106	RIM Testing Services	\N
002107	Seowonintech Co Ltd.	\N
002108	Nokia Danmark A/S	\N
002109	Nokia Danmark A/S	\N
00210a	byd:sign Corporation	\N
00210b	GEMINI TRAZE RFID PVT. LTD.	\N
00210c	Cymtec Systems, Inc.	\N
00210d	SAMSIN INNOTEC	\N
00210e	Orpak Systems L.T.D.	\N
00210f	Cernium Corp	\N
002110	Clearbox Systems	\N
002111	Uniphone Inc.	\N
002112	WISCOM SYSTEM CO.,LTD	\N
002113	Padtec S/A	\N
002114	Hylab Technology Inc.	\N
002115	PHYWE Systeme GmbH & Co. KG	\N
002116	Transcon Electronic Systems, spol. s r. o.	\N
002117	Tellord	\N
002118	Athena Tech, Inc.	\N
002119	Samsung Electro-Mechanics	\N
00211a	LInTech Corporation	\N
00211b	Cisco Systems	\N
00211c	Cisco Systems	\N
00211d	Dataline AB	\N
00211e	Motorola CHS	\N
00211f	SHINSUNG DELTATECH CO.,LTD.	\N
002120	Sequel Technologies, LLC	\N
002121	VRmagic GmbH	\N
002122	Chip-pro Ltd.	\N
002123	Aerosat Avionics	\N
002124	Optos Plc	\N
002125	KUK JE TONG SHIN Co.,LTD	\N
002126	Shenzhen Torch Equipment Co., Ltd.	\N
002127	TP-LINK Technology Co., Ltd.	\N
002128	Sun Microsystems Inc	\N
002129	Cisco-Linksys, LLC	\N
00212a	Audiovox Corporation	\N
00212b	MSA Auer	\N
00212c	SemIndia System Private Limited	\N
00212d	SCIMOLEX CORPORATION	\N
00212e	dresden-elektronik	\N
00212f	Phoebe Micro Inc.	\N
002130	Keico Hightech Inc.	\N
002131	Blynke Inc.	\N
002132	Masterclock, Inc.	\N
002133	Building B, Inc	\N
002134	Brandywine Communications	\N
002135	ALCATEL-LUCENT	\N
002136	Motorola Mobile Devices business (MDb)	\N
002137	Bay Controls, LLC	\N
002138	Cepheid	\N
002139	Escherlogic Inc.	\N
00213a	Winchester Systems Inc.	\N
00213b	Berkshire Products, Inc	\N
00213c	AliphCom	\N
00213d	Cermetek Microelectronics, Inc.	\N
00213e	TomTom	\N
00213f	A-Team Technology Ltd.	\N
002140	EN Technologies Inc.	\N
002141	RADLIVE	\N
002142	Advanced Control Systems doo	\N
002143	Motorola CHS	\N
002144	SS Telecoms	\N
002145	Semptian Technologies Ltd.	\N
002146	SCI Technology	\N
002147	Nintendo Co., Ltd.	\N
002148	Kaco Solar Korea	\N
002149	China Daheng Group ,Inc.	\N
00214a	Pixel Velocity, Inc	\N
00214b	Shenzhen HAMP Science & Technology Co.,Ltd	\N
00214c	SAMSUNG ELECTRONICS CO., LTD.	\N
00214d	Guangzhou Skytone Transmission Technology Com. Ltd.	\N
00214e	GS Yuasa Power Supply Ltd.	\N
00214f	ALPS Electric Co., Ltd	\N
002150	EYEVIEW ELECTRONICS	\N
002151	Millinet Co., Ltd.	\N
002152	General Satellite Trading Limited	\N
002153	SeaMicro Inc.	\N
002154	D-TACQ Solutions Ltd	\N
002155	Cisco Systems	\N
002156	Cisco Systems	\N
002157	National Datacast, Inc.	\N
002158	Style Flying Technology Co.	\N
002159	Juniper Networks	\N
00215a	Hewlett Packard	\N
00215b	Inotive	\N
00215c	Intel Corporate	\N
00215d	Intel Corporate	\N
00215e	IBM	\N
00215f	IHSE GmbH	\N
002160	Hidea Solutions Co. Ltd.	\N
002161	Yournet Inc.	\N
002162	Nortel	\N
002163	ASKEY COMPUTER CORP	\N
002164	Special Design Bureau for Seismic Instrumentation	\N
002165	Presstek Inc.	\N
002166	NovAtel Inc.	\N
002167	HWA JIN T&I Corp.	\N
002168	iVeia, LLC	\N
002169	Prologix, LLC.	\N
00216a	Intel Corporate	\N
00216b	Intel Corporate	\N
00216c	ODVA	\N
00216d	Soltech Co., Ltd.	\N
00216e	Function ATI (Huizhou) Telecommunications Co., Ltd.	\N
00216f	SymCom, Inc.	\N
002170	Dell Inc	\N
002171	Wesung TNC Co., Ltd.	\N
002172	Seoultek Valley	\N
002173	Ion Torrent Systems, Inc.	\N
002174	AvaLAN Wireless	\N
002175	Pacific Satellite International Ltd.	\N
002176	YMax Telecom Ltd.	\N
002177	W. L. Gore & Associates	\N
002178	Matuschek Messtechnik GmbH	\N
002179	IOGEAR, Inc.	\N
00217a	Sejin Electron, Inc.	\N
00217b	Bastec AB	\N
00217c	2Wire	\N
00217d	PYXIS S.R.L.	\N
00217e	Telit Communication s.p.a	\N
00217f	Intraco Technology Pte Ltd	\N
002180	Motorola CHS	\N
002181	Si2 Microsystems Limited	\N
002182	SandLinks Systems, Ltd.	\N
002183	VATECH HYDRO	\N
002184	POWERSOFT SRL	\N
002185	MICRO-STAR INT'L CO.,LTD.	\N
002186	USI	\N
002187	Imacs GmbH	\N
002188	Data Domain, Inc.	\N
002189	AppTech, Inc.	\N
00218a	Electronic Design and Manufacturing Company	\N
00218b	Wescon Technology, Inc.	\N
00218c	TopControl GMBH	\N
00218d	AP Router Ind. Eletronica LTDA	\N
00218e	MEKICS CO., LTD.	\N
00218f	Avantgarde Acoustic Lautsprechersysteme GmbH	\N
002190	Goliath Solutions	\N
002191	D-Link Corporation	\N
002192	Baoding Galaxy Electronic Technology  Co.,Ltd	\N
002193	Videofon MV	\N
002194	Ping Communication	\N
002195	GWD Media Limited	\N
002196	Telsey  S.p.A.	\N
002197	ELITEGROUP COMPUTER SYSTEM	\N
002198	Thai Radio Co, LTD	\N
002199	Vacon Plc	\N
00219a	Cambridge Visual Networks Ltd	\N
00219b	Dell Inc	\N
00219c	Honeywld Technology Corp.	\N
00219d	Adesys BV	\N
00219e	Sony Ericsson Mobile Communications	\N
00219f	SATEL OY	\N
0021a0	Cisco Systems	\N
0021a1	Cisco Systems	\N
0021a2	EKE-Electronics Ltd.	\N
0021a3	Micromint	\N
0021a4	Dbii Networks	\N
0021a5	ERLPhase Power Technologies Ltd.	\N
0021a6	Videotec Spa	\N
0021a7	Hantle System Co., Ltd.	\N
0021a8	Telephonics Corporation	\N
0021a9	Mobilink Telecom Co.,Ltd	\N
0021aa	Nokia Danmark A/S	\N
0021ab	Nokia Danmark A/S	\N
0021ac	Infrared Integrated Systems Ltd	\N
0021ad	Nordic ID Oy	\N
0021ae	ALCATEL-LUCENT FRANCE - WTD	\N
0021af	Radio Frequency Systems	\N
0021b0	Tyco Telecommunications	\N
0021b1	DIGITAL SOLUTIONS LTD	\N
0021b2	Fiberblaze A/S	\N
0021b3	Ross Controls	\N
0021b4	APRO MEDIA CO., LTD	\N
0021b5	Vyro Games Limited	\N
0021b6	Triacta Power Technologies Inc.	\N
0021b7	Lexmark International Inc.	\N
0021b8	Inphi Corporation	\N
0021b9	Universal Devices Inc.	\N
0021ba	Texas Instruments	\N
0021bb	Riken Keiki Co., Ltd.	\N
0021bc	ZALA COMPUTER	\N
0021bd	Nintendo Co., Ltd.	\N
0021be	Cisco, Service Provider Video Technology Group	\N
0021bf	Hitachi High-Tech Control Systems Corporation	\N
0021c0	Mobile Appliance, Inc.	\N
0021c1	ABB Oy / Distribution Automation	\N
0021c2	GL Communications Inc	\N
0021c3	CORNELL Communications, Inc.	\N
0021c4	Consilium AB	\N
0021c5	3DSP Corp	\N
0021c6	CSJ Global, Inc.	\N
0021c7	Russound	\N
0021c8	LOHUIS Networks	\N
0021c9	Wavecom Asia Pacific Limited	\N
0021ca	ART System Co., Ltd.	\N
0021cb	SMS TECNOLOGIA ELETRONICA LTDA	\N
0021cc	Flextronics International	\N
0021cd	LiveTV	\N
0021ce	NTC-Metrotek	\N
0021cf	The Crypto Group	\N
0021d0	Global Display Solutions Spa	\N
0021d1	Samsung Electronics Co.,Ltd	\N
0021d2	Samsung Electronics Co.,Ltd	\N
0021d3	BOCOM SECURITY(ASIA PACIFIC) LIMITED	\N
0021d4	Vollmer Werke GmbH	\N
0021d5	X2E GmbH	\N
0021d6	LXI Consortium	\N
0021d7	Cisco Systems	\N
0021d8	Cisco Systems	\N
0021d9	SEKONIC CORPORATION	\N
0021da	Automation Products Group Inc.	\N
0021db	Santachi Video Technology (Shenzhen) Co., Ltd.	\N
0021dc	TECNOALARM S.r.l.	\N
0021dd	Northstar Systems Corp	\N
0021de	Firepro Wireless	\N
0021df	Martin Christ GmbH	\N
0021e0	CommAgility Ltd	\N
0021e1	Nortel Networks	\N
0021e2	Creative Electronic GmbH	\N
0021e3	SerialTek LLC	\N
0021e4	I-WIN	\N
0021e5	Display Solution AG	\N
0021e6	Starlight Video Limited	\N
0021e7	Informatics Services Corporation	\N
0021e8	Murata Manufacturing Co., Ltd.	\N
0021e9	Apple, Inc	\N
0021ea	Bystronic Laser AG	\N
0021eb	ESP SYSTEMS, LLC	\N
0021ec	Solutronic GmbH	\N
0021ed	Telegesis	\N
0021ee	Full Spectrum Inc.	\N
0021ef	Kapsys	\N
0021f0	EW3 Technologies LLC	\N
0021f1	Tutus Data AB	\N
0021f2	EASY3CALL Technology Limited	\N
0021f3	Si14 SpA	\N
0021f4	INRange Systems, Inc	\N
0021f5	Western Engravers Supply, Inc.	\N
0021f6	Virtual Iron Software	\N
0021f7	ProCurve Networking by HP	\N
0021f8	Enseo, Inc.	\N
0021f9	WIRECOM Technologies	\N
0021fa	A4SP Technologies Ltd.	\N
0021fb	LG Electronics	\N
0021fc	Nokia Danmark A/S	\N
0021fd	DSTA S.L.	\N
0021fe	Nokia Danmark A/S	\N
0021ff	Cyfrowy Polsat SA	\N
002200	BLADE Network Technology	\N
002201	Aksys Networks Inc	\N
002202	Excito Elektronik i Skåne AB	\N
002203	Glensound Electronics Ltd	\N
002204	KORATEK	\N
002205	WeLink Solutions, Inc.	\N
002206	Cyberdyne Inc.	\N
002207	Inteno Broadband Technology AB	\N
002208	Certicom Corp	\N
002209	Omron Healthcare Co., Ltd	\N
00220a	Rearden Labs	\N
00220b	National Source Coding Center	\N
00220c	Cisco Systems	\N
00220d	Cisco Systems	\N
00220e	Indigo Security Co., Ltd.	\N
00220f	MoCA (Multimedia over Coax Alliance)	\N
002210	Motorola CHS	\N
002211	Rohati Systems	\N
002212	CAI Networks, Inc.	\N
002213	PCI CORPORATION	\N
002214	RINNAI KOREA	\N
002215	ASUSTek COMPUTER INC.	\N
002216	SHIBAURA VENDING MACHINE CORPORATION	\N
002217	Neat Electronics	\N
002218	Verivue Inc.	\N
002219	Dell Inc	\N
00221a	Audio Precision	\N
00221b	Morega Systems	\N
00221c	PRIVATE	\N
00221d	Freegene Technology LTD	\N
00221e	Media Devices Co., Ltd.	\N
00221f	eSang Technologies Co., Ltd.	\N
002220	Mitac Technology Corp	\N
002221	ITOH DENKI CO,LTD.	\N
002222	Schaffner Deutschland GmbH 	\N
002223	TimeKeeping Systems, Inc.	\N
002224	Good Will Instrument Co., Ltd.	\N
002225	Thales Avionics Ltd	\N
002226	Avaak, Inc.	\N
002227	uv-electronic GmbH	\N
002228	Breeze Innovations Ltd.	\N
002229	Compumedics Ltd	\N
00222a	SoundEar A/S	\N
00222b	Nucomm, Inc.	\N
00222c	Ceton Corp	\N
00222d	SMC Networks Inc.	\N
00222e	maintech GmbH	\N
00222f	Open Grid Computing, Inc.	\N
002230	FutureLogic Inc.	\N
002231	SMT&C Co., Ltd.	\N
002232	Design Design Technology Ltd	\N
002233	Pirelli Broadband Solutions	\N
002234	Corventis Inc.	\N
002235	Strukton Systems bv	\N
002236	VECTOR SP. Z O.O.	\N
002237	Shinhint Group	\N
002238	LOGIPLUS	\N
002239	Indiana Life Sciences Incorporated	\N
00223a	Scientific Atlanta, Cisco SPVT Group	\N
00223b	Communication Networks, LLC	\N
00223c	RATIO Entwicklungen GmbH	\N
00223d	JumpGen Systems, LLC	\N
00223e	IRTrans GmbH	\N
00223f	Netgear Inc.	\N
002240	Universal Telecom S/A	\N
002241	Apple, Inc	\N
002242	Alacron Inc.	\N
002243	AzureWave Technologies, Inc.	\N
002244	Chengdu Linkon Communications Device Co., Ltd	\N
002245	Leine & Linde AB	\N
002246	Evoc Intelligent Technology Co.,Ltd.	\N
002247	DAC ENGINEERING CO., LTD.	\N
002248	Microsoft Corporation	\N
002249	HOME MULTIENERGY SL	\N
00224a	RAYLASE AG	\N
00224b	AIRTECH TECHNOLOGIES, INC.	\N
00224c	Nintendo Co., Ltd.	\N
00224d	MITAC INTERNATIONAL CORP.	\N
00224e	SEEnergy Corp.	\N
00224f	Byzoro Networks Ltd.	\N
002250	Point Six Wireless, LLC	\N
002251	Lumasense Technologies	\N
002252	ZOLL Lifecor Corporation	\N
002253	Entorian Technologies	\N
002254	Bigelow Aerospace	\N
002255	Cisco Systems	\N
002256	Cisco Systems	\N
002257	3Com Europe Ltd	\N
002258	Taiyo Yuden Co., Ltd.	\N
002259	Guangzhou New Postcom Equipment Co.,Ltd.	\N
00225a	Garde Security AB	\N
00225b	Teradici Corporation	\N
00225c	Multimedia & Communication Technology	\N
00225d	Digicable Network India Pvt. Ltd.	\N
00225e	Uwin Technologies Co.,LTD	\N
00225f	Liteon Technology Corporation	\N
002260	AFREEY Inc.	\N
002261	Frontier Silicon Ltd	\N
002262	BEP Marine	\N
002263	Koos Technical Services, Inc.	\N
002264	Hewlett Packard	\N
002265	Nokia Danmark A/S	\N
002266	Nokia Danmark A/S	\N
002267	Nortel Networks	\N
002268	Hon Hai Precision Ind. Co., Ltd.	\N
002269	Hon Hai Precision Ind. Co., Ltd.	\N
00226a	Honeywell	\N
00226b	Cisco-Linksys, LLC	\N
00226c	LinkSprite Technologies, Inc.	\N
00226d	Shenzhen GIEC Electronics Co., Ltd.	\N
00226e	Gowell Electronic Limited	\N
00226f	3onedata Technology Co. Ltd.	\N
002270	ABK North America, LLC	\N
002271	Jäger Computergesteuerte Messtechnik GmbH	\N
002272	American Micro-Fuel Device Corp.	\N
002273	Techway	\N
002274	FamilyPhone AB	\N
002275	Belkin International, Inc.	\N
002276	Triple EYE B.V.	\N
002277	NEC Australia Pty Ltd	\N
002278	Shenzhen  Tongfang Multimedia  Technology Co.,Ltd.	\N
002279	Nippon Conlux Co., Ltd.	\N
00227a	Telecom Design	\N
00227b	Apogee Labs, Inc.	\N
00227c	Woori SMT Co.,ltd	\N
00227d	YE DATA INC.	\N
00227e	Chengdu 30Kaitian Communication Industry Co.Ltd	\N
00227f	Ruckus Wireless	\N
002280	A2B Electronics AB	\N
002281	Daintree Networks Inc	\N
002282	8086 Limited	\N
002283	Juniper Networks	\N
002284	DESAY A&V SCIENCE AND TECHNOLOGY CO.,LTD	\N
002285	NOMUS COMM SYSTEMS	\N
002286	ASTRON	\N
002287	Titan Wireless LLC	\N
002288	Sagrad, Inc.	\N
002289	Optosecurity Inc.	\N
00228a	Teratronik elektronische systeme gmbh	\N
00228b	Kensington Computer Products Group	\N
00228c	Photon Europe GmbH	\N
00228d	GBS Laboratories LLC	\N
00228e	TV-NUMERIC	\N
00228f	CNRS	\N
002290	Cisco Systems	\N
002291	Cisco Systems	\N
002292	Cinetal	\N
002293	ZTE Corporation	\N
002294	Kyocera Corporation	\N
002295	SGM Technology for lighting spa	\N
002296	LinoWave Corporation	\N
002297	XMOS Semiconductor	\N
002298	Sony Ericsson Mobile Communications	\N
002299	SeaMicro Inc.	\N
00229a	Lastar, Inc.	\N
00229b	AverLogic Technologies, Inc.	\N
00229c	Verismo Networks Inc	\N
00229d	PYUNG-HWA IND.CO.,LTD	\N
00229e	Social Aid Research Co., Ltd.	\N
00229f	Sensys Traffic AB	\N
0022a0	Delphi Corporation	\N
0022a1	Huawei Symantec Technologies Co.,Ltd.	\N
0022a2	Xtramus Technologies	\N
0022a3	California Eastern Laboratories	\N
0022a4	2Wire	\N
0022a5	Texas Instruments	\N
0022a6	Sony Computer Entertainment America	\N
0022a7	Tyco Electronics AMP GmbH	\N
0022a8	Ouman Finland Oy	\N
0022a9	LG Electronics Inc	\N
0022aa	Nintendo Co., Ltd.	\N
0022ab	Shenzhen Turbosight Technology Ltd	\N
0022ac	Hangzhou Siyuan Tech. Co., Ltd	\N
0022ad	TELESIS TECHNOLOGIES, INC.	\N
0022ae	Mattel Inc.	\N
0022af	Safety Vision	\N
0022b0	D-Link Corporation	\N
0022b1	Elbit Systems	\N
0022b2	4RF Communications Ltd	\N
0022b3	Sei S.p.A.	\N
0022b4	Motorola Mobile Devices	\N
0022b5	NOVITA	\N
0022b6	Superflow Technologies Group	\N
0022b7	GSS Grundig SAT-Systems GmbH	\N
0022b8	Norcott	\N
0022b9	Analogix Seminconductor, Inc	\N
0022ba	HUTH Elektronik Systeme GmbH	\N
0022bb	beyerdynamic GmbH & Co. KG	\N
0022bc	JDSU France SAS	\N
0022bd	Cisco Systems	\N
0022be	Cisco Systems	\N
0022bf	SieAmp Group of Companies	\N
0022c0	Shenzhen Forcelink Electronic Co, Ltd	\N
0022c1	Active Storage Inc.	\N
0022c2	Proview Eletronica do Brasil LTDA	\N
0022c3	Zeeport Technology Inc.	\N
0022c4	epro GmbH	\N
0022c5	INFORSON Co,Ltd.	\N
0022c6	Sutus Inc	\N
0022c7	SEGGER Microcontroller GmbH & Co. KG	\N
0022c8	Applied Instruments	\N
0022c9	Lenord, Bauer & Co GmbH	\N
0022ca	Anviz Biometric Tech. Co., Ltd.	\N
0022cb	IONODES Inc.	\N
0022cc	SciLog, Inc.	\N
0022cd	Ared Technology Co., Ltd.	\N
0022ce	Cisco, Service Provider Video Technology Group	\N
0022cf	PLANEX Communications INC	\N
0022d0	Polar Electro Oy	\N
0022d1	Albrecht Jung GmbH & Co. KG	\N
0022d2	All Earth Comércio de Eletrônicos LTDA.	\N
0022d3	Hub-Tech	\N
0022d4	ComWorth Co., Ltd.	\N
0022d5	Eaton Corp. Electrical Group Data Center Solutions - Pulizzi	\N
0022d6	Cypak AB	\N
0022d7	Nintendo Co., Ltd.	\N
0022d8	Shenzhen GST Security and Safety Technology Limited	\N
0022d9	Fortex Industrial Ltd.	\N
0022da	ANATEK, LLC	\N
0022db	Translogic Corporation	\N
0022dc	Vigil Health Solutions Inc.	\N
0022dd	Protecta Electronics Ltd	\N
0022de	OPPO Digital, Inc.	\N
0022df	TAMUZ Monitors	\N
0022e0	Atlantic Software Technologies S.r.L.	\N
0022e1	ZORT Labs, LLC.	\N
0022e2	WABTEC Transit Division	\N
0022e3	Amerigon	\N
0022e4	APASS TECHNOLOGY CO., LTD.	\N
0022e5	Fisher-Rosemount Systems Inc.	\N
0022e6	Intelligent Data	\N
0022e7	WPS Parking Systems	\N
0022e8	Applition Co., Ltd.	\N
0022e9	ProVision Communications	\N
0022ea	Rustelcom Inc.	\N
0022eb	Data Respons A/S	\N
0022ec	IDEALBT TECHNOLOGY CORPORATION	\N
0022ed	TSI Power Corporation	\N
0022ee	Algo Communication Products Ltd	\N
0022ef	Ibis Tek, LLC	\N
0022f0	3 Greens Aviation Limited	\N
0022f1	PRIVATE	\N
0022f2	SunPower Corp	\N
0022f3	SHARP CORPORATION	\N
0022f4	AMPAK Technology, Inc.	\N
0022f5	Advanced Realtime Tracking GmbH	\N
0022f6	Syracuse Research Corporation	\N
0022f7	Conceptronic	\N
0022f8	PIMA Electronic Systems Ltd.	\N
0022f9	Pollin Electronic GmbH	\N
0022fa	Intel Corporate	\N
0022fb	Intel Corporate	\N
0022fc	Nokia Danmark A/S	\N
0022fd	Nokia Danmark A/S	\N
0022fe	Microprocessor Designs Inc	\N
0022ff	NIVIS LLC	\N
002300	Cayee Computer Ltd.	\N
002301	Witron Technology Limited	\N
002302	Cobalt Digital, Inc.	\N
002303	LITE-ON IT Corporation	\N
002304	Cisco Systems	\N
002305	Cisco Systems	\N
002306	ALPS Electric Co., Ltd	\N
002307	FUTURE INNOVATION TECH CO.,LTD	\N
002308	Arcadyan Technology Corporation	\N
002309	Janam Technologies LLC	\N
00230a	ARBURG GmbH & Co KG	\N
00230b	Motorola CHS	\N
00230c	CLOVER ELECTRONICS CO.,LTD.	\N
00230d	Nortel Networks	\N
00230e	Gorba AG	\N
00230f	Hirsch Electronics Corporation	\N
002310	LNC Technology Co., Ltd.	\N
002311	Gloscom Co., Ltd.	\N
002312	Apple, Inc	\N
002313	Qool Technologies Ltd.	\N
002314	Intel Corporate	\N
002315	Intel Corporate	\N
002316	KISAN ELECTRONICS CO	\N
002317	Lasercraft Inc	\N
002318	Toshiba	\N
002319	Sielox LLC	\N
00231a	ITF Co., Ltd.	\N
00231b	Danaher Motion - Kollmorgen	\N
00231c	Fourier Systems Ltd.	\N
00231d	Deltacom Electronics Ltd	\N
00231e	Cezzer Multimedia Technologies	\N
00231f	Guangda Electronic & Telecommunication Technology Development Co., Ltd.	\N
002320	Nicira Networks	\N
002321	Avitech International Corp	\N
002322	KISS Teknical Solutions, Inc.	\N
002323	Zylin AS	\N
002324	G-PRO COMPUTER	\N
002325	IOLAN Holding	\N
002326	Fujitsu Limited	\N
002327	Shouyo Electronics CO., LTD	\N
002328	ALCON TELECOMMUNICATIONS CO., LTD.	\N
002329	DDRdrive LLC	\N
00232a	eonas IT-Beratung und -Entwicklung GmbH	\N
00232b	IRD A/S	\N
00232c	Senticare	\N
00232d	SandForce	\N
00232e	Kedah Electronics Engineering, LLC	\N
00232f	Advanced Card Systems Ltd.	\N
002330	DIZIPIA, INC.	\N
002331	Nintendo Co., Ltd.	\N
002332	Apple, Inc	\N
002333	Cisco Systems	\N
002334	Cisco Systems	\N
002335	Linkflex Co.,Ltd	\N
002336	METEL s.r.o.	\N
002337	Global Star Solutions ULC	\N
002338	OJ-Electronics A/S	\N
002339	Samsung Electronics	\N
00233a	Samsung Electronics Co.,Ltd	\N
00233b	C-Matic Systems Ltd	\N
00233c	Alflex	\N
00233d	novero GmbH	\N
00233e	Alcatel-Lucent-IPD	\N
00233f	Purechoice Inc	\N
002340	MiX Telematics	\N
002341	Siemens Building Technologies Fire & Security Products GmbH & Co. oHG	\N
002342	Coffee Equipment Company	\N
002343	TEM AG	\N
002344	Objective Interface Systems	\N
002345	Sony Ericsson Mobile Communications	\N
002346	Vestac	\N
002347	ProCurve Networking by HP	\N
002348	SAGEM COMMUNICATION	\N
002349	Helmholtz Centre Berlin for Material and Energy	\N
00234a	PRIVATE	\N
00234b	Inyuan Technology Inc.	\N
00234c	KTC AB	\N
00234d	Hon Hai Precision Ind. Co., Ltd.	\N
00234e	Hon Hai Precision Ind. Co., Ltd.	\N
00234f	Luminous Power Technologies Pvt. Ltd.	\N
002350	LynTec	\N
002351	2Wire	\N
002352	DATASENSOR S.p.A.	\N
002353	F E T Elettronica snc	\N
002354	ASUSTek COMPUTER INC.	\N
002355	Kinco Automation(Shanghai) Ltd.	\N
002356	Packet Forensics LLC	\N
002357	Pitronot Technologies and Engineering P.T.E. Ltd.	\N
002358	SYSTEL SA	\N
002359	Benchmark Electronics ( Thailand ) Public Company Limited	\N
00235a	COMPAL INFORMATION (KUNSHAN) CO., Ltd.	\N
00235b	Gulfstream	\N
00235c	Aprius, Inc.	\N
00235d	Cisco Systems	\N
00235e	Cisco Systems	\N
00235f	Silicon Micro Sensors GmbH	\N
002360	Lookit Technology Co., Ltd	\N
002361	Unigen Corporation	\N
002362	Goldline Controls	\N
002363	Zhuhai RaySharp Technology Co., Ltd.	\N
002364	Power Instruments Pte Ltd	\N
002365	ELKA-Elektronik GmbH	\N
002366	Beijing Siasun Electronic System Co.,Ltd.	\N
002367	UniControls a.s.	\N
002368	Motorola	\N
002369	Cisco-Linksys, LLC	\N
00236a	ClearAccess, Inc.	\N
00236b	Xembedded, Inc.	\N
00236c	Apple, Inc	\N
00236d	ResMed Ltd	\N
00236e	Burster GmbH & Co KG	\N
00236f	DAQ System	\N
002370	PRO-BEL LIMITED	\N
002371	SOAM Systel	\N
002372	MORE STAR INDUSTRIAL GROUP LIMITED	\N
002373	GridIron Systems, Inc.	\N
002374	Motorola CHS	\N
002375	Motorola CHS	\N
002376	HTC Corporation	\N
002377	Isotek Electronics Ltd	\N
002378	GN Netcom A/S	\N
002379	Union Business Machines Co. Ltd.	\N
00237a	RIM	\N
00237b	WHDI LLC	\N
00237c	NEOTION	\N
00237d	Hewlett Packard	\N
00237e	ELSTER GMBH	\N
00237f	PLANTRONICS	\N
002380	Nanoteq	\N
002381	Lengda Technology(Xiamen) Co.,Ltd.	\N
002382	Lih Rong Electronic Enterprise Co., Ltd.	\N
002383	InMage Systems Inc	\N
002384	GGH Engineering s.r.l.	\N
002385	ANTIPODE	\N
002386	Tour & Andersson AB	\N
002387	ThinkFlood, Inc.	\N
002388	V.T. Telematica S.p.a.	\N
002389	HANGZHOU H3C Technologies Co., Ltd.	\N
00238a	Ciena Corporation	\N
00238b	Quanta Computer Inc.	\N
00238c	PRIVATE	\N
00238d	Techno Design Co., Ltd.	\N
00238e	PIRELLI BROADBAND SOLUTIONS	\N
00238f	NIDEC COPAL CORPORATION	\N
002390	Algolware Corporation	\N
002391	Maxian	\N
002392	Proteus Industries Inc.	\N
002393	AJINEXTEK	\N
002394	Samjeon	\N
002395	Motorola CHS	\N
002396	ANDES TECHNOLOGY CORPORATION	\N
002397	Westell Technologies Inc.	\N
002398	Sky Control	\N
002399	VD Division, Samsung Electronics Co.	\N
00239a	EasyData Software GmbH	\N
00239b	Elster Integrated Solutions	\N
00239c	Juniper Networks	\N
00239d	Mapower Electronics Co., Ltd	\N
00239e	Jiangsu Lemote Technology Corporation Limited	\N
00239f	Institut für Prüftechnik	\N
0023a0	Hana CNS Co., LTD.	\N
0023a1	Trend Electronics Ltd	\N
0023a2	Motorola CHS	\N
0023a3	Motorola CHS	\N
0023a4	New Concepts Development Corp.	\N
0023a5	SageTV, LLC	\N
0023a6	E-Mon	\N
0023a7	Redpine Signals, Inc.	\N
0023a8	Marshall Electronics	\N
0023a9	Beijing Detianquan Electromechanical Equipment Co., Ltd	\N
0023aa	HFR, Inc.	\N
0023ab	Cisco Systems	\N
0023ac	Cisco Systems	\N
0023ad	Xmark Corporation	\N
0023ae	Dell Inc.	\N
0023af	Motorola Mobile Devices	\N
0023b0	COMXION Technology Inc.	\N
0023b1	Longcheer Technology (Singapore) Pte Ltd	\N
0023b2	Intelligent Mechatronic Systems Inc	\N
0023b3	Lyyn AB	\N
0023b4	Nokia Danmark A/S	\N
0023b5	ORTANA LTD	\N
0023b6	SECURITE COMMUNICATIONS / HONEYWELL	\N
0023b7	Q-Light Co., Ltd.	\N
0023b8	Sichuan Jiuzhou Electronic Technology Co.,Ltd	\N
0023b9	EADS Deutschland GmbH	\N
0023ba	Chroma	\N
0023bb	Schmitt Industries	\N
0023bc	EQ-SYS GmbH	\N
0023bd	Digital Ally, Inc.	\N
0023be	Cisco SPVTG	\N
0023bf	Mainpine, Inc.	\N
0023c0	Broadway Networks	\N
0023c1	Securitas Direct AB	\N
0023c2	SAMSUNG Electronics. Co. LTD	\N
0023c3	LogMeIn, Inc.	\N
0023c4	Lux Lumen	\N
0023c5	Radiation Safety and Control Services Inc	\N
0023c6	SMC Corporation	\N
0023c7	AVSystem	\N
0023c8	TEAM-R	\N
0023c9	Sichuan Tianyi Information Science & Technology Stock CO.,LTD	\N
0023ca	Behind The Set, LLC	\N
0023cb	Shenzhen Full-join Technology Co.,Ltd	\N
0023cc	Nintendo Co., Ltd.	\N
0023cd	TP-LINK TECHNOLOGIES CO., LTD.	\N
0023ce	KITA DENSHI CORPORATION	\N
0023cf	CUMMINS-ALLISON CORP.	\N
0023d0	Uniloc USA Inc.	\N
0023d1	TRG	\N
0023d2	Inhand Electronics, Inc.	\N
0023d3	AirLink WiFi Networking Corp.	\N
0023d4	Texas Instruments	\N
0023d5	WAREMA electronic GmbH	\N
0023d6	Samsung Electronics Co.,LTD	\N
0023d7	Samsung Electronics	\N
0023d8	Ball-It Oy	\N
0023d9	Banner Engineering	\N
0023da	Industrial Computer Source (Deutschland)GmbH	\N
0023db	saxnet gmbh	\N
0023dc	Benein, Inc	\N
0023dd	ELGIN S.A.	\N
0023de	Ansync Inc.	\N
0023df	Apple, Inc	\N
0023e0	INO Therapeutics LLC	\N
0023e1	Cavena Image Products AB	\N
0023e2	SEA Signalisation	\N
0023e3	Microtronic AG	\N
0023e4	IPnect co. ltd.	\N
0023e5	IPaXiom Networks	\N
0023e6	Pirkus, Inc.	\N
0023e7	Hinke A/S	\N
0023e8	Demco Corp.	\N
0023e9	F5 Networks, Inc.	\N
0023ea	Cisco Systems	\N
0023eb	Cisco Systems	\N
0023ec	Algorithmix GmbH	\N
0023ed	Motorola CHS	\N
0023ee	Motorola CHS	\N
0023ef	Zuend Systemtechnik AG	\N
0023f0	Shanghai Jinghan Weighing Apparatus Co. Ltd.	\N
0023f1	Sony Ericsson Mobile Communications	\N
0023f2	TVLogic	\N
0023f3	Glocom, Inc.	\N
0023f4	Masternaut	\N
0023f5	WILO SE	\N
0023f6	Softwell Technology Co., Ltd.	\N
0023f7	PRIVATE	\N
0023f8	ZyXEL Communications Corporation	\N
0023f9	Double-Take Software, INC.	\N
0023fa	RG Nets, Inc.	\N
0023fb	IP Datatel, Inc.	\N
0023fc	Ultra Stereo Labs, Inc	\N
0023fd	AFT Atlas Fahrzeugtechnik GmbH	\N
0023fe	Biodevices, SA	\N
0023ff	Beijing HTTC Technology Ltd.	\N
002400	Nortel Networks	\N
002401	D-Link Corporation	\N
002402	Op-Tection GmbH	\N
002403	Nokia Danmark A/S	\N
002404	Nokia Danmark A/S	\N
002405	Dilog Nordic AB	\N
002406	Pointmobile	\N
002407	TELEM SAS	\N
002408	Pacific Biosciences	\N
002409	The Toro Company	\N
00240a	US Beverage Net	\N
00240b	Virtual Computer Inc.	\N
00240c	DELEC GmbH	\N
00240d	OnePath Networks LTD.	\N
00240e	Inventec Besta Co., Ltd.	\N
00240f	Ishii Tool & Engineering Corporation	\N
002410	NUETEQ Technology,Inc.	\N
002411	PharmaSmart LLC	\N
002412	Benign Technologies Co, Ltd.	\N
002413	Cisco Systems	\N
002414	Cisco Systems	\N
002415	Magnetic Autocontrol GmbH	\N
002416	Any Use	\N
002417	Thomson Telecom Belgium	\N
002418	Nextwave Semiconductor	\N
002419	PRIVATE	\N
00241a	Red Beetle Inc.	\N
00241b	iWOW Communications Pte Ltd	\N
00241c	FuGang Electronic (DG) Co.,Ltd	\N
00241d	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
00241e	Nintendo Co., Ltd.	\N
00241f	DCT-Delta GmbH	\N
002420	NetUP Inc.	\N
002421	MICRO-STAR INT'L CO., LTD.	\N
002422	Knapp Logistik Automation GmbH	\N
002423	AzureWave Technologies (Shanghai) Inc.	\N
002424	Axis Network Technology	\N
002425	Shenzhenshi chuangzhicheng Technology Co.,Ltd	\N
002426	NOHMI BOSAI LTD.	\N
002427	SSI COMPUTER CORP	\N
002428	EnergyICT	\N
002429	MK MASTER INC.	\N
00242a	Hittite Microwave Corporation	\N
00242b	Hon Hai Precision Ind.Co.,Ltd.	\N
00242c	Hon Hai Precision Ind. Co., Ltd.	\N
00242e	Datastrip Inc.	\N
00242f	VirtenSys Inc	\N
002430	Ruby Tech Corp.	\N
002431	Uni-v co.,ltd	\N
002432	Neostar Technology Co.,LTD	\N
002433	Alps Electric Co., Ltd	\N
002434	Lectrosonics, Inc.	\N
002435	WIDE CORPORATION	\N
002436	Apple, Inc	\N
002437	Motorola - BSG	\N
002438	Brocade Communications Systems, Inc	\N
002439	Essential Viewing Systems Limited	\N
00243a	Ludl Electronic Products	\N
00243b	CSSI (S) Pte Ltd	\N
00243c	S.A.A.A.	\N
00243d	Emerson Appliance Motors and Controls	\N
00243f	Storwize, Inc.	\N
002440	Halo Monitoring, Inc.	\N
002441	Wanzl Metallwarenfabrik GmbH	\N
002442	Axona Limited	\N
002443	Nortel Networks	\N
002444	Nintendo Co., Ltd.	\N
002445	LiquidxStream Systems Inc.	\N
002446	MMB Research Inc.	\N
002447	Kaztek Systems	\N
002448	SpiderCloud Wireless, Inc	\N
002449	Shen Zhen Lite Star Electronics Technology Co., Ltd	\N
00244a	Voyant International	\N
00244b	PERCEPTRON INC	\N
00244c	Solartron Metrology Ltd	\N
00244d	Hokkaido Electronics Corporation	\N
00244e	RadChips, Inc.	\N
00244f	Asantron Technologies Ltd.	\N
002450	Cisco Systems	\N
002451	Cisco Systems	\N
002452	Silicon Software GmbH	\N
002453	Initra d.o.o.	\N
002454	Samsung Electronics Co., LTD	\N
002455	MuLogic BV	\N
002456	2Wire	\N
002458	PA Bastion CC	\N
002459	ABB STOTZ-KONTAKT GmbH	\N
00245a	Nanjing Panda Electronics Company Limited	\N
00245b	RAIDON TECHNOLOGY, INC.	\N
00245c	Design-Com Technologies Pty. Ltd.	\N
00245d	Terberg besturingstechniek B.V.	\N
00245e	Hivision Co.,ltd	\N
00245f	Vine Telecom CO.,Ltd.	\N
002460	Giaval Science Development Co. Ltd.	\N
002461	Shin Wang Tech.	\N
002462	Rayzone Corporation	\N
002463	Phybridge Inc	\N
002464	Bridge Technologies Co AS	\N
002465	Elentec	\N
002466	Unitron nv	\N
002467	AOC International (Europe) GmbH	\N
002468	Sumavision Technologies Co.,Ltd	\N
002469	Smart Doorphones	\N
00246a	Solid Year Co., Ltd.	\N
00246b	Coventive	\N
00246c	ARUBA NETWORKS, INC.	\N
00246d	Weinzierl Engineering GmbH	\N
00246e	Phihong USA Corp.	\N
00246f	Onda Communication spa	\N
002470	AUROTECH ultrasound AS.	\N
002471	Fusion MultiSystems dba Fusion-io	\N
002472	ReDriven Power Inc.	\N
002473	3Com Europe Ltd	\N
002474	Autronica Fire And Securirty	\N
002475	Compass System(Embedded Dept.)	\N
002476	TAP.tv	\N
002477	Tibbo Technology	\N
002478	Mag Tech Electronics Co Limited	\N
002479	Optec Displays, Inc.	\N
00247a	FU YI CHENG Technology Co., Ltd.	\N
00247b	Actiontec Electronics, Inc	\N
00247c	Nokia Danmark A/S	\N
00247d	Nokia Danmark A/S	\N
00247e	USI	\N
00247f	Nortel Networks	\N
002480	Meteocontrol GmbH	\N
002481	Hewlett Packard	\N
002482	Ruckus Wireless	\N
002483	LG Electronics	\N
002484	Bang and Olufsen Medicom a/s	\N
002485	ConteXtream Ltd	\N
002486	DesignArt Networks	\N
002487	Blackboard Inc.	\N
002488	Centre For Development Of Telematics	\N
002489	Vodafone Omnitel N.V.	\N
00248a	Kaga Electronics Co., Ltd.	\N
00248b	HYBUS CO., LTD.	\N
00248c	ASUSTek COMPUTER INC.	\N
00248d	Sony Computer Entertainment Inc.	\N
00248e	Infoware ZRt.	\N
00248f	DO-MONIX	\N
002490	Samsung Electronics Co.,LTD	\N
002491	Samsung Electronics	\N
002492	Motorola, Broadband Solutions Group	\N
002493	Motorola, Inc	\N
002494	Shenzhen Baoxin Tech CO., Ltd.	\N
002495	Motorola Mobile Devices	\N
002496	Ginzinger electronic systems	\N
002497	Cisco Systems	\N
002498	Cisco Systems	\N
002499	Aquila Technologies	\N
00249a	Beijing Zhongchuang Telecommunication Test Co., Ltd.	\N
00249b	Action Star Enterprise Co., Ltd.	\N
00249c	Bimeng Comunication System Co. Ltd	\N
00249d	NES Technology Inc.	\N
00249e	ADC-Elektronik GmbH	\N
00249f	RIM Testing Services	\N
0024a0	Motorola CHS	\N
0024a1	Motorola CHS	\N
0024a2	Hong Kong Middleware Technology Limited	\N
0024a3	Sonim Technologies Inc	\N
0024a4	Siklu Communication	\N
0024a5	Buffalo Inc.	\N
0024a6	TELESTAR DIGITAL GmbH	\N
0024a7	Advanced Video Communications Inc.	\N
0024a8	ProCurve Networking by HP	\N
0024a9	Ag Leader Technology	\N
0024aa	Dycor Technologies Ltd.	\N
0024ab	A7 Engineering, Inc.	\N
0024ac	Hangzhou DPtech Technologies Co., Ltd.	\N
0024ad	Adolf Thies Gmbh & Co. KG	\N
0024ae	SAGEM SECURITE	\N
0024af	EchoStar Technologies	\N
0024b0	ESAB AB	\N
0024b1	Coulomb Technologies	\N
0024b2	Netgear	\N
0024b3	Graf-Syteco GmbH & Co. KG	\N
0024b4	ESCATRONIC GmbH	\N
0024b5	Nortel Networks	\N
0024b6	Seagate Technology	\N
0024b7	GridPoint, Inc.	\N
0024b8	free alliance sdn bhd	\N
0024b9	Wuhan Higheasy Electronic Technology Development Co.Ltd	\N
0024ba	Texas Instruments	\N
0024bb	CENTRAL Corporation	\N
0024bc	HuRob Co.,Ltd	\N
0024bd	Hainzl Industriesysteme GmbH	\N
0024be	Sony Corporation	\N
0024bf	CIAT	\N
0024c0	NTI COMODO INC	\N
0024c1	Hangzhou Motorola Technologies LTD.	\N
0024c2	Asumo Co.,Ltd.	\N
0024c3	Cisco Systems	\N
0024c4	Cisco Systems	\N
0024c5	Meridian Audio Limited	\N
0024c6	Hager Electro SAS	\N
0024c7	Mobilarm Ltd	\N
0024c8	Broadband Solutions Group	\N
0024c9	Broadband Solutions Group	\N
0024ca	Tobii Technology AB	\N
0024cb	Autonet Mobile	\N
0024cc	Fascinations Toys and Gifts, Inc.	\N
0024cd	Willow Garage, Inc.	\N
0024ce	Exeltech Inc	\N
0024cf	Inscape Data Corporation	\N
0024d0	Shenzhen SOGOOD Industry CO.,LTD.	\N
0024d1	Thomson Inc.	\N
0024d2	Askey Computer	\N
0024d3	QUALICA Inc.	\N
0024d4	FREEBOX SA	\N
0024d5	Winward Industrial Limited	\N
0024d6	Intel Corporate	\N
0024d7	Intel Corporate	\N
0024d8	IlSung Precision	\N
0024d9	BICOM, Inc.	\N
0024da	Innovar Systems Limited	\N
0024db	Alcohol Monitoring Systems	\N
0024dc	Juniper Networks	\N
0024dd	Centrak, Inc.	\N
0024de	GLOBAL Technology Inc.	\N
0024df	Digitalbox Europe GmbH	\N
0024e0	DS Tech, LLC	\N
0024e1	Convey Computer Corp.	\N
0024e2	HASEGAWA ELECTRIC CO.,LTD.	\N
0024e3	CAO Group	\N
0024e4	Withings	\N
0024e5	Seer Technology, Inc	\N
0024e6	In Motion Technology Inc.	\N
0024e7	Plaster Networks	\N
0024e8	Dell Inc.	\N
0024e9	Samsung Electronics Co., Ltd., Storage System Division	\N
0024ea	iris-GmbH infrared & intelligent sensors	\N
0024eb	ClearPath Networks, Inc.	\N
0024ec	United Information Technology Co.,Ltd.	\N
0024ed	YT Elec. Co,.Ltd.	\N
0024ee	Wynmax Inc.	\N
0024ef	Sony Ericsson Mobile Communications	\N
0024f0	Seanodes	\N
0024f1	Shenzhen Fanhai Sanjiang Electronics Co., Ltd.	\N
0024f2	Uniphone Telecommunication Co., Ltd.	\N
0024f3	Nintendo Co., Ltd.	\N
0024f4	Kaminario Technologies Ltd.	\N
0024f5	NDS Surgical Imaging	\N
0024f6	MIYOSHI ELECTRONICS CORPORATION	\N
0024f7	Cisco Systems	\N
0024f8	Technical Solutions Company Ltd.	\N
0024f9	Cisco Systems	\N
0024fa	Hilger u. Kern GMBH	\N
0024fb	PRIVATE	\N
0024fc	QuoPin Co., Ltd.	\N
0024fd	Prosilient Technologies AB	\N
0024fe	AVM GmbH	\N
0024ff	QLogic Corporation	\N
002500	Apple, Inc	\N
002501	JSC "Supertel"	\N
002502	NaturalPoint	\N
002503	BLADE Network Technology	\N
002504	Valiant Communications Limited	\N
002505	eks Engel GmbH & Co. KG	\N
002506	A.I. ANTITACCHEGGIO ITALIA SRL	\N
002507	ASTAK Inc.	\N
002508	Maquet Cardiopulmonary AG	\N
002509	SHARETRONIC Group LTD	\N
00250a	Security Expert Co. Ltd	\N
00250b	CENTROFACTOR  INC	\N
00250c	Enertrac	\N
00250d	GZT Telkom-Telmor sp. z o.o.	\N
00250e	gt german telematics gmbh	\N
00250f	On-Ramp Wireless, Inc.	\N
002510	Pico-Tesla Magnetic Therapies	\N
002511	ELITEGROUP COMPUTER SYSTEM CO., LTD.	\N
002512	ZTE Corporation	\N
002513	CXP DIGITAL BV	\N
002514	PC Worth Int'l Co., Ltd.	\N
002515	SFR	\N
002516	Integrated Design Tools, Inc.	\N
002517	Venntis, LLC	\N
002518	Power PLUS Communications AG	\N
002519	Viaas Inc	\N
00251a	Psiber Data Systems Inc.	\N
00251b	Philips CareServant	\N
00251c	EDT	\N
00251d	DSA Encore, LLC	\N
00251e	ROTEL TECHNOLOGIES	\N
00251f	ZYNUS VISION INC.	\N
002520	SMA Railway Technology GmbH	\N
002521	Logitek Electronic Systems, Inc.	\N
002522	ASRock Incorporation	\N
002523	OCP Inc.	\N
002524	Lightcomm Technology Co., Ltd	\N
002525	CTERA Networks Ltd.	\N
002526	Genuine Technologies Co., Ltd.	\N
002527	Bitrode Corp.	\N
002528	Daido Signal Co., Ltd.	\N
002529	COMELIT GROUP S.P.A	\N
00252a	Chengdu GeeYa Technology Co.,LTD	\N
00252b	Stirling Energy Systems	\N
00252c	Entourage Systems, Inc.	\N
00252d	Kiryung Electronics	\N
00252e	Cisco SPVTG	\N
00252f	Energy, Inc.	\N
002530	Aetas Systems Inc.	\N
002531	Cloud Engines, Inc.	\N
002532	Digital Recorders	\N
002533	WITTENSTEIN AG	\N
002535	Minimax GmbH & Co KG	\N
002536	Oki Electric Industry Co., Ltd.	\N
002537	Runcom Technologies Ltd.	\N
002538	Samsung Electronics Co., Ltd., Memory Division	\N
002539	IfTA GmbH	\N
00253a	CEVA, Ltd.	\N
00253b	din Dietmar Nocker Facilitymanagement GmbH	\N
00253c	2Wire	\N
00253d	DRS Consolidated Controls	\N
00253e	Sensus Metering Systems	\N
002540	Quasar Technologies, Inc.	\N
002541	Maquet Critical Care AB	\N
002542	Pittasoft	\N
002543	MONEYTECH	\N
002544	LoJack Corporation	\N
002545	Cisco Systems	\N
002546	Cisco Systems	\N
002547	Nokia Danmark A/S	\N
002548	Nokia Danmark A/S	\N
002549	Jeorich Tech. Co.,Ltd.	\N
00254a	RingCube Technologies, Inc.	\N
00254b	Apple, Inc	\N
00254c	Videon Central, Inc.	\N
00254d	Singapore Technologies Electronics Limited	\N
00254e	Vertex Wireless Co., Ltd.	\N
00254f	ELETTROLAB Srl	\N
002550	Riverbed Technology	\N
002551	SE-Elektronic GmbH	\N
002552	VXI CORPORATION	\N
002553	PIRELLI BROADBAND SOLUTIONS	\N
002554	Pixel8 Networks	\N
002555	Visonic Technologies 1993 Ltd	\N
002556	Hon Hai Precision Ind. Co., Ltd.	\N
002557	Research In Motion	\N
002558	MPEDIA	\N
002559	Syphan Technologies Ltd	\N
00255a	Tantalus Systems Corp.	\N
00255b	CoachComm, LLC	\N
00255c	NEC Corporation	\N
00255d	Morningstar Corporation	\N
00255e	Shanghai Dare Technologies Co.,Ltd.	\N
00255f	SenTec AG	\N
002560	Ibridge Networks & Communications Ltd.	\N
002561	ProCurve Networking by HP	\N
002562	interbro Co. Ltd.	\N
002563	Luxtera Inc	\N
002564	Dell Inc.	\N
002565	Vizimax Inc.	\N
002566	Samsung Electronics Co.,Ltd	\N
002567	Samsung Electronics	\N
002568	Shenzhen Huawei Communication Technologies Co., Ltd	\N
002569	SAGEM COMMUNICATION	\N
00256a	inIT - Institut Industrial IT	\N
00256b	ATENIX E.E. s.r.l.	\N
00256c	"Azimut" Production Association JSC	\N
00256d	Broadband Forum	\N
00256e	Van Breda B.V.	\N
00256f	Dantherm Power	\N
002570	Eastern Communications Company Limited	\N
002571	Zhejiang Tianle Digital Electric Co.,Ltd	\N
002572	Nemo-Q International AB	\N
002573	ST Electronics (Info-Security) Pte Ltd	\N
002574	KUNIMI MEDIA DEVICE Co., Ltd.	\N
002575	FiberPlex Inc	\N
002576	NELI TECHNOLOGIES	\N
002577	D-BOX Technologies	\N
002578	JSC "Concern "Sozvezdie"	\N
002579	J & F Labs	\N
00257a	CAMCO Produktions- und Vertriebs-GmbH für Beschallungs- und Beleuchtungsanlagen	\N
00257b	STJ  ELECTRONICS  PVT  LTD	\N
00257c	Huachentel Technology Development Co., Ltd	\N
00257d	PointRed Telecom Private Ltd.	\N
00257e	NEW POS Technology Limited	\N
00257f	CallTechSolution Co.,Ltd	\N
002580	Equipson S.A.	\N
002581	x-star networks Inc.	\N
002582	Maksat Technologies (P) Ltd	\N
002583	Cisco Systems	\N
002584	Cisco Systems	\N
002585	KOKUYO S&T Co., Ltd.	\N
002586	TP-LINK Technologies Co., Ltd.	\N
002587	Vitality, Inc.	\N
002588	Genie Industries, Inc.	\N
002589	Hills Industries Limited	\N
00258a	Pole/Zero Corporation	\N
00258b	Mellanox Technologies Ltd	\N
00258c	ESUS ELEKTRONIK SAN. VE DIS. TIC. LTD. STI.	\N
00258d	Haier	\N
00258e	The Weather Channel	\N
00258f	Trident Microsystems, Inc.	\N
002590	Super Micro Computer, Inc.	\N
002591	NEXTEK, Inc.	\N
002592	Guangzhou Shirui Electronic Co., Ltd	\N
002593	DatNet Informatikai Kft.	\N
002594	Eurodesign BG LTD	\N
002595	Northwest Signal Supply, Inc	\N
002596	GIGAVISION srl	\N
002597	Kalki Communication Technologies	\N
002598	Zhong Shan City Litai Electronic Industrial Co. Ltd	\N
002599	Hedon e.d. B.V.	\N
00259a	CEStronics GmbH	\N
00259b	Beijing PKUNITY Microsystems Technology Co., Ltd	\N
00259c	Cisco-Linksys, LLC	\N
00259d	PRIVATE	\N
00259e	Huawei Technologies Co., Ltd.	\N
00259f	TechnoDigital Technologies GmbH	\N
0025a0	Nintendo Co., Ltd.	\N
0025a1	Enalasys	\N
0025a2	Alta Definicion LINCEO S.L.	\N
0025a3	Trimax Wireless, Inc.	\N
0025a4	EuroDesign embedded technologies GmbH	\N
0025a5	Walnut Media Network	\N
0025a6	Central Network Solution Co., Ltd.	\N
0025a7	Comverge, Inc.	\N
0025a8	Kontron (BeiJing) Technology Co.,Ltd	\N
0025a9	Shanghai Embedway Information Technologies Co.,Ltd	\N
0025aa	Beijing Soul Technology Co.,Ltd.	\N
0025ab	AIO LCD PC BU / TPV	\N
0025ac	I-Tech corporation	\N
0025ad	Manufacturing Resources International	\N
0025ae	Microsoft Corporation	\N
0025af	COMFILE Technology	\N
0025b0	Schmartz Inc	\N
0025b1	Maya-Creation Corporation	\N
0025b2	LFK-Lenkflugkörpersysteme GmbH	\N
0025b3	Hewlett Packard	\N
0025b4	Cisco Systems	\N
0025b5	Cisco Systems	\N
0025b6	Telecom FM	\N
0025b7	Costar  electronics, inc.,	\N
0025b8	Agile Communications, Inc.	\N
0025b9	Agilink Systems Corp.	\N
0025ba	Alcatel-Lucent IPD	\N
0025bb	INNERINT Co., Ltd.	\N
0025bc	Apple, Inc	\N
0025bd	Italdata Ingegneria dell'Idea S.p.A.	\N
0025be	Tektrap Systems Inc.	\N
0025bf	Wireless Cables Inc.	\N
0025c0	ZillionTV Corporation	\N
0025c1	Nawoo Korea Corp.	\N
0025c2	RingBell Co.,Ltd.	\N
0025c3	Nortel Networks	\N
0025c4	Ruckus Wireless	\N
0025c5	Star Link Communication Pvt. Ltd.	\N
0025c6	kasercorp, ltd	\N
0025c7	altek Corporation	\N
0025c8	S-Access GmbH	\N
0025c9	SHENZHEN HUAPU DIGITAL CO., LTD	\N
0025ca	LS Research, LLC	\N
0025cb	Reiner SCT	\N
0025cc	Mobile Communications Korea Incorporated	\N
0025cd	Skylane Optics	\N
0025ce	InnerSpace	\N
0025cf	Nokia Danmark A/S	\N
0025d0	Nokia Danmark A/S	\N
0025d1	Eastech Electronics (Taiwan) Inc.	\N
0025d2	InpegVision Co., Ltd	\N
0025d3	AzureWave Technologies, Inc	\N
0025d4	Fortress Technologies	\N
0025d5	Robonica (Pty) Ltd	\N
0025d6	The Kroger Co.	\N
0025d7	CEDO	\N
0025d8	KOREA MAINTENANCE	\N
0025d9	DataFab Systems Inc.	\N
0025da	Secura Key	\N
0025db	ATI Electronics(Shenzhen) Co., LTD	\N
0025dc	Sumitomo Electric Networks, Inc	\N
0025dd	SUNNYTEK INFORMATION CO., LTD.	\N
0025de	Probits Co., LTD.	\N
0025df	PRIVATE	\N
0025e0	CeedTec Sdn Bhd	\N
0025e1	SHANGHAI SEEYOO ELECTRONIC & TECHNOLOGY CO., LTD	\N
0025e2	Everspring Industry Co., Ltd.	\N
0025e3	Hanshinit Inc.	\N
0025e4	OMNI-WiFi, LLC	\N
0025e5	LG Electronics Inc	\N
0025e6	Belgian Monitoring Systems bvba	\N
0025e7	Sony Ericsson Mobile Communications	\N
0025e8	Idaho Technology	\N
0025e9	i-mate Development, Inc.	\N
0025ea	Iphion BV	\N
0025eb	Reutech Radar Systems (PTY) Ltd	\N
0025ec	Humanware	\N
0025ed	NuVo Technologies LLC	\N
0025ee	Avtex Ltd	\N
0025ef	I-TEC Co., Ltd.	\N
0025f0	Suga Electronics Limited	\N
0025f1	Motorola CHS	\N
0025f2	Motorola CHS	\N
0025f3	Nordwestdeutsche Zählerrevision	\N
0025f4	KoCo Connector AG	\N
0025f5	DVS Korea, Co., Ltd	\N
0025f6	netTALK.com, Inc.	\N
0025f7	Ansaldo STS USA	\N
0025f9	GMK electronic design GmbH	\N
0025fa	J&M Analytik AG	\N
0025fb	Tunstall Healthcare A/S	\N
0025fc	ENDA ENDUSTRIYEL ELEKTRONIK LTD. STI.	\N
0025fd	OBR Centrum Techniki Morskiej S.A.	\N
0025fe	Pilot Electronics Corporation	\N
0025ff	CreNova Technology GmbH	\N
002600	TEAC Australia Pty Ltd.	\N
002601	PRIVATE	\N
002602	SMART Temps LLC	\N
002603	Shenzhen Wistar Technology Co., Ltd	\N
002604	Audio Processing Technology Ltd	\N
002605	CC Systems AB	\N
002606	RAUMFELD GmbH	\N
002607	Enabling Technology Pty Ltd	\N
002608	Apple, Inc	\N
002609	Phyllis Co., Ltd.	\N
00260a	Cisco Systems	\N
00260b	Cisco Systems	\N
00260c	Dataram	\N
00260d	Micronetics, Inc.	\N
00260e	Ablaze Systems, LLC	\N
00260f	Linn Products Ltd	\N
002610	Apacewave Technologies	\N
002611	Licera AB	\N
002612	Space Exploration Technologies	\N
002613	Engel Axil S.L.	\N
002614	KTNF	\N
002615	Teracom Limited	\N
002616	Rosemount Inc.	\N
002617	OEM Worldwide	\N
002618	ASUSTek COMPUTER INC.	\N
002619	FRC	\N
00261a	Femtocomm System Technology Corp.	\N
00261b	LAUREL BANK MACHINES CO., LTD.	\N
00261c	NEOVIA INC.	\N
00261d	COP SECURITY SYSTEM CORP.	\N
00261e	QINGBANG ELEC(SZ) CO., LTD	\N
00261f	SAE Magnetics (H.K.) Ltd.	\N
002620	ISGUS GmbH	\N
002621	InteliCloud Technology Inc.	\N
002622	COMPAL INFORMATION (KUNSHAN) CO., LTD.	\N
002623	JRD Communication Inc	\N
002624	Thomson Inc.	\N
002625	MediaSputnik	\N
002626	Geophysical Survey Systems, Inc.	\N
002627	Truesell	\N
002628	companytec automação e controle ltda	\N
002629	Juphoon System Software Inc.	\N
00262a	Proxense, LLC	\N
00262b	Wongs Electronics Co. Ltd.	\N
00262c	IKT Advanced Technologies s.r.o.	\N
00262d	Wistron Corporation	\N
00262e	Chengdu Jiuzhou Electronic Technology Inc	\N
00262f	HAMAMATSU TOA ELECTRONICS	\N
002630	ACOREL S.A.S	\N
002631	COMMTACT LTD	\N
002632	Instrumentation Technologies d.d.	\N
002633	MIR - Medical International Research	\N
002634	Infineta Systems, Inc	\N
002635	Bluetechnix GmbH	\N
002636	Motorola Mobile Devices	\N
002637	Samsung Electro-Mechanics	\N
002638	Xia Men Joyatech Co., Ltd.	\N
002639	T.M. Electronics, Inc.	\N
00263a	Digitec Systems	\N
00263b	Onbnetech	\N
00263c	Bachmann GmbH & Co. KG	\N
00263d	MIA Corporation	\N
00263e	Trapeze Networks	\N
00263f	LIOS Technology GmbH	\N
002640	Baustem Broadband Technologies, Ltd.	\N
002641	Motorola, Inc	\N
002642	Motorola, Inc	\N
002643	Alps Electric Co., Ltd	\N
002644	Thomson Telecom Belgium	\N
002645	Circontrol S.A.	\N
002646	SHENYANG TONGFANG MULTIMEDIA TECHNOLOGY COMPANY LIMITED	\N
002647	WFE TECHNOLOGY CORP.	\N
002648	Emitech Corp.	\N
00264a	Apple, Inc	\N
00264c	Shanghai DigiVision Technology Co., Ltd.	\N
00264d	Arcadyan Technology Corporation	\N
00264e	Rail & Road Protec GmbH	\N
00264f	Krüger&Gothe GmbH	\N
002650	2Wire	\N
002651	Cisco Systems	\N
002652	Cisco Systems	\N
002653	DaySequerra Corporation	\N
002654	3Com Corporation	\N
002655	Hewlett Packard	\N
002656	Sansonic Electronics USA	\N
002657	OOO NPP EKRA	\N
002658	T-Platforms (Cyprus) Limited	\N
002659	Nintendo Co., Ltd.	\N
00265a	D-Link Corporation	\N
00265b	Hitron Technologies. Inc	\N
00265c	Hon Hai Precision Ind. Co.,Ltd.	\N
00265d	Samsung Electronics	\N
00265e	Hon Hai Precision Ind. Co.,Ltd.	\N
00265f	Samsung Electronics Co.,Ltd	\N
002660	Logiways	\N
002661	Irumtek Co., Ltd.	\N
002662	Actiontec Electronics, Inc	\N
002663	Shenzhen Huitaiwei Tech. Ltd, co.	\N
002664	Core System Japan	\N
002665	ProtectedLogic Corporation	\N
002666	EFM Networks	\N
002667	CARECOM CO.,LTD.	\N
002668	Nokia Danmark A/S	\N
002669	Nokia Danmark A/S	\N
00266a	ESSENSIUM NV	\N
00266b	SHINE UNION ENTERPRISE LIMITED	\N
00266c	Inventec	\N
00266d	MobileAccess Networks	\N
00266e	Nissho-denki Co.,LTD.	\N
00266f	Coordiwise Technology Corp.	\N
002670	Cinch Connectors	\N
002671	AUTOVISION Co., Ltd	\N
002672	AAMP of America	\N
002673	RICOH COMPANY LTD.	\N
002674	Electronic Solutions, Inc.	\N
002675	Aztech Electronics Pte Ltd	\N
002676	COMMidt AS	\N
002677	DEIF A/S	\N
002678	Logic Instrument SA	\N
002679	Euphonic Technologies, Inc.	\N
00267a	wuhan hongxin telecommunication technologies co.,ltd	\N
00267b	GSI Helmholtzzentrum für Schwerionenforschung GmbH	\N
00267c	Metz-Werke GmbH & Co KG	\N
00267d	A-Max Technology Macao Commercial Offshore Company Limited	\N
00267e	Parrot SA	\N
00267f	Zenterio AB	\N
002680	Lockie Innovation Pty Ltd	\N
002681	Interspiro AB	\N
002682	Gemtek Technology Co., Ltd.	\N
002683	Ajoho Enterprise Co., Ltd.	\N
002684	KISAN SYSTEM	\N
002685	Digital Innovation	\N
002686	Quantenna Communcations, Inc.	\N
002687	ALLIED TELESIS, K.K corega division.	\N
002688	Juniper Networks	\N
002689	General Dynamics Robotic Systems	\N
00268a	Terrier SC Ltd	\N
00268b	Guangzhou Escene Computer Technology Limited	\N
00268c	StarLeaf Ltd.	\N
00268d	CellTel S.p.A.	\N
00268e	Alta Solutions, Inc.	\N
00268f	MTA SpA	\N
002690	I DO IT	\N
002691	SAGEM COMMUNICATION	\N
002692	Mitsubishi Electric Co.	\N
002693	QVidium Technologies, Inc.	\N
002694	Senscient Ltd	\N
002695	ZT Group Int'l Inc	\N
002696	NOOLIX Co., Ltd	\N
002697	Cheetah Technologies, L.P.	\N
002698	Cisco Systems	\N
002699	Cisco Systems	\N
00269a	carina system co., ltd.	\N
00269b	SOKRAT Ltd.	\N
00269c	ITUS JAPAN CO. LTD	\N
00269d	M2Mnet Co., Ltd.	\N
00269e	Quanta Computer Inc	\N
00269f	PRIVATE	\N
0026a0	moblic	\N
0026a1	Megger	\N
0026a2	Instrumentation Technology Systems	\N
0026a3	FQ Ingenieria Electronica S.A.	\N
0026a4	Novus Produtos Eletronicos Ltda	\N
0026a5	MICROROBOT.CO.,LTD	\N
0026a6	TRIXELL	\N
0026a7	CONNECT SRL	\N
0026a8	DAEHAP HYPER-TECH	\N
0026a9	Strong Technologies Pty Ltd	\N
0026aa	Kenmec Mechanical Engineering Co., Ltd.	\N
0026ab	SEIKO EPSON CORPORATION	\N
0026ac	Shanghai LUSTER Teraband photonic Co., Ltd.	\N
0026ad	Arada Systems, Inc.	\N
0026ae	Wireless Measurement Ltd	\N
0026af	Duelco A/S	\N
0026b0	Apple, Inc	\N
0026b1	Harman/Navis	\N
0026b2	Setrix AG	\N
0026b3	Thales Communications Inc	\N
0026b4	Ford Motor Company	\N
0026b5	ICOMM Tele Ltd	\N
0026b6	Askey Computer	\N
0026b7	Kingston Technology Company, Inc.	\N
0026b8	Actiontec Electronics, Inc	\N
0026b9	Dell Inc	\N
0026ba	Motorola Mobile Devices	\N
0026bb	Apple, Inc	\N
0026bc	General Jack Technology Ltd.	\N
0026bd	JTEC Card & Communication Co., Ltd.	\N
0026be	Schoonderbeek Elektronica Systemen B.V.	\N
0026bf	ShenZhen Temobi Science&Tech Development Co.,Ltd	\N
0026c0	EnergyHub	\N
0026c1	ARTRAY CO., LTD.	\N
0026c2	SCDI Co. LTD	\N
0026c3	Insightek Corp.	\N
0026c4	Cadmos microsystems S.r.l.	\N
0026c5	Guangdong Gosun Telecommunications Co.,Ltd	\N
0026c6	Intel Corporate	\N
0026c7	Intel Corporate	\N
0026c8	System Sensor	\N
0026c9	Proventix Systems, Inc.	\N
0026ca	Cisco Systems	\N
0026cb	Cisco Systems	\N
0026cc	Nokia Danmark A/S	\N
0026cd	PurpleComm, Inc.	\N
0026ce	Kozumi USA Corp.	\N
0026cf	DEKA R&D	\N
0026d0	Semihalf	\N
0026d1	S Squared Innovations Inc.	\N
0026d2	Pcube Systems, Inc.	\N
0026d3	Zeno Information System	\N
0026d4	IRCA SpA	\N
0026d5	Ory Solucoes em Comercio de Informatica Ltda.	\N
0026d6	Ningbo Andy Optoelectronic Co., Ltd.	\N
0026d7	Xiamen BB Electron & Technology Co., Ltd.	\N
0026d8	Magic Point Inc.	\N
0026d9	Pace Micro Technology plc	\N
0026da	Universal Media Corporation /Slovakia/ s.r.o.	\N
0026db	Ionics EMS Inc.	\N
0026dc	Optical Systems Design	\N
0026dd	Fival Corporation	\N
0026de	FDI MATELEC	\N
0026df	TaiDoc Technology Corp.	\N
0026e0	ASITEQ	\N
0026e1	Stanford University, OpenFlow Group	\N
0026e2	LG Electronics	\N
0026e3	DTI	\N
0026e4	CANAL OVERSEAS	\N
0026e5	AEG Power Solutions	\N
0026e6	Visionhitech Co., Ltd.	\N
0026e7	Shanghai ONLAN Communication Tech. Co., Ltd.	\N
0026e8	Murata Manufacturing Co., Ltd.	\N
0026e9	SP Corp	\N
0026ea	Cheerchip Electronic Technology (ShangHai) Co., Ltd.	\N
0026eb	Advanced Spectrum Technology Co., Ltd.	\N
0026ec	Legrand Home Systems, Inc	\N
0026ed	zte corporation	\N
0026ee	TKM GmbH	\N
0026ef	Technology Advancement Group, Inc.	\N
0026f0	cTrixs International GmbH.	\N
0026f1	ProCurve Networking by HP	\N
0026f2	Netgear	\N
0026f3	SMC Networks	\N
0026f4	Nesslab	\N
0026f5	XRPLUS Inc.	\N
0026f6	Military Communication Institute	\N
0026f7	Infosys Technologies Ltd.	\N
0026f8	Golden Highway Industry Development Co., Ltd.	\N
0026f9	S.E.M. srl	\N
0026fa	BandRich Inc.	\N
0026fb	AirDio Wireless, Inc.	\N
0026fc	AcSiP Technology Corp.	\N
0026fd	Interactive Intelligence	\N
0026fe	MKD Technology Inc.	\N
0026ff	Research In Motion	\N
002700	Shenzhen Siglent Technology Co., Ltd.	\N
002701	INCOstartec GmbH	\N
002702	SolarEdge Technologies	\N
002703	Testech Electronics Pte Ltd	\N
002704	Accelerated Concepts, LLC	\N
002705	Sectronic	\N
002706	YOISYS	\N
002707	Lift Complex DS, JSC	\N
002708	Nordiag ASA	\N
002709	Nintendo Co., Ltd.	\N
00270a	IEE S.A.	\N
00270b	Adura Technologies	\N
00270c	Cisco Systems	\N
00270d	Cisco Systems	\N
00270e	Intel Corporate	\N
00270f	Envisionnovation Inc	\N
002710	Intel Corporate	\N
002711	LanPro Inc	\N
002712	MaxVision LLC	\N
002713	USI	\N
002714	Grainmustards, Co,ltd.	\N
002715	Rebound Telecom. Co., Ltd	\N
002716	Adachi-Syokai Co., Ltd.	\N
002717	CE Digital(Zhenjiang)Co.,Ltd	\N
002718	Suzhou NEW SEAUNION Video Technology Co.,Ltd	\N
002719	TP-LINK TECHNOLOGIES CO., LTD.	\N
00271a	Geenovo Technology Ltd.	\N
00271b	Alec Sicherheitssysteme GmbH	\N
00271c	MERCURY CORPORATION	\N
00271d	Comba Telecom Systems (China) Ltd.	\N
00271e	Xagyl Communications	\N
00271f	MIPRO Electronics Co., Ltd	\N
002720	NEW-SOL COM	\N
002721	Shenzhen Baoan Fenda Industrial Co., Ltd	\N
002722	Ubiquiti Networks	\N
003000	ALLWELL TECHNOLOGY CORP.	\N
003001	SMP	\N
003002	Expand Networks	\N
003003	Phasys Ltd.	\N
003004	LEADTEK RESEARCH INC.	\N
003005	Fujitsu Siemens Computers	\N
003006	SUPERPOWER COMPUTER	\N
003007	OPTI, INC.	\N
003008	AVIO DIGITAL, INC.	\N
003009	Tachion Networks, Inc.	\N
00300a	AZTECH Electronics Pte Ltd	\N
00300b	mPHASE Technologies, Inc.	\N
00300c	CONGRUENCY, LTD.	\N
00300d	MMC Technology, Inc.	\N
00300e	Klotz Digital AG	\N
00300f	IMT - Information Management T	\N
003010	VISIONETICS INTERNATIONAL	\N
003011	HMS FIELDBUS SYSTEMS AB	\N
003012	DIGITAL ENGINEERING LTD.	\N
003013	NEC Corporation	\N
003014	DIVIO, INC.	\N
003015	CP CLARE CORP.	\N
003016	ISHIDA CO., LTD.	\N
003017	BlueArc UK Ltd	\N
003018	Jetway Information Co., Ltd.	\N
003019	CISCO SYSTEMS, INC.	\N
00301a	SMARTBRIDGES PTE. LTD.	\N
00301b	SHUTTLE, INC.	\N
00301c	ALTVATER AIRDATA SYSTEMS	\N
00301d	SKYSTREAM, INC.	\N
00301e	3COM Europe Ltd.	\N
00301f	OPTICAL NETWORKS, INC.	\N
003020	TSI, Inc..	\N
003021	HSING TECH. ENTERPRISE CO.,LTD	\N
003022	Fong Kai Industrial Co., Ltd.	\N
003023	COGENT COMPUTER SYSTEMS, INC.	\N
003024	CISCO SYSTEMS, INC.	\N
003025	CHECKOUT COMPUTER SYSTEMS, LTD	\N
003026	HeiTel Digital Video GmbH	\N
003027	KERBANGO, INC.	\N
003028	FASE Saldatura srl	\N
003029	OPICOM	\N
00302a	SOUTHERN INFORMATION	\N
00302b	INALP NETWORKS, INC.	\N
00302c	SYLANTRO SYSTEMS CORPORATION	\N
00302d	QUANTUM BRIDGE COMMUNICATIONS	\N
00302e	Hoft & Wessel AG	\N
00302f	GE Aviation System	\N
003030	HARMONIX CORPORATION	\N
003031	LIGHTWAVE COMMUNICATIONS, INC.	\N
003032	MagicRam, Inc.	\N
003033	ORIENT TELECOM CO., LTD.	\N
003034	SET ENGINEERING	\N
003035	Corning Incorporated	\N
003036	RMP ELEKTRONIKSYSTEME GMBH	\N
003037	Packard Bell Nec Services	\N
003038	XCP, INC.	\N
003039	SOFTBOOK PRESS	\N
00303a	MAATEL	\N
00303b	PowerCom Technology	\N
00303c	ONNTO CORP.	\N
00303d	IVA CORPORATION	\N
00303e	Radcom Ltd.	\N
00303f	TurboComm Tech Inc.	\N
003040	CISCO SYSTEMS, INC.	\N
003041	SAEJIN T & M CO., LTD.	\N
003042	DeTeWe-Deutsche Telephonwerke	\N
003043	IDREAM TECHNOLOGIES, PTE. LTD.	\N
003044	Portsmith LLC	\N
003045	Village Networks, Inc. (VNI)	\N
003046	Controlled Electronic Manageme	\N
003047	NISSEI ELECTRIC CO., LTD.	\N
003048	Supermicro Computer, Inc.	\N
003049	BRYANT TECHNOLOGY, LTD.	\N
00304a	Fraunhofer IPMS	\N
00304b	ORBACOM SYSTEMS, INC.	\N
00304c	APPIAN COMMUNICATIONS, INC.	\N
00304d	ESI	\N
00304e	BUSTEC PRODUCTION LTD.	\N
00304f	PLANET Technology Corporation	\N
003050	Versa Technology	\N
003051	ORBIT AVIONIC & COMMUNICATION	\N
003052	ELASTIC NETWORKS	\N
003053	Basler AG	\N
003054	CASTLENET TECHNOLOGY, INC.	\N
003055	Renesas Technology America, Inc.	\N
003056	Beck IPC GmbH	\N
003057	QTelNet, Inc.	\N
003058	API MOTION	\N
003059	DIGITAL-LOGIC AG	\N
00305a	TELGEN CORPORATION	\N
00305b	Toko Inc.	\N
00305c	SMAR Laboratories Corp.	\N
00305d	DIGITRA SYSTEMS, INC.	\N
00305e	Abelko Innovation	\N
00305f	Hasselblad	\N
003060	Powerfile, Inc.	\N
003061	MobyTEL	\N
003062	PATH 1 NETWORK TECHNOL'S INC.	\N
003063	SANTERA SYSTEMS, INC.	\N
003064	ADLINK TECHNOLOGY, INC.	\N
003065	APPLE COMPUTER, INC.	\N
003066	RFM	\N
003067	BIOSTAR MICROTECH INT'L CORP.	\N
003068	CYBERNETICS TECH. CO., LTD.	\N
003069	IMPACCT TECHNOLOGY CORP.	\N
00306a	PENTA MEDIA CO., LTD.	\N
00306b	CMOS SYSTEMS, INC.	\N
00306c	Hitex Holding GmbH	\N
00306d	LUCENT TECHNOLOGIES	\N
00306e	HEWLETT PACKARD	\N
00306f	SEYEON TECH. CO., LTD.	\N
003070	1Net Corporation	\N
003071	Cisco Systems, Inc.	\N
003072	Intellibyte Inc.	\N
003073	International Microsystems, In	\N
003074	EQUIINET LTD.	\N
003075	ADTECH	\N
003076	Akamba Corporation	\N
003077	ONPREM NETWORKS	\N
003078	Cisco Systems, Inc.	\N
003079	CQOS, INC.	\N
00307a	Advanced Technology & Systems	\N
00307b	Cisco Systems, Inc.	\N
00307c	ADID SA	\N
00307d	GRE AMERICA, INC.	\N
00307e	Redflex Communication Systems	\N
00307f	IRLAN LTD.	\N
003080	CISCO SYSTEMS, INC.	\N
003081	ALTOS C&C	\N
003082	TAIHAN ELECTRIC WIRE CO., LTD.	\N
003083	Ivron Systems	\N
003084	ALLIED TELESYN INTERNAIONAL	\N
003085	CISCO SYSTEMS, INC.	\N
003086	Transistor Devices, Inc.	\N
003087	VEGA GRIESHABER KG	\N
003088	Siara Systems, Inc.	\N
003089	Spectrapoint Wireless, LLC	\N
00308a	NICOTRA SISTEMI S.P.A	\N
00308b	Brix Networks	\N
00308c	Quantum Corporation	\N
00308d	Pinnacle Systems, Inc.	\N
00308e	CROSS MATCH TECHNOLOGIES, INC.	\N
00308f	MICRILOR, Inc.	\N
003090	CYRA TECHNOLOGIES, INC.	\N
003091	TAIWAN FIRST LINE ELEC. CORP.	\N
003092	ModuNORM GmbH	\N
003093	Sonnet Technologies, Inc	\N
003094	Cisco Systems, Inc.	\N
003095	Procomp Informatics, Ltd.	\N
003096	CISCO SYSTEMS, INC.	\N
003097	AB Regin	\N
003098	Global Converging Technologies	\N
003099	BOENIG UND KALLENBACH OHG	\N
00309a	ASTRO TERRA CORP.	\N
00309b	Smartware	\N
00309c	Timing Applications, Inc.	\N
00309d	Nimble Microsystems, Inc.	\N
00309e	WORKBIT CORPORATION.	\N
00309f	AMBER NETWORKS	\N
0030a0	TYCO SUBMARINE SYSTEMS, LTD.	\N
0030a1	WEBGATE Inc.	\N
0030a2	Lightner Engineering	\N
0030a3	CISCO SYSTEMS, INC.	\N
0030a4	Woodwind Communications System	\N
0030a5	ACTIVE POWER	\N
0030a6	VIANET TECHNOLOGIES, LTD.	\N
0030a7	SCHWEITZER ENGINEERING	\N
0030a8	OL'E COMMUNICATIONS, INC.	\N
0030a9	Netiverse, Inc.	\N
0030aa	AXUS MICROSYSTEMS, INC.	\N
0030ab	DELTA NETWORKS, INC.	\N
0030ac	Systeme Lauer GmbH & Co., Ltd.	\N
0030ad	SHANGHAI COMMUNICATION	\N
0030ae	Times N System, Inc.	\N
0030af	Honeywell GmbH	\N
0030b0	Convergenet Technologies	\N
0030b1	aXess-pro networks GmbH	\N
0030b2	L-3 Sonoma EO	\N
0030b3	San Valley Systems, Inc.	\N
0030b4	INTERSIL CORP.	\N
0030b5	Tadiran Microwave Networks	\N
0030b6	CISCO SYSTEMS, INC.	\N
0030b7	Teletrol Systems, Inc.	\N
0030b8	RiverDelta Networks	\N
0030b9	ECTEL	\N
0030ba	AC&T SYSTEM CO., LTD.	\N
0030bb	CacheFlow, Inc.	\N
0030bc	Optronic AG	\N
0030bd	BELKIN COMPONENTS	\N
0030be	City-Net Technology, Inc.	\N
0030bf	MULTIDATA GMBH	\N
0030c0	Lara Technology, Inc.	\N
0030c1	HEWLETT-PACKARD	\N
0030c2	COMONE	\N
0030c3	FLUECKIGER ELEKTRONIK AG	\N
0030c4	Canon Imaging Systems Inc.	\N
0030c5	CADENCE DESIGN SYSTEMS	\N
0030c6	CONTROL SOLUTIONS, INC.	\N
0030c7	Macromate Corp.	\N
0030c8	GAD LINE, LTD.	\N
0030c9	LuxN, N	\N
0030ca	Discovery Com	\N
0030cb	OMNI FLOW COMPUTERS, INC.	\N
0030cc	Tenor Networks, Inc.	\N
0030cd	CONEXANT SYSTEMS, INC.	\N
0030ce	Zaffire	\N
0030cf	TWO TECHNOLOGIES, INC.	\N
0030d0	Tellabs	\N
0030d1	INOVA CORPORATION	\N
0030d2	WIN TECHNOLOGIES, CO., LTD.	\N
0030d3	Agilent Technologies	\N
0030d4	AAE Systems, Inc	\N
0030d5	DResearch GmbH	\N
0030d6	MSC VERTRIEBS GMBH	\N
0030d7	Innovative Systems, L.L.C.	\N
0030d8	SITEK	\N
0030d9	DATACORE SOFTWARE CORP.	\N
0030da	COMTREND CO.	\N
0030db	Mindready Solutions, Inc.	\N
0030dc	RIGHTECH CORPORATION	\N
0030dd	INDIGITA CORPORATION	\N
0030de	WAGO Kontakttechnik GmbH	\N
0030df	KB/TEL TELECOMUNICACIONES	\N
0030e0	OXFORD SEMICONDUCTOR LTD.	\N
0030e1	ACROTRON SYSTEMS, INC.	\N
0030e2	GARNET SYSTEMS CO., LTD.	\N
0030e3	SEDONA NETWORKS CORP.	\N
0030e4	CHIYODA SYSTEM RIKEN	\N
0030e5	Amper Datos S.A.	\N
0030e6	Draeger Medical Systems, Inc.	\N
0030e7	CNF MOBILE SOLUTIONS, INC.	\N
0030e8	ENSIM CORP.	\N
0030e9	GMA COMMUNICATION MANUFACT'G	\N
0030ea	TeraForce Technology Corporation	\N
0030eb	TURBONET COMMUNICATIONS, INC.	\N
0030ec	BORGARDT	\N
0030ed	Expert Magnetics Corp.	\N
0030ee	DSG Technology, Inc.	\N
0030ef	NEON TECHNOLOGY, INC.	\N
0030f0	Uniform Industrial Corp.	\N
0030f1	Accton Technology Corp.	\N
0030f2	CISCO SYSTEMS, INC.	\N
0030f3	At Work Computers	\N
0030f4	STARDOT TECHNOLOGIES	\N
0030f5	Wild Lab. Ltd.	\N
0030f6	SECURELOGIX CORPORATION	\N
0030f7	RAMIX INC.	\N
0030f8	Dynapro Systems, Inc.	\N
0030f9	Sollae Systems Co., Ltd.	\N
0030fa	TELICA, INC.	\N
0030fb	AZS Technology AG	\N
0030fc	Terawave Communications, Inc.	\N
0030fd	INTEGRATED SYSTEMS DESIGN	\N
0030fe	DSA GmbH	\N
0030ff	DATAFAB SYSTEMS, INC.	\N
003532	Electro-Metrics Corporation	\N
003a98	Cisco Systems	\N
003a99	Cisco Systems	\N
003a9a	Cisco Systems	\N
003a9b	Cisco Systems	\N
003a9c	Cisco Systems	\N
003a9d	NEC AccessTechnica, Ltd.	\N
003aaf	BlueBit Ltd.	\N
003cc5	WONWOO Engineering Co., Ltd	\N
003d41	Hatteland Computer AS	\N
004000	PCI COMPONENTES DA AMZONIA LTD	\N
004001	ZYXEL COMMUNICATIONS, INC.	\N
004002	PERLE SYSTEMS LIMITED	\N
004003	Emerson Process Management Power & Water Solutions, Inc.	\N
004004	ICM CO. LTD.	\N
004005	ANI COMMUNICATIONS INC.	\N
004006	SAMPO TECHNOLOGY CORPORATION	\N
004007	TELMAT INFORMATIQUE	\N
004008	A PLUS INFO CORPORATION	\N
004009	TACHIBANA TECTRON CO., LTD.	\N
00400a	PIVOTAL TECHNOLOGIES, INC.	\N
00400b	CISCO SYSTEMS, INC.	\N
00400c	GENERAL MICRO SYSTEMS, INC.	\N
00400d	LANNET DATA COMMUNICATIONS,LTD	\N
00400e	MEMOTEC, INC.	\N
00400f	DATACOM TECHNOLOGIES	\N
004010	SONIC SYSTEMS, INC.	\N
004011	ANDOVER CONTROLS CORPORATION	\N
004012	WINDATA, INC.	\N
004013	NTT DATA COMM. SYSTEMS CORP.	\N
004014	COMSOFT GMBH	\N
004015	ASCOM INFRASYS AG	\N
004016	ADC - Global Connectivity Solutions Division	\N
004017	Silex Technology America	\N
004018	ADOBE SYSTEMS, INC.	\N
004019	AEON SYSTEMS, INC.	\N
00401a	FUJI ELECTRIC CO., LTD.	\N
00401b	PRINTER SYSTEMS CORP.	\N
00401c	AST RESEARCH, INC.	\N
00401d	INVISIBLE SOFTWARE, INC.	\N
00401e	ICC	\N
00401f	COLORGRAPH LTD	\N
004020	Tyco Electronics (UK) Ltd	\N
004021	RASTER GRAPHICS	\N
004022	KLEVER COMPUTERS, INC.	\N
004023	LOGIC CORPORATION	\N
004024	COMPAC INC.	\N
004025	MOLECULAR DYNAMICS	\N
004026	Buffalo, Inc	\N
004027	SMC MASSACHUSETTS, INC.	\N
004028	NETCOMM LIMITED	\N
004029	COMPEX	\N
00402a	CANOGA-PERKINS	\N
00402b	TRIGEM COMPUTER, INC.	\N
00402c	ISIS DISTRIBUTED SYSTEMS, INC.	\N
00402d	HARRIS ADACOM CORPORATION	\N
00402e	PRECISION SOFTWARE, INC.	\N
00402f	XLNT DESIGNS INC.	\N
004030	GK COMPUTER	\N
004031	KOKUSAI ELECTRIC CO., LTD	\N
004032	DIGITAL COMMUNICATIONS	\N
004033	ADDTRON TECHNOLOGY CO., LTD.	\N
004034	BUSTEK CORPORATION	\N
004035	OPCOM	\N
004036	TRIBE COMPUTER WORKS, INC.	\N
004037	SEA-ILAN, INC.	\N
004038	TALENT ELECTRIC INCORPORATED	\N
004039	OPTEC DAIICHI DENKO CO., LTD.	\N
00403a	IMPACT TECHNOLOGIES	\N
00403b	SYNERJET INTERNATIONAL CORP.	\N
00403c	FORKS, INC.	\N
00403d	TERADATA	\N
00403e	RASTER OPS CORPORATION	\N
00403f	SSANGYONG COMPUTER SYSTEMS	\N
004040	RING ACCESS, INC.	\N
004041	FUJIKURA LTD.	\N
004042	N.A.T. GMBH	\N
004043	Nokia Siemens Networks GmbH & Co. KG.	\N
004044	QNIX COMPUTER CO., LTD.	\N
004045	TWINHEAD CORPORATION	\N
004046	UDC RESEARCH LIMITED	\N
004047	WIND RIVER SYSTEMS	\N
004048	SMD INFORMATICA S.A.	\N
004049	TEGIMENTA AG	\N
00404a	WEST AUSTRALIAN DEPARTMENT	\N
00404b	MAPLE COMPUTER SYSTEMS	\N
00404c	HYPERTEC PTY LTD.	\N
00404d	TELECOMMUNICATIONS TECHNIQUES	\N
00404e	FLUENT, INC.	\N
00404f	SPACE & NAVAL WARFARE SYSTEMS	\N
004050	IRONICS, INCORPORATED	\N
004051	GRACILIS, INC.	\N
004052	STAR TECHNOLOGIES, INC.	\N
004053	AMPRO COMPUTERS	\N
004054	CONNECTION MACHINES SERVICES	\N
004055	METRONIX GMBH	\N
004056	MCM JAPAN LTD.	\N
004057	LOCKHEED - SANDERS	\N
004058	KRONOS, INC.	\N
004059	YOSHIDA KOGYO K. K.	\N
00405a	GOLDSTAR INFORMATION & COMM.	\N
00405b	FUNASSET LIMITED	\N
00405c	FUTURE SYSTEMS, INC.	\N
00405d	STAR-TEK, INC.	\N
00405e	NORTH HILLS ISRAEL	\N
00405f	AFE COMPUTERS LTD.	\N
004060	COMENDEC LTD	\N
004061	DATATECH ENTERPRISES CO., LTD.	\N
004062	E-SYSTEMS, INC./GARLAND DIV.	\N
004063	VIA TECHNOLOGIES, INC.	\N
004064	KLA INSTRUMENTS CORPORATION	\N
004065	GTE SPACENET	\N
004066	HITACHI CABLE, LTD.	\N
004067	OMNIBYTE CORPORATION	\N
004068	EXTENDED SYSTEMS	\N
004069	LEMCOM SYSTEMS, INC.	\N
00406a	KENTEK INFORMATION SYSTEMS,INC	\N
00406b	SYSGEN	\N
00406c	COPERNIQUE	\N
00406d	LANCO, INC.	\N
00406e	COROLLARY, INC.	\N
00406f	SYNC RESEARCH INC.	\N
004070	INTERWARE CO., LTD.	\N
004071	ATM COMPUTER GMBH	\N
004072	Applied Innovation Inc.	\N
004073	BASS ASSOCIATES	\N
004074	CABLE AND WIRELESS	\N
004075	M-TRADE (UK) LTD	\N
004076	Sun Conversion Technologies	\N
004077	MAXTON TECHNOLOGY CORPORATION	\N
004078	WEARNES AUTOMATION PTE LTD	\N
004079	JUKO MANUFACTURE COMPANY, LTD.	\N
00407a	SOCIETE D'EXPLOITATION DU CNIT	\N
00407b	SCIENTIFIC ATLANTA	\N
00407c	QUME CORPORATION	\N
00407d	EXTENSION TECHNOLOGY CORP.	\N
00407e	EVERGREEN SYSTEMS, INC.	\N
00407f	FLIR Systems	\N
004080	ATHENIX CORPORATION	\N
004081	MANNESMANN SCANGRAPHIC GMBH	\N
004082	LABORATORY EQUIPMENT CORP.	\N
004083	TDA INDUSTRIA DE PRODUTOS	\N
004084	HONEYWELL ACS	\N
004085	SAAB INSTRUMENTS AB	\N
004086	MICHELS & KLEBERHOFF COMPUTER	\N
004087	UBITREX CORPORATION	\N
004088	MOBIUS TECHNOLOGIES, INC.	\N
004089	MEIDENSHA CORPORATION	\N
00408a	TPS TELEPROCESSING SYS. GMBH	\N
00408b	RAYLAN CORPORATION	\N
00408c	AXIS COMMUNICATIONS AB	\N
00408d	THE GOODYEAR TIRE & RUBBER CO.	\N
00408e	DIGILOG, INC.	\N
00408f	WM-DATA MINFO AB	\N
004090	ANSEL COMMUNICATIONS	\N
004091	PROCOMP INDUSTRIA ELETRONICA	\N
004092	ASP COMPUTER PRODUCTS, INC.	\N
004093	PAXDATA NETWORKS LTD.	\N
004094	SHOGRAPHICS, INC.	\N
004095	R.P.T. INTERGROUPS INT'L LTD.	\N
004096	Cisco Systems, Inc.	\N
004097	DATEX DIVISION OF	\N
004098	DRESSLER GMBH & CO.	\N
004099	NEWGEN SYSTEMS CORP.	\N
00409a	NETWORK EXPRESS, INC.	\N
00409b	HAL COMPUTER SYSTEMS INC.	\N
00409c	TRANSWARE	\N
00409d	DIGIBOARD, INC.	\N
00409e	CONCURRENT TECHNOLOGIES  LTD.	\N
00409f	LANCAST/CASAT TECHNOLOGY, INC.	\N
0040a0	GOLDSTAR CO., LTD.	\N
0040a1	ERGO COMPUTING	\N
0040a2	KINGSTAR TECHNOLOGY INC.	\N
0040a3	MICROUNITY SYSTEMS ENGINEERING	\N
0040a4	ROSE ELECTRONICS	\N
0040a5	CLINICOMP INTL.	\N
0040a6	Cray, Inc.	\N
0040a7	ITAUTEC PHILCO S.A.	\N
0040a8	IMF INTERNATIONAL LTD.	\N
0040a9	DATACOM INC.	\N
0040aa	VALMET AUTOMATION INC.	\N
0040ab	ROLAND DG CORPORATION	\N
0040ac	SUPER WORKSTATION, INC.	\N
0040ad	SMA REGELSYSTEME GMBH	\N
0040ae	DELTA CONTROLS, INC.	\N
0040af	DIGITAL PRODUCTS, INC.	\N
0040b0	BYTEX CORPORATION, ENGINEERING	\N
0040b1	CODONICS INC.	\N
0040b2	SYSTEMFORSCHUNG	\N
0040b3	PAR MICROSYSTEMS CORPORATION	\N
0040b4	NEXTCOM K.K.	\N
0040b5	VIDEO TECHNOLOGY COMPUTERS LTD	\N
0040b6	COMPUTERM  CORPORATION	\N
0040b7	STEALTH COMPUTER SYSTEMS	\N
0040b8	IDEA ASSOCIATES	\N
0040b9	MACQ ELECTRONIQUE SA	\N
0040ba	ALLIANT COMPUTER SYSTEMS CORP.	\N
0040bb	GOLDSTAR CABLE CO., LTD.	\N
0040bc	ALGORITHMICS LTD.	\N
0040bd	STARLIGHT NETWORKS, INC.	\N
0040be	BOEING DEFENSE & SPACE	\N
0040bf	CHANNEL SYSTEMS INTERN'L INC.	\N
0040c0	VISTA CONTROLS CORPORATION	\N
0040c1	BIZERBA-WERKE WILHEIM KRAUT	\N
0040c2	APPLIED COMPUTING DEVICES	\N
0040c3	FISCHER AND PORTER CO.	\N
0040c4	KINKEI SYSTEM CORPORATION	\N
0040c5	MICOM COMMUNICATIONS INC.	\N
0040c6	FIBERNET RESEARCH, INC.	\N
0040c7	RUBY TECH CORPORATION	\N
0040c8	MILAN TECHNOLOGY CORPORATION	\N
0040c9	NCUBE	\N
0040ca	FIRST INTERNAT'L COMPUTER, INC	\N
0040cb	LANWAN TECHNOLOGIES	\N
0040cc	SILCOM MANUF'G TECHNOLOGY INC.	\N
0040cd	TERA MICROSYSTEMS, INC.	\N
0040ce	NET-SOURCE, INC.	\N
0040cf	STRAWBERRY TREE, INC.	\N
0040d0	MITAC INTERNATIONAL CORP.	\N
0040d1	FUKUDA DENSHI CO., LTD.	\N
0040d2	PAGINE CORPORATION	\N
0040d3	KIMPSION INTERNATIONAL CORP.	\N
0040d4	GAGE TALKER CORP.	\N
0040d5	Sartorius Mechatronics T&H GmbH 	\N
0040d6	LOCAMATION B.V.	\N
0040d7	STUDIO GEN INC.	\N
0040d8	OCEAN OFFICE AUTOMATION LTD.	\N
0040d9	AMERICAN MEGATRENDS INC.	\N
0040da	TELSPEC LTD	\N
0040db	ADVANCED TECHNICAL SOLUTIONS	\N
0040dc	TRITEC ELECTRONIC GMBH	\N
0040dd	HONG TECHNOLOGIES	\N
0040de	ELETTRONICA SAN GIORGIO	\N
0040df	DIGALOG SYSTEMS, INC.	\N
0040e0	ATOMWIDE LTD.	\N
0040e1	MARNER INTERNATIONAL, INC.	\N
0040e2	MESA RIDGE TECHNOLOGIES, INC.	\N
0040e3	QUIN SYSTEMS LTD	\N
0040e4	E-M TECHNOLOGY, INC.	\N
0040e5	SYBUS CORPORATION	\N
0040e6	C.A.E.N.	\N
0040e7	ARNOS INSTRUMENTS & COMPUTER	\N
0040e8	CHARLES RIVER DATA SYSTEMS,INC	\N
0040e9	ACCORD SYSTEMS, INC.	\N
0040ea	PLAIN TREE SYSTEMS INC	\N
0040eb	MARTIN MARIETTA CORPORATION	\N
0040ec	MIKASA SYSTEM ENGINEERING	\N
0040ed	NETWORK CONTROLS INT'NATL INC.	\N
0040ee	OPTIMEM	\N
0040ef	HYPERCOM, INC.	\N
0040f0	MICRO SYSTEMS, INC.	\N
0040f1	CHUO ELECTRONICS CO., LTD.	\N
0040f2	JANICH & KLASS COMPUTERTECHNIK	\N
0040f3	NETCOR	\N
0040f4	CAMEO COMMUNICATIONS, INC.	\N
0040f5	OEM ENGINES	\N
0040f6	KATRON COMPUTERS INC.	\N
0040f7	Polaroid Corporation	\N
0040f8	SYSTEMHAUS DISCOM	\N
0040f9	COMBINET	\N
0040fa	MICROBOARDS, INC.	\N
0040fb	CASCADE COMMUNICATIONS CORP.	\N
0040fc	IBR COMPUTER TECHNIK GMBH	\N
0040fd	LXE	\N
0040fe	SYMPLEX COMMUNICATIONS	\N
0040ff	TELEBIT CORPORATION	\N
004252	RLX Technologies	\N
004501	Versus Technology, Inc.	\N
005000	NEXO COMMUNICATIONS, INC.	\N
005001	YAMASHITA SYSTEMS CORP.	\N
005002	OMNISEC AG	\N
005003	GRETAG MACBETH AG	\N
005004	3COM CORPORATION	\N
005006	TAC AB	\N
005007	SIEMENS TELECOMMUNICATION SYSTEMS LIMITED	\N
005008	TIVA MICROCOMPUTER CORP. (TMC)	\N
005009	PHILIPS BROADBAND NETWORKS	\N
00500a	IRIS TECHNOLOGIES, INC.	\N
00500b	CISCO SYSTEMS, INC.	\N
00500c	e-Tek Labs, Inc.	\N
00500d	SATORI ELECTORIC CO., LTD.	\N
00500e	CHROMATIS NETWORKS, INC.	\N
00500f	CISCO SYSTEMS, INC.	\N
005010	NovaNET Learning, Inc.	\N
005012	CBL - GMBH	\N
005013	Chaparral Network Storage	\N
005014	CISCO SYSTEMS, INC.	\N
005015	BRIGHT STAR ENGINEERING	\N
005016	SST/WOODHEAD INDUSTRIES	\N
005017	RSR S.R.L.	\N
005018	AMIT, Inc.	\N
005019	SPRING TIDE NETWORKS, INC.	\N
00501a	IQinVision	\N
00501b	ABL CANADA, INC.	\N
00501c	JATOM SYSTEMS, INC.	\N
00501e	Miranda Technologies, Inc.	\N
00501f	MRG SYSTEMS, LTD.	\N
005020	MEDIASTAR CO., LTD.	\N
005021	EIS INTERNATIONAL, INC.	\N
005022	ZONET TECHNOLOGY, INC.	\N
005023	PG DESIGN ELECTRONICS, INC.	\N
005024	NAVIC SYSTEMS, INC.	\N
005026	COSYSTEMS, INC.	\N
005027	GENICOM CORPORATION	\N
005028	AVAL COMMUNICATIONS	\N
005029	1394 PRINTER WORKING GROUP	\N
00502a	CISCO SYSTEMS, INC.	\N
00502b	GENRAD LTD.	\N
00502c	SOYO COMPUTER, INC.	\N
00502d	ACCEL, INC.	\N
00502e	CAMBEX CORPORATION	\N
00502f	TollBridge Technologies, Inc.	\N
005030	FUTURE PLUS SYSTEMS	\N
005031	AEROFLEX LABORATORIES, INC.	\N
005032	PICAZO COMMUNICATIONS, INC.	\N
005033	MAYAN NETWORKS	\N
005036	NETCAM, LTD.	\N
005037	KOGA ELECTRONICS CO.	\N
005038	DAIN TELECOM CO., LTD.	\N
005039	MARINER NETWORKS	\N
00503a	DATONG ELECTRONICS LTD.	\N
00503b	MEDIAFIRE CORPORATION	\N
00503c	TSINGHUA NOVEL ELECTRONICS	\N
00503e	CISCO SYSTEMS, INC.	\N
00503f	ANCHOR GAMES	\N
005040	Panasonic Electric Works Co., Ltd.	\N
005041	Coretronic Corporation	\N
005042	SCI MANUFACTURING SINGAPORE PTE, LTD.	\N
005043	MARVELL SEMICONDUCTOR, INC.	\N
005044	ASACA CORPORATION	\N
005045	RIOWORKS SOLUTIONS, INC.	\N
005046	MENICX INTERNATIONAL CO., LTD.	\N
005047	PRIVATE	\N
005048	INFOLIBRIA	\N
005049	Arbor Networks Inc	\N
00504a	ELTECO A.S.	\N
00504b	BARCONET N.V.	\N
00504c	Galil Motion Control	\N
00504d	Tokyo Electron Device Limited	\N
00504e	SIERRA MONITOR CORP.	\N
00504f	OLENCOM ELECTRONICS	\N
005050	CISCO SYSTEMS, INC.	\N
005051	IWATSU ELECTRIC CO., LTD.	\N
005052	TIARA NETWORKS, INC.	\N
005053	CISCO SYSTEMS, INC.	\N
005054	CISCO SYSTEMS, INC.	\N
005055	DOMS A/S	\N
005056	VMware, Inc.	\N
005057	BROADBAND ACCESS SYSTEMS	\N
005058	VegaStream Group Limted	\N
005059	iBAHN	\N
00505a	NETWORK ALCHEMY, INC.	\N
00505b	KAWASAKI LSI U.S.A., INC.	\N
00505c	TUNDO CORPORATION	\N
00505e	DIGITEK MICROLOGIC S.A.	\N
00505f	BRAND INNOVATORS	\N
005060	TANDBERG TELECOM AS	\N
005062	KOUWELL ELECTRONICS CORP.  **	\N
005063	OY COMSEL SYSTEM AB	\N
005064	CAE ELECTRONICS	\N
005065	TDK-Lambda Corporation	\N
005066	AtecoM GmbH advanced telecomunication modules	\N
005067	AEROCOMM, INC.	\N
005068	ELECTRONIC INDUSTRIES ASSOCIATION	\N
005069	PixStream Incorporated	\N
00506a	EDEVA, INC.	\N
00506b	SPX-ATEG	\N
00506c	Beijer Electronics Products AB	\N
00506d	VIDEOJET SYSTEMS	\N
00506e	CORDER ENGINEERING CORPORATION	\N
00506f	G-CONNECT	\N
005070	CHAINTECH COMPUTER CO., LTD.	\N
005071	AIWA CO., LTD.	\N
005072	CORVIS CORPORATION	\N
005073	CISCO SYSTEMS, INC.	\N
005074	ADVANCED HI-TECH CORP.	\N
005075	KESTREL SOLUTIONS	\N
005076	IBM	\N
005077	PROLIFIC TECHNOLOGY, INC.	\N
005078	MEGATON HOUSE, LTD.	\N
005079	PRIVATE	\N
00507a	XPEED, INC.	\N
00507b	MERLOT COMMUNICATIONS	\N
00507c	VIDEOCON AG	\N
00507d	IFP	\N
00507e	NEWER TECHNOLOGY	\N
00507f	DrayTek Corp.	\N
005080	CISCO SYSTEMS, INC.	\N
005081	MURATA MACHINERY, LTD.	\N
005082	FORESSON CORPORATION	\N
005083	GILBARCO, INC.	\N
005084	ATL PRODUCTS	\N
005086	TELKOM SA, LTD.	\N
005087	TERASAKI ELECTRIC CO., LTD.	\N
005088	AMANO CORPORATION	\N
005089	SAFETY MANAGEMENT SYSTEMS	\N
00508b	Hewlett Packard	\N
00508c	RSI SYSTEMS	\N
00508d	ABIT COMPUTER CORPORATION	\N
00508e	OPTIMATION, INC.	\N
00508f	ASITA TECHNOLOGIES INT'L LTD.	\N
005090	DCTRI	\N
005091	NETACCESS, INC.	\N
005092	RIGAKU INDUSTRIAL CORPORATION	\N
005093	BOEING	\N
005094	PACE MICRO TECHNOLOGY PLC	\N
005095	PERACOM NETWORKS	\N
005096	SALIX TECHNOLOGIES, INC.	\N
005097	MMC-EMBEDDED COMPUTERTECHNIK GmbH	\N
005098	GLOBALOOP, LTD.	\N
005099	3COM EUROPE, LTD.	\N
00509a	TAG ELECTRONIC SYSTEMS	\N
00509b	SWITCHCORE AB	\N
00509c	BETA RESEARCH	\N
00509d	THE INDUSTREE B.V.	\N
00509e	Les Technologies SoftAcoustik Inc.	\N
00509f	HORIZON COMPUTER	\N
0050a0	DELTA COMPUTER SYSTEMS, INC.	\N
0050a1	CARLO GAVAZZI, INC.	\N
0050a2	CISCO SYSTEMS, INC.	\N
0050a3	TransMedia Communications, Inc.	\N
0050a4	IO TECH, INC.	\N
0050a5	CAPITOL BUSINESS SYSTEMS, LTD.	\N
0050a6	OPTRONICS	\N
0050a7	CISCO SYSTEMS, INC.	\N
0050a8	OpenCon Systems, Inc.	\N
0050a9	MOLDAT WIRELESS TECHNOLGIES	\N
0050aa	KONICA MINOLTA HOLDINGS, INC.	\N
0050ab	NALTEC, Inc.	\N
0050ac	MAPLE COMPUTER CORPORATION	\N
0050ad	CommUnique Wireless Corp.	\N
0050ae	IWAKI ELECTRONICS CO., LTD.	\N
0050af	INTERGON, INC.	\N
0050b0	TECHNOLOGY ATLANTA CORPORATION	\N
0050b1	GIDDINGS & LEWIS	\N
0050b2	BRODEL AUTOMATION	\N
0050b3	VOICEBOARD CORPORATION	\N
0050b4	SATCHWELL CONTROL SYSTEMS, LTD	\N
0050b5	FICHET-BAUCHE	\N
0050b6	GOOD WAY IND. CO., LTD.	\N
0050b7	BOSER TECHNOLOGY CO., LTD.	\N
0050b8	INOVA COMPUTERS GMBH & CO. KG	\N
0050b9	XITRON TECHNOLOGIES, INC.	\N
0050ba	D-LINK	\N
0050bb	CMS TECHNOLOGIES	\N
0050bc	HAMMER STORAGE SOLUTIONS	\N
0050bd	CISCO SYSTEMS, INC.	\N
0050be	FAST MULTIMEDIA AG	\N
0050bf	MOTOTECH INC.	\N
0050c0	GATAN, INC.	\N
0050c1	GEMFLEX NETWORKS, LTD.	\N
0050c2	IEEE REGISTRATION AUTHORITY	\N
0050c4	IMD	\N
0050c5	ADS Technologies, Inc	\N
0050c6	LOOP TELECOMMUNICATION INTERNATIONAL, INC.	\N
0050c8	ADDONICS COMMUNICATIONS, INC.	\N
0050c9	MASPRO DENKOH CORP.	\N
0050ca	NET TO NET TECHNOLOGIES	\N
0050cb	JETTER	\N
0050cc	XYRATEX	\N
0050cd	DIGIANSWER A/S	\N
0050ce	LG INTERNATIONAL CORP.	\N
0050cf	VANLINK COMMUNICATION TECHNOLOGY RESEARCH INSTITUTE	\N
0050d0	MINERVA SYSTEMS	\N
0050d1	CISCO SYSTEMS, INC.	\N
0050d2	CMC Electronics Inc	\N
0050d3	DIGITAL AUDIO PROCESSING PTY. LTD.	\N
0050d4	JOOHONG INFORMATION &	\N
0050d5	AD SYSTEMS CORP.	\N
0050d6	ATLAS COPCO TOOLS AB	\N
0050d7	TELSTRAT	\N
0050d8	UNICORN COMPUTER CORP.	\N
0050d9	ENGETRON-ENGENHARIA ELETRONICA IND. e COM. LTDA	\N
0050da	3COM CORPORATION	\N
0050db	CONTEMPORARY CONTROL	\N
0050dc	TAS TELEFONBAU A. SCHWABE GMBH & CO. KG	\N
0050dd	SERRA SOLDADURA, S.A.	\N
0050de	SIGNUM SYSTEMS CORP.	\N
0050df	AirFiber, Inc.	\N
0050e1	NS TECH ELECTRONICS SDN BHD	\N
0050e2	CISCO SYSTEMS, INC.	\N
0050e3	Motorola, Inc.	\N
0050e4	APPLE COMPUTER, INC.	\N
0050e6	HAKUSAN CORPORATION	\N
0050e7	PARADISE INNOVATIONS (ASIA)	\N
0050e8	NOMADIX INC.	\N
0050ea	XEL COMMUNICATIONS, INC.	\N
0050eb	ALPHA-TOP CORPORATION	\N
0050ec	OLICOM A/S	\N
0050ed	ANDA NETWORKS	\N
0050ee	TEK DIGITEL CORPORATION	\N
0050ef	SPE Systemhaus GmbH	\N
0050f0	CISCO SYSTEMS, INC.	\N
0050f1	LIBIT SIGNAL PROCESSING, LTD.	\N
0050f2	MICROSOFT CORP.	\N
0050f3	GLOBAL NET INFORMATION CO., Ltd.	\N
0050f4	SIGMATEK GMBH & CO. KG	\N
0050f6	PAN-INTERNATIONAL INDUSTRIAL CORP.	\N
0050f7	VENTURE MANUFACTURING (SINGAPORE) LTD.	\N
0050f8	ENTREGA TECHNOLOGIES, INC.	\N
0050f9	SENSORMATIC ACD	\N
0050fa	OXTEL, LTD.	\N
0050fb	VSK ELECTRONICS	\N
0050fc	EDIMAX TECHNOLOGY CO., LTD.	\N
0050fd	VISIONCOMM CO., LTD.	\N
0050fe	PCTVnet ASA	\N
0050ff	HAKKO ELECTRONICS CO., LTD.	\N
005218	Wuxi Keboda Electron Co.Ltd	\N
006000	XYCOM INC.	\N
006001	InnoSys, Inc.	\N
006002	SCREEN SUBTITLING SYSTEMS, LTD	\N
006003	TERAOKA WEIGH SYSTEM PTE, LTD.	\N
006004	COMPUTADORES MODULARES SA	\N
006005	FEEDBACK DATA LTD.	\N
006006	SOTEC CO., LTD	\N
006007	ACRES GAMING, INC.	\N
006008	3COM CORPORATION	\N
006009	CISCO SYSTEMS, INC.	\N
00600a	SORD COMPUTER CORPORATION	\N
00600b	LOGWARE GmbH	\N
00600c	APPLIED DATA SYSTEMS, INC.	\N
00600d	Digital Logic GmbH	\N
00600e	WAVENET INTERNATIONAL, INC.	\N
00600f	WESTELL, INC.	\N
006010	NETWORK MACHINES, INC.	\N
006011	CRYSTAL SEMICONDUCTOR CORP.	\N
006012	POWER COMPUTING CORPORATION	\N
006013	NETSTAL MASCHINEN AG	\N
006014	EDEC CO., LTD.	\N
006015	NET2NET CORPORATION	\N
006016	CLARIION	\N
006017	TOKIMEC INC.	\N
006018	STELLAR ONE CORPORATION	\N
006019	Roche Diagnostics	\N
00601a	KEITHLEY INSTRUMENTS	\N
00601b	MESA ELECTRONICS	\N
00601c	TELXON CORPORATION	\N
00601d	LUCENT TECHNOLOGIES	\N
00601e	SOFTLAB, INC.	\N
00601f	STALLION TECHNOLOGIES	\N
006020	PIVOTAL NETWORKING, INC.	\N
006021	DSC CORPORATION	\N
006022	VICOM SYSTEMS, INC.	\N
006023	PERICOM SEMICONDUCTOR CORP.	\N
006024	GRADIENT TECHNOLOGIES, INC.	\N
006025	ACTIVE IMAGING PLC	\N
006026	VIKING Modular Solutions	\N
006027	Superior Modular Products	\N
006028	MACROVISION CORPORATION	\N
006029	CARY PERIPHERALS INC.	\N
00602a	SYMICRON COMPUTER COMMUNICATIONS, LTD.	\N
00602b	PEAK AUDIO	\N
00602c	LINX Data Terminals, Inc.	\N
00602d	ALERTON TECHNOLOGIES, INC.	\N
00602e	CYCLADES CORPORATION	\N
00602f	CISCO SYSTEMS, INC.	\N
006030	VILLAGE TRONIC ENTWICKLUNG	\N
006031	HRK SYSTEMS	\N
006032	I-CUBE, INC.	\N
006033	ACUITY IMAGING, INC.	\N
006034	ROBERT BOSCH GmbH	\N
006035	DALLAS SEMICONDUCTOR, INC.	\N
006036	AIT Austrian Institute of Technology GmbH	\N
006037	NXP Semiconductors	\N
006038	Nortel Networks	\N
006039	SanCom Technology, Inc.	\N
00603a	QUICK CONTROLS LTD.	\N
00603b	AMTEC spa	\N
00603c	HAGIWARA SYS-COM CO., LTD.	\N
00603d	3CX	\N
00603e	CISCO SYSTEMS, INC.	\N
00603f	PATAPSCO DESIGNS	\N
006040	NETRO CORP.	\N
006041	Yokogawa Electric Corporation	\N
006042	TKS (USA), INC.	\N
006043	iDirect, INC.	\N
006044	LITTON/POLY-SCIENTIFIC	\N
006045	PATHLIGHT TECHNOLOGIES	\N
006046	VMETRO, INC.	\N
006047	CISCO SYSTEMS, INC.	\N
006048	EMC CORPORATION	\N
006049	VINA TECHNOLOGIES	\N
00604a	SAIC IDEAS GROUP	\N
00604b	Safe-com GmbH & Co. KG	\N
00604c	SAGEM COMMUNICATION	\N
00604d	MMC NETWORKS, INC.	\N
00604e	CYCLE COMPUTER CORPORATION, INC.	\N
00604f	SUZUKI MFG. CO., LTD.	\N
006050	INTERNIX INC.	\N
006051	QUALITY SEMICONDUCTOR	\N
006052	PERIPHERALS ENTERPRISE CO., Ltd.	\N
006053	TOYODA MACHINE WORKS, LTD.	\N
006054	CONTROLWARE GMBH	\N
006055	CORNELL UNIVERSITY	\N
006056	NETWORK TOOLS, INC.	\N
006057	MURATA MANUFACTURING CO., LTD.	\N
006058	COPPER MOUNTAIN COMMUNICATIONS, INC.	\N
006059	TECHNICAL COMMUNICATIONS CORP.	\N
00605a	CELCORE, INC.	\N
00605b	IntraServer Technology, Inc.	\N
00605c	CISCO SYSTEMS, INC.	\N
00605d	SCANIVALVE CORP.	\N
00605e	LIBERTY TECHNOLOGY NETWORKING	\N
00605f	NIPPON UNISOFT CORPORATION	\N
006060	DAWNING TECHNOLOGIES, INC.	\N
006061	WHISTLE COMMUNICATIONS CORP.	\N
006062	TELESYNC, INC.	\N
006063	PSION DACOM PLC.	\N
006064	NETCOMM LIMITED	\N
006065	BERNECKER & RAINER INDUSTRIE-ELEKTRONIC GmbH	\N
006066	LACROIX TECHNOLGIE	\N
006067	ACER NETXUS INC.	\N
006068	Dialogic Corporation	\N
006069	Brocade Communications Systems, Inc.	\N
00606a	MITSUBISHI WIRELESS COMMUNICATIONS. INC.	\N
00606b	Synclayer Inc.	\N
00606c	ARESCOM	\N
00606d	DIGITAL EQUIPMENT CORP.	\N
00606e	DAVICOM SEMICONDUCTOR, INC.	\N
00606f	CLARION CORPORATION OF AMERICA	\N
006070	CISCO SYSTEMS, INC.	\N
006071	MIDAS LAB, INC.	\N
006072	VXL INSTRUMENTS, LIMITED	\N
006073	REDCREEK COMMUNICATIONS, INC.	\N
006074	QSC AUDIO PRODUCTS	\N
006075	PENTEK, INC.	\N
006076	SCHLUMBERGER TECHNOLOGIES RETAIL PETROLEUM SYSTEMS	\N
006077	PRISA NETWORKS	\N
006078	POWER MEASUREMENT LTD.	\N
006079	Mainstream Data, Inc.	\N
00607a	DVS GmbH	\N
00607b	FORE SYSTEMS, INC.	\N
00607c	WaveAccess, Ltd.	\N
00607d	SENTIENT NETWORKS INC.	\N
00607e	GIGALABS, INC.	\N
00607f	AURORA TECHNOLOGIES, INC.	\N
006080	MICROTRONIX DATACOM LTD.	\N
006081	TV/COM INTERNATIONAL	\N
006082	NOVALINK TECHNOLOGIES, INC.	\N
006083	CISCO SYSTEMS, INC.	\N
006084	DIGITAL VIDEO	\N
006085	Storage Concepts	\N
006086	LOGIC REPLACEMENT TECH. LTD.	\N
006087	KANSAI ELECTRIC CO., LTD.	\N
006088	WHITE MOUNTAIN DSP, INC.	\N
006089	XATA	\N
00608a	CITADEL COMPUTER	\N
00608b	ConferTech International	\N
00608c	3COM CORPORATION	\N
00608d	UNIPULSE CORP.	\N
00608e	HE ELECTRONICS, TECHNOLOGIE & SYSTEMTECHNIK GmbH	\N
00608f	TEKRAM TECHNOLOGY CO., LTD.	\N
006090	ABLE COMMUNICATIONS, INC.	\N
006091	FIRST PACIFIC NETWORKS, INC.	\N
006092	MICRO/SYS, INC.	\N
006093	VARIAN	\N
006094	IBM CORP.	\N
006095	ACCU-TIME SYSTEMS, INC.	\N
006096	T.S. MICROTECH INC.	\N
006097	3COM CORPORATION	\N
006098	HT COMMUNICATIONS	\N
006099	SBE, Inc.	\N
00609a	NJK TECHNO CO.	\N
00609b	ASTRO-MED, INC.	\N
00609c	Perkin-Elmer Incorporated	\N
00609d	PMI FOOD EQUIPMENT GROUP	\N
00609e	ASC X3 - INFORMATION TECHNOLOGY STANDARDS SECRETARIATS	\N
00609f	PHAST CORPORATION	\N
0060a0	SWITCHED NETWORK TECHNOLOGIES, INC.	\N
0060a1	VPNet, Inc.	\N
0060a2	NIHON UNISYS LIMITED CO.	\N
0060a3	CONTINUUM TECHNOLOGY CORP.	\N
0060a4	GRINAKER SYSTEM TECHNOLOGIES	\N
0060a5	PERFORMANCE TELECOM CORP.	\N
0060a6	PARTICLE MEASURING SYSTEMS	\N
0060a7	MICROSENS GmbH & CO. KG	\N
0060a8	TIDOMAT AB	\N
0060a9	GESYTEC MbH	\N
0060aa	INTELLIGENT DEVICES INC. (IDI)	\N
0060ab	LARSCOM INCORPORATED	\N
0060ac	RESILIENCE CORPORATION	\N
0060ad	MegaChips Corporation	\N
0060ae	TRIO INFORMATION SYSTEMS AB	\N
0060af	PACIFIC MICRO DATA, INC.	\N
0060b0	HEWLETT-PACKARD CO.	\N
0060b1	INPUT/OUTPUT, INC.	\N
0060b2	PROCESS CONTROL CORP.	\N
0060b3	Z-COM, INC.	\N
0060b4	GLENAYRE R&D INC.	\N
0060b5	KEBA GmbH	\N
0060b6	LAND COMPUTER CO., LTD.	\N
0060b7	CHANNELMATIC, INC.	\N
0060b8	CORELIS Inc.	\N
0060b9	NEC Infrontia Corporation	\N
0060ba	SAHARA NETWORKS, INC.	\N
0060bb	CABLETRON - NETLINK, INC.	\N
0060bc	KeunYoung Electronics & Communication Co., Ltd.	\N
0060bd	HUBBELL-PULSECOM	\N
0060be	WEBTRONICS	\N
0060bf	MACRAIGOR SYSTEMS, INC.	\N
0060c0	Nera Networks AS	\N
0060c1	WaveSpan Corporation	\N
0060c2	MPL AG	\N
0060c3	NETVISION CORPORATION	\N
0060c4	SOLITON SYSTEMS K.K.	\N
0060c5	ANCOT CORP.	\N
0060c6	DCS AG	\N
0060c7	AMATI COMMUNICATIONS CORP.	\N
0060c8	KUKA WELDING SYSTEMS & ROBOTS	\N
0060c9	ControlNet, Inc.	\N
0060ca	HARMONIC SYSTEMS INCORPORATED	\N
0060cb	HITACHI ZOSEN CORPORATION	\N
0060cc	EMTRAK, INCORPORATED	\N
0060cd	VideoServer, Inc.	\N
0060ce	ACCLAIM COMMUNICATIONS	\N
0060cf	ALTEON NETWORKS, INC.	\N
0060d0	SNMP RESEARCH INCORPORATED	\N
0060d1	CASCADE COMMUNICATIONS	\N
0060d2	LUCENT TECHNOLOGIES TAIWAN TELECOMMUNICATIONS CO., LTD.	\N
0060d3	AT&T	\N
0060d4	ELDAT COMMUNICATION LTD.	\N
0060d5	MIYACHI TECHNOS CORP.	\N
0060d6	NovAtel Wireless Technologies Ltd.	\N
0060d7	ECOLE POLYTECHNIQUE FEDERALE DE LAUSANNE (EPFL)	\N
0060d8	ELMIC SYSTEMS, INC.	\N
0060d9	TRANSYS NETWORKS INC.	\N
0060da	JBM ELECTRONICS CO.	\N
0060db	NTP ELEKTRONIK A/S	\N
0060dc	Toyo Network Systems  & System Integration Co. LTD	\N
0060dd	MYRICOM, INC.	\N
0060de	Kayser-Threde GmbH	\N
0060df	Brocade Communications Systems, Inc.	\N
0060e0	AXIOM TECHNOLOGY CO., LTD.	\N
0060e1	ORCKIT COMMUNICATIONS LTD.	\N
0060e2	QUEST ENGINEERING & DEVELOPMENT	\N
0060e3	ARBIN INSTRUMENTS	\N
0060e4	COMPUSERVE, INC.	\N
0060e5	FUJI AUTOMATION CO., LTD.	\N
0060e6	SHOMITI SYSTEMS INCORPORATED	\N
0060e7	RANDATA	\N
0060e8	HITACHI COMPUTER PRODUCTS (AMERICA), INC.	\N
0060e9	ATOP TECHNOLOGIES, INC.	\N
0060ea	StreamLogic	\N
0060eb	FOURTHTRACK SYSTEMS	\N
0060ec	HERMARY OPTO ELECTRONICS INC.	\N
0060ed	RICARDO TEST AUTOMATION LTD.	\N
0060ee	APOLLO	\N
0060ef	FLYTECH TECHNOLOGY CO., LTD.	\N
0060f0	JOHNSON & JOHNSON MEDICAL, INC	\N
0060f1	EXP COMPUTER, INC.	\N
0060f2	LASERGRAPHICS, INC.	\N
0060f3	Performance Analysis Broadband, Spirent plc	\N
0060f4	ADVANCED COMPUTER SOLUTIONS, Inc.	\N
0060f5	ICON WEST, INC.	\N
0060f6	NEXTEST COMMUNICATIONS PRODUCTS, INC.	\N
0060f7	DATAFUSION SYSTEMS	\N
0060f8	Loran International Technologies Inc.	\N
0060f9	DIAMOND LANE COMMUNICATIONS	\N
0060fa	EDUCATIONAL TECHNOLOGY RESOURCES, INC.	\N
0060fb	PACKETEER, INC.	\N
0060fc	CONSERVATION THROUGH INNOVATION LTD.	\N
0060fd	NetICs, Inc.	\N
0060fe	LYNX SYSTEM DEVELOPERS, INC.	\N
0060ff	QuVis, Inc.	\N
006440	Cisco Systems	\N
0070b0	M/A-COM INC. COMPANIES	\N
0070b3	DATA RECALL LTD.	\N
008000	MULTITECH SYSTEMS, INC.	\N
008001	PERIPHONICS CORPORATION	\N
008002	SATELCOM (UK) LTD	\N
008003	HYTEC ELECTRONICS LTD.	\N
008004	ANTLOW COMMUNICATIONS, LTD.	\N
008005	CACTUS COMPUTER INC.	\N
008006	COMPUADD CORPORATION	\N
008007	DLOG NC-SYSTEME	\N
008008	DYNATECH COMPUTER SYSTEMS	\N
008009	JUPITER SYSTEMS, INC.	\N
00800a	JAPAN COMPUTER CORP.	\N
00800b	CSK CORPORATION	\N
00800c	VIDECOM LIMITED	\N
00800d	VOSSWINKEL F.U.	\N
00800e	ATLANTIX CORPORATION	\N
00800f	STANDARD MICROSYSTEMS	\N
008010	COMMODORE INTERNATIONAL	\N
008011	DIGITAL SYSTEMS INT'L. INC.	\N
008012	INTEGRATED MEASUREMENT SYSTEMS	\N
008013	THOMAS-CONRAD CORPORATION	\N
008014	ESPRIT SYSTEMS	\N
008015	SEIKO SYSTEMS, INC.	\N
008016	WANDEL AND GOLTERMANN	\N
008017	PFU LIMITED	\N
008018	KOBE STEEL, LTD.	\N
008019	DAYNA COMMUNICATIONS, INC.	\N
00801a	BELL ATLANTIC	\N
00801b	KODIAK TECHNOLOGY	\N
00801c	NEWPORT SYSTEMS SOLUTIONS	\N
00801d	INTEGRATED INFERENCE MACHINES	\N
00801e	XINETRON, INC.	\N
00801f	KRUPP ATLAS ELECTRONIK GMBH	\N
008020	NETWORK PRODUCTS	\N
008021	Alcatel Canada Inc.	\N
008022	SCAN-OPTICS	\N
008023	INTEGRATED BUSINESS NETWORKS	\N
008024	KALPANA, INC.	\N
008025	STOLLMANN GMBH	\N
008026	NETWORK PRODUCTS CORPORATION	\N
008027	ADAPTIVE SYSTEMS, INC.	\N
008028	TRADPOST (HK) LTD	\N
008029	EAGLE TECHNOLOGY, INC.	\N
00802a	TEST SYSTEMS & SIMULATIONS INC	\N
00802b	INTEGRATED MARKETING CO	\N
00802c	THE SAGE GROUP PLC	\N
00802d	XYLOGICS INC	\N
00802e	CASTLE ROCK COMPUTING	\N
00802f	NATIONAL INSTRUMENTS CORP.	\N
008030	NEXUS ELECTRONICS	\N
008031	BASYS, CORP.	\N
008032	ACCESS CO., LTD.	\N
008033	FORMATION, INC.	\N
008034	SMT GOUPIL	\N
008035	TECHNOLOGY WORKS, INC.	\N
008036	REFLEX MANUFACTURING SYSTEMS	\N
008037	Ericsson Group	\N
008038	DATA RESEARCH & APPLICATIONS	\N
008039	ALCATEL STC AUSTRALIA	\N
00803a	VARITYPER, INC.	\N
00803b	APT COMMUNICATIONS, INC.	\N
00803c	TVS ELECTRONICS LTD	\N
00803d	SURIGIKEN CO.,  LTD.	\N
00803e	SYNERNETICS	\N
00803f	TATUNG COMPANY	\N
008040	JOHN FLUKE MANUFACTURING CO.	\N
008041	VEB KOMBINAT ROBOTRON	\N
008042	Emerson Network Power	\N
008043	NETWORLD, INC.	\N
008044	SYSTECH COMPUTER CORP.	\N
008045	MATSUSHITA ELECTRIC IND. CO	\N
008046	UNIVERSITY OF TORONTO	\N
008047	IN-NET CORP.	\N
008048	COMPEX INCORPORATED	\N
008049	NISSIN ELECTRIC CO., LTD.	\N
00804a	PRO-LOG	\N
00804b	EAGLE TECHNOLOGIES PTY.LTD.	\N
00804c	CONTEC CO., LTD.	\N
00804d	CYCLONE MICROSYSTEMS, INC.	\N
00804e	APEX COMPUTER COMPANY	\N
00804f	DAIKIN INDUSTRIES, LTD.	\N
008050	ZIATECH CORPORATION	\N
008051	FIBERMUX	\N
008052	TECHNICALLY ELITE CONCEPTS	\N
008053	INTELLICOM, INC.	\N
008054	FRONTIER TECHNOLOGIES CORP.	\N
008055	FERMILAB	\N
008056	SPHINX ELEKTRONIK GMBH	\N
008057	ADSOFT, LTD.	\N
008058	PRINTER SYSTEMS CORPORATION	\N
008059	STANLEY ELECTRIC CO., LTD	\N
00805a	TULIP COMPUTERS INTERNAT'L B.V	\N
00805b	CONDOR SYSTEMS, INC.	\N
00805c	AGILIS CORPORATION	\N
00805d	CANSTAR	\N
00805e	LSI LOGIC CORPORATION	\N
00805f	Hewlett Packard	\N
008060	NETWORK INTERFACE CORPORATION	\N
008061	LITTON SYSTEMS, INC.	\N
008062	INTERFACE  CO.	\N
008063	RICHARD HIRSCHMANN GMBH & CO.	\N
008064	WYSE TECHNOLOGY	\N
008065	CYBERGRAPHIC SYSTEMS PTY LTD.	\N
008066	ARCOM CONTROL SYSTEMS, LTD.	\N
008067	SQUARE D COMPANY	\N
008068	YAMATECH SCIENTIFIC LTD.	\N
008069	COMPUTONE SYSTEMS	\N
00806a	ERI (EMPAC RESEARCH INC.)	\N
00806b	SCHMID TELECOMMUNICATION	\N
00806c	CEGELEC PROJECTS LTD	\N
00806d	CENTURY SYSTEMS CORP.	\N
00806e	NIPPON STEEL CORPORATION	\N
00806f	ONELAN LTD.	\N
008070	COMPUTADORAS MICRON	\N
008071	SAI TECHNOLOGY	\N
008072	MICROPLEX SYSTEMS LTD.	\N
008073	DWB ASSOCIATES	\N
008074	FISHER CONTROLS	\N
008075	PARSYTEC GMBH	\N
008076	MCNC	\N
008077	BROTHER INDUSTRIES, LTD.	\N
008078	PRACTICAL PERIPHERALS, INC.	\N
008079	MICROBUS DESIGNS LTD.	\N
00807a	AITECH SYSTEMS LTD.	\N
00807b	ARTEL COMMUNICATIONS CORP.	\N
00807c	FIBERCOM, INC.	\N
00807d	EQUINOX SYSTEMS INC.	\N
00807e	SOUTHERN PACIFIC LTD.	\N
00807f	DY-4 INCORPORATED	\N
008080	DATAMEDIA CORPORATION	\N
008081	KENDALL SQUARE RESEARCH CORP.	\N
008082	PEP MODULAR COMPUTERS GMBH	\N
008083	AMDAHL	\N
008084	THE CLOUD INC.	\N
008085	H-THREE SYSTEMS CORPORATION	\N
008086	COMPUTER GENERATION INC.	\N
008087	OKI ELECTRIC INDUSTRY CO., LTD	\N
008088	VICTOR COMPANY OF JAPAN, LTD.	\N
008089	TECNETICS (PTY) LTD.	\N
00808a	SUMMIT MICROSYSTEMS CORP.	\N
00808b	DACOLL LIMITED	\N
00808c	NetScout Systems, Inc.	\N
00808d	WESTCOAST TECHNOLOGY B.V.	\N
00808e	RADSTONE TECHNOLOGY	\N
00808f	C. ITOH ELECTRONICS, INC.	\N
008090	MICROTEK INTERNATIONAL, INC.	\N
008091	TOKYO ELECTRIC CO.,LTD	\N
008092	Silex Technology, Inc.	\N
008093	XYRON CORPORATION	\N
008094	ALFA LAVAL AUTOMATION AB	\N
008095	BASIC MERTON HANDELSGES.M.B.H.	\N
008096	HUMAN DESIGNED SYSTEMS, INC.	\N
008097	CENTRALP AUTOMATISMES	\N
008098	TDK CORPORATION	\N
008099	KLOCKNER MOELLER IPC	\N
00809a	NOVUS NETWORKS LTD	\N
00809b	JUSTSYSTEM CORPORATION	\N
00809c	LUXCOM, INC.	\N
00809d	Commscraft Ltd.	\N
00809e	DATUS GMBH	\N
00809f	ALCATEL BUSINESS SYSTEMS	\N
0080a0	EDISA HEWLETT PACKARD S/A	\N
0080a1	MICROTEST, INC.	\N
0080a2	CREATIVE ELECTRONIC SYSTEMS	\N
0080a3	Lantronix	\N
0080a4	LIBERTY ELECTRONICS	\N
0080a5	SPEED INTERNATIONAL	\N
0080a6	REPUBLIC TECHNOLOGY, INC.	\N
0080a7	MEASUREX CORP.	\N
0080a8	VITACOM CORPORATION	\N
0080a9	CLEARPOINT RESEARCH	\N
0080aa	MAXPEED	\N
0080ab	DUKANE NETWORK INTEGRATION	\N
0080ac	IMLOGIX, DIVISION OF GENESYS	\N
0080ad	CNET TECHNOLOGY, INC.	\N
0080ae	HUGHES NETWORK SYSTEMS	\N
0080af	ALLUMER CO., LTD.	\N
0080b0	ADVANCED INFORMATION	\N
0080b1	SOFTCOM A/S	\N
0080b2	NETWORK EQUIPMENT TECHNOLOGIES	\N
0080b3	AVAL DATA CORPORATION	\N
0080b4	SOPHIA SYSTEMS	\N
0080b5	UNITED NETWORKS INC.	\N
0080b6	THEMIS COMPUTER	\N
0080b7	STELLAR COMPUTER	\N
0080b8	BUG, INCORPORATED	\N
0080b9	ARCHE TECHNOLIGIES INC.	\N
0080ba	SPECIALIX (ASIA) PTE, LTD	\N
0080bb	HUGHES LAN SYSTEMS	\N
0080bc	HITACHI ENGINEERING CO., LTD	\N
0080bd	THE FURUKAWA ELECTRIC CO., LTD	\N
0080be	ARIES RESEARCH	\N
0080bf	TAKAOKA ELECTRIC MFG. CO. LTD.	\N
0080c0	PENRIL DATACOMM	\N
0080c1	LANEX CORPORATION	\N
0080c2	IEEE 802.1 COMMITTEE	\N
0080c3	BICC INFORMATION SYSTEMS & SVC	\N
0080c4	DOCUMENT TECHNOLOGIES, INC.	\N
0080c5	NOVELLCO DE MEXICO	\N
0080c6	NATIONAL DATACOMM CORPORATION	\N
0080c7	XIRCOM	\N
0080c8	D-LINK SYSTEMS, INC.	\N
0080c9	ALBERTA MICROELECTRONIC CENTRE	\N
0080ca	NETCOM RESEARCH INCORPORATED	\N
0080cb	FALCO DATA PRODUCTS	\N
0080cc	MICROWAVE BYPASS SYSTEMS	\N
0080cd	MICRONICS COMPUTER, INC.	\N
0080ce	BROADCAST TELEVISION SYSTEMS	\N
0080cf	EMBEDDED PERFORMANCE INC.	\N
0080d0	COMPUTER PERIPHERALS, INC.	\N
0080d1	KIMTRON CORPORATION	\N
0080d2	SHINNIHONDENKO CO., LTD.	\N
0080d3	SHIVA CORP.	\N
0080d4	CHASE RESEARCH LTD.	\N
0080d5	CADRE TECHNOLOGIES	\N
0080d6	NUVOTECH, INC.	\N
0080d7	Fantum Engineering	\N
0080d8	NETWORK PERIPHERALS INC.	\N
0080d9	EMK Elektronik GmbH & Co. KG	\N
0080da	BRUEL & KJAER	\N
0080db	GRAPHON CORPORATION	\N
0080dc	PICKER INTERNATIONAL	\N
0080dd	GMX INC/GIMIX	\N
0080de	GIPSI S.A.	\N
0080df	ADC CODENOLL TECHNOLOGY CORP.	\N
0080e0	XTP SYSTEMS, INC.	\N
0080e1	STMICROELECTRONICS	\N
0080e2	T.D.I. CO., LTD.	\N
0080e3	CORAL NETWORK CORPORATION	\N
0080e4	NORTHWEST DIGITAL SYSTEMS, INC	\N
0080e5	LSI Logic Corporation	\N
0080e6	PEER NETWORKS, INC.	\N
0080e7	LYNWOOD SCIENTIFIC DEV. LTD.	\N
0080e8	CUMULUS CORPORATIION	\N
0080e9	Madge Ltd.	\N
0080ea	ADVA Optical Networking Ltd.	\N
0080eb	COMPCONTROL B.V.	\N
0080ec	SUPERCOMPUTING SOLUTIONS, INC.	\N
0080ed	IQ TECHNOLOGIES, INC.	\N
0080ee	THOMSON CSF	\N
0080ef	RATIONAL	\N
0080f0	Panasonic Communications Co., Ltd.	\N
0080f1	OPUS SYSTEMS	\N
0080f2	RAYCOM SYSTEMS INC	\N
0080f3	SUN ELECTRONICS CORP.	\N
0080f4	TELEMECANIQUE ELECTRIQUE	\N
0080f5	Quantel Ltd	\N
0080f6	SYNERGY MICROSYSTEMS	\N
0080f7	ZENITH ELECTRONICS	\N
0080f8	MIZAR, INC.	\N
0080f9	HEURIKON CORPORATION	\N
0080fa	RWT GMBH	\N
0080fb	BVM LIMITED	\N
0080fc	AVATAR CORPORATION	\N
0080fd	EXSCEED CORPRATION	\N
0080fe	AZURE TECHNOLOGIES, INC.	\N
0080ff	SOC. DE TELEINFORMATIQUE RTC	\N
008cfa	Inventec Corporation	\N
009000	DIAMOND MULTIMEDIA	\N
009001	NISHIMU ELECTRONICS INDUSTRIES CO., LTD.	\N
009002	ALLGON AB	\N
009003	APLIO	\N
009004	3COM EUROPE LTD.	\N
009005	PROTECH SYSTEMS CO., LTD.	\N
009006	HAMAMATSU PHOTONICS K.K.	\N
009007	DOMEX TECHNOLOGY CORP.	\N
009008	HanA Systems Inc.	\N
009009	i Controls, Inc.	\N
00900a	PROTON ELECTRONIC INDUSTRIAL CO., LTD.	\N
00900b	LANNER ELECTRONICS, INC.	\N
00900c	CISCO SYSTEMS, INC.	\N
00900d	Overland Storage Inc.	\N
00900e	HANDLINK TECHNOLOGIES, INC.	\N
00900f	KAWASAKI HEAVY INDUSTRIES, LTD	\N
009010	SIMULATION LABORATORIES, INC.	\N
009011	WAVTrace, Inc.	\N
009012	GLOBESPAN SEMICONDUCTOR, INC.	\N
009013	SAMSAN CORP.	\N
009014	ROTORK INSTRUMENTS, LTD.	\N
009015	CENTIGRAM COMMUNICATIONS CORP.	\N
009016	ZAC	\N
009017	Zypcom, Inc	\N
009018	ITO ELECTRIC INDUSTRY CO, LTD.	\N
009019	HERMES ELECTRONICS CO., LTD.	\N
00901a	UNISPHERE SOLUTIONS	\N
00901b	DIGITAL CONTROLS	\N
00901c	mps Software Gmbh	\N
00901d	PEC (NZ) LTD.	\N
00901e	Selesta Ingegneria S.p.A.	\N
00901f	ADTEC PRODUCTIONS, INC.	\N
009020	PHILIPS ANALYTICAL X-RAY B.V.	\N
009021	CISCO SYSTEMS, INC.	\N
009022	IVEX	\N
009023	ZILOG INC.	\N
009024	PIPELINKS, INC.	\N
009025	BAE Systems Australia (Electronic Systems) Pty Ltd	\N
009026	ADVANCED SWITCHING COMMUNICATIONS, INC.	\N
009027	INTEL CORPORATION	\N
009028	NIPPON SIGNAL CO., LTD.	\N
009029	CRYPTO AG	\N
00902a	COMMUNICATION DEVICES, INC.	\N
00902b	CISCO SYSTEMS, INC.	\N
00902c	DATA & CONTROL EQUIPMENT LTD.	\N
00902d	DATA ELECTRONICS (AUST.) PTY, LTD.	\N
00902e	NAMCO LIMITED	\N
00902f	NETCORE SYSTEMS, INC.	\N
009030	HONEYWELL-DATING	\N
009031	MYSTICOM, LTD.	\N
009032	PELCOMBE GROUP LTD.	\N
009033	INNOVAPHONE AG	\N
009034	IMAGIC, INC.	\N
009035	ALPHA TELECOM, INC.	\N
009036	ens, inc.	\N
009037	ACUCOMM, INC.	\N
009038	FOUNTAIN TECHNOLOGIES, INC.	\N
009039	SHASTA NETWORKS	\N
00903a	NIHON MEDIA TOOL INC.	\N
00903b	TriEMS Research Lab, Inc.	\N
00903c	ATLANTIC NETWORK SYSTEMS	\N
00903d	BIOPAC SYSTEMS, INC.	\N
00903e	N.V. PHILIPS INDUSTRIAL ACTIVITIES	\N
00903f	AZTEC RADIOMEDIA	\N
009040	Siemens Network Convergence LLC	\N
009041	APPLIED DIGITAL ACCESS	\N
009042	ECCS, Inc.	\N
009043	NICHIBEI DENSHI CO., LTD.	\N
009044	ASSURED DIGITAL, INC.	\N
009045	Marconi Communications	\N
009046	DEXDYNE, LTD.	\N
009047	GIGA FAST E. LTD.	\N
009048	ZEAL CORPORATION	\N
009049	ENTRIDIA CORPORATION	\N
00904a	CONCUR SYSTEM TECHNOLOGIES	\N
00904b	GemTek Technology Co., Ltd.	\N
00904c	EPIGRAM, INC.	\N
00904d	SPEC S.A.	\N
00904e	DELEM BV	\N
00904f	ABB POWER T&D COMPANY, INC.	\N
009050	TELESTE OY	\N
009051	ULTIMATE TECHNOLOGY CORP.	\N
009052	SELCOM ELETTRONICA S.R.L.	\N
009053	DAEWOO ELECTRONICS CO., LTD.	\N
009054	INNOVATIVE SEMICONDUCTORS, INC	\N
009055	PARKER HANNIFIN CORPORATION COMPUMOTOR DIVISION	\N
009056	TELESTREAM, INC.	\N
009057	AANetcom, Inc.	\N
009058	Ultra Electronics Ltd., Command and Control Systems	\N
009059	TELECOM DEVICE K.K.	\N
00905a	DEARBORN GROUP, INC.	\N
00905b	RAYMOND AND LAE ENGINEERING	\N
00905c	EDMI	\N
00905d	NETCOM SICHERHEITSTECHNIK GmbH	\N
00905e	RAULAND-BORG CORPORATION	\N
00905f	CISCO SYSTEMS, INC.	\N
009060	SYSTEM CREATE CORP.	\N
009061	PACIFIC RESEARCH & ENGINEERING CORPORATION	\N
009062	ICP VORTEX COMPUTERSYSTEME GmbH	\N
009063	COHERENT COMMUNICATIONS SYSTEMS CORPORATION	\N
009064	Thomson Inc.	\N
009065	FINISAR CORPORATION	\N
009066	Troika Networks, Inc.	\N
009067	WalkAbout Computers, Inc.	\N
009068	DVT CORP.	\N
009069	JUNIPER NETWORKS, INC.	\N
00906a	TURNSTONE SYSTEMS, INC.	\N
00906b	APPLIED RESOURCES, INC.	\N
00906c	Sartorius Hamburg GmbH	\N
00906d	CISCO SYSTEMS, INC.	\N
00906e	PRAXON, INC.	\N
00906f	CISCO SYSTEMS, INC.	\N
009070	NEO NETWORKS, INC.	\N
009071	Applied Innovation Inc.	\N
009072	SIMRAD AS	\N
009073	GAIO TECHNOLOGY	\N
009074	ARGON NETWORKS, INC.	\N
009075	NEC DO BRASIL S.A.	\N
009076	FMT AIRCRAFT GATE SUPPORT SYSTEMS AB	\N
009077	ADVANCED FIBRE COMMUNICATIONS	\N
009078	MER TELEMANAGEMENT SOLUTIONS, LTD.	\N
009079	ClearOne, Inc.	\N
00907a	Polycom, Inc.	\N
00907b	E-TECH, INC.	\N
00907c	DIGITALCAST, INC.	\N
00907d	Lake Communications	\N
00907e	VETRONIX CORP.	\N
00907f	WatchGuard Technologies, Inc.	\N
009080	NOT LIMITED, INC.	\N
009081	ALOHA NETWORKS, INC.	\N
009082	FORCE INSTITUTE	\N
009083	TURBO COMMUNICATION, INC.	\N
009084	ATECH SYSTEM	\N
009085	GOLDEN ENTERPRISES, INC.	\N
009086	CISCO SYSTEMS, INC.	\N
009087	ITIS	\N
009088	BAXALL SECURITY LTD.	\N
009089	SOFTCOM MICROSYSTEMS, INC.	\N
00908a	BAYLY COMMUNICATIONS, INC.	\N
00908b	PFU Systems, Inc.	\N
00908c	ETREND ELECTRONICS, INC.	\N
00908d	VICKERS ELECTRONICS SYSTEMS	\N
00908e	Nortel Networks Broadband Access	\N
00908f	AUDIO CODES LTD.	\N
009090	I-BUS	\N
009091	DigitalScape, Inc.	\N
009092	CISCO SYSTEMS, INC.	\N
009093	NANAO CORPORATION	\N
009094	OSPREY TECHNOLOGIES, INC.	\N
009095	UNIVERSAL AVIONICS	\N
009096	ASKEY COMPUTER CORP.	\N
009097	Sycamore Networks	\N
009098	SBC DESIGNS, INC.	\N
009099	ALLIED TELESIS, K.K.	\N
00909a	ONE WORLD SYSTEMS, INC.	\N
00909b	IMAJE	\N
00909c	Motorola, Inc.	\N
00909d	NovaTech Process Solutions, LLC	\N
00909e	Critical IO, LLC	\N
00909f	DIGI-DATA CORPORATION	\N
0090a0	8X8 INC.	\N
0090a1	Flying Pig Systems/High End Systems Inc.	\N
0090a2	CYBERTAN TECHNOLOGY, INC.	\N
0090a3	Corecess Inc.	\N
0090a4	ALTIGA NETWORKS	\N
0090a5	SPECTRA LOGIC	\N
0090a6	CISCO SYSTEMS, INC.	\N
0090a7	CLIENTEC CORPORATION	\N
0090a8	NineTiles Networks, Ltd.	\N
0090a9	WESTERN DIGITAL	\N
0090aa	INDIGO ACTIVE VISION SYSTEMS LIMITED	\N
0090ab	CISCO SYSTEMS, INC.	\N
0090ac	OPTIVISION, INC.	\N
0090ad	ASPECT ELECTRONICS, INC.	\N
0090ae	ITALTEL S.p.A.	\N
0090af	J. MORITA MFG. CORP.	\N
0090b0	VADEM	\N
0090b1	CISCO SYSTEMS, INC.	\N
0090b2	AVICI SYSTEMS INC.	\N
0090b3	AGRANAT SYSTEMS	\N
0090b4	WILLOWBROOK TECHNOLOGIES	\N
0090b5	NIKON CORPORATION	\N
0090b6	FIBEX SYSTEMS	\N
0090b7	DIGITAL LIGHTWAVE, INC.	\N
0090b8	ROHDE & SCHWARZ GMBH & CO. KG	\N
0090b9	BERAN INSTRUMENTS LTD.	\N
0090ba	VALID NETWORKS, INC.	\N
0090bb	TAINET COMMUNICATION SYSTEM Corp.	\N
0090bc	TELEMANN CO., LTD.	\N
0090bd	OMNIA COMMUNICATIONS, INC.	\N
0090be	IBC/INTEGRATED BUSINESS COMPUTERS	\N
0090bf	CISCO SYSTEMS, INC.	\N
0090c0	K.J. LAW ENGINEERS, INC.	\N
0090c1	Peco II, Inc.	\N
0090c2	JK microsystems, Inc.	\N
0090c3	TOPIC SEMICONDUCTOR CORP.	\N
0090c4	JAVELIN SYSTEMS, INC.	\N
0090c5	INTERNET MAGIC, INC.	\N
0090c6	OPTIM SYSTEMS, INC.	\N
0090c7	ICOM INC.	\N
0090c8	WAVERIDER COMMUNICATIONS (CANADA) INC.	\N
0090c9	DPAC Technologies	\N
0090ca	ACCORD VIDEO TELECOMMUNICATIONS, LTD.	\N
0090cb	Wireless OnLine, Inc.	\N
0090cc	Planex Communications	\N
0090cd	ENT-EMPRESA NACIONAL DE TELECOMMUNICACOES, S.A.	\N
0090ce	TETRA GmbH	\N
0090cf	NORTEL	\N
0090d0	Thomson Telecom Belgium	\N
0090d1	LEICHU ENTERPRISE CO., LTD.	\N
0090d2	ARTEL VIDEO SYSTEMS	\N
0090d3	GIESECKE & DEVRIENT GmbH	\N
0090d4	BindView Development Corp.	\N
0090d5	EUPHONIX, INC.	\N
0090d6	CRYSTAL GROUP	\N
0090d7	NetBoost Corp.	\N
0090d8	WHITECROSS SYSTEMS	\N
0090d9	CISCO SYSTEMS, INC.	\N
0090da	DYNARC, INC.	\N
0090db	NEXT LEVEL COMMUNICATIONS	\N
0090dc	TECO INFORMATION SYSTEMS	\N
0090dd	THE MIHARU COMMUNICATIONS CO., LTD.	\N
0090de	CARDKEY SYSTEMS, INC.	\N
0090df	MITSUBISHI CHEMICAL AMERICA, INC.	\N
0090e0	SYSTRAN CORP.	\N
0090e1	TELENA S.P.A.	\N
0090e2	DISTRIBUTED PROCESSING TECHNOLOGY	\N
0090e3	AVEX ELECTRONICS INC.	\N
0090e4	NEC AMERICA, INC.	\N
0090e5	TEKNEMA, INC.	\N
0090e6	ALi Corporation	\N
0090e7	HORSCH ELEKTRONIK AG	\N
0090e8	MOXA TECHNOLOGIES CORP., LTD.	\N
0090e9	JANZ COMPUTER AG	\N
0090ea	ALPHA TECHNOLOGIES, INC.	\N
0090eb	SENTRY TELECOM SYSTEMS	\N
0090ec	PYRESCOM	\N
0090ed	CENTRAL SYSTEM RESEARCH CO., LTD.	\N
0090ee	PERSONAL COMMUNICATIONS TECHNOLOGIES	\N
0090ef	INTEGRIX, INC.	\N
0090f0	Harmonic Video Systems Ltd.	\N
0090f1	DOT HILL SYSTEMS CORPORATION	\N
0090f2	CISCO SYSTEMS, INC.	\N
0090f3	ASPECT COMMUNICATIONS	\N
0090f4	LIGHTNING INSTRUMENTATION	\N
0090f5	CLEVO CO.	\N
0090f6	ESCALATE NETWORKS, INC.	\N
0090f7	NBASE COMMUNICATIONS LTD.	\N
0090f8	MEDIATRIX TELECOM	\N
0090f9	LEITCH	\N
0090fa	EMULEX Corp	\N
0090fb	PORTWELL, INC.	\N
0090fc	NETWORK COMPUTING DEVICES	\N
0090fd	CopperCom, Inc.	\N
0090fe	ELECOM CO., LTD.  (LANEED DIV.)	\N
0090ff	TELLUS TECHNOLOGY INC.	\N
0091d6	Crystal Group, Inc.	\N
009363	Uni-Link Technology Co., Ltd.	\N
0097ff	Heimann Sensor GmbH	\N
009d8e	CARDIAC RECORDERS, INC.	\N
00a000	CENTILLION NETWORKS, INC.	\N
00a001	DRS Signal Solutions	\N
00a002	LEEDS & NORTHRUP AUSTRALIA PTY LTD	\N
00a003	Siemens Switzerland Ltd., I B T HVP	\N
00a004	NETPOWER, INC.	\N
00a005	DANIEL INSTRUMENTS, LTD.	\N
00a006	IMAGE DATA PROCESSING SYSTEM GROUP	\N
00a007	APEXX TECHNOLOGY, INC.	\N
00a008	NETCORP	\N
00a009	WHITETREE NETWORK	\N
00a00a	Airspan	\N
00a00b	COMPUTEX CO., LTD.	\N
00a00c	KINGMAX TECHNOLOGY, INC.	\N
00a00d	THE PANDA PROJECT	\N
00a00e	VISUAL NETWORKS, INC.	\N
00a00f	Broadband Technologies	\N
00a010	SYSLOGIC DATENTECHNIK AG	\N
00a011	MUTOH INDUSTRIES LTD.	\N
00a012	B.A.T.M. ADVANCED TECHNOLOGIES	\N
00a013	TELTREND LTD.	\N
00a014	CSIR	\N
00a015	WYLE	\N
00a016	MICROPOLIS CORP.	\N
00a017	J B M CORPORATION	\N
00a018	CREATIVE CONTROLLERS, INC.	\N
00a019	NEBULA CONSULTANTS, INC.	\N
00a01a	BINAR ELEKTRONIK AB	\N
00a01b	PREMISYS COMMUNICATIONS, INC.	\N
00a01c	NASCENT NETWORKS CORPORATION	\N
00a01d	SIXNET	\N
00a01e	EST CORPORATION	\N
00a01f	TRICORD SYSTEMS, INC.	\N
00a020	CITICORP/TTI	\N
00a021	General Dynamics	\N
00a022	CENTRE FOR DEVELOPMENT OF ADVANCED COMPUTING	\N
00a023	APPLIED CREATIVE TECHNOLOGY, INC.	\N
00a024	3COM CORPORATION	\N
00a025	REDCOM LABS INC.	\N
00a026	TELDAT, S.A.	\N
00a027	FIREPOWER SYSTEMS, INC.	\N
00a028	CONNER PERIPHERALS	\N
00a029	COULTER CORPORATION	\N
00a02a	TRANCELL SYSTEMS	\N
00a02b	TRANSITIONS RESEARCH CORP.	\N
00a02c	interWAVE Communications	\N
00a02d	1394 Trade Association	\N
00a02e	BRAND COMMUNICATIONS, LTD.	\N
00a02f	PIRELLI CAVI	\N
00a030	CAPTOR NV/SA	\N
00a031	HAZELTINE CORPORATION, MS 1-17	\N
00a032	GES SINGAPORE PTE. LTD.	\N
00a033	imc MeBsysteme GmbH	\N
00a034	AXEL	\N
00a035	CYLINK CORPORATION	\N
00a036	APPLIED NETWORK TECHNOLOGY	\N
00a037	Mindray DS USA, Inc.	\N
00a038	EMAIL ELECTRONICS	\N
00a039	ROSS TECHNOLOGY, INC.	\N
00a03a	KUBOTEK CORPORATION	\N
00a03b	TOSHIN ELECTRIC CO., LTD.	\N
00a03c	EG&G NUCLEAR INSTRUMENTS	\N
00a03d	OPTO-22	\N
00a03e	ATM FORUM	\N
00a03f	COMPUTER SOCIETY MICROPROCESSOR & MICROPROCESSOR STANDARDS C	\N
00a040	APPLE COMPUTER	\N
00a041	INFICON	\N
00a042	SPUR PRODUCTS CORP.	\N
00a043	AMERICAN TECHNOLOGY LABS, INC.	\N
00a044	NTT IT CO., LTD.	\N
00a045	PHOENIX CONTACT GMBH & CO.	\N
00a046	SCITEX CORP. LTD.	\N
00a047	INTEGRATED FITNESS CORP.	\N
00a048	QUESTECH, LTD.	\N
00a049	DIGITECH INDUSTRIES, INC.	\N
00a04a	NISSHIN ELECTRIC CO., LTD.	\N
00a04b	TFL LAN INC.	\N
00a04c	INNOVATIVE SYSTEMS & TECHNOLOGIES, INC.	\N
00a04d	EDA INSTRUMENTS, INC.	\N
00a04e	VOELKER TECHNOLOGIES, INC.	\N
00a04f	AMERITEC CORP.	\N
00a050	CYPRESS SEMICONDUCTOR	\N
00a051	ANGIA COMMUNICATIONS. INC.	\N
00a052	STANILITE ELECTRONICS PTY. LTD	\N
00a053	COMPACT DEVICES, INC.	\N
00a054	PRIVATE	\N
00a055	Data Device Corporation	\N
00a056	MICROPROSS	\N
00a057	LANCOM Systems GmbH	\N
00a058	GLORY, LTD.	\N
00a059	HAMILTON HALLMARK	\N
00a05a	KOFAX IMAGE PRODUCTS	\N
00a05b	MARQUIP, INC.	\N
00a05c	INVENTORY CONVERSION, INC./	\N
00a05d	CS COMPUTER SYSTEME GmbH	\N
00a05e	MYRIAD LOGIC INC.	\N
00a05f	BTG Electronics Design BV	\N
00a060	ACER PERIPHERALS, INC.	\N
00a061	PURITAN BENNETT	\N
00a062	AES PRODATA	\N
00a063	JRL SYSTEMS, INC.	\N
00a064	KVB/ANALECT	\N
00a065	Symantec Corporation	\N
00a066	ISA CO., LTD.	\N
00a067	NETWORK SERVICES GROUP	\N
00a068	BHP LIMITED	\N
00a069	Symmetricom, Inc.	\N
00a06a	Verilink Corporation	\N
00a06b	DMS DORSCH MIKROSYSTEM GMBH	\N
00a06c	SHINDENGEN ELECTRIC MFG. CO., LTD.	\N
00a06d	MANNESMANN TALLY CORPORATION	\N
00a06e	AUSTRON, INC.	\N
00a06f	THE APPCON GROUP, INC.	\N
00a070	COASTCOM	\N
00a071	VIDEO LOTTERY TECHNOLOGIES,INC	\N
00a072	OVATION SYSTEMS LTD.	\N
00a073	COM21, INC.	\N
00a074	PERCEPTION TECHNOLOGY	\N
00a075	MICRON TECHNOLOGY, INC.	\N
00a076	CARDWARE LAB, INC.	\N
00a077	FUJITSU NEXION, INC.	\N
00a078	Marconi Communications	\N
00a079	ALPS ELECTRIC (USA), INC.	\N
00a07a	ADVANCED PERIPHERALS TECHNOLOGIES, INC.	\N
00a07b	DAWN COMPUTER INCORPORATION	\N
00a07c	TONYANG NYLON CO., LTD.	\N
00a07d	SEEQ TECHNOLOGY, INC.	\N
00a07e	AVID TECHNOLOGY, INC.	\N
00a07f	GSM-SYNTEL, LTD.	\N
00a080	SBE, Inc.	\N
00a081	ALCATEL DATA NETWORKS	\N
00a082	NKT ELEKTRONIK A/S	\N
00a083	ASIMMPHONY TURKEY	\N
00a084	Dataplex Pty Ltd	\N
00a085	PRIVATE	\N
00a086	AMBER WAVE SYSTEMS, INC.	\N
00a087	Zarlink Semiconductor Ltd.	\N
00a088	ESSENTIAL COMMUNICATIONS	\N
00a089	XPOINT TECHNOLOGIES, INC.	\N
00a08a	BROOKTROUT TECHNOLOGY, INC.	\N
00a08b	ASTON ELECTRONIC DESIGNS LTD.	\N
00a08c	MultiMedia LANs, Inc.	\N
00a08d	JACOMO CORPORATION	\N
00a08e	Nokia Internet Communications	\N
00a08f	DESKNET SYSTEMS, INC.	\N
00a090	TimeStep Corporation	\N
00a091	APPLICOM INTERNATIONAL	\N
00a092	H. BOLLMANN MANUFACTURERS, LTD	\N
00a093	B/E AEROSPACE, Inc.	\N
00a094	COMSAT CORPORATION	\N
00a095	ACACIA NETWORKS, INC.	\N
00a096	MITUMI ELECTRIC CO., LTD.	\N
00a097	JC INFORMATION SYSTEMS	\N
00a098	NetApp	\N
00a099	K-NET LTD.	\N
00a09a	NIHON KOHDEN AMERICA	\N
00a09b	QPSX COMMUNICATIONS, LTD.	\N
00a09c	Xyplex, Inc.	\N
00a09d	JOHNATHON FREEMAN TECHNOLOGIES	\N
00a09e	ICTV	\N
00a09f	COMMVISION CORP.	\N
00a0a0	COMPACT DATA, LTD.	\N
00a0a1	EPIC DATA INC.	\N
00a0a2	DIGICOM S.P.A.	\N
00a0a3	RELIABLE POWER METERS	\N
00a0a4	MICROS SYSTEMS, INC.	\N
00a0a5	TEKNOR MICROSYSTEME, INC.	\N
00a0a6	M.I. SYSTEMS, K.K.	\N
00a0a7	VORAX CORPORATION	\N
00a0a8	RENEX CORPORATION	\N
00a0a9	NAVTEL COMMUNICATIONS INC.	\N
00a0aa	SPACELABS MEDICAL	\N
00a0ab	NETCS INFORMATIONSTECHNIK GMBH	\N
00a0ac	GILAT SATELLITE NETWORKS, LTD.	\N
00a0ad	MARCONI SPA	\N
00a0ae	NUCOM SYSTEMS, INC.	\N
00a0af	WMS INDUSTRIES	\N
00a0b0	I-O DATA DEVICE, INC.	\N
00a0b1	FIRST VIRTUAL CORPORATION	\N
00a0b2	SHIMA SEIKI	\N
00a0b3	ZYKRONIX	\N
00a0b4	TEXAS MICROSYSTEMS, INC.	\N
00a0b5	3H TECHNOLOGY	\N
00a0b6	SANRITZ AUTOMATION CO., LTD.	\N
00a0b7	CORDANT, INC.	\N
00a0b8	SYMBIOS LOGIC INC.	\N
00a0b9	EAGLE TECHNOLOGY, INC.	\N
00a0ba	PATTON ELECTRONICS CO.	\N
00a0bb	HILAN GMBH	\N
00a0bc	VIASAT, INCORPORATED	\N
00a0bd	I-TECH CORP.	\N
00a0be	INTEGRATED CIRCUIT SYSTEMS, INC. COMMUNICATIONS GROUP	\N
00a0bf	WIRELESS DATA GROUP MOTOROLA	\N
00a0c0	DIGITAL LINK CORP.	\N
00a0c1	ORTIVUS MEDICAL AB	\N
00a0c2	R.A. SYSTEMS CO., LTD.	\N
00a0c3	UNICOMPUTER GMBH	\N
00a0c4	CRISTIE ELECTRONICS LTD.	\N
00a0c5	ZYXEL COMMUNICATION	\N
00a0c6	QUALCOMM INCORPORATED	\N
00a0c7	TADIRAN TELECOMMUNICATIONS	\N
00a0c8	ADTRAN INC.	\N
00a0c9	INTEL CORPORATION - HF1-06	\N
00a0ca	FUJITSU DENSO LTD.	\N
00a0cb	ARK TELECOMMUNICATIONS, INC.	\N
00a0cc	LITE-ON COMMUNICATIONS, INC.	\N
00a0cd	DR. JOHANNES HEIDENHAIN GmbH	\N
00a0ce	ASTROCOM CORPORATION	\N
00a0cf	SOTAS, INC.	\N
00a0d0	TEN X TECHNOLOGY, INC.	\N
00a0d1	INVENTEC CORPORATION	\N
00a0d2	ALLIED TELESIS INTERNATIONAL CORPORATION	\N
00a0d3	INSTEM COMPUTER SYSTEMS, LTD.	\N
00a0d4	RADIOLAN,  INC.	\N
00a0d5	SIERRA WIRELESS INC.	\N
00a0d6	SBE, INC.	\N
00a0d7	KASTEN CHASE APPLIED RESEARCH	\N
00a0d8	SPECTRA - TEK	\N
00a0d9	CONVEX COMPUTER CORPORATION	\N
00a0da	INTEGRATED SYSTEMS Technology, Inc.	\N
00a0db	FISHER & PAYKEL PRODUCTION	\N
00a0dc	O.N. ELECTRONIC CO., LTD.	\N
00a0dd	AZONIX CORPORATION	\N
00a0de	YAMAHA CORPORATION	\N
00a0df	STS TECHNOLOGIES, INC.	\N
00a0e0	TENNYSON TECHNOLOGIES PTY LTD	\N
00a0e1	WESTPORT RESEARCH ASSOCIATES, INC.	\N
00a0e2	Keisokugiken Corporation	\N
00a0e3	XKL SYSTEMS CORP.	\N
00a0e4	OPTIQUEST	\N
00a0e5	NHC COMMUNICATIONS	\N
00a0e6	DIALOGIC CORPORATION	\N
00a0e7	CENTRAL DATA CORPORATION	\N
00a0e8	REUTERS HOLDINGS PLC	\N
00a0e9	ELECTRONIC RETAILING SYSTEMS INTERNATIONAL	\N
00a0ea	ETHERCOM CORP.	\N
00a0eb	Encore Networks, Inc.	\N
00a0ec	TRANSMITTON LTD.	\N
00a0ed	Brooks Automation, Inc.	\N
00a0ee	NASHOBA NETWORKS	\N
00a0ef	LUCIDATA LTD.	\N
00a0f0	TORONTO MICROELECTRONICS INC.	\N
00a0f1	MTI	\N
00a0f2	INFOTEK COMMUNICATIONS, INC.	\N
00a0f3	STAUBLI	\N
00a0f4	GE	\N
00a0f5	RADGUARD LTD.	\N
00a0f6	AutoGas Systems Inc.	\N
00a0f7	V.I COMPUTER CORP.	\N
00a0f8	SYMBOL TECHNOLOGIES, INC.	\N
00a0f9	BINTEC COMMUNICATIONS GMBH	\N
00a0fa	Marconi Communication GmbH	\N
00a0fb	TORAY ENGINEERING CO., LTD.	\N
00a0fc	IMAGE SCIENCES, INC.	\N
00a0fd	SCITEX DIGITAL PRINTING, INC.	\N
00a0fe	BOSTON TECHNOLOGY, INC.	\N
00a0ff	TELLABS OPERATIONS, INC.	\N
00a2da	INAT GmbH	\N
00aa00	INTEL CORPORATION	\N
00aa01	INTEL CORPORATION	\N
00aa02	INTEL CORPORATION	\N
00aa3c	OLIVETTI TELECOM SPA (OLTECO)	\N
00b009	Grass Valley Group	\N
00b017	InfoGear Technology Corp.	\N
00b019	Casi-Rusco	\N
00b01c	Westport Technologies	\N
00b01e	Rantic Labs, Inc.	\N
00b02a	ORSYS GmbH	\N
00b02d	ViaGate Technologies, Inc.	\N
00b03b	HiQ Networks	\N
00b048	Marconi Communications Inc.	\N
00b04a	Cisco Systems, Inc.	\N
00b052	Intellon Corporation	\N
00b064	Cisco Systems, Inc.	\N
00b069	Honewell Oy	\N
00b06d	Jones Futurex Inc.	\N
00b080	Mannesmann Ipulsys B.V.	\N
00b086	LocSoft Limited	\N
00b08e	Cisco Systems, Inc.	\N
00b091	Transmeta Corp.	\N
00b094	Alaris, Inc.	\N
00b09a	Morrow Technologies Corp.	\N
00b09d	Point Grey Research Inc.	\N
00b0ac	SIAE-Microelettronica S.p.A.	\N
00b0ae	Symmetricom	\N
00b0b3	Xstreamis PLC	\N
00b0c2	Cisco Systems, Inc.	\N
00b0c7	Tellabs Operations, Inc.	\N
00b0ce	TECHNOLOGY RESCUE	\N
00b0d0	Dell Computer Corp.	\N
00b0db	Nextcell, Inc.	\N
00b0df	RELDATA Inc	\N
00b0e7	British Federal Ltd.	\N
00b0ec	EACEM	\N
00b0ee	Ajile Systems, Inc.	\N
00b0f0	CALY NETWORKS	\N
00b0f5	NetWorth Technologies, Inc.	\N
00b5d6	Omnibit Inc.	\N
00bac0	Biometric Access Company	\N
00bb01	OCTOTHORPE CORP.	\N
00bbf0	UNGERMANN-BASS INC.	\N
00bd3a	Nokia Corporation	\N
00c000	LANOPTICS, LTD.	\N
00c001	DIATEK PATIENT MANAGMENT	\N
00c002	SERCOMM CORPORATION	\N
00c003	GLOBALNET COMMUNICATIONS	\N
00c004	JAPAN BUSINESS COMPUTER CO.LTD	\N
00c005	LIVINGSTON ENTERPRISES, INC.	\N
00c006	NIPPON AVIONICS CO., LTD.	\N
00c007	PINNACLE DATA SYSTEMS, INC.	\N
00c008	SECO SRL	\N
00c009	KT TECHNOLOGY (S) PTE LTD	\N
00c00a	MICRO CRAFT	\N
00c00b	NORCONTROL A.S.	\N
00c00c	RELIA TECHNOLGIES	\N
00c00d	ADVANCED LOGIC RESEARCH, INC.	\N
00c00e	PSITECH, INC.	\N
00c00f	QUANTUM SOFTWARE SYSTEMS LTD.	\N
00c010	HIRAKAWA HEWTECH CORP.	\N
00c011	INTERACTIVE COMPUTING DEVICES	\N
00c012	NETSPAN CORPORATION	\N
00c013	NETRIX	\N
00c014	TELEMATICS CALABASAS INT'L,INC	\N
00c015	NEW MEDIA CORPORATION	\N
00c016	ELECTRONIC THEATRE CONTROLS	\N
00c017	Fluke Corporation	\N
00c018	LANART CORPORATION	\N
00c019	LEAP TECHNOLOGY, INC.	\N
00c01a	COROMETRICS MEDICAL SYSTEMS	\N
00c01b	SOCKET COMMUNICATIONS, INC.	\N
00c01c	INTERLINK COMMUNICATIONS LTD.	\N
00c01d	GRAND JUNCTION NETWORKS, INC.	\N
00c01e	LA FRANCAISE DES JEUX	\N
00c01f	S.E.R.C.E.L.	\N
00c020	ARCO ELECTRONIC, CONTROL LTD.	\N
00c021	NETEXPRESS	\N
00c022	LASERMASTER TECHNOLOGIES, INC.	\N
00c023	TUTANKHAMON ELECTRONICS	\N
00c024	EDEN SISTEMAS DE COMPUTACAO SA	\N
00c025	DATAPRODUCTS CORPORATION	\N
00c026	LANS TECHNOLOGY CO., LTD.	\N
00c027	CIPHER SYSTEMS, INC.	\N
00c028	JASCO CORPORATION	\N
00c029	Nexans Deutschland GmbH - ANS	\N
00c02a	OHKURA ELECTRIC CO., LTD.	\N
00c02b	GERLOFF GESELLSCHAFT FUR	\N
00c02c	CENTRUM COMMUNICATIONS, INC.	\N
00c02d	FUJI PHOTO FILM CO., LTD.	\N
00c02e	NETWIZ	\N
00c02f	OKUMA CORPORATION	\N
00c030	INTEGRATED ENGINEERING B. V.	\N
00c031	DESIGN RESEARCH SYSTEMS, INC.	\N
00c032	I-CUBED LIMITED	\N
00c033	TELEBIT COMMUNICATIONS APS	\N
00c034	TRANSACTION NETWORK	\N
00c035	QUINTAR COMPANY	\N
00c036	RAYTECH ELECTRONIC CORP.	\N
00c037	DYNATEM	\N
00c038	RASTER IMAGE PROCESSING SYSTEM	\N
00c039	Teridian Semiconductor Corporation	\N
00c03a	MEN-MIKRO ELEKTRONIK GMBH	\N
00c03b	MULTIACCESS COMPUTING CORP.	\N
00c03c	TOWER TECH S.R.L.	\N
00c03d	WIESEMANN & THEIS GMBH	\N
00c03e	FA. GEBR. HELLER GMBH	\N
00c03f	STORES AUTOMATED SYSTEMS, INC.	\N
00c040	ECCI	\N
00c041	DIGITAL TRANSMISSION SYSTEMS	\N
00c042	DATALUX CORP.	\N
00c043	STRATACOM	\N
00c044	EMCOM CORPORATION	\N
00c045	ISOLATION SYSTEMS, LTD.	\N
00c046	Blue Chip Technology Ltd	\N
00c047	UNIMICRO SYSTEMS, INC.	\N
00c048	BAY TECHNICAL ASSOCIATES	\N
00c049	U.S. ROBOTICS, INC.	\N
00c04a	GROUP 2000 AG	\N
00c04b	CREATIVE MICROSYSTEMS	\N
00c04c	DEPARTMENT OF FOREIGN AFFAIRS	\N
00c04d	MITEC, INC.	\N
00c04e	COMTROL CORPORATION	\N
00c04f	DELL COMPUTER CORPORATION	\N
00c050	TOYO DENKI SEIZO K.K.	\N
00c051	ADVANCED INTEGRATION RESEARCH	\N
00c052	BURR-BROWN	\N
00c053	Aspect Software Inc.	\N
00c054	NETWORK PERIPHERALS, LTD.	\N
00c055	MODULAR COMPUTING TECHNOLOGIES	\N
00c056	SOMELEC	\N
00c057	MYCO ELECTRONICS	\N
00c058	DATAEXPERT CORP.	\N
00c059	DENSO CORPORATION	\N
00c05a	SEMAPHORE COMMUNICATIONS CORP.	\N
00c05b	NETWORKS NORTHWEST, INC.	\N
00c05c	ELONEX PLC	\N
00c05d	L&N TECHNOLOGIES	\N
00c05e	VARI-LITE, INC.	\N
00c05f	FINE-PAL COMPANY LIMITED	\N
00c060	ID SCANDINAVIA AS	\N
00c061	SOLECTEK CORPORATION	\N
00c062	IMPULSE TECHNOLOGY	\N
00c063	MORNING STAR TECHNOLOGIES, INC	\N
00c064	GENERAL DATACOMM IND. INC.	\N
00c065	SCOPE COMMUNICATIONS, INC.	\N
00c066	DOCUPOINT, INC.	\N
00c067	UNITED BARCODE INDUSTRIES	\N
00c068	PHILIP DRAKE ELECTRONICS LTD.	\N
00c069	Axxcelera Broadband Wireless	\N
00c06a	ZAHNER-ELEKTRIK GMBH & CO. KG	\N
00c06b	OSI PLUS CORPORATION	\N
00c06c	SVEC COMPUTER CORP.	\N
00c06d	BOCA RESEARCH, INC.	\N
00c06e	HAFT TECHNOLOGY, INC.	\N
00c06f	KOMATSU LTD.	\N
00c070	SECTRA SECURE-TRANSMISSION AB	\N
00c071	AREANEX COMMUNICATIONS, INC.	\N
00c072	KNX LTD.	\N
00c073	XEDIA CORPORATION	\N
00c074	TOYODA AUTOMATIC LOOM	\N
00c075	XANTE CORPORATION	\N
00c076	I-DATA INTERNATIONAL A-S	\N
00c077	DAEWOO TELECOM LTD.	\N
00c078	COMPUTER SYSTEMS ENGINEERING	\N
00c079	FONSYS CO.,LTD.	\N
00c07a	PRIVA B.V.	\N
00c07b	ASCEND COMMUNICATIONS, INC.	\N
00c07c	HIGHTECH INFORMATION	\N
00c07d	RISC DEVELOPMENTS LTD.	\N
00c07e	KUBOTA CORPORATION ELECTRONIC	\N
00c07f	NUPON COMPUTING CORP.	\N
00c080	NETSTAR, INC.	\N
00c081	METRODATA LTD.	\N
00c082	MOORE PRODUCTS CO.	\N
00c083	TRACE MOUNTAIN PRODUCTS, INC.	\N
00c084	DATA LINK CORP. LTD.	\N
00c085	ELECTRONICS FOR IMAGING, INC.	\N
00c086	THE LYNK CORPORATION	\N
00c087	UUNET TECHNOLOGIES, INC.	\N
00c088	EKF ELEKTRONIK GMBH	\N
00c089	TELINDUS DISTRIBUTION	\N
00c08a	Lauterbach GmbH	\N
00c08b	RISQ MODULAR SYSTEMS, INC.	\N
00c08c	PERFORMANCE TECHNOLOGIES, INC.	\N
00c08d	TRONIX PRODUCT DEVELOPMENT	\N
00c08e	NETWORK INFORMATION TECHNOLOGY	\N
00c08f	Panasonic Electric Works Co., Ltd.	\N
00c090	PRAIM S.R.L.	\N
00c091	JABIL CIRCUIT, INC.	\N
00c092	MENNEN MEDICAL INC.	\N
00c093	ALTA RESEARCH CORP.	\N
00c094	VMX INC.	\N
00c095	ZNYX	\N
00c096	TAMURA CORPORATION	\N
00c097	ARCHIPEL SA	\N
00c098	CHUNTEX ELECTRONIC CO., LTD.	\N
00c099	YOSHIKI INDUSTRIAL CO.,LTD.	\N
00c09a	PHOTONICS CORPORATION	\N
00c09b	RELIANCE COMM/TEC, R-TEC	\N
00c09c	HIOKI E.E. CORPORATION	\N
00c09d	DISTRIBUTED SYSTEMS INT'L, INC	\N
00c09e	CACHE COMPUTERS, INC.	\N
00c09f	QUANTA COMPUTER, INC.	\N
00c0a0	ADVANCE MICRO RESEARCH, INC.	\N
00c0a1	TOKYO DENSHI SEKEI CO.	\N
00c0a2	INTERMEDIUM A/S	\N
00c0a3	DUAL ENTERPRISES CORPORATION	\N
00c0a4	UNIGRAF OY	\N
00c0a5	DICKENS DATA SYSTEMS	\N
00c0a6	EXICOM AUSTRALIA PTY. LTD	\N
00c0a7	SEEL LTD.	\N
00c0a8	GVC CORPORATION	\N
00c0a9	BARRON MCCANN LTD.	\N
00c0aa	SILICON VALLEY COMPUTER	\N
00c0ab	Telco Systems, Inc.	\N
00c0ac	GAMBIT COMPUTER COMMUNICATIONS	\N
00c0ad	MARBEN COMMUNICATION SYSTEMS	\N
00c0ae	TOWERCOM CO. INC. DBA PC HOUSE	\N
00c0af	TEKLOGIX INC.	\N
00c0b0	GCC TECHNOLOGIES,INC.	\N
00c0b1	GENIUS NET CO.	\N
00c0b2	NORAND CORPORATION	\N
00c0b3	COMSTAT DATACOMM CORPORATION	\N
00c0b4	MYSON TECHNOLOGY, INC.	\N
00c0b5	CORPORATE NETWORK SYSTEMS,INC.	\N
00c0b6	Overland Storage, Inc.	\N
00c0b7	AMERICAN POWER CONVERSION CORP	\N
00c0b8	FRASER'S HILL LTD.	\N
00c0b9	FUNK SOFTWARE, INC.	\N
00c0ba	NETVANTAGE	\N
00c0bb	FORVAL CREATIVE, INC.	\N
00c0bc	TELECOM AUSTRALIA/CSSC	\N
00c0bd	INEX TECHNOLOGIES, INC.	\N
00c0be	ALCATEL - SEL	\N
00c0bf	TECHNOLOGY CONCEPTS, LTD.	\N
00c0c0	SHORE MICROSYSTEMS, INC.	\N
00c0c1	QUAD/GRAPHICS, INC.	\N
00c0c2	INFINITE NETWORKS LTD.	\N
00c0c3	ACUSON COMPUTED SONOGRAPHY	\N
00c0c4	COMPUTER OPERATIONAL	\N
00c0c5	SID INFORMATICA	\N
00c0c6	PERSONAL MEDIA CORP.	\N
00c0c7	SPARKTRUM MICROSYSTEMS, INC.	\N
00c0c8	MICRO BYTE PTY. LTD.	\N
00c0c9	ELSAG BAILEY PROCESS	\N
00c0ca	ALFA, INC.	\N
00c0cb	CONTROL TECHNOLOGY CORPORATION	\N
00c0cc	TELESCIENCES CO SYSTEMS, INC.	\N
00c0cd	COMELTA, S.A.	\N
00c0ce	CEI SYSTEMS & ENGINEERING PTE	\N
00c0cf	IMATRAN VOIMA OY	\N
00c0d0	RATOC SYSTEM INC.	\N
00c0d1	COMTREE TECHNOLOGY CORPORATION	\N
00c0d2	SYNTELLECT, INC.	\N
00c0d3	OLYMPUS IMAGE SYSTEMS, INC.	\N
00c0d4	AXON NETWORKS, INC.	\N
00c0d5	Werbeagentur Jürgen Siebert	\N
00c0d6	J1 SYSTEMS, INC.	\N
00c0d7	TAIWAN TRADING CENTER DBA	\N
00c0d8	UNIVERSAL DATA SYSTEMS	\N
00c0d9	QUINTE NETWORK CONFIDENTIALITY	\N
00c0da	NICE SYSTEMS LTD.	\N
00c0db	IPC CORPORATION (PTE) LTD.	\N
00c0dc	EOS TECHNOLOGIES, INC.	\N
00c0dd	QLogic Corporation	\N
00c0de	ZCOMM, INC.	\N
00c0df	KYE Systems Corp.	\N
00c0e0	DSC COMMUNICATION CORP.	\N
00c0e1	SONIC SOLUTIONS	\N
00c0e2	CALCOMP, INC.	\N
00c0e3	OSITECH COMMUNICATIONS, INC.	\N
00c0e4	SIEMENS BUILDING	\N
00c0e5	GESPAC, S.A.	\N
00c0e6	Verilink Corporation	\N
00c0e7	FIBERDATA AB	\N
00c0e8	PLEXCOM, INC.	\N
00c0e9	OAK SOLUTIONS, LTD.	\N
00c0ea	ARRAY TECHNOLOGY LTD.	\N
00c0eb	SEH COMPUTERTECHNIK GMBH	\N
00c0ec	DAUPHIN TECHNOLOGY	\N
00c0ed	US ARMY ELECTRONIC	\N
00c0ee	KYOCERA CORPORATION	\N
00c0ef	ABIT CORPORATION	\N
00c0f0	KINGSTON TECHNOLOGY CORP.	\N
00c0f1	SHINKO ELECTRIC CO., LTD.	\N
00c0f2	TRANSITION NETWORKS	\N
00c0f3	NETWORK COMMUNICATIONS CORP.	\N
00c0f4	INTERLINK SYSTEM CO., LTD.	\N
00c0f5	METACOMP, INC.	\N
00c0f6	CELAN TECHNOLOGY INC.	\N
00c0f7	ENGAGE COMMUNICATION, INC.	\N
00c0f8	ABOUT COMPUTING INC.	\N
00c0f9	Motorola Embedded Computing Group	\N
00c0fa	CANARY COMMUNICATIONS, INC.	\N
00c0fb	ADVANCED TECHNOLOGY LABS	\N
00c0fc	ELASTIC REALITY, INC.	\N
00c0fd	PROSUM	\N
00c0fe	APTEC COMPUTER SYSTEMS, INC.	\N
00c0ff	DOT HILL SYSTEMS CORPORATION	\N
00cbbd	Cambridge Broadband Networks Ltd.	\N
00cf1c	COMMUNICATION MACHINERY CORP.	\N
00d000	FERRAN SCIENTIFIC, INC.	\N
00d001	VST TECHNOLOGIES, INC.	\N
00d002	DITECH CORPORATION	\N
00d003	COMDA ENTERPRISES CORP.	\N
00d004	PENTACOM LTD.	\N
00d005	ZHS ZEITMANAGEMENTSYSTEME	\N
00d006	CISCO SYSTEMS, INC.	\N
00d007	MIC ASSOCIATES, INC.	\N
00d008	MACTELL CORPORATION	\N
00d009	HSING TECH. ENTERPRISE CO. LTD	\N
00d00a	LANACCESS TELECOM S.A.	\N
00d00b	RHK TECHNOLOGY, INC.	\N
00d00c	SNIJDER MICRO SYSTEMS	\N
00d00d	MICROMERITICS INSTRUMENT	\N
00d00e	PLURIS, INC.	\N
00d00f	SPEECH DESIGN GMBH	\N
00d010	CONVERGENT NETWORKS, INC.	\N
00d011	PRISM VIDEO, INC.	\N
00d012	GATEWORKS CORP.	\N
00d013	PRIMEX AEROSPACE COMPANY	\N
00d014	ROOT, INC.	\N
00d015	UNIVEX MICROTECHNOLOGY CORP.	\N
00d016	SCM MICROSYSTEMS, INC.	\N
00d017	SYNTECH INFORMATION CO., LTD.	\N
00d018	QWES. COM, INC.	\N
00d019	DAINIPPON SCREEN CORPORATE	\N
00d01a	URMET  TLC S.P.A.	\N
00d01b	MIMAKI ENGINEERING CO., LTD.	\N
00d01c	SBS TECHNOLOGIES,	\N
00d01d	FURUNO ELECTRIC CO., LTD.	\N
00d01e	PINGTEL CORP.	\N
00d01f	CTAM PTY. LTD.	\N
00d020	AIM SYSTEM, INC.	\N
00d021	REGENT ELECTRONICS CORP.	\N
00d022	INCREDIBLE TECHNOLOGIES, INC.	\N
00d023	INFORTREND TECHNOLOGY, INC.	\N
00d024	Cognex Corporation	\N
00d025	XROSSTECH, INC.	\N
00d026	HIRSCHMANN AUSTRIA GMBH	\N
00d027	APPLIED AUTOMATION, INC.	\N
00d028	OMNEON VIDEO NETWORKS	\N
00d029	WAKEFERN FOOD CORPORATION	\N
00d02a	Voxent Systems Ltd.	\N
00d02b	JETCELL, INC.	\N
00d02c	CAMPBELL SCIENTIFIC, INC.	\N
00d02d	ADEMCO	\N
00d02e	COMMUNICATION AUTOMATION CORP.	\N
00d02f	VLSI TECHNOLOGY INC.	\N
00d030	Safetran Systems Corp	\N
00d031	INDUSTRIAL LOGIC CORPORATION	\N
00d032	YANO ELECTRIC CO., LTD.	\N
00d033	DALIAN DAXIAN NETWORK	\N
00d034	ORMEC SYSTEMS CORP.	\N
00d035	BEHAVIOR TECH. COMPUTER CORP.	\N
00d036	TECHNOLOGY ATLANTA CORP.	\N
00d037	Pace France	\N
00d038	FIVEMERE, LTD.	\N
00d039	UTILICOM, INC.	\N
00d03a	ZONEWORX, INC.	\N
00d03b	VISION PRODUCTS PTY. LTD.	\N
00d03c	Vieo, Inc.	\N
00d03d	GALILEO TECHNOLOGY, LTD.	\N
00d03e	ROCKETCHIPS, INC.	\N
00d03f	AMERICAN COMMUNICATION	\N
00d040	SYSMATE CO., LTD.	\N
00d041	AMIGO TECHNOLOGY CO., LTD.	\N
00d042	MAHLO GMBH & CO. UG	\N
00d043	ZONAL RETAIL DATA SYSTEMS	\N
00d044	ALIDIAN NETWORKS, INC.	\N
00d045	KVASER AB	\N
00d046	DOLBY LABORATORIES, INC.	\N
00d047	XN TECHNOLOGIES	\N
00d048	ECTON, INC.	\N
00d049	IMPRESSTEK CO., LTD.	\N
00d04a	PRESENCE TECHNOLOGY GMBH	\N
00d04b	LA CIE GROUP S.A.	\N
00d04c	EUROTEL TELECOM LTD.	\N
00d04d	DIV OF RESEARCH & STATISTICS	\N
00d04e	LOGIBAG	\N
00d04f	BITRONICS, INC.	\N
00d050	ISKRATEL	\N
00d051	O2 MICRO, INC.	\N
00d052	ASCEND COMMUNICATIONS, INC.	\N
00d053	CONNECTED SYSTEMS	\N
00d054	SAS INSTITUTE INC.	\N
00d055	KATHREIN-WERKE KG	\N
00d056	SOMAT CORPORATION	\N
00d057	ULTRAK, INC.	\N
00d058	CISCO SYSTEMS, INC.	\N
00d059	AMBIT MICROSYSTEMS CORP.	\N
00d05a	SYMBIONICS, LTD.	\N
00d05b	ACROLOOP MOTION CONTROL	\N
00d05c	TECHNOTREND SYSTEMTECHNIK GMBH	\N
00d05d	INTELLIWORXX, INC.	\N
00d05e	STRATABEAM TECHNOLOGY, INC.	\N
00d05f	VALCOM, INC.	\N
00d060	Panasonic Europe Ltd.	\N
00d061	TREMON ENTERPRISES CO., LTD.	\N
00d062	DIGIGRAM	\N
00d063	CISCO SYSTEMS, INC.	\N
00d064	MULTITEL	\N
00d065	TOKO ELECTRIC	\N
00d066	WINTRISS ENGINEERING CORP.	\N
00d067	CAMPIO COMMUNICATIONS	\N
00d068	IWILL CORPORATION	\N
00d069	TECHNOLOGIC SYSTEMS	\N
00d06a	LINKUP SYSTEMS CORPORATION	\N
00d06b	SR TELECOM INC.	\N
00d06c	SHAREWAVE, INC.	\N
00d06d	ACRISON, INC.	\N
00d06e	TRENDVIEW RECORDERS LTD.	\N
00d06f	KMC CONTROLS	\N
00d070	LONG WELL ELECTRONICS CORP.	\N
00d071	ECHELON CORP.	\N
00d072	BROADLOGIC	\N
00d073	ACN ADVANCED COMMUNICATIONS	\N
00d074	TAQUA SYSTEMS, INC.	\N
00d075	ALARIS MEDICAL SYSTEMS, INC.	\N
00d076	Bank of America	\N
00d077	LUCENT TECHNOLOGIES	\N
00d078	Eltex of Sweden AB	\N
00d079	CISCO SYSTEMS, INC.	\N
00d07a	AMAQUEST COMPUTER CORP.	\N
00d07b	COMCAM INTERNATIONAL INC	\N
00d07c	KOYO ELECTRONICS INC. CO.,LTD.	\N
00d07d	COSINE COMMUNICATIONS	\N
00d07e	KEYCORP LTD.	\N
00d07f	STRATEGY & TECHNOLOGY, LIMITED	\N
00d080	EXABYTE CORPORATION	\N
00d081	RTD Embedded Technologies, Inc.	\N
00d082	IOWAVE INC.	\N
00d083	INVERTEX, INC.	\N
00d084	NEXCOMM SYSTEMS, INC.	\N
00d085	OTIS ELEVATOR COMPANY	\N
00d086	FOVEON, INC.	\N
00d087	MICROFIRST INC.	\N
00d088	Motorola, Inc.	\N
00d089	DYNACOLOR, INC.	\N
00d08a	PHOTRON USA	\N
00d08b	ADVA Optical Networking Ltd	\N
00d08c	GENOA TECHNOLOGY, INC.	\N
00d08d	PHOENIX GROUP, INC.	\N
00d08e	NVISION INC.	\N
00d08f	ARDENT TECHNOLOGIES, INC.	\N
00d090	CISCO SYSTEMS, INC.	\N
00d091	SMARTSAN SYSTEMS, INC.	\N
00d092	GLENAYRE WESTERN MULTIPLEX	\N
00d093	TQ - COMPONENTS GMBH	\N
00d094	TIMELINE VISTA, INC.	\N
00d095	Alcatel-Lucent, Enterprise Business Group	\N
00d096	3COM EUROPE LTD.	\N
00d097	CISCO SYSTEMS, INC.	\N
00d098	Photon Dynamics Canada Inc.	\N
00d099	ELCARD OY	\N
00d09a	FILANET CORPORATION	\N
00d09b	SPECTEL LTD.	\N
00d09c	KAPADIA COMMUNICATIONS	\N
00d09d	VERIS INDUSTRIES	\N
00d09e	2WIRE, INC.	\N
00d09f	NOVTEK TEST SYSTEMS	\N
00d0a0	MIPS DENMARK	\N
00d0a1	OSKAR VIERLING GMBH + CO. KG	\N
00d0a2	INTEGRATED DEVICE	\N
00d0a3	VOCAL DATA, INC.	\N
00d0a4	ALANTRO COMMUNICATIONS	\N
00d0a5	AMERICAN ARIUM	\N
00d0a6	LANBIRD TECHNOLOGY CO., LTD.	\N
00d0a7	TOKYO SOKKI KENKYUJO CO., LTD.	\N
00d0a8	NETWORK ENGINES, INC.	\N
00d0a9	SHINANO KENSHI CO., LTD.	\N
00d0aa	CHASE COMMUNICATIONS	\N
00d0ab	DELTAKABEL TELECOM CV	\N
00d0ac	GRAYSON WIRELESS	\N
00d0ad	TL INDUSTRIES	\N
00d0ae	ORESIS COMMUNICATIONS, INC.	\N
00d0af	CUTLER-HAMMER, INC.	\N
00d0b0	BITSWITCH LTD.	\N
00d0b1	OMEGA ELECTRONICS SA	\N
00d0b2	XIOTECH CORPORATION	\N
00d0b3	DRS FLIGHT SAFETY AND	\N
00d0b4	KATSUJIMA CO., LTD.	\N
00d0b5	IPricot formerly DotCom	\N
00d0b6	CRESCENT NETWORKS, INC.	\N
00d0b7	INTEL CORPORATION	\N
00d0b8	Iomega Corporation	\N
00d0b9	MICROTEK INTERNATIONAL, INC.	\N
00d0ba	CISCO SYSTEMS, INC.	\N
00d0bb	CISCO SYSTEMS, INC.	\N
00d0bc	CISCO SYSTEMS, INC.	\N
00d0bd	Silicon Image GmbH	\N
00d0be	EMUTEC INC.	\N
00d0bf	PIVOTAL TECHNOLOGIES	\N
00d0c0	CISCO SYSTEMS, INC.	\N
00d0c1	HARMONIC DATA SYSTEMS, LTD.	\N
00d0c2	BALTHAZAR TECHNOLOGY AB	\N
00d0c3	VIVID TECHNOLOGY PTE, LTD.	\N
00d0c4	TERATECH CORPORATION	\N
00d0c5	COMPUTATIONAL SYSTEMS, INC.	\N
00d0c6	THOMAS & BETTS CORP.	\N
00d0c7	PATHWAY, INC.	\N
00d0c8	Prevas A/S	\N
00d0c9	ADVANTECH CO., LTD.	\N
00d0ca	Intrinsyc Software International Inc.	\N
00d0cb	DASAN CO., LTD.	\N
00d0cc	TECHNOLOGIES LYRE INC.	\N
00d0cd	ATAN TECHNOLOGY INC.	\N
00d0ce	ASYST ELECTRONIC	\N
00d0cf	MORETON BAY	\N
00d0d0	ZHONGXING TELECOM LTD.	\N
00d0d1	Sycamore Networks	\N
00d0d2	EPILOG CORPORATION	\N
00d0d3	CISCO SYSTEMS, INC.	\N
00d0d4	V-BITS, INC.	\N
00d0d5	GRUNDIG AG	\N
00d0d6	AETHRA TELECOMUNICAZIONI	\N
00d0d7	B2C2, INC.	\N
00d0d8	3Com Corporation	\N
00d0d9	DEDICATED MICROCOMPUTERS	\N
00d0da	TAICOM DATA SYSTEMS CO., LTD.	\N
00d0db	MCQUAY INTERNATIONAL	\N
00d0dc	MODULAR MINING SYSTEMS, INC.	\N
00d0dd	SUNRISE TELECOM, INC.	\N
00d0de	PHILIPS MULTIMEDIA NETWORK	\N
00d0df	KUZUMI ELECTRONICS, INC.	\N
00d0e0	DOOIN ELECTRONICS CO.	\N
00d0e1	AVIONITEK ISRAEL INC.	\N
00d0e2	MRT MICRO, INC.	\N
00d0e3	ELE-CHEM ENGINEERING CO., LTD.	\N
00d0e4	CISCO SYSTEMS, INC.	\N
00d0e5	SOLIDUM SYSTEMS CORP.	\N
00d0e6	IBOND INC.	\N
00d0e7	VCON TELECOMMUNICATION LTD.	\N
00d0e8	MAC SYSTEM CO., LTD.	\N
00d0e9	Advantage Century Telecommunication Corp.	\N
00d0ea	NEXTONE COMMUNICATIONS, INC.	\N
00d0eb	LIGHTERA NETWORKS, INC.	\N
00d0ec	NAKAYO TELECOMMUNICATIONS, INC	\N
00d0ed	XIOX	\N
00d0ee	DICTAPHONE CORPORATION	\N
00d0ef	IGT	\N
00d0f0	CONVISION TECHNOLOGY GMBH	\N
00d0f1	SEGA ENTERPRISES, LTD.	\N
00d0f2	MONTEREY NETWORKS	\N
00d0f3	SOLARI DI UDINE SPA	\N
00d0f4	CARINTHIAN TECH INSTITUTE	\N
00d0f5	ORANGE MICRO, INC.	\N
00d0f6	Alcatel Canada	\N
00d0f7	NEXT NETS CORPORATION	\N
00d0f8	FUJIAN STAR TERMINAL	\N
00d0f9	ACUTE COMMUNICATIONS CORP.	\N
00d0fa	Thales e-Security Ltd.	\N
00d0fb	TEK MICROSYSTEMS, INCORPORATED	\N
00d0fc	GRANITE MICROSYSTEMS	\N
00d0fd	OPTIMA TELE.COM, INC.	\N
00d0fe	ASTRAL POINT	\N
00d0ff	CISCO SYSTEMS, INC.	\N
00d11c	ACETEL	\N
00db45	THAMWAY CO.,LTD.	\N
00dd00	UNGERMANN-BASS INC.	\N
00dd01	UNGERMANN-BASS INC.	\N
00dd02	UNGERMANN-BASS INC.	\N
00dd03	UNGERMANN-BASS INC.	\N
00dd04	UNGERMANN-BASS INC.	\N
00dd05	UNGERMANN-BASS INC.	\N
00dd06	UNGERMANN-BASS INC.	\N
00dd07	UNGERMANN-BASS INC.	\N
00dd08	UNGERMANN-BASS INC.	\N
00dd09	UNGERMANN-BASS INC.	\N
00dd0a	UNGERMANN-BASS INC.	\N
00dd0b	UNGERMANN-BASS INC.	\N
00dd0c	UNGERMANN-BASS INC.	\N
00dd0d	UNGERMANN-BASS INC.	\N
00dd0e	UNGERMANN-BASS INC.	\N
00dd0f	UNGERMANN-BASS INC.	\N
00e000	Fujitsu Limited	\N
00e001	STRAND LIGHTING LIMITED	\N
00e002	CROSSROADS SYSTEMS, INC.	\N
00e003	NOKIA WIRELESS BUSINESS COMMUN	\N
00e004	PMC-SIERRA, INC.	\N
00e005	TECHNICAL CORP.	\N
00e006	SILICON INTEGRATED SYS. CORP.	\N
00e007	Avaya ECS Ltd	\N
00e008	AMAZING CONTROLS! INC.	\N
00e009	MARATHON TECHNOLOGIES CORP.	\N
00e00a	DIBA, INC.	\N
00e00b	ROOFTOP COMMUNICATIONS CORP.	\N
00e00c	MOTOROLA	\N
00e00d	RADIANT SYSTEMS	\N
00e00e	AVALON IMAGING SYSTEMS, INC.	\N
00e00f	SHANGHAI BAUD DATA	\N
00e010	HESS SB-AUTOMATENBAU GmbH	\N
00e011	Uniden Corporation	\N
00e012	PLUTO TECHNOLOGIES INTERNATIONAL INC.	\N
00e013	EASTERN ELECTRONIC CO., LTD.	\N
00e014	CISCO SYSTEMS, INC.	\N
00e015	HEIWA CORPORATION	\N
00e016	RAPID CITY COMMUNICATIONS	\N
00e017	EXXACT GmbH	\N
00e018	ASUSTEK COMPUTER INC.	\N
00e019	ING. GIORDANO ELETTRONICA	\N
00e01a	COMTEC SYSTEMS. CO., LTD.	\N
00e01b	SPHERE COMMUNICATIONS, INC.	\N
00e01c	Cradlepoint, Inc	\N
00e01d	WebTV NETWORKS, INC.	\N
00e01e	CISCO SYSTEMS, INC.	\N
00e01f	AVIDIA Systems, Inc.	\N
00e020	TECNOMEN OY	\N
00e021	FREEGATE CORP.	\N
00e022	Analog Devices Inc.	\N
00e023	TELRAD	\N
00e024	GADZOOX NETWORKS	\N
00e025	dit Co., Ltd.	\N
00e026	Redlake MASD LLC	\N
00e027	DUX, INC.	\N
00e028	APTIX CORPORATION	\N
00e029	STANDARD MICROSYSTEMS CORP.	\N
00e02a	TANDBERG TELEVISION AS	\N
00e02b	EXTREME NETWORKS	\N
00e02c	AST COMPUTER	\N
00e02d	InnoMediaLogic, Inc.	\N
00e02e	SPC ELECTRONICS CORPORATION	\N
00e02f	MCNS HOLDINGS, L.P.	\N
00e030	MELITA INTERNATIONAL CORP.	\N
00e031	HAGIWARA ELECTRIC CO., LTD.	\N
00e032	MISYS FINANCIAL SYSTEMS, LTD.	\N
00e033	E.E.P.D. GmbH	\N
00e034	CISCO SYSTEMS, INC.	\N
00e035	Emerson Network Power	\N
00e036	PIONEER CORPORATION	\N
00e037	CENTURY CORPORATION	\N
00e038	PROXIMA CORPORATION	\N
00e039	PARADYNE CORP.	\N
00e03a	CABLETRON SYSTEMS, INC.	\N
00e03b	PROMINET CORPORATION	\N
00e03c	AdvanSys	\N
00e03d	FOCON ELECTRONIC SYSTEMS A/S	\N
00e03e	ALFATECH, INC.	\N
00e03f	JATON CORPORATION	\N
00e040	DeskStation Technology, Inc.	\N
00e041	CSPI	\N
00e042	Pacom Systems Ltd.	\N
00e043	VitalCom	\N
00e044	LSICS CORPORATION	\N
00e045	TOUCHWAVE, INC.	\N
00e046	BENTLY NEVADA CORP.	\N
00e047	InFocus Corporation	\N
00e048	SDL COMMUNICATIONS, INC.	\N
00e049	MICROWI ELECTRONIC GmbH	\N
00e04a	ENHANCED MESSAGING SYSTEMS, INC	\N
00e04b	JUMP INDUSTRIELLE COMPUTERTECHNIK GmbH	\N
00e04c	REALTEK SEMICONDUCTOR CORP.	\N
00e04d	INTERNET INITIATIVE JAPAN, INC	\N
00e04e	SANYO DENKI CO., LTD.	\N
00e04f	CISCO SYSTEMS, INC.	\N
00e050	EXECUTONE INFORMATION SYSTEMS, INC.	\N
00e051	TALX CORPORATION	\N
00e052	Brocade Communications Systems, Inc	\N
00e053	CELLPORT LABS, INC.	\N
00e054	KODAI HITEC CO., LTD.	\N
00e055	INGENIERIA ELECTRONICA COMERCIAL INELCOM S.A.	\N
00e056	HOLONTECH CORPORATION	\N
00e057	HAN MICROTELECOM. CO., LTD.	\N
00e058	PHASE ONE DENMARK A/S	\N
00e059	CONTROLLED ENVIRONMENTS, LTD.	\N
00e05a	GALEA NETWORK SECURITY	\N
00e05b	WEST END SYSTEMS CORP.	\N
00e05c	MATSUSHITA KOTOBUKI ELECTRONICS INDUSTRIES, LTD.	\N
00e05d	UNITEC CO., LTD.	\N
00e05e	JAPAN AVIATION ELECTRONICS INDUSTRY, LTD.	\N
00e05f	e-Net, Inc.	\N
00e060	SHERWOOD	\N
00e061	EdgePoint Networks, Inc.	\N
00e062	HOST ENGINEERING	\N
00e063	CABLETRON - YAGO SYSTEMS, INC.	\N
00e064	SAMSUNG ELECTRONICS	\N
00e065	OPTICAL ACCESS INTERNATIONAL	\N
00e066	ProMax Systems, Inc.	\N
00e067	eac AUTOMATION-CONSULTING GmbH	\N
00e068	MERRIMAC SYSTEMS INC.	\N
00e069	JAYCOR	\N
00e06a	KAPSCH AG	\N
00e06b	W&G SPECIAL PRODUCTS	\N
00e06c	AEP Systems International Ltd	\N
00e06d	COMPUWARE CORPORATION	\N
00e06e	FAR SYSTEMS S.p.A.	\N
00e06f	Motorola, Inc.	\N
00e070	DH TECHNOLOGY	\N
00e071	EPIS MICROCOMPUTER	\N
00e072	LYNK	\N
00e073	NATIONAL AMUSEMENT NETWORK, INC.	\N
00e074	TIERNAN COMMUNICATIONS, INC.	\N
00e075	Verilink Corporation	\N
00e076	DEVELOPMENT CONCEPTS, INC.	\N
00e077	WEBGEAR, INC.	\N
00e078	BERKELEY NETWORKS	\N
00e079	A.T.N.R.	\N
00e07a	MIKRODIDAKT AB	\N
00e07b	BAY NETWORKS	\N
00e07c	METTLER-TOLEDO, INC.	\N
00e07d	NETRONIX, INC.	\N
00e07e	WALT DISNEY IMAGINEERING	\N
00e07f	LOGISTISTEM s.r.l.	\N
00e080	CONTROL RESOURCES CORPORATION	\N
00e081	TYAN COMPUTER CORP.	\N
00e082	ANERMA	\N
00e083	JATO TECHNOLOGIES, INC.	\N
00e084	COMPULITE R&D	\N
00e085	GLOBAL MAINTECH, INC.	\N
00e086	CYBEX COMPUTER PRODUCTS	\N
00e087	LeCroy - Networking Productions Division	\N
00e088	LTX CORPORATION	\N
00e089	ION Networks, Inc.	\N
00e08a	GEC AVERY, LTD.	\N
00e08b	QLogic Corp.	\N
00e08c	NEOPARADIGM LABS, INC.	\N
00e08d	PRESSURE SYSTEMS, INC.	\N
00e08e	UTSTARCOM	\N
00e08f	CISCO SYSTEMS, INC.	\N
00e090	BECKMAN LAB. AUTOMATION DIV.	\N
00e091	LG ELECTRONICS, INC.	\N
00e092	ADMTEK INCORPORATED	\N
00e093	ACKFIN NETWORKS	\N
00e094	OSAI SRL	\N
00e095	ADVANCED-VISION TECHNOLGIES CORP.	\N
00e096	SHIMADZU CORPORATION	\N
00e097	CARRIER ACCESS CORPORATION	\N
00e098	AboCom Systems, Inc.	\N
00e099	SAMSON AG	\N
00e09a	Positron Inc.	\N
00e09b	ENGAGE NETWORKS, INC.	\N
00e09c	MII	\N
00e09d	SARNOFF CORPORATION	\N
00e09e	QUANTUM CORPORATION	\N
00e09f	PIXEL VISION	\N
00e0a0	WILTRON CO.	\N
00e0a1	HIMA PAUL HILDEBRANDT GmbH Co. KG	\N
00e0a2	MICROSLATE INC.	\N
00e0a3	CISCO SYSTEMS, INC.	\N
00e0a4	ESAOTE S.p.A.	\N
00e0a5	ComCore Semiconductor, Inc.	\N
00e0a6	TELOGY NETWORKS, INC.	\N
00e0a7	IPC INFORMATION SYSTEMS, INC.	\N
00e0a8	SAT GmbH & Co.	\N
00e0a9	FUNAI ELECTRIC CO., LTD.	\N
00e0aa	ELECTROSONIC LTD.	\N
00e0ab	DIMAT S.A.	\N
00e0ac	MIDSCO, INC.	\N
00e0ad	EES TECHNOLOGY, LTD.	\N
00e0ae	XAQTI CORPORATION	\N
00e0af	GENERAL DYNAMICS INFORMATION SYSTEMS	\N
00e0b0	CISCO SYSTEMS, INC.	\N
00e0b1	Alcatel-Lucent, Enterprise Business Group	\N
00e0b2	TELMAX COMMUNICATIONS CORP.	\N
00e0b3	EtherWAN Systems, Inc.	\N
00e0b4	TECHNO SCOPE CO., LTD.	\N
00e0b5	ARDENT COMMUNICATIONS CORP.	\N
00e0b6	Entrada Networks	\N
00e0b7	PI GROUP, LTD.	\N
00e0b8	GATEWAY 2000	\N
00e0b9	BYAS SYSTEMS	\N
00e0ba	BERGHOF AUTOMATIONSTECHNIK GmbH	\N
00e0bb	NBX CORPORATION	\N
00e0bc	SYMON COMMUNICATIONS, INC.	\N
00e0bd	INTERFACE SYSTEMS, INC.	\N
00e0be	GENROCO INTERNATIONAL, INC.	\N
00e0bf	TORRENT NETWORKING TECHNOLOGIES CORP.	\N
00e0c0	SEIWA ELECTRIC MFG. CO., LTD.	\N
00e0c1	MEMOREX TELEX JAPAN, LTD.	\N
00e0c2	NECSY S.p.A.	\N
00e0c3	SAKAI SYSTEM DEVELOPMENT CORP.	\N
00e0c4	HORNER ELECTRIC, INC.	\N
00e0c5	BCOM ELECTRONICS INC.	\N
00e0c6	LINK2IT, L.L.C.	\N
00e0c7	EUROTECH SRL	\N
00e0c8	VIRTUAL ACCESS, LTD.	\N
00e0c9	AutomatedLogic Corporation	\N
00e0ca	BEST DATA PRODUCTS	\N
00e0cb	RESON, INC.	\N
00e0cc	HERO SYSTEMS, LTD.	\N
00e0cd	SENSIS CORPORATION	\N
00e0ce	ARN	\N
00e0cf	INTEGRATED DEVICE TECHNOLOGY, INC.	\N
00e0d0	NETSPEED, INC.	\N
00e0d1	TELSIS LIMITED	\N
00e0d2	VERSANET COMMUNICATIONS, INC.	\N
00e0d3	DATENTECHNIK GmbH	\N
00e0d4	EXCELLENT COMPUTER	\N
00e0d5	Emulex Corporation	\N
00e0d6	COMPUTER & COMMUNICATION RESEARCH LAB.	\N
00e0d7	SUNSHINE ELECTRONICS, INC.	\N
00e0d8	LANBit Computer, Inc.	\N
00e0d9	TAZMO CO., LTD.	\N
00e0da	Alcatel North America ESD	\N
00e0db	ViaVideo Communications, Inc.	\N
00e0dc	NEXWARE CORP.	\N
00e0dd	ZENITH ELECTRONICS CORPORATION	\N
00e0de	DATAX NV	\N
00e0df	KEYMILE GmbH	\N
00e0e0	SI ELECTRONICS, LTD.	\N
00e0e1	G2 NETWORKS, INC.	\N
00e0e2	INNOVA CORP.	\N
00e0e3	SK-ELEKTRONIK GmbH	\N
00e0e4	FANUC ROBOTICS NORTH AMERICA, Inc.	\N
00e0e5	CINCO NETWORKS, INC.	\N
00e0e6	INCAA DATACOM B.V.	\N
00e0e7	RAYTHEON E-SYSTEMS, INC.	\N
00e0e8	GRETACODER Data Systems AG	\N
00e0e9	DATA LABS, INC.	\N
00e0ea	INNOVAT COMMUNICATIONS, INC.	\N
00e0eb	DIGICOM SYSTEMS, INCORPORATED	\N
00e0ec	CELESTICA INC.	\N
00e0ed	SILICOM, LTD.	\N
00e0ee	MAREL HF	\N
00e0ef	DIONEX	\N
00e0f0	ABLER TECHNOLOGY, INC.	\N
00e0f1	THAT CORPORATION	\N
00e0f2	ARLOTTO COMNET, INC.	\N
00e0f3	WebSprint Communications, Inc.	\N
00e0f4	INSIDE Technology A/S	\N
00e0f5	TELES AG	\N
00e0f6	DECISION EUROPE	\N
00e0f7	CISCO SYSTEMS, INC.	\N
00e0f8	DICNA CONTROL AB	\N
00e0f9	CISCO SYSTEMS, INC.	\N
00e0fa	TRL TECHNOLOGY, LTD.	\N
00e0fb	LEIGHTRONIX, INC.	\N
00e0fc	HUAWEI TECHNOLOGIES CO., LTD.	\N
00e0fd	A-TREND TECHNOLOGY CO., LTD.	\N
00e0fe	CISCO SYSTEMS, INC.	\N
00e0ff	SECURITY DYNAMICS TECHNOLOGIES, Inc.	\N
00e6d3	NIXDORF COMPUTER CORP.	\N
020701	RACAL-DATACOM	\N
021c7c	PERQ SYSTEMS CORPORATION	\N
026086	LOGIC REPLACEMENT TECH. LTD.	\N
02608c	3COM CORPORATION	\N
027001	RACAL-DATACOM	\N
0270b0	M/A-COM INC. COMPANIES	\N
0270b3	DATA RECALL LTD	\N
029d8e	CARDIAC RECORDERS INC.	\N
02aa3c	OLIVETTI TELECOMM SPA (OLTECO)	\N
02bb01	OCTOTHORPE CORP.	\N
02c08c	3COM CORPORATION	\N
02cf1c	COMMUNICATION MACHINERY CORP.	\N
02e6d3	NIXDORF COMPUTER CORPORATION	\N
040ae0	XMIT AG COMPUTER NETWORKS	\N
040ec2	ViewSonic Mobile China Limited	\N
041e64	Apple, Inc	\N
042234	Wireless Standard Extensions	\N
042bbb	PicoCELA, Inc.	\N
042f56	ATOCS (Shenzhen) LTD	\N
044faa	Ruckus Wireless	\N
04766e	ALPS Co,. Ltd.	\N
0494a1	CATCH THE WIND INC	\N
049f81	Simena, LLC	\N
04a3f3	Emicon	\N
04b3b6	Seamap (UK) Ltd	\N
04b466	BSP Co., Ltd.	\N
04c05b	Tigo Energy	\N
04c880	Samtec Inc	\N
04e0c4	TRIUMPH-ADLER AG	\N
04e548	Cohda Wireless Pty Ltd	\N
04fe7f	Cisco Systems	\N
080001	COMPUTERVISION CORPORATION	\N
080002	BRIDGE COMMUNICATIONS INC.	\N
080003	ADVANCED COMPUTER COMM.	\N
080004	CROMEMCO INCORPORATED	\N
080005	SYMBOLICS INC.	\N
080006	SIEMENS AG	\N
080007	APPLE COMPUTER INC.	\N
080008	BOLT BERANEK AND NEWMAN INC.	\N
080009	HEWLETT PACKARD	\N
08000a	NESTAR SYSTEMS INCORPORATED	\N
08000b	UNISYS CORPORATION	\N
08000c	MIKLYN DEVELOPMENT CO.	\N
08000d	INTERNATIONAL COMPUTERS LTD.	\N
08000e	NCR CORPORATION	\N
08000f	MITEL CORPORATION	\N
080011	TEKTRONIX INC.	\N
080012	BELL ATLANTIC INTEGRATED SYST.	\N
080013	EXXON	\N
080014	EXCELAN	\N
080015	STC BUSINESS SYSTEMS	\N
080016	BARRISTER INFO SYS CORP	\N
080017	NATIONAL SEMICONDUCTOR	\N
080018	PIRELLI FOCOM NETWORKS	\N
080019	GENERAL ELECTRIC CORPORATION	\N
08001a	TIARA/ 10NET	\N
08001b	EMC Corporation	\N
08001c	KDD-KOKUSAI DEBNSIN DENWA CO.	\N
08001d	ABLE COMMUNICATIONS INC.	\N
08001e	APOLLO COMPUTER INC.	\N
08001f	SHARP CORPORATION	\N
080020	SUN MICROSYSTEMS INC.	\N
080021	3M COMPANY	\N
080022	NBI INC.	\N
080023	Panasonic Communications Co., Ltd.	\N
080024	10NET COMMUNICATIONS/DCA	\N
080025	CONTROL DATA	\N
080026	NORSK DATA A.S.	\N
080027	CADMUS COMPUTER SYSTEMS	\N
080028	Texas Instruments	\N
080029	MEGATEK CORPORATION	\N
08002a	MOSAIC TECHNOLOGIES INC.	\N
08002b	DIGITAL EQUIPMENT CORPORATION	\N
08002c	BRITTON LEE INC.	\N
08002d	LAN-TEC INC.	\N
08002e	METAPHOR COMPUTER SYSTEMS	\N
08002f	PRIME COMPUTER INC.	\N
080030	NETWORK RESEARCH CORPORATION	\N
080031	LITTLE MACHINES INC.	\N
080032	TIGAN INCORPORATED	\N
080033	BAUSCH & LOMB	\N
080034	FILENET CORPORATION	\N
080035	MICROFIVE CORPORATION	\N
080036	INTERGRAPH CORPORATION	\N
080037	FUJI-XEROX CO. LTD.	\N
080038	CII HONEYWELL BULL	\N
080039	SPIDER SYSTEMS LIMITED	\N
08003a	ORCATECH INC.	\N
08003b	TORUS SYSTEMS LIMITED	\N
08003c	SCHLUMBERGER WELL SERVICES	\N
08003d	CADNETIX CORPORATIONS	\N
08003e	CODEX CORPORATION	\N
08003f	FRED KOSCHARA ENTERPRISES	\N
080040	FERRANTI COMPUTER SYS. LIMITED	\N
080041	RACAL-MILGO INFORMATION SYS..	\N
080042	JAPAN MACNICS CORP.	\N
080043	PIXEL COMPUTER INC.	\N
080044	DAVID SYSTEMS INC.	\N
080045	CONCURRENT COMPUTER CORP.	\N
080046	Sony Corporation	\N
080047	SEQUENT COMPUTER SYSTEMS INC.	\N
080048	EUROTHERM GAUGING SYSTEMS	\N
080049	UNIVATION	\N
08004a	BANYAN SYSTEMS INC.	\N
08004b	PLANNING RESEARCH CORP.	\N
08004c	HYDRA COMPUTER SYSTEMS INC.	\N
08004d	CORVUS SYSTEMS INC.	\N
08004e	3COM EUROPE LTD.	\N
08004f	CYGNET SYSTEMS	\N
080050	DAISY SYSTEMS CORP.	\N
080051	EXPERDATA	\N
080052	INSYSTEC	\N
080053	MIDDLE EAST TECH. UNIVERSITY	\N
080055	STANFORD TELECOMM. INC.	\N
080056	STANFORD LINEAR ACCEL. CENTER	\N
080057	EVANS & SUTHERLAND	\N
080058	SYSTEMS CONCEPTS	\N
080059	A/S MYCRON	\N
08005a	IBM CORPORATION	\N
08005b	VTA TECHNOLOGIES INC.	\N
08005c	FOUR PHASE SYSTEMS	\N
08005d	GOULD INC.	\N
08005e	COUNTERPOINT COMPUTER INC.	\N
08005f	SABER TECHNOLOGY CORP.	\N
080060	INDUSTRIAL NETWORKING INC.	\N
080061	JAROGATE LTD.	\N
080062	GENERAL DYNAMICS	\N
080063	PLESSEY	\N
080064	AUTOPHON AG	\N
080065	GENRAD INC.	\N
080066	AGFA CORPORATION	\N
080067	COMDESIGN	\N
080068	RIDGE COMPUTERS	\N
080069	SILICON GRAPHICS INC.	\N
08006a	ATT BELL LABORATORIES	\N
08006b	ACCEL TECHNOLOGIES INC.	\N
08006c	SUNTEK TECHNOLOGY INT'L	\N
08006d	WHITECHAPEL COMPUTER WORKS	\N
08006e	MASSCOMP	\N
08006f	PHILIPS APELDOORN B.V.	\N
080070	MITSUBISHI ELECTRIC CORP.	\N
080071	MATRA (DSIE)	\N
080072	XEROX CORP UNIV GRANT PROGRAM	\N
080073	TECMAR INC.	\N
080074	CASIO COMPUTER CO. LTD.	\N
080075	DANSK DATA ELECTRONIK	\N
080076	PC LAN TECHNOLOGIES	\N
080077	TSL COMMUNICATIONS LTD.	\N
080078	ACCELL CORPORATION	\N
080079	THE DROID WORKS	\N
08007a	INDATA	\N
08007b	SANYO ELECTRIC CO. LTD.	\N
08007c	VITALINK COMMUNICATIONS CORP.	\N
08007e	AMALGAMATED WIRELESS(AUS) LTD	\N
08007f	CARNEGIE-MELLON UNIVERSITY	\N
080080	AES DATA INC.	\N
080081	,ASTECH INC.	\N
080082	VERITAS SOFTWARE	\N
080083	Seiko Instruments Inc.	\N
080084	TOMEN ELECTRONICS CORP.	\N
080085	ELXSI	\N
080086	KONICA MINOLTA HOLDINGS, INC.	\N
080087	XYPLEX	\N
080088	Brocade Communications Systems, Inc.	\N
080089	KINETICS	\N
08008a	PERFORMANCE TECHNOLOGY	\N
08008b	PYRAMID TECHNOLOGY CORP.	\N
08008c	NETWORK RESEARCH CORPORATION	\N
08008d	XYVISION INC.	\N
08008e	TANDEM COMPUTERS	\N
08008f	CHIPCOM CORPORATION	\N
080090	SONOMA SYSTEMS	\N
081443	UNIBRAIN S.A.	\N
081651	Shenzhen Sea Star Technology Co.,Ltd	\N
08181a	zte corporation	\N
08184c	A. S. Thomas, Inc.	\N
081ff3	Cisco Systems	\N
082ad0	SRD Innovations Inc.	\N
084e1c	H2A Systems, LLC	\N
08512e	Orion Diagnostica Oy	\N
087618	ViE Technologies Sdn. Bhd.	\N
087695	Auto Industrial Co., Ltd.	\N
089f97	LEROY AUTOMATION	\N
08bbcc	AK-NORD EDV VERTRIEBSGES. mbH	\N
08f2f4	Net One Partners Co.,Ltd.	\N
08f6f8	GET Engineering	\N
08fae0	Fohhn Audio AG	\N
0c15c5	SDTEC Co., Ltd.	\N
0c17f1	TELECSYS	\N
0c1dc2	SeAH Networks	\N
0c2755	Valuable Techologies Limited	\N
0c6076	Hon Hai Precision Ind. Co.,Ltd.	\N
0c7d7c	Kexiang Information Technology Co, Ltd.	\N
0c8230	SHENZHEN MAGNUS TECHNOLOGIES CO.,LTD	\N
0c826a	Wuhan Huagong Genuine Optics Technology Co., Ltd	\N
0c8411	A.O. Smith Water Products	\N
0ca42a	OB Telecom Electronic Technology Co., Ltd	\N
0cc3a7	Meritec	\N
0cc9c6	Samwin Hong Kong Limited	\N
0cd502	Westell	\N
0cd7c2	Axium Technologies, Inc.	\N
0cddef	Nokia Corporation	\N
0ce709	Fox Crypto B.V.	\N
0ce936	ELIMOS srl	\N
0ceee6	Hon Hai Precision Ind. Co.,Ltd.	\N
0cef7c	AnaCom Inc	\N
100000	PRIVATE	\N
10005a	IBM CORPORATION	\N
1000e8	NATIONAL SEMICONDUCTOR	\N
10090c	Janome Sewing Machine Co., Ltd.	\N
1010b6	McCain Inc	\N
10189e	Elmo Motion Control	\N
102d96	Looxcie Inc.	\N
104369	Soundmax Electronic Limited 	\N
10445a	Shaanxi Hitech Electronic Co., LTD	\N
1045f8	LNT-Automation GmbH	\N
1056ca	Peplink International Ltd.	\N
1062c9	Adatis GmbH & Co. KG	\N
1065a3	Panamax Inc.	\N
10880f	DARUMA TELECOMUNICAÇÕES E INFORMÁTICA S/A	\N
10b7f6	Plastoform Industries Ltd.	\N
10baa5	GANA I&C CO., LTD	\N
10c73f	Midas Klark Teknik Ltd	\N
10ca81	PRECIA	\N
10ccdb	AXIMUM PRODUITS ELECTRONIQUES	\N
10e6ae	Source Technologies, LLC	\N
1100aa	PRIVATE	\N
145412	Entis Co., Ltd.	\N
146e0a	PRIVATE	\N
147373	TUBITAK UEKAE	\N
14a62c	S.M. Dezac S.A.	\N
14a86b	ShenZhen Telacom Science&Technology Co., Ltd	\N
1801e3	Elektrobit Wireless Communications Ltd	\N
180675	DILAX Intelcom GmbH	\N
180c77	Westinghouse Electric Company, LLC	\N
181714	DAEWOOIS	\N
183bd2	BYD Precision Manufacture Company Ltd.	\N
1880ce	Barberry Solutions Ltd	\N
1886ac	Nokia Danmark A/S	\N
18a905	Hewlett Packard	\N
18b209	Torrey Pines Logic, Inc	\N
18c086	Broadcom Corporation	\N
18fc9f	Changhe Electronics Co., Ltd.	\N
1c0fcf	Sypro Optics GmbH	\N
1c129d	IEEE PES PSRC/SUB     	\N
1c17d3	Cisco Systems	\N
1c3a4f	AccuSpec Electronics, LLC	\N
1c4bd6	AzureWave	\N
1c6f65	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
1c8f8a	Phase Motion Control SpA	\N
1caff7	D-LINK INTERNATIONAL PTE LIMITED	\N
1cbdb9	D-LINK INTERNATIONAL PTE LIMITED	\N
1cc1de	Hewlett Packard	\N
1cf061	SCAPS GmbH	\N
201257	Most Lucky Trading Ltd	\N
2021a5	LG Electronics Inc	\N
202cb7	Kong Yue Electronics & Information Industry (Xinhui) Ltd.	\N
20415a	Smarteh d.o.o.	\N
2046f9	Advanced Network Devices (dba:AND)	\N
204e6b	Axxana(israel) ltd	\N
2059a0	Paragon Technologies Inc.	\N
206a8a	Wistron InfoComm Manufacturing(Kunshan)Co.,Ltd.	\N
207c8f	Quanta Microsystems,Inc.	\N
20bfdb	DVL	\N
20d906	Iota, Inc.	\N
240b2a	Viettel Group	\N
241f2c	Calsys, Inc.	\N
2421ab	Sony Ericsson Mobile Communications	\N
243c20	Dynamode Group	\N
244597	GEMUE Gebr. Mueller Apparatebau	\N
24828a	Prowave Technologies Ltd.	\N
24a42c	KOUKAAM a.s.	\N
24a937	PURE Storage	\N
24af4a	Alcatel-Lucent-IPD	\N
24b6b8	FRIEM SPA	\N
24bf74	PRIVATE	\N
24cf21	Shenzhen State Micro Technology Co., Ltd	\N
24d2cc	SmartDrive Systems Inc.	\N
24dbad	ShopperTrak RCT Corporation	\N
28068d	ITL, LLC	\N
284846	GridCentric Inc.	\N
284c53	Intune Networks	\N
286ed4	HUAWEI TECHNOLOGIES CO.,LTD	\N
2872c5	Smartmatic Corp	\N
288915	CashGuard Sverige AB	\N
2893fe	Cisco Systems	\N
28cd4c	Individual Computers GmbH	\N
28e794	Microtime Computer Inc.	\N
28ef01	PRIVATE	\N
28fbd3	Shanghai RagenTek Communication Technology Co.,Ltd.	\N
2c0623	Win Leader Inc.	\N
2c1984	IDN Telecom, Inc.	\N
2c3427	ERCO &amp; GENER	\N
2c3a28	Fagor Electrï¿½nica	\N
2c3f3e	Alge-Timing GmbH	\N
2c6bf5	Juniper networks	\N
2c8158	Hon Hai Precision Ind. Co.,Ltd	\N
2c9127	Eintechno Corporation	\N
2ca780	True Technologies Inc.	\N
2ca835	RIM	\N
2ccd27	Precor Inc	\N
2ccd43	Summit Technology Group	\N
3018cf	DEOS control systems GmbH	\N
3032d4	Hanilstm Co., Ltd.	\N
3037a6	Cisco Systems	\N
304174	ALTEC LANSING LLC	\N
30469a	NETGEAR	\N
30525a	NST Co., LTD	\N
30694b	RIM	\N
307c30	RIM	\N
30e48e	Vodafone UK	\N
30efd1	Alstom Strongwish (Shenzhen) Co., Ltd.	\N
34159e	Apple, Inc	\N
342109	Jensen Scandinavia AS	\N
347e39	Nokia Danmark A/S	\N
348302	iForcom Co., Ltd	\N
34862a	Heinz Lackmann GmbH & Co KG	\N
34aaee	Mikrovisatos Servisas UAB	\N
34ba51	Se-Kure Controls, Inc.	\N
34c3ac	Samsung Electronics	\N
34c69a	Enecsys Ltd	\N
34ce94	Parsec (Pty) Ltd	\N
34e0d7	DONGGUAN QISHENG ELECTRONICS INDUSTRIAL CO., LTD	\N
34ef44	2Wire	\N
34ef8b	NTT Communications Corporation	\N
34f39b	WizLAN Ltd.	\N
38229d	PIRELLI BROADBAND SOLUTIONS 	\N
3863f6	3NOD MULTIMEDIA(SHENZHEN)CO.,LTD	\N
389f83	OTN Systems N.V.	\N
38bb23	OzVision America LLC	\N
38c7ba	CS Services Co.,Ltd.	\N
38e7d8	HTC Corporation	\N
38e8df	b gmbh medien + datenbanken	\N
38e98c	Reco S.p.A.	\N
3c05ab	Product Creation Studio	\N
3c106f	ALBAHITH TECHNOLOGIES 	\N
3c1915	GFI Chrono Time	\N
3c1a79	Huayuan Technology CO.,LTD	\N
3c1cbe	JADAK LLC	\N
3c2db7	Texas Instruments	\N
3c39c3	JW Electronics Co., Ltd.	\N
3c4c69	Infinity System S.L.	\N
3cb17f	Wattwatchers Pty Ld	\N
3cdf1e	Cisco Systems	\N
3ce5a6	Hangzhou H3C Technologies Co., Ltd.	\N
3cea4f	2Wire	\N
3cf52c	DSPECIALISTS GmbH	\N
3cf72a	Nokia Corporation	\N
4001c6	3COM EUROPE LTD	\N
4012e4	Compass-EOS	\N
401597	Protect America, Inc.	\N
4025c2	Intel Corporate	\N
402ba1	Sony Ericsson Mobile Communications AB	\N
404022	ZIV	\N
40406b	Icomera	\N
404a03	ZyXEL Communications Corporation	\N
40520d	Pico Technology	\N
406186	MICRO-STAR INT'L CO.,LTD	\N
40618e	Hort-Plan	\N
4083de	Motorola	\N
408493	Clavister AB	\N
408a9a	TITENG CO., Ltd.	\N
409558	Aisino Corporation	\N
4097d1	BK Electronics cc	\N
40a6a4	PassivSystems Ltd	\N
40d32d	Apple, Inc	\N
40d40e	Biodata Ltd	\N
40ecf8	Siemens AG	\N
40ef4c	Fihonest communication co.,Ltd	\N
40f52e	Leica Microsystems (Schweiz) AG	\N
44376f	Young Electric Sign Co	\N
444e1a	Samsung Electronics Co.,Ltd	\N
44568d	PNC Technologies  Co., Ltd.	\N
4456b7	Spawn Labs, Inc	\N
445829	Cisco SPVTG	\N
446c24	Reallin Electronic Co.,Ltd	\N
448312	Star-Net	\N
4487fc	ELITEGROUP COMPUTER SYSTEM CO., LTD.	\N
448e81	VIG	\N
44a42d	TCT Mobile Limited	\N
44a689	PROMAX ELECTRONICA SA	\N
44a8c2	SEWOO TECH CO., LTD	\N
44c233	Guangzhou Comet Technology Development Co.Ltd	\N
44c9a2	Greenwald Industries	\N
44d63d	Talari Networks	\N
44e49a	OMNITRONICS PTY LTD	\N
44f459	Samsung Electronics	\N
481249	Luxcom Technologies Inc.	\N
481bd2	Intron Scientific co., ltd.	\N
48343d	IEP GmbH	\N
484487	Cisco SPVTG	\N
485b39	ASUSTek COMPUTER INC.	\N
486fd2	StorSimple Inc	\N
487119	SGB GROUP LTD.	\N
48aa5d	Store Electronic Systems	\N
48eb30	ETERNA TECHNOLOGY, INC.	\N
48f8e1	Alcatel Lucent WT	\N
48fcb8	Woodstream Corporation	\N
4c022e	CMR KOREA CO., LTD	\N
4c3089	Thales Rail Signalling Solutions GmbH	\N
4c322d	TELEDATA NETWORKS	\N
4c4b68	Mobile Device, Inc. 	\N
4c5499	Huawei Device Co., Ltd	\N
4c5dcd	Oy Finnish Electric Vehicle Technologies Ltd	\N
4c60d5	airPointe of New Hampshire	\N
4c63eb	Application Solutions (Electronics and Vision) Ltd	\N
4c8b55	Grupo Digicon	\N
4c9ee4	Hanyang Navicom Co.,Ltd.	\N
4cbaa3	Bison Electronics Inc.	\N
4cc452	Shang Hai Tyd. Electon Technology Ltd.	\N
4cc602	Radios, Inc.	\N
4ce676	BUFFALO INC.	\N
4cf737	SamJi Electronics Co., Ltd	\N
50252b	Nethra Imaging Incorporated	\N
502a7e	Smart electronic GmbH	\N
502a8b	Telekom Research and Development Sdn Bhd	\N
502da2	Intel Corporate	\N
502df4	Phytec Messtechnik GmbH	\N
506313	Hon Hai Precision Ind. Co.,Ltd. 	\N
506f9a	Wi-Fi Alliance	\N
50934f	Gradual Tecnologia Ltda.	\N
50a6e3	David Clark Company	\N
50c58d	Juniper Networks	\N
50f003	Open Stack, Inc.	\N
5403f5	EBN Technology Corp.	\N
543131	Raster Vision Ltd	\N
544249	Sony Corporation	\N
545fa9	Teracom Limited	\N
5475d0	Cisco Systems	\N
547fee	Cisco Systems	\N
548922	Zelfy Inc	\N
5492be	Samsung Electronics Co.,Ltd	\N
549a16	Uzushio Electric Co.,Ltd.	\N
54b620	SUHDOL E&C Co.Ltd.	\N
54d46f	Cisco SPVTG	\N
580556	Elettronica GF S.r.L.	\N
58170c	Sony Ericsson Mobile Communications AB	\N
583cc6	Omneality Ltd.	\N
5849ba	Chitai Electronic Corp.	\N
584cee	Digital One Technologies, Limited	\N
5850e6	Best Buy Corporation	\N
58570d	Danfoss Solar Inverters	\N
586ed6	PRIVATE	\N
58946b	Intel Corporate	\N
58b035	Apple, Inc	\N
58b9e1	Crystalfontz America, Inc.	\N
58e747	Deltanet AG	\N
58f67b	Xia Men UnionCore Technology LTD.	\N
58fd20	Bravida Sakerhet AB	\N
5c0e8b	Motorola	\N
5c1437	Thyssenkrupp Aufzugswerke GmbH	\N
5c338e	Alpha Networkc Inc.	\N
5c35da	There Corporation Oy	\N
5c57c8	Nokia Corporation	\N
5c8778	Cybertelbridge co.,ltd	\N
5cac4c	Hon Hai Precision Ind. Co.,Ltd.	\N
5cd135	Xtreme Power Systems	\N
5ce223	Delphin Technology AG	\N
5ce286	Nortel Networks	\N
5cff35	Wistron Corporation	\N
601283	Soluciones Tecnologicas para la Salud y el Bienestar SA	\N
601d0f	Midnite Solar	\N
60380e	Alps Electric Co.,	\N
60391f	ABB Ltd	\N
6089b7	KAEL Mï¿½HENDISLIK ELEKTRONIK TICARET SANAYI Limited ï¿½irketi	\N
608d17	Sentrus Government Systems Division, Inc	\N
609aa4	GVI SECURITY INC.	\N
609f9d	CloudSwitch	\N
60b3c4	Elber Srl	\N
60d0a9	Samsung Electronics Co.,Ltd	\N
60d30a	Quatius Limited	\N
60f13d	JABLOCOM s.r.o.	\N
60fb42	Apple, Inc	\N
641084	HEXIUM Technical Development Co., Ltd.	\N
64168d	Cisco Systems	\N
6416f0	Shehzhen Huawei Communication Technologies Co., Ltd.	\N
644bc3	Shanghai WOASiS Telecommunications Ltd., Co.	\N
644f74	LENUS Co., Ltd.	\N
6465c0	Nuvon, Inc	\N
646707	Beijing Omnific Technology, Ltd.	\N
64680c	COMTREND	\N
646e6c	Radio Datacom LLC	\N
647bd4	Texas Instruments	\N
647d81	YOKOTA INDUSTRIAL CO,.LTD	\N
6487d7	PIRELLI BROADBAND SOLUTIONS 	\N
649c8e	Texas Instruments	\N
64a837	Juni Korea Co., Ltd	\N
64b9e8	Apple, Inc	\N
64bc11	CombiQ AB	\N
64c6af	AXERRA Networks Ltd	\N
64d02d	DRAYTEK FRANCE	\N
64d4da	Intel Corporate	\N
64db18	OpenPattern	\N
64ed57	Motorola MDb/Broadband	\N
64f970	Kenade Electronics Technology Co.,LTD.	\N
681fd8	Advanced Telemetry	\N
684b88	Galtronics Telemetry Inc.	\N
68784c	Nortel Networks	\N
687924	ELS-GmbH & Co. KG	\N
687f74	Cisco-Linksys, LLC	\N
688540	IGI Mobile, Inc.	\N
689234	Ruckus Wireless	\N
68a1b7	Honghao Mingchuan Technology (Beijing) CO.,Ltd.	\N
68aad2	DATECS LTD.,	\N
68ca00	Octopus Systems Limited	\N
68cc9c	Mine Site Technologies	\N
68e41f	Unglaube Identech GmbH	\N
68ebae	Samsung Electronics Co.,Ltd	\N
68efbd	Cisco Systems	\N
6c0e0d	Sony Ericsson Mobile Communications AB	\N
6c0f6a	JDC Tech Co., Ltd.	\N
6c1811	Decatur Electronics	\N
6c22ab	Ainswoth Game Technology 	\N
6c32de	Indieon Technologies Pvt. Ltd.	\N
6c3e9c	KE Knestel Elektronik GmbH	\N
6c5cde	SunReports, Inc.	\N
6c5e7a	Ubiquitous Internet Telecom Co., Ltd	\N
6c626d	Micro-Star INT'L CO., LTD	\N
6c6f18	Stereotaxis, Inc.	\N
6c7039	Novar GmbH 	\N
6c8cdb	Otus Technologies Ltd	\N
6c8d65	Wireless Glue Networks, Inc.	\N
6c92bf	Inspur Electronic Information Industry Co.,Ltd.	\N
6cac60	Venetex Corp	\N
6cbee9	Alcatel-Lucent-IPD	\N
6cd68a	LG Electronics Inc	\N
6cdc6a	Promethean Limited	\N
6cf049	GIGA-BYTE TECHNOLOGY CO.,LTD.	\N
6cfdb9	Proware Technologies Co Ltd.	\N
6cffbe	MPB Communications Inc.	\N
701a04	Liteon Tech Corp.	\N
701aed	ADVAS CO., LTD.	\N
702b1d	E-Domus International Limited	\N
702f97	Aava Mobile Oy	\N
703c39	SEAWING Kft	\N
705812	Panasonic AVC Networks Company	\N
705ab6	COMPAL INFORMATION (KUNSHAN) CO., LTD. 	\N
7071bc	PEGATRON CORPORATION	\N
7072cf	EdgeCore Networks	\N
70828e	OleumTech Corporation	\N
70d57e	Scalar Corporation	\N
70d5e7	Wellcore Corporation	\N
70d880	Upos System sp. z o.o.	\N
70f1a1	Liteon Technology Corporation	\N
70f395	USI	\N
743256	NT-ware Systemprg GmbH	\N
746b82	MOVEK 	\N
747e1a	Red Embedded Design Limited	\N
748ef8	Brocade Communications Systems, Inc.	\N
74b9eb	Fujian Goldcat Electronic Technology Co.,Ltd.	\N
74d850	Evrisko Systems	\N
74e537	RADSPIN	\N
74f06d	AzureWave Technologies, Inc.	\N
74f07d	BnCOM Co.,Ltd	\N
74f726	Neuron Robotics	\N
781185	NBS Payment Solutions Inc.	\N
7812b8	ORANTEK LIMITED	\N
78192e	NASCENT Technology	\N
781dba	HUAWEI TECHNOLOGIES CO.,LTD	\N
7825ad	SAMSUNG ELECTRONICS CO., LTD.	\N
7830e1	UltraClenz, LLC	\N
784476	Zioncom technology co.,ltd	\N
785c72	Hioso Technology Co., Ltd.	\N
7866ae	ZTEC Instruments, Inc.	\N
787f62	GiK mbH	\N
78818f	Server Racks Australia Pty Ltd	\N
7884ee	INDRA ESPACIO S.A.	\N
78929c	Intel Corporate	\N
78998f	MEDILINE ITALIA SRL	\N
78a2a0	Nintendo Co., Ltd.	\N
78a6bd	DAEYEON Control&Instrument Co,.Ltd	\N
78a714	Amphenol	\N
78b81a	INTER SALES A/S	\N
78c40e	H&D Wireless 	\N
78dd08	Hon Hai Precision Ind. Co.,Ltd. 	\N
78e400	Hon Hai Precision Ind. Co.,Ltd. 	\N
78e7d1	Hewlett Packard	\N
78ec22	Shanghai Qihui Telecom Technology Co., LTD	\N
7c051e	RAFAEL LTD.	\N
7c08d9	Shanghai Engineering Research Center for Broadband Technologies and Applications	\N
7c1476	AE Partners S.a.s	\N
7c1eb3	2N TELEKOMUNIKACE a.s.	\N
7c2064	Alcatel Lucent IPD	\N
7c2cf3	Secure Electrans Ltd	\N
7c2e0d	Blackmagic Design	\N
7c2f80	Gigaset Communications GmbH	\N
7c3bd5	Imago Group	\N
7c3e9d	PATECH	\N
7c6c8f	AMS NEVE LTD	\N
7c6d62	Apple, Inc	\N
7c6f06	Caterpillar Trimble Control Technologies	\N
7c7673	ENMAS GmbH	\N
7c7be4	Z&#39;SEDAI KENKYUSHO CORPORATION	\N
7ca29b	D.SignT GmbH &amp; Co. KG	\N
7ccb0d	Aaxeon Technologies Inc.	\N
7ccfcf	Shanghai SEARI Intelligent System Co., Ltd	\N
7ce044	NEON Inc	\N
7cf098	Bee Beans Technologies, Inc.	\N
800010	ATT BELL LABORATORIES	\N
80177d	Nortel Networks	\N
8038fd	LeapFrog Enterprises, Inc.	\N
803b9a	ghe-ces electronic ag	\N
80501b	Nokia Corporation	\N
806629	Prescope Technologies CO.,LTD.	\N
80711f	Juniper Networks	\N
807d1b	Neosystem Co. Ltd.	\N
8081a5	TONGQING COMMUNICATION EQUIPMENT (SHENZHEN) Co.,Ltd	\N
80912a	Lih Rong electronic Enterprise Co., Ltd.	\N
80a1d7	Shanghai DareGlobal Technologies Co.,Ltd 	\N
80b289	Forworld Electronics Ltd.	\N
80baac	TeleAdapt Ltd	\N
80c63f	Remec Broadband Wireless , LLC	\N
80c862	Openpeak, Inc	\N
80ee73	Shuttle Inc.	\N
80f593	IRCO Sistemas de Telecomunicaciï¿½n S.A.	\N
842141	Shenzhen Ginwave Technologies Ltd.	\N
842914	EMPORIA TELECOM Produktions- und VertriebsgesmbH & Co KG	\N
842b2b	Dell Inc.	\N
844823	WOXTER TECHNOLOGY Co. Ltd	\N
849000	Arnold &amp; Richter Cine Technik	\N
84a991	Cyber Trans Japan Co.,Ltd.	\N
84c727	Gnodal Ltd	\N
84c7a9	C3PO S.A.	\N
84f64c	Cross Point BV	\N
8841c1	ORBISAT DA AMAZONIA IND E AEROL SA	\N
8843e1	Cisco Systems	\N
884b39	Siemens AG, Healthcare Sector	\N
88532e	Intel Corporate	\N
888717	CANON INC.	\N
8891dd	Racktivity	\N
8894f9	Gemicom Technology, Inc.	\N
8897df	Entrypass Corporation Sdn. Bhd.	\N
889821	TERAON	\N
88a5bd	QPCOM INC.	\N
88ae1d	COMPAL INFORMATION(KUNSHAN)CO.,LTD	\N
88b627	Gembird Europe BV	\N
88ba7f	Qfiednet Co., Ltd.	\N
88ed1c	Cudo Communication Co., Ltd.	\N
88fd15	LINEEYE CO., LTD	\N
8c53f7	A&D ENGINEERING CO., LTD.	\N
8c541d	LGE 	\N
8c56c5	Nintendo Co., Ltd.	\N
8c598b	C Technologies AB	\N
8c640b	BS Storitve d.o.o.	\N
8c736e	Fujitsu Limited	\N
8c7cb5	Hon Hai Precision Ind. Co.,Ltd.	\N
8c8401	PRIVATE	\N
8c9236	Aus.Linx Technology Co., Ltd.	\N
8ca982	Intel Corporate	\N
8cd628	Ikor Metering	\N
9018ae	Shanghai Meridian Technologies, Co. Ltd.	\N
9027e4	Apple	\N
903d5a	Shenzhen Wision Technology Holding Limited	\N
903d6b	Zicon Technology Corp.	\N
904716	RORZE CORPORATION	\N
904ce5	Hon Hai Precision Ind. Co.,Ltd.	\N
90513f	Elettronica Santerno	\N
9055ae	Ericsson, EAB/RWI/K	\N
906dc8	DLG Automação Industrial Ltda	\N
907f61	Chicony Electronics Co., Ltd.	\N
90840d	Apple, Inc	\N
90a2da	GHEO SA	\N
90a7c1	Pakedge Device and Software Inc.	\N
90e0f0	IEEE P1722 	\N
90e6ba	ASUSTek COMPUTER INC.	\N
90f278	Radius Gateway	\N
90fba6	Hon Hai Precision Ind.Co.Ltd	\N
940c6d	TP-LINK Technologies Co.,Ltd.	\N
94236e	Shenzhen Junlan Electronic Ltd	\N
942e63	Finsécur	\N
944452	Belkin International, Inc.	\N
94592d	EKE Building Technology Systems Ltd	\N
945b7e	TRILOBIT LTDA.	\N
949c55	Alta Data Technologies	\N
94ba31	Visiontec da Amazônia Ltda.	\N
94c4e9	PowerLayer Microsystems HongKong Limited	\N
94f692	Geminico co.,Ltd.	\N
9803a0	ABB n.v. Power Quality Products	\N
982d56	Resolution Audio	\N
9835b8	Assembled Products Corporation	\N
986dc8	TOSHIBA MITSUBISHI-ELECTRIC INDUSTRIAL SYSTEMS CORPORATION	\N
9889ed	Anadem Information Inc.	\N
988b5d	SAGEM COMMUNICATION	\N
988edd	Raychem International	\N
98bc99	Edeltech Co.,Ltd.	\N
98d88c	Nortel Networks	\N
98dcd9	UNITEC Co., Ltd.	\N
98fc11	Cisco-Linksys, LLC	\N
9c1874	Nokia Danmark A/S	\N
9c28bf	Continental Automotive Czech Republic s.r.o.	\N
9c4e20	Cisco Systems	\N
9c4e8e	ALT Systems Ltd	\N
9c55b4	I.S.E. S.r.l.	\N
9c5b96	NMR Corporation	\N
9c5e73	Calibre UK Ltd	\N
9c7514	Wildix srl	\N
9cafca	Cisco Systems	\N
9cb206	PROCENTEC	\N
9cc077	PrintCounts, LLC	\N
9ccd82	CHENG UEI PRECISION INDUSTRY CO.,LTD	\N
9cebe8	BizLink (Kunshan) Co.,Ltd	\N
9cf61a	UTC Fire and Security	\N
a00798	Samsung Electronics	\N
a01859	Shenzhen Yidashi Electronics Co Ltd	\N
a0231b	TeleComp R&amp;D Corp.	\N
a02ef3	United Integrated Services Co., Led.	\N
a03a75	PSS Belgium N.V.	\N
a04025	Actioncable, Inc.	\N
a04041	SAMWONFA Co.,Ltd.	\N
a04e04	Nokia Corporation	\N
a055de	Pace Micro Technology plc	\N
a0593a	V.D.S. Video Display Systems srl	\N
a05dc1	TMCT Co., LTD.	\N
a05de7	DIRECTV, Inc.	\N
a06986	Wellav Technologies Ltd	\N
a06a00	Verilink Corporation	\N
a07332	Cashmaster International Limited	\N
a088b4	Intel Corporate	\N
a09805	OpenVox Communication Co Ltd	\N
a098ed	Shandong Intelligent Optical Communication Development Co., Ltd.	\N
a09a5a	Time Domain	\N
a0b9ed	Skytap	\N
a0bfa5	CORESYS	\N
a40cc3	Cisco Systems	\N
a438fc	Plastic Logic	\N
a4561b	MCOT Corporation	\N
a479e4	KLINFO Corp	\N
a4ad00	Ragsdale Technology	\N
a4adb8	Vitec Group, Camera Dynamics Ltd	\N
a4ae9a	Maestro Wireless Solutions ltd.	\N
a4b121	Arantia 2010 S.L.	\N
a4b1ee	H. ZANDER GmbH & Co. KG	\N
a4badb	Dell Inc.	\N
a4c2ab	Hangzhou LEAD-IT Information & Technology Co.,Ltd	\N
a4da3f	Bionics Corp.	\N
a4de50	Total Walther GmbH	\N
a4e7e4	Connex GmbH	\N
a4ed4e	Motorola Mobile Devices	\N
a85bb0	Shenzhen Dehoo Technology Co.,Ltd	\N
a863df	DISPLï¿½AIRE CORPORATION	\N
a870a5	UniComm Inc.	\N
a87b39	Nokia Corporation	\N
a87e33	Nokia Danmark A/S	\N
a893e6	JIANGXI JINGGANGSHAN CKING COMMUNICATION TECHNOLOGY CO.,LTD	\N
a8995c	aizo ag	\N
a8b1d4	Cisco Systems	\N
a8c222	TM-Research Inc.	\N
a8cb95	EAST BEST CO., LTD.	\N
a8ce90	CVC	\N
a8e3ee	Sony Computer Entertainment Inc.	\N
a8f274	Samsung Electronics	\N
a8f470	Fujian Newland Communication Science Technologies Co.,Ltd.	\N
a8f94b	Eltex Enterprise Ltd.	\N
aa0000	DIGITAL EQUIPMENT CORPORATION	\N
aa0001	DIGITAL EQUIPMENT CORPORATION	\N
aa0002	DIGITAL EQUIPMENT CORPORATION	\N
aa0003	DIGITAL EQUIPMENT CORPORATION	\N
aa0004	DIGITAL EQUIPMENT CORPORATION	\N
ac44f2	Revolabs Inc	\N
ac5135	MPI TECH	\N
ac583b	Human Assembler, Inc.	\N
ac6706	Ruckus Wireless	\N
ac8317	Shenzhen Furtunetel Communication Co., Ltd	\N
ac867e	Create New Technology (HK) Limited Company	\N
ac9a96	Lantiq Deutschland GmbH	\N
ac9b84	Smak Tecnologia e Automacao	\N
aca016	Cisco Systems	\N
acbe75	Ufine Technologies Co.,Ltd.	\N
acbeb6	Visualedge Technology Co., Ltd.	\N
acce8f	HWA YAO TECHNOLOGIES CO., LTD	\N
acd180	Crexendo Business Solutions, Inc.	\N
acde48	PRIVATE	\N
ace348	MadgeTech, Inc	\N
ace9aa	Hay Systems Ltd	\N
acea6a	GENIX INFOCOMM CO., LTD.	\N
b05b1f	THERMO FISHER SCIENTIFIC S.P.A.	\N
b09074	Fulan Electronics Limited	\N
b09134	Taleo	\N
b0973a	E-Fuel Corporation	\N
b0aa36	GUANGDONG OPPO MOBILE TELECOMMUNICATIONS CORP.,LTD.	\N
b0c69a	Juniper Networks	\N
b0c8ad	People Power Company	\N
b0e39d	CAT SYSTEM CO.,LTD.	\N
b0e754	2Wire	\N
b0e97e	Advanced Micro Peripherals	\N
b40832	TC Communications	\N
b42cbe	Direct Payment Solutions Limited	\N
b439d6	ProCurve Networking by HP	\N
b43db2	Degreane Horizon	\N
b4417a	ShenZhen Gongjin Electronics Co.,Ltd	\N
b45861	CRemote, LLC	\N
b482fe	Askey Computer Corp	\N
b4b5af	Minsung Electronics	\N
b4ed19	Pie Digital, Inc.	\N
b4ed54	Wohler Technologies	\N
b8616f	Accton Wireless Broadband(AWB), Corp.	\N
b86491	CK Telecom Ltd	\N
b8653b	Bolymin, Inc.	\N
b894d2	Retail Innovation HTT AB	\N
b8a3e0	BenRui Technology Co.,Ltd	\N
b8ac6f	Dell Inc	\N
b8b1c7	BT&COM CO.,LTD	\N
b8f732	Aryaka Networks Inc	\N
bc0543	AVM GmbH	\N
bc4e3c	CORE STAFF CO., LTD.	\N
bc6a16	tdvine	\N
bc7737	Intel Corporate	\N
bc7dd1	Radio Data Comms	\N
bc9da5	DASCOM Europe GmbH	\N
bca9d6	Cyber-Rain, Inc.	\N
bcb181	SHARP CORPORATION	\N
bcd5b6	d2d technologies	\N
bcf2af	devolo AG	\N
c00d7e	Additech, Inc.	\N
c01e9b	Pixavi AS	\N
c02250	PRIVATE	\N
c038f9	Nokia Danmark A/S	\N
c03f0e	NETGEAR	\N
c06c0f	Dobbs Stanford	\N
c08b6f	S I Sistemas Inteligentes Eletronicos Ltda	\N
c09134	ProCurve Networking by HP	\N
c09c92	COBY	\N
c0bae6	Application Solutions (Safety and Security) Ltd	\N
c0cfa3	Creative Electronics &amp; Software, Inc.	\N
c0d044	SAGEMCOM	\N
c0e422	Texas Instruments	\N
c416fa	Prysm Inc	\N
c417fe	Hon Hai Precision Ind. Co.,Ltd.	\N
c4198b	Dominion Voting Systems Corporation	\N
c41ece	HMI Sources Ltd.	\N
c42c03	Apple	\N
c44619	Hon Hai Precision Ind. Co.,Ltd. 	\N
c45976	Fugoo	\N
c47d4f	Cisco Systems	\N
c4823f	Fujian Newland Auto-ID Tech. Co,.Ltd.	\N
c4aaa1	SUMMIT DEVELOPMENT, spol.s r.o.	\N
c4e17c	U2S co.	\N
c4fce4	DishTV NZ Ltd	\N
c802a6	Beijing Newmine Technology	\N
c80aa9	Quanta Computer Inc.	\N
c82e94	Halfa Enterprise Co., Ltd.	\N
c835b8	Ericsson, EAB/RWI/K	\N
c83a35	Tenda Technology Co., Ltd.	\N
c848f5	MEDISON Xray Co., Ltd	\N
c84c75	Cisco Systems	\N
c86c1e	Display Systems Ltd	\N
c86cb6	Optcom Co., Ltd.	\N
c87248	Aplicom Oy	\N
c87e75	Samsung Electronics Co.,Ltd	\N
c8873b	Net Optics	\N
c8979f	Nokia Corporation	\N
c8aacc	PRIVATE	\N
c8c13c	RuggedTek Hangzhou Co., Ltd	\N
c8d1d1	AGAiT Technology Corporation	\N
c8d2c1	Jetlun (Shenzhen) Corporation	\N
c8ef2e	Beijing Gefei Tech. Co., Ltd 	\N
cc0080	TRUST SYSTEM Co.,	\N
cc2218	InnoDigital Co., Ltd.	\N
cc43e3	Trump s.a.	\N
cc5076	Ocom Communications, Inc.	\N
cc5459	OnTime Networks AS	\N
cc5c75	Weightech Com. Imp. Exp. Equip. Pesagem Ltda	\N
cc69b0	Global Traffic Technologies, LLC	\N
ccb888	AnB Securite s.a.	\N
cccc4e	Sun Fountainhead USA. Corp 	\N
ccea1c	DCONWORKS  Co., Ltd	\N
ccfcb1	Wireless Technology, Inc.	\N
d03761	Texas Instruments	\N
d05875	Active Control Technology Inc.	\N
d0b33f	SHENZHEN TINNO MOBILE TECHNOLOGY CO.,LTD.	\N
d0d286	Beckman Coulter Biomedical K.K.	\N
d0e347	Yoga	\N
d0e40b	Wearable Inc.	\N
d0f0db	Ericsson	\N
d4000d	Phoenix Broadband Technologies, LLC.	\N
d411d6	ShotSpotter, Inc.	\N
d41f0c	TVI Vision Oy	\N
d44ca7	Informtekhnika & Communication, LLC	\N
d45297	nSTREAMS Technologies, Inc.	\N
d46cbf	Goodrich ISR	\N
d479c3	Cameronet GmbH &amp; Co. KG	\N
d4823e	Argosy Technologies, Ltd.	\N
d49a20	Apple, Inc	\N
d49c28	JayBird Gear LLC	\N
d4aaff	MICRO WORLD 	\N
d4c766	Acentic GmbH	\N
d4f143	IPROAD.,Inc	\N
d81bfe	TWINLINX CORPORATION	\N
d828c9	General Electric Consumer and Industrial	\N
d83062	Apple, Inc	\N
d842ac	FreeComm Data Communication Co.,Ltd.	\N
d84606	Silicon Valley Global Marketing	\N
d84b2a	Cognitas Technologies, Inc.	\N
d8543a	Texas Instruments	\N
d85d4c	TP-LINK Technologies Co.,Ltd.	\N
d87157	Lenovo Mobile Communication Technology Ltd.	\N
d87533	Nokia Corporation	\N
d87988	Hon Hai Precision Ind. Co., Ltd.	\N
d8a25e	Apple	\N
d8ae90	Itibia Technologies	\N
d8c3fb	DETRACOM	\N
d8d385	Hewlett Packard	\N
d8d67e	GSK CNC EQUIPMENT CO.,LTD	\N
d8e72b	OnPATH Technologies	\N
dc0265	Meditech Kft	\N
dc1d9f	U & B tech	\N
dc2c26	Iton Technology Limited	\N
dc3350	TechSAT GmbH	\N
dc49c9	CASCO SIGNAL LTD	\N
dca971	Intel Corporate	\N
dce2ac	Lumens Digital Optics Inc.	\N
dce71c	AUG Elektronik GmbH	\N
dcfad5	STRONG Ges.m.b.H.	\N
e01cee	Bravo Tech, Inc.	\N
e02630	Intrigue Technologies, Inc.	\N
e02636	Nortel Networks	\N
e0271a	TTC Next-generation Home Network System WG	\N
e0589e	Laerdal Medical	\N
e064bb	DigiView S.r.l.	\N
e087b1	Nata-Info Ltd.	\N
e08fec	REPOTEC CO., LTD.	\N
e09153	XAVi Technologies Corp.	\N
e091f5	NETGEAR	\N
e0abfe	Orb Networks, Inc.	\N
e0bc43	C2 Microsystems, Inc.	\N
e0c286	Aisai Communication Technology Co., Ltd.	\N
e0ca4d	Shenzhen Unistar Communication Co.,LTD	\N
e0cb4e	ASUSTek COMPUTER INC.	\N
e0e751	Nintendo Co., Ltd.	\N
e41f13	IBM	\N
e43593	Hangzhou GoTo technology Co.Ltd	\N
e4751e	Getinge Sterilization AB	\N
e47cf9	Samsung Electronics Co., LTD	\N
e497f0	Shanghai VLC Technologies Ltd. Co.	\N
e4ab46	UAB Selteka	\N
e4ad7d	SCL Elements	\N
e4ffdd	ELECTRON INDIA	\N
e8056d	Nortel Networks	\N
e80688	Apple Inc. 	\N
e80b13	Akib Systems Taiwan, INC	\N
e83a97	OCZ Technology Group	\N
e84ece	Nintendo Co., Ltd.	\N
e85b5b	LG ELECTRONICS INC	\N
e85e53	Infratec Datentechnik GmbH	\N
e86cda	Supercomputers and Neurocomputers Research Center	\N
e89d87	Toshiba	\N
e8a4c1	Deep Sea Electronics PLC	\N
e8daaa	VideoHome Technology Corp.	\N
e8dff2	PRF Co., Ltd.	\N
e8e1e2	Energotest	\N
e8e5d6	Samsung Electronics Co.,Ltd 	\N
e8e776	Shenzhen Kootion Technology Co., Ltd	\N
ec3091	Cisco Systems	\N
ec43e6	AWCER Ltd.	\N
ec4476	Cisco Systems	\N
ec5c69	MITSUBISHI HEAVY INDUSTRIES MECHATRONICS SYSTEMS,LTD.	\N
ec6c9f	Chengdu Volans Technology CO.,LTD	\N
ec7c74	Justone Technologies Co., Ltd.	\N
ec8ead	DLX	\N
ec9b5b	Nokia Corporation	\N
ecb106	Acuro Networks, Inc	\N
ecc882	Cisco Systems	\N
eccd6d	Allied Telesis, Inc.	\N
ecd00e	MiraeRecognition Co., Ltd.	\N
ecde3d	Lamprey Networks, Inc.	\N
ece09b	Samsung electronics CO., LTD	\N
ece9f8	Guang Zhou TRI-SUN Electronics Technology  Co., Ltd	\N
ecfe7e	BlueRadios, Inc.	\N
f02408	Talaris (Sweden) AB	\N
f0264c	Dr. Sigrist AG	\N
f02fd8	Bi2-Vision	\N
f04335	DVN(Shanghai)Ltd.	\N
f04bf2	JTECH Communications, Inc.	\N
f06281	ProCurve Networking by HP	\N
f06853	Integrated Corporation	\N
f077d0	Xcellen	\N
f07bcb	Hon Hai Precision Ind. Co.,Ltd. 	\N
f09cbb	RaonThink Inc.	\N
f0ad4e	Globalscale Technologies, Inc.	\N
f0b6eb	Poslab Technology Co., Ltd.	\N
f0bcc8	MaxID (Pty) Ltd	\N
f0bdf1	Sipod Inc.	\N
f0c24c	Zhejiang FeiYue Digital Technology Co., Ltd	\N
f0d767	Axema Passagekontroll AB	\N
f0de71	Shanghai EDO Technologies Co.,Ltd.	\N
f0def1	Wistron InfoComm Manufacturing(Kunshan)Co	\N
f0ec39	Essec	\N
f0ed1e	Bilkon Bilgisayar Kontrollu Cih. Im.Ltd.	\N
f40b93	Research In Motion	\N
f445ed	Portable Innovation Technology Ltd.	\N
f450eb	Telechips Inc	\N
f45595	HENGBAO Corporation LTD.	\N
f45ff7	DQ Technology Inc.	\N
f46349	Diffon Corporation	\N
f47626	Viltechmeda UAB 	\N
f4acc1	Cisco Systems	\N
f4c795	WEY Elektronik AG	\N
f4ce46	Hewlett Packard	\N
f4fc32	Texas Instruments	\N
f80f41	Wistron InfoComm(ZhongShan) Corporation	\N
f81edf	Apple, Inc	\N
f8472d	X2gen Digital Corp. Ltd	\N
f852df	VNL Europe AB	\N
f86ecf	Arcx Inc	\N
f871fe	The Goldman Sachs Group, Inc.	\N
f8811a	OVERKIZ	\N
f88def	Tenebraex	\N
f8912a	GLP German Light Products GmbH	\N
f893f3	VOLANS	\N
f89d0d	Control Technology Inc.	\N
f8ac6d	Deltenna Ltd	\N
f8c091	Highgates Technology	\N
f8d756	Simm Tronic Limited 	\N
f8dc7a	Variscite LTD	\N
f8e968	Egker Kft.	\N
f8fb2f	Santur Corporation	\N
fc0877	Prentke Romich Company	\N
fc4463	Universal Audio	\N
fc6198	NEC Personal Products, Ltd	\N
fc683e	Directed Perception, Inc	\N
fccce4	Ascon Ltd.	\N
fccf62	BLADE Network Technology	\N
fce192	Sichuan Jinwangtong Electronic Science&Technology Co,.Ltd	\N
fce23f	CLAY PAKY SPA	\N
fcfaf7	Shanghai Baud Data Communication Co.,Ltd.	\N
fcfbfb	Cisco Systems	\N
\.


--
-- Data for Name: esia_maintenance_configuration; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_maintenance_configuration (maintenance_conf_id, maintenance_conf_type, maintenance_conf_start_date, maintenance_conf_create, maintenance_conf_service_id, maintenance_conf_during, maintenance_conf_recurring, maintenance_conf_during_firsterror) FROM stdin;
\.


--
-- Data for Name: esia_maintenance_planned; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_maintenance_planned (planned_id, planned_type, planned_start, planned_end, planned_during, planned_service_id) FROM stdin;
\.


--
-- Data for Name: esia_manufacturer; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_manufacturer (manufacturer_oid, manufacturer_name, manufacturer_logo) FROM stdin;
9	Cisco	/data/images/Cisco-logo.png
1916	Extreme Network	\N
11	HP	/data/images/HP-logo.png
43	3Com	/data/images/3Com-logo.png
\.


--
-- Data for Name: esia_memberof; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_memberof (member_group_id, member_user_id) FROM stdin;
1	1
1	2
1	23
1	24
\.


--
-- Data for Name: esia_model; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_model (model_id, model_name, model_manufacturer_oid, model_html_tpl) FROM stdin;
2	WS-C2950G-24-EI	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-c2950g-24-ei.html
9	WS-C2950-24	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C2950-24.html
10	WS-C2950T-24	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C2950T-24.html
11	WS-C2950T-48-SI	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C2950T-48-SI.html
12	WS-C3524-XL	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C3524-XL.html
13	WS-C3750-24TS	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C3750-24TS.html
14	WS-C3750-24P	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C3750-24P.html
15	WS-C2960-24TT-L	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C2960-24TT-L.html
16	WS-C3750E-24TD	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C3750E-24TD.html
17	WS-C3560-24TS	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C3560-24TS.html
18	WS-C2924-XL	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C2924-XL.html
19	WS-C2960-24PC-L	9	/usr/local/esia/public_html/data/switch_template/Cisco.ws-C2960-24PC-L.html
21	J4813A	11	/usr/local/esia/public_html/data/switch_template/HP.J4813A.html
\.


--
-- Data for Name: esia_os; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_os (os_id, os_name, os_version, os_template_path, os_manufacturer_oid) FROM stdin;
1	IOS	12.1(22)EA4	/usr/local/esia/public_html/data/switch_template_configuration/Cisco/switch/IOS/12.1(22).xml	9
2	IOS	12.1(22)EA1	/usr/local/esia/public_html/data/switch_template_configuration/Cisco/switch/IOS/12.1(22).xml	9
3	\N	F.05.55	/usr/local/esia/public_html/data/switch_template_configuration/HP/switch/F.05.55.xml	11
\.


--
-- Data for Name: esia_php_class; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_php_class (class_id, class_name, class_type, class_path, class_manufacturer_oid) FROM stdin;
6	CiscoSwitch	switch	/usr/local/esia/public_html/data/libs/CiscoSwitch.class.php	9
16	HPSwitch	switch	/usr/local/esia/public_html/data/libs/HPSwitch.class.php	11
17	ComSwitch	switch	/usr/local/esia/public_html/data/libs/3ComSwitch.class.php	43
\.


--
-- Data for Name: esia_plugin_react_program; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_plugin_react_program (react_plugin_id, react_program_id, react_parameter, react_ok, react_warning, react_critical, react_unknown) FROM stdin;
\.


--
-- Data for Name: esia_plugins; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_plugins (plugin_id, plugin_name, plugin_path, plugin_type, plugin_trap_oid, plugin_default_parameter, plugin_default_interval_time, plugin_description, plugin_delete, plugin_priority, plugin_html_template, plugin_manufacturer, plugin_tag, plugin_cron_type, plugin_cron_only_existing) FROM stdin;
1	CRON_RSS_SYNCHRONIZATION	/usr/local/esia/public_html/cron/sync-rss.php	cron	\N	\N	3600000	Synchronisation des flux RSS	f	5	\N	ESIA	ESIA,RSS,CRON	minut	t
\.


--
-- Data for Name: esia_privilege_grantedto; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_privilege_grantedto (grant_group_id, grant_privilege_id, grant_acces) FROM stdin;
1	1	1
1	2	1
1	3	1
1	4	1
1	5	1
1	6	1
1	7	1
1	8	1
1	9	1
1	10	1
1	11	1
1	12	1
1	13	1
1	14	1
1	15	1
1	16	1
1	17	1
1	18	1
7	6	1
\.


--
-- Data for Name: esia_privileges; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_privileges (privilege_id, privilege_name, privilege_site, privilege_description) FROM stdin;
1	esia_administration	1	Accès à l'interface d'administration d'Esia
2	esia_device	1	Autorise la création, la modification des appareils et l'ajout de plugin
3	esia_groups	1	Gestion des groupes
4	esia_users	1	Gestion des utilisateurs
5	esia_access	1	Gestion des accès
6	device_view	0	Droit d accès à un noeud
7	map_view	1	accès à la google map
8	map_administration	1	accès à la configuration de la google map
9	switch_interface_view	0	Autorise l'accès aux interfaces des switchs
10	switch_conf_vlan_access	0	Autorise la configuration du vlan sur les interfaces (en mode static et dynamic chez cisco) (vlan untagged)
11	switch_conf_vlan_trunk	0	Autorise la configuration du vlan des interfaces en mode trunk (tagged)
12	switch_conf_voip	0	Autorise la configuration de la VoIP
13	switch_conf_reset_access	0	Autorise le 'reset' des interfaces en mode static et dynamic
14	switch_conf_reset_trunk	0	Autorise le reset des interfaces en mode trunk
15	device_administration	1	\N
16	acquit_access	1	Autorise l'acquittement
17	esia_user_share_dashboard	1	autorise l'utilisateur à partager des tableaux de bord
18	esia_maintenance	1	Autorise l'accès à la maintenance des noeuds/service
\.


--
-- Data for Name: esia_program; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_program (program_id, program_name, program_path, program_default_parameter, program_html_template) FROM stdin;
1	SEND_MAIL	/usr/local/esia/plugins/alerting/send_mail_html.pl	-e $ERROR -t "$USERMAIL"	/theme/plugins/SEND_MAIL.php?htmlid=:id&htmlid_link=:linkid
4	SEND_SMS	/usr/local/esia/plugins/alerting/send_sms.pl	-e $ERROR -t <numeros au format 00324x> -m "$USERMAIL"	/theme/plugins/SEND_SMS_MAIL.php?htmlid=:id&htmlid_link=:linkid
\.


--
-- Data for Name: esia_rdp; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_rdp (rdp_id, rdp_device_id, rdp_software, rdp_soft_id) FROM stdin;
\.


--
-- Data for Name: esia_search; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_search (search_keyword, search_description, search_file, search_function, search_name) FROM stdin;
idesc	switch plugin: keyword if they want search description of interface	/plugin/cloudswitch/sw_search_addon.php	search_interfaceDescription	SEARCH_INT_DESCRIPTION
mac	switch plugin: keyword if they want search mac-address	/plugin/cloudswitch/sw_search_addon.php	search_mac	SEARCH_INT_MAC_ADDRESS
node	Default search node 	search.php	SearchNodeNameOrDescription	SEARCH_NODE
plug	switch plugin: keyword if they want search plug interface	/plugin/cloudswitch/sw_search_addon.php	search_plug	SEARCH_INT_PLUG
\.


--
-- Data for Name: esia_service_pattern_included; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_service_pattern_included (included_id, included_name, included_pattern_id, included_plugin_id, included_parameter, included_interval, included_program_id, included_program_parameter, included_unknown, included_warning, included_critical, included_priority, included_ok) FROM stdin;
\.


--
-- Data for Name: esia_service_patterns; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_service_patterns (pattern_id, pattern_name, pattern_description) FROM stdin;
\.


--
-- Data for Name: esia_service_pending_execute; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_service_pending_execute (pending_service_id, pending_type) FROM stdin;
\.


--
-- Data for Name: esia_service_response_program; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_service_response_program (response_service_id, response_program_id, response_parameter, response_ok, response_warning, response_critical, response_unknown) FROM stdin;
\.


--
-- Data for Name: esia_service_rra_file; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_service_rra_file (rra_file_id, rra_service_id, rra_graph_num, rra_file_path) FROM stdin;
\.


--
-- Data for Name: esia_services; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_services (service_plugin_id, service_device_id, service_interval_time, service_parameter, service_previous_state, service_last_exec_time, service_last_stdout, service_id, service_name, service_time_added, service_delete, service_included_id, service_priority) FROM stdin;
\.


--
-- Data for Name: esia_switch; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_switch (switch_device_id, switch_model_id, switch_class_id, switch_os_id) FROM stdin;
\.


--
-- Data for Name: esia_theme; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_theme (theme_id, theme_name) FROM stdin;
3	default
\.


--
-- Data for Name: esia_users; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_users (user_id, user_name, user_password, user_lang, user_inscription_time, user_email, user_change_password, user_theme_id, user_tuto_view, visibility_user, home_page, user_source) FROM stdin;
1	esia-01	2528d5b2187ea1090502e2abdd56ee33154701ca	fr	2018-02-14 11:06:43.603824	n.biersart@esia-sa.com	1	3	238	f	/index.php	LOCAL
2	esia-02	7f7413459dd33ae1d8d8075eb51d7b00e8550f58	fr	2018-02-14 10:44:37.2059	j.george@esia-sa.com	1	3	238	f	/index.php	LOCAL
24	esia-04	752c8a38e7ac2fb70c2ea764c0965a6e43f79dc4	fr	2018-02-12 16:23:01.900469	v.marin@esia-sa.com	1	3	238	f	/index.php	LOCAL
23	esia-03	8441d7ddd9d9e2dbd22bb48064865e91564ff869	fr	2018-11-14 16:01:42.654763	q.george@esia-sa.com	1	3	238	f	/index.php	LOCAL
\.


--
-- Data for Name: esia_web_plugin; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.esia_web_plugin (web_id, web_name, web_dir_lang, web_php_menu, web_index_page, web_admin_page, web_activate, web_tpl_method, web_plugin_apache_path, web_page_control, web_show_main_menu, web_icon) FROM stdin;
3	WEBP_LOCATION	lang/	map_menu.inc.php	map.php	map_admin.php	f	universal	/plugin/map/	0	t	/plugin/map/map.png
\.


--
-- Data for Name: gdpr_register; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.gdpr_register (register_id, register_user_name, register_date, register_action, register_type, register_data, register_finality) FROM stdin;
\.


--
-- Data for Name: gesa_connected; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.gesa_connected (connected_gesa_id, connected_device_id) FROM stdin;
\.


--
-- Data for Name: gesa_info; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.gesa_info (gesa_device_id, gesa_max_slot, gesa_sn, gesa_version_base, gesa_version_nrpe, gesa_version_plugin, gesa_version_web, gesa_repo_global, gesa_repo_gesa, gesa_last_update, gesa_rsbp, gesa_local_ip, gesa_macadress) FROM stdin;
\.


--
-- Data for Name: rrd_plugins; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rrd_plugins (rrd_plugin_id, rrd_type, rrd_regex, rrd_time, rrd_id, rrd_name, rrd_min, rrd_max, rrd_vertical_label, rrd_graph_type, rrd_graph_color, rrd_regex_threshold, rrd_graph_num, rrd_calc) FROM stdin;
\.


--
-- Data for Name: rss_category; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rss_category (category_id, category_name, category_description) FROM stdin;
\.


--
-- Data for Name: rss_channel; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rss_channel (channel_id, channel_name, channel_description, channel_language, channel_last_publication, channel_image, channel_type, channel_url, channel_enable, channel_notdelete, channel_group_link) FROM stdin;
1	Watchguard - endpoint			\N		REMOTE	https://www.watchguard.com/wgrd-blog/feed/endpoint-security	f	f	\N
2	Axis - Firmware			\N		REMOTE	http://feeds.axis.com/axisfirmware	f	f	\N
3	Qnap - News			\N		REMOTE	https://www.qnap.com/fr-fr/news/feed	f	f	\N
4	Pfsense - News			\N		REMOTE	https://www.netgate.com/blog/rss.xml	f	f	\N
5	Cisco - security			\N		REMOTE	https://sec.cloudapps.cisco.com/security/center/psirtrss20/CiscoSecurityAdvisory.xml	f	f	\N
6	Synology - News			\N		REMOTE	https://www.synology.com/api/rssfeed/news/en-global	f	f	\N
7	Proxmox			\N		REMOTE	https://my.proxmox.com/en/announcements/rss	f	f	\N
8	Zyxel - USG			\N		REMOTE	https://community.zyxel.com/en/categories/security-gateway-news-and-release/feed.rss	f	f	\N
9	Stormshield - security			\N		REMOTE	https://advisories.stormshield.eu/feed/	f	f	\N
10	OPNSense			\N		REMOTE	https://forum.opnsense.org/index.php?board=11.0;type=rss;action=.xml	f	f	\N
11	Ubuntu - News			\N		REMOTE	https://ubuntu-news.org/feed/	f	f	\N
12	Microsoft - security			\N		REMOTE	https://api.msrc.microsoft.com/update-guide/rss	f	f	\N
13	APC - News			\N		REMOTE	https://blog.apc.com/category/solutions-services/feed/	f	f	\N
14	HPE - News			\N		REMOTE	https://www.hpe.com/us/en/newsroom/rss.xml	f	f	\N
15	Citrix - security			\N		REMOTE	https://support.citrix.com/feed/products/all/securitybulletins.rss	f	f	\N
16	RedHat - security			\N		REMOTE	https://www.redhat.com/en/rss/blog/channel/security	f	f	\N
17	Debian - News			\N		REMOTE	https://www.debian.org/News/news	f	f	\N
18	Redhat - Enterprise			\N		REMOTE	https://www.redhat.com/en/rss/blog/channel/red-hat-enterprise-linux	f	f	\N
19	Sophos - UTM			\N		REMOTE	https://community.sophos.com/utm-firewall/b/blog/rss	f	f	\N
20	Sophos - XG			\N		REMOTE	https://community.sophos.com/sophos-xg-firewall/b/blog/rss	f	f	\N
21	Fortinet - PSIRT			\N		REMOTE	https://feeds.fortinet.com/fortinet/blog/psirt&x=1	f	f	\N
22	Dell - Product			\N		REMOTE	https://www.dell.com/en-us/blog/categories/products/feed/	f	f	\N
23	Sophos - produit			\N		REMOTE	https://news.sophos.com/fr-fr/category/products-services/feed	f	f	\N
24	Citrix - Software			\N		REMOTE	https://support.citrix.com/feed/products/all/softwareupdates.rss	f	f	\N
25	VMware - VSphere			\N		REMOTE	https://docs.vmware.com/en/VMware-vSphere/rn_rss.xml	f	f	\N
26	Fortinet - Firmware			\N		REMOTE	https://pub.kb.fortinet.com/rss/firmware.xml	f	f	\N
27	Riello UPS- News			\N		REMOTE	https://www.riello-ups.fr/feed	f	f	\N
28	Fortinet - security			\N		REMOTE	https://filestore.fortinet.com/fortiguard/rss/ir.xml	f	f	\N
29	Watchguard - network			\N		REMOTE	https://www.watchguard.com/wgrd-blog/feed/network-security	f	f	\N
30	Debian - security			\N		REMOTE	https://www.debian.org/security/dsa-long	f	f	\N
31	Esia - News	\N	fr-be	\N	\N	REMOTE	https://rss.esia-sa.com/rss.php?flow=ESIA+-+NEWS	t	t	RSS_GROUP_ALL
\.


--
-- Data for Name: rss_link_channel_to_category; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rss_link_channel_to_category (link_channel_id, link_category_id) FROM stdin;
\.


--
-- Data for Name: rss_message; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rss_message (message_id, message_title, message_description, message_link, message_author, message_image, message_channel_id, message_sign, message_date_publish, message_group_view, message_categories) FROM stdin;
\.


--
-- Data for Name: rss_user_read; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rss_user_read (read_user_id, read_message_id) FROM stdin;
\.


--
-- Data for Name: rss_user_subscribe; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.rss_user_subscribe (subscribe_user_id, subscribe_channel_id) FROM stdin;
1	31
2	31
24	31
23	31
\.


--
-- Data for Name: sms_send; Type: TABLE DATA; Schema: public; Owner: esia
--

COPY public.sms_send (sms_id, sms_to, sms_message, sms_date) FROM stdin;
\.


--
-- Name: dashboard_dashboard_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.dashboard_dashboard_id_seq', 2, true);


--
-- Name: dashboard_tile_tile_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.dashboard_tile_tile_id_seq', 14, true);


--
-- Name: dashboard_view_view_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.dashboard_view_view_id_seq', 25, true);


--
-- Name: dashboard_widget_group_widget_group_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.dashboard_widget_group_widget_group_id_seq', 8, true);


--
-- Name: dashboard_widget_widget_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.dashboard_widget_widget_id_seq', 14, true);


--
-- Name: device_tab_tab_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.device_tab_tab_id_seq', 1, false);


--
-- Name: esia_communication_message_message_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_communication_message_message_id_seq', 1, false);


--
-- Name: esia_cron_error_cron_error_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_cron_error_cron_error_id_seq', 1, false);


--
-- Name: esia_cron_plugin_cron_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_cron_plugin_cron_id_seq', 1, true);


--
-- Name: esia_device_advance_config_adv_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_device_advance_config_adv_id_seq', 1, false);


--
-- Name: esia_device_advance_config_type_device_config_type_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_device_advance_config_type_device_config_type_id_seq', 2, true);


--
-- Name: esia_device_device_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_device_device_id_seq', 10, true);


--
-- Name: esia_device_group_device_group_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_device_group_device_group_id_seq', 1, false);


--
-- Name: esia_device_type_type_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_device_type_type_id_seq', 18, true);


--
-- Name: esia_error_happen_happen_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_error_happen_happen_id_seq', 1, false);


--
-- Name: esia_errors_error_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_errors_error_id_seq', 16, true);


--
-- Name: esia_groups_group_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_groups_group_id_seq', 7, true);


--
-- Name: esia_interface_interface_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_interface_interface_id_seq', 1, false);


--
-- Name: esia_interface_node_connected_connected_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_interface_node_connected_connected_id_seq', 1, false);


--
-- Name: esia_maintenance_configuration_maintenance_conf_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_maintenance_configuration_maintenance_conf_id_seq', 1, false);


--
-- Name: esia_maintenance_planned_planned_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_maintenance_planned_planned_id_seq', 1, false);


--
-- Name: esia_manufacturer_class_class_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_manufacturer_class_class_id_seq', 17, true);


--
-- Name: esia_model_model_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_model_model_id_seq', 21, true);


--
-- Name: esia_os_os_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_os_os_id_seq', 3, true);


--
-- Name: esia_plugin_plugin_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_plugin_plugin_id_seq', 1, true);


--
-- Name: esia_privilege_privilege_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_privilege_privilege_id_seq', 18, true);


--
-- Name: esia_program_program_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_program_program_id_seq', 4, true);


--
-- Name: esia_rdp_rdp_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_rdp_rdp_id_seq', 1, false);


--
-- Name: esia_service_rra_file_rra_file_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_service_rra_file_rra_file_id_seq', 1, false);


--
-- Name: esia_services_service_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_services_service_id_seq', 1, false);


--
-- Name: esia_template_template_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_template_template_id_seq', 14, true);


--
-- Name: esia_theme_theme_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_theme_theme_id_seq', 3, true);


--
-- Name: esia_tpl_service_sertpl_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_tpl_service_sertpl_id_seq', 68, true);


--
-- Name: esia_users_user_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_users_user_id_seq', 24, true);


--
-- Name: esia_web_plugin_web_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.esia_web_plugin_web_id_seq', 9, true);


--
-- Name: gdpr_register_register_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.gdpr_register_register_id_seq', 1, false);


--
-- Name: rrd_plugins_rrd_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.rrd_plugins_rrd_id_seq', 1, false);


--
-- Name: rss_category_category_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.rss_category_category_id_seq', 1, false);


--
-- Name: rss_channel_channel_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.rss_channel_channel_id_seq', 31, true);


--
-- Name: rss_message_message_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.rss_message_message_id_seq', 1, false);


--
-- Name: sms_send_sms_id_seq; Type: SEQUENCE SET; Schema: public; Owner: esia
--

SELECT pg_catalog.setval('public.sms_send_sms_id_seq', 1, false);


--
-- Name: esia_group_device_access access_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_group_device_access
    ADD CONSTRAINT "access_PrimaryKey" PRIMARY KEY (access_group_id, access_device_id);


--
-- Name: esia_php_class class_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_php_class
    ADD CONSTRAINT "class_PrimaryKey" PRIMARY KEY (class_id);


--
-- Name: esia_configuration config_primary_key; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_configuration
    ADD CONSTRAINT config_primary_key PRIMARY KEY (config_key);


--
-- Name: esia_interface_connected_to conint_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface_connected_to
    ADD CONSTRAINT "conint_primaryKey" PRIMARY KEY (conint_switch_interface_id, conint_interface_id2);


--
-- Name: esia_cron_plugin cron_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_cron_plugin
    ADD CONSTRAINT "cron_primaryKey" PRIMARY KEY (cron_id);


--
-- Name: dashboard_user_access dashboard_user_access_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_user_access
    ADD CONSTRAINT "dashboard_user_access_primaryKey" PRIMARY KEY (board_user_id, board_dashboard_id, board_user_access);


--
-- Name: dashboard_view dashboard_view_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_view
    ADD CONSTRAINT "dashboard_view_primaryKey" PRIMARY KEY (view_id);


--
-- Name: dashboard dashboards_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard
    ADD CONSTRAINT "dashboards_primaryKey" PRIMARY KEY (dashboard_id);


--
-- Name: esia_device device_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device
    ADD CONSTRAINT "device_PrimaryKey" PRIMARY KEY (device_id);


--
-- Name: esia_device_advance_config_type device_config_type_NameUNique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config_type
    ADD CONSTRAINT "device_config_type_NameUNique" UNIQUE (device_config_type_name);


--
-- Name: esia_device_advance_config_type device_config_type_Primary_key; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config_type
    ADD CONSTRAINT "device_config_type_Primary_key" PRIMARY KEY (device_config_type_id);


--
-- Name: esia_device_group_in device_group_in_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_group_in
    ADD CONSTRAINT "device_group_in_PrimaryKey" PRIMARY KEY (group_in_device_id, group_in_device_group_id);


--
-- Name: esia_device device_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device
    ADD CONSTRAINT device_name_unique UNIQUE (device_name);


--
-- Name: esia_errors error_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_errors
    ADD CONSTRAINT error_name_unique UNIQUE (error_name);


--
-- Name: esia_errors error_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_errors
    ADD CONSTRAINT "error_primaryKey" PRIMARY KEY (error_id);


--
-- Name: esia_communication_message_link_user esia_communication_message_link_user_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_communication_message_link_user
    ADD CONSTRAINT "esia_communication_message_link_user_primaryKey" PRIMARY KEY (message_link_id, user_link_id);


--
-- Name: esia_communication_message esia_communication_message_primarykey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_communication_message
    ADD CONSTRAINT esia_communication_message_primarykey PRIMARY KEY (message_id);


--
-- Name: esia_consumption esia_consumption_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_consumption
    ADD CONSTRAINT esia_consumption_pkey PRIMARY KEY (conso_device_id);


--
-- Name: esia_cron_error esia_cron_error_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_cron_error
    ADD CONSTRAINT esia_cron_error_pkey PRIMARY KEY (cron_error_id);


--
-- Name: esia_device_advance_config esia_device_advance_config_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config
    ADD CONSTRAINT "esia_device_advance_config_PrimaryKey" PRIMARY KEY (adv_id);


--
-- Name: esia_device_group esia_device_group_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_group
    ADD CONSTRAINT "esia_device_group_PrimaryKey" PRIMARY KEY (device_group_id);


--
-- Name: esia_interface_node_connected esia_interface_node_connected_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface_node_connected
    ADD CONSTRAINT esia_interface_node_connected_pkey PRIMARY KEY (connected_id);


--
-- Name: esia_mac esia_mac_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_mac
    ADD CONSTRAINT "esia_mac_primaryKey" PRIMARY KEY (mac);


--
-- Name: esia_program esia_programPrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_program
    ADD CONSTRAINT "esia_programPrimaryKey" PRIMARY KEY (program_id);


--
-- Name: esia_plugin_react_program esia_react_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_plugin_react_program
    ADD CONSTRAINT "esia_react_PrimaryKey" PRIMARY KEY (react_plugin_id, react_program_id);


--
-- Name: esia_service_response_program esia_response_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_response_program
    ADD CONSTRAINT "esia_response_PrimaryKey" PRIMARY KEY (response_service_id, response_program_id);


--
-- Name: esia_theme esia_theme_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_theme
    ADD CONSTRAINT esia_theme_pkey PRIMARY KEY (theme_id);


--
-- Name: esia_device_type esia_type_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_type
    ADD CONSTRAINT "esia_type_primaryKey" PRIMARY KEY (type_id);


--
-- Name: gdpr_register gdpr_register_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gdpr_register
    ADD CONSTRAINT "gdpr_register_PrimaryKey" PRIMARY KEY (register_id);


--
-- Name: gesa_connected gesa_connected_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gesa_connected
    ADD CONSTRAINT "gesa_connected_PrimaryKey" PRIMARY KEY (connected_gesa_id, connected_device_id);


--
-- Name: gesa_info gesa_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gesa_info
    ADD CONSTRAINT "gesa_primaryKey" PRIMARY KEY (gesa_device_id);


--
-- Name: gesa_info gesa_sn_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gesa_info
    ADD CONSTRAINT gesa_sn_unique UNIQUE (gesa_sn);


--
-- Name: esia_privilege_grantedto grantedto_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_privilege_grantedto
    ADD CONSTRAINT "grantedto_PrimaryKey" PRIMARY KEY (grant_group_id, grant_privilege_id);


--
-- Name: esia_groups group_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_groups
    ADD CONSTRAINT group_name_unique UNIQUE (group_name);


--
-- Name: esia_groups groups_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_groups
    ADD CONSTRAINT "groups_PrimaryKey" PRIMARY KEY (group_id);


--
-- Name: esia_error_happen happen_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen
    ADD CONSTRAINT "happen_primaryKey" PRIMARY KEY (happen_id);


--
-- Name: esia_interface interface_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface
    ADD CONSTRAINT "interface_primaryKey" PRIMARY KEY (interface_id);


--
-- Name: device_tab_linked linked_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab_linked
    ADD CONSTRAINT "linked_primaryKey" PRIMARY KEY (tab_linked_device_id, tab_linked_tab_id);


--
-- Name: esia_mac_address mac_address_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_mac_address
    ADD CONSTRAINT "mac_address_primaryKey" PRIMARY KEY (mac_address, mac_interface_id, mac_vlan_id);


--
-- Name: esia_maintenance_configuration maintenance_conf_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_maintenance_configuration
    ADD CONSTRAINT "maintenance_conf_PrimaryKey" PRIMARY KEY (maintenance_conf_id);


--
-- Name: esia_manufacturer manufacturer_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_manufacturer
    ADD CONSTRAINT manufacturer_name_unique UNIQUE (manufacturer_name);


--
-- Name: esia_manufacturer manufacturer_primarykey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_manufacturer
    ADD CONSTRAINT manufacturer_primarykey PRIMARY KEY (manufacturer_oid);


--
-- Name: esia_memberof memberof_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_memberof
    ADD CONSTRAINT "memberof_PrimaryKey" PRIMARY KEY (member_group_id, member_user_id);


--
-- Name: esia_model model_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_model
    ADD CONSTRAINT model_name_unique UNIQUE (model_name);


--
-- Name: esia_model model_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_model
    ADD CONSTRAINT "model_primaryKey" PRIMARY KEY (model_id);


--
-- Name: esia_os os_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_os
    ADD CONSTRAINT "os_primaryKey" PRIMARY KEY (os_id);


--
-- Name: esia_service_pending_execute pending_primary_key; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_pending_execute
    ADD CONSTRAINT pending_primary_key PRIMARY KEY (pending_service_id);


--
-- Name: esia_maintenance_planned planned_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_maintenance_planned
    ADD CONSTRAINT "planned_primaryKey" PRIMARY KEY (planned_id);


--
-- Name: esia_plugins plugin_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_plugins
    ADD CONSTRAINT plugin_name_unique UNIQUE (plugin_name);


--
-- Name: esia_plugins plugin_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_plugins
    ADD CONSTRAINT "plugin_primaryKey" PRIMARY KEY (plugin_id);


--
-- Name: esia_privileges privilege_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_privileges
    ADD CONSTRAINT "privilege_PrimaryKey" PRIMARY KEY (privilege_id);


--
-- Name: esia_privileges privilege_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_privileges
    ADD CONSTRAINT privilege_name_unique UNIQUE (privilege_name);


--
-- Name: esia_rdp rdp_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_rdp
    ADD CONSTRAINT "rdp_primaryKey" PRIMARY KEY (rdp_id);


--
-- Name: rss_user_read read_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_user_read
    ADD CONSTRAINT "read_primaryKey" PRIMARY KEY (read_user_id, read_message_id);


--
-- Name: esia_service_rra_file rra_file_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_rra_file
    ADD CONSTRAINT "rra_file_primaryKey" PRIMARY KEY (rra_file_id);


--
-- Name: rrd_plugins rrd_Primarykey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rrd_plugins
    ADD CONSTRAINT "rrd_Primarykey" PRIMARY KEY (rrd_id);


--
-- Name: rrd_plugins rrd_plugin_and_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rrd_plugins
    ADD CONSTRAINT rrd_plugin_and_name_unique UNIQUE (rrd_plugin_id, rrd_name);


--
-- Name: rss_category rss_category_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_category
    ADD CONSTRAINT rss_category_pkey PRIMARY KEY (category_id);


--
-- Name: rss_channel rss_channel_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_channel
    ADD CONSTRAINT rss_channel_pkey PRIMARY KEY (channel_id);


--
-- Name: rss_link_channel_to_category rss_link_channel_to_category_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_link_channel_to_category
    ADD CONSTRAINT rss_link_channel_to_category_pkey PRIMARY KEY (link_channel_id, link_category_id);


--
-- Name: rss_message rss_message_pkey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_message
    ADD CONSTRAINT rss_message_pkey PRIMARY KEY (message_id);


--
-- Name: esia_search search_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_search
    ADD CONSTRAINT "search_PrimaryKey" PRIMARY KEY (search_keyword);


--
-- Name: esia_service_pattern_included sertpl_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_pattern_included
    ADD CONSTRAINT "sertpl_primaryKey" PRIMARY KEY (included_id);


--
-- Name: esia_services service_primarKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_services
    ADD CONSTRAINT "service_primarKey" PRIMARY KEY (service_id);


--
-- Name: sms_send sms_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.sms_send
    ADD CONSTRAINT "sms_primaryKey" PRIMARY KEY (sms_id);


--
-- Name: rss_user_subscribe subscribe_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_user_subscribe
    ADD CONSTRAINT "subscribe_primaryKey" PRIMARY KEY (subscribe_user_id, subscribe_channel_id);


--
-- Name: esia_switch switch_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_switch
    ADD CONSTRAINT "switch_primaryKey" PRIMARY KEY (switch_device_id);


--
-- Name: device_tab tab_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab
    ADD CONSTRAINT tab_name_unique UNIQUE (tab_name);


--
-- Name: device_tab tab_primary_key; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab
    ADD CONSTRAINT tab_primary_key PRIMARY KEY (tab_id);


--
-- Name: esia_service_patterns template_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_patterns
    ADD CONSTRAINT template_name_unique UNIQUE (pattern_name);


--
-- Name: esia_service_patterns template_primary_key; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_patterns
    ADD CONSTRAINT template_primary_key PRIMARY KEY (pattern_id);


--
-- Name: dashboard_tile tile_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_tile
    ADD CONSTRAINT "tile_PrimaryKey" PRIMARY KEY (tile_id);


--
-- Name: esia_communication_message title_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_communication_message
    ADD CONSTRAINT title_unique UNIQUE (title);


--
-- Name: esia_device_type type_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_type
    ADD CONSTRAINT type_name_unique UNIQUE (type_name);


--
-- Name: esia_users user_PrimaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_users
    ADD CONSTRAINT "user_PrimaryKey" PRIMARY KEY (user_id);


--
-- Name: esia_users user_name_Unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_users
    ADD CONSTRAINT "user_name_Unique" UNIQUE (user_name);


--
-- Name: esia_mac_address_vendor vendor_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_mac_address_vendor
    ADD CONSTRAINT "vendor_primaryKey" PRIMARY KEY (vendor_mac_address);


--
-- Name: esia_web_plugin web_name_unique; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_web_plugin
    ADD CONSTRAINT web_name_unique UNIQUE (web_name);


--
-- Name: esia_web_plugin web_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_web_plugin
    ADD CONSTRAINT "web_primaryKey" PRIMARY KEY (web_id);


--
-- Name: dashboard_widget_group widget_group_primary_key; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_widget_group
    ADD CONSTRAINT widget_group_primary_key PRIMARY KEY (widget_group_id);


--
-- Name: dashboard_widget widget_primaryKey; Type: CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_widget
    ADD CONSTRAINT "widget_primaryKey" PRIMARY KEY (widget_id);


--
-- Name: access_index; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX access_index ON public.esia_group_device_access USING btree (access_group_id, access_device_id);


--
-- Name: access_index_deviceId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "access_index_deviceId" ON public.esia_group_device_access USING btree (access_device_id);


--
-- Name: access_index_groupId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "access_index_groupId" ON public.esia_group_device_access USING btree (access_group_id);


--
-- Name: config_visibility; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX config_visibility ON public.esia_configuration USING btree (config_visibility);


--
-- Name: cron_index_plugin_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX cron_index_plugin_id ON public.esia_cron_plugin USING btree (cron_plugin_id);


--
-- Name: device_group_index_parent; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX device_group_index_parent ON public.esia_device_group USING btree (device_group_parent);


--
-- Name: device_index_delete; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX device_index_delete ON public.esia_device USING btree (device_delete);


--
-- Name: device_index_device_description; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX device_index_device_description ON public.esia_device USING btree (device_description NULLS FIRST);


--
-- Name: device_index_device_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX device_index_device_id ON public.esia_device USING btree (device_id);


--
-- Name: device_index_device_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE UNIQUE INDEX device_index_device_name ON public.esia_device USING btree (device_name);


--
-- Name: device_index_device_type; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX device_index_device_type ON public.esia_device USING btree (device_type);


--
-- Name: device_index_group_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX device_index_group_name ON public.esia_device USING btree (device_group_name);


--
-- Name: error_index_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX error_index_id ON public.esia_errors USING btree (error_id);


--
-- Name: error_index_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX error_index_name ON public.esia_errors USING btree (error_name);


--
-- Name: error_index_type; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX error_index_type ON public.esia_errors USING btree (error_type);


--
-- Name: exe_index_device_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX exe_index_device_id ON public.esia_services USING btree (service_device_id);


--
-- Name: exe_index_plugin_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX exe_index_plugin_id ON public.esia_services USING btree (service_plugin_id);


--
-- Name: fki_class_foreignKey_esia_device_type; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_class_foreignKey_esia_device_type" ON public.esia_php_class USING btree (class_type);


--
-- Name: fki_class_foreignKey_manufacturer_oid; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_class_foreignKey_manufacturer_oid" ON public.esia_php_class USING btree (class_manufacturer_oid);


--
-- Name: fki_device_config_type_ForeignKey_web_plugin_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_device_config_type_ForeignKey_web_plugin_id" ON public.esia_device_advance_config_type USING btree (device_config_type_webplugin_id);


--
-- Name: fki_included_ForeignKay_program_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_included_ForeignKay_program_id" ON public.esia_service_pattern_included USING btree (included_program_id);


--
-- Name: fki_service_foreignkey_pattern_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX fki_service_foreignkey_pattern_id ON public.esia_services USING btree (service_included_id);


--
-- Name: fki_switch_foreignKey_class_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_switch_foreignKey_class_id" ON public.esia_switch USING btree (switch_class_id);


--
-- Name: fki_switch_foreignKey_model_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_switch_foreignKey_model_id" ON public.esia_switch USING btree (switch_model_id);


--
-- Name: fki_switch_foreignKey_os_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "fki_switch_foreignKey_os_id" ON public.esia_switch USING btree (switch_os_id);


--
-- Name: gesa_index_deviceId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "gesa_index_deviceId" ON public.gesa_connected USING btree (connected_device_id);


--
-- Name: grant_index; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX grant_index ON public.esia_privilege_grantedto USING btree (grant_group_id, grant_privilege_id);


--
-- Name: grant_index_access; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX grant_index_access ON public.esia_privilege_grantedto USING btree (grant_acces);


--
-- Name: grant_index_groupId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "grant_index_groupId" ON public.esia_privilege_grantedto USING btree (grant_group_id);


--
-- Name: grant_index_privilegeId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "grant_index_privilegeId" ON public.esia_privilege_grantedto USING btree (grant_privilege_id);


--
-- Name: group_in_index_deviceId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "group_in_index_deviceId" ON public.esia_device_group_in USING btree (group_in_device_id);


--
-- Name: group_in_index_groupId; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "group_in_index_groupId" ON public.esia_device_group_in USING btree (group_in_device_group_id);


--
-- Name: group_index_group_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX group_index_group_id ON public.esia_groups USING btree (group_id);


--
-- Name: group_index_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX group_index_name ON public.esia_groups USING btree (group_name);


--
-- Name: happen_ForeignKey_user_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX "happen_ForeignKey_user_id" ON public.esia_error_happen USING btree (happen_user_id);


--
-- Name: happen_idx_timestamp_error; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX happen_idx_timestamp_error ON public.esia_error_happen USING btree (happen_timestamp DESC) WITH (deduplicate_items='true');


--
-- Name: happen_index_acquitte; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX happen_index_acquitte ON public.esia_error_happen USING btree (happen_acquitte) WHERE (happen_acquitte = 0);


--
-- Name: happen_index_device_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX happen_index_device_id ON public.esia_error_happen USING btree (happen_device_id);


--
-- Name: happen_index_error_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX happen_index_error_id ON public.esia_error_happen USING btree (happen_error_id);


--
-- Name: happen_index_interface_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX happen_index_interface_id ON public.esia_error_happen USING btree (happen_interface_id);


--
-- Name: happen_index_service_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX happen_index_service_id ON public.esia_error_happen USING btree (happen_service_id);


--
-- Name: interface_index_device_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX interface_index_device_id ON public.esia_interface USING btree (interface_device_id);


--
-- Name: interface_index_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX interface_index_name ON public.esia_interface USING btree (interface_name);


--
-- Name: interface_index_plug; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX interface_index_plug ON public.esia_interface USING btree (interface_plug);


--
-- Name: mac_index_interface_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX mac_index_interface_id ON public.esia_mac_address USING btree (mac_interface_id);


--
-- Name: mac_index_ip_address; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX mac_index_ip_address ON public.esia_mac_address USING btree (mac_ip_address);


--
-- Name: mac_index_mac_address; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX mac_index_mac_address ON public.esia_mac_address USING btree (mac_address);


--
-- Name: member_index; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX member_index ON public.esia_memberof USING btree (member_group_id, member_user_id);


--
-- Name: member_index_group_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX member_index_group_id ON public.esia_memberof USING btree (member_group_id);


--
-- Name: member_index_user_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX member_index_user_id ON public.esia_memberof USING btree (member_user_id);


--
-- Name: plugin_index_plugin_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX plugin_index_plugin_id ON public.esia_plugins USING btree (plugin_id);


--
-- Name: plugin_index_plugin_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX plugin_index_plugin_name ON public.esia_plugins USING btree (plugin_name);


--
-- Name: privilege_index_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX privilege_index_id ON public.esia_privileges USING btree (privilege_id);


--
-- Name: privilege_index_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX privilege_index_name ON public.esia_privileges USING btree (privilege_name);


--
-- Name: rrd_index_plugin_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX rrd_index_plugin_id ON public.rrd_plugins USING btree (rrd_plugin_id);


--
-- Name: search_index_search_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX search_index_search_name ON public.esia_search USING btree (search_name);


--
-- Name: service_index_previous_state; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX service_index_previous_state ON public.esia_services USING btree (service_previous_state);


--
-- Name: tile_index_dashboard_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX tile_index_dashboard_id ON public.dashboard_tile USING btree (tile_dashboard_id);


--
-- Name: type_index_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX type_index_name ON public.esia_device_type USING btree (type_name);


--
-- Name: type_index_type_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE UNIQUE INDEX type_index_type_id ON public.esia_device_type USING btree (type_id);


--
-- Name: type_index_web_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX type_index_web_id ON public.esia_device_type USING btree (type_web_id);


--
-- Name: user_index_email; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX user_index_email ON public.esia_users USING btree (user_email);


--
-- Name: user_index_user_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX user_index_user_id ON public.esia_users USING btree (user_id);


--
-- Name: user_index_user_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX user_index_user_name ON public.esia_users USING btree (user_name);


--
-- Name: web_index_activate; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX web_index_activate ON public.esia_web_plugin USING btree (web_activate);


--
-- Name: web_index_name; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX web_index_name ON public.esia_web_plugin USING btree (web_name);


--
-- Name: web_index_web_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE UNIQUE INDEX web_index_web_id ON public.esia_web_plugin USING btree (web_id);


--
-- Name: widget_index_tile_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX widget_index_tile_id ON public.dashboard_widget USING btree (widget_tile_id);


--
-- Name: widget_index_view_id; Type: INDEX; Schema: public; Owner: esia
--

CREATE INDEX widget_index_view_id ON public.dashboard_widget USING btree (widget_view_id);


--
-- Name: esia_device deleteDevice; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER "deleteDevice" BEFORE DELETE ON public.esia_device FOR EACH ROW EXECUTE FUNCTION public.trigger_delete_device();


--
-- Name: esia_services deleteService; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER "deleteService" BEFORE DELETE ON public.esia_services FOR EACH ROW EXECUTE FUNCTION public.trigger_delete_service();


--
-- Name: esia_users deleteUser; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER "deleteUser" BEFORE DELETE ON public.esia_users FOR EACH ROW EXECUTE FUNCTION public.trigger_delete_user();


--
-- Name: esia_cron_plugin notifyCronDaemon; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER "notifyCronDaemon" AFTER INSERT OR UPDATE OF cron_interval, cron_log_file, cron_retries, cron_retries_interval, cron_parameter ON public.esia_cron_plugin FOR EACH ROW EXECUTE FUNCTION public.trigger_notify_servicecron();


--
-- Name: rss_channel trigger_channels; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER trigger_channels AFTER INSERT OR UPDATE OF channel_group_link ON public.rss_channel FOR EACH ROW EXECUTE FUNCTION public.trigger_channels();


--
-- Name: esia_communication_message trigger_delete_link_message_user; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER trigger_delete_link_message_user BEFORE DELETE ON public.esia_communication_message FOR EACH ROW EXECUTE FUNCTION public.link_message_user();


--
-- Name: esia_memberof trigger_memberof; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER trigger_memberof AFTER INSERT OR UPDATE ON public.esia_memberof FOR EACH ROW EXECUTE FUNCTION public.trigger_insert_update_memberof();


--
-- Name: esia_device_group updateGroupPath; Type: TRIGGER; Schema: public; Owner: esia
--

CREATE TRIGGER "updateGroupPath" AFTER INSERT OR UPDATE OF device_group_name, device_group_parent ON public.esia_device_group FOR EACH ROW EXECUTE FUNCTION public.trigger_update_group_path();


--
-- Name: esia_group_device_access access_ForeignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_group_device_access
    ADD CONSTRAINT "access_ForeignKey_device_id" FOREIGN KEY (access_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_group_device_access access_ForeignKey_group_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_group_device_access
    ADD CONSTRAINT "access_ForeignKey_group_id" FOREIGN KEY (access_group_id) REFERENCES public.esia_groups(group_id);


--
-- Name: esia_php_class class_foreignKey_manufacturer_oid; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_php_class
    ADD CONSTRAINT "class_foreignKey_manufacturer_oid" FOREIGN KEY (class_manufacturer_oid) REFERENCES public.esia_manufacturer(manufacturer_oid);


--
-- Name: esia_interface_connected_to conint_foreignKey_interface_id1; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface_connected_to
    ADD CONSTRAINT "conint_foreignKey_interface_id1" FOREIGN KEY (conint_switch_interface_id) REFERENCES public.esia_interface(interface_id);


--
-- Name: esia_interface_connected_to conint_foreignKey_interface_id2; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface_connected_to
    ADD CONSTRAINT "conint_foreignKey_interface_id2" FOREIGN KEY (conint_interface_id2) REFERENCES public.esia_interface(interface_id);


--
-- Name: esia_interface_node_connected connected_foreignkey_interface; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface_node_connected
    ADD CONSTRAINT connected_foreignkey_interface FOREIGN KEY (connected_interface_id) REFERENCES public.esia_interface(interface_id);


--
-- Name: esia_consumption conso_fk_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_consumption
    ADD CONSTRAINT conso_fk_device_id FOREIGN KEY (conso_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_cron_error cron_error_foreignKey_cron_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_cron_error
    ADD CONSTRAINT "cron_error_foreignKey_cron_id" FOREIGN KEY (cron_id) REFERENCES public.esia_cron_plugin(cron_id);


--
-- Name: esia_cron_plugin cron_foreignKey_plugin_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_cron_plugin
    ADD CONSTRAINT "cron_foreignKey_plugin_id" FOREIGN KEY (cron_plugin_id) REFERENCES public.esia_plugins(plugin_id);


--
-- Name: dashboard_user_access dashboard_user_access_dashboard_id_foreignKey; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_user_access
    ADD CONSTRAINT "dashboard_user_access_dashboard_id_foreignKey" FOREIGN KEY (board_dashboard_id) REFERENCES public.dashboard(dashboard_id);


--
-- Name: dashboard_user_access dashboard_user_id_foreignKey; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_user_access
    ADD CONSTRAINT "dashboard_user_id_foreignKey" FOREIGN KEY (board_user_id) REFERENCES public.esia_users(user_id);


--
-- Name: dashboard_widget dashboard_view_id_foreignKey; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_widget
    ADD CONSTRAINT "dashboard_view_id_foreignKey" FOREIGN KEY (widget_view_id) REFERENCES public.dashboard_view(view_id);


--
-- Name: esia_device device_ForeignKey_type_name; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device
    ADD CONSTRAINT "device_ForeignKey_type_name" FOREIGN KEY (device_type) REFERENCES public.esia_device_type(type_name);


--
-- Name: esia_device_advance_config_type device_config_type_ForeignKey_web_plugin_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config_type
    ADD CONSTRAINT "device_config_type_ForeignKey_web_plugin_id" FOREIGN KEY (device_config_type_webplugin_id) REFERENCES public.esia_web_plugin(web_id);


--
-- Name: esia_device_group_in device_group_in_ForeignKey_device_group_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_group_in
    ADD CONSTRAINT "device_group_in_ForeignKey_device_group_id" FOREIGN KEY (group_in_device_group_id) REFERENCES public.esia_device_group(device_group_id);


--
-- Name: esia_device_group_in device_group_in_ForeignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_group_in
    ADD CONSTRAINT "device_group_in_ForeignKey_device_id" FOREIGN KEY (group_in_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_communication_message_link_user esia_communication_message_link_user_message_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_communication_message_link_user
    ADD CONSTRAINT esia_communication_message_link_user_message_id FOREIGN KEY (message_link_id) REFERENCES public.esia_communication_message(message_id);


--
-- Name: esia_communication_message_link_user esia_communication_message_link_user_user_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_communication_message_link_user
    ADD CONSTRAINT esia_communication_message_link_user_user_id FOREIGN KEY (user_link_id) REFERENCES public.esia_users(user_id);


--
-- Name: esia_device_advance_config esia_device_advance_config_ForignKey_type_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config
    ADD CONSTRAINT "esia_device_advance_config_ForignKey_type_id" FOREIGN KEY (adv_config_type_id) REFERENCES public.esia_device_advance_config_type(device_config_type_id);


--
-- Name: esia_device_advance_config esia_device_advance_config_foreingKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_advance_config
    ADD CONSTRAINT "esia_device_advance_config_foreingKey_device_id" FOREIGN KEY (adv_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_device_group esia_device_group_ForeignKey_device_group_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_group
    ADD CONSTRAINT "esia_device_group_ForeignKey_device_group_id" FOREIGN KEY (device_group_parent) REFERENCES public.esia_device_group(device_group_id);


--
-- Name: esia_plugin_react_program esia_react_ForeignKey_plugin_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_plugin_react_program
    ADD CONSTRAINT "esia_react_ForeignKey_plugin_id" FOREIGN KEY (react_plugin_id) REFERENCES public.esia_plugins(plugin_id);


--
-- Name: esia_plugin_react_program esia_react_ForeignKey_program_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_plugin_react_program
    ADD CONSTRAINT "esia_react_ForeignKey_program_id" FOREIGN KEY (react_program_id) REFERENCES public.esia_program(program_id);


--
-- Name: esia_service_response_program esia_response_ForeignKey_program_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_response_program
    ADD CONSTRAINT "esia_response_ForeignKey_program_id" FOREIGN KEY (response_program_id) REFERENCES public.esia_program(program_id);


--
-- Name: esia_service_response_program esia_response_ForeignKey_service_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_response_program
    ADD CONSTRAINT "esia_response_ForeignKey_service_id" FOREIGN KEY (response_service_id) REFERENCES public.esia_services(service_id);


--
-- Name: gesa_connected gesa_connected_ForeignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gesa_connected
    ADD CONSTRAINT "gesa_connected_ForeignKey_device_id" FOREIGN KEY (connected_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: gesa_connected gesa_connected_ForeignKey_gesa; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gesa_connected
    ADD CONSTRAINT "gesa_connected_ForeignKey_gesa" FOREIGN KEY (connected_gesa_id) REFERENCES public.esia_device(device_id);


--
-- Name: gesa_info gesa_foreignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.gesa_info
    ADD CONSTRAINT "gesa_foreignKey_device_id" FOREIGN KEY (gesa_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_privilege_grantedto grant_ForeignKey_group_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_privilege_grantedto
    ADD CONSTRAINT "grant_ForeignKey_group_id" FOREIGN KEY (grant_group_id) REFERENCES public.esia_groups(group_id);


--
-- Name: esia_privilege_grantedto grant_ForeignKey_privilege_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_privilege_grantedto
    ADD CONSTRAINT "grant_ForeignKey_privilege_id" FOREIGN KEY (grant_privilege_id) REFERENCES public.esia_privileges(privilege_id);


--
-- Name: esia_error_happen happen_ForeignKey_error_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen
    ADD CONSTRAINT "happen_ForeignKey_error_id" FOREIGN KEY (happen_error_id) REFERENCES public.esia_errors(error_id);


--
-- Name: esia_error_happen happen_ForeignKey_interface_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen
    ADD CONSTRAINT "happen_ForeignKey_interface_id" FOREIGN KEY (happen_interface_id) REFERENCES public.esia_interface(interface_id);


--
-- Name: esia_error_happen happen_ForeignKey_service_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen
    ADD CONSTRAINT "happen_ForeignKey_service_id" FOREIGN KEY (happen_service_id) REFERENCES public.esia_services(service_id);


--
-- Name: esia_error_happen happen_ForeignKey_user_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen
    ADD CONSTRAINT "happen_ForeignKey_user_id" FOREIGN KEY (happen_user_id) REFERENCES public.esia_users(user_id);


--
-- Name: esia_error_happen happen_foreignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_error_happen
    ADD CONSTRAINT "happen_foreignKey_device_id" FOREIGN KEY (happen_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_service_pattern_included included_ForeignKay_program_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_pattern_included
    ADD CONSTRAINT "included_ForeignKay_program_id" FOREIGN KEY (included_program_id) REFERENCES public.esia_program(program_id);


--
-- Name: esia_interface interface_foreignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_interface
    ADD CONSTRAINT "interface_foreignKey_device_id" FOREIGN KEY (interface_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: rss_link_channel_to_category link_foreignKey_category_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_link_channel_to_category
    ADD CONSTRAINT "link_foreignKey_category_id" FOREIGN KEY (link_category_id) REFERENCES public.rss_category(category_id);


--
-- Name: rss_link_channel_to_category link_foreignKey_channel_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_link_channel_to_category
    ADD CONSTRAINT "link_foreignKey_channel_id" FOREIGN KEY (link_channel_id) REFERENCES public.rss_channel(channel_id);


--
-- Name: device_tab_linked linked_foreignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab_linked
    ADD CONSTRAINT "linked_foreignKey_device_id" FOREIGN KEY (tab_linked_device_id) REFERENCES public.esia_device(device_id) MATCH FULL;


--
-- Name: device_tab_linked linked_foreignKey_device_tab; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab_linked
    ADD CONSTRAINT "linked_foreignKey_device_tab" FOREIGN KEY (tab_linked_tab_id) REFERENCES public.device_tab(tab_id) MATCH FULL;


--
-- Name: esia_mac_address mac_foreignKey_interface_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_mac_address
    ADD CONSTRAINT "mac_foreignKey_interface_id" FOREIGN KEY (mac_interface_id) REFERENCES public.esia_interface(interface_id);


--
-- Name: esia_maintenance_configuration maintenance_conf_ForeignKey_service_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_maintenance_configuration
    ADD CONSTRAINT "maintenance_conf_ForeignKey_service_id" FOREIGN KEY (maintenance_conf_service_id) REFERENCES public.esia_services(service_id);


--
-- Name: esia_memberof memberof_ForeignKey_group_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_memberof
    ADD CONSTRAINT "memberof_ForeignKey_group_id" FOREIGN KEY (member_group_id) REFERENCES public.esia_groups(group_id);


--
-- Name: esia_memberof memberof_ForeignKey_user_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_memberof
    ADD CONSTRAINT "memberof_ForeignKey_user_id" FOREIGN KEY (member_user_id) REFERENCES public.esia_users(user_id);


--
-- Name: esia_model model_foreingKey_manufacturer_oid; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_model
    ADD CONSTRAINT "model_foreingKey_manufacturer_oid" FOREIGN KEY (model_manufacturer_oid) REFERENCES public.esia_manufacturer(manufacturer_oid);


--
-- Name: esia_os os_foreignKey_manufacturer_oid; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_os
    ADD CONSTRAINT "os_foreignKey_manufacturer_oid" FOREIGN KEY (os_manufacturer_oid) REFERENCES public.esia_manufacturer(manufacturer_oid);


--
-- Name: esia_maintenance_planned planned_foreignKey_service_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_maintenance_planned
    ADD CONSTRAINT "planned_foreignKey_service_id" FOREIGN KEY (planned_service_id) REFERENCES public.esia_services(service_id);


--
-- Name: esia_rdp rdp_foreignkey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_rdp
    ADD CONSTRAINT rdp_foreignkey_device_id FOREIGN KEY (rdp_device_id) REFERENCES public.esia_device(device_id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: rss_user_read read_foreignKey_message_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_user_read
    ADD CONSTRAINT "read_foreignKey_message_id" FOREIGN KEY (read_message_id) REFERENCES public.rss_message(message_id);


--
-- Name: rss_user_read read_foreignKey_user_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_user_read
    ADD CONSTRAINT "read_foreignKey_user_id" FOREIGN KEY (read_user_id) REFERENCES public.esia_users(user_id);


--
-- Name: esia_service_rra_file rra_file_foreignKey_service_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_rra_file
    ADD CONSTRAINT "rra_file_foreignKey_service_id" FOREIGN KEY (rra_service_id) REFERENCES public.esia_services(service_id);


--
-- Name: rrd_plugins rrd_plugin_ForeignKey_plugin_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rrd_plugins
    ADD CONSTRAINT "rrd_plugin_ForeignKey_plugin_id" FOREIGN KEY (rrd_plugin_id) REFERENCES public.esia_plugins(plugin_id) ON DELETE CASCADE;


--
-- Name: rss_message rss_message_foreignKey_channel; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_message
    ADD CONSTRAINT "rss_message_foreignKey_channel" FOREIGN KEY (message_channel_id) REFERENCES public.rss_channel(channel_id);


--
-- Name: esia_service_pattern_included sertpl_foreignKey_plugin_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_pattern_included
    ADD CONSTRAINT "sertpl_foreignKey_plugin_id" FOREIGN KEY (included_plugin_id) REFERENCES public.esia_plugins(plugin_id) ON DELETE CASCADE;


--
-- Name: esia_service_pattern_included sertpl_foreign_key_template_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_service_pattern_included
    ADD CONSTRAINT sertpl_foreign_key_template_id FOREIGN KEY (included_pattern_id) REFERENCES public.esia_service_patterns(pattern_id);


--
-- Name: esia_services service_ForeignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_services
    ADD CONSTRAINT "service_ForeignKey_device_id" FOREIGN KEY (service_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_services service_ForeignKey_included_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_services
    ADD CONSTRAINT "service_ForeignKey_included_id" FOREIGN KEY (service_included_id) REFERENCES public.esia_service_pattern_included(included_id);


--
-- Name: esia_services service_ForeignKey_plugin _id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_services
    ADD CONSTRAINT "service_ForeignKey_plugin _id" FOREIGN KEY (service_plugin_id) REFERENCES public.esia_plugins(plugin_id);


--
-- Name: rss_user_subscribe subscribe_foreignKey_channel; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_user_subscribe
    ADD CONSTRAINT "subscribe_foreignKey_channel" FOREIGN KEY (subscribe_channel_id) REFERENCES public.rss_channel(channel_id);


--
-- Name: rss_user_subscribe subscribe_foreignKey_user; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.rss_user_subscribe
    ADD CONSTRAINT "subscribe_foreignKey_user" FOREIGN KEY (subscribe_user_id) REFERENCES public.esia_users(user_id);


--
-- Name: esia_switch switch_foreignKey_class_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_switch
    ADD CONSTRAINT "switch_foreignKey_class_id" FOREIGN KEY (switch_class_id) REFERENCES public.esia_php_class(class_id);


--
-- Name: esia_switch switch_foreignKey_device_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_switch
    ADD CONSTRAINT "switch_foreignKey_device_id" FOREIGN KEY (switch_device_id) REFERENCES public.esia_device(device_id);


--
-- Name: esia_switch switch_foreignKey_model_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_switch
    ADD CONSTRAINT "switch_foreignKey_model_id" FOREIGN KEY (switch_model_id) REFERENCES public.esia_model(model_id);


--
-- Name: esia_switch switch_foreignKey_os_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_switch
    ADD CONSTRAINT "switch_foreignKey_os_id" FOREIGN KEY (switch_os_id) REFERENCES public.esia_os(os_id);


--
-- Name: device_tab tab_foreignKey_web_plugin; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.device_tab
    ADD CONSTRAINT "tab_foreignKey_web_plugin" FOREIGN KEY (tab_web_id) REFERENCES public.esia_web_plugin(web_id) MATCH FULL;


--
-- Name: dashboard_tile tile_dashboard_id_ForeignKey; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_tile
    ADD CONSTRAINT "tile_dashboard_id_ForeignKey" FOREIGN KEY (tile_dashboard_id) REFERENCES public.dashboard(dashboard_id);


--
-- Name: esia_device_type type_web_id_ForeignKey; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_device_type
    ADD CONSTRAINT "type_web_id_ForeignKey" FOREIGN KEY (type_web_id) REFERENCES public.esia_web_plugin(web_id);


--
-- Name: esia_users user_foreignKey_theme_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.esia_users
    ADD CONSTRAINT "user_foreignKey_theme_id" FOREIGN KEY (user_theme_id) REFERENCES public.esia_theme(theme_id);


--
-- Name: dashboard_view view_foreignkey_group_id; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_view
    ADD CONSTRAINT view_foreignkey_group_id FOREIGN KEY (view_group_id) REFERENCES public.dashboard_widget_group(widget_group_id);


--
-- Name: dashboard_widget widget_tile_id_foreignKey; Type: FK CONSTRAINT; Schema: public; Owner: esia
--

ALTER TABLE ONLY public.dashboard_widget
    ADD CONSTRAINT "widget_tile_id_foreignKey" FOREIGN KEY (widget_tile_id) REFERENCES public.dashboard_tile(tile_id);


--
-- PostgreSQL database dump complete
--

